/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
package com.radview.util.datastructures;


public class SynchronizedQueue {

    private Queue SynchQueue = new Queue();

    public synchronized void clear() {
        SynchQueue.clear();
    }

    public synchronized void Put(Object item) {
        boolean QueueIsEmpty = (Size() == 0);
        SynchQueue.put(item);
        if (QueueIsEmpty) notify();
    }

    public synchronized void PutOnTop(Object item) {
        boolean QueueIsEmpty = (Size() == 0);
        SynchQueue.putOnTop(item);
        if (QueueIsEmpty) notify();
    }

    public synchronized Object Get() throws EmptyQueueException{
        return SynchQueue.get();
    }

    public Object GetAndWait() {
        if (Size() > 0) {
            try {
                return Get();
            }
            catch (EmptyQueueException e) {}  //Throwable
        }
        else {
						synchronized(this) {
							if (Size() == 0) {
								try {
						      wait();
								}
								catch (InterruptedException e) {}
							}
						}
            try {
                return Get();
            }
            catch (EmptyQueueException e) {}  //Throwable
        }
        return null;
    }

    public Object GetAndWait(int TimeoutInMiliSeconds)
                                throws TimeoutExpiredException{
        if (Size() > 0) {
            try {
                return Get();
            }
            catch (EmptyQueueException e) {}  //Throwable
        }
        else {
						synchronized(this) {
							if (Size() == 0) {
								try {
							    wait(TimeoutInMiliSeconds);
								}
								catch (InterruptedException e) {}
							}
						}
						if (Size() > 0) {
                try {
                    return Get();
                }
                catch (EmptyQueueException e) {}  //Throwable
            }
            else { // timeout expired
                throw new TimeoutExpiredException();
            }
        }
        return null;

    }

    public synchronized int Size() {
        return SynchQueue.size();
    }
}
