/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
 
/*
 *
 * Copyright (c) 1996 Brian Maso. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted. 
 * 
 * A Mutex is just a Semaphore with a maximum owner count of 1.
 *
 */

package com.radview.util.synch;

/**
 * A Mutex is a single-owner Semaphore. That is, it is an owned Synch
 * object which has only one owner at a time. When it is owned, the
 * Mutex' state is unsignaled. When it is unowned, the Mutex' state
 * is signaled.
 *
 * <P>A thread gains ownership of the Mutex automatically when it waits
 * for the Mutex in <I>Scheduler.waitFor()</I> and is released. The thread
 * must then release ownership of the Mutex using <I>Semaphore.release()</I>.
 * Only the owner thread may call <I>release()</I>.
 *
 * <P>If a thread dies before releasing ownership of a Mutex, then the
 * Mutex is permanently unsignaled. No other thread will ever be able to
 * gain ownership, and any calls to <I>Scheduler.waitFor()</I> will result
 * in an indefinite wait.
 *
 * @see Semaphore
 *
 * @version 1.0
 * @author Brian Maso
 */

public class Mutex extends Semaphore {

	/**
	 * Constructor for the Mutex class. It takes no
	 * arguments.
	 */
	public Mutex() {
		super(1);
	}

}
