/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
 
/*
 *
 * Copyright (c) 1996 Brian Maso. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted. 
 *
 * A TickingClock is a subclass of the Clock class. The only
 * difference is that a TickingClock only allows each waiting
 * Thread a single gain per period. This is handled by having
 * an internal list of Threads which have gained the TickingClock
 * this period (ie, since the last time it was signaled).
 *
 */

package com.radview.util.synch;

/**
 * A TickingClock is a slight variation of the Clock Synch. A
 * TickingClock will allow a single thread to be released by
 * the Scheduler only once per cycle. By contrast the Clock
 * Synch will allow the same thread to be released by the
 * Scheduler any number of times during the second half of its
 * cycle. The TickingClock class has no public methods of its
 * own other than its constructor.
 *
 * @see Clock
 *
 * @version 1.0
 * @author Brian Maso
 */

import java.util.Vector;

public class TickingClock extends Clock {
	Vector _vectOwnersThisPeriod = new Vector();
	
	/**
	 * You only need to specify the half-period of the TickingClock's cycle
	 * to the TickingClock class constructor. Every two half-periods, the
	 * TickingClock will release all threads waiting for it in the Scheduler
	 * exactly once.
	 *
	 * @param	half_period
	 *					Every two<I>half_period</I> millseconds the
	 *					TickingClock will signal any waiting threads
	 *					exactly once.
	 */
	public TickingClock(long half_period) {
		super(half_period);
	}
	
	/* *********
	 * void setState(int state)
	 *
	 * Override setState() so the vector of owners this
	 * period is cleared before super version is run.
	 *
	 ********* */
	
        protected void setState(int state) {
		if(signaled == state)
			_vectOwnersThisPeriod.removeAllElements();
		super.setState(state);
	}
	
	/* *********
	 * void gain(Thread t)
	 * 
	 * Add t to list of owners this period before running
	 * super version of this method.
	 *
	 ********* */
	
	void gain(Thread t) {
		_vectOwnersThisPeriod.addElement(t);
		super.gain(t);
		_hashOwnership.remove(t);
	}
	
	/* *********
	 * boolean reserve(Thread t)
	 *
	 * Must return false if t is on list of Threads which have
	 * already gained ownership this period.
	 *
	 ********* */
	
	boolean reserve(Thread t) {
		if(_vectOwnersThisPeriod.contains(t))
			return false;
			
		return super.reserve(t);
	}
}
