/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
package com.radview.util.testtalk;

import com.radview.util.datastructures.SynchronizedQueue;

// class which handle the client.
// The class handles 3 queues : Data,Ack (for the client use),
//                              control (for internal use of TestTalk)
// The client can fetch/wait message on each of its queues.
class ClientC {

    private SynchronizedQueue DataQ = new SynchronizedQueue();
    private SynchronizedQueue AckQ = new SynchronizedQueue();
    private SynchronizedQueue ControlQ = new SynchronizedQueue();

    // method for internal use. Used to write message in client's queue
    protected void PutMessageInClientQueue(
                            byte     QueueType,
                            MessageC Message)
                                throws WrongQueueTypeException {
        switch(QueueType) {
            case TestTalkConstants.DATAQ :
                DataQ.Put(Message);
                break;
            case TestTalkConstants.ACKQ :
                AckQ.Put(Message);
                break;
            case TestTalkConstants.CONTROLQ :
                ControlQ.Put(Message);
                break;
            default:
                throw new WrongQueueTypeException();
        }
    }

    // get message from queue. The get can be done with wait/nowait
    // option, and wait + timeout
    protected MessageC GetMessageFromClientQueue(
                            byte    QueueType,
                            boolean Wait,
                            int     TimeoutInMiliSeconds)
                               throws TimeoutExpiredException,
                                      EmptyQueueException,
                                      WrongQueueTypeException {
      try {

        switch(QueueType) {

            case TestTalkConstants.DATAQ :
                if (Wait) {
                    if (TimeoutInMiliSeconds < 0)
                        return (MessageC)DataQ.GetAndWait();
                    else
                        return (MessageC)DataQ.
                                    GetAndWait(TimeoutInMiliSeconds);
                }
                else
                    return (MessageC)DataQ.Get();

            case TestTalkConstants.ACKQ :
                if (Wait) {
                    if (TimeoutInMiliSeconds < 0)
                        return (MessageC)AckQ.GetAndWait();
                    else
                        return (MessageC)AckQ.
                                    GetAndWait(TimeoutInMiliSeconds);
                }
                else
                    return (MessageC)AckQ.Get();

            case TestTalkConstants.CONTROLQ :
                if (Wait) {
                    if (TimeoutInMiliSeconds < 0)
                        return (MessageC)ControlQ.GetAndWait();
                    else
                        return (MessageC)ControlQ.
                                    GetAndWait(TimeoutInMiliSeconds);
                }
                else
                    return (MessageC)ControlQ.Get();

            default:
                throw new WrongQueueTypeException();
        }
      }
      catch (com.radview.util.datastructures.EmptyQueueException e) {
            throw new com.radview.util.testtalk.EmptyQueueException(
                                                e.getMessage());
      }
      catch (com.radview.util.datastructures.TimeoutExpiredException e) {
            throw new com.radview.util.testtalk.TimeoutExpiredException(
                                                e.getMessage());
      }

    }

    // clear queue
    protected void ClearQueue(byte QueueType)
                                    throws WrongQueueTypeException {
        switch(QueueType) {
            case TestTalkConstants.DATAQ :
                DataQ.clear();
                break;
            case TestTalkConstants.ACKQ :
                AckQ.clear();
                break;
            case TestTalkConstants.CONTROLQ :
                ControlQ.clear();
                break;
            default:
                throw new WrongQueueTypeException();
        }
    }

    // return the queue size
    protected int QueueSize(byte QueueType)
                                    throws WrongQueueTypeException {
        switch(QueueType) {
            case TestTalkConstants.DATAQ :
                return DataQ.Size();
            case TestTalkConstants.ACKQ :
                return AckQ.Size();
            case TestTalkConstants.CONTROLQ :
                return ControlQ.Size();
            default:
                throw new WrongQueueTypeException();
        }

    }

}
