/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
package com.radview.util.testtalk;

import java.net.Socket;
import java.io.DataInputStream;

// radview classes
import com.radview.util.general.DebugMessages;

// class which handles all the messages to be read from different
// TestTalk. There is an object & thread of current class for every
// TestTalk connected to current TestTalk.
class TestTalkReaderC extends Thread {

    private TestTalkToTestTalkC TestTalkToTestTalk;
    private Socket      TargetSocket;
    private boolean     Send;
    private String      TargetComputerName;
    private int         TargetPort;

    private DataInputStream IS;

    // Ctor
    TestTalkReaderC(TestTalkToTestTalkC testtalktotesttalk,
                    String              targetcomputername,
                    int                 targetport,
                    Socket              targetsocket,
                    boolean             send) {

        // On send mode we know the target host & name & target socket.
        // On send = false we know only socket

        TestTalkToTestTalk = testtalktotesttalk;
        TargetComputerName = targetcomputername;
        TargetPort = targetport;
        TargetSocket = targetsocket;
        Send = send;

    }

    public void inter() {

        setPriority(MAX_PRIORITY);

        // the socket is known in both modes of activation (send and
        // not send). Thus, create input stream.
        try {
            IS = new DataInputStream(
                            TargetSocket.getInputStream());
        }
        catch (Exception e) {  //Throwable

            if (Send) {

                DebugMessages.WriteMessage(
                    "TestTalk : Error occured when creating input stream " +
                    "to socket of host " + TargetComputerName +
                    ":" + TargetPort + ", " + e);

                TestTalkToTestTalk.UnregisterTestTalk(
                                    TargetComputerName,TargetPort);
                try {
                    if (TargetSocket != null) TargetSocket.close();
                }
                catch (Exception er) {}  //Throwable
                // on send, the writer is activated the reader. Thus,
                // has to kill the writer.
                MessageC KillMessage = new MessageC();
                KillMessage.Type = TestTalkConstants.KILL_YOURSELF;
                KillMessage.Data =
                    "Error occured when creating input stream " +
                    "to socket of host " + TargetComputerName +
                    ":" + TargetPort + ", " + e;
                TestTalkToTestTalk.Send(KillMessage);
            }
            else {
                DebugMessages.WriteMessage(
                    "TestTalk : Error occured when creating input stream " +
                    " to socket, " + e);

                try {
                    if (TargetSocket != null) TargetSocket.close();
                }
                catch (Exception er) {}  //Throwable
            }
            stop(); // deprecated

        }
        if (!Send) {

            // get special message which contains the host name & port.
            // We still don't know these values. They are passed by
            // the first & special message.

            if (TestTalkConstants.DEBUG_MODE) {
                DebugMessages.WriteMessage("TestTalk : Connection is created by other host");
            }

            PackedBufC PackedBuf = new PackedBufC();
            try {
                PackedBuf.ReadFromInputStream(IS);
            }
            catch (Exception e) {  //Throwable
                DebugMessages.WriteMessage(
                    "TestTalk : Error occured when creating input stream " +
                    " to socket, " + e);

                try {
                    if (TargetSocket != null) TargetSocket.close();
                }
                catch (Exception er) {}  //Throwable
                stop(); // deprecated
            }

            PackedBuf.Data = (PackedBuf.Data).substring(0,(PackedBuf.Data).length()-1);
            int SeparatorIndex = (PackedBuf.Data).indexOf(":");
            if (SeparatorIndex != -1) {
                TargetComputerName = (PackedBuf.Data).
                                    substring(0,SeparatorIndex);
                TargetPort = Integer.parseInt((PackedBuf.Data).
                                    substring(SeparatorIndex+1));
            }
            else {
                TargetComputerName = PackedBuf.Data;
                TargetPort = TestTalkConstants.DEFAULT_PORT;
            }

            if (TestTalkConstants.DEBUG_MODE) {
                DebugMessages.WriteMessage(
                    "TestTalk : Connection is created by : " +
                    TargetComputerName + " " + TargetPort);
            }

            // register the host as connected host
            TestTalkToTestTalk.RegisterTestTalk(
                                    TargetComputerName,TargetPort);

            // on this mode, after the first message has received, we
            // already knows the host name & port & the socket. So,
            // when activating the writer we know everything.
            TestTalkToTestTalk.CreateWriter(
                                    TargetComputerName,
                                    TargetPort,
                                    TargetSocket);

        }

        MessageC Message;
        PackedBufC PackedBuf = new PackedBufC();
        MessageC ControlMessage = new MessageC();
        ClientC Client;

        while (true) {

            // read message from socket

            try {
                // read the message and convert to message object
                PackedBuf.ReadFromInputStream(IS);
                Message = new MessageC();
                Message.FromPackedBuf(PackedBuf);

                // arrange the control Message to be sent back
                ControlMessage.FromHost = Message.ToHost;
                ControlMessage.FromClient = Message.ToClient;
                ControlMessage.ToHost = Message.FromHost;
                ControlMessage.ToClient = Message.FromClient;
                ControlMessage.QueueType = TestTalkConstants.CONTROLQ;
                ControlMessage.SubType = TestTalkConstants.VERSION;

                // search for the target client
                Client = TestTalkToTestTalk.ClientObjectOf(
                                                Message.ToClient);
                if (Client == null) {
                    // when client not found send failure message
                    ControlMessage.Type = TestTalkConstants.SEND_FAIL;
                    ControlMessage.Data =
                        "Client " + Message.ToClient +
                        " does not exist in host " + Message.ToHost;
                    TestTalkToTestTalk.Send(ControlMessage);
                }
                else {
                    // send success
                    if (Message.QueueType != TestTalkConstants.CONTROLQ) {
                        // success message is sent only for real
                        // messages and not for control messages.
                        ControlMessage.Type = TestTalkConstants.SEND_SUCCESS;
                        ControlMessage.Data =
                            "Successfully added to client queue";
                        TestTalkToTestTalk.SendUrgent(ControlMessage);
                    }
                    // put message in client queue
                    try {
                        Client.PutMessageInClientQueue(
                                        Message.QueueType,Message);
                    }
                    catch (WrongQueueTypeException e) {}
                }
            }
            catch (Exception e) {  //Throwable
                DebugMessages.WriteMessage(
                    "TestTalk : Error occured when reading from stream " +
                    "of host " + TargetComputerName + ":" +
                    TargetPort + ", " + e);

                TestTalkToTestTalk.UnregisterTestTalk(
                                        TargetComputerName,TargetPort);
                try {
                    if (TargetSocket != null) TargetSocket.close();
                }
                catch (Exception er) {}  //Throwable
                // send kill message to the writer
                MessageC KillMessage = new MessageC();
                KillMessage.Type = TestTalkConstants.KILL_YOURSELF;
                KillMessage.Data =
                    "Error occured when reading from stream " +
                    "of host " + TargetComputerName + ":" +
                    TargetPort + ", " + e;
                TestTalkToTestTalk.Send(KillMessage);
                stop(); // deprecated
            }
        }
    }

    public void run() {
        inter();
    }
}
