/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
package com.radview.util.testtalk;

import java.net.Socket;
import java.io.BufferedOutputStream;

// radview classes
import com.radview.util.datastructures.SynchronizedQueue;
import com.radview.util.general.DebugMessages;

// class which handles all the messages to be written to different
// TestTalk. There is an object & thread of current class for every
// TestTalk connected to current TestTalk.
class TestTalkWriterC extends Thread {

    private SynchronizedQueue WriterQueue = new SynchronizedQueue();
                                        // Queue of all messages
                                        // waiting to be sent.

    private TestTalkToTestTalkC TestTalkToTestTalk;
    private int         TargetPort;
    private boolean     Send;
    private String      TargetComputerName;
    private Socket      TargetSocket;

    private BufferedOutputStream OS;

    // Ctor
    TestTalkWriterC(TestTalkToTestTalkC testtalktotesttalk,
                    String              targetcomputername,
                    int                 targetport,
                    Socket              targetsocket,
                    boolean             send) {

        TestTalkToTestTalk = testtalktotesttalk;
        TargetComputerName = targetcomputername;
        TargetPort = targetport;
        TargetSocket = targetsocket;
        Send = send;

    }

    // put message to be sent in queue
    protected void PutMessageInWriterQueue(MessageC Message) {
        WriterQueue.Put(Message);
    }
    protected void PutMessageInWriterQueueOnTop(MessageC Message) {
        WriterQueue.PutOnTop(Message);
    }

    // on error in channel has to go over all waiting to be sent
    // message and send fail message
    private void SendFailMessagesToAllSenders(String DataMessage) {

        MessageC Message;

        try {
            while (true) {

                // fetch message
                Message = (MessageC)WriterQueue.Get();

                // need to send fail just to real messages
                if (Message.QueueType != TestTalkConstants.CONTROLQ) {
                    // create message object
                    MessageC ControlMessage = new MessageC();

                    // fill fail control message
                    ControlMessage.FromHost = Message.ToHost;
                    ControlMessage.FromClient = Message.ToClient;
                    ControlMessage.ToHost = Message.FromHost;
                    ControlMessage.ToClient = Message.FromClient;
                    ControlMessage.QueueType = TestTalkConstants.CONTROLQ;
                    ControlMessage.SubType = TestTalkConstants.VERSION;
                    ControlMessage.Type = TestTalkConstants.SEND_FAIL;
                    ControlMessage.Data = DataMessage;

                    // put message in client's queue
                    try {
                        (TestTalkToTestTalk.ClientObjectOf(
                                ControlMessage.ToClient)).
                            PutMessageInClientQueue(TestTalkConstants.CONTROLQ,
                                                    ControlMessage);
                    }
                    catch (Exception er) {}  //Throwable

                }
            }
        }
        catch (com.radview.util.datastructures.EmptyQueueException er) {}

    }

    public void inter() {

        setPriority(MAX_PRIORITY);

        if (Send) {
            // the writer thread is activated in send mode. Thus,
            // we know the target host name and target host port.
            // We still don't know the socket. It has not been
            // created yet.

            if (TestTalkConstants.DEBUG_MODE) {
                DebugMessages.WriteMessage(
                    "TestTalk : Create connection to : " +
                    TargetComputerName + " " + TargetPort);
            }

            // create socket
            try {
                TargetSocket = new Socket(TargetComputerName,TargetPort);
            }
            catch (Exception e) {  //Throwable
                DebugMessages.WriteMessage(
                    "TestTalk : could not create connection to host " +
                    TargetComputerName + ":" + TargetPort +
                    ", " + e);

                TestTalkToTestTalk.UnregisterTestTalk(
                                TargetComputerName,TargetPort);
                // on error send fail messages to all waiting clients
                SendFailMessagesToAllSenders(
                    "could not create connection to host " +
                    TargetComputerName + " at port " + TargetPort +
                    ", " + e);
                try {
                    if (TargetSocket != null) TargetSocket.close();
                }
                catch (Exception er) {}  //Throwable
                stop(); // deprecated
            }

            // create output stream
            try {
                if (TargetSocket != null)
                    OS = new BufferedOutputStream(
                                TargetSocket.getOutputStream());
            }
            catch (Exception e) {  //Throwable
                DebugMessages.WriteMessage(
                    "TestTalk : Error occured when creating output stream " +
                    "to socket of host " + TargetComputerName +
                    ":" + TargetPort +", " + e);

                TestTalkToTestTalk.UnregisterTestTalk(
                                    TargetComputerName,TargetPort);
                SendFailMessagesToAllSenders(
                    "Error occured when creating output stream " +
                    "to socket of host " + TargetComputerName +
                    ":" + TargetPort +", " + e);
                try {
                    if (TargetSocket != null) TargetSocket.close();
                }
                catch (Exception er) {}  //Throwable
                stop(); // deprecated
            }

            // send special message to the computer to identify my host
            // and my port
            PackedBufC PackedBuf = new PackedBufC();

            PackedBuf.Type = TestTalkConstants.NEW_CONNECTION;
            PackedBuf.SubType = TestTalkConstants.VERSION;
            PackedBuf.Data = TestTalkToTestTalk.NetLocalHost() + ":" +
                             TestTalkToTestTalk.NetLocalPort() +
                             TestTalkConstants.C_NULL_TERMINATOR;

            try {
                PackedBuf.WriteToOutputStream(OS);
            }
            catch (Exception e) {  //Throwable
                DebugMessages.WriteMessage(
                    "TestTalk : Error occured when writing " +
                    "to socket of host " + TargetComputerName +
                    ":" + TargetPort +", " + e);

                TestTalkToTestTalk.UnregisterTestTalk(
                                    TargetComputerName,TargetPort);
                SendFailMessagesToAllSenders(
                    "Error occured when creating output stream " +
                    "to socket of host " + TargetComputerName +
                    ":" + TargetPort +", " + e);
                try {
                    if (TargetSocket != null) TargetSocket.close();
                }
                catch (Exception er) {}  //Throwable
                stop(); // deprecated
            }

            // create reader thread
            TestTalkToTestTalk.CreateReaderInSendOption(
                                                TargetComputerName,
                                                TargetPort,
                                                TargetSocket);

        }
        else {
            // the writer thread is not activated in send mode. It
            // means that current host received message which activated
            // the reader, and the writer is activated by the reader.
            // On that case we know the target host & port and also
            // the socket. (The socket has been created earlier by
            // the reader)

            // create output stream
            try {
                OS = new BufferedOutputStream(
                                TargetSocket.getOutputStream());
            }
            catch (Exception e) {  //Throwable
                DebugMessages.WriteMessage(
                    "TestTalk : Error occured when creating output stream " +
                    "to socket of host " + TargetComputerName +
                    ":" + TargetPort + ", " + e);

                TestTalkToTestTalk.UnregisterTestTalk(
                                    TargetComputerName,TargetPort);
                SendFailMessagesToAllSenders(
                    "Error occured when creating output stream " +
                    "to socket of host " + TargetComputerName +
                    ":" + TargetPort + ", " + e);
                try {
                    if (TargetSocket != null) TargetSocket.close();
                }
                catch (Exception er) {}  //Throwable
                stop(); // deprecated
            }
        }

        // infinite loop for messages
        MessageC Message;
        while (true) {

            // wait for message in queue
            Message = (MessageC)WriterQueue.GetAndWait();
            if (Message.Type == TestTalkConstants.KILL_YOURSELF) {
                // this message is send only by the reader when it gets
                // error in the socket it kills the writer and kills
                // itself. It send alos the error message in the data.
                // Thus, the writer send the error message to all
                // senders in the data field.
                SendFailMessagesToAllSenders(
                    "Connection Lost, " + Message.Data);
                stop(); // deprecated
                // no need to unregister host because it is done by
                // the reader
            }

            // send message
            try {
                (Message.ToPackedBuf()).WriteToOutputStream(OS);
            }
            catch (Exception e) {  //Throwable

                DebugMessages.WriteMessage(
                    "TestTalk : Cannot write to output stream of host " +
                    TargetComputerName + ":" + TargetPort +
                    ", " + e);

                TestTalkToTestTalk.UnregisterTestTalk(
                                    TargetComputerName,TargetPort);

                // put the message back in queue
                WriterQueue.Put(Message);

                // send fail messages to all waiting clients
                SendFailMessagesToAllSenders("Connection Lost" +
                ", " + e);

                try {
                    if (TargetSocket != null) TargetSocket.close();
                }
                catch (Exception er) {}  //Throwable
                stop();

                // when the writer stops it doesn't try to kill
                // the reader. I assume that the reader will find
                // the error by itself.
            }
        }
    }
    public void run() {
        inter();
    }
}
