/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_DOM_ITEM_H__
#define __WL_DOM_ITEM_H__


#include <RVCString.h>
#include <ordered.h>
#include <collectable.h>
#include <collstr.h>
#include <hashdict.h>
#include <collint.h>

#include <platform.h>

#define STORAGE_LIMIT 4096

class DllExport CWLItemFrame : public RVCollectable
{
public:
  RVCString           frameSource;   //! the frame's source
  RVCollectableString frameUrl;      //! the frame's URL
  RVCollectableString frameLocation; //! the frame's URL

public:
  CWLItemFrame(const char *source, const char* url, const char *loc) :
      frameSource(source) , frameUrl(url) , frameLocation(loc)
      {}

  virtual ~CWLItemFrame() {}
  const char *getSource() { return frameSource.data(); }
  const char *getUrl() { return frameUrl.data(); }
  const char *getLocation() { return frameLocation.data(); }
};


class CWLDomRepository;

class DllExport CWLDomItem : public RVCollectable
{
private:
  RVCollectableString *root;    //! will hold the name of the root
  RVHashDictionary  byUrl;      //! indexing by URL
  RVHashDictionary  byLocation; //! indexing by Location

public:
  CWLDomItem() : root(NULL) {}
	virtual ~CWLDomItem();

	void add(const char *url,
           const char *location,
           const char *data,
           bool isRoot = false);

  CWLItemFrame *get(const char *url, const char *location);
  const char   *getRootUrl();

	void dump(CWLDomRepository& rep );

	void unDump(CWLDomRepository& rep);

  void printAll();

};


class DllExport CWLDomRepository
{
private:
	RVCString dumpedStorage; //! a temporary storage while saving.
	int storagePointer;

private:
  bool testFill(int size);

//! a set of routined to aid serialize an item into/from 'dumpedStorage'.
public:
	virtual void dumpInt32(unsigned int i32);
	virtual void dumpBuffer(const char *data);
	virtual unsigned int unDumpInt32();
	virtual const char *unDumpBuffer();
	virtual void initDump() { storagePointer = 0; dumpedStorage = "";}
	virtual void initUnDump() { storagePointer = 0; }
	virtual const char *storage() { return dumpedStorage.data(); }
	virtual size_t buffLength() { return dumpedStorage.length();}
  virtual void fillStorage(const char *data);

public:
	CWLDomRepository() {}
	virtual ~CWLDomRepository() {}

	virtual int add(CWLDomItem *item,int domId = -1) = 0;
	virtual CWLDomItem *get(int itemId) = 0;
  virtual bool fill() = 0;
  virtual void flush() = 0;
	virtual bool save(const char *name) = 0;
	virtual bool open(const char *name) = 0;
  virtual int  length() = 0;
  virtual void printAll() {};
  virtual RVHashDictionary* GetItemHashDictionary()=0;

};





class DllExport CWLExpectedDomRepository : public CWLDomRepository
{
private:
	//RVOrdered  itemList;
	RVHashDictionary itemList;
 	FILE *fp;
	bool m_bNewVersion;


public:
	CWLExpectedDomRepository();
	virtual ~CWLExpectedDomRepository()
	{
		itemList.clearAndDestroy();
	}

	virtual int add(CWLDomItem *item,int domId = -1);
	virtual CWLDomItem *get(int itemId);
	virtual bool replace(int itemId, CWLDomItem *item);
	virtual bool save(const char *name);
	virtual bool open(const char *name);
  virtual bool fill();
	bool fillVersion();
  virtual void flush();
  virtual void printAll();
  virtual int length() { return itemList.entries(); }
  virtual RVHashDictionary* GetItemHashDictionary(){ return &itemList; }

};




#endif

