/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_DOCUMENT__
#define __WL_DOCUMENT__

#include <RVCString.h>
#include <hashdict.h>
#include <thr.h>
#include "wlsrc.h"
#include "wllocation.h"
#include "wlurlarray.h"
#include "wlurlbase.h"
#include "wlheaders.h"
#include "wlframelist.h"
//#include "wlformlist.h"
#include "wlhtmlfuncs.h"
#include "platform.h"
//#include "wltable.h"
#include "wlparseflags.h"
#include "wlmeta.h"
#include "wlIdentLocation.h"



class frameContainer;
class parseFrame;
class CWLHtmlVirtualQueue;
class CWLXmlDocument;
class formList;
class CWLTableCollection;
class CWLLink;

enum findFrameInDocStatus {
  findFrameInDocStatus_OK,
  findFrameInDocStatus_ERROR,
  findFrameInDocStatus_NAME_MATCH_NUMBER_MISMATCH_WARNING,
  findFrameinDocStatus_NUMBER_MATCH_NAME_MISMATCH_WARNING
};

//========================CWLDOCUMENT================================
class DllExport CWLDocument : public RVCollectable {
public:
	bool		m_ParseInnerHTML;
	CWLLink*	m_CurLink;
private:
	CWLIdentLocation m_FrameTarget;
	CWLLocation  m_Location;
//	CWLLocation  m_Referrer;
	RVCString    m_Name;
	RVCString    m_Id;
	RVCString    m_Target;
	RVCString		 m_FrameName;
	int					 m_BufFlag;
	bool         m_FrameValid;

  WLLogMessage::Severity m_MaxSeverity;

  bool         m_DirectXmlParsing;

RVCriticalSection*     m_RefCountMutex;
private:
	unsigned int	m_CloneRefCount;
	RVCString		buff;
	RVCString		buffAdd;
private:
	size_t			buffPtr;
	readStateE	readState;
	bool				titleFound;
	bool				baseSet;
  //! these come from the HTTP Get/Post operation
	CWLHeaders  m_Header;
	unsigned    m_StatusNumber;
	unsigned		m_PortNum;
	RVCString   m_StatusLine;
	RVCString   m_ProtocolVersion;
	RVCString		m_Title;
	//! or use a CWLReply instance

	// zion
	frameContainer* frameCont;
protected:
	RVCString    m_DocUrl;
	RVCString    m_OriginalUrl;
	RVCString    m_Referrer;
	long	     m_Index;
	CWLDocument* m_DocPtr;
	urlBase	     m_UrlBase;
	bool	     m_DontParse;
        bool         m_ConvertToUTF8;
public:
	CWLHtmlVirtualQueue*		m_QueuePtr;
	int         m_iBuffPos;

public:
	parseFrame*			m_Frames;	// list of CWLDocument
	CWLLinkArray*		m_Links;
	formList*			m_Forms;
	parseSrc*			m_SrcList;
	parseSrc*			m_EmbedList;
	RVOrdered*			m_AppletList;
	CWLCSSArray*		m_StyleLinks;
	RVOrdered*			m_StyleSheets;
	RVCString*			m_HtmlPagePtr;
	RVOrdered*			m_Scripts;
	RVOrdered*			m_Xmls;
	CWLTableCollection* m_Tables;
	CWLMetaCollection*   m_MetaTags;
	RVOrdered*			m_wlAll;

private:
	void parseAll();
	void append(const char* newData, size_t size);
	void addNewScriptElementToArray(RVCString& tempBuff, size_t scriptEndIndex);
	void addNewStyleElementToArray(RVCString& tempBuff, size_t styleEndIndex);
	elementReadState extractBodyAttributes(RVCString& tempBuff);
	elementReadState extractTitle(RVCString& tempBuff);
	elementReadState extractBase(RVCString& tempBuff);
	elementReadState extractHref(RVCString& tempBuff, anchorType type);
	elementReadState extractFrame(RVCString& tempBuff);
	elementReadState extractForm(RVCString& tempBuff);
	elementReadState extractImg(RVCString& tempBuff);
	elementReadState extractEmbed(RVCString& tempBuff);
	elementReadState extractBgsound(RVCString& tempBuff);
	elementReadState extractLink(RVCString& tempBuff);
	elementReadState extractTable(RVCString& tempBuff);
	void parseSession();

public:
	CWLDocument();
	CWLDocument(	RVCString& name, RVCString& frameName, RVCString& url, RVCString& origUrl,
					const char* id,  const char* target, CWLUserParseFlags&  parseFlags,
					frameContainer*	container, CWLDocument* docPtr
				);
	CWLDocument& operator=( CWLDocument& aDocument) ;
	virtual ~CWLDocument();
	virtual void	parse(const char* _newData, size_t size, bool* isDone = NULL);
	void SaveSource(const char* newData, size_t size);
	elementReadState addToXmls(RVCString& tempBuff);
	elementReadState addToScripts(RVCString& tempBuff);
	elementReadState addToStyles(RVCString& tempBuff);
	elementReadState addToApplets(RVCString& tempBuff, const char *tagName);
	void	redirectionHandler(RVCString& newUrl);
	void	insertUri(RVCString& uri);
	void	insertHeader(const char* name, const char* value);
	void	insertHost(RVCString& name, unsigned port);
	void	incCloneRefCount(){++m_CloneRefCount;}
	void	decCloneRefCount(){--m_CloneRefCount;}
	void	insertStatus(unsigned value, int n, RVCString& line);
	int		pageTreeSearch(int typeOfSearch, int formIndex,
											const char* pat, const char* fn, void** retVal);
	unsigned int	getRefCount();
	unsigned int	incRefCount();
	unsigned int	decRefCount();

	frameContainer* GetFrameContainer(){return frameCont;}

  CWLLocation&	GetLocation()       { return  m_Location;	}
	CWLIdentLocation& GetFrameTarget(){ return m_FrameTarget; }
  void SetFrameTarget(const CWLIdentLocation& frameTarget) { m_FrameTarget = frameTarget; }
	virtual RVCString&		GetDocUrl()	{return  m_DocUrl;}
	RVCString&		GetName()						{return  m_Name;}
	RVCString&		GetFrameName()			{return  m_FrameName;}
	urlBase*			GetBase()						{return  &m_UrlBase;}
//	CWLLocation&	GetReferrer()				{return  m_Referrer;}
	CWLHeaders&		GetHeader()					{return  m_Header;}
	RVCString&		GetProtocolVersion(){return  m_ProtocolVersion;}
	void				SetDocUrl(const char *url);
	void				SetBase(RVCString& url);
	void        SetInvalidFrame() { m_FrameValid = false; }
	const char*	Id();
	const char*	Target();
	bool        IsValid() { return m_FrameValid; }
	void				SetId(const char* id);
	const char*	GetTitle(){return m_Title.data();}
	int					GetFrameByIndex(RVCString&, int, CWLDocument** );
	int					GetFrameByUrl(const char* , CWLDocument** );
	findFrameInDocStatus GetFrameByLocation(const RVCString& identLocationStr , CWLDocument*& frame);
	int					GetFrameByFrameName(const char*, CWLDocument** );
	int					GetHrefByIndex(int , CWLLink** );
	int					GetHrefByName(const char* , CWLLink** );
	int					GetHrefByUrl(const char* , CWLLink** );
	int					GetFormAction(int , CWLLocation** )  ;
	int					GetFieldValueByPage(const char*, const char**);
	int					GetFieldValueByForm(int,const char*, const char** );
	int					GetHeaderValue(const char* name, const char*& value);
	int					GetAllHeaderValues(const char* name, const char*& values);
	int					GetUri(const char*&);
	int					GetStatusLine(const char*&);
	int					GetHost(const char*&);
	int					GetHostName(const char*&);
	unsigned		GetPortNum();
	unsigned		GetStatusNumber();
	const char* GetStatusLine(){return m_StatusLine.data();}
	CWLXmlDocument*			Xml(const char* xmlId);
	virtual bool			isEqual(const RVCollectable* c)const {return (this==c);}
	virtual int compareTo(const RVCollectable* a) const{return 0;};

  void StartXmlDirectParsing() ;
  void EndXmlDirectParsing() ;

  RVCString& GetOriginalUrl(){ return m_OriginalUrl; }
  RVCString& GetReferrer()	 { return m_Referrer; }
  long       GetIndex()      { return m_Index; }
  void       SetIndex(long p_i)       { m_Index = p_i; }

protected:
    CWLTransactionsObject m_TransObj;
	CWLTransactionsObject *m_CurrentTransObj;
public:
	CWLTransactionsObject* GetTransactionObj(){return &m_TransObj;}
	CWLTransactionsObject* GetFirstTransactionObj()
		{ m_CurrentTransObj = &m_TransObj; return m_CurrentTransObj; }
	CWLTransactionsObject* GetNextTransactionObj()
		{ m_CurrentTransObj = m_CurrentTransObj->next(); return m_CurrentTransObj;}
	void insertTransObjToQueue(CWLTransactionsObject *tObj);

  WLLogMessage::Severity GetSeverity();
  WLLogMessage::Severity SetSeverity(WLLogMessage::Severity severity);

private:
 	RVCString		m_TruncatedBytes;

  // @@Chaim - needed?
 	int					m_CharMode;

protected:
  int m_iUrlOffset;
	long m_Charset;
	bool m_UserDefinedCharset;
	bool m_FoundBodyTag;
public:
  void setUrlOffset(int uo) { m_iUrlOffset = uo; }
  int  getUrlOffset() { return m_iUrlOffset; }
	void SetCharset(long charset, bool userdefined);
	long GetCharset();
	bool GetSCharset(char* pcStr);
};

#endif
