/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <slistcol.h>                              //! 1
#include <thr.h>
#include <thr.h>

class CWLQueue {
   private:
      RVMutexLock								m_Mutex;                                //! 1
      RVCondition								m_NotEmpty;                             //! 2
//    RVCondition								notFull;                              // :
      RVSlistCollectablesQueue  m_Queue;                                //! 3
			size_t										maxSize;//! get rid of ...
   public:
      CWLFrameQueue(size_t size=0)                                 //! 4
         :maxSize(size), notEmpty(m_Mutex), notFull(m_Mutex)														// 5
      {
      }
      RVCollectable* read()
      {
         RVMutexLock::LockGuard guard(m_Mutex);                //! 6
         while(m_Queue.isEmpty()) {                            //! 7
            m_NotEmpty.wait();                                 //! 8
         }
         result = m_Queue.get();
//         notFull.signal();                                   //! 9
         return result;
      }
      void insert(RVCollectable* item)
      {
         RVMutexLock::LockGuard guard(m_Mutex); 
//       while(maxSize != 0 && queue.entries() < maxSize) { // 10
//          notFull.wait();                                 // 11
         }
         m_Queue.append(item);
         m_NotEmpty.signal();                                 //! 12
      }
			void clearAndDestroy()
			{
				m_Queue.clearAndDestroy();
			}
};
/*!
 //1 Declare a mutex member that we can use to provide mutually exclusive access to the internal linked-list. 
 //2 Declare two condition-variables to represent the two queue conditions of interest; a queue that is not empty, and a queue that is not
 //     full. 
 //3
 //     Declare a linked-list collection to use as the internal implementation of the actual queue. 
 //4
 //     The PCQueue constructor accepts a maximum size argument; this size is used to limit the growth of the queue, if desired. A value of
 //    zero implies that there is no preset limit to the number of entries that may be written. 
 //5
 //     The mutex must be passed to the member initializers for the RVCondition instances so they will know what mutex to release and
 //    reacquire during a wait() operation. 
 //6
 //     Acquire the mutex to prohibit other threads from changing the internal linked-list while the list condition is evaluated and the next
 //     available entry is removed. 
 //7
 //     If the list is empty, proceed to the wait operation described in the next comment, otherwise, proceed to step #9 and remove an entry
 //    from the beginning of the list. 
 //8
 //     Call the wait() member on the notEmpty condition-variable; the wait function will unlock the mutex, and begin waiting for another
 //     thread to signal that an entry has been added to the list. 
 //9
 //     Call the signal() member on the notFull condition-variable to inform a waiting writer, if any, that the list is no longer full, since an
 //     entry has just been removed! 
 //10
 //     If the maxSize is zero, indicating that the queue is not size limited, or if the current number of entries in the list is less than the
 //     established maximum size, proceed to step #13 and write the new value at the end of the list. 
 //11
 //     Call the wait() member on the notFull condition-variable; the wait function will unlock the mutex, and begin waiting for another
 //     thread to signal that an entry has been removed from the list. 
 //12
 //     Call the signal() member on the notEmpty condition-variable to inform a waiting writer, if any, that the list can no longer be empty,
 //     since an entry has just been added! 
*/