/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_OPTION__
#define __WL_OPTION__
#include <RVCString.h>
#include <collectable.h>
#include "wldocument.h"

#ifdef __TV_UNIX__
#include <windows.h>
#endif

class CWLOption : public RVCollectable  
{
public:
	CWLDocument* m_DocPtr;

private:
	RVCString m_Text;
	RVCString m_Value;
	RVCString m_Id;
	bool			m_Selected;


public:
	CWLOption() { m_StartInnerHTML = -1; m_LenInnerHTML = -1; m_Selected = false; }
	CWLOption(CWLDocument* docPtr, const char* value, const char* text, const char* id, bool selected)
		: m_DocPtr(docPtr)
	{
		if(value)
			m_Value	 = value;
		if(text)
			m_Text	 = text;
		if(id)
			m_Id = id;
		m_Selected = selected;
		m_StartInnerHTML = -1; 
		m_LenInnerHTML = -1;
	}

	virtual ~CWLOption(){}

public:
	CWLOption& operator=(CWLOption& anOption)
	{
		m_DocPtr        = anOption.m_DocPtr;
		m_Value			= anOption.Value();
		m_Text			= anOption.Text();
		m_Selected	= anOption.Selected();
		m_Id				= anOption.Id();
		m_StartInnerHTML = anOption.m_StartInnerHTML; 
		m_LenInnerHTML = anOption.m_LenInnerHTML;
		return *this;
	}
	const char*	Text()		{return m_Text.data();}
	const char*	Value()		
	{
		return m_Value.data();
	}
	const char* Id(){return m_Id.data();}
	bool				Selected(){return m_Selected;}

private:
	int             m_StartInnerHTML;
	int             m_LenInnerHTML;

public:
	void SetInnerHTMLOffset(int start, int len) { m_StartInnerHTML = start; m_LenInnerHTML = len; }
	void SetInnerHTMLStart(int start) { m_StartInnerHTML = start; }
	void SetInnerHTMLLen(int len)     { m_LenInnerHTML = len; }
	int GetStartInnerHTML() {return m_StartInnerHTML; }
	int GetLenInnerHTML() {return m_LenInnerHTML; }
	RVCSubString GetInnerHTML()
	{ 
		if (m_DocPtr && m_DocPtr->m_HtmlPagePtr) {
			RVCString &temp = *(m_DocPtr->m_HtmlPagePtr);
			return temp(GetStartInnerHTML(), GetLenInnerHTML());
		} else {
			RVCString temp;
			return temp(0,0);
		}
	}
};


#endif
