/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_TABLE__
#define __WL_TABLE__

#include <RVCString.h>
#include <ordered.h>
#include <collectable.h>
#include <platform.h>
#include "wlhtmlfuncs.h"
#include "wlsrc.h"
// #include "wldocument.h"
class CWLDocument;

#define     TD   1
#define     TH   2
#define     TD_NAME "TD"
#define     TH_NAME "TH"

class CWLTable;
class DllExport CWLTableCell : public RVCollectable
{
private:
	int             cellType;
	RVCString       innerHtml;   //! the cell content
	RVCString				innerText;   //! only visiable text content
	RVCString				id;
	int				rowSpan;	 //! Row Span (from the cell tag)
	int				colSpan;	 //! Column Span (from the cell tag)
	int				cellIndex;   //! the ordinal index of the cell in its row
	CWLDocument    *m_DocPtr;

private:
	void readCellAttributes(RVCString& buff, int *index, urlBase *urlBasePtr);

public:
	CWLTableCell(CWLDocument *docPtr, int type, char *content = "", int rspan = 1, int cspan = 1):
	     m_DocPtr(docPtr),
	     cellType(type) , innerHtml(content) , innerText("") , cellIndex(-1) ,
	     rowSpan(rspan) , colSpan(cspan)
		{
//			Html2Text();
		}

	virtual ~CWLTableCell() {}

	int getCellType() const { return cellType; }
	const char *getCellInnerHtml() const  { return (const char *)innerHtml; }
	const char *getCellInnerText() const  { return (const char *)innerText; }
	const char *getCellId() const  { return (const char *)id; }
	int getCellIndex() const { return cellIndex; }
	int getColSpan() const { return colSpan; }
	int getRowSpan() const { return rowSpan; }
	void setCellIndex(int index) { cellIndex = index; }
	void parse(RVCString& buff, int startIndex, int endIndex, urlBase *urlBasePtr);


};

DllExport ostream& operator<<(ostream& o, const CWLTableCell& x);


class DllExport CWLTableRow : public RVOrdered
{
private:
	int             rowIndex;   //! the ordinal index of the row in the table
	int             totalCells; //! the total number of cells in the table
	                            //! up and including this row

public:
	CWLTableRow(int index, int total) : rowIndex(index) , totalCells(total)
		{}
	virtual ~CWLTableRow() { clearAndDestroy(); }

	bool addToRow(CWLTableCell *cell);
	int getTotalCells() const { return totalCells; }
	int getRowSize() const { return entries(); }
	int getRowIndex() const { return rowIndex; }
	CWLTableCell *getRowCell(int position) const;

};

DllExport ostream& operator<<(ostream& o, const CWLTableRow& x);



class DllExport CWLCellCollection : public RVOrdered
{
private:
	CWLTable* m_pOwnerTable;
public:
	//yuval - add pointer to the table that produced the cell collection
	//we need in order to use getCellRange
	CWLCellCollection(CWLTable* ownerTable = NULL):m_pOwnerTable(ownerTable) {}
	virtual ~CWLCellCollection() { clear(); }  //! NO clearAndDestroy!!! holds pointers to existing table

	int length() const { return entries(); }
	CWLTableCell*& operator()(size_t i) { return (CWLTableCell *&)((*this)(i)); }
	CWLTable*& getOwnerTable(){return m_pOwnerTable; }

};

DllExport ostream& operator<<(ostream& o, const CWLCellCollection& x);


class DllExport CWLTable : public RVOrdered
{
private:
	int				cols;	   //! the number of cols taken from <TABLE>
	int             numCells;  //! the number of cells in the table
	RVCString		tableID;   //! Table ID (if exists)

	tableStates		state;	   //! for table parsing
	int				cellStart; //! for table parsing
	CWLTableCell    *cell;     //! the current cell
	RVCString       *outerHtml;//! wil hold the table's HTML (if required)
	CWLDocument		*m_DocPtr;

private:
	tableTagType findTableTag(RVCString& buff, int *index);
	elementReadState readTableAttributes(RVCString& buff, int *index, urlBase *urlBasePtr);

public:
	CWLTable(CWLDocument *docPtr, int n_cols=0);
	CWLTable(CWLDocument *docPtr, char *id,int n_cols=0);
	virtual ~CWLTable();
	bool newRow();
	bool addCell(CWLTableCell *cell);
	CWLTableRow *getTableRow(int position) const;
	CWLTableCell *getTableCell(int rowPosition, int cellInRow) const;
	CWLTableCell *getTableCell(int position) const;
	CWLCellCollection *getCellRange(char *range) const;
	CWLCellCollection *getCells();
	int getNumCells() const { return numCells; }
	int getNumRows() const { return entries(); }
	int getNumCols() const { return cols; }
	const char *getTableID() const;
	const char *getOuterHtml();
	tableParseResults parse(RVCString& buff, int *index, urlBase *urlBasePtr);

};

DllExport ostream& operator<<(ostream& o, const CWLTable& x);



class DllExport CWLTableCollection : public RVOrdered
{
private:
	CWLDocument		   *m_DocPtr;
	RVOrdered			TableStack;
	int					parseIndex;
	tableParseStates	state;

private:
	void clearTableTags(RVCString& buff, int toIndex);

public:
	CWLTableCollection(CWLDocument *docPtr) : m_DocPtr(docPtr), parseIndex(0) , state(START_PARSE)
		{}
	virtual ~CWLTableCollection() { clearAndDestroy(); }
	elementReadState parse(RVCString& buff, urlBase *urlBasePtr);
	CWLTable *findByID(char *id);
};


bool  DllExport translateSpreadtoCoord(char* spreadRange, int *x_1, int *y_1, int *x_2 , int *y_2);

#ifdef _DEBUG
DllExport CWLTable *debugCreateTable(CWLDocument *docPtr, char *name, int col, int rows, int cols);
#endif

#endif

