/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_URL__
#define __WL_URL__

#include <collstr.h>
#include <colclass.h>
#include <ordered.h>
#include <collectable.h>
#include <stdio.h>
#include <ordered.h>
#include <RVCString.h>
#include "wlarray.h"
#include "platform.h"
#include "wlbuffpart.h"
#include "urlcmp.h"
#include "wltransactiondata.h"

class CWLDocument;
	  	

// ==============================CWLURL===========================
class DllExport CWLUrl : public RVCollectable {
public:
	CWLDocument* m_DocPtr;
private:
  CWLBuffPart    m_Protocol;
  CWLBuffPart    m_Hostname;
  CWLBuffPart    m_Host; //! ==hostname:port
  CWLBuffPart    m_Uri;  //! ...host[:port]/uri....->[?,#, end]  
  CWLBuffPart	 m_Path;  //! the path part of m_Uri
  CWLBuffPart	 m_File;  //! The file part of m_Uri
  CWLBuffPart	 m_FileExt;  //! The extenssion part of m_Uri
  CWLBuffPart    m_Search;//! ....?..?->................[
  CWLBuffPart    m_Hash;
  RVCString      m_Href; //! == protocol://host/uri?search#hash
  CWLBuffPart    m_Commands;
  char           *m_Name; //! name = href, no name extraction any more
  unsigned        m_Port;
  char		     *m_Target;
  char		 	 *m_Id;
  //char		     *m_InnerHTML;
  RVCString      *m_InnerText;
  CWLSearchArray  m_Kvps;  //! decoding 
  char           *m_Source;	//! the original string as passed to the constructor
  RVCString		  m_OriginalUrl;
  RVCString		  m_Referrer;
  int             m_StartInnerHTML;
  int             m_LenInnerHTML;

public:
  bool 		  m_UrlAnalyzed;
  bool		  m_SearchAnalyzed;
  bool        m_HostDirty;
  bool        m_HrefDirty;
  bool        m_HrefValid;
  bool        m_PortValid;            

  static char *extenssionCharacters;

public:
  CWLUrl();
  CWLUrl(CWLDocument *docPtr);
  CWLUrl(CWLDocument *docPtr, RVCString& href, RVCString& origUrl, RVCString& referrer, const char* name, const char* id, const char* target);
  virtual ~CWLUrl();   
  CWLUrl(const CWLUrl&); 
  CWLUrl& operator=(const CWLUrl&) ;

  bool operator==(CWLUrl& aUrl) const ;
  //! get & set functions
  const char* Protocol();     
  const char* Hostname() ;    
  const char* Host();         
  const char* Uri();          
  const char* Search();                                         
  const char* Hash();          
  const char* Commands();                                    
  const char* Name();          
  const char* GetName();          
  RVCString&  Href_();                                       
  const char* Href() { return (const char *)(Href_()); }          
  unsigned    Port();          
  bool        Valid();         
  bool        PortValid();     
  const char* Target();
  const char* Id();
//  const char* InnerHTML();
  virtual const char* GetInnerText();
  CWLSearchArray& Array();     
  virtual void		  printAll();
  char*       Source() { return m_Source; }

  void setProtocol(const char* protocol); 
  void setHostname(const char* hostname); 
  void setHost(const char* host);         
  void setUri(const char* uri);           
  void setSearch(const char* search);     
  void setHash(const char* hash);         
  void setHref(const char* href);         
  void setPort(unsigned port);            
  const char* CWLUrl::getOriginalUrl();
  const char* CWLUrl::getReferrer();
  bool isAnalyzed();
  int  analyzeHref();

  bool urlCompare(CWLUrl& aUrl, CWLUrlCmp& res);
  bool propertiesCompare(CWLUrl& aUrl, CWLUrlCmp& res);
  
  //! define this class as a friend in order to allow it
  //! to access private members of another CWLUrl....
  friend bool urlCompare(CWLUrl& aUrl, CWLUrlCmp& res);
  friend bool propertiesCompare(CWLUrl& aUrl, CWLUrlCmp& res);

  RVCString& GetReferrer()    { return m_Referrer; }
  RVCString& GetOriginalUrl() { return m_OriginalUrl; }

  void SetInnerHTMLOffset(int start, int len) { m_StartInnerHTML = start; m_LenInnerHTML = len; }
  int GetStartInnerHTML() {return m_StartInnerHTML; }
  int GetLenInnerHTML() {return m_LenInnerHTML; }
  virtual RVCSubString GetInnerHTML();

private:
  void rebuild();
};

#endif

