/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wldocument.h"
#include "wlhtmlvirtualqueue.h"
#include "wlurlbase.h"
#include <RVCString.h>
#include <hashdict.h>
#include <ordered.h>
#include <collectable.h>
#include <collstr.h>

/*
enum objType{
	PARAMETER_REF,
	ELEMENT,
	ENTITY,
	READ,
	ERR
};
*/

enum entityType{
	ENTITY_EXTERNAL,
	ENTITY_INTERNAL,
	ENTITY_PARAMETER_EXTERNAL,
	ENTITY_PARAMETER_INTERNAL,
	ENTITY_NDATA,
	DOCTYPE_EXTERNAL,
	DOCTYPE_INTERNAL,
	PARAMETER_REFERENCE
};

enum enumParseState{
	PARSE_INIT,
	PARSE_PARAMETER_REF,
	PARSE_ENTITY,
	PARSE_ELEMENT,
	PARSE_ATTLIST,
	PARSE_NOTATION,
	PARSE_IGNORE_INCLUDE,
	PARSE_IGNORE,
	PARSE_INCLUDE,
	PARSE_COMMENT,
	PARSE_ERR,
	PARSE_DONE,
	PARSE_READ_MORE
};

enum enumEntityState{
	PARSE_ENTITY_INIT,
	PARSE_ENTITY_SEARCH_NAME,
	PARSE_ENTITY_SEARCH_EXTERNAL,
	PARSE_ENTITY_PUBLIC,
	PARSE_ENTITY_SYSTEM,
	PARSE_ENTITY_LOCAL,
	PARSE_ENTITY_PARAM,
	PARSE_ENTITY_TO_END,
	PARSE_ENTITY_FINISH
};



class CWLXmlEntityResolver : public CWLDocument{
private:
	RVOrdered							m_ExternalEntitiesAndParameterRefs;//! all external reference objects and parameterRefs
	RVHashDictionary			m_ParameterEntitiesDictionary;//! parameter external reference objects

	RVCString							m_DocTypeErrorString;
	RVCString							m_Data;
	size_t								startEntityPtr, endEntityPtr, ltPtr, gtPtr;
	size_t								totalLength;
	size_t								strPtr;
	int										m_StartLocationParent;
	int										m_EndLocationParent;
	int										m_AddToParentEnd;
	entityType						m_EntityType;//! INTERNAL, EXTERNAL
	enumParseState				parseState;
	enumEntityState				parseEntityState;
	bool									isParamEntityDec;
	bool									isParsingInclude;
	entityType						parseEntityType;
	RVCString							word, url, parameterEntityName;
	RVCString							m_DictName;

public:
	CWLXmlEntityResolver(entityType ent, CWLHtmlVirtualQueue* queue, int start, int end, RVCString* url, RVCString* internalStr, CWLDocument *docPtr, bool p_isDtd=false);
	virtual						~CWLXmlEntityResolver();
	void							parse(const char* data, size_t size, bool* isDone = NULL);
	enumParseState		findNextObject(bool* isDone);
	enumParseState		typeOfObject();
	enumParseState		findIgnoreInclude();
	enumParseState		parameterReferenceHandler();
	enumParseState		entityDeclarationHandler();
	enumParseState		elementAttlistNotationDeclarationHandler();
	enumParseState    ignoreHandler();
	enumParseState		includeHandler();
	enumParseState		commentHandler();
	void							appendStringWithQuotes(RVCString& value, size_t start, size_t end);
	const char*				entityFoldSelf(RVCString& value, RVCString& error);
	size_t						GetStartLocationParent();
	size_t						GetEndLocationParent();
	entityType				GetType();
	const char*				GetData();
	const char*       GetError();
	void							appendToErrorString(char* description, size_t start, size_t length, char* ptr = NULL);
	void							setDictionaryName(const char* name);
	virtual bool	isEqual(const RVCollectable* c) const;
	virtual int				compareTo(const RVCollectable* a) const;
};

