/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlregularexpressions.h"
#include "wlformlist.h"
#include "wlform.h"


formList::formList(CWLDocument* docPtr) : 
	m_DocPtr(docPtr), createNewForm(true)
{
  typeHref  = -1;
  stateFlag = 0;
  buffPtr		=0;
}

formList& formList::operator=(const formList& aformList)
{
	size_t _entries = aformList.entries();

	if(!aformList.isEmpty())
		for(size_t i = 0; i < _entries; ++i)
		{
			CWLForm* newFormPtr =  new CWLForm(m_DocPtr);
			(*newFormPtr) = *(CWLForm*)aformList[i];
			insert(newFormPtr);
		}	
		return (*this);
}



elementReadState formList::parse(RVCString& buff, urlBase* urlBasePtr)
{
	if(createNewForm)
	{	
		formPtr = new CWLForm(m_DocPtr);
		insert(formPtr);
		createNewForm = false;
	}

	switch(formPtr->parse(buff, urlBasePtr))
	{
		case(READ_MORE):
			return READ_MORE;

		case(END_OF_ELEMENT):
			createNewForm = true;
			break;//!switch
		
		default:
			break;
	}
	return END_OF_ELEMENT;
}


CWLForm* formList::Form(const char* formName)
{	
	size_t entries_ = entries();
	if(entries_  < 1)
		return NULL;
	for(size_t formIndex = 0; formIndex < entries_; ++formIndex)
	{
		if( ( ((CWLForm*)at(formIndex))->Name() ) == formName )
			return (CWLForm*)at(formIndex);
	}
	return NULL;
}
		

int formList::findActionByIndex(size_t formIndex, CWLLocation** action)
{	
	size_t ent = entries();
	if(entries() < 1)
		return -3; //!no forms in page
	if(entries() <= formIndex)
		return -4;//!number of forms smaller tham index

	CWLForm* formEntryPtr = (CWLForm*)at(formIndex);  
	CWLLocation* action_ = (formEntryPtr->Action_());
	*action = action_;
	return 0;        	
}


int formList::findEntryByName(const char* name_,const char** valuePtrAdr )
{
  int errCode;
  size_t _entries = entries();

  if(_entries < 1)
    return -4; //!no forms in page
  
  for(unsigned int i = 0; i < _entries; ++i)
  {
    CWLForm* formEntryPtr = (CWLForm*)at(i);
    if(((errCode = formEntryPtr->findEntryByName(name_, *valuePtrAdr))) == 0)
      return 0;  
  }
  
  return errCode;//!name pattern does not exist in page
}


int formList::findEntryByIndexAndName(size_t index_, const char* name_,const char** valuePtrAdr)
{
  int errCode;
  if(entries() < 1)
    return -4; //!no forms in page
  
  if (entries() <= index_)
    return -3;//!number of forms in page smaller than index

  CWLForm* formEntryPtr = (CWLForm*)at(index_);
  
  if(((errCode = formEntryPtr->findEntryByName(name_, *valuePtrAdr ))) == 0)
    return 0;      
  
  return errCode;//!no name match
}

RVCollectable *formList::comparedObject(int index)
{
	  return ((CWLForm *)at(index))->Action_();
}

void formList::printAll()
{
  for ( unsigned int i = 0; i < entries(); i++)  {
    CWLForm* pfrm = (CWLForm*)at(i);
    cout << "Form No. " << i + 1 << endl;
    cout << "=========================" << endl;
    RVOrderedIterator iter(pfrm->Elements());
    while (iter()) {
			CWLElement* elePtr = (CWLElement*)iter.key();
      cout << "Name = " <<  elePtr->Name() << "  ---  "
           << "Value = " << elePtr->Value() << endl;
    }
    cout << endl;
  }      
}
