/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlregularexpressions.h"
#include "wldocument.h"
#include <string.h>
#ifdef _DEBUG
//#include <iostream.h>
#endif
#include "tracedll.h"
#include "wllocation.h"
#include "wlhtmlvirtualqueue.h"
#include "wlframecontainer.h"
#include "wlscript.h"
#include "wlstyle.h"
#include "wltable.h"

//Galtech
#include "gt_unicode.h"

#ifdef __TV_UNIX__
#include <ctype.h>
#endif

class CWLXmlDocument;

extern void addNewXmlElementToArray(RVCString& tempBuff, size_t xmlEndIndex, CWLDocument *docPtr);
extern void transformScriptToXml(CWLScript* script, CWLDocument *docPtr);
extern CWLXmlDocument* GetXmlById(const char* xmlId, RVOrdered* Xmls);
const RVSize_T BIGSIZE = 50000;

//some helper functions - oooo
static void SkipSpaces(char*& str)
{
	while (*str && (*str == ' ' || *str == '\t'))
		str++;
}

static void GetValue(char* Value, char*&str)
{
	while (*str && !isspace(*str))
		*Value++ = *str++;
	*Value = '\0';
}

CWLUserParseFlags& CWLUserParseFlags::operator=(CWLUserParseFlags& aCWLUserParseFlags)
{
	m_Parse					= aCWLUserParseFlags.m_Parse;
	m_ParseContents = aCWLUserParseFlags.m_ParseContents;
	m_ParseLinks		= aCWLUserParseFlags.m_ParseLinks;
	m_ParseForms		= aCWLUserParseFlags.m_ParseForms;
	m_SaveSource		= aCWLUserParseFlags.m_SaveSource;
	m_ParseTables		= aCWLUserParseFlags.m_ParseTables;
	m_ParseXml 			= aCWLUserParseFlags.m_ParseXml;
	m_ParseScripts	= aCWLUserParseFlags.m_ParseScripts;
	m_XmlStandAlone	= aCWLUserParseFlags.m_XmlStandAlone;
	m_ParseImages   = aCWLUserParseFlags.m_ParseImages;
	m_ParseEmbeds   = aCWLUserParseFlags.m_ParseEmbeds;
	m_ParseApplets  = aCWLUserParseFlags.m_ParseApplets;
	m_ParseCss = aCWLUserParseFlags.m_ParseCss;
	m_ParseMetas  = aCWLUserParseFlags.m_ParseMetas;
	m_ParseOthers = aCWLUserParseFlags.m_ParseOthers;
	m_ParseInnerHTML = aCWLUserParseFlags.m_ParseInnerHTML;
	m_ConvertToUTF8 = aCWLUserParseFlags.m_ConvertToUTF8;
	return (*this)	;
}

CWLDocument::CWLDocument(	RVCString& name, RVCString& frameName, RVCString& url,
						    RVCString& origUrl,
						    const char* id,  const char* target,
							CWLUserParseFlags&   parseFlags,
							frameContainer*		 container,
							CWLDocument*         docPtr
						 ):
m_DocPtr(docPtr), m_DocUrl(url), m_OriginalUrl(origUrl), m_Referrer((docPtr?(const char*)docPtr->GetDocUrl():(const char*)"")),
						m_Location(url, origUrl, RVCString((docPtr?(const char*)docPtr->GetDocUrl():(const char*)"")), frameName.data(), id, target),
						m_BufFlag(parseFlags.m_SaveSource), m_TransObj(NULL), m_ConvertToUTF8(parseFlags.m_ConvertToUTF8)
{
	m_ParseInnerHTML = (m_BufFlag) ? parseFlags.m_ParseInnerHTML : false;
	m_TransObj.SetObj(this);
	m_TransObj.SetObjType(FRAME);
	m_RefCountMutex = new RVCriticalSection;
	frameCont				= container;
	m_CloneRefCount = 0;
	m_FrameName			= frameName;
  m_MetaTags = NULL;
  m_Name				= name;
  m_wlAll               = NULL;
  m_SrcList				= NULL;
  m_EmbedList			= NULL;
  m_AppletList			= NULL;
  m_StyleLinks			= NULL;
  m_StyleSheets			= NULL;
  m_Links					= NULL;
  m_Forms					= NULL;
  m_Frames				= NULL;
	m_Tables        = NULL;
	m_Scripts       = NULL;
	m_Xmls					= NULL;
  m_StatusNumber	= 0;
  m_PortNum				= 0;
	titleFound			= baseSet				= false;
	buffPtr			  	= 0;
	readState				= START_APPEND;
  m_DirectXmlParsing = false ;

  m_UrlBase.setBase(url);//!set the base
	if(id)
		m_Id				= id;
	if(target)
		m_Target		= target;

	if (parseFlags.m_ParseMetas)
		m_MetaTags      = new CWLMetaCollection(this);
	m_Frames		=	new parseFrame(name, parseFlags, frameCont, this);
  if (parseFlags.m_ParseContents) {
    m_SrcList	  =	new parseSrc(this);
	m_EmbedList   =	new parseSrc(this);
	m_AppletList  =	new RVOrdered;
	m_StyleSheets = new RVOrdered;
	m_StyleLinks  = new CWLCSSArray(this);
	m_wlAll       = new RVOrdered;
  } else {
	  if (parseFlags.m_ParseImages)
		m_SrcList	  =	new parseSrc(this);
	  if (parseFlags.m_ParseEmbeds)
		m_EmbedList   =	new parseSrc(this);
	  if (parseFlags.m_ParseApplets)
		m_AppletList  =	new RVOrdered;
	  if (parseFlags.m_ParseCss) {
		m_StyleSheets = new RVOrdered;
		m_StyleLinks  = new CWLCSSArray(this);
	  }
	  if (parseFlags.m_ParseOthers)
		m_wlAll       = new RVOrdered;
  }

  if (parseFlags.m_ParseLinks)
    m_Links		=	new CWLLinkArray(this);
	if(parseFlags.m_ParseTables)
		m_Tables		= new CWLTableCollection(this);
  if (parseFlags.m_ParseForms)
    m_Forms		=	new formList(this);
	if(parseFlags.m_ParseScripts || parseFlags.m_ParseXml)
		m_Scripts		= new RVOrdered;
	if(parseFlags.m_ParseXml)//!xmlFlag)
		m_Xmls			= new RVOrdered;
	if(parseFlags.m_SaveSource)
		m_HtmlPagePtr = new RVCString(BIGSIZE);
	else
		m_HtmlPagePtr = NULL;

  m_TransObj.GetTransaction()->SetName(url);
  m_CurrentTransObj = &m_TransObj;
  if (docPtr) {
	m_Index = docPtr->m_Frames->entries();
    m_QueuePtr = docPtr->m_QueuePtr ;
  } else {
	m_Index = -1;
    m_QueuePtr = container->GetQueuePtr() ;
  }
  if (parseFlags.m_Parse == CWLUserParseFlags::DontParse)
	  m_DontParse = true;
  else
	  m_DontParse = false;

  m_MaxSeverity = WLLogMessage::Success;
  m_iBuffPos = 0;
  m_iUrlOffset = -1;
  m_CurLink = NULL;

  m_Charset = CP_DEFAULT;
  m_UserDefinedCharset = false;
	m_CharMode = 1;
  m_FoundBodyTag = false;
}

CWLDocument::CWLDocument() : m_TransObj(NULL), m_ConvertToUTF8(false)
{
  m_TransObj.SetObj(this);
  m_TransObj.SetObjType(FRAME);
  m_RefCountMutex = NULL;
  m_wlAll               = NULL;
  m_SrcList				= NULL;
  m_EmbedList			= NULL;
  m_AppletList			= NULL;
  m_StyleSheets			= NULL;
  m_StyleLinks			= NULL;
  m_Links					= NULL;
  m_Forms					= NULL;
  m_Frames				= NULL;
	m_Tables        = NULL;
	m_MetaTags      = NULL;
	m_Scripts       = NULL;
	m_Xmls					= NULL;
	m_HtmlPagePtr		= NULL;
	m_StatusNumber = 0;
  m_DirectXmlParsing = false ;
  m_CurrentTransObj = &m_TransObj;
  m_Index = -1;
  m_DontParse = true;
  m_MaxSeverity = WLLogMessage::Success;
  m_iBuffPos = 0;
  m_iUrlOffset = -1;
  m_CurLink = NULL;
  m_Charset = CP_DEFAULT;
  m_UserDefinedCharset = false;
  m_FoundBodyTag = false;
  m_BufFlag = 0;
	m_CharMode = 1;
}


/// BUGGY!!!! Talk with Yuval
CWLDocument& CWLDocument::operator=(CWLDocument& aDocument)
{
	m_FrameTarget   	  = aDocument.GetFrameTarget();
	m_Location					= aDocument.GetLocation();
	m_Name							=	aDocument.GetName();
	m_Referrer					=	aDocument.GetReferrer();
	m_Header						=	aDocument.GetHeader();
	m_ProtocolVersion   =	aDocument.GetProtocolVersion();

	m_FrameValid = true;

	if(m_wlAll)
		m_wlAll       = new RVOrdered;
	if(m_Frames)
		m_Frames			= new parseFrame(this);
	if(m_Links)
		m_Links				= new CWLLinkArray(this);
	if(m_Forms)
		m_Forms				= new formList;
	if(m_SrcList)
		m_SrcList 	  = new parseSrc(this);
	if(m_EmbedList)
		m_EmbedList	  = new parseSrc(this);
	if(m_AppletList)
		m_AppletList  = new RVOrdered;
	if(m_StyleLinks)
		m_StyleLinks  = new CWLCSSArray(this);
	if(m_StyleSheets)
		m_StyleSheets = new RVOrdered;
	if(m_HtmlPagePtr)
		m_HtmlPagePtr = new RVCString((*aDocument.m_HtmlPagePtr).data());
	else
		m_HtmlPagePtr= NULL;




	*m_wlAll     = *aDocument.m_wlAll;
	*m_Frames    = *aDocument.m_Frames;
	*m_Links		 = *aDocument.m_Links;
	*m_Forms		 = *aDocument.m_Forms;
	*m_SrcList	 = *aDocument.m_SrcList;
	*m_EmbedList = *aDocument.m_EmbedList;
	*m_AppletList = *aDocument.m_AppletList;
	*m_StyleSheets = *aDocument.m_StyleSheets;
	*m_StyleLinks  = *aDocument.m_StyleLinks;

  m_TransObj = aDocument.m_TransObj ;
	m_DontParse = aDocument.m_DontParse;
	m_MaxSeverity = aDocument.m_MaxSeverity;
  m_iBuffPos = m_iBuffPos;

  m_Charset = aDocument.m_Charset;
  m_UserDefinedCharset = aDocument.m_UserDefinedCharset;
	m_CharMode = aDocument.m_CharMode;

  m_ConvertToUTF8 = aDocument.m_ConvertToUTF8;

  return (*this)	;
}


CWLDocument::~CWLDocument()
{
//  WlTraceMsg ("Inside ~CWLDocument for : %d ",(int)this);

  if(m_RefCountMutex)
		delete m_RefCountMutex;
  if (m_SrcList)
    delete m_SrcList;
  if (m_EmbedList)
	delete m_EmbedList;
  if (m_wlAll) {
	  if (!m_wlAll->isEmpty())
		  m_wlAll->clearAndDestroy();
	  delete m_wlAll;
  }
  if (m_AppletList) {
	  if (!m_AppletList->isEmpty())
		  m_AppletList->clearAndDestroy();
	  delete m_AppletList;
  }
  if (m_StyleSheets) {
	  if (!m_StyleSheets->isEmpty())
		  m_StyleSheets->clearAndDestroy();
	  delete m_StyleSheets;
  }
  if (m_StyleLinks)
	  delete m_StyleLinks;
  if (m_Links)
    delete m_Links;
  if (m_Forms)
    delete m_Forms;
  if (m_Frames)
    delete m_Frames;
	if(m_HtmlPagePtr)
		delete m_HtmlPagePtr;
	if(m_Scripts)
	{
		m_Scripts->clearAndDestroy();
		delete m_Scripts;
	}
	if(m_Tables)
		delete m_Tables;
	if(m_MetaTags)
		delete m_MetaTags;
	if(m_Xmls)
	{
		m_Xmls->clearAndDestroy();
		delete m_Xmls;
	}
}

void	CWLDocument::parse(const char* _newData, size_t size, bool* isDone)
{
	if(m_DontParse && !m_BufFlag)
		return;

	if(size == 0)
		return;

  //Check if the content is already in UCS2	          
  if (size > 2 && !strncmp(_newData, "\xFE\xFF", 2)) //unicode, also ucs2 also utf18
  {
    char *pcData = new char[size*3 + 1];
    memcpy(pcData, _newData+2, size-2);

	  int sz = UCS2ToANSI(CP_UTF8, (const unsigned short *)(_newData+2), (size-2)/2, pcData, (size-2)/2);
		m_Charset = CP_UTF8;
    
	  if(m_BufFlag)
		  m_HtmlPagePtr->append(pcData, sz);

	  append(pcData, sz);
    delete []pcData ;
    return;
  } 
  else if (size > 2 && !strncmp(_newData, "\xFF\xFE", 2)) //little endian unicode, also ucs2le
	{
    char *pcData = new char[size*3 + 1];
    memcpy(pcData, _newData+2, size-2);

	  int sz = UCS2ToANSI(CP_UTF8, (const unsigned short *)(_newData+2), (size-2)/2, pcData, (size-2)/2);
		m_Charset = CP_UTF8;
    
	  if(m_BufFlag)
		  m_HtmlPagePtr->append(pcData, sz);

	  append(pcData, sz);
    delete []pcData ;
    return;
	}
  else if (size  > 3 && !strncmp(_newData, "\xEF\xBB\xBF", 3)) //UTF8
  {
    char *pcData = new char[size];
    memcpy(pcData, _newData+3, size-3);

// @@Chaim - already in UTF8
//	  int sz = UCS2ToANSI(CP_UTF8, (const unsigned short *)(_newData+3), (size-3)/2, pcData, (size-3)/2);
		m_Charset = CP_UTF8;
    
	  if(m_BufFlag)
		  m_HtmlPagePtr->append(pcData, size-3);

	  append(pcData, size-3);
    delete []pcData ;
    return;
  }

	if (!m_FoundBodyTag && !m_UserDefinedCharset)
	{	// look for charset setting
		RVCString newData;
		if (buff.length() > 0)
		{
			newData.append(buff, buff.length());
		}

    /*
    if (m_TruncatedBytes.length() >  0)  // truncated bytes from previous chunk
		{
			newData.append(m_TruncatedBytes, m_TruncatedBytes.length());
		}
    */
		newData.append(_newData, size);

		RVCRegexp REbeginBodyTag("<[bB][oO][dD][yY]");
		RVCRegexp REbeginMetaTag("<[mM][eE][tT][aA]");
		size_t dummy, lenB, lenM = 0;
		UINT uCharsetValue = -1;
  	size_t attValueStart = RV_NPOS;

		while (!((lenM = REbeginMetaTag.index(newData, &dummy, lenM)) == RV_NPOS || m_FoundBodyTag))
		{ // found META tag
			size_t buffPtr = lenM+5;
			lenM++;
			RVCString attName, attValue, content, charset;
			while(1)
			{
				attState metaState = htmlFindNextAttribute(newData, attName, attValue, buffPtr, attValueStart);
				if(metaState == HTML_CURRENT_ATT_COMPLETE)
				{
					switch(kindOfAttribute(attName))
					{
					case CHARSET:
						charset = attValue.data();
					case CONTENT:
						content = attValue.data();
						break;
					}
				}
				else if (metaState == HTML_FOUND_DELIMITER)
				{
					if (!charset.isNull())
					{
						uCharsetValue = UL_CodePageNameToID(charset.data());
					}
					else if (!content.isNull())
					{
						char *pcContent = new char[content.length()+1];
						strcpy(pcContent, content.data());
						char* pIndex = strtok(pcContent, ";");
						while (pIndex)
						{
							SkipSpaces(pIndex);
							//there's a charset index
							if (!strnicmp(pIndex, "charset", 7)) {
								pIndex += 7;
								SkipSpaces(pIndex);
								//read the value
								if (*pIndex == '=')
								{
									++pIndex;
									SkipSpaces(pIndex);
									char code[100];
									GetValue(code, pIndex);
									uCharsetValue = UL_CodePageNameToID(code);
									WlTraceMsg("Found Charset %s", code);
									break;
								}
							}
							pIndex = strtok(NULL, ";");
						}// while
						bool bBreak = (!pIndex || !pIndex[0]);
						delete [] pcContent;
						if (bBreak)
							break;
					}
				}
				else 
					break;
			}
		}
		
		if ((lenB = REbeginBodyTag.index(newData, &dummy)) != RV_NPOS)
			m_FoundBodyTag = true;

		if (uCharsetValue != -1)
			m_Charset = uCharsetValue;
	}

	//int nNewSize = (size + m_TruncatedBytes.length())*3 +1;
  int nNewSize = (size )*3 +1;

  char *tempBuff = new char[nNewSize];
  memset(tempBuff,'\0',nNewSize);
	//RVCString Newbuff("");
  /*
	if (m_TruncatedBytes.length())
	{  // append truncated double byte chars from previous chunk
		Newbuff.append(m_TruncatedBytes);
		m_TruncatedBytes = "";  //reset
	}
  */
	//Newbuff.append(_newData);

  // @@Chaim
  // This is for the actual conversion
	//int num = ANSIToUTF8(m_Charset, Newbuff, nNewSize, tempBuff, nNewSize);

  // @@Chaim - charset issue in netizen
  int num;

  // @@Chaim - YES
//  if (m_ConvertToUTF8 && m_Charset){
  if (m_Charset){
    num = ANSIToUTF8(m_Charset, _newData, size, tempBuff, nNewSize);
	WlTraceMsg("Converted %d bytes from %d bytes", num, size);
  }
  else {
    memcpy(tempBuff, _newData, size);
    num = size;
  }

	if(m_BufFlag)
		m_HtmlPagePtr->append(tempBuff, num);

	append(tempBuff, num);

	delete [] tempBuff;

  // @@Chaim
  // This is to check if there are any truncated bytes (ineffecient)
  /*
	int nBytesConveted;
	wchar_t *wbuff = new wchar_t[nNewSize];
	num = UL_MultiByteToWideChar(m_Charset, 0, Newbuff, nNewSize, wbuff, nNewSize, &nBytesConveted, &m_CharMode);
	delete [] wbuff;
	if ((int)nNewSize > nBytesConveted)  // store truncated bytes
	{
		const char* temp = Newbuff;
		m_TruncatedBytes.append(temp+nBytesConveted, (int)nNewSize-nBytesConveted);
	}
   */
  
  
  // @@Chaim - old code
  /*
	if(m_BufFlag)
		m_HtmlPagePtr->append(newData, size);

	if(!m_DontParse)
		append(newData, size);
  */
}

void	CWLDocument::SaveSource(const char* newData, size_t size)
{
	if(m_BufFlag)
		m_HtmlPagePtr->append(newData, size);
}

// 25.2.99

void CWLDocument::append(const char* newData, size_t size)
{
	if(size > 0)
		buff.append(newData, size);
	else
		return;
	size_t  lt = 0;
	while(1)
	{
	switch(readState)
	{
		case(START_APPEND):
  		  while((readState == START_APPEND)
			&& ((lt = buff.index("<")) != RV_NPOS)) {
		    if(buff.length() - lt > 8)//!mimimum of "noframe " == 9, white space included
			{
			  m_iBuffPos += lt + 1;
			  buff.remove(0, lt + 1);

			  char firstChar = toupper(buff(0));

			  switch(firstChar) {
			  case '/' : 
				  switch(toupper(buff(1))) {
				  case 'A' :
					if (isElementOfType("/a", 2, buff, '>', this))
						m_CurLink = NULL;
					break;
				  }
				  break;
			  case '!' :
			      if(isElementOfType("!--", 3, buff, '~', this))
				    readState = IN_COMMENT;
				  break;
			  case 'A' :
				  m_CurLink = NULL;
				  if(m_Links && isElementOfType("a", 1, buff, ' ', this))
				    readState = IN_HREF;
				  else if(m_Links && isElementOfType("area", 4, buff, ' ', this))
				    readState = IN_AREA;
   				  else if (m_AppletList && isElementOfType("applet", 6, buff, ' ', this))
				    readState = IN_APPLET;
				  break;
			  case 'B' :
				  if(isElementOfType("base", 4, buff, ' ', this))
					readState = IN_BASE;
	              else if(m_SrcList && isElementOfType("body", 4, buff, ' ', this))
					readState = IN_BODY_TAG;
  				  else if(m_EmbedList && isElementOfType("bgsound", 7, buff, ' ', this))
			  	    readState = IN_BGSOUND;
				  break;
			  case 'E' :
				  if(m_EmbedList && isElementOfType("embed", 5, buff, ' ', this))
				    readState = IN_EMBED;
				  break;
			  case 'F' :
				  if(m_Forms && isElementOfType("form", 4, buff, ' ', this))
				    readState = IN_FORM;
  			      else if(isElementOfType("frame", 5, buff, ' ', this))
				    readState = IN_FRAME;
				  break;
			  case 'I' :
				  if(m_SrcList && isElementOfType("img", 3, buff, ' ', this))
				    readState = IN_IMG;
	  			  else if(isElementOfType("iframe", 6, buff, ' ', this))
				    readState = IN_IFRAME_TAG;
				  break;
			  case 'L' :
  				  if(m_StyleLinks && isElementOfType("link", 4, buff, ' ', this))
			  	    readState = IN_LINK;
				  break;
			  case 'M' :
	  			  if(m_MetaTags && isElementOfType("meta", 4, buff, ' ', this))
					readState = IN_META;
				  break;
			  case 'N' :
			      if(isElementOfType("noframes", 8, buff, '>', this))
				    readState = IN_NOFRAME;
/*!	webload currently doesn't support scripts, so NOSCRIPT section
	should be parsed as usual
				else if(isElementOfType("noscript", buff, '>'))
				    readState = IN_NOSCRIPT;
*/
				  break;
			  case 'O' :
  				  if(m_AppletList && isElementOfType("object", 6, buff, ' ', this))
				    readState = IN_OBJECT;
				  break;
			  case 'S' :
			      if(isElementOfType("script", 6, buff, '>', this))
				    readState = IN_SCRIPT; //!script element - must parse or rid of
				  else if(isElementOfType("style", 5, buff, '>', this))
					readState = IN_STYLE;
				  break;
			  case 'T' :
				  if(m_Tables && isElementOfType("table", 5, buff, '>', this))
			 	    readState = IN_TABLES;
				  else if (!titleFound && isElementOfType("title", 5, buff, '>', this))
				    readState = IN_TITLE;
				  break;
			  case 'X' :
				  if(m_Xmls && isElementOfType("xml", 3, buff, '>', this))
			  	    readState = IN_XML;
				  break;
			  default:
				  break;
			  } //! switch
			}
			else//!if -- buffLength - lt <= 7
				return;
		  }			//!while

		  if(readState == START_APPEND)//!no '<' found remove rest of buff and read more
		  {
			m_iBuffPos += buff.length();
		    buff.remove(0);
		    return;
		  }
		  break;//!switch
		case(IN_BODY_TAG): //! in <body
			switch(extractBodyAttributes(buff))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_TITLE)://!in <title
			switch(extractTitle(buff))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_BASE)://!in <base
			switch(extractBase(buff))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_META)://!in <meta
			switch(m_MetaTags->parse(buff,&m_UrlBase))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_SCRIPT)://!in <script
			if (m_Scripts) {
				switch(addToScripts(buff))
				{
					case(END_OF_ELEMENT):
						readState = START_APPEND;
						break;
					case(READ_MORE):
						return;
				}
				break;
			} else {
				switch(skipHtmlBlock(buff,"</script>", 9, this))
				{
					case(END_OF_ELEMENT):
						readState = START_APPEND;
						break;
					case(READ_MORE):
						return;
				}
				break;
			}

		case(IN_COMMENT):
			switch(skipHtmlBlock(buff,"-->", 3, this))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_HREF):
			switch(extractHref(buff, A_ANCHOR))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_AREA):
			switch(extractHref(buff, AREA_ANCHOR))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_FRAME):
			switch(extractFrame(buff))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_IFRAME_TAG):
			switch(extractFrame(buff))
			{
				case(END_OF_ELEMENT):
					readState = IN_IFRAME_BODY;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_IFRAME_BODY):
			switch(skipHtmlBlock(buff,"</iframe>", 9, this))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_NOFRAME):
			switch(skipHtmlBlock(buff,"</noframes>", 11, this))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_NOSCRIPT):
			switch(skipHtmlBlock(buff,"</noscript>", 11, this))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_FORM):
			switch(extractForm(buff))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_IMG):
			switch(extractImg(buff))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_BGSOUND):
			switch(extractBgsound(buff))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_EMBED):
			switch(extractEmbed(buff))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_OBJECT):
			switch(addToApplets(buff, "object"))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_APPLET):
			switch(addToApplets(buff, "applet"))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_LINK):
			switch(extractLink(buff))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_STYLE)://!in <style
			switch(addToStyles(buff))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_XML):
			switch(addToXmls(buff))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		case(IN_TABLES):
			switch(extractTable(buff))
			{
				case(END_OF_ELEMENT):
					readState = START_APPEND;
					break;
				case(READ_MORE):
					return;
			}
			break;
		default:
			break;
	}
	}
}

//new 25.2.99
elementReadState CWLDocument::addToScripts(RVCString& tempBuff)
{
	RVCRegexp REscriptEnd("</[sS][cC][rR][iI][pP][tT]>");
	RVCString scriptsString;
	size_t tempBuffLength = tempBuff.length();
	if(tempBuffLength > 8)
	{
		size_t scriptEndIndex = tempBuff.index(REscriptEnd);;
		if(scriptEndIndex != RV_NPOS)
		{
			scriptsString = tempBuff(0, scriptEndIndex);
			if(m_Scripts)
  			addNewScriptElementToArray(scriptsString, scriptEndIndex);
			m_iBuffPos += scriptEndIndex + 9;
			tempBuff.remove(0, scriptEndIndex + 9);
			return END_OF_ELEMENT;
		}
	}
		//!length <= 8  or </script> not found
	return READ_MORE;
}

elementReadState CWLDocument::addToStyles(RVCString& tempBuff)
{
	RVCRegexp REstyleEnd("</[sS][tT][yY][lL][eE]>");
	RVCString styleString;
	size_t tempBuffLength = tempBuff.length();
	if(tempBuffLength > 7)
	{
		size_t styleEndIndex = tempBuff.index(REstyleEnd);;
		if(styleEndIndex != RV_NPOS)
		{
			styleString = tempBuff(0, styleEndIndex);
			if(m_StyleSheets)
  			addNewStyleElementToArray(styleString, styleEndIndex);
			m_iBuffPos += styleEndIndex + 8;
			tempBuff.remove(0, styleEndIndex + 8);
			return END_OF_ELEMENT;
		}
	}
		//!length <= 7  or </style> not found
	return READ_MORE;
}

elementReadState CWLDocument::addToXmls(RVCString& tempBuff)
{
	RVCString xmlString;
	size_t tempBuffLength = tempBuff.length();
	if(tempBuffLength > 5)
	{
		RVCRegexp RExmlEnd("</[xX][mM][lL]>");
		size_t xmlEndIndex = tempBuff.index(RExmlEnd);;
		if(xmlEndIndex != RV_NPOS)
		{
			xmlString = tempBuff(0, xmlEndIndex);
			addNewXmlElementToArray(xmlString, xmlEndIndex, this);
			m_iBuffPos += xmlEndIndex + 6;
			tempBuff.remove(0, xmlEndIndex + 6);
			return END_OF_ELEMENT;
		}
	}
		//!length <= 5  or </xml> not found
	return READ_MORE;
}

void CWLDocument::addNewScriptElementToArray(RVCString& tempBuff, size_t scriptEndIndex)
{
	bool b_transformScriptToXml = false;
	RVCString attName, attValue, url, origUrl;
	size_t buffPtr = 0, attValueStart = RV_NPOS, attValueStartF = RV_NPOS;
	attState scriptState;
	CWLScript* script = new CWLScript(this);
	m_Scripts->insert(script);
	while(1)
	{
		scriptState = htmlFindNextAttribute(tempBuff, attName, attValue, buffPtr, attValueStart);
		if(scriptState == HTML_FOUND_DELIMITER)
		{
			//!done - have to get the innerHTML of the script - includes <!--
			buffPtr = tempBuff.first('>') + 1;
			RVCString text(tempBuff(buffPtr, scriptEndIndex - buffPtr));
			script->SetText(text.data());
			if(!script->m_OriginalUrl.isNull()) {
				CWLTransactionsObject* TOptr = script->GetTransactionObj();
				if (attValueStartF != RV_NPOS) {
					TOptr->SetUrlOffset(m_iBuffPos + attValueStartF);
					script->setUrlOffset(m_iBuffPos + attValueStartF);
				}
				insertTransObjToQueue(TOptr);
			}
			if(b_transformScriptToXml)
				transformScriptToXml(script, this);
//				transformScriptToXml(script, m_Xmls, m_QueuePtr, &m_DocUrl, &m_UrlBase);
			return;
		}
		else if(scriptState == HTML_CURRENT_ATT_COMPLETE)
			//!find out what the att is - if we support it add to object
		{
			switch(kindOfAttribute(attName)) {
			case EVENT:
				script->SetEvent(attValue.data());
				break;
			case ID:
				//!update the att in the script object
				script->SetId(attValue.data());
				break;
			case LANGUAGE:
				if(attValue.compareTo("xml", RVCString::ignoreCase) == 0)
/*! *******************************************
						<script langauge = xml ..>
							<xmlroot>
							..
							</xmlroot>
						</script>
********************************************/
					b_transformScriptToXml = true;
				script->SetLanguage(attValue.data());
				break;
			case SRC:
				attValueStartF = attValueStart;
				script->m_OriginalUrl = attValue;
				url = m_UrlBase.calcUrl(attValue);
				script->SetSrc(url.data());
				break;
			}
		}
		else
			//!error we cannot be in a state where we need more data as we have a complete element
			return;
	}
	return;
}

void CWLDocument::addNewStyleElementToArray(RVCString& tempBuff, size_t styleEndIndex)
{
	RVCString attName, attValue, url, origUrl;
	size_t buffPtr = 0, attValueStart = RV_NPOS;
	attState styleState;
	CWLStyle* style = new CWLStyle(this);
	m_StyleSheets->insert(style);
	while(1)
	{
		styleState = htmlFindNextAttribute(tempBuff, attName, attValue, buffPtr, attValueStart);
		if(styleState == HTML_FOUND_DELIMITER)
		{
			//!done - have to get the innerHTML of the style - includes <!--
			buffPtr = tempBuff.first('>') + 1;
			RVCString text(tempBuff(buffPtr, styleEndIndex - buffPtr));
			style->parse(text, text.length(), &m_UrlBase, buffPtr);
			return;
		}
		else if(styleState == HTML_CURRENT_ATT_COMPLETE)
			//!find out what the att is - if we support it add to object
		{
			switch(kindOfAttribute(attName)){
			case ID:
				style->SetId(attValue.data());
				break;
			}
		}
		else
			//!error we cannot be in a state where we need more data as we have a complete element
			return;
	}
	return;
}

elementReadState CWLDocument::extractTitle(RVCString& tempBuff)
{
	size_t tempBuffLength = tempBuff.length();
	RVCRegexp REtitle("</[tT][iI][tT][lL][eE]");
	size_t titleEndIndex = tempBuff.index(REtitle);
	if(tempBuffLength > 7 && titleEndIndex != RV_NPOS)
	{
		titleFound = true;
		m_Title = tempBuff(1, titleEndIndex - 1);
		m_iBuffPos += titleEndIndex + 7;
		tempBuff.remove(0, titleEndIndex + 7);
		return END_OF_ELEMENT;
	}
	return READ_MORE;
}


elementReadState CWLDocument::extractHref(RVCString& tempBuff, anchorType type)
{
	return m_Links->parse(tempBuff, type, &m_UrlBase);

}

elementReadState CWLDocument::extractFrame(RVCString& tempBuff)
{
	return m_Frames->parse(tempBuff, &m_UrlBase);

}

elementReadState CWLDocument::extractForm(RVCString& tempBuff)
{
	return m_Forms->parse(tempBuff, &m_UrlBase);
}

elementReadState CWLDocument::extractImg(RVCString& tempBuff)
{
	return m_SrcList->parse(tempBuff, &m_UrlBase, "img");

}

elementReadState CWLDocument::extractEmbed(RVCString& tempBuff)
{
	return m_EmbedList->parse(tempBuff, &m_UrlBase, "embed");

}

elementReadState CWLDocument::extractLink(RVCString& tempBuff)
{
	return m_StyleLinks->parse(tempBuff, &m_UrlBase);

}

elementReadState CWLDocument::extractBgsound(RVCString& tempBuff)
{
	return m_EmbedList->parse(tempBuff, &m_UrlBase, "bgsound");

}

elementReadState CWLDocument::addToApplets(RVCString& tempBuff, const char* tagName)
{
	RVCString appletsString;
	size_t appletEndIndex, tempBuffLength = tempBuff.length();
	if(tempBuffLength > 8)
	{
		if(!stricmp(tagName, "object"))
			appletEndIndex = tempBuff.index(RVCRegexp("</[oO][bB][jJ][eE][cC][tT]>"));
		else
			appletEndIndex = tempBuff.index(RVCRegexp("</[aA][pP][pP][lL][eE][tT]>"));

		if(appletEndIndex != RV_NPOS)
		{
			CWLApplet *applet = new CWLApplet(this);
			m_AppletList->insert(applet);
			appletsString = tempBuff(0, appletEndIndex);
			applet->parse(appletsString, &m_UrlBase, appletEndIndex, tagName);
			m_iBuffPos += appletEndIndex + 9;
			tempBuff.remove(0, appletEndIndex + 9);
			return END_OF_ELEMENT;
		}
	}
		//!length <= 8  or </script> not found
	return READ_MORE;

}

elementReadState CWLDocument::extractTable(RVCString& tempBuff)
{
	return m_Tables->parse(tempBuff, &m_UrlBase);

}

elementReadState CWLDocument::extractBase(RVCString& tempBuff)
{
	RVCString attName, attValue;
	size_t buffPtr = 0, attValueStart = RV_NPOS;
	while(1)
	{
		attState readState = htmlFindNextAttribute(tempBuff, attName, attValue, buffPtr, attValueStart);
		if(readState == HTML_FOUND_DELIMITER)
		{
			size_t _lt = tempBuff.index(">");
			m_iBuffPos += _lt + 1;
			tempBuff.remove(0, _lt + 1);
			return END_OF_ELEMENT;
		}
		else if(readState == HTML_CURRENT_ATT_COMPLETE)
		{
			m_iBuffPos += buffPtr;
			tempBuff.remove(0, buffPtr);
			buffPtr = 0;
			switch(kindOfAttribute(attName)) {
			case HREF:
			//!find out what the att is - if we support it add to object
		  //!set the url base in case it the url
				m_UrlBase.setBase(attValue);
				break;
			}
		}
		else
			return READ_MORE;
	}
	return READ_MORE;
}

elementReadState CWLDocument::extractBodyAttributes(RVCString& tempBuff)
{
	RVCString attName, attValue, url, origUrl;
	size_t buffPtr = 0, attValueStart = RV_NPOS, attValueStartF = RV_NPOS;
	while(1)
	{
		attState readState = htmlFindNextAttribute(tempBuff, attName, attValue, buffPtr, attValueStart);
		if(readState == HTML_FOUND_DELIMITER)
		{
			if (m_wlAll && !url.isNull()) {
			CWLEmbed* srcEntryPtr = new CWLEmbed("body", "background", url, origUrl, this, "", "", "");
				if(!m_wlAll->insert(srcEntryPtr))
					delete srcEntryPtr;
				else if (!origUrl.isNull()) {
					CWLTransactionsObject* TOptr = srcEntryPtr->GetTransactionObj();
					if (attValueStartF != RV_NPOS) {
						TOptr->SetUrlOffset(m_iBuffPos + attValueStartF);
						srcEntryPtr->setUrlOffset(m_iBuffPos + attValueStartF);
					}
					insertTransObjToQueue(TOptr);
				}
			}
			size_t _lt = tempBuff.index(">");
			m_iBuffPos += _lt + 1;
			tempBuff.remove(0, _lt + 1);
			return END_OF_ELEMENT;
		}
		else if(readState == HTML_CURRENT_ATT_COMPLETE)
		{
			switch(kindOfAttribute(attName)) {
			case BACKGROUND:
				attValueStartF = attValueStart;
				origUrl = attValue;
				url = m_UrlBase.calcUrl(attValue);
				break;
			}
	/*bug		m_iBuffPos += buffPtr; 
			tempBuff.remove(0, buffPtr);
			buffPtr = 0;*/
		}
		else
			return READ_MORE;
	}
	return READ_MORE;
}
//move to wlhtmlfuncs
/*
elementReadState CWLDocument::extractComment(RVCString& tempBuff)
{
	size_t tempBuffLength = tempBuff.length();
	RVCRegexp REcommentEnd("-->");
	size_t commentEndIndex = tempBuff.index(REcommentEnd);
	if(tempBuffLength > 2 && commentEndIndex != RV_NPOS)
	{
		tempBuff.remove(0, commentEndIndex + 3);
		return END_OF_ELEMENT;
	}
	//len < 2  || no --> found
	return READ_MORE;
}
*/

void CWLDocument::redirectionHandler(RVCString& newUrl)
{
	m_Header.clearAndDestroy();	//!clear the headers array
	m_DocUrl = newUrl.data();
	m_UrlBase.setBase(newUrl);
	m_Location.setHref(newUrl.data());

	titleFound = false;
	readState = START_APPEND;

	if(!m_Title.isNull())
		m_Title.remove(0);

  if(m_HtmlPagePtr && !m_HtmlPagePtr->isNull())
		m_HtmlPagePtr->remove(0);

  m_iBuffPos = 0;
  m_CurLink = NULL ;

/*
	if(m_wlAll && !m_wlAll->isEmpty())
		m_wlAll->clearAndDestroy();

	if(m_Links && !m_Links->isEmpty())
		m_Links->clearAndDestroy();

	if(m_Frames && !m_Frames->isEmpty())
		m_Frames->clearAndDestroy();

	if(m_Forms && !m_Forms->isEmpty())
		m_Forms->clearAndDestroy();

	if(m_SrcList && !m_SrcList->isEmpty())
		m_SrcList->clearAndDestroy();

	if(m_EmbedList && !m_EmbedList->isEmpty())
		m_EmbedList->clearAndDestroy();

	if(m_AppletList && !m_AppletList->isEmpty())
		m_AppletList->clearAndDestroy();

	if(m_StyleSheets && !m_StyleSheets->isEmpty())
		m_StyleSheets->clearAndDestroy();

	if(m_StyleLinks && !m_StyleLinks->isEmpty())
		m_StyleLinks->clearAndDestroy();

	if(m_Scripts && !m_Scripts->isEmpty())
		m_Scripts->clearAndDestroy();

	if(m_Xmls && !m_Xmls->isEmpty())
		m_Xmls->clearAndDestroy();

	if(m_Tables && !m_Tables->isEmpty())
		m_Tables->clearAndDestroy();

	if(m_MetaTags && !m_MetaTags->isEmpty())
		m_MetaTags->clearAndDestroy();
*/
}

void CWLDocument::insertUri(RVCString& uri)
{
	m_Location.setUri(uri.data());
}


void CWLDocument::insertHost(RVCString& name, unsigned port)
{
	m_Location.setHostname(name.data());
	m_PortNum = port;
}

unsigned int CWLDocument::getRefCount()
{
  unsigned int i;
  {
    RVCriticalSection::LockGuard lock(*m_RefCountMutex);
    i = m_CloneRefCount;
  }
  return i;
}

unsigned int CWLDocument::incRefCount()
{
  unsigned int i;

  {
    RVCriticalSection::LockGuard lock(*m_RefCountMutex);
    ++m_CloneRefCount;
    i = m_CloneRefCount;
  }
//  WlTraceMsg ("Increasing reference count for frame : %d - now is: %d",(int)this,i);
  return i;
}

unsigned int CWLDocument::decRefCount()
{
  unsigned int i;

  {
    RVCriticalSection::LockGuard lock(*m_RefCountMutex);
//!    if(m_CloneRefCount)
    m_CloneRefCount--;
    i = m_CloneRefCount;
  }
//  WlTraceMsg ("Decreasing reference count for frame : %d - now is: %d",(int)this,i);
	return i;
}

void CWLDocument::insertStatus(unsigned  value, int n, RVCString& line)
{
	//Shahar - Sun response problem....
	if (n > 0) {
	m_ProtocolVersion = line(0, n - 1);
	if (line.length() > size_t(n + 3)) {
		line.remove(0, n+3);
		line = line.strip(line.both, ' ');
		m_StatusLine = line;
	}
	}
	else {
		m_ProtocolVersion = line;
		m_StatusLine = line;
	}
	m_StatusNumber = value;

}

//! have to change according to new definition of headerss
void CWLDocument::insertHeader(const char* name, const char* value)
{
	GetHeader().insertKeyAndValue(name, value);
}

//!done
int CWLDocument::pageTreeSearch(int typeOfSearch, int formIndex,
                           const char* pat,const char* fn, void** retVal)
{
	RVSlistCollectablesQueue framePtrQ;
	CWLDocument* CWLDocumentPtr = this;
	int roundCounter = 1;
	int retCode, errorCode;
	do
	{
		parseFrame*  flist;
		if (CWLDocumentPtr->m_Frames)
			flist = CWLDocumentPtr->m_Frames;
		else
			continue;
		RVOrderedIterator iter(*flist);
		do {
			switch(typeOfSearch)
			{
			case 0:
				if ((retCode = CWLDocumentPtr->GetFrameByFrameName(pat, (CWLDocument**)retVal)) > -1)
				return (0);
				if(roundCounter-- > 0)
				errorCode = retCode;
				break;
			case 1:
				if ((retCode = CWLDocumentPtr->GetFrameByUrl(pat, (CWLDocument**)retVal)) > -1)
				return (0);
				if(roundCounter-- > 0)
				errorCode = retCode;
				break;
			case 2:
				if ((retCode = CWLDocumentPtr->GetHrefByName(pat, (CWLLink**)retVal)) > -1)
					return (0);
				if(roundCounter-- > 0)
					errorCode = retCode;
					break;
			case 3:
				if ((retCode = CWLDocumentPtr->GetHrefByUrl(pat, (CWLLink**)retVal)) > -1)
					return (0);
				if(roundCounter-- > 0)
					errorCode = retCode;
					break;
			case 4:
				if ((retCode = CWLDocumentPtr->GetFieldValueByPage(fn, (const char**)retVal)) > -1)
					return (0);
				if(roundCounter-- > 0)
					errorCode = retCode;
				break;
			case 5:
				if ((retCode = CWLDocumentPtr->GetFieldValueByForm(formIndex, fn, (const char**)retVal)) > -1)
					return (0);
				if(roundCounter-- > 0)
					errorCode = retCode;
				break;
			default:
				return -2;
			}
			if (iter())
			{
				CWLDocumentPtr = (CWLDocument*)iter.key();
				framePtrQ.insert(CWLDocumentPtr);
			}
			else
				CWLDocumentPtr = NULL;
		} while(CWLDocumentPtr);
  } while (CWLDocumentPtr = (CWLDocument*)framePtrQ.get());
	return  errorCode;
}


int CWLDocument::GetFrameByIndex(RVCString& pn, int in, CWLDocument** frameValue)
{
  if (m_Frames)
    return(m_Frames->getNameByIndex(pn, in, frameValue));
  else
  return (-5);//! No parsing of frames
}


int CWLDocument::GetFrameByUrl(const char* urlN, CWLDocument** frameValue)
{
  if (m_Frames)
    return (m_Frames->findEntryByUrlMatch(urlN, frameValue));
  else
  return (-5);//! No parsing of frames
}

findFrameInDocStatus CWLDocument::GetFrameByLocation(const RVCString& identLocationStr , CWLDocument*& frame)
{
  CWLIdentLocation      identLocation (identLocationStr);

  //! WE initialize the return status in default error
  findFrameInDocStatus retStatus = findFrameInDocStatus_ERROR;
  //! Call the frames search method which will search and return the frame
  //! for the ident loacation given
  findFrameStatus retFrameStatus = m_Frames->findEntryByLocationMatch(identLocation, frame);

  if (retFrameStatus == findFrameStatus_OK) {
    retStatus = findFrameInDocStatus_OK;
  }
  else if (retFrameStatus == findFrameStatus_NAME_MATCH_NUMBER_MISMATCH_WARNING) {
    retStatus = findFrameInDocStatus_NAME_MATCH_NUMBER_MISMATCH_WARNING;
  }
  else if (retFrameStatus == findFrameStatus_NUMBER_MATCH_NAME_MISMATCH_WARNING) {
    retStatus = findFrameinDocStatus_NUMBER_MATCH_NAME_MISMATCH_WARNING;
  }
  return (retStatus);
}

int CWLDocument::GetFrameByFrameName(const char* frameNameN, CWLDocument** frameValue)
{
  if (m_Frames)
    return (m_Frames->findEntryByFrameNameMatch(frameNameN, frameValue));
  else
  return (-5);//! No parsing of frames
}

int CWLDocument::GetHrefByIndex(int in, CWLLink** hrefPtr)
{
  if (m_Links)
    return (m_Links->getUrlByIndex(in, hrefPtr));
  else
    return (-5);//! No parsing of links
}


int CWLDocument::GetHrefByName(const char* hrefName, CWLLink** hrefPtr)
{
  if (m_Links)
    return (m_Links->findEntryByInnerHTMLMatch(hrefName, hrefPtr));
  else
    return (-5);//! No parsing of links
}

int CWLDocument::GetHrefByUrl(const char* url_, CWLLink** hrefPtr)
{
  if (m_Links)
    return (m_Links->findEntryByUrlMatch(url_, hrefPtr));
  else
    return (-5);//! No parsing done to links
}

int CWLDocument::GetFormAction(int formIndex, CWLLocation** action)
{
	if(m_Forms)
		return(m_Forms->findActionByIndex(formIndex, action));
	else
		return(-5);//!no parsing of forms
}

int CWLDocument::GetFieldValueByPage(const char* fieldName, const char** fieldValue)
{
  if (m_Forms)
    return (m_Forms->findEntryByName(fieldName, fieldValue));
  else
    return (-5);//! No parsing of forms
}


int CWLDocument::GetFieldValueByForm(int fi,const  char* fieldName, const char** fieldValue)
{
  if (m_Forms)
    return (m_Forms->findEntryByIndexAndName(fi, fieldName, fieldValue));
  else
    return (-6);//!No parsing of forms
}


int CWLDocument::GetHeaderValue(const char* name, const char*& value)
{
	value = m_Header.Value(name);
	if (value)
		return 0;

	return -2;
}

int CWLDocument::GetAllHeaderValues(const char* name, const char*& values)
{
	values = m_Header.getAllValues(name);
	if(values)
		return 0;
	return -2;
}

unsigned CWLDocument::GetStatusNumber()
{
	return m_StatusNumber ;
}


int CWLDocument::GetStatusLine(const char*& value)
{
	if (!m_StatusLine.isNull()) {
		value = m_StatusLine.data();
		return 0;
	}
	return -2;
}


int CWLDocument::GetHost(const char*& value)
{
	if ((value = m_Location.Host()) != "") {
		return 0;
	}
	return -2;
}


int CWLDocument::GetHostName(const char*& value)
{
	if ((value = m_Location.Hostname()) != "") {
		return 0;
	}
	return -2;
}


unsigned CWLDocument::GetPortNum()
{

	return m_Location.Port();
}


int CWLDocument::GetUri(const char*& uri)
{
	if ((uri = m_Location.Uri()) != "")
		return 0;
	return -2;
}

const char* CWLDocument::Id()
{
	return m_Id.isNull() ? NULL : m_Id.data();
}

const char* CWLDocument::Target()
{
	return m_Target.isNull() ? NULL : m_Target.data();
}

void CWLDocument::SetDocUrl(const char *url)
{
	m_DocUrl = url;
	m_Location.setHref(url);
	SetBase(m_DocUrl);
}

void CWLDocument::SetBase(RVCString& url)
{
	m_UrlBase.setBase(url);
}

void CWLDocument::SetId(const char* id)
{
	m_Id = id;
}

CWLXmlDocument* CWLDocument::Xml(const char* xmlId)
{
	return GetXmlById(xmlId, m_Xmls);
}

void CWLDocument::StartXmlDirectParsing()
{
  if (!m_DirectXmlParsing) {
  char StartXmlPage[] = "<HTML><XML ID=\"Xml\">" ;
  m_DirectXmlParsing = true ;
  append(StartXmlPage, strlen(StartXmlPage) );
  }
}

void CWLDocument::EndXmlDirectParsing()
{

  if (m_DirectXmlParsing) {
    char EndXmlPage[] = "</XML></HTML>" ;
    append(EndXmlPage, strlen(EndXmlPage));
    m_DirectXmlParsing = false ;
  }
}

void CWLDocument::insertTransObjToQueue(CWLTransactionsObject *tObj)
{
	if (tObj->GetObjType() != FRAME) {
		m_CurrentTransObj->setNext(tObj);
		m_CurrentTransObj = tObj;
	}
	m_QueuePtr->insert(tObj);
}

WLLogMessage::Severity CWLDocument::GetSeverity()
{
  return m_MaxSeverity;
}

WLLogMessage::Severity CWLDocument::SetSeverity(WLLogMessage::Severity severity)
{
  if (severity > m_MaxSeverity)
    m_MaxSeverity = severity ;

  return m_MaxSeverity;
}

void CWLDocument::SetCharset(long charset, bool userdefined) {
  m_Charset = charset; m_UserDefinedCharset = userdefined;
}

long CWLDocument::GetCharset(){
  return m_Charset;
}

bool CWLDocument::GetSCharset(char* pcStr){
  return UL_CodePageIDToName(m_Charset, pcStr);
}
