/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlregularexpressions.h"
#include <RVCString.h>	
#include "wlstyle.h"
#include "wldocument.h"
#include "wlurlbase.h"
#include "platform.h"


CWLStyle::CWLStyle(CWLDocument* docPtr)
	: m_DocPtr(docPtr), m_TransObj(NULL)
{
	m_TransObj.SetObj(this);
	m_TransObj.SetObjType(STYLE);
	m_Index = m_DocPtr->m_StyleSheets->entries();
	m_ParentStyleSheet = NULL;
	m_Referrer = m_DocPtr->GetDocUrl();
	m_UrlBase.setBase(m_Referrer);
	m_BuffIndex = 0;
	// m_OriginalUrl == "", m_Href == ""
	m_iUrlOffset = -1;
}

CWLStyle::CWLStyle(CWLDocument* docPtr, CWLStyle* parentSS, RVCString& href)
	: m_DocPtr(docPtr), m_ParentStyleSheet(parentSS), m_OriginalUrl(href), m_TransObj(NULL)
{
	m_TransObj.SetObj(this); 
	m_TransObj.SetObjType(STYLE);
	m_Index = parentSS->m_Imports.entries();
	m_Referrer = parentSS->m_Href;
	m_Href = parentSS->m_UrlBase.calcUrl(m_OriginalUrl);
	m_UrlBase.setBase(m_Href);
	m_BuffIndex = 0;
    m_TransObj.GetTransaction()->SetName(m_Href);
	m_iUrlOffset = -1;
}

void CWLStyle::parse(RVCString& newData, size_t dataSize, urlBase* urlBasePtr, size_t p_offset)
{
	size_t beginUrl, endUrl, beginComment, endComment, beginImport, endImport;
	RVCString url;
  enum styleType {COMMENT, IMPORT, URL};
	styleType state;
	size_t buffPrevLength = m_Buff.length();
	size_t urlOffset ;

	if (dataSize > 0)
		m_Buff.append(newData, dataSize);
	
	while(1) {
		beginComment = m_Buff.index("/*", m_BuffIndex);
		beginImport  = m_Buff.index("@import", m_BuffIndex);
		beginUrl     = m_Buff.index("url(", m_BuffIndex);

		if (beginComment != RV_NPOS 
			&& (beginImport == RV_NPOS  || beginComment < beginImport)
			&& (beginUrl    == RV_NPOS  || beginComment < beginUrl))
			state = COMMENT;
		else if (beginImport != RV_NPOS && beginUrl != RV_NPOS
				&& beginImport < beginUrl)
			state = IMPORT;
		else if (beginUrl != RV_NPOS)
			state = URL;
		else
			return;

		if(state == URL) {
			if ((endUrl = m_Buff.index(")", beginUrl)) == RV_NPOS)
				return; //! no closing ")"
			beginUrl += 4;
			url = m_Buff(beginUrl, endUrl - beginUrl);
			url = url.strip(RVCString::both, '"');
			RVCString href = m_UrlBase.calcUrl(url);
			CWLCss *some_obj = new CWLCss(href, url, m_DocPtr, NULL, NULL);
			if (!m_DocPtr->m_wlAll || !m_DocPtr->m_wlAll->insert(some_obj))
				delete some_obj;
			else if (!url.isNull()) {
				CWLTransactionsObject* TOptr = some_obj->GetTransactionObj();
				urlOffset = m_DocPtr->m_iBuffPos + p_offset - buffPrevLength + beginUrl;
				TOptr->SetUrlOffset(urlOffset);
				some_obj->setUrlOffset(urlOffset);
				m_DocPtr->insertTransObjToQueue(TOptr);
			}
			m_BuffIndex = endUrl + 1;

		} else if(state == IMPORT) {
			if ((endUrl = m_Buff.index(")", beginImport)) == RV_NPOS)
				return; //! no closing ")"
			if ((endImport = m_Buff.index(";", beginImport)) == RV_NPOS)
				return; //! no ';' after @import
			if (beginUrl > endImport || endUrl > endImport || beginUrl > endUrl) {
				m_BuffIndex = endImport + 1; //! something bad, point after ";"
				continue; //! 
			}
			beginUrl += 4;
			url = m_Buff(beginUrl, endUrl - beginUrl);
			url = url.strip(RVCString::both, '"');
			CWLStyle *style = new CWLStyle(m_DocPtr, this, url);
			if (!m_Imports.insert(style))
				delete style;
			else if (!url.isNull()) {
				CWLTransactionsObject* TOptr = style->GetTransactionObj();
				urlOffset = m_DocPtr->m_iBuffPos + p_offset - buffPrevLength + beginUrl;
				TOptr->SetUrlOffset(urlOffset);
				style->setUrlOffset(urlOffset);
				m_DocPtr->insertTransObjToQueue(TOptr);
			}
			m_BuffIndex = endImport + 1;
		
		} else { //! if(state == COMMENT)  
			endComment = m_Buff.index("*/", beginComment);
			if (endComment == RV_NPOS)
				m_BuffIndex = beginComment;
			else
				m_BuffIndex = endComment + 2;
		}
	}
}

void CWLStyle::printAll()
{
	cout<<"ID:       "<<m_Id.data()<<'\n';
	cout<<"HREF:     "<<m_Href.data()<<'\n';
	cout<<"CSSTEXT:  "<<'\n'<<m_Buff.data()<<'\n'<<'\n';
	cout<<"IMPORTS#: "<<m_Imports.entries() << endl;
}
