/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlregularexpressions.h"
#include "wltable.h"
#include <string.h>
#include <stdlib.h>
#include <regexp.h>
#include "innerhtml.h"
#ifdef __TV_UNIX__
#include <ctype.h>
#endif
#include "wllocation.h"
#include "wlhtmlvirtualqueue.h" 
#include "wldocument.h"

//!
//! Gets a cell attributes (ROWSPAN, COLSPAN).
//! The routine TAKES FOR GRANTED that buff contains the entire cell!!!
void CWLTableCell::readCellAttributes(RVCString& buff, int *index, urlBase *urlBasePtr)
{
	size_t buff_index = *index, attValueStart = RV_NPOS, attValueStartF = RV_NPOS;
	RVCString url, origUrl;
	RVCString attName, attValue;

	while(1)
	{
		attState state = htmlFindNextAttribute(buff, attName, attValue, buff_index, attValueStart);
		*index = buff_index;
		if( state == HTML_CURRENT_ATT_COMPLETE ) 
		{
			switch(kindOfAttribute(attName)){
			case BACKGROUND:
				attValueStartF = attValueStart;
				origUrl = attValue;
				url = urlBasePtr->calcUrl(attValue);
				break;
			case COLSPAN:
				colSpan = atoi((const char *)attValue);
				break;
			case ID:
				id =  attValue.data();
				break;
			case ROWSPAN:
				rowSpan = atoi((const char *)attValue);
				break;
			}
		}
		else if( state == HTML_FOUND_DELIMITER ) {
			RVOrdered *doc_SrcT = m_DocPtr->m_wlAll;
			//parseSrc *doc_SrcT = m_DocPtr->m_SrcList;
			if (!origUrl.isNull() && doc_SrcT) {
				CWLEmbed* srcEntryPtr = new CWLEmbed("td", "background", url, origUrl, m_DocPtr, "", id.data(), "");
				if(!doc_SrcT->insert(srcEntryPtr))
					delete srcEntryPtr;
				else {
					CWLTransactionsObject* TOptr = srcEntryPtr->GetTransactionObj();
					if (attValueStartF != RV_NPOS) {
						size_t uo = m_DocPtr->m_iBuffPos + attValueStartF;
						srcEntryPtr->setUrlOffset(uo);
						TOptr->SetUrlOffset(uo);
					}
					m_DocPtr->insertTransObjToQueue(TOptr);
				}
			}	
			break;//!while
		}
	}

	return;
		
}


//!
//! fills a cell with data taken from the HTML source
//! It is TAKEN for GRANTED that startIndex points to the location rigth after the openning
//! tag name and endIndex points on the location right after the closing tag
void CWLTableCell::parse(RVCString& buff, int startIndex, int endIndex, urlBase *urlBasePtr)
{
	int index = startIndex;
	if( startIndex == endIndex )  //!empty cell
		return;

	//! first read any attribute of the cell
	readCellAttributes(buff, &index, urlBasePtr);

	if (index >= endIndex)
		return; //! some problem (e.g. <TD &nbsp;</TD>)

	//! now set the innerHTML property
	//! This code will be changed once we will ove to pointers - Uri
	if( createInnerHtml() )
	{
		innerHtml = buff(index,(endIndex-index+1));
		removeNewLine(innerHtml);
		if( ! innerHtml.isNull() ) 
			Html2Text(innerHtml,0,innerHtml.length()-1,innerText);
	}
	else
		Html2Text(buff,index,endIndex,innerText);
}


ostream& operator<<(ostream& o, const CWLTableCell& x)
{
	o << "[<" << (x.getCellType()==TD ? "TD>:" : "TH>:") << x.getCellIndex()<<" id = "<<x.getCellId();
	o << " \"" << x.getCellInnerHtml() << "\"]";	
	return o;
}


//! Adds a new cell at the end of a row
bool CWLTableRow::addToRow(CWLTableCell *cell)
{
	if( insert((RVCollectable *)cell) == NULL )
		return false;

	cell->setCellIndex(entries() - 1);
	++totalCells;

	return true;
	
}

//! returns the cell according to position or null if cell does not exist
//! position start from zero (0)
CWLTableCell *CWLTableRow::getRowCell(int position) const
{
	CWLTableCell *cell;
	try {
		cell = (CWLTableCell *) at(position);
	}
	catch(...)
	{
		return NULL;
	}

	return cell;
}

DllExport ostream& operator<<(ostream& o, const CWLTableRow& x)
{
	CWLTableCell *cell;

	o << "Row " << x.getRowIndex() << "(" << x.getTotalCells() << ") :";
	RVOrderedIterator i(x);

	while( cell = (CWLTableCell *)i() )
		o << *cell;

	o << endl;

	return o;
}



DllExport ostream& operator<<(ostream& o, const CWLCellCollection& x)
{
	CWLTableCell *cell;

	o << "Collection  :";
	RVOrderedIterator i(x);

	while( cell = (CWLTableCell *)i() )
		o << *cell;

	o << endl;

	return o;
}

CWLTable::CWLTable(CWLDocument *docPtr, int n_cols) :	m_DocPtr(docPtr), cols(n_cols), numCells(0) ,tableID(""),
													state(NEW_TABLE) , cell(NULL) , outerHtml(NULL)
{}

CWLTable::CWLTable(CWLDocument *docPtr, char *id,int n_cols) : m_DocPtr(docPtr), cols(n_cols), numCells(0) ,
																tableID(id), state(NEW_TABLE) , cell(NULL) , outerHtml(NULL)
{}

CWLTable::~CWLTable()
{
	if( outerHtml )
		delete outerHtml;

	clearAndDestroy();
}


bool CWLTable::newRow()
{
	CWLTableRow  *row = new CWLTableRow(entries(),numCells);

	if( insert((RVCollectable *)row) == NULL )
	{
		delete row;
		return false;
	}
	else
		return true;

}

bool CWLTable::addCell(CWLTableCell *cell)
{
	CWLTableRow  *row = (CWLTableRow *)last();
	if( row->addToRow(cell) )
	{
		++numCells;
		return true;
	}
	else
		return false;

}


CWLTableRow *CWLTable::getTableRow(int position) const
{
	CWLTableRow   *row;
	try {
		row = (CWLTableRow *) at(position);
	}
	catch(...)
	{
		return NULL;
	}

	return row;
}


CWLTableCell *CWLTable::getTableCell(int rowPosition, int cellPosition) const
{
	CWLTableRow   *row;
	try {
		row = (CWLTableRow *) at(rowPosition);
	}
	catch(...)
	{
		return NULL;
	}

	return row->getRowCell(cellPosition);
}

//! get a cell from the table according to its ordinal position.
//! implements a binary search to find the row that holds the cell.
//! last result is remembered in order to optimize iterations.
CWLTableCell *CWLTable::getTableCell(int position) const
{
	CWLTableRow  *row;
	int            lower,upper;   //!lower & upper bounds for the binary search
	static int     current;
	static int     totalInRow;

	static int lastPosition;

	if( position < 0 || position >= numCells ) //! out of bounds
		return NULL;

	if(position == (lastPosition + 1)) //! Ahhha!! iteration. use previous results
	{
		if( position >= totalInRow )  //! beyond this row
			++current;  //! go to the next one

		row = (CWLTableRow *) at(current);
		totalInRow = row->getTotalCells();
	}
	else   //! Sorry. need to search all over again..
	{
		lower = 0;
		upper = entries();  //! start with the entire range of rows
		
		while(upper > lower)
		{
			current = (upper - lower) / 2 + lower; //! look at the middle..
			row = (CWLTableRow *) at(current);
			totalInRow = row->getTotalCells();
			if( position >= totalInRow )  //! beyond this row
				lower = current;
			else if( position < totalInRow - int(row->entries()) ) //! below this row
				upper = current;
			else
				break;
		}
	}

	return row->getRowCell((position - (totalInRow - row->entries())));

}



CWLCellCollection *CWLTable::getCellRange(char *range) const
{
	CWLCellCollection *result = new CWLCellCollection;
	CWLTableRow            *row;
	CWLTableCell      *cell;
	int x1,y1,x2,y2,x,y;

	if( translateSpreadtoCoord(range,&x1,&y1,&x2,&y2))
	{
		for(y = y1 ; y <= y2 ; ++y)
		{
			if( (row = getTableRow(y)) == NULL ) //! error
			{
//				delete result;
//				return NULL;
				return result;
			}

			for(x = x1 ; x <= x2 ; ++x)
			{
				if( (cell = row->getRowCell(x)) != NULL ) //! cell exists
					if( result->insert((RVCollectable *)cell) == NULL ) //! error
					{
						delete result;
						return NULL;
					}
			}
		}

		return result;
	}
	else
	{
		delete result;
		return NULL;
	}
}



CWLCellCollection *CWLTable::getCells() 
{
	CWLCellCollection *result = new CWLCellCollection(this);
	CWLTableRow       *row;
	CWLTableCell      *cell;	
	for (int y = 0 ; y < getNumRows() ; ++y)
	{
		if( (row = getTableRow(y)) == NULL ) //! error
		{
			delete result;
			return NULL;
		}
//yuval
//?? - getNumCells() == total number of cell in table, not in current row.
//		for (int x = 0 ; x < getNumCells() ; x++)
		for (int x = 0 ; x < row->getRowSize() ; ++x)
		{
			if ( (cell = row->getRowCell(x)) != NULL ) //! cell exists
				if ( result->insert((RVCollectable *)cell) == NULL ) //! error
				{
					delete result;
					return NULL;
				}
		}
	}

	return result;
}

				
				

DllExport ostream& operator<<(ostream& o, const CWLTable& x)
{
	CWLTableRow  *r;

	o << "Table    ID=" << x.getTableID() << endl;

	RVOrderedIterator i(x);
	while( r = (CWLTableRow *)i() )
		o << *r;

	o << "Total number of cells in table: " << x.getNumCells() << endl;
	return o;
}

const char* CWLTable::getTableID() const
{
	return tableID.data();
}

const char *CWLTable::getOuterHtml()
{
	CWLTableRow       *row;
	CWLTableCell      *cell;
	char               tempBuf[100];

	if( outerHtml )  //! property already exists, return it.
		return (const char *) *outerHtml;
	else
		outerHtml = new RVCString();

	//! otherwise, calculate it
	outerHtml->append("<table "); 
	if( *(getTableID()) != '\0' ) //! there is TableID
	{
		outerHtml->append(" id=");
		outerHtml->append(getTableID());
	}
	if( cols > 0 ) //! there is cols
	{
		outerHtml->append(" cols=");
		sprintf(tempBuf,"%d",cols);
		outerHtml->append(tempBuf);
	}
	outerHtml->append(">\n");

	for (int y = 0 ; y < getNumRows() ; ++y)
	{
		if( (row = getTableRow(y)) == NULL ) //! error
			break;
		else
			outerHtml->append("<tr>\n");

		for (int x = 0 ; x < row->getRowSize() ; ++x)
		{
			if ( (cell = row->getRowCell(x)) != NULL ) //! cell exists
			{
				outerHtml->append("<");
				outerHtml->append((cell->getCellType() == TD ? "td" : "th"));
				if(cell->getRowSpan() > 1)
				{
					outerHtml->append(" rowspan=");
					sprintf(tempBuf,"%d",cell->getRowSpan());
					outerHtml->append(tempBuf);
				}
				if(cell->getColSpan() > 1)
				{
					outerHtml->append(" colspan=");
					sprintf(tempBuf,"%d",cell->getColSpan());
					outerHtml->append(tempBuf);
				}
				outerHtml->append(">\n");
				outerHtml->append(cell->getCellInnerHtml());
				outerHtml->append("<");
				outerHtml->append((cell->getCellType() == TD ? "/td>\n" : "/th>\n"));
			}
		}
		outerHtml->append("</tr>\n");
	}
	outerHtml->append("</table>");

	return (const char *) *outerHtml;
}



tableTagType CWLTable::findTableTag(RVCString& buff, int *index)
{
    RVCRegexp innerTagsRE("</?[tT][dDrRhH]");
    RVCRegexp tableRE("</?[tT][aA][bB][lL][eE]");
	
    size_t index1,index2,index3,index4;
    
    index1 = buff.index(innerTagsRE,*index);
    index2 = buff.index(tableRE,*index);
	index3 = buff.index("<!--",*index);

	//! skip comments if any....
	if( index3 != RV_NPOS ) 
	{
		if( index1 == RV_NPOS || index3 < index1 )
		{
			if( index2 == RV_NPOS || index3 < index2 )
			{
				index4 = buff.index("-->",index3+4);
				if( index4 == RV_NPOS ) //! end comment not found
				{
					return TAG_READ_MORE;
				}
				else
					*index = index4+3;
					if( *index >= int(buff.length()) )
						return TAG_READ_MORE;
					else
						return TAG_CONTINUE;
			}
		}
	}

	if(index1 == RV_NPOS)
		if(index2 == RV_NPOS)  //! nothing found
		{
            return TAG_READ_MORE;
		}
        else                   //! only TABLE or /TABLE found
        {
            if(buff[index2+1] == '/')
            {
                *index = index2+7;
                return TABLE_END;
            }
            else
            {
                //yuval
                //we need to change the <TABLE into <FABLE here
                //              buff[index2 + 1] = 'F';
                *index = index2+6;
                return TABLE_START;
            }
        }
     else
		 if( index2 == RV_NPOS || index1 < index2 ) //! found inner tag
		 {
			 if(buff[index1+1] != '/')
			 {
				 *index = index1+3;
				 switch(buff[index1+2])
				 {
				 case 'd':
				 case 'D':
					 return TD_START;
					 
				 case 'r':
				 case 'R':
					 return TR_START;
					 
				 case 'h':
				 case 'H':
					 return TH_START;
				 }
			 }
			 else
			 {
				 *index = index1+4;
				 switch(buff[index1+3])
				 {
				 case 'd':
				 case 'D':
					 return TD_END;
					 
				 case 'r':
				 case 'R':
					 return TR_END;
					 
				 case 'h':
				 case 'H':
					 return TH_END;
				 }
			 }
			 
		 }
		 else  //! TABLE or /TABLE were found BEFORE an inner tag
		 {
			 if(buff[index2+1] == '/')
			 {
				 *index = index2+7;
				 return TABLE_END;
			 }
			 else
			 {
				 *index = index2+6;
				 return TABLE_START;
			 }
		 }
	return TAG_READ_MORE;		 
}




//!
//! Gets a cell attributes (ROWSPAN, COLSPAN).
//! The routine TAKES FOR GRANTED that buff contains the entire cell!!!
elementReadState CWLTable::readTableAttributes(RVCString& buff, int *index, urlBase *urlBasePtr)
{
	RVCString attName, attValue;
	RVCString url, origUrl;
	size_t  buff_ptr = *index, attValueStart = RV_NPOS, attValueStartF = RV_NPOS;

	while(1)
	{
		attState state = htmlFindNextAttribute(buff, attName, attValue, buff_ptr, attValueStart);
		*index = buff_ptr;
		if( state == HTML_CURRENT_ATT_COMPLETE )  //! anything else of an attribute
		{
			switch(kindOfAttribute(attName)) {
			case BACKGROUND:
				attValueStartF = attValueStart;
				origUrl = attValue;
				url = urlBasePtr->calcUrl(attValue);
				break;
			case COLS:
				cols = atoi((const char *)attValue);
				break;
			case ID:
				tableID = attValue;
				break;
			}
		}
		else if (state == HTML_FOUND_DELIMITER) {
			//parseSrc *doc_SrcsT = m_DocPtr->m_SrcList;
			RVOrdered *doc_SrcsT = m_DocPtr->m_wlAll;
			if (doc_SrcsT) {
			CWLEmbed* srcEntryPtr = new CWLEmbed("table", "background", url, origUrl, m_DocPtr, "", tableID.data(), "");
				if(!doc_SrcsT->insert(srcEntryPtr))
					delete srcEntryPtr;
				else if (!origUrl.isNull()) {
					CWLTransactionsObject* TOptr = srcEntryPtr->GetTransactionObj();
					if (attValueStartF != RV_NPOS) {
						size_t uo = m_DocPtr->m_iBuffPos + attValueStartF;
						TOptr->SetUrlOffset(uo);
						srcEntryPtr->setUrlOffset(uo);
					}
					m_DocPtr->insertTransObjToQueue(TOptr);
				}
			}
			break;//!while
		} else  //!HTML_READ_MORE_DATA
			return READ_MORE;
	}	

	return END_OF_ELEMENT;	
		
}


tableParseResults CWLTable::parse(RVCString& buff, int *index, urlBase *urlBasePtr)
{
	tableTagType   tag;
	
	if(state == NEW_TABLE)  //! needs to read table's attributes
		if( readTableAttributes(buff, index, urlBasePtr) == READ_MORE )
			return TABLE_READ_MORE;
		else
			state = IN_TABLE;  //! ok. start parsing the content
		
    while(1) //! loop
    {
        tag = findTableTag(buff, index);
        switch(tag)
        {
        case TABLE_START:
            switch(state)
            {
            case IN_TABLE:
            case IN_TR:
                return TABLE_START_TABLE; //! end of one and start of other.
                
            case IN_TH:
            case IN_TD:
                return TABLE_PUSH_TABLE;  //! recursive table inside a cell
            }
            
            break;
            
        case TABLE_END:
            switch(state)
            {
            case IN_TABLE:
            case IN_TR:
                return TABLE_POP_TABLE; //! end of table.
                
            case IN_TH:
            case IN_TD:
                if(cell)
                {
                    cell->parse(buff,cellStart, (*index)-8, urlBasePtr); //! fill the cell with data
                    cell = NULL;  //! dismiss the pointer
                }
                   
                return TABLE_POP_TABLE;
            }
            
            break;
               
        case TR_START:
            switch(state)
            {
            case IN_TABLE:
            case IN_TR:
                newRow();
                state = IN_TR;
                break;
                   
            case IN_TH:
            case IN_TD:
                if(cell)
                {
                    cell->parse(buff,cellStart, (*index)-4, urlBasePtr); //! fill the cell with data
                    cell = NULL;  //! dismiss the pointer
                }
				newRow();
                state = IN_TR;
                break;
            }
               
            break;
                       
        case TR_END:
            switch(state)
            {
            case IN_TABLE:
            case IN_TH:
            case IN_TD:
                break;  //! do nothing. continue
                
            case IN_TR:
                state = IN_TABLE;  //! close the row
                break;
            }
            break;
                        
        case TD_START:
            switch (state)
            {
            case IN_TABLE:
                newRow();
                cellStart = *index;  //! save the start of the cell
                cell = new CWLTableCell(m_DocPtr, TD);
				addCell(cell);  //! insert it to the table
                state = IN_TD;
                break;
                   
            case IN_TR:
                cellStart = *index;  //! save the start of the cell
                cell = new CWLTableCell(m_DocPtr, TD);
				addCell(cell);  //! insert it to the table
                state = IN_TD;
                break;
                
            case IN_TD:
            case IN_TH:
                if(cell)   //! close cell
                    cell->parse(buff,cellStart, (*index)-4, urlBasePtr); //! fill the cell with data
                
                cellStart = *index;  //! save the start of the cell
                cell = new CWLTableCell(m_DocPtr, TD);
				addCell(cell);  //! insert it to the table
                state = IN_TD;
                break;
            }
            break;
                               
        case TD_END:
            switch(state)
            {
            case IN_TABLE:
            case IN_TR:
            case IN_TH:
                break;  //! do nothing. continue.
                
            case IN_TD:
                if(cell)
                {
                    cell->parse(buff,cellStart, (*index)-5, urlBasePtr); //! fill the cell with data
                    cell = NULL;  //! dismiss the pointer
                }
                state = IN_TR;
                break;
            }
               
            break;
               
               
        case TH_START:
            switch (state)
            {
            case IN_TABLE:
                newRow();
                cellStart = *index;  //! save the start of the cell
                cell = new CWLTableCell(m_DocPtr, TH);
				addCell(cell);  //! insert it to the table
                state = IN_TH;
				break;
				
			case IN_TR:
				cellStart = *index;  //! save the start of the cell
				cell = new CWLTableCell(m_DocPtr, TH);
				addCell(cell);  //! insert it to the table
				state = IN_TH;
				break;
				
			case IN_TD:
			case IN_TH:
				if(cell)   //! close cell
					cell->parse(buff,cellStart, (*index)-4, urlBasePtr); //! fill the cell with data
				
				cellStart = *index;  //! save the start of the cell
				cell = new CWLTableCell(m_DocPtr, TH);
				addCell(cell);  //! insert it to the table
				state = IN_TH;
				break;
			}
			break;
			
        case TH_END:
			switch(state)
			{
			case IN_TABLE:
			case IN_TR:
			case IN_TD:
				break;  //! do nothing. continue.
				
			case IN_TH:
				if(cell)
				{
					cell->parse(buff,cellStart, (*index)-5, urlBasePtr); //! fill the cell with data
					cell = NULL;  //! dismiss the pointer
				}
				state = IN_TR;
				break;
			}
			
			break;
			
        case TAG_READ_MORE:
            return TABLE_READ_MORE;
                
        }
	}
	
	
}




CWLTable *CWLTableCollection::findByID(char *id)
{
	CWLTable  *table;
	RVOrderedIterator it((RVOrdered&) *this);

	while( table = (CWLTable *)it() ){
		const char* tableId = table->getTableID();
		if(tableId != NULL)
			if( ! strcmp(id, tableId) )
				return table;
	}

	return NULL;
}

void CWLTableCollection::clearTableTags(RVCString& buff, int clearToIndex)
{	
	RVCRegexp tableRE("<[tT][aA][bB][lL][eE]");
	int pos = 0;

	while( (pos = buff.index(tableRE,pos)) != RV_NPOS )
		if(pos < clearToIndex)
			buff.replace(pos,6,"<ttttt");  //! replace with dummy tag
		else 
			return;

}


elementReadState CWLTableCollection::parse(RVCString& buff, urlBase *urlBasePtr)
{
	CWLTable  *table;
	tableParseResults  result;

	while(1)  //! loop
	{
		switch(state)
		{
		case START_PARSE:
			table = new CWLTable(m_DocPtr);
			insert((RVCollectable *)table); //! add to the collection
			TableStack.push((RVCollectable *)table); //!push to the stack as the actve table
			
			state = IN_PARSE;
			break;
			
		case IN_PARSE:
			table = (CWLTable *)TableStack.top(); //! get the active table
			
			result = table->parse(buff, &parseIndex, urlBasePtr);
			switch(result)
			{
			case TABLE_READ_MORE:
				return READ_MORE;
				
			case TABLE_PUSH_TABLE:
				state = START_PARSE;  //! a recursive table was found. Start to parse it.
				break;
				
			case TABLE_POP_TABLE:  //! end of table
				TableStack.pop();
				if(TableStack.isEmpty()) //! no more tables to parse
				{
					state = START_PARSE;  //! preper for next time
					clearTableTags(buff, parseIndex); //! remove TABLE tags to avoid re-parsing
					parseIndex = 0;
					return END_OF_ELEMENT; //! finito!!
				}
				
				break; //! otherwise continue....
				
			case TABLE_START_TABLE:  //! end of table and the start of another one
				TableStack.pop();
				state = START_PARSE;
				break;
			}
			
			break;
		}
	}

	state = START_PARSE;  //! preper for next time	
	clearTableTags(buff, parseIndex); //! remove TABLE tags to avoid re-parsing
	parseIndex = 0;
	return END_OF_ELEMENT;

}



//! Translate a "spreadsheet-style" range to regular coordinates.
//! spreadsheet-style uses letters (one or two) to indicate columns
//! and digits (starting from 1) to indicate rows.
//! Example:  "A1:C8" or "AC7:BB9"
//! This routine will ALWAYS return the upper-left corner as x1,y1
//! and the lower-right corner as x2,y2 regardless of the way the
//! spreadsheet range was defined

#define translateLetter(l) ((tolower((int)(l))-'a'))
#define translateDigit(l)  (((l)-'1'))

bool translateOneCoord(char *spreadHalfRange, int *x, int *y)
{
	int length = strlen(spreadHalfRange);
	if( length < 2 || length > 5 )
		return false;  //! no go..
	else if( length == 2 ) //! should be letter+digit
	{
		if( isalpha(tolower((int)spreadHalfRange[0])) )
				*x = translateLetter(spreadHalfRange[0]);
		else
			return false;

		if( isdigit((int)spreadHalfRange[1]) )
			*y = translateDigit(spreadHalfRange[1]);
		else
			return false;

	}
	else  //! 3 or more chars.
	{
		if( isalpha(tolower((int)spreadHalfRange[0])) )
				*x = translateLetter(spreadHalfRange[0]);
		else
			return false;

		if( isalpha(tolower((int)spreadHalfRange[1])) )
				*x = ((*x+1) * 26) + translateLetter(spreadHalfRange[1]);
		else
			return false;

		*y = atoi((spreadHalfRange+2));
		if( *y == 0 )
			return false;
	}
	return true;
}


bool DllExport translateSpreadtoCoord(char* spreadRange, int *x_1, int *y_1, int *x_2 , int *y_2)
{
	char temp[20];
	int  x1,y1,x2,y2;
	int  length = strlen(spreadRange);
	char  *p;
	
	if(length < 5 || length > 11)
		return false; //! min xx:xx max xxxxx:xxxxx
	else
		strcpy(temp,spreadRange);
	
	p = strchr(temp,':');
	if( !p )
		return false;
	else
		*p = '\0'; //! split the string on the location of ':'
	
	if( *temp ) //! first char is not null
		if( ! translateOneCoord(temp, &x1, &y1) )
			return false;
		
	if( *(p+1) ) //! second half is not null
		if( ! translateOneCoord((p+1), &x2, &y2) )
			return false;
		
	//! Now return the coordinates where x1,y1 is the upper left
	//! and x2,y2 is the lower right corners
		
	//!*x_1 = (x1 <= x2) ? x1 : x2;
	//!*y_1 = (y1 <= y2) ? y1 : y2;
	//!*x_2 = (x1 <= x2) ? x2 : x1;
	//!*y_2 = (y1 <= y2) ? y2 : y1;
	
	*x_1 = x1;
	*y_1 = y1;
	*x_2 = x2;
	*y_2 = y2;
	
	return true;		
			
}



#ifdef _DEBUG

DllExport CWLTable *debugCreateTable(CWLDocument *docPtr, char *name, int col, int rows, int cols)
{
	CWLTable *table = new CWLTable(docPtr, name, col);
	char s[10];

	for(int r = 0 ; r < rows ; ++r)
	{
		table->newRow();
		for(int c = 0 ; c < cols ; ++c)
			if( r == 0 ) //! first row
			{
				sprintf(s,"H%2d",c+1);
				table->addCell(new CWLTableCell(docPtr, TH,s));
			}
			else //! other rows
			{
				sprintf(s,"%2d.%2d",r+1,c+1);
				table->addCell(new CWLTableCell(docPtr, TD,s));
			}
	}

	return table;
}

#endif
