/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wltransactiondata.h"
#include "wldocument.h"
#include "wlscript.h"
#include "wlxml.h"
#include "wlstyle.h"
#include "float.h"
#include <math.h>


CWLEvent::CWLEvent(
           double dTime,
           const char* reason, 
           const char* description, 
           WLLogMessage::Severity severity/*WLLogMessage::Success*/, 
           const char* extraDataType/*NULL*/, 
           void* extraData/*NULL*/, 
           unsigned long len/*0*/,
           bool bLog/*true*/)
{
  m_dTime         = dTime ;
  m_sReason       = reason ? reason : "" ;
  m_sDescription  = description ? description : "" ;
  m_Severity      = severity ;
  m_sExtraDataType= extraDataType ? extraDataType : "" ;
  m_ExtraData     = NULL ;
  m_ulLength      = len ;
  m_bLog          = bLog;

  if (extraData)
  {
    m_ExtraData = (void*)new char[m_ulLength+1];
    memcpy(m_ExtraData, extraData, m_ulLength);
    ((char*)m_ExtraData)[m_ulLength] = '\0';
  }
  
}

CWLEvent::~CWLEvent()
{
  if (m_ExtraData)
    delete []m_ExtraData;
}

CWLEvent::CWLEvent(const CWLEvent& cpyEvent)
{
  m_dTime         = cpyEvent.m_dTime ;
  m_sReason       = cpyEvent.m_sReason ;
  m_sDescription  = cpyEvent.m_sDescription ;
  m_Severity      = cpyEvent.m_Severity ;
  m_sExtraDataType= cpyEvent.m_sExtraDataType ;
  m_ExtraData     = NULL ;
  m_ulLength      = cpyEvent.m_ulLength ;
  m_bLog          = cpyEvent.m_bLog ;

  if (cpyEvent.m_ExtraData)
  {
    m_ExtraData = (void*)new char[m_ulLength+1];
    memcpy(m_ExtraData, cpyEvent.m_ExtraData, m_ulLength);
    ((char*)m_ExtraData)[m_ulLength] = '\0';
  }
}

CWLEvent& CWLEvent::operator=(const CWLEvent& cpyEvent)
{
  m_dTime         = cpyEvent.m_dTime ;
  m_sReason       = cpyEvent.m_sReason ;
  m_sDescription  = cpyEvent.m_sDescription ;
  m_Severity      = cpyEvent.m_Severity ;
  m_sExtraDataType= cpyEvent.m_sExtraDataType ;
  m_ExtraData     = NULL ;
  m_ulLength      = cpyEvent.m_ulLength ;
  m_bLog          = cpyEvent.m_bLog ;

  if (cpyEvent.m_ExtraData)
  {
    m_ExtraData = (void*)new char[m_ulLength+1];
    memcpy(m_ExtraData, cpyEvent.m_ExtraData, m_ulLength);
    ((char*)m_ExtraData)[m_ulLength] = '\0';
  }

  return *this;
}



//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
  
CWLTransactionData::CWLTransactionData(const char* pName/*NULL*/, double dTime/*0*/, TransactionType eType/*TRANS_GENERAL*/)
{
  m_sName          = (pName ? pName : "");
  m_sOrigName      =  m_sName ;
  m_dEndTrans      = m_dBeginTrans = dTime ;
	m_dBeginSubTrans = FLT_MAX;
  m_dEndTrans      = 0 ;
  m_bSave          = false ;
  m_Severity       = WLLogMessage::Success ;
  m_bFirstSubTrans = false ;
  m_bErrorUnder    = false ;
  m_eTransType     = eType ;
}

CWLTransactionData::CWLTransactionData(const CWLTransactionData& cpyTrans)
{
  m_dBeginTrans     = cpyTrans.m_dBeginTrans ;
  m_dEndTrans       = cpyTrans.m_dEndTrans ;
  m_sDescription    = cpyTrans.m_sDescription ;
  m_sName           = cpyTrans.m_sName ;
  m_sFailureReason  = cpyTrans.m_sFailureReason ;
  m_Severity        = cpyTrans.m_Severity ;
  m_bSave           = cpyTrans.m_bSave ;
  m_bFirstSubTrans  = cpyTrans.m_bFirstSubTrans ;
  m_bErrorUnder     = cpyTrans.m_bErrorUnder ;
  m_EventList       = cpyTrans.m_EventList ;
  m_eTransType      = cpyTrans.m_eTransType ;

  m_sOrigName      =  m_sName ;

}

CWLTransactionData::~CWLTransactionData()
{
  // Remove & delete list items
  m_EventList.clearAndDestroy(); 
}

void CWLTransactionData::SetName(const char*pName)
{
	if (pName)
		m_sName = pName;
	else
		m_sName = "";
  
  m_sOrigName      =  m_sName ;

  //m_sName = (pName ? pName : ""); oooo
}

void CWLTransactionData::SetBeginTransaction(double dTime)
{
  m_dBeginTrans = dTime ;
}

void CWLTransactionData::SetBeginSubTransaction(double dTime)
{
  m_dBeginSubTrans = dTime ;
}

void CWLTransactionData::SetEndTransaction(double dTime, WLLogMessage::Severity severity, bool save)
{
  m_dEndTrans = dTime ;
  m_Severity  = severity ;
  m_bSave    |= save ;
}

void CWLTransactionData::SetEndTransactionFailure(double dTime, WLLogMessage::Severity severity, const char* reason)
{
  m_dEndTrans = dTime ;
  m_Severity  = severity ;
  m_sFailureReason  = reason ? reason : "" ;
}

void CWLTransactionData::SetFailureReason(double dTime, const char* reason, const char* description)
{
  m_dEndTrans       = dTime ;
  m_sFailureReason  = reason ? reason : "" ;
  m_sDescription    = description ? description : "" ;
}

void CWLTransactionData::RecordEvent(    
                                      double dTime,
                                      const char* reason, 
                                      const char* description, 
                                      WLLogMessage::Severity severity/*WLLogMessage::Success*/, 
                                      const char* extraDataType/*NULL*/, 
                                      void* extraData/*NULL*/, 
                                      unsigned long len/*0*/,
                                      bool bLog/*true*/)
{
  m_EventList.insert(new CWLEvent(dTime, reason, description, severity, extraDataType, extraData, len, bLog));
}

void CWLTransactionData::AddEvent(    
                                      const char* reason, 
                                      const char* description, 
                                      WLLogMessage::Severity severity/*WLLogMessage::Success*/, 
                                      const char* extraDataType/*NULL*/, 
                                      void* extraData/*NULL*/, 
                                      unsigned long len/*0*/,
                                      bool bLog/*true*/)
{
  double time = m_dEndTrans;
  if (!time)
    time = TVPSysTime();
  m_EventList.insert(new CWLEvent(time, reason, description, severity, extraDataType, extraData, len,bLog));
}

CWLEvent* CWLTransactionData::GetNextEvent(int &pos)
{
  if ( (pos >= 0) && (pos < int(m_EventList.entries())) )
    return (CWLEvent*)m_EventList.at(pos++); 

  return NULL ;
}

void CWLTransactionData::SetSeverity(WLLogMessage::Severity severity)
{
  if (severity > m_Severity)
    m_Severity=severity;
}
  
void CWLTransactionData::SetFailureReason(const char* reason, const char* description/*NULL*/)
{
  m_sDescription = description ? description : "";
  m_sFailureReason = reason ? reason : "";
}

void CWLTransactionData::ResetSeverity()
{
	m_Severity        = WLLogMessage::Success ;
	m_sFailureReason  = "" ;
  m_sDescription    = "" ;
  m_bErrorUnder     = false ;

  // Remove & delete list items
  m_EventList.clearAndDestroy(); 
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

CWLTransactionsObject::CWLTransactionsObject(RVCollectable* pObject, HtmlQueueItemType objType /*NONE*/)
   : m_pObject(pObject), m_ObjType(objType), m_NextTransObj(NULL), m_bCached(false), m_nUrlOffset(-1)
{ 
  m_nFileSize = 0 ;
  m_Checksum  = 0 ;
}
 

CWLTransactionsObject::CWLTransactionsObject(const CWLTransactionsObject& cpyObj)
{
  m_Trans = cpyObj.m_Trans ; 
  m_SubTransList = cpyObj.m_SubTransList ; 
  m_bCached      = false;
  m_NextTransObj = NULL; // ???
  m_nFileSize = cpyObj.m_nFileSize ;
  m_Checksum  = cpyObj.m_Checksum ;
}

// Chaim - copy constructor for Parse Once.
CWLTransactionsObject::CWLTransactionsObject(CWLTransactionsObject& cpyObj, bool CopySubTransactionList)
{
  m_Trans = cpyObj.m_Trans ;
  m_NextTransObj = NULL; 
  m_nFileSize = cpyObj.m_nFileSize ;
  m_Checksum  = cpyObj.m_Checksum ;

  if (CopySubTransactionList) m_SubTransList = cpyObj.m_SubTransList ;
//  else m_SubTransList = NULL;
  m_Trans.SetName (cpyObj.GetObjUrl());
  m_bCached = cpyObj.m_bCached;

  m_ObjType = GENERAL;

//  RVCString OriginalUrl = cpyObj.getOriginalUrl();
//  RVCString Referer = cpyObj.getReferer();

//  m_pObject = (RVCollectable*) new CWLGeneralObject (OriginalUrl, Referer);
  m_pObject = (RVCollectable*) new CWLGeneralObject (cpyObj.GetOriginalUrl(), cpyObj.GetReferer());
//  m_ObjType = cpyObj.m_ObjType;


//  switch(m_ObjType)
//  {
    // The actual object is copied only for images (the constructor is used by ParseOnce which
    // needs only images to be fully handled.
//    case IMAGE  : m_pObject = (RVCollectable*) new CWLEmbed ((const CWLEmbed&) *cpyObj.m_pObject); break;
//    default:;
}

// Chaim - For ParseOnce
CWLGeneralObject::CWLGeneralObject(RVCString& OriginalUrl, RVCString& Referrer)
{
  m_OriginalUrl = *OriginalUrl;
  m_Referrer = *Referrer;
}
CWLGeneralObject::CWLGeneralObject(const char* OriginalUrl, const char* Referrer)
{
  m_OriginalUrl = OriginalUrl;
  m_Referrer = Referrer;
}

CWLTransactionData* CWLTransactionsObject::AddSubTransaction(const char* pName, double dTime/*0*/, CWLTransactionData::TransactionType eType/*TRANS_GENERAL*/)
{
  return (CWLTransactionData*)m_SubTransList.insert(new CWLTransactionData(pName, dTime, eType));
}

CWLTransactionsObject::~CWLTransactionsObject()
{
  // Remove & delete list items
  m_SubTransList.clearAndDestroy();
}

CWLTransactionData* CWLTransactionsObject::GetNextTrans(int &pos)
{
  CWLTransactionData* pTrans = NULL ;
    
  if (!pos)//first transaction
    pTrans = &m_Trans ;
  else if (pos > 0 && pos < NumOfTransactions())
    pTrans = (CWLTransactionData*)m_SubTransList.at(pos-1);

  pos++ ;

  return pTrans ;
}

CWLTransactionData* CWLTransactionsObject::FindTrans(const char* pName)
{
  CWLTransactionData* pTrans ;
  
  for (int pos = 0 ; (pTrans = GetNextTrans(pos)) && (strcmp(pTrans->GetName(), pName)) ; )
    ;

  return pTrans ;
}


void  CWLTransactionsObject::AddEvent(const char* reason, 
                                      const char* description, 
                                      WLLogMessage::Severity severity/*WLLogMessage::Success*/, 
                                      const char* extraDataType/*NULL*/, 
                                      void* extraData/*NULL*/, 
                                      unsigned long len/*0*/)
{
  double time = GetTransaction()->GetTransEnd();
  if (!time)
    time = TVPSysTime();
  GetTransaction()->RecordEvent(time,
                                reason, 
                                description, 
                                severity, 
                                extraDataType, 
                                extraData, 
                                len);
}

void CWLTransactionsObject::SetFailureReason(const char* reason, const char* description)
{
  GetTransaction()->SetFailureReason(GetTransaction()->GetTransEnd(),reason, description);
}

const char* CWLTransactionsObject::GetOriginalUrl()
{
  const char* pOriginalUrl=NULL;

  switch( GetObjType())
  {
    case FRAME  : pOriginalUrl = ((CWLDocument*)GetObject())->GetOriginalUrl(); break;
    case XML    : pOriginalUrl = ((CWLXmlDocument*)GetObject())->GetOriginalUrl(); break;
    case DTD    : pOriginalUrl = ((CWLXmlDocument*)GetObject())->GetOriginalUrl(); break;
    case IMAGE  : pOriginalUrl = ((CWLLocation*)GetObject())->GetOriginalUrl(); break;
    case LINK   : pOriginalUrl = ((CWLCss*)GetObject())->GetOriginalUrl(); break;
    case STYLE  : pOriginalUrl = ((CWLStyle*)GetObject())->GetOriginalUrl(); break;
    case EMBED  : pOriginalUrl = ((CWLEmbed*)GetObject())->GetOriginalUrl(); break; 
    case APPLET : pOriginalUrl = ((CWLApplet*)GetObject())->GetOriginalUrl(); break;
    case SCRIPT : pOriginalUrl = ((CWLScript*)GetObject())->GetOriginalUrl(); break;

    // Chaim - for ParseOnce
    case GENERAL: pOriginalUrl = ((CWLGeneralObject*)GetObject())->GetOriginalUrl(); break;

    default:;
  }

  return pOriginalUrl;
}

const char* CWLTransactionsObject::GetReferer()
{
  const char* pReferrer=NULL;

  switch( GetObjType())
  {
    case FRAME  : pReferrer = ((CWLDocument*)GetObject())->GetReferrer(); break;
    case XML    : pReferrer = ((CWLXmlDocument*)GetObject())->GetReferrer(); break;
   case DTD    : pReferrer = ((CWLXmlDocument*)GetObject())->GetReferrer(); break;
    case IMAGE  : pReferrer = ((CWLLocation*)GetObject())->GetReferrer(); break;
    case LINK   : pReferrer = ((CWLCss*)GetObject())->GetReferrer(); break;
    case STYLE  : pReferrer = ((CWLStyle*)GetObject())->GetReferrer(); break;
    case EMBED  : pReferrer = ((CWLEmbed*)GetObject())->GetReferrer(); break; 
    case APPLET : pReferrer = ((CWLApplet*)GetObject())->GetReferrer(); break;
    case SCRIPT : pReferrer = ((CWLScript*)GetObject())->GetReferrer(); break;

    // Chaim - for ParseOnce
    case GENERAL: pReferrer = ((CWLGeneralObject*)GetObject())->GetReferrer(); break;

    default:;
  }

  return pReferrer;
}


// Chaim - for Parse Once.
void CWLTransactionsObject::Reset()
{
  m_SubTransList.clearAndDestroy();
  m_Trans.GetEventList()->clearAndDestroy();
}
void CWLTransactionsObject::ResetFileSize(int p_nFileSize/*0*/)
{
  m_nFileSize = p_nFileSize ;
}

void CWLTransactionsObject::ResetChecksum(CHECKSUM_VAR p_CheckSum/*0*/)
{
  m_Checksum = p_CheckSum ;
}

int CWLTransactionsObject::AddFileSize(int p_nFileSize)
{
  return (m_nFileSize += p_nFileSize) ;
}

CHECKSUM_VAR CWLTransactionsObject::CalcCheckSum(const void* p_pvBuff, int p_nLength)
{
  CHECKSUM_VAR Checksum = 0 ;
  CHECKSUM_VAR Modulus = 0 ;
  int nModulus = 0 ;
  int nLength = p_nLength / sizeof(CHECKSUM_VAR);
  CHECKSUM_VAR* pBuff = (CHECKSUM_VAR*)p_pvBuff ;

  for ( int nIndex = 0 ; nIndex < nLength ; nIndex++)
  {
    Checksum ^= pBuff[nIndex] ;
  }

  //there is a remainder
  if ( nModulus = p_nLength % sizeof(CHECKSUM_VAR) )
  {
    memcpy( (void*)&Modulus, (char*)p_pvBuff + p_nLength-nModulus, nModulus );
    Checksum ^= Modulus ;
  }

  return Checksum ;
}

CHECKSUM_VAR CWLTransactionsObject::AddChecksum(const void* p_pvBuff, int p_nLength)
{
  return (m_Checksum^=CalcCheckSum(p_pvBuff, p_nLength));
}

int CWLTransactionsObject::GetFileSize()
{
  return m_nFileSize ;
}

CHECKSUM_VAR CWLTransactionsObject::GetCheckSum()
{
  return m_Checksum ;
}

double CWLTransactionsObject::GetSubTransTime(CWLTransactionData::TransactionType eType)
{
  CWLTransactionData* pTrans ;
  double dTime = 0 ;
  
  for (int pos = 0 ; (pTrans = GetNextTrans(pos)) ; )
  {
    if ( pTrans->GetTransType() == eType )
      dTime += ( pTrans->GetTransEnd() - pTrans->GetTransBegin() ) ;
  }

  dTime *= 1000 ;
  dTime = ceil(dTime) / (double)1000 ;
  return dTime ;
}

