/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "wlregularexpressions.h"
#include "wlxml.h"
#include <ordered.h>
#include <RVCString.h>
#include "wlhtmlvirtualqueue.h"
#include "wlhtmlfuncs.h"
#include "wlxmlhelperfuncs.h"
#include "wlscript.h"
#include "wlstyle.h"

CWLXmlDocument* GetXmlById(const char* xmlId, RVOrdered* Xmls)
{
	size_t entries_ = Xmls->entries();
	if(entries_  < 1)
		return NULL;
	for(size_t xmlIndex = 0; xmlIndex < entries_; ++xmlIndex)
	{
		if((RVCString) ( ((CWLXmlDocument*)((*Xmls).at(xmlIndex)))->GetId() ) == xmlId )
			return (CWLXmlDocument*)(Xmls->at(xmlIndex));
	}
	return NULL;
}

void addNewXmlElementToArray(RVCString& tempBuff, size_t xmlEndIndex, CWLDocument *docPtr)
{
	RVOrdered* Xmls = docPtr->m_Xmls;
	RVCString* docUrlPtr = &docPtr->GetDocUrl();
	urlBase* urlBasePtr = docPtr->GetBase();
	RVCString attName, attValue;
	size_t buffPtr = 0, attValueStart = RV_NPOS;
	attState xmlState;
	CWLXmlDocument* xml = new CWLXmlDocument(docPtr);
	//
	if(Xmls)
		Xmls->insert(xml);
	while(1)
	{
		xmlState = htmlFindNextAttribute(tempBuff, attName, attValue, buffPtr, attValueStart);
		if(xmlState == HTML_FOUND_DELIMITER)
		{
			//!done - have to get the innerHTML of the xml - includes <!--
			buffPtr = tempBuff.first('>') + 1;
			if(!(xml->GetSrc())){
				//RVCString text(tempBuff(buffPtr, xmlEndIndex - buffPtr));
				xml->SetInnerHTML(tempBuff.data() + buffPtr, xmlEndIndex - buffPtr);
				//xml->SetInnerHTML(text.data());
				xml->SetBase(*docUrlPtr);
				xml->parse();
			}
			if(!Xmls)
				delete xml;
			return;
		}
		else if(xmlState == HTML_CURRENT_ATT_COMPLETE)
			//!find out what the att is - if we support it add to object
		{
			switch(kindOfAttribute(attName)) {
			case ID:
				//!update the att in the xml object
				xml->SetId(attValue.data());
				break;
			case SRC:
				if(Xmls) {
				RVCString temp = urlBasePtr->calcUrl(attValue);
//				xml->SetSrc(attValue.data());
				xml->SetSrc(temp.data());
				xml->SetDocUrl(temp.data()) ;
				//m_DocUrl = urlBasePtr->calcUrl(attValue) ;
				//m_urlBase.setBase(m_DocUrl);
				temp = xml->GetDocUrl();
				xml->SetBase(temp);
				if (!attValue.isNull()) {
					CWLTransactionsObject* TOptr = xml->GetTransactionObj();
					if (attValueStart != RV_NPOS) {
						size_t uo = docPtr->m_iBuffPos + attValueStart;
						TOptr->SetUrlOffset(uo);
						xml->setUrlOffset(uo);
					}
					docPtr->insertTransObjToQueue(TOptr);
				}
				}
				break;
			}
		}
		else
			//!HTML_READ_MORE_DATA
			return;
	}
	return;
}

void transformScriptToXml(CWLScript* script, CWLDocument *docPtr)
{
	RVOrdered *Xmls = docPtr->m_Xmls;
	RVCString* docUrlPtr = &docPtr->GetDocUrl();
	urlBase* urlBasePtr = docPtr->GetBase();
	//!create new xml object
	if(!Xmls)
		return;
	CWLXmlDocument* xml = new CWLXmlDocument(docPtr);
	Xmls->insert(xml);
	//!fill in the different fields
	xml->SetId(script->GetId());
    const char* scriptSrc = script->GetSrc();
    if(scriptSrc){
		xml->SetSrc(scriptSrc);
		RVCString temp = urlBasePtr->calcUrl(RVCString(scriptSrc));
		xml->SetDocUrl(temp.data()) ;
		docPtr->insertTransObjToQueue(xml->GetTransactionObj());
	}
	else{
			xml->SetInnerHTML(script->GetText(), script->GetTextLength());
			xml->parse();
	}
}



enumWhiteValue xmlSkipWhiteSpaces(RVCString& str, size_t length, size_t& strPtr, char& firstNoneWhite)
{
	if(strPtr >= length)
		return WHITE_READ_MORE;
	if(!htmlIsWhiteSpace(str(strPtr)) ){
		firstNoneWhite = str(strPtr);
		return WHITE_NONE_TO_SKIP;
	}
	else
		while(htmlIsWhiteSpace(str(strPtr)) && strPtr < length - 1)
			++strPtr;
	if(!htmlIsWhiteSpace(str(strPtr))){
		firstNoneWhite = str(strPtr);
		return WHITE_SKIPPED_ALL;
	}
	return WHITE_READ_MORE;
}

//!function that takes:
//! - buffer
//! - pointer to a location in the buffer, return value - first white location after word
//! - length of the buffer
//! - reference to string to return

//! return values: false(READ_MORE), true(READ_COMPLETE_WORD)

bool readNextWord(RVCString& str, size_t length, size_t& strPtr, size_t& beginWordPtr)
{
	bool isQuoted = false;
	char firstNoneWhite, currentChar, quoteChar;
	switch(xmlSkipWhiteSpaces(str, length, strPtr, firstNoneWhite)){
		case(WHITE_READ_MORE):
			return false;
		case(WHITE_NONE_TO_SKIP):
		case(WHITE_SKIPPED_ALL):
			break;
	}
	size_t tmpStrPtr = beginWordPtr = strPtr;//!\" \" or other char
	if(firstNoneWhite == '"'){
		isQuoted  = true;
		quoteChar = '"';
	}
	else if(firstNoneWhite == '\''){
		isQuoted = true;
		quoteChar = '\'';
	}
	while(tmpStrPtr < length - 1){
		++tmpStrPtr;
		currentChar = str(tmpStrPtr);
		if(htmlIsWhiteSpace(currentChar) && !isQuoted){
			strPtr = tmpStrPtr;
			return true;
		}
		else if(isQuoted && currentChar == quoteChar){
			strPtr = tmpStrPtr + 1;
			return true;
		}
	}
	return  false;
}

bool	wordEquals(RVCString& str, size_t strPtr, size_t wordPtr,const char* strChar)
{
	size_t i = 0;
	for(size_t ptr = wordPtr; ptr < strPtr; ++ptr)
		if(str(ptr) != strChar[i++])
			return false;
	return true;
}

void appendNoNull(RVCString& obj,  const char* str, size_t& newLen)
{
	if(!str)
		return;
	int nullCount = 0, nullLoc;
	const char* endStr = str + newLen;
	if(strlen(str) == newLen){
		obj.append(str, newLen);
		return;
	}
	else {
		do{
			nullLoc = strlen(str);
			obj.append(str, nullLoc);
			++nullCount;
			str += nullLoc + 1;
		}
		while(str < endStr);
	}
	newLen -= nullCount;
}



