/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// WLGenericConnection.h: interface for the CWLGenericConnection class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_WLGENERICCONNECTION_H__24B682BF_9CB0_4481_91E5_E74EDC3B8BBC__INCLUDED_)
#define AFX_WLGENERICCONNECTION_H__24B682BF_9CB0_4481_91E5_E74EDC3B8BBC__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "gethostip.h"
#include <collectable.h>
#include <ordered.h>
#include <RVCString.h>


#define MAX_SESSIONS 15

#define UNKNOWN_PROTOCOL 0
#define HTTP_PROTOCOL    1
#define SSL_PROTOCOL     2
#define JAVASCRIPT_PROTOCOL    3

/*#ifndef TRANSPORT_LAYER_TRACE
#define TRANSPORT_LAYER_TRACE
#endif // TRANSPORT_LAYER_TRACE*/


class CWLHttpRequest;
class CWLTransactionData;
class CWLWorkspace;
class CWLCommandManager;
class CWLConnectionsPool;
class CWLTransactionsObject;

/*
 *	Error code enumerator type.
 */
typedef enum{
	SSL_NO_ERROR=0,
	SSL_ERROR_
} SSL_ERROR_CODE;

typedef enum{
	DEFAULT=0,
	ALL,
	LOW,
	MEDIUM,
	HIGH
} SSL_CRYPTO_STRENGTH;

class CWLGenericConnection :  public RVCollectable
{
public:
	/* Constructor and destructor */
	CWLGenericConnection( CWLHttpRequest* pHttpReq );
	virtual ~CWLGenericConnection();

	/* Public methods */
public:
	virtual bool Init( CWLCommandManager* cm, CWLConnectionsPool* cp, int socketDescriptor, bool createNewSession = false, const char* certFileName = NULL )=0;
	virtual bool Init( CWLCommandManager* cm, CWLConnectionsPool* cp, sockaddr_in& addr, const char* host, int port );
	virtual int Connect()=0;
	virtual int Send( const char* buffer, int length, int threadNum = 0 )=0;
	virtual int Recv( char* buffer, int* buffsize, int flags, int* err_code, bool* closed, int threadNum = 0 )=0;
	virtual void CloseConnection()=0;
	virtual void CleanUp()=0;

	virtual bool SetCiphers( int strength = ALL );

	static CWLGenericConnection* ChangeConnectionProtocol( int protocol, sockaddr_in& address, const char* host, int port );

protected:
	int EstablishTcpConnection( sockaddr_in& address, RVCString& host, int port, bool usingProxy, bool& timedOut );
	virtual int EstablishConnection( RVCString& host, int port, bool usingProxy, int threadNum );

public:
	int OpenConnection( RVCString& host, int port, bool usingProxy, int threadNum );

	virtual int SetCryptographicStrength ( int strength );// Does nothing. Is redefined in CWLSecureSslConnection

	bool IsMatching(sockaddr_in& address, const char* host, int port);
	bool IsAvailable() { return m_Available; }
	bool IsOpen() { return m_TcpConnected; }
	void Lock() { m_Available = false; }
	void Unlock() { m_Available = true; }
	void Release() { Unlock(); }
	bool isActive(int threadNum = 94);

	void SetTransactionObj( CWLTransactionsObject* pTransObj ) { m_pTransObj = pTransObj; }
	void SetTransactionData( CWLTransactionData* pTransData ) { m_pTransData = pTransData; }
	CWLTransactionData* GetTransactionData() { return m_pTransData; }

	int GetHttpStatus()  { return m_httpStatus; }
	void ClearHttpStatus() { m_httpStatus=0; }
	void SetUrlType(int urlType) { m_UrlType = urlType; }
	CWLTransactionsObject* GetTransObj() {return m_pTransObj ;}
	CWLHttpRequest* GetHttpRequest(){return m_pHttpRequest;}
	void SetHttpRequest(CWLHttpRequest* pHttpRequest){m_pHttpRequest = pHttpRequest;}
	CWLCommandManager* GetCM() {return m_Cm;}

	int SetSocketMode(int mode);
	void CloseConnection(bool forceClose, int threadNum = 94);
	void CloseSocket();

	int SelectForRead(int *errnum, int inputTimeout);
	int SelectForWrite(int *errnum, int inputTimeout);
	static int LastError();
	int ReportError(const char* frmName, int frmId, ...);
	void SetErrorCode(int err);

public:
	static CWLGenericConnection* GetConnection( int protocol, const char* host, int port, int threadNum, CWLHttpRequest* pHttpReq );
	static unsigned GetHostByName(const char *host, char* *addresses, short* length, int* err);
	static char* ip(sockaddr_in* addr);
	char* ip();
	int port(){ return m_Port; }
	int sock(){ return m_Socket; }
	void setSocket( int socket );

	/* Data members */

protected:
	int m_Socket;
	int m_Port;

	bool m_TcpConnected;

protected:
	CWLCommandManager* m_Cm;
	CWLConnectionsPool* m_Cp;
	CWLTransactionData* m_pTransData;
	CWLWorkspace* m_Ws;
	CWLHttpRequest* m_pHttpRequest;
	CWLTransactionsObject* m_pTransObj;
	int m_httpStatus;
	int m_UrlType;


	RVCString m_Host;
#ifdef __TV_WINDOWS__
	sockaddr_in m_SockAddress;
#endif// __TV_WINDOWS__

	RVCString m_ErrorString;
	RVCString m_ErrorCode;

	bool m_Available;
	int m_BlockMode;

public:
	int m_Index; // reserved for debugging
	int m_Protocol;

private:
	static RVOrdered m_Session;

};

#endif // !defined(AFX_WLGENERICCONNECTION_H__24B682BF_9CB0_4481_91E5_E74EDC3B8BBC__INCLUDED_)
