/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __HTTP_COOKIES__
#define __HTTP_COOKIES__

#include <time.h>
#include <RVCString.h>
#include <collectable.h>
#include <bintree.h>
#include <thr.h>
#include <thr.h>


#include "platform.h"
#include "workspace.h"

#define COOKIE_RESPONSE "Set-cookie"


class HTTPCookie : public RVCollectable  {
private:
  RVCString  name, value;
  RVCString  path;
  RVCString  domain;
  time_t     expire;

public:
  HTTPCookie(RVCString& n, RVCString& v, RVCString& p, RVCString& d, time_t t)
    : name(n), value(v), path(p), domain(d), expire(t) {}
  HTTPCookie(RVCString& n, RVCString& p, RVCString& d)
    : name(n), path(p), domain(d) {}
  virtual ~HTTPCookie()  {}

  RVCString& getName() { return (name); }
  RVCString& getValue() {return (value); }
  RVCString& getPath() { return (path); }
  RVCString& getDomain() { return (domain); }
  time_t     getExpire() { return (expire); }

  virtual int compareTo(const RVCollectable* a) const;
};


class DllExport CWLCookies : public RVBinaryTree  {
private:
	RVReadersWriterLock m_CookieLock;
  CWLWorkspace*  m_workspace;

public:
  CWLCookies(CWLWorkspace* workspace) : m_workspace(workspace) {}
  virtual ~CWLCookies() { clearAndDestroy(); }


  int  parseCookieIntoTree(RVCString& response, 
													 RVCString& host, 
													 RVCString& uri,
                           bool       CheckExpiration,
													 int&       errCode);

  int  makeCookieRequest(RVCString& host, 
                         RVCString& uri, 
                         RVCString& cookie,
                         bool       CheckPath,
                         bool       CheckDomain,
                         bool       CheckExpiration);

  int  InsertCookie(RVCString& n, RVCString& v, RVCString& p, RVCString& d, RVCString& t);
  void ClearAllCookies();
  int  DeleteCookie(RVCString& name, RVCString& path, RVCString& domain);
};

#endif // __HTTP_COOKIES__
