/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************************
*
*   Author  : Itaj Sherman & Yossi Nakash
*   Date    : 10.07.20000
*   File    : HttpEmbedderReport.h
*   Purpose : Classes calling to callback methods in iHttpProbingClient.
*   Classes : cHttpEmbedderReportInfo - super class for situations in callbacks.
*             cHttpEmbedderReportInfo... - callback classes which call virtual 
*               methods in iHttpProbingClient.
*
*******************************************************************************/

#ifndef _HTTP_EMBEDDER_REPORT_H_
#define _HTTP_EMBEDDER_REPORT_H_

#include "loadengineglobals.h"
#include "rtamscript.h"
#include "loadreportinfo.h"
#include "wlhtmlvirtualqueue.h"
#include "callbackdata.h"


/******************************************************************************
*
*   Class   : cHttpEmbedderReportInfo
*   Purpose : Super class for callback classes (classes for indicating event).
*
*******************************************************************************/
class cHttpEmbedderReportInfo: public iSCActionReportInfo 
{

public:
  explicit cHttpEmbedderReportInfo(){}
		
protected:

};


/******************************************************************************
*
*   Classes : cHttpEmbedderReportInfo...
*   Purpose : Each class indicates different event, and is calling the 
*             iHttpProbingClient virtual method in the method fApply.  
*             fApply is called by iLoadProbingclient::fEmbedderReporter.
*
*******************************************************************************/
class cHttpEmbedderReportInfoPageStartDownload: public cHttpEmbedderReportInfo
{

public:
  explicit cHttpEmbedderReportInfoPageStartDownload(CUrlData* pUrlData, long lExpectedDom) : m_pUrlData(pUrlData), m_lExpectedDomId(lExpectedDom), m_lActiveDomId(0){}
  long GetDomId() {return m_lActiveDomId;}

protected:
  CUrlData* m_pUrlData ;
  long      m_lActiveDomId ;
  long      m_lExpectedDomId ;

private:
  virtual void fApply(
    tSCActionReporterReturn&,
    iLoadSingleClient& pPBC,
     iReportThreadState* threadState);
};


class cHttpEmbedderReportInfoPageAbortDownload: public cHttpEmbedderReportInfo
{

public:
  explicit cHttpEmbedderReportInfoPageAbortDownload(CUrlData* pUrlData) : m_pUrlData(pUrlData){}
	
protected:
  CUrlData* m_pUrlData ;

private:
  virtual void fApply(
    tSCActionReporterReturn&,
    iLoadSingleClient& pPBC,
     iReportThreadState* threadState);
};


class cHttpEmbedderReportInfoPageEndDownload: public cHttpEmbedderReportInfo
{

public:
  explicit cHttpEmbedderReportInfoPageEndDownload(CUrlData* pUrlData) : m_pUrlData(pUrlData){}
	
protected:
  CUrlData* m_pUrlData ;

private:
  virtual void fApply(
    tSCActionReporterReturn&,
    iLoadSingleClient& pPBC,
     iReportThreadState* threadState);
};


class cHttpEmbedderReportInfoObjectStartDownload: public cHttpEmbedderReportInfo
{

public:
  explicit cHttpEmbedderReportInfoObjectStartDownload(CUrlData* pUrlData) : m_pUrlData(pUrlData){}

protected:
  CUrlData* m_pUrlData ;
 
private:
  virtual void fApply(
    tSCActionReporterReturn&,
    iLoadSingleClient& pPBC,
     iReportThreadState* threadState);
};


class cHttpEmbedderReportInfoObjectAbortDownload: public cHttpEmbedderReportInfo
{

public:
  explicit cHttpEmbedderReportInfoObjectAbortDownload(CUrlDataEx* pUrlDataEx) : m_pUrlDataEx(pUrlDataEx){} 
	
protected:
  CUrlDataEx* m_pUrlDataEx ;

private:
  virtual void fApply(
    tSCActionReporterReturn&,
    iLoadSingleClient& pPBC,
     iReportThreadState* threadState);
};


class cHttpEmbedderReportInfoObjectEndDownload: public cHttpEmbedderReportInfo
{

public:
  explicit cHttpEmbedderReportInfoObjectEndDownload(CallbackData* pData) : m_pData(pData){}
	
protected:
  CallbackData* m_pData ;

private:
  virtual void fApply(
    tSCActionReporterReturn&,
    iLoadSingleClient& pPBC,
     iReportThreadState* threadState);
};

class cHttpEmbedderReportInfoUseCachedObject: public cHttpEmbedderReportInfo
{

public:
  explicit cHttpEmbedderReportInfoUseCachedObject(CUrlDataEx* pUrlDataEx) : m_pUrlDataEx(pUrlDataEx){}
 
protected:
  CUrlDataEx* m_pUrlDataEx ;

private:
  virtual void fApply(
    tSCActionReporterReturn&,
    iLoadSingleClient& pPBC,
     iReportThreadState* threadState);
};




#endif //_HTTP_EMBEDDER_REPORT_H_
