/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/*
  This header file defines the implementations of all the protocol classes,
  except for the inheritor of iLEPRotocolVCInMix, which is the command manager and is thus defined there.
*/

#ifndef __HTTPProtocol_H
#define __HTTPProtocol_H

#include "wlleprotocol.h"
#include "platform.h"
#include "tvtables.h"

#ifdef __TV_WINDOWS__
#include "xmlerrorreporter.h"
#endif // __TV_WINDOWS__

class CHLProtocolProcess;
class CHLProtocolSession;
class CHLProtocolAgendaInMix;
class CHLProtocolAgendaMachineThread;
class CHLProtocolVCInMix;

class CLoadGeneralThreadInstanceScript;
class CLoadGeneralThreadInstance;

class CWLCommandManager;

//The Protocol global static pointer.  
extern CHLProtocolProcess* piHLProtocolProcess;

//The function 'NewProtocolProcces' is the only function that will be exported 
//from the dll. The function will create an instance of a class that 
//inherits the base class iLEProtocolProcess, and return a pointer to
// the interface 'iLEProtocolProcess'.
ExportLEProtocol iLEProtocolProcess* NewProtocolProcess();


////////////////////////////////////////////////////////////////
//////////////// class CHLProtocolProcess //////////////////////
////////////////////////////////////////////////////////////////


//the protocol will have one class like this for each Process.
class CHLProtocolProcess : public iLEProtocolProcess
{
public:
	explicit CHLProtocolProcess();
public:
	~CHLProtocolProcess();

public:
	// the functions of the class iLEProtocolProcess
	virtual iLEProtocolSession& NewProtocolSession();
	virtual iLEProtocolAgendaMachineThread& NewProtocolAgendaMachineThread(WLProtocolThreadType);
	virtual iLEProtocolJSRuntime& NewProtocolJSRuntime();
	virtual bool ProcessInitialize();
	virtual bool ProcessInitializeJava(cRVJVM&, JNIEnv*);
	virtual bool ProcessFinalizeJava(cRVJVM&, JNIEnv*);
	virtual bool ProcessTermination();

// HttpLoad stuff
  void SetCallbackEnabled(bool aOn) { m_CallbackEnabled = aOn;};
  bool GetCallbackEnabled() { return m_CallbackEnabled;};

private:
  bool  m_CallbackEnabled;

};





////////////////////////////////////////////////////////////////
//////////////// class CHLProtocolSession //////////////////////
////////////////////////////////////////////////////////////////


//the protocol will have one class like this for each Session.
class CHLProtocolSession : public iLEProtocolSession
{
public:
	explicit CHLProtocolSession(iLEProtocolProcess& father);
	~CHLProtocolSession();

public:
	// the functions of the class iLEProtocolSession
	virtual iLEProtocolAgendaInMix& NewProtocolAgendaInMix();
	virtual bool SessionInitialize();
	virtual bool SessionTermination();

};




////////////////////////////////////////////////////////////////
//////////////// class CHLProtocolJSRuntime ////////////////////
////////////////////////////////////////////////////////////////

class CHLProtocolJSRuntime : public iLEProtocolJSRuntime
{
public:
	explicit CHLProtocolJSRuntime(iLEProtocolProcess & father);
	~CHLProtocolJSRuntime();

public:
	virtual bool JSRuntimeInitialize(JSRuntime* rt);
	virtual bool JSRuntimeAfterNewMainContext(JSContext* cx);
	virtual bool JSRuntimeBeforeDestroyMainContext(JSContext* cx);

private:
	JSRuntime* m_JSRuntime;
};




////////////////////////////////////////////////////////////////
//////////////// class CHLProtocolAgendaInMix //////////////////
////////////////////////////////////////////////////////////////

class CHLProtocolAgendaInMix : public iLEProtocolAgendaInMix
{
public:
	explicit CHLProtocolAgendaInMix(iLEProtocolSession & father);
	~CHLProtocolAgendaInMix();

public:
	// the functions of the class iLEProtocolAgendaInMix
	virtual iLEProtocolVCInMix& NewProtocolVCInMix();//??
	virtual bool BeforeInitAgenda();
	virtual bool AfterTerminateAgenda();
  virtual bool JSInitializeForScript(JSContext* cx, JSObject* pAgendaGlobal, JSObject* pGeneralGlobal);
  virtual bool JSTerminateForScript(JSContext* cx, JSObject* pAgendaGlobal, JSObject* pGeneralGlobal);
};



////////////////////////////////////////////////////////////////
////////////// class CHLProtocolAgendaMachineThread ////////////
////////////////////////////////////////////////////////////////

class CHLProtocolAgendaMachineThread : public iLEProtocolAgendaMachineThread
{
public:
	explicit CHLProtocolAgendaMachineThread(iLEProtocolProcess& father, WLProtocolThreadType);
	~CHLProtocolAgendaMachineThread();
public:
	// the functions of the class iLEProtocolAgendaInMix
	virtual bool MachineThreadInitialize();
	virtual bool MachineThreadTermination();//to do
};


////////////////////////////////////////////////////////////////
//////////////// class CHLXMLErrorReporter /////////////////////
////////////////////////////////////////////////////////////////

// @@ofer until we will get rid of the xmlparser module
#ifdef __TV_WINDOWS__
class CHLXMLErrorReporter : public iXMLVirtualErrorReporter
{
public:
  explicit CHLXMLErrorReporter();
  virtual ~CHLXMLErrorReporter();
private:
  bool vReport(JSContext* cx, WLLogMessage::Severity, const char*);
};
#else
class CHLXMLErrorReporter
{
public:
  explicit CHLXMLErrorReporter();
  virtual ~CHLXMLErrorReporter();
private:
  bool vReport(JSContext* cx, WLLogMessage::Severity, const char*);
};
#endif // __TV_WINDOWS__


#endif // __HTTPProtocol_H


