/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : HttpRequest.h
 *
 *	Author : Zion Menahem
 *
 *	Description : 
 *
 *	History : 
 *
 *	Modifications : 
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/
#ifndef _HttpRequest_h_
#define _HttpRequest_h_

/** include files **/
#include <stdio.h>
#include <string.h>
#include <fstream.h>
#include <ordered.h>
#include <collstr.h>
#include <thr.h>

#include <platform.h>
#include <tvpsync.h>
#include "loadengineglobals.h"
#include "wlframecontainer.h"
#include "httpresponse.h"

#include "rvjsapi.h"

#include "httphost.h"
#include "wlerrorlib.h"
#include "wlurlbase.h"
#include "callbackdata.h"
#include "wlhttpcache.h"


#define BREAK    1
#define CONTINUE 2

/** local definitions **/
#define CWLHttpValue RVCString
#define CWLPage frameContainer

typedef enum
{
  Unreachable=0,
  Reachable=1
} StatusConn;

typedef enum
{
  Master,
	Slave
} THREAD_MODE;

#ifdef __TV_WINDOWS__
enum AUTH_STATUS;
#endif


/** default settings **/

/** external functions **/
extern void  ParseCharForHttp();

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/

/** public functions **/

/** private functions **/

/** classes definitions **/
class	CWLCommandManager;
class	CWLWorkspace;
class CWLConnectionsPool;
class	CWLConnection;
class	CWLCookies;
class	CWLPage;
class CWLClientAuthentication;
class CWLTransactionsObject;
class CWLMultiPart;
class CWLHttpCacheEntry;

#ifdef __OPEN_SSL__
class CWLGenericConnection;
#endif // __OPEN_SSL__


class myCollectableStr : public RVCollectableString {
public:
  myCollectableStr() {}
  myCollectableStr(const RVCString& s) : RVCollectableString(s) {}
  myCollectableStr(const char* c) : RVCollectableString(c) {}
  myCollectableStr(const RVCSubString& ss) : RVCollectableString(ss) {}
  virtual ~myCollectableStr() {}

  virtual bool isEqual(const RVCollectable* c) const;
};




class WLBlock : public RVCollectable {
private:
public:
	RVCString _data;

	WLBlock() {}
	WLBlock(char* string) { _data = string; }
	virtual ~WLBlock() {}
	virtual int compareTo(const RVCollectable* rwc) const { return _data.compareTo(((WLBlock*)rwc)->_data); }
	virtual bool isEqual(const RVCollectable* rwc) const { return (compareTo(rwc) == 0 ? TRUE : FALSE); }
	virtual unsigned hash() const { unsigned sum = 0; size_t i = _data.length(); while (i > 0) sum += (unsigned)_data[--i]; return sum; }
	void AddString(char* string) { _data += ", "; _data += string; }
};


// handle the HTTP protocol
class CWLHttpRequest {

#ifndef __GNUC__
  // ATTENTION GNU compiler does not know how to handle static enum definitions
  // so we omit the static ONLT from GNU 
  static 
#endif //__GNUC__
  enum{ AUTH_NONE, AUTH_BASIC, AUTH_NTLM, AUTH_BASIC_NTLM};

private:

	CWLCommandManager*  m_Cm;	// pointer to the command manager
	CWLWorkspace*       m_Ws;	// pointer to the workspace's variables
	CWLConnectionsPool* m_ConnPool;	// pointer to the connections pool

#ifndef __OPEN_SSL__
	CWLConnection*      m_Connection;	// pointer to the thread's connection
#else
	CWLGenericConnection* m_Connection;
	CWLGenericConnection* m_TmpSslConnection;
#endif // __OPEN_SSL__


//	CWLSslConnection*   m_Ssl;	// pointer to the SSL object
	CWLCookies*         m_Cookies;	// pointer to the cookies object
	CWLPage*			      m_Page;		// pointer to the parsed page
	CWLDocument*			  m_Frame;	// pointer to the parsed frame
  CWLTransactionsObject* m_pTransObj ; // pointer to transaction object

	CWLHttpCacheEntry*  m_pCacheEntry;   // pointer to the cached object (if exist)
  CallbackData        m_CBData ;
  bool                m_CBDataClosed ;

  bool                m_ExplicitUrl ; //Indicates that the current fetched url 
                                      //was written explicitly in the agenda 
                                      //and is not fetched because of GetContents

  int                m_Version;	// HTTP version (1.0/1.1)

	int                m_ThreadNum;
	THREAD_MODE        m_ThreadMode;
	HtmlQueueItemType m_UrlType;
  RVCString          m_Url;
  RVCString          m_FullUrl;
  int                m_protocol;
  int                m_previousProtocol;
  RVCString          m_host;
  int                m_port;
  RVCString          m_uri;

  RVCString          m_server_host;
  RVCString          m_server_uri;
	int                m_server_port;
	// for Keep-Alive
	bool               m_RequestKeepAlive;
	bool               m_ResponseKeepAlive;

	CWLHttpResponse    m_Response;

  bool               m_usingProxy;

	bool							 m_DoRedirections;
	int                m_NumRedirections;

	bool               m_AuthInit;
	bool               m_WaitForNTResponse;
	bool               m_DuringChallengeResponse;
  bool               m_bAuthDone ;
  bool               m_bPAuthDone ;

  RVCString          m_headers;
  RVCString          m_form_data;
  RVCString          m_content_type;
	RVCString          m_dataFilename;
	long               m_dataLength;

	// NT-Authentication
	int                m_Iteration;
	int                m_Phase;
	bool               m_Exhausted;
	CWLClientAuthentication*  m_Authorization;

	// for file parsing
	RVCString          m_Path;

  //
  urlBase            m_Base; // an urlBase for redirection calculation

  char*              m_pRecvBuff; // allocated dynamically according to workspace size
  int                m_nRecvBufferSize;
  bool               m_bFirstTimeWait ;
  CWLMultiPart*      m_pMultiPart ;

public:
  double             m_dStartReceive ;
  double             m_dStartWait ;

private:
	void        startSlave();

  char*       TraceRecvBuff(int Len, char* Buff);
	int         Head_Get(HttpFuncs method);
	int         Post(HttpFuncs method); 
        void        FormatFormdataForDisplay(RVCString formdata, RVCString& retBuff);
	int         Connect(const char* method);

	int         GetConnTimeout();
	int         RedirectionHandler();
  int         AuthorizationInitialization(const char*& authValue,bool* exhaust, int* err);
	int        AuthorizationHandler(bool* exhasted, int* err);
  bool        AuthorizationBASIC(const char* username, const char* password, const char* Header, RVCString& Authentication);
#ifdef __TV_WINDOWS__
  bool        AuthorizationNTLM(int* err, const char* authValue, const char* username, const char* password, const char* Header, RVCString& Authentication);
#endif
	bool        CacheHandler( HttpFuncs p_eMethod, RVCString& p_sUrl, HtmlQueueItemType p_uObjType, bool& p_bFreshObject, bool& p_bPartialImage );
	int         FetchUrl( HttpFuncs method, bool inRetry = false );
	int         DoHttpCommand( HttpFuncs method, HtmlQueueItemType p_uObjType );

	int         ReadData(CWLWorkspace*);
	void        initLocalVariables();
	int         ParseHeader();
	int         ParseProxyHeader(RVCString& Headers);
	int         ParseFormData();
	int					ParseDataBlocks();

	int         CollectFormData(RVCString& query);

	void        ParseCharFunc(const char* , RVCString& );
	int         ReadResponse(HttpFuncs method, double*);

	int         DetermineProtocol(RVCString& url, int *errnum);
	int         ParseUri();
	void        GetHostAndUri(const RVCString& Url, RVCString& lhost, RVCString& luri, int& lport);

  //Galtech
#ifndef _ANSI_ONLY
	int			GetServerEncoding();
	RVCString   ConvertToServerEncoding(const char* str);
#endif

  
	int					acquireConnection();
	void        releaseConnection();
	int         openConnection();
	void        closeConnection(bool force = false);
	int         selectForRead(int* errnum, int inputTimeout);
	int         NewselectForRead(int inputTimeout);
	int	        send(const char* buff, int len, int protocol);
	int         recv(char* buff, int* len, int protocol, bool* closed, int* errnum);
  const char* GetReferer() const;

	CWLHttpCache&  getHttpCache();
	void           updateCachedHeadersAndContent( CWLHttpCacheEntry* m_pCacheEntry );
	int            retrieveAndSetCachedContent( CWLHttpCacheEntry* p_pCacheEntry );

#ifdef __TV_WINDOWS__
	AUTH_STATUS authorizationInit();
	void        authorizationTerminate();
	void        authorizationClean();
	void        authorizationReset();
  // Chaim - Passing filter as arg to allow us to add filters for sending data to IDE in callback in the future (20/11/06) 
  bool        ShouldNavigate(RVCString url, const char* filter);
#endif

public:
  CWLHttpRequest();
  ~CWLHttpRequest();

	bool        Init(CWLCommandManager*, CWLConnectionsPool*, int);
	void        SetFrame(CWLDocument* frame) { if (m_ThreadMode == Master) m_Frame = frame; }
	void        SetMaster() { m_ThreadMode = Master; }
	void        SlaveThread();
	int         CondDoCommand(HttpFuncs		method, 
														bool					first,
														CWLTransactionsObject*	pTransObj, 
                            HtmlQueueItemType objType,
														RVCString&		path, 
														int&					retStatus);

	int         DoCommand( HttpFuncs method, HtmlQueueItemType p_uObjType );

  void        PrintParseTree();
	virtual int	ParseResponse (const char* recv_buff, 
														 const int&  RecieveSize, 
														 HttpFuncs method,
														 int&				 ContentCurrentLen,
														 bool&			 Finished,
														 double&		 ProcessTime,
                             RVCString&  sBuffer);
  CWLDocument* const GetFrame(){return m_Frame;}
  bool         IsFirstTimeWait(){return m_bFirstTimeWait;}
  void         SetFirstTimeWait(bool bSet){m_bFirstTimeWait=bSet;}
  bool         GetAuthState(){return m_DuringChallengeResponse;}

  CWLCommandManager* GetCm()  {return m_Cm;}

  void        CloseCBData();
};


#endif _HttpRequest_h_
 
