/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __HTTPSSL_H__
#define __HTTPSSL_H__

#include <hashdict.h>

class CWLCommandManager;
class CWLConnectionsPool;
class CWLDBRef;


#define UNKNOWN_PROTOCOL 0
#define HTTP_PROTOCOL    1
#define SSL_PROTOCOL     2

#ifndef __SSL__
#define SSLContext      char 
#define SSLDHParams     char

#else
#include "generalsslfunctions.h"

#define X509NoCertSupplied -6973
#define X509BadCertificate -6972
#define X509CertificateFile -6971

#endif //  __SSL__

#include "wlerrorlib.h"

int         SSLDetermineVersion(const char* version);
#ifdef __SSL__

const RVCString SSL_CERT_NO_VERIFICATION = "SSL_CERT_NO_VERIFICATION";
const RVCString SSL_CERT_VALIDATE_ONLY = "SSL_CERT_VALIDATE_ONLY";
const RVCString SSL_CERT_VERIFY_ALL = "SSL_CERT_VERIFY_ALL";
#endif



// this class holds template ssl context (SSLContext)
// which will be duplicated for each transaction.
// keys that will identify each object are:
// protocol version, resumable (session reference), certificate filename
class CWLSSLContext : public RVCollectable {
private:
	CWLConnectionsPool* m_Cp;

	SSLContext*         m_Ctx;
  SSLDHParams         m_DHParams;
  void*               m_Random;

public:
	CWLDBRef*           m_DBRef;
	RVCString		        m_CertificateFile;
	int                 m_Version;
	int                 m_CryptographicStrength;
	bool                m_Resumable;

	// for SSL errors handling
	RVCString               m_ErrorString;
	WLLogMessage::Severity  m_ErrorSeverity;

public:
	CWLSSLContext();
	CWLSSLContext(CWLConnectionsPool* cp);
	~CWLSSLContext();

	int          Init(int         version, 
                    bool        resumable, 
                    const char* certFile, 
                    const char* certPassword, 
                    const char* certVerification, 
                    int*        errCode,
                    int         CryptoStrength);

	bool         isMatching(int         version, 
                          bool        resumable, 
                          const char* certFile,
                          int         CryptoStrength);
	bool         isResumable() { return m_Resumable; }
	SSLContext*  GetContext() { return m_Ctx; }
	static int   DetermineVersion(const char* version);
	static int   DetermineCryptographicStrength(const char* CryptoStrength);
};




// this is a duplicated ssl context object. it should be created when
// the socket is opened, and destroyed when the connection is closed.
class CWLSslConnection {
private:
  // SSL variables.
  SSLContext*     m_Ctx;	// duplicated context
	CWLSSLContext*  m_Template;	// template context

	in_addr  m_Address;
	int      m_Port;

	bool     m_Closed;

public:
	// for SSL errors handling
	RVCString               m_ErrorString;
	WLLogMessage::Severity  m_ErrorSeverity;

public:
	CWLSslConnection() { m_Ctx = NULL; m_Template = NULL; m_Closed = true; }
	CWLSslConnection(CWLSSLContext* ctx, in_addr addr, int port);
	~CWLSslConnection();

/*
	int              Init(int prt, int sslpver, RVCString* certVerification,
																	 CWLConnectionsPool* cp,
																	 const char* certFile, const char* certPassword);
*/
	void             Init(CWLSSLContext* ctx, in_addr addr, int port);
	int	             AfterConnection(void *ioRef);
	SSLContext*      GetCtx() { return m_Ctx; }

	void             Close();
	void             Delete();
	bool             SetError(int sslErr, int errCode = 0);
};

#endif //  __HTTPSSL_H__
