/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// WLSecureSslConnection.cpp: implementation of the CWLSecureSslConnection class.
//
//////////////////////////////////////////////////////////////////////
#ifdef __OPEN_SSL__

//#include <Winsock2.h>
#include <OpenSSL\symhacks.h>

#include "tracedll.h"
#include "WLSecureSslConnection.h"
#include "WLSslCiphersCollection.h"

#include "httpdefines.h"
#include "commandmanager.h"
#include "workspace.h"
#include "wltransactiondata.h"
#include "connpool.h"


#define max( a, b ) ( ( (a) > (b) ) ? (a) : (b) )
#define min( a, b ) ( ( (a) < (b) ) ? (a) : (b) )

//////////////////////////////////////////////////////////////////////
// this callback function is used by OpenSSL library internally
//////////////////////////////////////////////////////////////////////
int pem_passwd_cb(char *buffer, int size, int rwflag, void *password)
{
	strncpy(buffer, (char *)(password), size);
	buffer[size - 1] = '\0';
	return(strlen(buffer));
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWLSecureSslConnection::CWLSecureSslConnection( CWLHttpRequest* pHttpReq )
: CWLGenericConnection( pHttpReq )
, m_Context( NULL )
, m_Ssl( NULL )
, m_Method(NULL)
, m_SslConnected( false )
{
	m_CryptoStrength = "ALL:COMPLEMENTOFALL";
	m_Protocol = SSL_PROTOCOL;
	

	//////////////////////////////////////////////////////////////////////////
#ifdef TRANSPORT_LAYER_TRACE
	char fname[256] = {'\0'};
	sprintf( fname, "V:\\Log\\ssl_resp_%d.htm", m_FileNum );
	m_FileNum++;
	m_File = fopen( fname, "w" );
#endif // TRANSPORT_LAYER_TRACE
	//////////////////////////////////////////////////////////////////////////	
}

CWLSecureSslConnection::CWLSecureSslConnection( CWLGenericConnection* pConnection )
: CWLGenericConnection( *pConnection )
, m_Context( NULL )
, m_Ssl( NULL )
, m_Method(NULL)
, m_SslConnected( false )
{
	m_CryptoStrength = "ALL:COMPLEMENTOFALL";
	m_Protocol = SSL_PROTOCOL;

	//////////////////////////////////////////////////////////////////////////
#ifdef TRANSPORT_LAYER_TRACE
	char fname[256] = {'\0'};
	sprintf( fname, "V:\\Log\\ssl_resp_%d.txt", m_FileNum );
	m_FileNum++;
	m_File = fopen( fname, "w" );
#endif // TRANSPORT_LAYER_TRACE
	//////////////////////////////////////////////////////////////////////////
}

CWLSecureSslConnection::~CWLSecureSslConnection()
{
  CWLGenericConnection::CloseConnection(true);
	CleanUp();

	m_SslConnected = false;

	//////////////////////////////////////////////////////////////////////////
#ifdef TRANSPORT_LAYER_TRACE
	if( m_File )
		fclose( m_File );
#endif // TRANSPORT_LAYER_TRACE
	//////////////////////////////////////////////////////////////////////////
	if( NULL != m_Ssl ){
		SSL_free( m_Ssl );
		m_Ssl = NULL;
	}
	
	if( NULL != m_Context ){
		SSL_CTX_free( m_Context );
		m_Context = NULL;
	}
}

bool CWLSecureSslConnection::Init( CWLCommandManager* cm, CWLConnectionsPool* cp, int sd, bool createNewSession/*=false*/, const char* certFileName/*=NULL*/ )
{
	int err = 0;

	m_Cm = cm;
	m_Cp = cp;
	
	if( createNewSession ){
		m_Method = SSLv3_client_method();
		m_Context = SSL_CTX_new( m_Method );
		if( NULL == m_Context )
			return false;

		if( ( NULL != certFileName ) && ( '\0' != certFileName[0] ) ){
			if ( SSL_CTX_use_certificate_file( m_Context, certFileName, SSL_FILETYPE_PEM ) <= 0 ){
				return false;
			}
		}
		if( m_Ws->m_CertificatePassword && '\0' != m_Ws->m_CertificatePassword[0] )
		SSL_CTX_set_default_passwd_cb_userdata( m_Context, (void*)m_Ws->m_CertificatePassword );

		m_Ssl = SSL_new ( m_Context );
		if( NULL == m_Ssl )
			return false;
  }

	if( cp->GetCipherSuite()->Update( this ) )
		return false;

	m_Socket = sd;

	err = SSL_set_fd( m_Ssl, m_Socket );
	if( -1 == err )
		return false;

	return true;
}


bool CWLSecureSslConnection::Init( CWLCommandManager* cm, CWLConnectionsPool* cp, sockaddr_in& addr, const char* host, int port )
{
	return CWLGenericConnection::Init( cm, cp, addr, host, port );
}

bool CWLSecureSslConnection::InitSslConnection()
{
	int err = 0;

	// set the client method according to SslVersion
	if( NULL == m_Method )
		switch( m_Ws->m_SslVersion ){
		case SSL_Version_2_0:
			m_Method = SSLv2_client_method();
			break;

		case SSL_Version_3_0_Only:
		case SSL_Version_3_0:
			m_Method = SSLv3_client_method();
			break;

		case TLS_Version_1_0_With_2_0_Hello:
		case TLS_Version_1_0_Only:
		case TLS_Version_1_0:
			m_Method = TLSv1_client_method();
			break;

		default:
			m_Method = SSLv23_client_method();
		}
		
	if( NULL == m_Method )
		return false;

	// create the context
	if( NULL == m_Context )
		m_Context = SSL_CTX_new( m_Method );
	if( NULL == m_Context )
		return false;

	// tune the context
	switch( m_Ws->m_SslVersion ){
	case SSL_Version_3_0_Only:
		SSL_CTX_set_options( m_Context, SSL_OP_NO_SSLv2 | SSL_OP_NO_TLSv1  );
		break;
	case TLS_Version_1_0_Only:
		SSL_CTX_set_options( m_Context, SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 );
		break;
	}

	// set the certificate file
	if( ( NULL != m_Ws->m_CertificateFile ) && ( '\0' != m_Ws->m_CertificateFile[0] ) ){
		//if ( SSL_CTX_use_certificate_chain_file( m_Context, m_Ws->m_CertificateFile ) <= 0 )
		if ( SSL_CTX_use_certificate_file( m_Context, m_Ws->m_CertificateFile, SSL_FILETYPE_PEM ) <= 0 ){
			// bug #60 fixed - add error report
			ReportError( SSL_ERRORS_FILE, 6972  );
			return false;
		}

		// set password for private key if the private key is encrypted
		// bug #59 fixed - do not check the password
			SSL_CTX_set_default_passwd_cb( m_Context, pem_passwd_cb );
			SSL_CTX_set_default_passwd_cb_userdata( m_Context, (void*)m_Ws->m_CertificatePassword );
	
		int err = 0;
		//set private key file if any
		if( 1 == (err = SSL_CTX_use_RSAPrivateKey_file( m_Context, m_Ws->m_CertificateFile, SSL_FILETYPE_PEM )) || 
			1 == (err = SSL_CTX_use_PrivateKey_file( m_Context, m_Ws->m_CertificateFile, SSL_FILETYPE_PEM )) ){

			// check the consistency of a private key with the corresponding certificate 
			if( 1 != SSL_CTX_check_private_key( m_Context ) ){
				// bug #60 fixed - add error report
				ReportError( SSL_ERRORS_FILE, 6972  );
				return false;
			}
		} else {
			// bug #60 fixed - add error report
			ReportError( SSL_ERRORS_FILE, 6972  );
			return false;
		}
	}

	// create the SSL session
	if( NULL == m_Ssl )
		m_Ssl = SSL_new ( m_Context );
	if( NULL == m_Ssl )
		return false;

	// clear session before new connection
	int i = SSL_clear( m_Ssl );

	SSL_set_mode( m_Ssl, SSL_MODE_AUTO_RETRY );

	// TBD
	// set the peer certificate verification mode
/*	SSL_set_verify( m_Ssl, SSL_VERIFY_PEER, NULL );*/

	if( !m_Cp->GetCipherSuite()->Update( this ) )
		return false;

	// set the socket for SSL session
	err = SSL_set_fd( m_Ssl, m_Socket );
	if( -1 == err )
		return false;

	return true;
}



int CWLSecureSslConnection::Connect()
{
	int err = 0;

	if( NULL == m_Ssl ){
		m_SslConnected = false;
		return RET_ERROR;
	}

	if (m_Cm->isAbort())
		return IS_ABORT;

	err = SSL_connect ( m_Ssl );

	if( 1 != err ){
		int err_code = 0;
		err_code = SSL_get_error( m_Ssl, err );


		if( err_code == SSL_ERROR_SYSCALL )
			return RET_RETRY_FETCH;

		m_SslConnected = false;
		return RET_ERROR;
	}
		
	//TBD
/*	// check the results of peer certificate verification
	if( X509_V_OK != SSL_get_verify_result( m_Ssl ) )
		return RET_ERROR;*/

	err = GetAndCheckCertificateFromServer();
	if( RET_OK != err ){
		m_SslConnected = false;
		return RET_ERROR;
	}

	m_SslConnected = true;
	return RET_OK;
}

int CWLSecureSslConnection::EstablishConnection( RVCString& host, int port, bool usingProxy, int threadNum )
{
	if (m_Cm->isAbort())
		return IS_ABORT;

	int tcpStatus = CWLGenericConnection::EstablishConnection( host, port, usingProxy, threadNum );
	
	if( KEPT_ALIVE != tcpStatus && RET_OK != tcpStatus ){
		return RET_ERROR;
	}

	if( m_SslConnected )
		return KEPT_ALIVE;

	if( !InitSslConnection() )
		return RET_ERROR;

	return Connect();
}

int CWLSecureSslConnection::GetAndCheckCertificateFromServer()
{
	X509* server_cert = NULL;
	char* str = NULL;

	if( ( NULL == m_Ssl ) )
		return RET_ERROR;

	/* Get server's certificate (note: beware of dynamic allocation) - opt */
	server_cert = SSL_get_peer_certificate( m_Ssl );
	if( NULL == server_cert ){
		// check if the cipher with no authentication is being used
		// in this case no certificate is being sent by server
		SSL_CIPHER* currCipher = SSL_get_current_cipher( m_Ssl );
		if( NULL == currCipher )
			return RET_ERROR;

		if( ( currCipher->algorithms & RV_SSL_AUTH_MASK ) == RV_SSL_aNULL ){
			return RET_OK;
		}

		// else return error
		return RET_ERROR;
	}

	// get the certificate subject name if needed
	str = X509_NAME_oneline( X509_get_subject_name( server_cert ), 0, 0 );
	if( NULL == str )
		return RET_ERROR;


	OPENSSL_free (str);

	// get the certificate issuer name if needed
	str = X509_NAME_oneline( X509_get_issuer_name( server_cert ), 0, 0 );
	if( NULL == str )
		return RET_ERROR;

	OPENSSL_free( str );

	/*	We could do all sorts of certificate verification stuff here before
		deallocating the certificate. */

	X509_free (server_cert);

	return RET_OK;
}

int CWLSecureSslConnection::Send( const char* buffer, int length, int threadNum )
{
	int sentBytes = 0;	// number of bytes sent
	int done = 0;
	int lastRes = 0;
	RVCString tempbuf;
	int tempLen = 0;
	int nSendBytesCnt = 0;
	int nMaxSendSize = max( m_Ws->m_SendBufferSize, MIN_BUFFER_SIZE );
	bool bUseFixedBufSize = m_Ws->m_SendBufferSize != DEFAULT_SEND_BUFFER_SIZE;

	if (m_Ws->m_LowLevelUnescape){
        UnescapeString(buffer, length, m_Ws->m_LowLevelUnescape, m_Ws->m_EscapeChar, tempbuf);
		tempLen = tempbuf.length();
	}
	else {
		tempbuf = buffer;
		tempLen = length;
	}


	do {
		if (m_Cm->isAbort())
			return IS_ABORT;

		if( !m_SslConnected || ( NULL == m_Ssl ) )
			return RET_ERROR;

		sentBytes = 0;

		if( bUseFixedBufSize )
			nSendBytesCnt = min( nMaxSendSize, (tempLen-done) );
		else
			nSendBytesCnt = tempLen-done;

		sentBytes = SSL_write( m_Ssl, (const char*)(tempbuf.data()+done), nSendBytesCnt );

		if( sentBytes > 0 )
			done += sentBytes;
		else{
			if( 0 <= sentBytes ){
				SSL_get_error( m_Ssl, lastRes );
				printf( "\nSSL_write ERROR: %d.\n", lastRes );
				break;
			}
		}

	}while( sentBytes > 0 && done < length );

	if ( done != length){
		ReportError( HTTP_ERROR_FILE, 20 );
		return RET_ERROR;
	}

	return RET_OK;
}


int CWLSecureSslConnection::Recv( char* buffer, int* buffsize, int flags, int* err_code, bool* closed, int threadNum )
{
	int recvdBytes = 0;

	if( !m_SslConnected || ( NULL == m_Ssl ) )
		return RET_ERROR;

	if( m_Cm->isAbort() )
		return IS_ABORT;

	*err_code = 0;

	recvdBytes = SSL_read( m_Ssl, buffer, *buffsize );
	*buffsize = recvdBytes;

	//////////////////////////////////////////////////////////////////////////
#ifdef TRANSPORT_LAYER_TRACE
		if( m_File && recvdBytes > 0 ){
			fwrite( buffer, sizeof(char), recvdBytes, m_File );
		}
#endif // TRANSPORT_LAYER_TRACE
	//////////////////////////////////////////////////////////////////////////

	*err_code = SSL_get_error( m_Ssl, recvdBytes );


	switch( *err_code )
	{
		case SSL_ERROR_NONE: // no error occured
			return RET_OK;

		case SSL_ERROR_ZERO_RETURN: // connection was closed on the server side
			*closed = true;
			// Chaim - if we return a successful status, we shouldn't set the err code because it's not checked
			// by the caller
			// *err_code = SSL_ERROR_ZERO_RETURN;
			*err_code = SSL_ERROR_NONE;
			return RET_OK;

		case SSL_ERROR_WANT_READ: // A non-blocking socket operation could not be completed immediately
			*err_code = EWOULDBLOCK;
			return RET_ERROR;

		case SSL_ERROR_SYSCALL: // IO error or end of data
			if( 0 == recvdBytes ){ // end of data
				//*buffsize = 0; // the buffsize should be set to 0 for correct handling
				// Chaim - if we return a successful status, we shouldn't set the err code because it's not checked
				// by the caller
				//*err_code = SSL_ERROR_ZERO_RETURN;
				*err_code = SSL_ERROR_NONE;
				*closed = true;						// signals that the connection was closed gracefully
				return RET_OK;
			}
			if( -1 == recvdBytes ){ // IO error
				*buffsize = 0; // the buffsize should be set to 0 for correct handling
				*err_code = SOCKET_ERROR;
				*closed = true;						// signals that the connection was closed gracefully
				return RET_ERROR;
			}

		case SSL_ERROR_SSL: // SSL library error
			*buffsize = 0; // the buffsize should be set to 0 for correct handling
			*err_code = SSL_ERROR_SSL;
			*closed = true;
			return RET_ERROR;

		default: // unknown error
			*closed = true;
			return RET_ERROR;
	}

	*closed = true;
	return RET_ERROR;
}

void CWLSecureSslConnection::CloseConnection()
{
	int err = -1;
	if( ( !m_TcpConnected || !m_SslConnected ) || ( NULL == m_Ssl ) || ( -1 == m_Socket ) ){
		return;
	}

	err = SSL_shutdown( m_Ssl );
	m_SslConnected = false;

	CWLGenericConnection::CloseConnection( true );
}

void CWLSecureSslConnection::CleanUp()
{
	if( !m_TcpConnected || !m_SslConnected )
		return;

	if( m_Ssl ){
		SSL_free ( m_Ssl );
		m_Ssl = NULL;
	}

	if( m_Context ){
		SSL_CTX_free ( m_Context );
		m_Context = NULL;
	}

	m_Socket = -1;
}


bool CWLSecureSslConnection::SetCiphers( int strength/* = SSL_AllCrypto*/ )
{
	char strStrength[32];

	switch( strength ){
	case SSL_AllCrypto:
		sprintf( strStrength, "%s", "ALL:eNULL" );
		break;
	case SSL_ExportCryptoOnly:
		sprintf( strStrength, "%s", "EXPORT" );
		break;
	case SSL_StrongCryptoOnly:
		sprintf( strStrength, "%s", "HIGH" );
		break;
	default:
		sprintf( strStrength, "%s", "DEFAULT" );
	}

	if( ! SSL_set_cipher_list( m_Ssl, strStrength ) )
	return false;

	return true;
}


SSL_ERROR_CODE CWLSecureSslConnection::SetSessionId( const unsigned char* buffer, int bufsize )
{
	int err = 0;
/*
	if( NULL == m_Ssl )
		return SSL_ERROR_;
	
	err = SSL_set_session_id_context( m_Ssl, buffer, bufsize );
	if( 1 != err )
		return SSL_ERROR_;
*/
	return SSL_NO_ERROR;
}

SSL_ERROR_CODE CWLSecureSslConnection::StoreSession( unsigned char*& destination )
{
	if( NULL == m_Ssl )
		return SSL_ERROR_;

	int size = i2d_SSL_SESSION( SSL_get_session( m_Ssl ), &destination );
	if( 0 == size ){
		return SSL_ERROR_;
	}

	return SSL_NO_ERROR;
}

SSL_ERROR_CODE CWLSecureSslConnection::RestoreSession( const unsigned char*& source, int length )
{
	SSL_SESSION* pSes = NULL;

	d2i_SSL_SESSION( &pSes, &source, length);
	if( NULL == pSes ){
		return SSL_ERROR_;
	}

	if( NULL != m_Ssl )
		SSL_SESSION_free( SSL_get_session( m_Ssl ) );

	SSL_set_session( m_Ssl, pSes );

	return SSL_NO_ERROR;
}

int CWLSecureSslConnection::GetSessionBinarySize( int & err_code )
{
	err_code = SSL_ERROR_;
	if( NULL == m_Ssl )
		return 0;

	err_code = SSL_NO_ERROR;
	return i2d_SSL_SESSION( SSL_get_session( m_Ssl ), NULL );
}


int CWLSecureSslConnection::SetCryptographicStrength( int strength )
{
	switch( strength ){
	case ALL:
		m_CryptoStrength = "COMPLEMENTOFALL";
		break;
	case LOW:
		m_CryptoStrength = "LOW";
		break;
	case MEDIUM:
		m_CryptoStrength = "MEDIUM";
		break;
	case HIGH:
		m_CryptoStrength = "HIGH";
		break;
	default:
		m_CryptoStrength = "DEFAULT";
	}

	return RET_OK;
}


//////////////////////////////////////////////////////////////////////////
// Static methods
//////////////////////////////////////////////////////////////////////////
void CWLSecureSslConnection::LibraryInit()
{
	if( !m_LibraryIsInitialized ){
		/*
		 *	Potential BUG !!!
		 *	Must be synchronized???.
		 */
		m_LibraryIsInitialized = true;

		SSL_library_init();                      /* initialize library */
		SSL_load_error_strings();                /* readable error messages */
		srand( 1000 );
		OpenSSL_add_all_algorithms();

		m_pSslCryptoLock = new RVCriticalSection[CRYPTO_num_locks()];
		if( NULL == m_pSslCryptoLock ){
			WlTraceMsg( "SSL library was not initialized correctly." );
		}

		CRYPTO_set_locking_callback( crypto_lock_cb );
		CRYPTO_set_id_callback( crypto_thread_id );
	}
}


void CWLSecureSslConnection::DestroyCryptoLockers()
{
	if( NULL != m_pSslCryptoLock ){
		delete [] m_pSslCryptoLock;
		m_pSslCryptoLock = NULL;
	}
}


int CWLSecureSslConnection::DetermineVersion( const char* version )
{
	if (!version)
		return (SSL_Version_Undetermined);

	RVCString sslVerStr(version);

  // Chaim - check default value first - igonre case not needed here
	if (sslVerStr.compareTo("SSL_Version_3_0_With_2_0_Hello", sslVerStr.ignoreCase) == 0)
		return (SSL_Version_3_0_With_2_0_Hello);
	else if (sslVerStr.compareTo("SSL_Version_2_0", sslVerStr.ignoreCase) == 0)
		return (SSL_Version_2_0);
	else if (sslVerStr.compareTo("SSL_Version_3_0", sslVerStr.ignoreCase) == 0)
		return (SSL_Version_3_0);
	else if (sslVerStr.compareTo("SSL_Version_3_0_Only", sslVerStr.ignoreCase) == 0)
		return (SSL_Version_3_0_Only);
	else if (sslVerStr.compareTo("TLS_Version_1.0_With_2_0_Hello", sslVerStr.ignoreCase) == 0)
		return (TLS_Version_1_0_With_2_0_Hello);
	else if (sslVerStr.compareTo("TLS_Version_1_0_Only", sslVerStr.ignoreCase) == 0)
		return (TLS_Version_1_0_Only);
	else if (sslVerStr.compareTo("TLS_Version_1_0", sslVerStr.ignoreCase) == 0)
		return (TLS_Version_1_0);
	else if (sslVerStr.compareTo("SSL_Version_Undetermined", sslVerStr.ignoreCase) == 0)
		return (SSL_Version_Undetermined);

	return (SSL_Version_Undetermined);
}


int CWLSecureSslConnection::DetermineCryptographicStrength(const char* CryptoStrength)
{
	if (!CryptoStrength)
		return (SSL_AllCrypto);

	RVCString CryptoStrengthStr(CryptoStrength);
  if (CryptoStrengthStr.compareTo("SSL_AllCrypto", RVCString::ignoreCase) == 0)
		return (SSL_AllCrypto);
  if (CryptoStrengthStr.compareTo("SSL_StrongCryptoOnly", RVCString::ignoreCase) == 0)
		return (SSL_StrongCryptoOnly);
  if (CryptoStrengthStr.compareTo("SSL_ExportCryptoOnly", RVCString::ignoreCase) == 0)
		return (SSL_ExportCryptoOnly);
  if (CryptoStrengthStr.compareTo("SSL_ServerGatedCrypto", RVCString::ignoreCase) == 0)
		return (SSL_ServerGatedCrypto);

	return (SSL_Version_Undetermined);
}


void CWLSecureSslConnection::crypto_lock_cb(int mode, int type, const char *file, int line){
	if (mode & CRYPTO_LOCK){
		m_pSslCryptoLock[type].acquireWrite();
	} else {
		m_pSslCryptoLock[type].release();
	}
}


unsigned long CWLSecureSslConnection::crypto_thread_id()
{
   unsigned long ret = 0;

#ifdef WIN32
   ret = GetCurrentThreadId();
#else
   ret = (int)pthread_self();
#endif

   return(ret);
}


//////////////////////////////////////////////////////////////////////////
// Static data members
//////////////////////////////////////////////////////////////////////////
#ifdef TRANSPORT_LAYER_TRACE
int CWLSecureSslConnection::m_FileNum = 0;
#endif // TRANSPORT_LAYER_TRACE

bool CWLSecureSslConnection::m_LibraryIsInitialized = false;

RVCriticalSection* CWLSecureSslConnection::m_pSslCryptoLock = NULL;

#endif // __OPEN_SSL__
