/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : GeneralSSLFunctions.CPP
 *
 *	Author : Shahar Kassif
 *
 *	Description : General SSL functions need to be used by the SSL PLUS
 *
 *	History : 
 *
 *	Modifications : 
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/

/** include files **/

#ifdef __SSL__

#include "generalsslfunctions.h"
#ifndef __TV_UNIX__
#include "cdglobal.h"
#endif

#include <stdlib.h>
#include <string.h>
#include <time.h>

/** local definitions **/
#define MIN_ELEMENTS  16

extern "C" 
{

void* CD_malloc(uint32 size)
{   
  return malloc(size == 0 ? 1 : size);
}

void CD_free(void *block)
{   
  if(block)
  {
  free(block);
    block=NULL;
  }
}

void* CD_realloc(void *block, uint32 newSize)
{   if (block == 0)
        return CD_malloc(newSize);
    else
        return realloc(block, newSize == 0 ? 1 : newSize);
}

void CD_memset(void *block, int value, uint32 length)
{   
  memset(block, value, length);
}

/* Even though it's called memcpy, it needs to handle overlapping regions */
void CD_memcpy(void *dest, void *src, uint32 length)
{   
  memmove(dest, src, length);
}

int CD_memcmp(void *a, void *b, uint32 length)
{   
  return memcmp(a, b, length);
}

uint32 CD_time(void)
{   
  return time(0);
}
}
static void GenerateRandomString(unsigned char* seedRandom, int seedLen)
{
  int minElm;
  if (seedLen < MIN_ELEMENTS*2)
    minElm = seedLen/2;
  else
    minElm = MIN_ELEMENTS;

  CD_memset(seedRandom, (int)'\0', (unsigned int) seedLen);
  srand((unsigned)time(0));
  int numElm = rand() % minElm + minElm;
  for (int i = 0; i <= numElm; i++) 
    seedRandom[i] = (unsigned char)(rand() % 223 + 32);
}

SSLErr SSLAlloc (SSLBuffer *buf, void *allocRef)
{
	buf->data = (unsigned char*)malloc((size_t) buf->length);
  if (!buf->data)
    return SSLMemoryErr;
  return SSLNoErr;  
}

SSLErr SSLFree (SSLBuffer *buf, void *allocRef)
{
  if(buf->data)
  {
  free(buf->data);
  buf->data = 0;
  }
  return SSLNoErr;
}

SSLErr SSLRealloc (SSLBuffer *buf, uint32 newSize, void *allocRef)
{
  unsigned char   *newData;
	newData = (unsigned char*)realloc(buf->data, (size_t) newSize);
  if (!newData)
    return SSLMemoryErr;
  buf->data = newData;
  return SSLNoErr;
}

SSLErr WLSSLRandomFunc(SSLBuffer data, void* randomRef)
{   
#ifdef BSAFE
  B_ALGORITHM_OBJ random = (B_ALGORITHM_OBJ)randomRef;
    
  if ((result = B_GenerateRandomBytes(random, data.data, (unsigned int)data.length, (A_SURRENDER_CTX*)NULL_PTR )) != 0)//stav
    return SSLUnknownErr;
#else
/*//@@@@@yossi - yahali's fix, enter all this section into comment, because it causes problems in multiThread
  EZCryptoObj random = (EZCryptoObj)randomRef;
  if ((result = EZSeedRandom(random,data.data, (unsigned int)data.length)) != 0) {
    return SSLUnknownErr;
  }
*/
#endif // BSAFE
  return SSLNoErr;
}

SSLErr WLSeedRandomAndSet(SSLContext *ctx, void** randomRef)
{  
#ifdef BSAFE
  B_ALGORITHM_METHOD      *chooser[] = { &AM_MD5_RANDOM, 0 };
    
  if (B_CreateAlgorithmObject((B_ALGORITHM_OBJ *)randomRef) != 0)
    return SSLUnknownErr;
  if (B_SetAlgorithmInfo((B_ALGORITHM_OBJ)*randomRef, AI_MD5Random, 0) != 0)
    return SSLUnknownErr;
  if (B_RandomInit((B_ALGORITHM_OBJ)*randomRef, chooser, (A_SURRENDER_CTX*)NULL_PTR ) != 0)//stav
    return SSLUnknownErr;
    
  unsigned char seedRandom[256];
  int           seedLen = sizeof(seedRandom);

  GenerateRandomString(seedRandom, seedLen);  
	if (B_RandomUpdate((B_ALGORITHM_OBJ)*randomRef, seedRandom, seedLen, (A_SURRENDER_CTX*)NULL_PTR ) != 0)
      return SSLUnknownErr;
    
  SSLSetRandomRef(ctx, *randomRef);
#else

  EZCreateObject((EZCryptoObj *)randomRef);
  EZInitRandom((EZCryptoObj)*randomRef);

  unsigned char seedRandom[256];
  int           seedLen = sizeof(seedRandom);

  GenerateRandomString(seedRandom, seedLen);  
  if (EZSeedRandom((EZCryptoObj)*randomRef, seedRandom, seedLen) != 0)
      return SSLUnknownErr;
  SSLSetRandomRef(ctx, *randomRef);
#endif // BSAFE
  return SSLNoErr;
}

void WLSSLDestroyRandom(void** randomRef)
{
#ifdef BSAFE
  B_ALGORITHM_OBJ random = (B_ALGORITHM_OBJ)*randomRef;
  B_DestroyAlgorithmObject(&random);
#else
  EZCryptoObj random = (EZCryptoObj)*randomRef;
  EZDestroyObject(&random);
#endif // BSAFE
  *randomRef = NULL;
}


SSLErr SSLTime(uint32 *timeResult, void *timeRef)
{
  *timeResult = time(0);
  return SSLNoErr;
}

SSLErr SSLConvertTime(uint32 *time, void *timeRef)
{
  return SSLNoErr;
}

#ifndef __TV_UNIX__
#ifdef BSAFE
/* Glue functions for BSAFE
 *
 * These functions allow BSAFE to use the SSL Plus linked callbacks to allocate memory.
 *	This means that all allocations by the toolkit will occur in one place.
 */

/* Copyright (C) RSA Data Security, Inc. created 1992.
  
   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */
extern "C" 
{
void 
T_memset (POINTER p, int c, unsigned int count)
{	CD_memset (p, (unsigned char) c, (uint32) count);
}

void
T_memcpy (POINTER d, POINTER s, unsigned int count)
{	CD_memcpy (d, s, (uint32) count);
}

void
T_memmove (POINTER d, POINTER s, unsigned int count)
{	CD_memcpy (d, s, (uint32) count);	/* CD_memcpy actually does memmove */
}

int
T_memcmp (POINTER s1, POINTER s2, unsigned int count)
{	return CD_memcmp(s1, s2, (uint32) count);
}

POINTER
T_malloc(unsigned int size)
{	return (POINTER) CD_malloc((uint32) size);
}

POINTER
T_realloc (POINTER p, unsigned int size)
{	return (POINTER) CD_realloc (p, (uint32) size);
}

void 
T_free (POINTER p)
{	CD_free (p);
}

unsigned int
T_strlen (char *c)
{
        return strlen(c);
}

void 
T_strcpy(char *dest, char *src)
{
        (void) strcpy(dest, src);
}
int
T_strcmp(char *str1, char *str2)
{
	return strcmp(str1, str2);
}
}
#endif // BSAFE
#endif // _TV_UNIX__

#endif // __SSL__
