/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <stdlib.h>
#include "httpdefines.h"
#include "httpcookies.h"
#include "TraceDll.h"

#define EXPIRES        "expires="
#define PATH           "path="
#define DOMAIN_NAME         "domain="
#define COOKIE_REQUEST "Cookie: "

const RVCString MonthConst ("JanFebMarAprMayJunJulAugSepOctNovDec");  
const RVCString DomainConst (".COM.EDU.NET.ORG.GOV.MIL.INT");  

// Methods of HTTPCookie.
int HTTPCookie::compareTo(const RVCollectable* a) const
{
  HTTPCookie* b = (HTTPCookie*)a;

  if (name != b->getName())
    return ((name < b->getName()) ? 1 : -1);
  if (path != b->getPath())
    return ((path < b->getPath()) ? 1 : -1);
  if (domain != b->getDomain())
    return ((domain < b->getDomain()) ? 1 : -1);

  return (0);
}

// General help function for parsing and checking cookies strings
//	  getMonthInNum - Returns according to month string it's number
//		  							If not found then return -1
//    parseNameAndValue - extract from a given string the name and
//												the value of the cookie if the string is 
//												not in the following syntax "name=value"
//												return RET_ERROR
//		convertStrToTime  - Convert a string date formated as : 
//												"wdy DD-Mon-YYYY HH:MM:SS GMT" 
//												(wdy = day in week)	into a time in 
//												miliseconds.
//		getOneAttr				- extract upon a attribute name it's value
//												from the string. If attribute not found
//												return RET_ERROR
//		getCookieAttrs    - Return all the attributes of a cookie,
//												path domain and expireation time
//												if it failes translating one of the 
//												attributes it return RET_ERROR
//												and do not continue any further
//		checkCookieDomain	- Chech if a specified domain name built
//												properly. Domain name must have 3 periods
//												in it or 2 if it ends with 7 specfied suffixs
//

int getMonthInNum(const char* month)
{
	size_t	MonthPos = MonthConst.index (month,strlen(month), 0 , RVCString::ignoreCase);
	div_t		DivResult;

	if (MonthPos != RV_NPOS) {
		DivResult = div(MonthPos,3);
    return (DivResult.quot);
  }
	else {
		return (-1);
	}
}


int parseNameAndValue(RVCString& data, RVCString& name, RVCString& value)
{
  size_t fe = data.first('=');
  if (fe == RV_NPOS  ||  fe == 0)
    return RET_ERROR;

  size_t len = data.length() - fe - 1;
  if (len < 0)
    return RET_ERROR;

  name  = data(0, fe);
  // Remove all trailing and leading blanks if exists
	name = name.strip(RVCString::both);
  value = data(fe+1, len);
  // Remove all trailing and leading blanks if exists
	value = value.strip(RVCString::both);


  return RET_OK;
}


time_t convertStrToTime(RVCString& s_time)
{
	size_t uStrLength = s_time.length();

  struct tm ltm;
  memset(&ltm, 0, sizeof(ltm));

  size_t loc =  s_time.index(", ");
  if (loc == RV_NPOS)
    return (-1);
  //if (s_time.length() != loc+24)
  //  return (-1);

  RVCString date;
  loc += 2;
	if ( uStrLength < loc )
		return -1;

  date = s_time(loc, 2);
  ltm.tm_mday = atoi((const char*)date);
  loc += 3;
	if ( uStrLength < loc )
		return -1;

  date = s_time(loc, 3);
  ltm.tm_mon  = getMonthInNum((const char*)date);
  loc += 4;
	if ( uStrLength < loc )
		return -1;

  size_t ylen = s_time.index(" ", loc) ;
  ylen = ((ylen == RV_NPOS) ? s_time.length() : ylen ) - loc ; 
  date = s_time(loc, ylen);
  ltm.tm_year = atoi((const char*)date);
	// If date bigger then 1900 we substructe 1900 
	// in order that mktime will work
	// else if the year is less then 80 then it is two digit year
	// and we add to it 100 in order that it will be as like 4 digit year 
	// which from which we subtructed 1900
	// other wise it is a year between 1980-1999.

  if (ltm.tm_year > 1970)
    ltm.tm_year -= 1900;
  else if (ltm.tm_year <= 1970 && ltm.tm_year > 999)
    ltm.tm_year = 80;
	else if (ltm.tm_year < 80)
		ltm.tm_year += 100;
  loc += ylen+1;
	if ( uStrLength < loc )
		return -1;

  date = s_time(loc, 2);
  ltm.tm_hour = atoi((const char*)date);
  loc += 3;
	if ( uStrLength < loc )
		return -1;

  date = s_time(loc, 2);
  ltm.tm_min  = atoi((const char*)date);
  loc += 3;
	if ( uStrLength < loc )
		return -1;

  date = s_time(loc, 2);
  ltm.tm_sec  = atoi((const char*)date);

  // Will work onlt till 2038
  return (mktime(&ltm));
}


int getOneAttr(RVCString& data, const char* attrName, RVCString& attr)
{
  size_t loc = data.index(attrName, 0, data.ignoreCase);
  if (loc != RV_NPOS)  {
    size_t len;
    size_t start = loc + strlen(attrName);
    size_t sc = data.index(";", start);
    if (sc == RV_NPOS)
      len = data.length() - start;
    else
      len = sc - start;
    if (len <= 0)  {
      return RET_ERROR;
    }

    attr.remove(0);
    attr = data(start, len);
		// Remove all trailing and leading blanks if exists
		attr = attr.strip(RVCString::both);
  }

  return RET_OK;
}


int getCookieAttrs(RVCString& data, 
                   RVCString& path, 
                   RVCString& domain, 
                   RVCString& s_time,
                   bool       CheckExpiration)
{
  int status = RET_OK;
  status = getOneAttr(data, PATH, path);
  // If we failed in analyzing the path then
  // we assume that the path is for the all domain
  if (status == RET_ERROR) {
    path = "/";
    status = RET_OK;
  }

  status = getOneAttr(data, DOMAIN_NAME, domain);
  if (status == RET_ERROR)
    return (status);

  //  This part is operating only if user specifically asked
  //  for activating the expiration parsing
  //  the default behaviour is that expiration date is not checked
  //  which mean that it will be sent in any case
  if (CheckExpiration) {
    status = getOneAttr(data, EXPIRES, s_time);
    if (status == RET_ERROR)
      return (status);
  }
  else {
    s_time.remove(0);
  }
    

  return status;
}
int checkCookieDomain (RVCString& domain) 
{
	int				NumPeriod = 0;
	size_t		PeriodPos = 0;
	size_t		domainLen	= domain.length();
	RVCString	HelpStr		= domain;
	
	PeriodPos = HelpStr.index(".");
	while (PeriodPos != RV_NPOS) {
		NumPeriod++;
		HelpStr = HelpStr(PeriodPos+1,domainLen-PeriodPos-1);
		PeriodPos = HelpStr.index(".");
	}

	HelpStr.toUpper();
	if ((NumPeriod>2) || ((NumPeriod==2) && (DomainConst.contains (HelpStr.data())))) {
		return true;
	}
	else {
		return false;
	}
}


// called from each thread
int CWLCookies::parseCookieIntoTree(RVCString& response, 
                                    RVCString& host, 
                                    RVCString& uri, 
                                    bool CheckExpiration, 
                                    int& errCode)
{
  RVCString res(response);
  RVCString name, value;
  RVCString path(uri);
  RVCString domain(host);
  RVCString internal_host(host);


  RVCString s_time;
  size_t    sc;
	int				SlashPos;

	internal_host.toUpper();

	errCode = 0;

	// Check if the response that was deliver contain data
  if (response.isNull()) {
    errCode = 56;
		return RET_ERROR;
	}

  // Strip from path the characters which apear
	// after the last slash (if there are so)

	SlashPos = path.last ('/');
	if (SlashPos != RV_NPOS) {
		path = path.remove (SlashPos + 1);
	}
	
	res = res.strip(res.leading, ' ');
  sc = res.first(';');
  if (sc == RV_NPOS)
    sc = res.length();

	// Parse the name and the value of the cookie
  RVCString data = res(0,sc);
  if (parseNameAndValue(data, name, value) == RET_ERROR) {
    errCode = 57;
		return RET_ERROR;
	}

	// Parse the path, domain and expiraion time of the cookie
  res.remove(0, sc + 1);
  if (!res.isNull())  {
    if (getCookieAttrs(res, path, domain, s_time, CheckExpiration) == RET_ERROR) {
	    errCode = 57;
			return RET_ERROR;
		}
  }
	// Checking if we are trying to set a cookie to a different domain
	// from the current

        // Chaim - crash if cookie domain would match host except it starts with a '.'. Saw it in amazon.
        domain = domain.strip(RVCString::leading, '.');
	domain.toUpper();
	path.toUpper();

	if (domain.length() > internal_host.length() || (internal_host(internal_host.length()-domain.length(),domain.length())) != domain) {
		errCode = 58;
	}

	// Checking if we are trying to set a cookie to a domain
	// that is not valid. Domain name must contain 3 periods
	// or 2 periods if it ends with one of the following :
	// COM,EDU,NET,ORG,GOV,MIL and INT
	// This test should be activated by a specific flag <TBD>
	// return a specific error notice
	// if (!checkCookieDomain (host)) {
	// }

	// Inserting the cookie to the tree and checking if it was finished ok
	if (InsertCookie(name, value, path, domain, s_time) == RET_ERROR) {
		errCode = 59;
	}

	if (errCode == 0) {
		return RET_OK;
	}
	else {
		return RET_ERROR;
	}

}


int CWLCookies::makeCookieRequest(RVCString& host, 
                                  RVCString& uri, 
                                  RVCString& cookie,
                                  bool CheckPath, 
                                  bool CheckDomain, 
                                  bool CheckExpiration)
{
	RVCString internal_host(host);

	internal_host.toUpper();
	
  cookie.remove(0);

  if (entries()==0)
    return 0;

	// protect cookies when reading from them
	RVReadersWriterLock::WriteLockGuard reader(m_CookieLock);

  RVBinaryTreeIterator iter(*this);

  HTTPCookie*		IterCookie = NULL;
	RVCString			IterDomain;
	RVCString			IterPath;
	time_t				IterTime;
	time_t				NowTime = time(0);
	size_t				HostLen;
	size_t				IterDomainLen;

WlTraceMsg("***                --------------------");
  while (iter())  {
    IterCookie = (HTTPCookie*)iter.key();
    if (!IterCookie) {
      return RET_ERROR;
		}
		else {
			IterDomain		= IterCookie->getDomain();
			IterDomainLen = IterDomain.length();
			IterPath			= IterCookie->getPath();
		}

WlTraceMsg("*** Checking cookie: %s=%s, domain=%s, path=%s, expires=%d", IterCookie->getName().data(), IterCookie->getValue().data(), IterDomain.data(), IterPath.data(), IterCookie->getExpire());

		// First we check the matching of host.
		// Match in host name means that the host we have in hand 
		// matches the stored host tail for example :
		// Stored host value : "aaa.com" will match any host
		// such as "www.aaa.com","aaa.com" etc.
    if (CheckDomain) {
		  HostLen = internal_host.length();
		  if (IterDomainLen > HostLen) {
WlTraceMsg("*** Domain mismatch-Host name is longer: [%s != %s]", IterDomain.data(), internal_host.data());
        continue;
		  }
		  if ((internal_host (HostLen-IterDomainLen,IterDomainLen)) != IterDomain) {
WlTraceMsg("*** Domain length mismatch: [%s != %s]", IterDomain.data(), internal_host.data());
        continue;
		  }
    }
		// Then we check if the path value apears completle in the uri
		// that was given and from the start of it
		// for example matching will accour if IterPath= "/f" and uri="/fff/ggg"
		// but not if IterPath="/f" and uri="/a/f"
    if (CheckPath) {
      if (uri.index(IterPath,0,RVCString::ignoreCase) != 0) {
WlTraceMsg("*** Path mismatch: [%s != %s]", IterPath.data(), uri.data());
        continue;
		  }
    }

		// And last we check if the cookie that we have is not expired yet
		// We could delete from the tree cookies that expired but
    if (CheckExpiration) {
      NowTime = time(0);
		  IterTime = IterCookie->getExpire();
      if (IterTime > 0 && IterTime < NowTime) {
WlTraceMsg("*** Expires mismatch: [%d < %d]", IterTime, NowTime);
        continue;
		  }
    }

		// if all went well then we add this cookie to the 
		// out cookie string
    cookie.append(IterCookie->getName());
    cookie.append("=");
    cookie.append(IterCookie->getValue());
    cookie.append("; ");
WlTraceMsg("*** Cookie match");
  }
  // Remove extra chars from end of cookie string
	// adn add the cookie header
  if (!cookie.isNull())  {
    cookie.remove(cookie.length() - 2);
    cookie.prepend(COOKIE_REQUEST);
WlTraceMsg("*** Adding cookie header: %s", cookie.data());
  }

WlTraceMsg("***                --------------------\n");

	return (0);
}


// the function is called from the script (thread 0) or from parseCookieIntoTree function
// by each existing thread
int CWLCookies::InsertCookie(RVCString& n, RVCString& v, 
                                     RVCString& p, RVCString& d, RVCString& t)
{
  time_t  time;
	RVCString internal_d (d);
	RVCString internal_p (p);

	internal_d.toUpper();
	internal_p.toUpper();
	
  if (t.isNull() != TRUE)  {
		// If day it is not handled here
		// if month is not legal then it is not handled here
    time = convertStrToTime(t);
  }
  else
    time = 0;

  HTTPCookie* cki = new HTTPCookie(n, v, internal_p, internal_d, time);

	// protect cookies when inserting a new one
	RVReadersWriterLock::WriteLockGuard writer(m_CookieLock);

  if (find((RVCollectable*)cki) != NULL && v.length() == 0 &&
    m_workspace->m_DeleteEmptyCookies){
    WlTraceMsg("*** Cookie deleted: %s=%s, domain=%s, path=%s, expires=%d", n, v, internal_d, internal_p, time);
    removeAndDestroy((RVCollectable*)cki);
    return RET_OK;
  }

  removeAndDestroy((RVCollectable*)cki);
  if (insert(cki) == NULL)
    return RET_ERROR;

WlTraceMsg("*** Cookie inserted: %s=%s, domain=%s, path=%s, expires=%d", n, v, internal_d, internal_p, time);
	return (RET_OK);
}


// the function is called from the script (thread 0) only
void CWLCookies::ClearAllCookies()
{
	// protect cookies when inserting a new one
	RVReadersWriterLock::WriteLockGuard writer(m_CookieLock);
  clearAndDestroy();
}


// the function is called from the script (thread 0) only
int CWLCookies::DeleteCookie(RVCString& name, RVCString& path, RVCString& domain)
{
  path.toUpper();
  domain.toUpper();
  HTTPCookie cki(name, path, domain);

	// protect cookies when inserting a new one
	RVReadersWriterLock::WriteLockGuard writer(m_CookieLock);

  HTTPCookie *ck = (HTTPCookie *)remove(&cki);
  if (ck != NULL)  {
    delete ck;
    return RET_OK;
  }

	return RET_ERROR;
}
