/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/*
 * File name : httpheaderhandler.cpp
 * Author    : Yossi Nakash
 * Created   : 3/14/01
 * ==========:
 * Abstract  : implementation of CHLHeaderHandler
 *
 * Version control stuff:
 * $Revision:: 1        $
 * $Author::            $
 */

#include "httpheaderhandler.h"
#include <string.h>
#include <assert.h>
#include <platform.h>
#include "httpdefines.h"


CHLHeaderHandler::CHLHeaderHandler()
{}


CHLHeaderHandler::~CHLHeaderHandler()
{}


CHLHeaderHandler::CHLHeaderHandler( const CHLHeaderHandler& src )
{}


CHLHeaderHandler& CHLHeaderHandler::operator=( const CHLHeaderHandler& src )
{ assert(0); return *this; }


CHLHeaderHandler::HttpHeaders CHLHeaderHandler::GetHeaderId(const char* p_pcHeader)
{
  HttpHeaders RetID = EN_HEADER_NONE ;

  if (!p_pcHeader || !p_pcHeader[0])
    return RetID ;

  int nHeaderLength = strlen(p_pcHeader);
  char* pcHeader = new char[strlen(p_pcHeader)+12];
  pcHeader[0] = '\0';
  strcpy(pcHeader, p_pcHeader);
  TVPStrlwr((char*)pcHeader);

  switch (pcHeader[0])
  {
  case 'a' :{
    switch (pcHeader[1])
    {
    case 'c' : if (!strcmp(pcHeader, "accept"))
                  RetID = EN_HEADER_ACCEPT ; 
               else if (!strcmp(pcHeader, "accept-encoding"))
                 RetID = EN_HEADER_ACCEPT_ENCODING; 
               break;
    case 'u' : if (!strcmp(pcHeader, "authorization"))
                  RetID = EN_HEADER_AUTHORIZATION ; break ;
    default:;
    }
    break;}

  case 'b' : if (!strcmp(pcHeader, "basic"))
                RetID = EN_TOKEN_BASIC ; break ;

  case 'c' :{
    switch (pcHeader[3])
    {
    case 'k' : if (!strcmp(pcHeader, "cookie"))
                  RetID = EN_HEADER_COOKIE ; break ;
    case 'n' :{
      switch(pcHeader[4])
      {
      case 'k' : if (!strcmp(pcHeader, "chunked"))
                    RetID = EN_TOKEN_CHUNKED ; break ;
      case 'e' : if (!strcmp(pcHeader, "connection"))
                    RetID = EN_HEADER_CONNECTION ; break ;
      default:;
      }
      break;}

    case 's' : if (!strcmp(pcHeader, "close"))
                  RetID = EN_TOKEN_CLOSE ; break ;
    case 't' :{
  	  if ( nHeaderLength > 8 )
	  {
      switch(pcHeader[8])
      {
      case 'l' : if (!strcmp(pcHeader, "content-length"))
                    RetID = EN_HEADER_CONTENT_LENGTH ; break ;
      case 't' : if (!strcmp(pcHeader, "content-type"))
                    RetID = EN_HEADER_CONTENT_TYPE ; break ;
      case 'e' : if (!strcmp(pcHeader, "content-encoding"))
                    RetID = EN_HEADER_CONTENT_ENCODING ; break ;
      default:;
      }
	  }
      break;}

    default:;
    }
    break;}

	case 'd':
		if ( !strcmp( pcHeader, HEADER_DATE ) )
			RetID = EN_HEADER_DATE;
		break;

	case 'e':
		if ( nHeaderLength > 1 )
		{
			switch ( pcHeader[1] )
			{
				case 't':
					if ( !strcmp( pcHeader, HEADER_ETAG ) )
						RetID = EN_HEADER_ETAG;
					break;

				case 'x':
					if ( !strcmp( pcHeader, HEADER_EXPIRES ) )
						RetID = EN_HEADER_EXPIRES;
					break;

				default:
					break;
			}
		}
		break;

  case 'h' : if (!strcmp(pcHeader, "host"))
                RetID = EN_HEADER_HOST ; break ;

  case 'i' : if (!strcmp(pcHeader, "if-modified-since"))
                RetID = EN_HEADER_IF_MODIFIED_SINCE ; break ;

  case 'k' : if (!strcmp(pcHeader, "keep-alive"))
                RetID = EN_TOKEN_KEEP_ALIVE ; break ;

  case 'l' :
    if (nHeaderLength>1)
    {
	    switch (pcHeader[1])
		  {
				case 'o':
					if ( !strcmp( pcHeader, "location" ) )
						RetID = EN_HEADER_LOCATION ; break ;

				case 'a':
					if ( !strcmp( pcHeader, HEADER_LAST_MODIFIED ) )
						RetID = EN_HEADER_LAST_MODIFIED;
					break;

			  default:
					break;
		  }
    }
		break;

  case 'n' : if (!strcmp(pcHeader, "ntlm"))
                RetID = EN_TOKEN_NTLM ; break ;

  case 'p' :{
	if ( nHeaderLength > 11 )
	{
    switch (pcHeader[11])
    {
    case 'c' : if (!strcmp(pcHeader, "proxy-connection"))
                  RetID = EN_HEADER_PROXY_CONNECTION ; break ;
    case 'n' : if (!strcmp(pcHeader, "proxy-authenticate"))
                  RetID = EN_HEADER_PROXY_AUTHENTICATE ; break ;
    case 'r' : if (!strcmp(pcHeader, "proxy-authorization"))
                  RetID = EN_HEADER_PROXY_AUTHORIZATION ; break ;
    default:;
    }
	}
    break;
}

  case 'r' : if (!strcmp(pcHeader, "referer"))
                RetID = EN_HEADER_REFERER ; break ;

  case 's' : if (!strcmp(pcHeader, "set-cookie"))
                RetID = EN_HEADER_COOKIE_RESPONSE ; break ;

  case 't' :{
    switch (pcHeader[9])
    {
    case 'c' : if (!strcmp(pcHeader, "transfer-coding"))
                  RetID = EN_HEADER_TRANSFER_CODING ; break ;
    case 'e' : if (!strcmp(pcHeader, "transfer-encoding"))
                  RetID = EN_HEADER_TRANSFER_ENCODING ; break ;
    default:;
    }
    break;}

  case 'u' : if (!strcmp(pcHeader, "user-agent"))
                RetID = EN_HEADER_USER_AGENT ; break ;

  case 'w' : if (!strcmp(pcHeader, "www-authenticate"))
                RetID = EN_HEADER_AUTHENTICATE ; break ;

  default:;
  }

  delete[] pcHeader;
  return RetID ;
}

