/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <regexp.h>
#include <collstr.h>
#include "tvenv.h"
#include "wlerrorlib.h"
#include "httpdefines.h"
#include "httphost.h"
#include "connpool.h"
#include "tracedll.h"

#include "WLGenericConnection.h"


int HTTPAddr::convertHost(RVCString& h, int p, int* err)
{  
	host = h;
	port = p;
  sin.sin_family = AF_INET;
  if ((sin.sin_port = htons(port)) == 0) 
    return RET_ERROR;
  
	if (m_Addresses) {
		free(m_Addresses);
		m_Addresses = NULL;
	}

	// this function will enable internal round-robin when caching DNS reply
	m_AddressCount = CWLGenericConnection::GetHostByName(host.data(), &m_Addresses, &m_AddressLength, err);
	if (m_AddressCount == 0)
		return RET_ERROR;

	m_CurrentAddress = 0;

	return RET_OK;
}

bool HTTPAddr::getSockAddrIn(sockaddr_in& addr, bool resolveOnly)
{
	if (!resolveOnly)
		m_CurrentAddress++;

	m_CurrentAddress = m_CurrentAddress % m_AddressCount;
	// first copy the current ip
	memmove((char*)&(sin.sin_addr), &(m_Addresses[m_CurrentAddress * m_AddressLength]), m_AddressLength);
	// then duplicate the common properties of the host (port, family...)
	memmove((void*)&(addr), (void*)&sin, sizeof(sockaddr_in));

	return true;
}


bool HTTPAddr::isEqual(const RVCollectable* t) const
{
	HTTPAddr* c = (HTTPAddr*)t;

	if (host == c->getHost()  &&  port == c->getPort())
		return TRUE;
	else
		return FALSE;
}


HTTPAddr* CWLDnsTable::resolveItem(RVCString& host, int port, int* err)
{
	HTTPAddr* addr = new HTTPAddr;
	if (addr->convertHost(host, port, err) == RET_ERROR) {
		delete addr;
		return (NULL);
	}

  WlTraceMsg ("DNS: Ip address was resolved for %s", host.data()) ;
	return addr;
}


HTTPAddr* CWLDnsTable::addItem(RVCString& host, int port, int* err)
{
	HTTPAddr  faddr(host, port);
	HTTPAddr* p;

  RVReadersWriterLock::WriteLockGuard writer(m_ItemLock);
	if ((p = (HTTPAddr*)find(&faddr)) != NULL)
		return (p);
	
	HTTPAddr* addr = resolveItem(host, port, err);
	if (addr)
		insert(addr);

	return(addr);
}


HTTPAddr* CWLDnsTable::getItem(RVCString& host, int port, bool useCache, int* err, bool* actualLookup)
{
  *actualLookup = false;
	if (useCache) {
		RVReadersWriterLock::ReadLockGuard reader(m_ItemLock);
		RVSetIterator iter(*this);
	  while (iter()) {
		  HTTPAddr* addr = (HTTPAddr*)iter.key();
      if (host == addr->getHost()  &&  port == addr->getPort()) {
        WlTraceMsg ("DNS: Ip address for %s was fetched from cache.", host.data()) ;
			  return addr;
      }
	  }
	}
  else {
    *actualLookup = true;
		return resolveItem(host, port, err);
  }

  *actualLookup = true;
	return addItem(host, port, err);
}

