/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifdef _USE_SMARTHEAP_
#include <smrtheap.hpp>
#endif // _USE_SMARTHEAP_
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "platform.h"
#include "tvtables.h"
#include "tvenv.h"
#include "tracedll.h"
#include "tvpsync.h"
#include "wlerrorlib.h"
#include "loadengineglobals.h"

int WLHttpInit(bool bCallback);
int WLHttpTerminate();

#ifdef __TV_UNIX__
#include <pthread.h>
#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
void HttpLoadTerminateThread(int sig)
{
  pthread_exit((void *)-1);
}
#endif

int main(int argc, char *argv[])
{
#ifdef __TV_UNIX__
  char thisFileName[MAX_PATH];
  if( !TVEInit(TVPGetPathDirectory( (const char *)realpath(argv[0],thisFileName ))))
  {
    TVPFatalErrorLog("tvenv.dll: Init Terminated with error: (%s)",TVEError());
    exit(-1);
  }
  // create signal handler
  const char *SignalMask = TVEGetEnv(ENV_SIGNAL_MASK);
  TVPSignalHandler *hnd = new TVPSignalHandler(SignalMask?SignalMask:"");
  // Terminate Thread on various signals
  hnd->TVPSigSet(SIGBUS,HttpLoadTerminateThread);
  hnd->TVPSigSet(SIGSEGV,HttpLoadTerminateThread);

#ifdef _USE_SMARTHEAP_
  MemRegisterTask();
#endif // _USE_SMARTHEAP_

#endif
  // WARNING - Do not change this constant without consulting with Shahar ***
  char TraceProgram[50] = "HttpLoad";
  switch (argc) {
    case 5:
      strcat (TraceProgram, "#") ;
      strcat (TraceProgram, argv[4]) ;
      break ;
    case 3:
      strcat (TraceProgram, argv[2]) ;
      break ;
    default:
      break;
  };
  CTraceMsg::Init (TraceProgram) ;
  WlTraceMsg("HttpLoad started with the following parameters:");
  int i ;
  for (i = 0 ; i < argc ; i++)
      WlTraceMsg ("Parameter %d:   %s" , i, argv[i]);
  WLHttpInit(false);

// added code for openning file for debug only for convenience running doNothing.exe
  int argc2 = argc;
	char** argv2 = argv;

  char argvData[10][100];
	char* argvp[10];

	if(argc==2) // open file & read data from it.
  {
		argv2 = argvp;

		argc2 = 0;
		argvp[0] = argv[0];

		FILE* InputFile = fopen( argv[1], "r" );
		assert(InputFile);
		while( !feof(InputFile) ) {
			int status = fscanf( InputFile, "%s", argvData[argc2] );
			if( (status!=0) && (status!=EOF) ) {
				argvp[argc2+1] = argvData[argc2];
				++argc2;
				}
			}
		++argc2;
	}

  // @@Migration - the bool arg tells load engine we're protocol
  if (argc2==5)
    LoadGeneratorMain(argc2, argv2, false);
  else if (argc2==3)
    LoadGeneratorSlave(argc2, argv2, false);
  else if (argc2==4)
    LoadGeneratorTest(argc2, argv2, false);
  else
#ifdef __TV_WINDOWS__
    MessageBox(0, "Illegal number of Parameters", "Load Generator", MB_OK);
#else
    printf("Illegal number of Parameters");
#endif
  WlTraceMsg("Before TVSTerminate");
  TVSTerminate();
  WlTraceMsg("Before WLERRTerminate");
  WLERRTerminate();

  WlTraceMsg("Before WLHttpTerminate");
	WLHttpTerminate();

  WlTraceMsg("Before ExitProcess");
#ifdef _USE_SMARTHEAP_
  /* unregister with SmartHeap */
  MemUnregisterTask();
#endif // _USE_SMARTHEAP_

#ifndef __TV_UNIX__
  TerminateProcess(GetCurrentProcess(),0);
#else
  kill(getpid(), SIGKILL);
#endif

  return 0;

}
