/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : jsCallback.cpp
 *
 *	Author : Chaim Shraga
 *
 *	Description : 
 *
 *	History : 
 *
 *	Modifications : 
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/

#include "rvjsapi.h"

#include "loadengineglobals.h"
#include "httpdefines.h"
#include "jshttpnames.h"
#include "commandmanager.h"
#include "callbackdata.h"
#include "httpembedderreport.h"
#include "connpool.h"

#define LOAD_EXCEPTION_FRM_FILE "loadengine.frm"
#define reportError  ptrHttp->GetConnPool()->ReportError	// not depended by the (HTTP) thread

static JSPropertySpec callback_props[] = {
	{0}
};


static void CallbackFinalize(JSContext *cx, JSObject *obj)
{
    CallbackData *p_CallbackData = (CallbackData*)JS_GetPrivate(cx, obj);
	if (p_CallbackData != NULL)
	{
		delete(p_CallbackData);
	}
}

JSClass callback_class = {
    WLJSNAME_CALLBACK_CLASS,
    0,
    JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
    JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	CallbackFinalize
};

//
// Download functions
//

static JSBool
callback_PageStartDownload(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {

		if (!ptrHttp->IsCallbackEnabled()){
			return JS_TRUE;
		}

		CallbackData *p_CallbackData = (CallbackData*)JS_GetPrivate(cx, obj);
		if (p_CallbackData != NULL){

			// @@Chaim - what to pass as expected DOM ID?
			cHttpEmbedderReportInfoPageStartDownload StartDwnld((CUrlData*)p_CallbackData, 0);
			StartDwnld.fReport(cx);
		}
	}
	return JS_TRUE;
}

static JSBool
callback_PageEndDownload(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {

		if (!ptrHttp->IsCallbackEnabled()){
			return JS_TRUE;
		}

		CallbackData *p_CallbackData = (CallbackData*)JS_GetPrivate(cx, obj);
		if (p_CallbackData != NULL){

			// @@Chaim - dummy empty DOM
//			CWLDocument aDom;
//			aDom.SetDocUrl(p_CallbackData->m_pcUrl);
			p_CallbackData->SetDom(NULL);
			p_CallbackData->SetGetFrames(false);
			cHttpEmbedderReportInfoPageEndDownload EndDwnld((CUrlData*)(CUrlDataEx*)p_CallbackData);
			EndDwnld.fReport(cx);
		}
	}
	return JS_TRUE;
}

static JSBool
callback_PageAbortDownload(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	// @@Chaim
	return JS_TRUE;
}

static JSBool
callback_ObjectStartDownload(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {

		if (!ptrHttp->IsCallbackEnabled()){
			return JS_TRUE;
		}

		CallbackData *p_CallbackData = (CallbackData*)JS_GetPrivate(cx, obj);
		if (p_CallbackData != NULL){

			cHttpEmbedderReportInfoObjectStartDownload StartDwnld((CUrlData*)p_CallbackData);
			StartDwnld.fReport(cx);
		}
	}
	return JS_TRUE;
}

static JSBool
callback_ObjectEndDownload(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {

		if (!ptrHttp->IsCallbackEnabled()){
			return JS_TRUE;
		}

		CallbackData *p_CallbackData = (CallbackData*)JS_GetPrivate(cx, obj);
		if (p_CallbackData != NULL){

			p_CallbackData->SetUrlType(FRAME) ;
			p_CallbackData->SetHttpStatus(RET_OK) ;
			p_CallbackData->SetMimeType("text/html") ;
			p_CallbackData->SetFileSize(0);
			p_CallbackData->SetCheckSum(0);
			cHttpEmbedderReportInfoObjectEndDownload EndDwnld(p_CallbackData);
			EndDwnld.fReport(cx);
		}
	}
	return JS_TRUE;
}

static JSBool
callback_ObjectAbortDownload(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	// @@Chaim
	return JS_TRUE;
}


// Callback data functions
static JSBool
callback_AddRequestHeader(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {

		if (!ptrHttp->IsCallbackEnabled()){
			return JS_TRUE;
		}

		if (!JSVAL_IS_STRING(argv[0])) {
				reportError(LOAD_EXCEPTION_FRM_FILE, 273, "WLCallback.AddRequestHeader", 1);
			return JS_FALSE;
		}

		const char* sHeader     = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));
		CallbackData *p_CallbackData = (CallbackData*)JS_GetPrivate(cx, obj);
		if (p_CallbackData != NULL){
			p_CallbackData->AddRequestHeader(sHeader);
		}
	}
	return JS_TRUE;
}

static JSBool
callback_AddResponseHeader(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {

		if (!ptrHttp->IsCallbackEnabled()){
			return JS_TRUE;
		}

		if (!JSVAL_IS_STRING(argv[0])) {
			reportError(LOAD_EXCEPTION_FRM_FILE, 273, "WLCallback.AddRequestHeader", 1);
			return JS_FALSE;
		}

		const char* sHeader     = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));
		CallbackData *p_CallbackData = (CallbackData*)JS_GetPrivate(cx, obj);
		if (p_CallbackData != NULL){
			p_CallbackData->AddResponseHeader(sHeader);
		}
	}
	return JS_TRUE;
}

static JSBool
callback_SetUrl(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {

		if (!ptrHttp->IsCallbackEnabled()){
			return JS_TRUE;
		}

		if (!JSVAL_IS_STRING(argv[0])) {
			reportError(LOAD_EXCEPTION_FRM_FILE, 273, "WLCallback.SetUrl", 1);
			return JS_FALSE;
		}

		const char* sUrl     = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));
		CallbackData *p_CallbackData = (CallbackData*)JS_GetPrivate(cx, obj);
		if (p_CallbackData != NULL){
			p_CallbackData->SetUrl(sUrl);
			p_CallbackData->SetFinalUrl(sUrl);
			p_CallbackData->SetOriginalUrl(sUrl);
		}
	}
	return JS_TRUE;
}

static JSBool
callback_SetContent(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {

		if (!ptrHttp->IsCallbackEnabled()){
			return JS_TRUE;
		}

		if (!JSVAL_IS_STRING(argv[0])) {
			reportError(LOAD_EXCEPTION_FRM_FILE, 273, "WLCallback.SetContent", 1);
			return JS_FALSE;
		}

		const char* sContent     = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));
		CallbackData *p_CallbackData = (CallbackData*)JS_GetPrivate(cx, obj);
		if (p_CallbackData != NULL){
			p_CallbackData->SetObjContent((void*)sContent) ;
			p_CallbackData->SetObjContentLength(strlen(sContent)) ;
		}
	}
	return JS_TRUE;
}

static JSBool
callback_Reset(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {

		if (!ptrHttp->IsCallbackEnabled()){
			return JS_TRUE;
		}

		CallbackData *p_CallbackData = (CallbackData*)JS_GetPrivate(cx, obj);
		if (p_CallbackData != NULL){
			p_CallbackData->Reset();
		}
	}
	return JS_TRUE;
}

//
// @@Chaim - verify with Yossi which properties to add (Mime type etc)
//

static JSFunctionSpec callback_methods[] = {
	{WLJSNAME_CALLBACK_PAGE_START_DOWNLOAD,			callback_PageStartDownload,				0},
	{WLJSNAME_CALLBACK_PAGE_END_DOWNLOAD,			callback_PageEndDownload,				0},
	{WLJSNAME_CALLBACK_PAGE_ABORT_DOWNLOAD,			callback_PageAbortDownload,				0},
	{WLJSNAME_CALLBACK_OBJECT_START_DOWNLOAD,		callback_ObjectStartDownload,			0},
	{WLJSNAME_CALLBACK_OBJECT_END_DOWNLOAD,			callback_ObjectEndDownload,				0},
	{WLJSNAME_CALLBACK_OBJECT_ABORT_DOWNLOAD,		callback_ObjectAbortDownload,			0},
	{WLJSNAME_CALLBACK_OBJECT_ADD_REQUEST_HEADER,	callback_AddRequestHeader,				0},
	{WLJSNAME_CALLBACK_OBJECT_ADD_RESPONSE_HEADER,	callback_AddResponseHeader,				0},
	{WLJSNAME_CALLBACK_OBJECT_SET_URL,				callback_SetUrl,						0},
	{WLJSNAME_CALLBACK_OBJECT_SET_CONTENT,			callback_SetContent,					0},
	{WLJSNAME_CALLBACK_OBJECT_RESET,				callback_Reset,							0},
	{0}
};


static JSBool CallbackCtor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

  CallbackData* p_CallbackData = new CallbackData();
  JS_SetPrivate(cx, obj, p_CallbackData);
  
  *rval = OBJECT_TO_JSVAL(obj);
	return JS_TRUE;
}

bool DllExport WLAddCallbackClass(JSContext *cx, JSObject *obj)
{
	// define a class of type Cookie
	if (!JS_InitClass(cx, obj, NULL, &callback_class, CallbackCtor, 1, NULL, callback_methods, NULL, NULL))
		return false;

	return true;
}
