/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "rvjsapi.h"
#include "wlfuncs.h"
#include "wlform.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "jsprivate.h"
#include "TraceDll.h"
#include "propertieshandler.h"

static JSBool
form_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside form_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/

  // check if the property is a function (defined by us or by the user)
  if (JS_TypeOfValue(cx,*vp) == JSTYPE_FUNCTION) 
    return JS_TRUE;

	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);

  CWLFrame* frame = prv->GetParent();

	CWLForm* form = (CWLForm*)prv->GetObj();
	if (!form)
		return JS_TRUE;


	if (form && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);


    switch (PropertiesHandler::GetPropertyId(prop))
    {
    case PropertiesHandler::EN_ELEMENTS : {	// return the Elements property
			CWLArray* elements = &(form->Elements());
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_ELEMENT, (void**)&elements, vp);
		}

    case PropertiesHandler::EN_ACTION : {	// return the Action property
			CWLLocation* action = &(form->Action());
			return WLSetActionValue(cx, obj, frame, action, vp);
		}

    case PropertiesHandler::EN_METHOD : {	// return the Method property
			const char* method = form->Method().data();
			return WLSetStringValue(cx, obj, prop, method, vp);
		}

    case PropertiesHandler::EN_NAME : {	// return the Name property
			const char* name = form->Name().data();
			return WLSetStringValue(cx, obj, prop, name, vp);
		}

    case PropertiesHandler::EN_ENCODING : {	// return the Name property
			const char* encoding = form->Encoding().data();
			return WLSetStringValue(cx, obj, prop, encoding, vp);
		}

    case PropertiesHandler::EN_TARGET : {	// return the Target property
			const char* target = form->Target().data();
			return WLSetStringValue(cx, obj, prop, target, vp);
		}

    case PropertiesHandler::EN_LENGTH : {	// return the Length property
			 unsigned length = form->Length();
			return WLSetIntValue(cx, obj, prop, length, vp);
		}

    case PropertiesHandler::EN_ID : {	// return the id property
			const char* id = form->Id();
			return WLSetStringValue(cx, obj, prop, id, vp);
		}

    case PropertiesHandler::EN_TAGNAME : {	// return the tagName property
			return WLSetStringValue(cx, obj, prop, "FORM", vp);
		}

    case PropertiesHandler::EN_WLSOURCE_INDEX : { // return the WLSourceIndex property
			int index = form->GetIndex();
			return WLSetIntValue(cx, obj, prop, index, vp);
		}

    case PropertiesHandler::EN_INNERHTML : {	// return the innerHTML property
			RVCString innerHtml = form->GetInnerHTML();
			return WLSetStringValue(cx, obj, prop, innerHtml.data(), vp);
		}

    default :{	// return the Element property -document.<form>.<element>
			CWLElement* element = form->Element(prop);
			return WLSetElementValue(cx, obj, frame, element, vp);
		}
    }
	}

	return JS_TRUE;
}

static JSBool
form_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside form_setProperty");
/*
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLForm* form = (CWLForm*)prv->GetObj();
	if (form && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

		if (!strcmp(prop, WLJSNAME_PROP_ACTION)) {	// return the Action property
//			form->setAction(WLLocationValue(cx, *vp));
			form->Action() = *WLLocationValue(cx, *vp);
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_METHOD)) {	// return the Method property
//			form->setMethod(WLStringValue(*vp));
			form->Method() = WLStringValue(*vp);
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_NAME)) {	// return the Name property
//			form->setName(WLStringValue(*vp));
			form->Name() = WLStringValue(*vp);
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_SPIDERSESSION)) {	// return the SPIDERSESSION property
			form->setSPIDERSESSION(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_ELEMENTS)) {	// return the Elements property
			form->setElements(WLArrayValue(cx, *vp));
			return JS_TRUE;
		}
	}
*/

	return JS_TRUE;
}


JSBool form_ClassEnumerate(JSContext *cx, JSObject *obj ){


	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);

	CWLForm* form = (CWLForm*)prv->GetObj();
  if (form){
    // Define properties for elements
    for (int i = form->Length() -1; i >= 0; i--){
      JS_DefineProperty(cx, obj, (const char*)(i), JSVAL_VOID, NULL, NULL, JSPROP_ENUMERATE | JSPROP_INDEX);
    }
  }

  JS_DefineProperty(cx, obj, WLJSNAME_PROP_TARGET, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_NAME, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_ID, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_TAG_NAME, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_INNER_TEXT, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_INNERHTML, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_ENCODING, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_NAME, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_METHOD, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_ACTION, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_ELEMENTS, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);

  return JS_TRUE;
}

static void
form_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside form_finalize");
	WLFinalize(cx, obj);
}

JSClass wl_FormClass = {
	WLJSNAME_CLASS_FORM,
	JSCLASS_HAS_PRIVATE,
  JS_PropertyStub,
	JS_PropertyStub,
	form_getProperty,
	form_setProperty,
  form_ClassEnumerate,
	JS_ResolveStub,
	JS_ConvertStub,
	form_finalize
};


/*
static JSBool
form_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	// set rval to name
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLForm* form = (CWLForm*)prv->GetObj();
	if (!form)
		return JS_TRUE;

	const char* name = form->Name();
	JSString* str = JS_NewStringCopyZ(cx, name);
	if (str)
		*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}

//??? to be implemented ???
static JSBool
form_submit(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
/ *
	// use method on action
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_FALSE;

	CWLForm* form = (CWLForm*)prv->GetObj();
	if (!form)
		return JS_FALSE;

	// set the 'wlHttp' object's properties: Headers, FormData and POST the action
	jsval v;
	JS_EvaluateScript(cx, obj, WLJSNAME_OBJECT_HTTP, strlen(WLJSNAME_OBJECT_HTTP), NULL, 0, &v);
	if (WLIsObjectOfClass(cx, v, WLJSNAME_CLASS_HTTP)) {
		JSObject* http = JSVAL_TO_OBJECT(v);
		JS_GetProperty(cx, http, WLJSNAME_PROP_FORMDATA, &v);
		JSObject* formdata = JSVAL_TO_OBJECT(v);
		for (int i = 0; i < form->length(); i++) {
			jsval key = STRING_TO_JSVAL(JS_NewStringCopyZ(cx, form->Elements()->Key(i)));
			jsval val = STRING_TO_JSVAL(JS_NewStringCopyZ(cx, form->Elements()->Value(i)));
			JS_SetProperty(cx, formdata, key, &val);
		}
		char command[128];
		sprintf(command, "%s.%s(%s)", WLJSNAME_OBJECT_HTTP, WLJSNAME_PROP_FORMDATA, form->Action()->Href());
		JS_EvaluateScript
	}
* /

	return JS_TRUE;
}


static JSFunctionSpec form_methods[] = {
	{wl_toString_str,	form_toString,	0},
	//{"submit",				form_submit,		0},
	{0}
};
*/


/*
static JSBool
InitFormObject(JSContext *cx, JSObject *obj, jsint length, jsval *vector)
{
WlTraceMsg("initializing form object properties");
	CWLForm* form = new CWLForm;
	JS_SetPrivate(cx, obj, (void*)form);

	if (!WLInitHiddenProperty(cx, obj, WLJSNAME_PROP_DOM_SCOPE))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_ELEMENTS))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_ACTION))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_METHOD))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_NAME))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_SPIDERSESSION))
		return JS_FALSE;

	return JS_TRUE;
}


static JSBool
WLForm(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	JSBool ok;

	ok = InitFormObject(cx, obj, 0, NULL);
	if (ok)
		*rval = OBJECT_TO_JSVAL(obj);

	return ok;
}


bool WLAddFormObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddFormObject(%x, %x)", cx, obj);
	// create an instance of type WLHttp
	JSObject* form = JS_DefineObject(cx, obj, WLJSNAME_OBJECT_FORM, &wl_FormClass, NULL, JSPROP_PERMANENT);
	if (!form)
		return false;

	// define it's properties & structures
	if (!InitFormObject(cx, form, 0, NULL))
		return false;

	return true;
}
*/


JSObject* WLNewFormObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewFormObject(%x, %x)", cx, obj);
	// create an instance of type WLForm
	JSObject* proto = NULL;
	JSObject* form = WLNewObject(cx, &wl_FormClass, proto, obj);
	if (!form)
		return NULL;

	return form;
}


bool DllExport WLAddFormClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddFormClass(%x, %x)", cx, obj);
	// define a class of type WLHttp
	//if (!JS_InitClass(cx, obj, NULL, &wl_FormClass, WLForm, 1, NULL, form_methods, NULL, NULL))
	if (!JS_InitClass(cx, obj, NULL, &wl_FormClass, NULL, 1, NULL, NULL, NULL, NULL))
		return false;

	return true;
}
