/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : jsHtml.cpp
 *
 *	Author : Zion Menahem
 *
 *	Description : 
 *
 *	History : 
 *
 *	Modifications : 
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/

/** include files **/
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "rvjsapi.h"

#include <platform.h>
#include "wldocument.h"
#include "wllocation.h"
#include "loadengineglobals.h"
#include "jshttpnames.h"
#include "parsefuncs.h"
#include "jsreplynames.h"
#include "wlfuncs.h"
#include "jsprivate.h"
#include "commandmanager.h"
#include "wljsreply.h"
#include "replyprops.h"


/** local definitions **/

/** default settings **/

/** external functions **/

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/

/** classes definitions **/

/** public functions **/

/** private functions **/


/*********************************************************************************
    All the changes were undone, and the explanation below is the opposite.
    The commented part of the code, did not delete the specified javascript objects,
    only replaced the pointers to a new c++ objects.
    The new code, deletes the javascript objects, and creates new ones.
    This code was replaced because of the change entered in all the getProperty
    functions. we do not create new object in every call to getProperty. If object exists,
    we return it. These changes should save performance, when there are many calls to 
    getProperty, but will damage performance when there is only http_get in the agenda.
**********************************************************************************/

bool WLBuildHtmlParseTree_WebLoad(JSContext* cx, JSObject* obj, CWLCommandManager* ptrHttp, bool isParent)
{
	CWLDocument* wlDoc = ptrHttp->GetParsedHtmlTree();
	if (!wlDoc)	// response was not parsed
		return false;

	//the parameter obj here is the wlhttp object - we need it's parent- the wllocal -
	//under which we have to "produce" the document object
	// if isParent==true then obj is already the parent
	if (!isParent)
		obj = JS_GetParent(cx, obj);

	jsval v;
	JSBool res;

	res = JS_LookupProperty(cx, obj, WLJSNAME_OBJECT_DOCUMENT, &v);
	if (res && JSVAL_IS_OBJECT(v)) {
		JSObject* jsdoc = JSVAL_TO_OBJECT(v);
		WLFinalize(cx, jsdoc);
		WLSetNewObject(cx, obj, jsdoc, wlDoc, WLJSNAME_CLASS_FRAME, (void*)wlDoc);
	}
	else {
		// error, can't find parsed document
	}

/*
	if (WLDeleteProperty(cx, obj, WLJSNAME_OBJECT_DOCUMENT)) {
		JSObject* jsdoc;
		if (WLAddDomObject(cx, obj, &jsdoc))
			WLSetNewObject(cx, obj, jsdoc, wlDoc, WLJSNAME_CLASS_FRAME, (void*)wlDoc);
		else {
			// error, can't create document object
		}
	}
*/

	return true;
}

/**** the function that create the objects: window, location & frames ****/
/***************************Browser like object*****************************************/
bool WLBuildHtmlParseTree_Browser(JSContext* cx, JSObject* obj, CWLCommandManager* ptrHttp, bool isParent)
{
	CWLDocument* wlDoc = (CWLDocument*)ptrHttp->GetParsedHtmlTree();
	if(!wlDoc)
		return false;

	CWLLocation* wlLoc = &(wlDoc->GetLocation());
	if(!wlLoc)
		return false;

	RVOrdered* wlFrames = wlDoc->m_Frames;	
	if (!wlFrames)	// response was not parsed
		return false;

	// if isParent==true then obj is already the parent
	if (!isParent)
		obj = JS_GetParent(cx, obj);

	jsval v;
	JSBool res;
	
	res = JS_LookupProperty(cx, obj, WLJSNAME_OBJECT_WINDOW, &v);
	if (res && JSVAL_IS_OBJECT(v)) {
		JSObject* jswindow = JSVAL_TO_OBJECT(v);
		WLFinalize(cx, jswindow);
		WLSetNewObject(cx, obj, jswindow, wlDoc, WLJSNAME_CLASS_WINDOW, (void*)wlDoc);
	}
	else {
		// error, can't find window object
	}

/*
	if (WLDeleteProperty(cx, obj, WLJSNAME_OBJECT_WINDOW)) {
		JSObject* jsdoc;
		if (WLAddWindowObject(cx, obj, &jsdoc))
			WLSetNewObject(cx, obj, jsdoc, wlDoc, WLJSNAME_CLASS_WINDOW, (void*)wlDoc);
		else {
			// error, can't create window object
		}
	}
*/

	res = JS_LookupProperty(cx, obj, WLJSNAME_OBJECT_LOCATION, &v);
	if (res && JSVAL_IS_OBJECT(v)) {
		JSObject* jslocation = JSVAL_TO_OBJECT(v);
		WLFinalize(cx, jslocation);
		WLSetNewObject(cx, obj, jslocation, wlDoc, WLJSNAME_CLASS_BROWSER_LOCATION, (void*)wlLoc);
	}
	else {
		// error, can't find location object
	}

/*
	if (WLDeleteProperty(cx, obj, WLJSNAME_OBJECT_LOCATION)) {
		JSObject* jslocation;
		if (WLAddLocationObject(cx, obj, &jslocation))
			WLSetNewObject(cx, obj, jslocation, wlDoc, WLJSNAME_CLASS_BROWSER_LOCATION, (void*)wlLoc);
		else {
			// error, can't create location object
		}
	}
*/

	res = JS_LookupProperty(cx, obj, WLJSNAME_OBJECT_FRAMES, &v);
	if (res && JSVAL_IS_OBJECT(v)) {
		JSObject* jsframes = JSVAL_TO_OBJECT(v);
		WLFinalize(cx, jsframes);
		WLSetNewObject(cx, obj, jsframes, wlDoc, WLJSNAME_CLASS_FRAMES, (void*)wlFrames);
	}
	else {
		// error, can't find frames object
	}	
	
/*
	if (WLDeleteProperty(cx, obj, WLJSNAME_OBJECT_FRAMES)) {
		JSObject* jsframes;
		if (WLAddFramesObject(cx, obj, &jsframes))
			WLSetNewObject(cx, obj, jsframes, wlDoc, WLJSNAME_CLASS_FRAMES, (void*)wlFrames);
		else {
			// error, can't create frames object
		}	
	}
*/

  return true;
}

/******************************************************************************\

	HTML Object

\******************************************************************************/

enum HtmlId {
};

static JSPropertySpec html_props[] = {
	{0}
};


JSClass html_class = {
    WLJSNAME_HTML_CLASS,
    0,
    JS_PropertyStub,
		JS_PropertyStub,
		JS_PropertyStub,
		JS_PropertyStub,
		JS_EnumerateStub,
		JS_ResolveStub,
		JS_ConvertStub,
		JS_FinalizeStub
};


static JSBool
html_GetFrameByIndex(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		CWLFrame* frame = WLParse_GetFrameByIndex(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (frame)
			return WLSetFrameValue(cx, obj, frame, frame, rval);
	}

	return JS_TRUE;
}


static JSBool
html_GetFrameByUrl(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		CWLFrame* frame = WLParse_GetFrameByUrl(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (frame)
			return WLSetFrameValue(cx, obj, frame, frame, rval);
	}

	return JS_TRUE;
}

static JSBool
html_GetFrameByLocation(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		CWLFrame* frame = WLParse_GetFrameByLocation(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (frame)
			return WLSetFrameValue(cx, obj, frame, frame, rval);
	}

	return JS_TRUE;
}


static JSBool
html_GetFrameUrl(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		CWLFrame* frame = WLParse_GetFrameByIndex(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;
		CWLUrl* url = WLParse_GetFrameUrl(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (url)
			return WLSetLinkValue(cx, obj, frame, url, rval);
	}

	return JS_TRUE;
}

static JSBool
html_GetLinkByIndex(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		CWLFrame* frame = (CWLFrame*)GetFrame(ptrHttp, cx, 1, argc, argv[1]);

		CWLUrl* link = WLParse_GetLinkByIndex(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (link)
			return WLSetLinkValue(cx, obj, frame, link, rval);
	}

	return JS_TRUE;
}


static JSBool
html_GetLinkByName(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		CWLFrame* frame = (CWLFrame*)GetFrame(ptrHttp, cx, 1, argc, argv[1]);
		CWLUrl* link = WLParse_GetLinkByName(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (link)
			return WLSetLinkValue(cx, obj, frame, link, rval); 
	}

	return JS_TRUE;
}


static JSBool
html_GetLinkByUrl(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		CWLFrame* frame = (CWLFrame*)GetFrame(ptrHttp, cx, 1, argc, argv[1]);
		CWLUrl* link = WLParse_GetLinkByUrl(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (link)
			return WLSetLinkValue(cx, obj, frame, link, rval);  
	}

	return JS_TRUE;
}


static JSBool
html_GetFieldValue(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		const char* value = WLParse_GetFieldValue(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (value)
			return WLSetStringValue(cx, obj, NULL, value, rval);
	}

	return JS_TRUE;
}


static JSBool
html_GetFieldValueInForm(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		const char* value = WLParse_GetFieldValueInForm(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (value)
			return WLSetStringValue(cx, obj, NULL, value, rval);
	}

	return JS_TRUE;
}


static JSBool
html_GetQSFieldValue(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		char* value = WLParse_GetQSFieldValue(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

    if (value) {
      JSBool ret = WLSetStringValue(cx, obj, NULL, value, rval);
      free(value);
      return ret;
    }
	}

	return JS_TRUE;
}


static JSBool
html_GetFormAction(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		CWLFrame* frame = (CWLFrame*)GetFrame(ptrHttp, cx, 1, argc, argv[1]);
		CWLLocation* location = WLParse_GetFormAction(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (location)
			return WLSetLocationValue(cx, obj, frame, location, rval);
	}

	return JS_TRUE;
}


static JSBool
html_GetHeaderValue(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		const char* value = WLParse_GetHeaderValue(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (value)
			return WLSetStringValue(cx, obj, NULL, value, rval);
	}

	return JS_TRUE;
}


static JSBool
html_GetStatusNumber(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		unsigned value = WLParse_GetStatusNumber(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (value != -1)
			*rval = INT_TO_JSVAL(value);
	}

	return JS_TRUE;
}


static JSBool
html_GetStatusLine(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		const char* value = WLParse_GetStatusLine(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (value)
			return WLSetStringValue(cx, obj, NULL, value, rval);
	}

	return JS_TRUE;
}


static JSBool
html_GetHost(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		const char* value = WLParse_GetHost(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (value)
			return WLSetStringValue(cx, obj, NULL, value, rval);
	}

	return JS_TRUE;
}


static JSBool
html_GetHostName(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		const char* value = WLParse_GetHostName(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (value)
			return WLSetStringValue(cx, obj, NULL, value, rval);
	}

	return JS_TRUE;
}


static JSBool
html_GetPortNum(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		unsigned value = WLParse_GetPortNum(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (value != -1)
			*rval = INT_TO_JSVAL(value);
	}

	return JS_TRUE;
}


static JSBool
html_GetUri(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		const char* value = WLParse_GetUri(ptrHttp, cx, argc, argv);
    if (ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (value)
			return WLSetStringValue(cx, obj, NULL, value, rval);
	}

	return JS_TRUE;
}




static JSFunctionSpec html_methods[] = {
	{WLJSNAME_HTML_GET_FRAME_BY_INDEX,			html_GetFrameByIndex,			2},
	{WLJSNAME_HTML_GET_FRAME_BY_URL,				html_GetFrameByUrl,				2},
	{WLJSNAME_HTML_GET_FRAME_BY_LOCATION,		html_GetFrameByLocation,	1},
	{WLJSNAME_HTML_GET_FRAME_URL,						html_GetFrameUrl,					1},
	{WLJSNAME_HTML_GET_LINK_BY_INDEX,				html_GetLinkByIndex,			2},
	{WLJSNAME_HTML_GET_LINK_BY_NAME,				html_GetLinkByName,				2},
	{WLJSNAME_HTML_GET_LINK_BY_URL,					html_GetLinkByUrl,				2},
	{WLJSNAME_HTML_GET_FIELD_VALUE,					html_GetFieldValue,				2},
	{WLJSNAME_HTML_GET_FIELD_VALUE_IN_FORM,	html_GetFieldValueInForm,	3},
	{WLJSNAME_HTML_GET_QS_FIELD_VALUE,			html_GetQSFieldValue,			2},
	{WLJSNAME_HTML_GET_FORM_ACTION,					html_GetFormAction,				2},
	{WLJSNAME_HTML_GET_HEADER_VALUE,				html_GetHeaderValue,			2},
	{WLJSNAME_HTML_GET_STATUS_NUMBER,				html_GetStatusNumber,			1},
	{WLJSNAME_HTML_GET_STATUS_LINE,					html_GetStatusLine,				1},
	{WLJSNAME_HTML_GET_HOST,								html_GetHost,							1},
	{WLJSNAME_HTML_GET_HOST_NAME,						html_GetHostName,					1},
	{WLJSNAME_HTML_GET_PORT_NUM,						html_GetPortNum,					1},
	{WLJSNAME_HTML_GET_URI,									html_GetUri,							1},
	{0}
};


// synchronize this array with the html_props array
char* wlHtml_values[] = {
	NULL
};

static jsval GetDefaultValue(JSContext* cx, int id)
{
	if (!wlHtml_values[id] || !wlHtml_values[id][0])
		return JSVAL_VOID;
//		return JSVAL_NULL;

	JSString* str = JS_NewStringCopyZ(cx, wlHtml_values[id]);
	return STRING_TO_JSVAL(str);
}


static JSBool
WLHtml(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	// add the HTML object properties
	for (int i = 0; html_props[i].name; i++)
		if (!JS_DefineProperty(cx, obj, html_props[i].name, GetDefaultValue(cx, i),
								html_props[i].getter, html_props[i].setter, html_props[i].flags))
			return JS_FALSE;
	
	return JS_TRUE;
}


bool DllExport WLAddHtmlClass(JSContext *cx, JSObject *obj)
{
	// define a class of type Cookie
	if (!JS_InitClass(cx, obj, NULL, &html_class, NULL, 1, NULL, html_methods, NULL, NULL))
		return false;

	return true;
}

bool DllExport WLAddHtmlObject(JSContext *cx, JSObject *obj)
{
	// create an instance of type Http
	JSObject* wlHtml = JS_DefineObject(cx, obj, WLJSNAME_HTML_OBJECT, &html_class, NULL, JSPROP_PERMANENT);
	if (!wlHtml)
		return false;

	// call it's constructor
	jsval v;
	if (WLHtml(cx, wlHtml, 0, NULL, &v) != JS_TRUE)
		return false;

	return true;
}
