/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : jsHttp.cpp
 *
 *	Author : Zion Menahem
 *
 *	Description :
 *
 *	History :
 *
 *	Modifications :
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/

/** include files **/
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include <openssl/ssl.h>

#include "jshttp.h"

#include "loadengineglobals.h"
#include "httpdefines.h"
#include "jsreplynames.h"
#include "commandmanager.h"
#include "connpool.h"
#include "wljsobjects.h"
#include "workspace.h"
#include "wlfuncs.h"
#include "jsprivate.h"
#include "replyprops.h"
#include "jshttpobj.h"
#include "parsefuncs.h"

#include "connpool.h"

//#define __WL_TRACE__
#include "tracedll.h"
#include "httpembedderreport.h"
#include "domitem.h"
#include "associativearray.h"
#include "gt_strwrapper.h"

#include "commandmanager.h"
#include "WLSslCiphersCollection.h"


/** local definitions **/
#define WLJSNAME_FORMDATA_CLASS	"FORMDATA"
#define WLJSNAME_HEADER_CLASS		"HEADER"
#define WLJSNAME_DATA_CLASS			"DATA"
#define WLJSNAME_DATAFILE_CLASS	"DATAFILE"

static JSBool	AddDataStructures(JSContext* cx, JSObject* obj, CWLHttpPrivateData* HttpVars);
static JSBool HttpArrays_Set(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
static JSBool HttpArrays_Get(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);


extern bool WLBuildHtmlParseTree_WebLoad(JSContext* cx, JSObject* obj, CWLCommandManager* ptrHttp, bool isParent = false);
extern bool WLBuildHtmlParseTree_Browser(JSContext* cx, JSObject* obj, CWLCommandManager* ptrHttp, bool isParent = false);
extern void EncodeString(const char* val, RVCString& result, bool encode);
/** default settings **/

/** external functions **/

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/
class CWLVariables;

/** classes definitions **/

/** public functions **/

/** private functions **/
static int reportError(CWLCommandManager* ptrHttp, const char* frmName, int frmId, ...)
{
  WLLogMessage::Severity sev = WLERRGetErrorSeverity(frmName, frmId);

  char    buf[10240];
  va_list ap;

  va_start(ap, frmId);
  vsprintf(buf, WLERRGetErrorFormat(frmName,frmId), ap);
  va_end(ap);

  bool rtn;
	if (sev > WLLogMessage::Note)
		rtn = ptrHttp->reportMultiThread(sev, (const char*)buf);

  return (rtn ? RET_OK : RET_ERROR );
}


bool SetParsedDom(JSContext* cx, JSObject* obj, CWLCommandManager* ptrHttp)
{
	CWLDocument* wlDoc = (CWLDocument*)ptrHttp->GetParsedHtmlTree();
	if (!wlDoc)	// response was not parsed
		return false;

	jsval v;
	obj = JS_GetParent(cx, obj);
	JSBool res = JS_LookupProperty(cx, obj, WLJSNAME_OBJECT_DOCUMENT, &v);
	if (res && JSVAL_IS_OBJECT(v)) {
		JSObject* jsdoc = JSVAL_TO_OBJECT(v);
		WLFinalize(cx, jsdoc);
		WLSetNewObject(cx, obj, jsdoc, wlDoc, WLJSNAME_CLASS_FRAME, (void*)wlDoc);
	}
	else {
		// error, can't find parsed document property
	}

	return true;
}


bool SetExpectedDom(JSContext* cx, JSObject* obj, jsval v)
{
	CWLDocument* exp = NULL;

	if (JSVAL_IS_OBJECT(v)) {
		CWLPrivate* prv = NULL;
		if (JS_GetClass(cx, JSVAL_TO_OBJECT(v))->flags & JSCLASS_HAS_PRIVATE)
			prv = (CWLPrivate*)JS_GetPrivate(cx, JSVAL_TO_OBJECT(v));
		if (prv)
			exp = (CWLDocument*)prv->GetObj();
	}

	if (!exp)	// parameter is not an object or 'empty'
		return false;

	// set the expected document property to point to exp
	obj = JS_GetParent(cx, obj);
	if (exp) {
		int res = JS_LookupProperty(cx, obj, WLJSNAME_OBJECT_EXPECTED_DOM, &v);
		if (res && JSVAL_IS_OBJECT(v)) {
			JSObject* jsexp = JSVAL_TO_OBJECT(v);
			WLFinalize(cx, jsexp);
			WLSetNewObject(cx, obj, jsexp, exp, WLJSNAME_CLASS_FRAME, (void*)exp);
		}
		else {
			// error, can't find expected document property
		}
	}

	return true;
}


/*********************************************************************\

	Http Object

\*********************************************************************/

/*
enum HttpId {
	WLHTTP_USER_NAME, WLHTTP_PASSWORD, WLHTTP_PROXY, WLHTTP_PROXY_USER_NAME,
	WLHTTP_PROXY_PASSWORD, WLHTTP_PARSE, WLHTTP_PARSE_FORMS, WLHTTP_PARSE_LINKS,
	WLHTTP_PARSE_SCRIPTS, WLHTTP_PARSE_TABLES, WLHTTP_PARSE_XML, WLHTTP_VERIFY_XML,WLHTTP_XML_STANDALONE,
	WLHTTP_TIMER_NAME, WLHTTP_VERSION, WLHTTP_URL, WLHTTP_OUTFILE,
	WLHTTP_GET_CONTENTS, WLHTTP_GET_FRAME_CONT,
	WLHTTP_CACHE_DNS, WLHTTP_KEEP_ALIVE,
//	WLHTTP_DO_REDIRECTIONS, WLHTTP_USE_SLOW_CONN,
	WLHTTP_ERASE,
	WLSSL_VERSION, WLSSL_USE_CACHE,
	PROP_SESSION_TYPE, PROP_REFERRER, PROP_SESSION_CONTROL,PROP_LOCATE_SESSION_ID, PROP_USE_SESSION_ID
};

//**************************** For enum !!!!!
static JSPropertySpec HttpProps[] = {
	{WLJSNAME_HTTP_USER_NAME,				WLHTTP_USER_NAME,				JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_PASSWORD,				WLHTTP_PASSWORD,				JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_PROXY,						WLHTTP_PROXY,						JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_PROXY_USER_NAME,	WLHTTP_PROXY_USER_NAME,	JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_PROXY_PASSWORD,	WLHTTP_PROXY_PASSWORD,	JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_PARSE,						WLHTTP_PARSE,						JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_PARSE_FORMS,			WLHTTP_PARSE_FORMS,			JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_PARSE_LINKS,			WLHTTP_PARSE_LINKS,			JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_PARSE_SCRIPTS,		WLHTTP_PARSE_SCRIPTS,		JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_PARSE_TABLES,		WLHTTP_PARSE_TABLES,		JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_PARSE_XML,				WLHTTP_PARSE_XML,				JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_XML_STANDALONE,	WLHTTP_XML_STANDALONE,	JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_VERIFY_XML,			WLHTTP_VERIFY_XML,			JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_TIMER_NAME,			WLHTTP_TIMER_NAME,			JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_VERSION,					WLHTTP_VERSION,					JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_URL,							WLHTTP_URL,							JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_OUTFILE,					WLHTTP_OUTFILE,					JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_GET_CONTENTS,		WLHTTP_GET_CONTENTS,		JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_GET_FRAME_CONT,	WLHTTP_GET_FRAME_CONT,	JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_CACHE_DNS,				WLHTTP_CACHE_DNS,				JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_HTTP_KEEP_ALIVE,			WLHTTP_KEEP_ALIVE,			JSPROP_ENUMERATE | JSPROP_PERMANENT},
//	{WLJSNAME_HTTP_DO_REDIRECTIONS,	WLHTTP_DO_REDIRECTIONS,	JSPROP_ENUMERATE | JSPROP_PERMANENT},
//	{WLJSNAME_HTTP_USE_SLOW_CONN,		WLHTTP_USE_SLOW_CONN,		JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_SSL_VERSION,					WLSSL_VERSION,					JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{WLJSNAME_SSL_USE_CACHE,				WLSSL_USE_CACHE,				JSPROP_ENUMERATE | JSPROP_PERMANENT},


	// the erase property for the wlHttp object should be ReadOnly !
//	{WLJSNAME_HTTP_ERASE,				WLHTTP_ERASE,						JSPROP_ENUMERATE | JSPROP_PERMANENT},
	{0}
};
*/



static JSBool
http_SetUrlProperty(JSContext *cx, JSObject *obj, jsval value)
{
	CWLHttpPrivateData* pHttpVariablesData = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
	if (pHttpVariablesData) {
		JSString* str = JS_ValueToString(cx, value);
		if (str) {
      CLEPropertyValue PropVal(JS_GetStringBytes(str));
      pHttpVariablesData->m_Variables->Set(CWLWorkspace::UrlIndex(), PropVal);
			return JS_TRUE;
		}
		// report a url error
	}

	return JS_TRUE;
}


static JSBool
http_GetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
	if (!JSVAL_IS_STRING(id))
		return JS_TRUE;

	CWLHttpPrivateData* pHttpVariablesData = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
	if (pHttpVariablesData) {
		pHttpVariablesData->m_Variables->JsVarGet(cx, obj, id, vp);
	}

	return JS_TRUE;
}


static JSBool
http_SetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
	//if (strcmp(JS_GetStringBytes(JSVAL_TO_STRING(id)), WLJSNAME_HTTP_URL) == 0)
	//	return http_SetUrlProperty(cx, obj, *vp);

	CWLHttpPrivateData* pHttpVariablesData = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
	if (pHttpVariablesData) {
		return pHttpVariablesData->m_Variables->JsVarSet(cx, obj, id, vp);
	}

	return JS_TRUE;
}


static void
http_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside http_finalize");
	CWLHttpPrivateData* HttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
	delete HttpVars;
}



JSClass js_HttpClass = {
    WLJSNAME_HTTP_CLASS,
    JSCLASS_HAS_PRIVATE,
    JS_PropertyStub,	// add Property
		JS_PropertyStub,	// del Property
		http_GetProperty,	//JS_PropertyStub,	// get Property
		http_SetProperty,	//JS_PropertyStub,	// set Property
    JS_EnumerateStub,
		JS_ResolveStub,
		JS_ConvertStub,
		http_finalize//		JS_FinalizeStub
};


void DeleteHttpProperties(JSContext *cx, JSObject *obj)
{
	// erase all properties
	CWLHttpPrivateData* HttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
	HttpVars->m_Variables->ResetVariablesArray();
	if (HttpVars->m_FormdataBlock)
		HttpVars->m_FormdataBlock->clearAndDestroy();
	if (HttpVars->m_HeaderBlock)
		HttpVars->m_HeaderBlock->clearAndDestroy();
	if (HttpVars->m_DataBlock)
		HttpVars->m_DataBlock->clearAndDestroy();
	if (HttpVars->m_DatafileBlock)
		HttpVars->m_DatafileBlock->clearAndDestroy();

}

static JSBool
http_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	JSBool ok = JS_TRUE;
	jsval v;

	if (JS_GetProperty(cx, obj, WLJSNAME_HTTP_URL, &v))
		*rval = v;

	return ok;
}

/******************************************************************
  Return the IP address of the connection the thread made.
  If MultiIPSupport is false nothing is returned (undefined).
******************************************************************/
static JSBool
http_GetIPAddress(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  bool bMultiIPSupport = false ;
  bool bMultiIPSupportPerClient = false ;
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
  CWLHttpPrivateData* pHttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
  if (pHttpVars)
    bMultiIPSupport = pHttpVars->m_Variables->Get(WLJSNAME_HTTP_MULTI_IP_SUPPORT).m_Value.BoolVal ;
    bMultiIPSupportPerClient = (ptrHttp->GetWorkspace()->m_MultiIPType == CWLWorkspace::PerClient);

	if (bMultiIPSupport && ptrHttp)
  {
	// Chaim - MultiIP - recalc IP address
	sockaddr_in   bind_addr;
	int	i;
	if (bMultiIPSupportPerClient){
		i = ptrHttp->GetLGThreadNum() % CWLConnectionsPool::m_gLocalAddr.GetNum();
	}
	else {
		i = CWLConnectionsPool::GetMultiIPAddressIndex();
	}
	bind_addr.sin_addr.s_addr = CWLConnectionsPool::m_gLocalAddr.GetAt(i)->s_addr;
    char * pIPAddress = inet_ntoa(bind_addr.sin_addr) ;

    JSString* str = JS_NewStringCopyZ(cx, pIPAddress);
	  if (str)
		  *rval = STRING_TO_JSVAL(str);
  }
  return JS_TRUE;
}
//return JS_FALSE only in case of isAbort()
static JSBool
http_DoCommand(HttpFuncs cmd, JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval, int &res)
{
	if (argc > 0)
		http_SetUrlProperty(cx, obj, argv[0]);

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp)
  {
    CWLHttpPrivateData* HttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);

	  if (argc > 1 && cmd != HTTP_FUNCS_HEAD)
    {
		  // argument must be a string
		  if (JSVAL_IS_STRING(argv[1]))
			  ptrHttp->m_sTransName = JS_GetStringBytes(JSVAL_TO_STRING(argv[1]));
		  else
			  reportError(ptrHttp, HTTP_ERROR_FILE, 240);
    }

  // get expected dom id
  if (ptrHttp->IsCallbackEnabled())
  {
    jsval ExpectedomId ;
    if (JS_GetProperty(cx, obj, "ExpectedDOMID", &ExpectedomId) && ExpectedomId != JSVAL_VOID)
      JS_ValueToInt32(cx, ExpectedomId, (int32 *)&ptrHttp->m_lExpectedDomId);
  }

    {
			CJSGCRequestSuspendResume gc(cx);
			res = ptrHttp->DoCommand(HttpVars, cmd, cx);
      ptrHttp->UpdateGetTime();  // update the time of end transaction.
      ptrHttp->endMultiThread();

      if (!ptrHttp->isAbort())
      {
        ptrHttp->addTimer(PAGE_TIME, ptrHttp->GetPageTime());
        ptrHttp->addCounter(TOTAL_PAGES);
      }
		}
		*rval = INT_TO_JSVAL(res);

    WLBuildHtmlParseTree_WebLoad(cx, obj, ptrHttp);
		WLBuildHtmlParseTree_Browser(cx, obj, ptrHttp);

		if (ptrHttp->isAbort())
			return JS_FALSE;
	}

	return JS_TRUE;
}

static JSBool
http_Head(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  JSBool ret = JS_TRUE ;

  int DoCommandRes;

  // in case of isAbort() do not enter data to statistics
  if (!http_DoCommand(HTTP_FUNCS_HEAD, cx, obj, argc, argv, rval, DoCommandRes))
    return JS_FALSE ;
/*
  // we do not delete the workspace variables, because they are still in use
  CWLHttpPrivateData* pHttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
  if (pHttpVars)   // check the 'erase' property and delete data blocks if 'TRUE'
    if (pHttpVars->m_Variables->Get(CWLWorkspace::EraseIndex()).m_Value.BoolVal)
		  DeleteHttpProperties(cx, obj);
*/
  CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
  CWLDocument* pFrame = ptrHttp->GetPage()->GetGlobalDoc();
	CWLArray* headers = &pFrame->GetHeader();
	ret &= WLSetArrayValue(cx, obj, pFrame, WLJSNAME_CLASS_HEADER, (void**)&headers, rval);

  ptrHttp->GetPage()->SetPreviousDomAsActive();

  WLBuildHtmlParseTree_WebLoad(cx, obj, ptrHttp);
  WLBuildHtmlParseTree_Browser(cx, obj, ptrHttp);

  return ret ;
}


static JSBool
http_Get(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  bool bFoldTree = false ;
  JSBool ret = JS_TRUE ;
  CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
  CWLHttpPrivateData* pHttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);

  int DoCommandRes;

  // in case of isAbort() do not enter data to statistics
	if (!http_DoCommand(HTTP_FUNCS_GET, cx, obj, argc, argv, rval, DoCommandRes))
    return JS_FALSE;

  ptrHttp->UpdateAllTransactions();

	if (pHttpVars)
  {
    bFoldTree = pHttpVars->m_Variables->Get(WLJSNAME_HTTP_FOLD_TRANSACTIONS_TREE).m_Value.BoolVal ;
	}

  if (bFoldTree)
  {
    ret &= (int)ptrHttp->InsertAllTransactions();

    if (pHttpVars)   // check the 'erase' property and delete data blocks if 'TRUE'
      if (pHttpVars->m_Variables->Get(CWLWorkspace::EraseIndex()).m_Value.BoolVal)
		    DeleteHttpProperties(cx, obj);
  }

  if (ptrHttp->IsCallbackEnabled())
  {
    ptrHttp->CloseCBData();
    
    if (ret != JS_TRUE)
    {
      cHttpEmbedderReportInfoPageAbortDownload AbortDwnld((CUrlData*)ptrHttp->GetCallbackData());
      AbortDwnld.fReport(cx);
    }
    else
    {
      CWLDocument* pDom = ptrHttp->GetParsedHtmlTree();
      if (pDom)
      {
        ptrHttp->GetCallbackData()->SetDom(pDom);
        ptrHttp->GetCallbackData()->SetFinalUrl(pDom->GetDocUrl());
        ptrHttp->GetCallbackData()->SetGetFrames(ptrHttp->GetWorkspace()->m_GetFrames);
      }
      WlTraceMsg("End download: \"%s\", \"%s\"", ptrHttp->GetCallbackData()->m_pcUrl, ptrHttp->GetCallbackData()->m_pcFinalUrl);
      cHttpEmbedderReportInfoPageEndDownload EndDwnld((CUrlData*)ptrHttp->GetCallbackData());
      EndDwnld.fReport(cx);
    }
  }

  return ret;
}


static JSBool
http_Post(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  bool bFoldTree = false ;
  JSBool ret = JS_TRUE ;
  CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
  CWLHttpPrivateData* pHttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);

  int DoCommandRes;

  // in case of isAbort() do not enter data to statistics
	if (!http_DoCommand(HTTP_FUNCS_POST, cx, obj, argc, argv, rval, DoCommandRes))
    return JS_FALSE;

  ptrHttp->UpdateAllTransactions();

  if (pHttpVars)
  {
    bFoldTree = pHttpVars->m_Variables->Get(WLJSNAME_HTTP_FOLD_TRANSACTIONS_TREE).m_Value.BoolVal ;
	}

  if (bFoldTree)
  {
    ret &= (int)ptrHttp->InsertAllTransactions();

    if (pHttpVars)   // check the 'erase' property and delete data blocks if 'TRUE'
      if (pHttpVars->m_Variables->Get(CWLWorkspace::EraseIndex()).m_Value.BoolVal)
		    DeleteHttpProperties(cx, obj);
  }

  // Check that we actually had a URL to navigate to
  if (ptrHttp->IsCallbackEnabled())
  {
    ptrHttp->CloseCBData();

    if (ret != JS_TRUE)
    {
      cHttpEmbedderReportInfoPageAbortDownload AbortDwnld((CUrlData*)ptrHttp->GetCallbackData());
      AbortDwnld.fReport(cx);
    }
    else
    {
      CWLDocument* pDom = ptrHttp->GetParsedHtmlTree();
      if (pDom)
      {
        ptrHttp->GetCallbackData()->SetDom(pDom);
        ptrHttp->GetCallbackData()->SetFinalUrl(pDom->GetDocUrl());
        ptrHttp->GetCallbackData()->SetGetFrames(ptrHttp->GetWorkspace()->m_GetFrames);
      }
      WlTraceMsg("End download: \"%s\", \"%s\", \"%s\"", ptrHttp->GetWorkspace()->m_Url, ptrHttp->GetCallbackData()->m_pcUrl, ptrHttp->GetCallbackData()->m_pcFinalUrl);
      cHttpEmbedderReportInfoPageEndDownload EndDwnld((CUrlData*)ptrHttp->GetCallbackData());
      EndDwnld.fReport(cx);
    }
  }

  return ret;
}

static JSBool
http_CloseConnection(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp)
		ptrHttp->ConnectionClose();

	return JS_TRUE;
}

static JSBool
http_ClearDnsCache(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp)
		ptrHttp->ClearDnsCache();

	return JS_TRUE;
}

static JSBool
http_ClearSslCache(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp)
		ptrHttp->ClearSslCache();

	return JS_TRUE;
}

static JSBool
http_InsertTransactions(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  JSBool bRet = false;
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp)
    bRet = ptrHttp->InsertAllTransactions();

  CWLHttpPrivateData* HttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
  if (HttpVars)   // check the 'erase' property and delete data blocks if 'TRUE'
    if (HttpVars->m_Variables->Get(CWLWorkspace::EraseIndex()).m_Value.BoolVal)
		  DeleteHttpProperties(cx, obj);

  return (bRet ? JS_TRUE : JS_FALSE) ;
}

static JSBool
http_AddEvent(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
  bool bAutoNamedTransactions = true;
	CWLHttpPrivateData* pHttpVariablesData = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
	if (pHttpVariablesData)
  {
    bAutoNamedTransactions = pHttpVariablesData->m_Variables->Get(WLJSNAME_HTTP_AUTO_NAMED_TRANS).m_Value.BoolVal ;
	}
  if (argc!=4 && argc!=3)
  {
      WLErrorException e(HTTP_ERROR_FILE, 309);
		  return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
	if (ptrHttp && argc==4)
  {
    if ( !JSVAL_IS_STRING(argv[1]) || !JSVAL_IS_STRING(argv[2]) || !JSVAL_IS_INT(argv[3]))
    {
      WLErrorException e(HTTP_ERROR_FILE, 309);
		  return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }

    //we check if the frame is a FRAME or a WINDOW in GetWindowOrFrame function.
  	CWLDocument* frame = GetWindowOrFrame(ptrHttp, cx, 1, argc, argv[0]);
    if (!frame)
    {
      WLErrorException e(HTTP_ERROR_FILE, 309);
		  return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }

    const char* sReason     = JS_GetStringBytes(JS_ValueToString(cx, argv[1]));
    const char* sDescription= JS_GetStringBytes(JS_ValueToString(cx, argv[2]));
    WLLogMessage::Severity severity = (WLLogMessage::Severity)JSVAL_TO_INT(argv[3]);

    if (bAutoNamedTransactions)
      frame->GetTransactionObj()->AddEvent(sReason, sDescription, severity);
    else
    {
      ptrHttp->VerificationRecordEvent(sReason,
                                  sDescription,
  														    TVPSysTime(),
                                  severity);
      ptrHttp->reportTime( severity, sReason, TVPSysTime() );
    }
  }
  else if (ptrHttp && argc==3)
  {
    // check if the severity parameter is int or not.
    if ( !JSVAL_IS_INT(argv[2]))
    {
      WLErrorException e(HTTP_ERROR_FILE, 310);
		  return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }

    //we check if the frame is a FRAME or a WINDOW in GetWindowOrFrame function.
  	CWLDocument* frame = GetWindowOrFrame(ptrHttp, cx, 1, argc, argv[0]);
    if (!frame)
    {
      WLErrorException e(HTTP_ERROR_FILE, 309);
		  return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }

    const char *name="Undefined", *description="";
    const char* data="";

    JSObject* EventObj = JSVAL_TO_OBJECT(argv[1]);
    WLLogMessage::Severity severity = (WLLogMessage::Severity)JSVAL_TO_INT(argv[2]);
    jsval nameProperty, descriptionProperty, getString;

// should replace those strings to defines.
    if (JS_GetProperty(cx, EventObj, "Name", &nameProperty)==JS_TRUE)
      name = JS_GetStringBytes(JS_ValueToString(cx, nameProperty));

    if (JS_GetProperty(cx, EventObj, "Description", &descriptionProperty)==JS_TRUE)
        description = JS_GetStringBytes(JS_ValueToString(cx, descriptionProperty));

    if( JS_CallFunctionName( cx, EventObj, "ToString", 0, 0, &getString) )
    {
       data = JS_GetStringBytes( JS_ValueToString( cx, getString ) );
    }
    else
    {
//should write an appropriate error !
      WLErrorException e(HTTP_ERROR_FILE, 309);
		  return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }

    if (bAutoNamedTransactions)
    {
      frame->GetTransactionObj()->AddEvent(
                    name,
                    description,
                    severity,
                    "Functional Test",    ///event type - have to be changed !
                    (void*)data,
                    strlen(data));
    }
    else
    {
      ptrHttp->VerificationRecordEvent(name,
                                  description,
  														    TVPSysTime(),
                                  severity,
                                  "Functional Test",    ///event type - have to be changed !
                                  (void*)data,
                                  strlen(data));
    }
  }

  return JS_TRUE;
}

static JSBool
http_SetSeverity(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
  if (argc!=2)
  {
    WLErrorException e(HTTP_ERROR_FILE, 310);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
	else if (ptrHttp)
  {
    if ( !JSVAL_IS_INT(argv[1]))
    {
      WLErrorException e(HTTP_ERROR_FILE, 310);
		  return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }

    //we check if the frame is a FRAME or a WINDOW in GetWindowOrFrame function.
  	CWLDocument* frame = GetWindowOrFrame(ptrHttp, cx, 1, argc, argv[0]);
    if (!frame)
    {
      WLErrorException e(HTTP_ERROR_FILE, 310);
		  return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }

    WLLogMessage::Severity severity = (WLLogMessage::Severity)JSVAL_TO_INT(argv[1]);

    if (frame)
      frame->GetTransactionObj()->GetTransaction()->SetSeverity(severity);
  }

  return JS_TRUE;
}

static JSBool
http_SetFailureReason(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);

  if (argc!=3)
  {
    WLErrorException e(HTTP_ERROR_FILE, 311);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
	else if (ptrHttp)
  {
    if ( !JSVAL_IS_STRING(argv[1]) || !JSVAL_IS_STRING(argv[2]))
    {
      WLErrorException e(HTTP_ERROR_FILE, 311);
		  return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }

    //we check if the frame is a FRAME or a WINDOW in GetWindowOrFrame function.
  	CWLDocument* frame = GetWindowOrFrame(ptrHttp, cx, 1, argc, argv[0]);
    if (!frame)
    {
      WLErrorException e(HTTP_ERROR_FILE, 311);
		  return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }

    const char* sReason     = JS_GetStringBytes(JS_ValueToString(cx, argv[1]));
    const char* sDescription= JS_GetStringBytes(JS_ValueToString(cx, argv[2]));

    frame->GetTransactionObj()->SetFailureReason(sReason, sDescription);
  }

  return JS_TRUE;
}

static JSBool
HttpArrays_Clear(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  CWLDataBlock* DataArray = NULL;
  bool DoClear = false ;

	CWLHttpPrivateData* HttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
  if (HttpVars) {
    //We assume that this object was defined with a class.
    //Thus we don't check if the JS_GetClass returns null.
    const char* ClassName = JS_GetClass(cx,obj)->name ;
    if (!strcmp(ClassName, WLJSNAME_FORMDATA_CLASS))
      DataArray = HttpVars->m_FormdataBlock ;
    else if (!strcmp(ClassName, WLJSNAME_HEADER_CLASS))
      DataArray = HttpVars->m_HeaderBlock ;
    else if (!strcmp(ClassName, WLJSNAME_DATA_CLASS))
      DataArray = HttpVars->m_DataBlock ;
    else if (!strcmp(ClassName, WLJSNAME_DATAFILE_CLASS))
      DataArray = HttpVars->m_DatafileBlock ;

    if (DataArray) {
      if (argc == 0) {
        DataArray->clearAndDestroy();
      }
      else {
        char* KeyName ;
        uintN i ;
        for (i = 0 ; i < argc; i++) {
          KeyName = JS_GetStringBytes(JS_ValueToString(cx, argv[i]));
//        JS_DeleteProperty(cx, obj, KeyName )
          DataArray->removeAllKeysAndValues(KeyName) ;
        };
      } ;
    } ;
  };

  return JS_TRUE ;
};


// some debug functions...
#ifdef _DEBUG

static JSBool
http_ParseHtmlFile(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 1 || !JSVAL_IS_STRING(argv[0]))
		return JS_TRUE;

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		ptrHttp->ParseHtmlFile(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
		WLBuildHtmlParseTree_WebLoad(cx, obj, ptrHttp);
		WLBuildHtmlParseTree_Browser(cx, obj, ptrHttp);
	}

	return JS_TRUE;
}

static JSBool
http_SetEndOfRound(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 1 || !JSVAL_IS_STRING(argv[0]))
		return JS_TRUE;

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp)
		ptrHttp->DebugFunction(DEBUG_SET_END_OF_ROUND);

	return JS_TRUE;
}

static JSBool
http_ReadResponseFromFile(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 2 || !JSVAL_IS_STRING(argv[0]) || !JSVAL_IS_INT(argv[1]))
		return JS_TRUE;

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		ptrHttp->ReadResponseFromFile(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])), JSVAL_TO_INT(argv[1]));
		WLBuildHtmlParseTree_WebLoad(cx, obj, ptrHttp);
		WLBuildHtmlParseTree_Browser(cx, obj, ptrHttp);
	}

	return JS_TRUE;
}

static JSBool
http_CreateTable(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc < 4 ||
			!JSVAL_IS_STRING(argv[0]) ||
			!JSVAL_IS_INT(argv[1]) ||
			!JSVAL_IS_INT(argv[2]) ||
			!JSVAL_IS_INT(argv[3]))
		return JS_TRUE;

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		ptrHttp->CreateTable(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])), JSVAL_TO_INT(argv[1]), JSVAL_TO_INT(argv[2]), JSVAL_TO_INT(argv[3]));
		WLBuildHtmlParseTree_WebLoad(cx, obj, ptrHttp);
		WLBuildHtmlParseTree_Browser(cx, obj, ptrHttp);
	}

	return JS_TRUE;
}

static JSBool
http_Write(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	printf("Version: %d\n", JS_GetVersion(cx));
	JSString* str = JS_ValueToString(cx, argv[0]);
	if (str) {
		printf(JS_GetStringBytes(str));
		printf("\n");
	}

	return JS_TRUE;
}

#endif
// debug functions


static JSFunctionSpec http_methods[] = {
	{wl_toString_str,						http_toString,				0},
	{WLJSNAME_HTTP_HEAD,				http_Head,						1},
	{WLJSNAME_HTTP_GET,					http_Get,							1},
	{WLJSNAME_HTTP_POST,				http_Post,						1},
	{WLJSNAME_HTTP_CLOSE_CONN,	http_CloseConnection,	0},
	{WLJSNAME_DNS_CLEAR_CACHE,	http_ClearDnsCache,		0},
	{WLJSNAME_SSL_CLEAR_CACHE,	http_ClearSslCache,		0},

	{WLJSNAME_HTTP_INSERT_TRANSACTIONS,	http_InsertTransactions,		0},
  {WLJSNAME_HTTP_ADD_EVENT,   http_AddEvent,        4},
  {WLJSNAME_HTTP_SET_SEVERITY,http_SetSeverity,     2},
  {WLJSNAME_HTTP_SET_FAILURE_REASON, http_SetFailureReason,       3},
  {WLJSNAME_HTTP_GET_IP_ADDRESS, http_GetIPAddress, 0 },
#ifdef _DEBUG
	{"ParseHtmlFile",						http_ParseHtmlFile,					1},
	{"SetEndOfRound",						http_SetEndOfRound,					0},
	{"ReadRespnoseFromFile",		http_ReadResponseFromFile,	2},
	{"RRFF",										http_ReadResponseFromFile,	2},
	{"CreateTable",							http_CreateTable,           4},
	{"Write",										http_Write,									1},
#endif
	{0}

};
static JSFunctionSpec HttpArrays_methods[] = {
	{WLJSNAME_HTTP_ARRAY_CLEAR,	HttpArrays_Clear,	1, JSPROP_READONLY, 0},
	{WLJSNAME_HTTP_ARRAY_SET,	HttpArrays_Set,	  2, JSPROP_READONLY, 0},
	{WLJSNAME_HTTP_ARRAY_GET,	HttpArrays_Get,	  1, JSPROP_READONLY, 0},
	{0,0,0,0,0}
};

//Utilities for data arrays
bool GetValueFromDataBlock(CWLDataBlock* block, const char* key, const char* & retVal)
{
  //check if block if empty
  if (block) {
	  for (size_t i = 0; i < block->entries(); i++)
		  if (!strcmp(key, block->Key(i))) {
			  retVal = block->Value(i);
			  return true;
		  }
  }

	retVal = NULL;
	return false;
}
void setBlockKeyAndValue(CWLDataBlock* & block, const char* key, const char* value)
{
	if (!block)
		block = new CWLDataBlock;
	if (!block)
		return;

	block->insertKeyAndValue(key, value);
}

/*********************************************************************\

	HEADER object

\*********************************************************************/

static JSBool
header_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside header_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
  JSType ValueType = JS_TypeOfValue(cx, *vp) ;
  if (ValueType == JSTYPE_FUNCTION) return JS_TRUE ;

  const char* key = JS_GetStringBytes(JS_ValueToString(cx, id));
	const char* value = NULL;

	CWLHttpPrivateData* HttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
	if (HttpVars) {
   	*vp = JSVAL_VOID;
		if (GetValueFromDataBlock(HttpVars->m_HeaderBlock, key, value)) {
			if (value) {
				JSString* str = JS_NewStringCopyZ(cx, value);
				*vp = STRING_TO_JSVAL(str);
      }
    }
  }

	return JS_TRUE;
}


static JSBool
header_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{

	const char* key = JS_GetStringBytes(JS_ValueToString(cx, id));
	const char* value = JS_GetStringBytes(JS_ValueToString(cx, *vp));

	CWLHttpPrivateData* HttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
  if (HttpVars)
    setBlockKeyAndValue(HttpVars->m_HeaderBlock, key, value);

	return JS_TRUE;
}



JSClass js_HeaderClass = {
    WLJSNAME_HEADER_CLASS,
    JSCLASS_HAS_PRIVATE,
    JS_PropertyStub,	// add Property
		JS_PropertyStub,	// del Property
		header_getProperty,	// get Property
		header_setProperty,	// set Property
    JS_EnumerateStub,
		JS_ResolveStub,
		JS_ConvertStub,
		JS_FinalizeStub
};


bool DllExport WLAddJSHeaderClass(JSContext *cx, JSObject *obj)
{
	// define a class of type Cookie
	if (!JS_InitClass(cx, obj, NULL, &js_HeaderClass, NULL, 1, NULL, HttpArrays_methods, NULL, NULL))
		return false;

	return true;
}

/*********************************************************************\

	FORMDATA object

\*********************************************************************/

static JSBool
form_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside form_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
  JSType ValueType = JS_TypeOfValue(cx, *vp) ;
  if (ValueType == JSTYPE_FUNCTION) return JS_TRUE ;

  const char* key = JS_GetStringBytes(JS_ValueToString(cx, id));
	const char* value = NULL;

	CWLHttpPrivateData* HttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
	if (HttpVars) {
 		*vp = JSVAL_VOID;
		if (GetValueFromDataBlock(HttpVars->m_FormdataBlock, key, value)) {
			if (value) {
				JSString* str = JS_NewStringCopyZ(cx, value);
				*vp = STRING_TO_JSVAL(str);
      }
    }
  }

	return JS_TRUE;
}


static JSBool
form_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{

	const char* key = JS_GetStringBytes(JS_ValueToString(cx, id));
	const char* value = JS_GetStringBytes(JS_ValueToString(cx, *vp));

	CWLHttpPrivateData* HttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
  if (HttpVars)
    setBlockKeyAndValue(HttpVars->m_FormdataBlock, key, value);

	return JS_TRUE;
}


JSClass js_FormClass = {
    WLJSNAME_FORMDATA_CLASS,
    JSCLASS_HAS_PRIVATE,
    JS_PropertyStub,	// add Property
		JS_PropertyStub,	// del Property
		form_getProperty,	// get Property
		form_setProperty,	// set Property
    JS_EnumerateStub,
		JS_ResolveStub,
		JS_ConvertStub,
		JS_FinalizeStub
};

bool DllExport WLAddJSFormClass(JSContext *cx, JSObject *obj)
{
	// define a class of type Cookie
	if (!JS_InitClass(cx, obj, NULL, &js_FormClass, NULL, 1, NULL, HttpArrays_methods, NULL, NULL))
		return false;

	return true;
}



/*********************************************************************\

	DATA object

\*********************************************************************/

static JSBool
data_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside data_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
  JSType ValueType = JS_TypeOfValue(cx, *vp) ;
  if (ValueType == JSTYPE_FUNCTION) return JS_TRUE ;

	if (!JSVAL_IS_STRING(id))
		return JS_TRUE;

	const char* key = JS_GetStringBytes(JS_ValueToString(cx, id));
	const char* value = NULL;

	CWLHttpPrivateData* HttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
	if (HttpVars) {
   	*vp = JSVAL_VOID;
		if (GetValueFromDataBlock(HttpVars->m_DataBlock, key, value)) {
			if (value) {
				JSString* str = JS_NewStringCopyZ(cx, value);
				*vp = STRING_TO_JSVAL(str);
      }
    }
  }

	return JS_TRUE;
}


static JSBool
data_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
	// accept only strings as property (field name)
	if (!JSVAL_IS_STRING(id))
		return JS_TRUE;

	const char* key = JS_GetStringBytes(JS_ValueToString(cx, id));
	const char* value = JS_GetStringBytes(JS_ValueToString(cx, *vp));

	CWLHttpPrivateData* HttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
  if (HttpVars)
    setBlockKeyAndValue(HttpVars->m_DataBlock, key, value);


	return JS_TRUE;
}


JSClass js_DataClass = {
    WLJSNAME_DATA_CLASS,
    JSCLASS_HAS_PRIVATE,
    JS_PropertyStub,	// add Property
		JS_PropertyStub,	// del Property
		data_getProperty,	// get Property
		data_setProperty,	// set Property
    JS_EnumerateStub,
		JS_ResolveStub,
		JS_ConvertStub,
		JS_FinalizeStub
};

bool DllExport WLAddJSDataClass(JSContext *cx, JSObject *obj)
{
	// define a class of type Cookie
	if (!JS_InitClass(cx, obj, NULL, &js_DataClass, NULL, 1, NULL, HttpArrays_methods, NULL, NULL))
		return false;

	return true;
}



/*********************************************************************\

	DATAFILE object

\*********************************************************************/

static JSBool
datafile_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside datafile_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
  JSType ValueType = JS_TypeOfValue(cx, *vp) ;
  if (ValueType == JSTYPE_FUNCTION) return JS_TRUE ;

  if (!JSVAL_IS_STRING(id))
		return JS_TRUE;

	const char* key = JS_GetStringBytes(JS_ValueToString(cx, id));
	const char* value = NULL;

	CWLHttpPrivateData* HttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
	if (HttpVars) {
   	*vp = JSVAL_VOID;
		if (GetValueFromDataBlock(HttpVars->m_DatafileBlock, key, value)) {
			if (value) {
				JSString* str = JS_NewStringCopyZ(cx, value);
				*vp = STRING_TO_JSVAL(str);
      }
    }
  }

	return JS_TRUE;
}


static JSBool
datafile_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
	// accept only strings as property (field name)
	if (!JSVAL_IS_STRING(id))
		return JS_TRUE;

	const char* key = JS_GetStringBytes(JS_ValueToString(cx, id));
	const char* value = JS_GetStringBytes(JS_ValueToString(cx, *vp));

	CWLHttpPrivateData* HttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, obj);
  if (HttpVars)
    setBlockKeyAndValue(HttpVars->m_DatafileBlock, key, value);

	return JS_TRUE;
}


JSClass js_DatafileClass = {
    WLJSNAME_DATAFILE_CLASS,
    JSCLASS_HAS_PRIVATE,
    JS_PropertyStub,	// add Property
		JS_PropertyStub,	// del Property
		datafile_getProperty,	// get Property
		datafile_setProperty,	// set Property
    JS_EnumerateStub,
		JS_ResolveStub,
		JS_ConvertStub,
		JS_FinalizeStub
};

bool DllExport WLAddJSDatafileClass(JSContext *cx, JSObject *obj)
{
	// define a class of type Cookie
	if (!JS_InitClass(cx, obj, NULL, &js_DatafileClass, NULL, 1, NULL, HttpArrays_methods, NULL, NULL))
		return false;

	return true;
}


static JSBool
AddDataStructures(JSContext* cx, JSObject* obj, CWLHttpPrivateData* HttpVars)
{
	// add an object instance (FORMDATA) to Http object
	JSObject* form = JS_DefineObject(cx, obj, WLJSNAME_HTTP_FORMDATA_STRUCT, &js_FormClass, NULL, 0);
	if (!form)
		return JS_FALSE;

  // set a pointer to the variables class
	JS_SetPrivate(cx, form, HttpVars);


	// add an object instance (HEADER) to Http object
	JSObject* header = JS_DefineObject(cx, obj, WLJSNAME_HTTP_HEADER_STRUCT, &js_HeaderClass, NULL, 0);
	if (!header)
		return JS_FALSE;

  // set a pointer to the variables class
	JS_SetPrivate(cx, header, HttpVars);


	// add an object instance (DATA) to Http object
	JSObject* data = JS_DefineObject(cx, obj, WLJSNAME_HTTP_DATA_STRUCT, &js_DataClass, NULL, 0);
	if (!data)
		return JS_FALSE;

	// set a pointer to the variables class
	JS_SetPrivate(cx, data, HttpVars);


	// add an object instance (DATAFILE) to Http object
	JSObject* datafile = JS_DefineObject(cx, obj, WLJSNAME_HTTP_DATAFILE_STRUCT, &js_DatafileClass, NULL, 0);
	if (!datafile)
		return JS_FALSE;

	// set a pointer to the variables class
	JS_SetPrivate(cx, datafile, HttpVars);


	return JS_TRUE;
}

static JSBool
HttpArrays_Set(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

  //We assume that this object was defined with a class.
  //Thus we don't check if the JS_GetClass returns null.
  const char* ClassName = JS_GetClass(cx,obj)->name ;

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
    if (argc != 2) {
      reportError(ptrHttp, HTTP_ERROR_FILE, 307, ClassName);
      return JS_FALSE ;
    }

    if (!strcmp(ClassName, WLJSNAME_FORMDATA_CLASS))
      return form_setProperty(cx, obj, argv[0] , &argv[1]) ;
    else if (!strcmp(ClassName, WLJSNAME_HEADER_CLASS))
      return header_setProperty(cx, obj, argv[0] , &argv[1]);
    else if (!strcmp(ClassName, WLJSNAME_DATA_CLASS))
      return data_setProperty(cx, obj, argv[0] , &argv[1]) ;
    else if (!strcmp(ClassName, WLJSNAME_DATAFILE_CLASS))
      return datafile_setProperty(cx, obj, argv[0] , &argv[1]) ;
  }

  return JS_TRUE;

}

static JSBool
HttpArrays_Get(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

  //We assume that this object was defined with a class.
  //Thus we don't check if the JS_GetClass returns null.
  const char* ClassName = JS_GetClass(cx,obj)->name ;

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
    if (argc != 1) {
      reportError(ptrHttp, HTTP_ERROR_FILE, 308, ClassName);
      return JS_FALSE ;
    }

    if (!strcmp(ClassName, WLJSNAME_FORMDATA_CLASS))
      return form_getProperty(cx, obj, argv[0] , rval) ;
    else if (!strcmp(ClassName, WLJSNAME_HEADER_CLASS))
      return header_getProperty(cx, obj, argv[0] , rval);
    else if (!strcmp(ClassName, WLJSNAME_DATA_CLASS))
      return data_getProperty(cx, obj, argv[0] , rval) ;
    else if (!strcmp(ClassName, WLJSNAME_DATAFILE_CLASS))
      return datafile_getProperty(cx, obj, argv[0] , rval) ;
  }

  return JS_TRUE;

}

static JSBool
InitHttpObject(JSContext *cx, JSObject *obj, jsint length, jsval *vector, bool wlObj, CLEVariablesData* pVariablesDataParent)
{
  CLEVariablesData* vars = new CLEVariablesData(pVariablesDataParent, CLEPropertyValue::PL_PRIVATE );
  CWLHttpPrivateData* HttpVars = new CWLHttpPrivateData (vars);

  JS_SetPrivate(cx, obj, (void*)HttpVars);

	if (AddDataStructures(cx, obj, HttpVars) != JS_TRUE)
		return JS_FALSE;

	// add the 'Erase' property
	// should be added as read-only for the wlHttp object
  //AViad
  //vars->SetBoolVariable(WLJSNAME_HTTP_ERASE, true);

	//vars->SetBuildInValue(wlObj);

	return JS_TRUE;
}


static JSBool
WLHttp(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	jsint length;
	jsval *vector;
	JSBool ok = JS_FALSE;

	if (argc == 0) {
		length = 0;
		vector = NULL;
	}
	else {
		length = argc;
		vector = argv;
	}
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp)
		ok = InitHttpObject(cx, obj, length, vector, false, ptrHttp->GetVariablesParent());
	if (ok)
		*rval = OBJECT_TO_JSVAL(obj);

	return ok;
}

bool DllExport WLAddHttpClass(JSContext *cx, JSObject *obj)
{
	// define a class of type WLHttp
	if (!JS_InitClass(cx, obj, NULL, &js_HttpClass, WLHttp, 1, NULL, http_methods, NULL, NULL))
		return false;

	return true;
}


bool DllExport WLAddHttpObject(JSContext *cx, JSObject *obj, CLEVariablesData* pVariablesDataParent)
{
WlTraceMsg("Adding %s", WLJSNAME_HTTP_OBJECT);
	// create an instance of type WLHttp
	JSObject* wlHttp = JS_DefineObject(cx, obj, WLJSNAME_HTTP_OBJECT, &js_HttpClass, NULL, JSPROP_PERMANENT);
	if (!wlHttp)
		return false;

	// define it's properties & structures
	if (!InitHttpObject(cx, wlHttp, 0, NULL, true, pVariablesDataParent))
		return false;

	return true;
}



// ==========================================================================

// this section defines global HTTP functions

static JSBool execDomFunction(int mode, JSContext* cx, JSObject* obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (argc < 1) {
		// filename not supplied
		if (ptrHttp)
			reportError(ptrHttp, HTTP_ERROR_FILE, 231, WLJSNAME_CREATE_DOM);
		return JS_TRUE;
	}

	if (!JSVAL_IS_STRING(argv[0])) {
		// parameter must be a string
		if (ptrHttp)
			reportError(ptrHttp, HTTP_ERROR_FILE, 232, WLJSNAME_CREATE_DOM);
		return JS_TRUE;
	}

	if (ptrHttp) {
		char* filename = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
    //Concatinate LoadEngine temporary path name to the file name.
		//Galtech
		RVCString leFileName =  CArrangeMachineTempFiles::GetWlTempFullPath (cx, WLECopiedFile, ToANSI(filename)) ;

    CWLDocument* dom = NULL;

		CWLHttpPrivateData* HttpVars = NULL;
		if (!(ptrHttp->GetWorkspace()->IsInit())) {
			// get the wlHttp object in order to get it's private (the variables object)
			jsval v;
			if (JS_GetProperty(cx, obj, WLJSNAME_HTTP_OBJECT, &v) && JSVAL_IS_OBJECT(v))
				HttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, JSVAL_TO_OBJECT(v));
		}

		// let the GC work
		int res;
		{
			CJSGCRequestSuspendResume gc(cx);
			res = ptrHttp->CreateDom(HttpVars, leFileName, &dom, mode);
		}

		if (dom)
			// create new object that will hold the created dom
			WLSetWindowValue(cx, obj, dom, dom, rval);

    if (res == RET_ERROR && ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (ptrHttp->isAbort())
			return JS_FALSE;
	}

	return JS_TRUE;
}

static JSBool CreateDom(JSContext* cx, JSObject* obj, uintN argc, jsval *argv, jsval *rval)
{
	return execDomFunction(FullDom, cx, obj, argc, argv, rval);
}


static JSBool CreateTable(JSContext* cx, JSObject* obj, uintN argc, jsval *argv, jsval *rval)
{
	return execDomFunction(OnlyTables, cx, obj, argc, argv, rval);
}

/**************************************************************************************
    Function : CreateExpectedDom
    Input    : DomRepository object - holding all snapshots recorded by AAT
               int ItemId - The specific item in repository to create DOM of.
    Output   : creating new DOM object.
**************************************************************************************/
static JSBool CreateExpectedDom(JSContext* cx, JSObject* obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (argc != 2)
  {
    WLErrorException e(HTTP_ERROR_FILE, 312);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

	if (ptrHttp)
  {
    int res, nItemId;
		CWLDocument* dom = NULL;
		CWLHttpPrivateData* HttpVars = NULL;
    CWLDomRepository *pDomRepository=NULL;

		if (!(ptrHttp->GetWorkspace()->IsInit()))
    {
      jsval v;
			if (JS_GetProperty(cx, obj, WLJSNAME_HTTP_OBJECT, &v) && JSVAL_IS_OBJECT(v))
				HttpVars = (CWLHttpPrivateData*)JS_GetPrivate(cx, JSVAL_TO_OBJECT(v));
		}

    // get the DomRepository object from private (the variables object)
    if (!WLIsObjectOfClass(cx, argv[0], WLJSNAME_DOM_REPOSITORY_CLASS) || !JSVAL_IS_INT(argv[1]) )
    {
      WLErrorException e(HTTP_ERROR_FILE, 312);
		  return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }

    JSObject* obj = JSVAL_TO_OBJECT(argv[0]);
		pDomRepository = (CWLDomRepository*)JS_GetPrivate(cx, obj);

    nItemId = JSVAL_TO_INT(argv[1]);

		// let the GC work
		{
			CJSGCRequestSuspendResume gc(cx);
      CWLDomItem*   pDomItem = pDomRepository->get(nItemId);
      if (!pDomItem) //illegal repository item
      {
        WLErrorException e(HTTP_ERROR_FILE, 313);
		    return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
      }

      res = ptrHttp->CreateExpectedDom(HttpVars, pDomItem, &dom);
		}

		if (dom && res != RET_ERROR)  // create new object that will hold the created dom
			WLSetWindowValue(cx, obj, dom, dom, rval);

    if (res == RET_ERROR && ptrHttp->GetErrorSeverity() >= WLLogMessage::Error)
     	return JS_FALSE;

		if (ptrHttp->isAbort())
			return JS_FALSE;
	}

	return JS_TRUE;
}

static JSBool EncodeBuffer(JSContext* cx, JSObject* obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (argc < 1) {
		// buffer not supplied
		if (ptrHttp)
			reportError(ptrHttp, HTTP_ERROR_FILE, 231, WLJSNAME_ENCODE_STRING);
		return JS_TRUE;
	}
	if (!JSVAL_IS_STRING(argv[0])) {
		// parameter must be a string
		if (ptrHttp)
			reportError(ptrHttp, HTTP_ERROR_FILE, 232, WLJSNAME_ENCODE_STRING);
		return JS_TRUE;
	}

	RVCString res("");
	char* string = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
	//Galtech:  encoding should be done for ANSI codes and not UTF8
	EncodeString(ToANSI(string, GetMonitorCharSet()), res, true);

	JSString* str = JS_NewStringCopyZ(cx, res.data());
	if (str)
		*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}

// ================  SSL functions  ===================

#ifdef __OPEN_SSL__

static bool GetAndCheckCM(JSContext* cx, CWLCommandManager** pCM, const char* func)
{
	*pCM = WLGetCommandManagerPtr(cx);

	if (!*pCM)
		return false;

  // Till will have a strong demand for changing the
	// threads amount before an http command was made
  // we disallow the user to change the ciphers after
  // he had started doing http
  if ((*pCM)->WasHttpCommand() ) {
	  reportError(*pCM, CIPHER_ERROR_FILE, 10, func);
		return false;
  }

	return true;
}

static bool CheckCipherCommandArgs(JSContext* cx, uintN argc, jsval* argv, CWLCommandManager** pCM, const char* func, const char** str)
{
	if (argc < 1) {
		// suite name not supplied
		reportError(*pCM, CIPHER_ERROR_FILE, 11, func);
		return false;
	}

	if (!JSVAL_IS_STRING(argv[0])) {
		// suite name must be a string
		reportError(*pCM, CIPHER_ERROR_FILE, 12, func);
		return false;
	}

	*str = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));

	return true;
}

static void UpdateAll(CWLCommandManager* pCM)
{
	// execute the command for each of the SSL context templates exist !
	CWLGenericConnection* conn = NULL;

	RVOrdered* sessionsPool = pCM->GetConnPool()->GetConnectionsPool();
	for (size_t i = 0; i < sessionsPool->entries(); i++) {
		conn = ((CWLGenericConnection*)(sessionsPool->at(i)));
		pCM->GetConnPool()->GetCipherSuite()->Update( conn );
	}
}

static JSBool SslEnableCipherName(JSContext* cx, JSObject* obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager*  pCM   = WLGetCommandManagerPtr(cx);
	if( NULL != pCM )
		reportError(pCM, CIPHER_ERROR_FILE, 70 );
	return JS_FALSE;
}

static JSBool SslDisableCipherName(JSContext* cx, JSObject* obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager*  pCM   = WLGetCommandManagerPtr(cx);
	if( NULL != pCM )
		reportError(pCM, CIPHER_ERROR_FILE, 70 );
	return JS_FALSE;
}



static bool CheckIntArgs(JSContext* cx, uintN argc, jsval* argv, CWLCommandManager** pCM, const char* func, int& val)
{
	*pCM = WLGetCommandManagerPtr(cx);

	if (!*pCM)
		return false;

	if (argc < 1) {
		// suite name not supplied
		if (*pCM)
			reportError(*pCM, CIPHER_ERROR_FILE, 13, func);
		return false;
	}

	if (!JSVAL_IS_INT(argv[0])) {
		// suite name  must be a string
		if (*pCM)
			reportError(*pCM, CIPHER_ERROR_FILE, 14, func);
		return false;
	}

	val = JSVAL_TO_INT(argv[0]);

	return true;
}

static JSBool SslCipherSuiteCommand(JSContext* cx, JSObject* obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager* pCM = NULL;
	const char* name = NULL;
	int CipherSuiteCommand=CWLSslCiphersCollection::CIPHER_NULL_COMMAND;

  if (GetAndCheckCM(cx, &pCM, WLJSNAME_SSL_CIPHER_SUITE_COMMAND)) {
    if (CheckCipherCommandArgs(cx, argc, argv, &pCM, WLJSNAME_SSL_CIPHER_SUITE_COMMAND, &name)) {
      if (name) {
        if (!stricmp(name , "EnableAll"))
          CipherSuiteCommand=CWLSslCiphersCollection::CIPHER_ENABLE_ALL;
        else if (!stricmp(name , "DisableAll"))
          CipherSuiteCommand=CWLSslCiphersCollection::CIPHER_DISABLE_ALL;
        else if (!stricmp(name , "ShowAll"))
          CipherSuiteCommand=CWLSslCiphersCollection::CIPHER_SHOW_ALL;
        else if (!stricmp(name , "ShowEnabled"))
          CipherSuiteCommand=CWLSslCiphersCollection::CIPHER_SHOW_ENABLED;

        if (CipherSuiteCommand!=CWLSslCiphersCollection::CIPHER_NULL_COMMAND) {
      	        pCM->GetConnPool()->GetCipherSuite()->DoCommand(CipherSuiteCommand, name);
        }
      }
    }
  }

	return JS_TRUE;
}


static JSBool SslEnableCipherId(JSContext* cx, JSObject* obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager*  pCM   = WLGetCommandManagerPtr(cx);
	if( NULL != pCM )
		reportError(pCM, CIPHER_ERROR_FILE, 70 );
	return JS_FALSE;
}

static JSBool SslDisableCipherId(JSContext* cx, JSObject* obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager*  pCM   = WLGetCommandManagerPtr(cx);
	if( NULL != pCM )
		reportError(pCM, CIPHER_ERROR_FILE, 70 );
	return JS_FALSE;
}

static JSBool SslEnableStrength(JSContext* cx, JSObject* obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager* pCM = WLGetCommandManagerPtr(cx);
	if( NULL != pCM )
		reportError(pCM, CIPHER_ERROR_FILE, 70 );
	return JS_FALSE;
}

static JSBool SslGetCipherCount(JSContext* cx, JSObject* obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager* pCM = WLGetCommandManagerPtr(cx);

	if (!pCM)
		return JS_TRUE;

	*rval = INT_TO_JSVAL( pCM->GetConnPool()->GetCipherSuite()->GetCipherCount() );

	return JS_TRUE;
}
// This is an internal use function that return the information of a cipher according
// to SSL id or by cipher name
static JSBool SslGetCipherInfo(JSContext* cx, JSObject* obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager*  pCM   = WLGetCommandManagerPtr(cx);
	if( NULL != pCM )
		reportError(pCM, CIPHER_ERROR_FILE, 70 );
	return JS_FALSE;
}
// This function gets a cipher name and return it's SSL id
static JSBool SslGetCipherId(JSContext* cx, JSObject* obj, uintN argc, jsval *argv, jsval *rval)
{
	int         indx;
	const char* name = NULL;

	CWLCommandManager* pCM = WLGetCommandManagerPtr(cx);
	if (!pCM)
		return JS_TRUE;

	if (argc < 1)
		return JS_TRUE;

  if (JSVAL_IS_STRING(argv[0])) {
		name = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
		if (name) {
			indx = pCM->GetConnPool()->GetCipherSuite()->NameToId( name );
			*rval = INT_TO_JSVAL(indx);
		}
	}
	return JS_TRUE;
}
// This function gets a cipher SSL id and returns the cipher name
static JSBool SslGetCipherName(JSContext* cx, JSObject* obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager* pCM = WLGetCommandManagerPtr(cx);

	if (!pCM)
		return JS_TRUE;

	int indx;
	const char* name = NULL;

  if (argc < 1)
		return JS_TRUE;

	if (JSVAL_IS_INT(argv[0]))
		indx = JSVAL_TO_INT(argv[0]);
	else
		return JS_TRUE;

	name = pCM->GetConnPool()->GetCipherSuite()->IdToName( indx );

   	if( NULL != name ){
		JSString* str = JS_NewStringCopyZ(cx, name);
      *rval = STRING_TO_JSVAL(str);
    }

	return JS_TRUE;
}
// This function returns the current strength

static JSBool SslGetCipherStrength(JSContext* cx, JSObject* obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager* pCM = WLGetCommandManagerPtr(cx);

	if (!pCM)
		return JS_TRUE;

	char* strength = NULL;

//	extern void SSLGetStrengthList(char** strengthList);
	pCM->GetConnPool()->GetCipherSuite()->GetStrengthList( &strength );
	JSString* str = JS_NewStringCopyZ(cx, strength);
	free(strength);
	if (str)
		*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}

#endif __OPEN_SSL__


static JSFunctionSpec http_functions[] = {
	{WLJSNAME_CREATE_DOM,					CreateDom,				2},
	{WLJSNAME_CREATE_TABLE,				CreateTable,			2},
  {WLJSNAME_CREATE_EXPECTED_DOM,CreateExpectedDom,2},
	{WLJSNAME_ENCODE_STRING,			EncodeBuffer,			1},

#ifdef __OPEN_SSL__

	{WLJSNAME_SSL_CIPHER_SUITE_COMMAND, SslCipherSuiteCommand,1},
	{WLJSNAME_SSL_ENABLE_CIPHER_NAME,		SslEnableCipherName,	1},
	{WLJSNAME_SSL_DISABLE_CIPHER_NAME,	SslDisableCipherName,	1},
	{WLJSNAME_SSL_ENABLE_CIPHER_ID,			SslEnableCipherId,		1},
	{WLJSNAME_SSL_DISABLE_CIPHER_ID,		SslDisableCipherId,		1},
	{WLJSNAME_SSL_ENABLE_STRENGTH,			SslEnableStrength,		1},
	{WLJSNAME_SSL_GET_CIPHER_COUNT,			SslGetCipherCount,		0},
	{WLJSNAME_SSL_GET_CIPHER_INFO,			SslGetCipherInfo,			1},
	{WLJSNAME_SSL_GET_CIPHER_STRENGTH,	SslGetCipherStrength,	0},
	{WLJSNAME_SSL_GET_CIPHER_NAME,	    SslGetCipherName,	    1},
	{WLJSNAME_SSL_GET_CIPHER_ID,	      SslGetCipherId,	      1},

#endif __OPEN_SSL__
	{0}
};


bool WLAddHttpFunctions(JSContext* cx, JSObject* obj)
{
	if (!JS_DefineFunctions(cx, obj, http_functions))
		return false;

	return true;
}
