/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "rvjsapi.h"
#include "wlframecontainer.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "wlfuncs.h"
#include "jsprivate.h"
#include "TraceDll.h"

#include "propertieshandler.h"

static JSBool
img_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside img_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
 	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLEmbed* location = (CWLEmbed*)prv->GetObj();
	if (!location)
		return JS_TRUE;

	CWLFrame* frame = (CWLFrame*)prv->GetParent();
	if (!frame)
		return JS_TRUE;

	if (location && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

    switch (PropertiesHandler::GetPropertyId(prop))
    {
    case PropertiesHandler::EN_PROTOCOL : {	// return the Protocol property
			const char* protocol = location->Protocol();
			return WLSetStringValue(cx, obj, prop, protocol, vp);
		}

    case PropertiesHandler::EN_SRC : {	// return the Protocol property
			const char* src = location->Href();
			return WLSetStringValue(cx, obj, prop, src, vp);
		}

    case PropertiesHandler::EN_WLORIGURL : {	// return the wlOrigUrl property
			const char* origUrl = location->GetOriginalUrl();
			return WLSetStringValue(cx, obj, prop, origUrl, vp);
		}

    case PropertiesHandler::EN_ID : {	// return the id property
			const char* id = location->Id();
			return WLSetStringValue(cx, obj, prop, id, vp);
		}

    case PropertiesHandler::EN_WLSOURCE_INDEX : { // return the WLSourceIndex property
			int index = location->GetIndex();
			return WLSetIntValue(cx, obj, prop, index, vp);
		}

    case PropertiesHandler::EN_NAME : { // return the name property
			const char* name = location->GetName();
			return WLSetStringValue(cx, obj, prop, name, vp);
		}

    case PropertiesHandler::EN_FILE_SIZE : { // return the FileSize property
			int nFileSize = location->GetTransactionObj()->GetFileSize();
			return WLSetIntValue(cx, obj, prop, nFileSize, vp);
		}

    case PropertiesHandler::EN_CHECKSUM : { // return the CheckSum property
			int nCheckSum = location->GetTransactionObj()->GetCheckSum();
			return WLSetIntValue(cx, obj, prop, nCheckSum, vp);
		}

    case PropertiesHandler::EN_OPEN_TIME : { // return the OpenTime property
			double dTime = location->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_OPEN_CONNECTION);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_SEND_TIME : { // return the SendTime property
			double dTime = location->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_SEND);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_WAIT_TIME : { // return the WaitTime property
			double dTime = location->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_WAIT);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_RECEIVE_TIME : { // return the ReceiveTime property
			double dTime = location->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_RECEIVE);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_WIDTH : { // return the width property
			const char* width = location->GetWidth();
			return WLSetStringValue(cx, obj, prop, width, vp);
		}

    case PropertiesHandler::EN_HEIGHT : { // return the height property
			const char* height = location->GetHeight();
			return WLSetStringValue(cx, obj, prop, height, vp);
		}

    case PropertiesHandler::EN_ALT : { // return the alt property
			const char* alt = location->GetAlt();
			return WLSetStringValue(cx, obj, prop, alt, vp);
		}

    case PropertiesHandler::EN_USEMAP : { // return the useMap property
			const char* usemap = location->GetUseMap();
			return WLSetStringValue(cx, obj, prop, usemap, vp);
		}

    case PropertiesHandler::EN_LOWSRC : { // return the lowsrc property
			const char* lowsrc = location->GetLowsrc();
			return WLSetStringValue(cx, obj, prop, lowsrc, vp);
		}

    case PropertiesHandler::EN_TITLE : { // return the title property
			const char* title = location->GetTitle();
			return WLSetStringValue(cx, obj, prop, title, vp);
		}

    default:;
    }
	}

//???	return JS_GetProperty(cx, obj, id, vp);
	return JS_TRUE;
}


static JSBool
img_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside img_setProperty");
/*
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLLocation* location = (CWLLocation*)prv->GetObj();
	if (location && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

		if (!strcmp(prop, WLJSNAME_PROP_PROTOCOL)) {	// set the Protocol property
			location->setProtocol(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HOSTNAME)) {	// return the Hostname property
			location->setHostname(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_PORT)) {	// return the Port property
			location->setPort(WLIntValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HOST)) {	// return the Host property
			location->setHost(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_PATHNAME)) {	// return the Uri property
			location->setUri(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HASH)) {	// return the Hash property
			location->setHash(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_KVP)) {	// return the KVP property
//!!!???			location->setKvp(WLArrayValue(cx, *vp));	???
//			location->Array() = *WLArrayValue(cx, *vp);
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_SEARCH)) {	// return the Search property
			location->setSearch(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HREF)) {	// return the Href property
			location->setHref(WLStringValue(*vp));
			return JS_TRUE;
		}
	}
*/

	return JS_TRUE;
}

JSBool img_ClassEnumerate(JSContext *cx, JSObject *obj ){

  JS_DefineProperty(cx, obj, WLJSNAME_PROP_ID, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_NAME, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_PROTOCOL, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_WIDTH, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_HEIGHT, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_ALT, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_LOWSRC, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_SRC, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);

  return JS_TRUE;
}

static void
img_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside img_finalize");
	WLFinalize(cx, obj);
}



JSClass wl_ImgClass = {
	WLJSNAME_CLASS_IMG,
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	img_getProperty,
	img_setProperty,
	img_ClassEnumerate,
	JS_ResolveStub,
	JS_ConvertStub,
	img_finalize
};


/*
static JSBool
img_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLLocation* location = (CWLLocation*)prv->GetObj();
	if (!location)
		return JS_TRUE;

	// set rval to Href property
	const char* href = location->Href();
	JSString* str = JS_NewStringCopyZ(cx, href);
	if (str)
		*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}


static JSFunctionSpec img_methods[] = {
	{wl_toString_str,	img_toString,	0},
	{0}
};
*/


/*
static JSBool
InitImgObject(JSContext *cx, JSObject *obj, jsint length, jsval *vector)
{
WlTraceMsg("initializing img object properties");
	// create the CPP object


	// add it's properties
	if (!WLInitHiddenProperty(cx, obj, WLJSNAME_PROP_DOM_SCOPE))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_PROTOCOL))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_HOSTNAME))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_PORT))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_HOST))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_SEARCH))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_KVP))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_HASH))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_SRC))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_HREF))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_PROTOCOL))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_HOSTNAME))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_PORT))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_HOST))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_SEARCH))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_KVP))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_SEARCH_PAIRS))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_HASH))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_HREF))
		return JS_FALSE;

	return JS_TRUE;
}


static JSBool
WLImg(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	JSBool ok;

//	JS_Lock(JS_GetRuntime(cx));
	ok = InitImgObject(cx, obj, 0, NULL);
//	JS_Unlock(JS_GetRuntime(cx));
	if (ok)
		*rval = OBJECT_TO_JSVAL(obj);

	return ok;
}


bool DllExport WLAddImgObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddImgObject(%x, %x)", cx, obj);
	// create an instance of type WLDocument
	JSObject* img = JS_DefineObject(cx, obj, WLJSNAME_OBJECT_IMG, &wl_ImgClass, NULL, JSPROP_PERMANENT);
	if (!img)
		return false;

	JS_SetPrivate(cx, img, NULL);
	
	// define it's properties & structures
	if (!InitImgObject(cx, img, 0, NULL))
		return false;

	return true;
}
*/


JSObject* WLNewImgObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewImgObject(%x, %x)", cx, obj);
	// create an instance of type WLLink
	JSObject* proto = NULL;
	JSObject* img = WLNewObject(cx, &wl_ImgClass, proto, obj);
	if (!img)
		return NULL;

	return img;
}


bool DllExport WLAddImgClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddImgClass(%x, %x)", cx, obj);
	if (!JS_InitClass(cx, obj, NULL, &wl_ImgClass, NULL, 1, NULL, NULL, NULL, NULL))
		return false;

	return true;
}
