/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "rvjsapi.h"

#include "wlerrorlib.h"
#include "domitem.h"
#include "LoadReportInfo.h"
#include "wlfuncs.h"
#include "loadengineglobals.h"
#include "jsreplynames.h"

#define WLJSNAME_ITERATOR_GET_NEXT_KEY "GetNextKey"
#define ErrorRegardingParameter 1


static void IteratorFinalize(JSContext *cx, JSObject *obj)
{
	RVHashDictionaryIterator *pIterator= (RVHashDictionaryIterator*)JS_GetPrivate(cx, obj);
  if( pIterator )
    delete pIterator;
}

static JSBool
IteratorGetNextKey(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  int nItemId = -1 ;
  if (argc != 0)  {
    WLErrorException e(JS_DOM_ERROR_FILE, 120, WLJSNAME_CLASS_ITERATOR"."WLJSNAME_ITERATOR_GET_NEXT_KEY, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

	RVHashDictionaryIterator *pIterator= (RVHashDictionaryIterator*)JS_GetPrivate(cx, obj);

	if (pIterator->operator()())
  {
    RVCollectableInt* key = (RVCollectableInt*)pIterator->key();
    nItemId = (int)*key;
  }

  *rval = INT_TO_JSVAL(nItemId) ;

  return JS_TRUE ;
}



static JSClass IteratorClass = {
  WLJSNAME_CLASS_ITERATOR,
  JSCLASS_HAS_PRIVATE,
  JS_PropertyStub,	// add Property
  JS_PropertyStub,	// del Property
  JS_PropertyStub,	// get Property
  JS_PropertyStub,	// set Property
  JS_EnumerateStub,	
	JS_ResolveStub,
	JS_ConvertStub,
	IteratorFinalize
};

JSBool IteratorInit( JSContext* cx, JSObject* obj, RVHashDictionary& container ) {

  JSBool rtn = JS_FALSE;

  RVHashDictionaryIterator* iter = new RVHashDictionaryIterator(container);
  if(iter) {
     JS_SetPrivate( cx, obj, iter );
     rtn = JS_TRUE;
  }

  return rtn;
}




JSObject* IteratorNewObject( JSContext* cx, RVHashDictionary& container ) {

  JSObject* rtn = JS_NewObject( cx, &IteratorClass, NULL, NULL );
  if(rtn) {
    if( !IteratorInit( cx, rtn, container ) ) {
       rtn = NULL;
    }
  }

  return rtn;
}

// there is no constructor for the mean time, because I'm not allowing the class to be 
// created in javascript. You can create it only in C++ (example: DomRepository), you have
// to supply it with RVHashDictionary. 
/*
static JSBool IteratorCtor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

  if (argc != 1)  {
    WLErrorException e(JS_DOM_ERROR_FILE, ErrorRegardingParameter);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  JSBool rtn = JS_FALSE;
  if (pRep) {
    if (pRep->GetItemHashDictionary()) {
      if( IteratorInit( cx, obj, pRep->GetItemHashDictionary() ) ) {
        rtn = JS_TRUE;
      }
    }
  }
  return rtn;
}
*/




static JSFunctionSpec Iterator_methods[] = {
	{WLJSNAME_ITERATOR_GET_NEXT_KEY,	IteratorGetNextKey,	1},
	{0}
};


bool AddIteratorClass(JSContext *cx, JSObject *obj)
{
  if (!JS_InitClass(cx, obj, NULL, &IteratorClass, NULL, 0, NULL, Iterator_methods, NULL, NULL))
		return false;
  
  return true;
}
