/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlfuncs.h"
#include <string.h>
#include "rvjsapi.h"
#include "wloption.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "jsprivate.h"
#include "TraceDll.h"

#include "propertieshandler.h"

static JSBool
option_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside option_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLOption* option = (CWLOption*)prv->GetObj();
	if (!option)
		return JS_TRUE;

	if (option && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

    switch (PropertiesHandler::GetPropertyId(prop))
    {
    case PropertiesHandler::EN_TEXT : {	// return the Name property
			const char* text = option->Text();
			return WLSetStringValue(cx, obj, prop, text, vp);
		}

    case PropertiesHandler::EN_VALUE : {	// return the Type property
			const char* value = option->Value();
			return WLSetStringValue(cx, obj, prop, value, vp);
		}

    case PropertiesHandler::EN_SELECTED : {	// return the Value property
			bool selected = option->Selected();
			return WLSetBoolValue(cx, obj, prop, selected, vp);
		}

    case PropertiesHandler::EN_DEFAULT_SELECTED : {	// return the Value property
			bool selected = option->Selected();
			return WLSetBoolValue(cx, obj, prop, selected, vp);
		}

    case PropertiesHandler::EN_ID : {	// return the id property
			const char* id = option->Id();
			return WLSetStringValue(cx, obj, prop, id, vp);
		}

    case PropertiesHandler::EN_INNERHTML : {	// return the innerHTML property
			RVCString innerHtml = option->GetInnerHTML();
			return WLSetStringValue(cx, obj, prop, innerHtml.data(), vp);
		}

    default:;
    }
	}

	return JS_TRUE;
}

static JSBool
option_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside option_setProperty");
/*
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLOption* option= (CWLOption*)prv->GetObj();
	if (option && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

		if (!strcmp(prop, WLJSNAME_PROP_TEXT)) {	// return the Name property
			option->setText(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_VALUE)) {	// return the Value property
			option->setValue(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_SELECTED)) {	// return the Options property
			element->setOptions(WLArrayValue(cx, *vp));
			return JS_TRUE;
		}
	}
*/

	return JS_TRUE;
}

static void
option_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside option_finalize");
	WLFinalize(cx, obj);
}

JSClass wl_OptionClass = {
	WLJSNAME_CLASS_OPTION,
	JSCLASS_HAS_PRIVATE,
  JS_PropertyStub,
	JS_PropertyStub,
	option_getProperty,
	option_setProperty,
  JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	option_finalize
};


/*
static JSBool
option_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	// set rval to name
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLOption* option = (CWLOption*)prv->GetObj();
	if (!option)
		return JS_TRUE;

	const char* value = option->Value();
	JSString* str = JS_NewStringCopyZ(cx, value);
	if (str)
		*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}


static JSFunctionSpec option_methods[] = {
	{wl_toString_str,	option_toString,	0},
	{0}
};
*/


/*
static JSBool
InitOptionObject(JSContext *cx, JSObject *obj, jsint length, jsval *vector)
{
WlTraceMsg("initializing option object properties");

	CWLOption* option = new CWLOption;
	JS_SetPrivate(cx, obj, (void*)option);

	if (!WLInitHiddenProperty(cx, obj, WLJSNAME_PROP_DOM_SCOPE))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_TEXT))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_VALUE))
		return JS_FALSE;

/ *
	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_TYPE))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_OPTIONS))
		return JS_FALSE;
* /

	return JS_TRUE;
}


static JSBool
WLOption(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	JSBool ok;

	ok = InitOptionObject(cx, obj, 0, NULL);
	if (ok)
		*rval = OBJECT_TO_JSVAL(obj);

	return ok;
}


bool WLAddOptionObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddOptionObject(%x, %x)", cx, obj);
	// create an instance of type WLHttp
	JSObject* option = JS_DefineObject(cx, obj, WLJSNAME_OBJECT_ELEMENT, &wl_OptionClass, NULL, JSPROP_PERMANENT);
	if (!option)
		return false;

	// define it's properties & structures
	if (!InitOptionObject(cx, option, 0, NULL))
		return false;

	return true;
}
*/


JSObject* WLNewOptionObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewOptionObject(%x, %x)", cx, obj);
	// create an instance of type WLElement
	JSObject* proto = NULL;
	JSObject* option = WLNewObject(cx, &wl_OptionClass, proto, obj);
	if (!option)
		return NULL;

	return option;
}


bool DllExport WLAddOptionClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddOptionClass(%x, %x)", cx, obj);
	// define a class of type WLHttp
	//if (!JS_InitClass(cx, obj, NULL, &wl_OptionClass, WLOption, 1, NULL, option_methods, NULL, NULL))
	if (!JS_InitClass(cx, obj, NULL, &wl_OptionClass, NULL, 1, NULL, NULL, NULL, NULL))
		return false;

	return true;
}
