/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "rvjsapi.h"
#include "wltable.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "wlfuncs.h"
#include "jsprivate.h"
#include "TraceDll.h"

#include "propertieshandler.h"

static JSBool
table_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside table_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	char* key = NULL;
	CWLFrame* frame = (CWLFrame*)prv->GetParent();
	CWLTable* table = (CWLTable*)prv->GetObj();
	if (!table)
		return JS_TRUE;

	if (JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

		if (!prop)
			return JS_TRUE;

    switch (PropertiesHandler::GetPropertyId(prop))
    {
    case PropertiesHandler::EN_ID : {	// return the id property
			const char* id = table->getTableID();
			return WLSetStringValue(cx, obj, prop, id, vp);
		}

    case PropertiesHandler::EN_COLS : {	// return the cols property
			int cols = table->getNumCols();
			return WLSetIntValue(cx, obj, prop, cols, vp);
		}

    case PropertiesHandler::EN_ROWS : {	// return the rows property
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_ROW, (void**)&table, vp);
		}

    case PropertiesHandler::EN_CELLS : {	// return the cells property
			CWLCellCollection* cells = table->getCells();
			return WLSetCellArrayValue(cx, obj, frame, cells, vp);
			//return WLSetArrayValue(cx, obj, WLJSNAME_CLASS_CELL, (void**)&cells, vp);
			//return WLSetArrayValue(cx, obj, WLJSNAME_CLASS_TABLE_CELL, (void**)&cells, vp);
		}

    case PropertiesHandler::EN_OUTERHTML : {	// return the outerHTML property
			const char* outerHtml = table->getOuterHtml();
			return WLSetStringValue(cx, obj, prop, outerHtml, vp);
		}

    default:;
    }
	}
	return JS_TRUE;
}


static JSBool
table_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside table_setProperty");
	return JS_TRUE;
}

JSBool table_ClassEnumerate(JSContext *cx, JSObject *obj ){

  JS_DefineProperty(cx, obj, WLJSNAME_PROP_INNERHTML, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_OUTERHTML, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_CELLS, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_ROWS, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  // @@Chaim - needed?
//  JS_DefineProperty(cx, obj, WLJSNAME_PROP_COLS, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_ID, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);

  return JS_TRUE;
}

static void
table_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside table_finalize(%x, %x)", cx, obj);
	WLFinalize(cx, obj);
}



JSClass wl_TableClass = {
	WLJSNAME_CLASS_TABLE,
	JSCLASS_HAS_PRIVATE,	//ptr to c++ structure that info regarding the object it represents
	JS_PropertyStub,			//callback to C++
	JS_PropertyStub,			//--"--
	table_getProperty,
	table_setProperty,
	table_ClassEnumerate,
	JS_ResolveStub,
	JS_ConvertStub,
	table_finalize
};


/*
static JSBool
table_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("table_toString");
	// set rval to id
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLTable* table = (CWLTable*)prv->GetObj();
	if (!table)
		return JS_TRUE;

	const char* id = table->getTableID();
	JSString* str = JS_NewStringCopyZ(cx, id);
	if (str)
		*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}


static JSBool
table_valueOf(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("table_valueOf");
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLTable* table = (CWLTable*)prv->GetObj();
	if (!table)
		return JS_TRUE;

	*rval = INT_TO_JSVAL(table->getNumCols());

	return JS_TRUE;
}


static JSFunctionSpec table_methods[] = {
	{wl_toString_str,		table_toString,	0},
	{wl_valueOf_str,		table_valueOf,	0},
	{0}
};
*/


static JSBool
InitTableObject(JSContext *cx, JSObject *obj, jsint argc, jsval *argv)
{
WlTraceMsg("initializing Table object properties");

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_LENGTH))
		return JS_FALSE;

	return JS_TRUE;
}


JSObject* WLNewTableObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewTableObject(%x, %x)", cx, obj);
	// create an instance of type WLTable
	JSObject* proto = NULL;
	JSObject* table = WLNewObject(cx, &wl_TableClass, proto, obj);
	if (!table)
		return NULL;

	return table;
}


bool DllExport WLAddTableClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddTableClass(%x, %x)", cx, obj);
	if (!JS_InitClass(cx, obj, NULL, &wl_TableClass, NULL, 1, NULL, NULL, NULL, NULL))
		return false;

	return true;
}
