/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/*
 * File name : propertieshandler.cpp
 * Author    : Yossi Nakash
 * Created   : 3/7/01
 * ==========:
 * Abstract  : PropertiesHandler implementation
 *
 * Version control stuff:
 * $Revision:: 1        $
 * $Author::            $
 */

#include "propertieshandler.h"
#include <string.h>
#include <assert.h>


PropertiesHandler::PropertiesHandler()
{}


PropertiesHandler::~PropertiesHandler()
{}


PropertiesHandler::PropertiesHandler( const PropertiesHandler& src )
{}


PropertiesHandler& PropertiesHandler::operator=( const PropertiesHandler& src )
{ assert(0); return *this;}


PropertiesHandler::ClassNameId PropertiesHandler::GetClassId(const char* p_pcClassName)
{
  ClassNameId eRet = EN_CLASS_NONE ;
  if (p_pcClassName && strlen(p_pcClassName) > 2)
  {
    // all classes begin with wl, so we start compare with the third char.
    char cIdentifier = p_pcClassName[2] ;

    switch (cIdentifier)
    {
    case 'A' :
      cIdentifier = p_pcClassName[3] ;
      switch (cIdentifier)
      {
      case 'c' : if (!strcmp(p_pcClassName,"WLAction"))
                   eRet = EN_CLASS_ACTION; break;
      case 'p' : if (!strcmp(p_pcClassName,"WLApplet"))
                  eRet = EN_CLASS_APPLET; break;
      case 'r' : if (!strcmp(p_pcClassName,"WLArray"))
                   eRet = EN_CLASS_ARRAY; break;
      default:;
      }
      break;

    case 'B' : if (!strcmp(p_pcClassName,"WLBrowserLocation"))
                 eRet = EN_CLASS_BROWSER_LOCATION;break;

    case 'C' :
      cIdentifier = p_pcClassName[3] ;
      switch (cIdentifier)
      {
      case 'e' : if (!strcmp(p_pcClassName,"WLCell"))
                   eRet = EN_CLASS_CELL; break;
      case 's' : if (!strcmp(p_pcClassName,"WLCss"))
                   eRet = EN_CLASS_CSS; break;
      default:;
      }
      break;

    case 'D' : if (!strcmp(p_pcClassName,"WLDocument"))
                 eRet = EN_CLASS_DOCUMENT; break;

    case 'E' :
      cIdentifier = p_pcClassName[3] ;
      switch (cIdentifier)
      {
      case 'l' : if (!strcmp(p_pcClassName,"WLElement"))
                   eRet = EN_CLASS_ELEMENT ; break ;
      case 'm' : if (!strcmp(p_pcClassName,"WLEmbed"))
                   eRet = EN_CLASS_EMBED; break;
      default:;
      }
      break;

    case 'F' :
      if (!strcmp(p_pcClassName,"WLForm"))
        eRet = EN_CLASS_FORM ;
      else if (!strcmp(p_pcClassName,"WLFrame"))
        eRet = EN_CLASS_FRAME;
      else if (!strcmp(p_pcClassName,"WLFrames"))
        eRet = EN_CLASS_FRAMES ;
      break;

    case 'H' : if (!strcmp(p_pcClassName,"WLHeader"))
                 eRet = EN_CLASS_HEADER ; break;

    case 'I' : if (!strcmp(p_pcClassName,"WLImg"))
                 eRet = EN_CLASS_IMG; break;

    case 'K' : if (!strcmp(p_pcClassName,"WLKvp"))
                 eRet = EN_CLASS_KVP; break;

    case 'L' :
      cIdentifier = p_pcClassName[3] ;
      switch (cIdentifier)
      {
      case 'i' : if (!strcmp(p_pcClassName,"WLLink"))
                   eRet = EN_CLASS_LINK ; break ;
      case 'o' : if (!strcmp(p_pcClassName,"WLLocation"))
                   eRet = EN_CLASS_LOCATION; break;
      default:;
      }
      break;

    case 'M' : if (!strcmp(p_pcClassName,"WLMeta"))
                 eRet = EN_CLASS_META; break;

    case 'O' : if (!strcmp(p_pcClassName,"WLOption"))
                 eRet = EN_CLASS_OPTION; break;

    case 'R' :
      if (!strcmp(p_pcClassName,"WLReply"))
       eRet = EN_CLASS_REPLY;
      else if (!strcmp(p_pcClassName,"WLRow"))
       eRet = EN_CLASS_ROW ;
      else if (!strcmp(p_pcClassName,"WLRowCell"))
       eRet = EN_CLASS_ROW_CELL;
      break;

    case 'S' :
      cIdentifier = p_pcClassName[3] ;
      switch (cIdentifier)
      {
      case 'c' : if (!strcmp(p_pcClassName,"WLScript"))
                   eRet = EN_CLASS_SCRIPT; break;
      case 't' : if (!strcmp(p_pcClassName,"WLString"))
                   eRet = EN_CLASS_STRING; break;
      default:;
      }
      break;

    case 'T' :
      if (!strcmp(p_pcClassName,"WLTable"))
        eRet = EN_CLASS_TABLE;
      else if (!strcmp(p_pcClassName,"WLTableCell"))
        eRet = EN_CLASS_TABLE_CELL;
      break;

    case 'W' : if (!strcmp(p_pcClassName,"WLWindow"))
                 eRet = EN_CLASS_WINDOW; break;

    case 'X' : if (!strcmp(p_pcClassName,"WLXml"))
                 eRet = EN_CLASS_XML; break;
    default:;
    }
  }

  return eRet;
}


PropertiesHandler::PropertyNameId PropertiesHandler::GetPropertyId(const char* p_pcPropertyName)
{
  PropertyNameId eRet = EN_NONE ;
  if (p_pcPropertyName && strlen(p_pcPropertyName) > 1)
  {
    // all classes begin with wl, so we start compare with the third char.
    char cIdentifier = p_pcPropertyName[0] ;

    switch (cIdentifier)
    {
    case 'a' :
      cIdentifier = p_pcPropertyName[1];
      switch (cIdentifier)
      {
      case 'c' : if (!strcmp(p_pcPropertyName, "action"))
                   eRet = EN_ACTION; break;
      case 'l' : if (!strcmp(p_pcPropertyName, "alt"))
                   eRet = EN_ALT; break;
      case 'p' : if (!strcmp(p_pcPropertyName, "applets"))
                   eRet = EN_APPLETS; break;
      default:;
      }
      break;

    case 'C' : if (!strcmp(p_pcPropertyName, "CheckSum"))
                 eRet = EN_CHECKSUM; break;

    case 'c' :
      cIdentifier = p_pcPropertyName[1];
      switch (cIdentifier)
      {
      case 'e' :
        if (!strcmp(p_pcPropertyName, "cellIndex"))
          eRet = EN_CELL_INDEX;
        else if (!strcmp(p_pcPropertyName, "cells"))
          eRet = EN_CELLS ;
        break;

      case 'h' : if (!strcmp(p_pcPropertyName, "checked"))
                   eRet = EN_CHECKED; break;

      case 'o' :
        if (!strcmp(p_pcPropertyName, "cols"))
          eRet = EN_COLS;
        else if (!strcmp(p_pcPropertyName, "commands"))
          eRet = EN_COMMANDS;
        else if (!strcmp(p_pcPropertyName, "content"))
          eRet = EN_CONTENT;
        break;

      default:;
      }
      break;

    case 'd' :
      cIdentifier = p_pcPropertyName[1];
      switch (cIdentifier)
      {
      case 'e' :
        if (!strcmp(p_pcPropertyName, "defaultchecked"))
          eRet = EN_DEFAULT_CHECKED;
        else if (!strcmp(p_pcPropertyName, "defaultselected"))
          eRet = EN_DEFAULT_SELECTED;
        else if (!strcmp(p_pcPropertyName, "defaultvalue"))
          eRet = EN_DEFAULT_VALUE;
        break;

      case 'o' : if (!strcmp(p_pcPropertyName, "document"))
                   eRet = EN_DOCUMENT;break;
      default:;
      }
      break;

    case 'e' :
      cIdentifier = p_pcPropertyName[1];
      switch (cIdentifier)
      {
      case 'l' : if (!strcmp(p_pcPropertyName, "elements"))
                   eRet = EN_ELEMENTS; break;
      case 'm' : if (!strcmp(p_pcPropertyName, "embeds"))
                   eRet = EN_EMBEDS; break;
      case 'n' : if (!strcmp(p_pcPropertyName, "encoding"))
                   eRet = EN_ENCODING; break;
      case 'v' : if (!strcmp(p_pcPropertyName, "event"))
                   eRet = EN_EVENT; break;
      default:;
      }
      break;

    case 'F' : if (!strcmp(p_pcPropertyName, "FileSize"))
                   eRet = EN_FILE_SIZE; break;

    case 'f' :
      cIdentifier = p_pcPropertyName[1];
      switch (cIdentifier)
      {
      case 'o' : if (!strcmp(p_pcPropertyName, "forms"))
                   eRet = EN_FORMS; break;
      case 'r' : if (!strcmp(p_pcPropertyName, "frames"))
                   eRet = EN_FRAMES; break;
      default:;
      }
      break;

    case 'h' :
      cIdentifier = p_pcPropertyName[1];
      switch (cIdentifier)
      {
      case 'a' : if (!strcmp(p_pcPropertyName, "hash"))
                   eRet = EN_HASH; break;
      case 'e' : if (!strcmp(p_pcPropertyName, "headers"))
                   eRet = EN_HEADERS;
                 else if (!strcmp(p_pcPropertyName, "height"))
                   eRet = EN_HEIGHT ; break;
      case 'o' :
        if (!strcmp(p_pcPropertyName, "host"))
          eRet = EN_HOST ;
        else if (!strcmp(p_pcPropertyName, "hostname"))
          eRet = EN_HOSTNAME ;
        break;
      case 'r' : if (!strcmp(p_pcPropertyName, "href"))
                   eRet = EN_HREF; break;
      case 't' : if (!strcmp(p_pcPropertyName, "httpEquiv"))
                   eRet = EN_HTTPEQUIV; break;
      default:;
      }
      break;

    case 'i' :
      cIdentifier = p_pcPropertyName[1];
      switch (cIdentifier)
      {
      case 'd' : if (!strcmp(p_pcPropertyName, "id"))
                   eRet = EN_ID; break;
      case 'm' : if (!strcmp(p_pcPropertyName, "images"))
                   eRet = EN_IMAGES; break;
      case 'n' :
        if (!strcmp(p_pcPropertyName, "innerHTML"))
          eRet = EN_INNERHTML ;
        else if (!strcmp(p_pcPropertyName, "innerText"))
          eRet = EN_INNERTEXT ; break;
      default:;
      }
      break;

    case 'k' :
      cIdentifier = p_pcPropertyName[1];
      switch (cIdentifier)
      {
      case 'e' : if (!strcmp(p_pcPropertyName, "key"))
                   eRet = EN_KEY ; break;
      case 'v' : if (!strcmp(p_pcPropertyName, "kvp"))
                   eRet = EN_KVP; break;
      default:;
      }
      break;

    case 'l' :
      cIdentifier = p_pcPropertyName[1];
      switch (cIdentifier)
      {
      case 'a' : if (!strcmp(p_pcPropertyName, "language"))
                   eRet = EN_LANGUAGE; break;
      case 'e' : if (!strcmp(p_pcPropertyName, "length"))
                   eRet = EN_LENGTH; break;
      case 'i' : if (!strcmp(p_pcPropertyName, "links"))
                   eRet = EN_LINKS; break;
      case 'o' : if (!strcmp(p_pcPropertyName, "location"))
                   eRet = EN_LOCATION;
                 else if (!strcmp(p_pcPropertyName, "lowsrc"))
                   eRet = EN_LOWSRC; break;
      default:;
      }
      break;

    case 'm' : if (!strcmp(p_pcPropertyName, "method"))
                   eRet = EN_METHOD; break;

    case 'n' : if (!strcmp(p_pcPropertyName, "name"))
                   eRet = EN_NAME; break;

    case 'O' : if (!strcmp(p_pcPropertyName, "OpenTime"))
                   eRet = EN_OPEN_TIME; break;

    case 'o' :
      cIdentifier = p_pcPropertyName[1];
      switch (cIdentifier)
      {
      case 'p' : if (!strcmp(p_pcPropertyName, "options"))
                   eRet = EN_OPTIONS; break;
      case 'u' : if (!strcmp(p_pcPropertyName, "outerHTML"))
                   eRet = EN_OUTERHTML; break;
      default:;
      }
      break;

    case 'p' :
      cIdentifier = p_pcPropertyName[1];
      switch (cIdentifier)
      {
      case 'a' : if (!strcmp(p_pcPropertyName, "pathname"))
                   eRet = EN_PATHNAME; break;
      case 'o' : if (!strcmp(p_pcPropertyName, "port"))
                   eRet = EN_PORT; break;
      case 'r' : if (!strcmp(p_pcPropertyName, "protocol"))
                   eRet = EN_PROTOCOL; break;
      default:;
      }
      break;

    case 'R' : if (!strcmp(p_pcPropertyName, "ReceiveTime"))
                 eRet = EN_RECEIVE_TIME ; break;

    case 'r' :
      cIdentifier = p_pcPropertyName[1];
      switch (cIdentifier)
      {
      case 'e' :
        if (!strcmp(p_pcPropertyName, "referrer"))
          eRet = EN_REFERRER;
        else if (!strcmp(p_pcPropertyName, "reply"))
          eRet = EN_REPLY;
        break;
      case 'o' :
        if (!strcmp(p_pcPropertyName, "rowIndex"))
          eRet = EN_ROWINDEX;
        else if (!strcmp(p_pcPropertyName, "rows"))
          eRet = EN_ROWS;
        break;
      default:;
      }
      break;

    case 'S' : if (!strcmp(p_pcPropertyName, "SPIDERSESSION"))
                 eRet = EN_SPIDERSESSION ;
               else if (!strcmp(p_pcPropertyName, "SendTime"))
                 eRet = EN_SEND_TIME; break;

    case 's' :
      cIdentifier = p_pcPropertyName[1];
      switch (cIdentifier)
      {
      case 'c' : if (!strcmp(p_pcPropertyName, "scripts"))
                   eRet = EN_SCRIPTS; break;
      case 'e' :
        if (!strcmp(p_pcPropertyName, "search"))
          eRet = EN_SEARCH;
        else if (!strcmp(p_pcPropertyName, "selected"))
          eRet = EN_SELECTED;
        else if (!strcmp(p_pcPropertyName, "selectedindex"))
          eRet = EN_SELECTED_INDEX;
        break;
      case 'o' : if (!strcmp(p_pcPropertyName, "source"))
                   eRet = EN_SOURCE; break;
      case 'r' : if (!strcmp(p_pcPropertyName, "src"))
                   eRet = EN_SRC; break;
      case 't' :
        if (!strcmp(p_pcPropertyName, "statusLine"))
          eRet = EN_STATUS_LINE;
        else if (!strcmp(p_pcPropertyName, "statusNumber"))
          eRet = EN_STATUS_NUMBER;
        else if (!strcmp(p_pcPropertyName, "stylesheets"))
          eRet = EN_STYLESHEETS;
        else if (!strcmp(p_pcPropertyName, "styleSheets"))
          eRet = EN_STYLESHEETS;
        break;
      default:;
      }
      break;

    case 't' :
      cIdentifier = p_pcPropertyName[1];
      switch (cIdentifier)
      {
      case 'a' :
        if (!strcmp(p_pcPropertyName, "tagName"))
          eRet = EN_TAGNAME;
        else if (!strcmp(p_pcPropertyName, "target"))
          eRet = EN_TARGET;
        break;
      case 'e' : if (!strcmp(p_pcPropertyName, "text"))
                   eRet = EN_TEXT; break;
      case 'i' : if (!strcmp(p_pcPropertyName, "title"))
                   eRet = EN_TITLE; break;
      case 'y' : if (!strcmp(p_pcPropertyName, "type"))
                   eRet = EN_TYPE; break;
      default:;
      }
      break;

    case 'u' :
      cIdentifier = p_pcPropertyName[1];
      switch (cIdentifier)
      {
      case 'r' : if (!strcmp(p_pcPropertyName, "uri"))
                   eRet = EN_URI; break;

      case 's' : if (!strcmp(p_pcPropertyName, "useMap"))
                   eRet = EN_USEMAP; break;

      default:;
      }
      break;

    case 'U' : if (!strcmp(p_pcPropertyName, "URL"))
                   eRet = EN_URL; break;

    case 'v' :
      cIdentifier = p_pcPropertyName[1];
      switch (cIdentifier)
      {
      case 'a' :
        if (!strcmp(p_pcPropertyName, "value"))
          eRet = EN_VALUE; break;
      case 'e' : if (!strcmp(p_pcPropertyName, "version"))
                   eRet = EN_VERSION; break;
      default:;
      }
      break;

    case 'w' :
    case 'W' ://all the cases are variables that begin with 'wl'
      cIdentifier = p_pcPropertyName[2];
      switch (cIdentifier)
      {
      case 'd' : if (!strcmp(p_pcPropertyName, "width"))
                     eRet = EN_WIDTH; break;

      case 'H' :
        cIdentifier = p_pcPropertyName[3];
        switch (cIdentifier)
        {
        case 'a' : if (!strcmp(p_pcPropertyName, "wlHash"))
                     eRet = EN_WLHASH; break;
        case 'e' : if (!strcmp(p_pcPropertyName, "wlHeaders"))
                     eRet = EN_WLHEADERS; break;
        case 'i' :
          if (!strcmp(p_pcPropertyName, "WLHiddenPropertyClassName"))
            eRet = EN_WLHIDDEN_PROPERTY_CLASS_NAME;
          else if (!strcmp(p_pcPropertyName, "WLHiddenPropertyDomScope"))
            eRet = EN_WLHIDDEN_PROPERTY_DOM_SCOPE;
          break;
        case 'o' :
          if (!strcmp(p_pcPropertyName, "wlHost"))
            eRet = EN_WLHOST;
          else if (!strcmp(p_pcPropertyName, "wlHostName"))
            eRet = EN_WLHOSTNAME;
          break;
        case 'r' : if (!strcmp(p_pcPropertyName, "wlHref"))
                     eRet = EN_WLHREF; break;
        default:;
        }
        break;

      case 'i' : if (!strcmp(p_pcPropertyName, "WaitTime"))
                     eRet = EN_WAIT_TIME ; break;

      case 'I' : if (!strcmp(p_pcPropertyName, "wlInnerImg"))
                  eRet = EN_INNERIMG ; break;

      case 'M' : if (!strcmp(p_pcPropertyName, "wlMetas"))
                     eRet = EN_WLMETAS; break;

      case 'O' : if (!strcmp(p_pcPropertyName, "wlOrigUrl"))
                     eRet = EN_WLORIGURL; break;
      case 'P' :
        cIdentifier = p_pcPropertyName[3];
        switch (cIdentifier)
        {
        case 'a' : if (!strcmp(p_pcPropertyName, "wlPathName"))
                     eRet = EN_WLPATH_NAME; break;
        case 'o' : if (!strcmp(p_pcPropertyName, "wlPost"))
                     eRet = EN_WLPOST; break;
        case 'r' : if (!strcmp(p_pcPropertyName, "wlProtocol"))
                     eRet = EN_WLPROTOCOL; break;
        default:;
        }
        break;

      case 'S' :
        cIdentifier = p_pcPropertyName[3];
        switch (cIdentifier)
        {
        case 'e' :
          if (!strcmp(p_pcPropertyName, "wlSearch"))
            eRet = EN_WLSEARCH;
          else if (!strcmp(p_pcPropertyName, "wlSearchPairs"))
            eRet = EN_WLSEARCHPAIRS;
          break;
        case 'o' :
          if (!strcmp(p_pcPropertyName, "wlSource"))
            eRet = EN_WLSOURCE;
          else if (!strcmp(p_pcPropertyName, "wlSourceIndex"))
            eRet = EN_WLSOURCE_INDEX;
          break;
        case 't' :
          if (!strcmp(p_pcPropertyName, "wlStatusLine"))
            eRet = EN_WLSTATUS_LINE;
          else if (!strcmp(p_pcPropertyName, "wlStatusNumber"))
            eRet = EN_WLSTATUS_NUMBER;
          break;
        default:;
        }
        break;

      case 'T' : if (!strcmp(p_pcPropertyName, "wlTables"))
                     eRet = EN_WLTABLES; break;

      case 'V' : if (!strcmp(p_pcPropertyName, "wlVersion"))
                     eRet = EN_WLVERSION; break;

      case 'X' : if (!strcmp(p_pcPropertyName, "wlXmls"))
                     eRet = EN_WLXMLS; break;
      default:;
      }
    default:;
    }
  }

  return eRet;
}


