/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <time.h>
#include <slistcol.h>
#include <thr.h>
#include <thr.h>
#include <hashdict.h>
#include <collstr.h>
#include <slistcolque.h>
#include "platform.h"
#include "tvpsync.h"

#include "httpdefines.h"
#include "wlhttpcache.h"

#define VALIDATOR_STRONG  1
#define VALIDATOR_WEAK    2

extern time_t convertStrToTime( RVCString& s_time );


CWLHttpCacheEntry::CWLHttpCacheEntry( const char* p_psUrl, const char* p_psReferer, HtmlQueueItemType p_eType )
//: m_Lock( m_Mutex )
{
	m_uMethod = HTTP_CACHE_TYPE_FRESH;

	if ( p_psUrl )
		m_sResourceUrl = p_psUrl;

	if (p_psReferer)
		m_sReferer = p_psReferer;
	
	m_eResourceType = p_eType;

	m_uExpirationTime = -1;
	m_uArrivalTime = -1;
	m_uModifiedTime = -1;

	m_eStatus = STATUS_NEW;
        m_sResourceType = "";
  m_ByteCount = 0;
  m_eZipCode = ZipCode_NO;
}

CWLHttpCacheEntry::~CWLHttpCacheEntry()
{
}

bool CWLHttpCacheEntry::isHeuristicFresh( time_t p_uCurrTime )
{
	// probably use current time, arrival time, creation (modified) time, type, size to determine
	// whether this object is fresh
	return false;
}

// C@@haim - currently, expiration is never set so IsFresh always returns false
bool CWLHttpCacheEntry::IsFresh( unsigned p_uMethod )
{
	// if expires exist and is later (bigger) than now, return true
	// else check if heuristically fresh
	// else return false

	m_uMethod = p_uMethod;

	// current time
	time_t uCurrTime;
	time ( &uCurrTime );

	// compare
	if ( uCurrTime <= m_uExpirationTime )
		return true;

	// check heuristics
	if ( m_uMethod == HTTP_CACHE_TYPE_HEURISTICS )
		return isHeuristicFresh( uCurrTime );

	return false;
}

bool CWLHttpCacheEntry::IsAlreadyExpired()
{
	// if 'Expires' exist and is invalid on equal 'Date', it's considered as already expired
	return true;
}

bool CWLHttpCacheEntry::GenerateConditionalHeaders( unsigned p_uHttpVersion, RVCString& p_sConditionalHeaders )
{
	//if ( m_eStatus == STATUS_APPEND )
	//	m_Lock.wait();

	RVCString sConditionalHeaders;
	//generateIfMatchHeader( sConditionalHeaders );
	generateIfNoneMatchHeader( sConditionalHeaders );
	/*
	if ( m_sValidator.first( "W/" ) != 0 &&
		   m_sValidator.first( "w/" ) != 0 )
	*/
		generateIfModifiedSinceHeader( sConditionalHeaders );
	p_sConditionalHeaders.append( sConditionalHeaders );

	//m_Lock.signalAll();
	return true;
}

bool CWLHttpCacheEntry::generateIfModifiedSinceHeader( RVCString& p_sIfModifiedSinceHeader )
{
	if ( !m_sModifiedTime.isNull() )
	{
		p_sIfModifiedSinceHeader.append( CS_HEADER_IF_MODIFIED_SINCE );
		p_sIfModifiedSinceHeader.append( HEADER_SEPARATOR);
		p_sIfModifiedSinceHeader.append( m_sModifiedTime );
		p_sIfModifiedSinceHeader.append( CRLF );
		return true;
	}

	return false;
}

bool CWLHttpCacheEntry::generateIfMatchHeader( RVCString& p_sIfMatchHeader )
{
	if ( !m_sValidator.isNull() )
	{
		p_sIfMatchHeader.append( CS_HEADER_IF_MATCH );
		p_sIfMatchHeader.append( HEADER_SEPARATOR);
		p_sIfMatchHeader.append( m_sValidator );
		p_sIfMatchHeader.append( CRLF );
		return true;
	}

	return false;
}

bool CWLHttpCacheEntry::generateIfNoneMatchHeader( RVCString& p_sIfNoneMatchHeader )
{
	if ( !m_sValidator.isNull() )
	{
		p_sIfNoneMatchHeader.append( CS_HEADER_IF_NONE_MATCH );
		p_sIfNoneMatchHeader.append( HEADER_SEPARATOR);
		p_sIfNoneMatchHeader.append( m_sValidator );
		p_sIfNoneMatchHeader.append( CRLF );
		return true;
	}

	return false;
}

void CWLHttpCacheEntry::SetContentType( const char* p_psContentType )
{
	if ( p_psContentType )
		m_sResourceType = p_psContentType;
}

RVCString& CWLHttpCacheEntry::GetContentType(){
  return m_sResourceType;
}

void CWLHttpCacheEntry::SetExpires( const char* p_psExpires )
{
	if ( p_psExpires )
		m_sExpirationTime = p_psExpires;
	// expiration time
	if ( m_uExpirationTime == -1 && !m_sExpirationTime.isNull() )
		m_uExpirationTime = convertStrToTime( m_sExpirationTime );
}

void CWLHttpCacheEntry::SetLastModified( const char* p_psLastModified )
{
	if ( p_psLastModified )
		m_sModifiedTime = p_psLastModified;
	m_uModifiedTime = -1;

	/*
	if ( p_psLastModified )
	{
		if ( sValues.isNull() )
		{
			sValues.append( m_sResourceUrl );
			sValues.append( "\n" );
		}
		sValues.append( "Last-Modified: " );
		sValues.append( p_psLastModified );
		sValues.append( "\n" );
	}
	*/
}

void CWLHttpCacheEntry::SetETag( const char* p_psETag )
{
	if ( p_psETag )
		m_sValidator = p_psETag;

	/*
	if ( p_psETag )
	{
		if ( sValues.isNull() )
		{
			sValues.append( m_sResourceUrl );
			sValues.append( "\n" );
		}
		sValues.append( "ETag: " );
		sValues.append( p_psETag );
		sValues.append( "\n" );
	}
	*/
}

void CWLHttpCacheEntry::SetDate( const char* p_psDate )
{
	if ( p_psDate )
		m_sArrivalTime = p_psDate;
	m_uArrivalTime = -1;
}

void CWLHttpCacheEntry::Reset()
{
/*
	m_sResourceType.remove( 0 );

	m_sValidator.remove( 0 );
	
	m_sExpirationTime.remove( 0 );
	m_uExpirationTime = -1;
	
	m_sArrivalTime.remove( 0 );
	m_uArrivalTime = -1;
	
	m_sModifiedTime.remove( 0 );
	m_uModifiedTime = -1;
*/
	//sValues.remove( 0 );
  //m_ByteCount = 0;
}

void CWLHttpCacheEntry::Touch( unsigned p_uRoundNum )
{
	m_uLastAccess = p_uRoundNum;
}

unsigned CWLHttpCacheEntry::GetLastAccess()
{
	return m_uLastAccess;
}

bool CWLHttpCacheEntry::IsMatchingTypes( HtmlQueueItemType p_eTag, unsigned p_uTypes )
{
	// compare type of resource to it's tag
	switch ( p_eTag )
	{
		case FRAME:
			if ( p_uTypes & HTTP_CACHE_CACHE_HTML )
				return true;
			break;

		case SCRIPT:
			if ( p_uTypes & HTTP_CACHE_CACHE_JS )
				return true;
			break;

		case STYLE:
		case LINK:
			if ( p_uTypes & HTTP_CACHE_CACHE_CSS )
				return true;
			break;

		case XML:
		case DTD:
			if ( p_uTypes & HTTP_CACHE_CACHE_XML )
				return true;
			break;

		case IMAGE:
		case EMBED:
			if ( p_uTypes & HTTP_CACHE_CACHE_IMAGE )
				return true;
			break;

		case APPLET:
			if ( p_uTypes & HTTP_CACHE_CACHE_APPLET )
				return true;
			break;

		case GENERAL:
		default:
			break;
	}

	return false;
}

bool CWLHttpCacheEntry::IsOfType( unsigned p_uTypes )
{
	// check if resource url contains a query string
	if ( p_uTypes & HTTP_CACHE_CACHE_DYNAMIC && m_sResourceUrl.first( '?' ) != RV_NPOS )
		return true;

	// check resource tag type
	return IsMatchingTypes( m_eResourceType, p_uTypes );
}

bool CWLHttpCacheEntry::IsReady()
{
	return m_eStatus == STATUS_READY;
}
void CWLHttpCacheEntry::UpdateContent( const char* p_psContentType,
																			 const char* p_psExpires,
																			 const char* p_psDate,
																			 const char* p_psETag,
																			 const char* p_psLastModified,
																			 const char* p_psContent, unsigned p_uLength )
{
	//RVMutexLock::LockGuard guard( m_Mutex );

	SetContentType( p_psContentType );
	SetExpires( p_psExpires );
	SetDate( p_psDate );
	SetETag( p_psETag );
	SetLastModified( p_psLastModified );
	AppendContent( p_psContent, p_uLength );
}

void CWLHttpCacheEntry::ClearContent()
{
	//m_Lock.wait();

	m_sContent.remove( 0 );
}

void CWLHttpCacheEntry::AppendContent( const char* p_psContent, unsigned p_uLength )
{
	// appending is relevant only if validators exists or object is fresh
	m_eStatus = STATUS_APPEND;

  if ( p_psContent && ( ( !m_sArrivalTime.isNull() && !m_sValidator.isNull() ) || IsFresh( m_uMethod ) ) ){
		m_sContent.append( p_psContent, p_uLength );
    m_ByteCount += p_uLength;
  }
}

void CWLHttpCacheEntry::ContentBegin()
{
	// content is starting to load into cache
	m_eStatus = STATUS_APPEND;
	//m_Lock.wait();
}

void CWLHttpCacheEntry::ContentDone()
{
	// content is fully loaded into cache signal all waiting threads
	m_eStatus = STATUS_READY;
	//m_Lock.signalAll();
}

const char* CWLHttpCacheEntry::GetContentData()
{
	//m_Lock.wait();

	return m_sContent.data();
}

size_t CWLHttpCacheEntry::GetContentLength()
{
	//m_Lock.wait();

	return m_sContent.length();
}

RVCString& CWLHttpCacheEntry::GetContent()
{
	return m_sContent;
}

void CWLHttpCacheEntry::SetContent( RVCString p_sContent )
{
	m_sContent = p_sContent;
  m_ByteCount = m_sContent.length();
}


ZipTypeCode CWLHttpCacheEntry::GetEZipCode()
{
  return m_eZipCode;
}

void CWLHttpCacheEntry::SetEZipCode( ZipTypeCode p_eZipCode )
{
  m_eZipCode = p_eZipCode;
}

// ============================================================================================


CWLHttpCache::CWLHttpCache( unsigned p_uTimeToLive )
{
	//m_uTotal = 0;
	m_uTimeToLive = p_uTimeToLive;
}

CWLHttpCache::~CWLHttpCache()
{
	m_CacheDictionary.clearAndDestroy();
}

void CWLHttpCache::DeleteOldEntries( unsigned p_uCurrRound )
{
	if ( m_uTimeToLive == 0 ) // live forever
		return;

	unsigned uDieTime = p_uCurrRound - m_uTimeToLive;
	RVHashDictionaryIterator iter( m_CacheDictionary );
	while ( iter() )
	{
		CWLHttpCacheEntry* pEntry = (CWLHttpCacheEntry*)iter.value();
		if ( pEntry && pEntry->GetLastAccess() <= uDieTime )
		{
			delete iter.key();
			delete pEntry;
			iter.remove();
			//m_CacheDictionary.removeAndDestroy( pEntry );
		}
	}
}

void CWLHttpCache::ClearAndDestroy()
{
	m_CacheDictionary.clearAndDestroy();
}

CWLHttpCacheEntry*  CWLHttpCache::CreateNewEntry( unsigned p_uRoundNum, RVCollectableString* p_psKey, RVCollectableString* p_psReferer, HtmlQueueItemType p_eType, unsigned& p_uEntryStatus )
{
	// this lock is used in order to prevent the creation of more than 1 instance from the given key
	RVMutexLock::LockGuard guard( m_Mutex );

	p_uEntryStatus = CACHE_ENTRY_MISS;

	// entry might have been created by the locking thread
	CWLHttpCacheEntry* pCacheEntry = (CWLHttpCacheEntry*)m_CacheDictionary.findValue( p_psKey );
	if ( pCacheEntry )
	{
		pCacheEntry->Touch( p_uRoundNum );
		if ( !pCacheEntry->IsReady() )
			return NULL; // this will cause a full (unconditional) request for this resource

		p_uEntryStatus = CACHE_ENTRY_FOUND;
		return pCacheEntry;
	}

	pCacheEntry = new CWLHttpCacheEntry( p_psKey->data(), p_psReferer->data(), p_eType );
	m_CacheDictionary.insertKeyAndValue( new RVCollectableString( p_psKey->data() ), pCacheEntry );
	pCacheEntry->Touch( p_uRoundNum );

	p_uEntryStatus = CACHE_ENTRY_NEW;
	return pCacheEntry;
}

CWLHttpCacheEntry*  CWLHttpCache::UpdateCacheEntry(CWLHttpCacheEntry* pCacheEntry){
	return (CWLHttpCacheEntry*)m_CacheDictionary.insertKeyAndValue(new RVCollectableString( pCacheEntry->m_sResourceUrl.data()), pCacheEntry );
}

void CWLHttpCache::SetCachedTypes( unsigned p_uCachedTypes )
{
	m_uCachedTypes = p_uCachedTypes;
}

bool CWLHttpCache::IsCachedType( HtmlQueueItemType p_eTag, const char* p_psUrl )
{
	// if not caching url with a query string and this url contains one - it shouldn't be cached
	if ( !(m_uCachedTypes & HTTP_CACHE_CACHE_DYNAMIC) && strchr( p_psUrl, '?' ) )
		return false;

	return CWLHttpCacheEntry::IsMatchingTypes( p_eTag, m_uCachedTypes );
}

// delete all entries that should not be kept in the cache
void CWLHttpCache::RemoveTypes( unsigned p_uTypes )
{
	RVHashDictionaryIterator iter( m_CacheDictionary );
	while ( iter() )
	{
		CWLHttpCacheEntry* pEntry = (CWLHttpCacheEntry*)iter.value();
		if ( pEntry && pEntry->IsOfType( p_uTypes ) )
		{
			delete iter.key();
			delete pEntry;
			iter.remove();
			//m_CacheDictionary.removeAndDestroy( pEntry );
		}
	}
}

/*
void CWLHttpCache::ContentAdded( unsigned p_uLength )
{
	m_uTotal += p_uLength;
}

size_t CWLHttpCache::GetTotal()
{
	return m_uTotal;
}

size_t CWLHttpCache::ContentSize()
{
	size_t uSum = 0;
	RVHashDictionaryIterator iter( m_CacheDictionary );
	while ( iter() )
	{
		CWLHttpCacheEntry* pEntry = (CWLHttpCacheEntry*)iter.value();
		if ( pEntry )
			uSum += pEntry->GetContentLength();
	}

	return uSum;
}
*/
