/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlqueue.h"
#include <stdio.h>
#include <iostream.h>
#include "tracedll.h"
#include "wlurl.h"
#include "wltransactiondata.h"
#include "commandmanager.h"
#include "httpembedderreport.h"
#include "workspace.h"
#include "httpdefines.h"

CWLQueue::CWLQueue(unsigned int numThreads, CWLCommandManager* p_pCm)
					       :m_MaxNumberThreads(numThreads), 
								  m_SlaveSignal(m_NotEmptyMutex),
									m_MasterSignal(m_NotEmptyMutex),
                  m_pCm(p_pCm)
{
	m_NumThreadsWaiting = 0;	
	m_EndOfSession  = false;
	m_MasterWaiting = true;
	m_Terminated = false;
  m_OnlyMasterRead = false;
  m_UseBackup = false;
  WlTraceMsg("Initing queue");
}
	
CWLQueue::~CWLQueue()
{
	clearAndDestroy();
}

void CWLQueue::setNumMaxThreads(unsigned int maxThreads )
{
	m_MaxNumberThreads = maxThreads;
}

HtmlQueueItemType CWLQueue::masterRead(RVCollectable** result)
{
  *result = NULL ;

  RVMutexLock::LockGuard guard(m_NotEmptyMutex); 

  while (m_TransactionsQueue.isEmpty() && !m_Terminated)
  {
		if(m_NumThreadsWaiting >= m_MaxNumberThreads - 1)
		{
			WlTraceMsg("in master read, found that all other threads are waiting - exiting");
			m_MasterWaiting	= true;
			return NONE;
		}
		WlTraceMsg("master read - Queues are empty");
		m_NumThreadsWaiting++;
		m_MasterWaiting = true;
		WlTraceMsg("m_MasterWaiting true and m_NumThreadsWaiting: %d ",m_NumThreadsWaiting);
		//WlTraceMsg("m_MasterWaiting = true and m_NumThreadsWaiting= %d, m_NumThreadsWaiting);
		m_MasterSignal.wait();
		WlTraceMsg("master out of wait");
		m_MasterWaiting = false;
		m_NumThreadsWaiting--;
	}

	if (m_Terminated) {
		m_MasterWaiting = false;
		return TERMINATE;
	}
	if(*result = m_TransactionsQueue.get()) {
		m_MasterWaiting = false;
		return ((CWLTransactionsObject*)*result)->GetObjType();
	}

  m_MasterWaiting	= true;
	return NONE;
}		


HtmlQueueItemType CWLQueue::slaveRead(RVCollectable** result)
{
  *result = NULL ;

  RVMutexLock::LockGuard guard(m_NotEmptyMutex);                

  while ( (m_MasterWaiting || m_TransactionsQueue.isEmpty() || isOnlyMasterRead() ) && !m_Terminated)
	{ 
		m_NumThreadsWaiting++;
		WlTraceMsg("in slave read - m_NumThreadsWaiting:%d ",m_NumThreadsWaiting);
		if(m_NumThreadsWaiting >= m_MaxNumberThreads)
		{
			WlTraceMsg("m_NumThreadsWaiting reached MAX - signalling");
			m_MasterSignal.signal();
		}	 	
		WlTraceMsg("thread waiting for signal");			                           											 								             
		m_SlaveSignal.wait();     
		if(m_EndOfSession)
			return RELEASE;	                            
		m_NumThreadsWaiting--;
  }
	WlTraceMsg("after signal to slave, num waiting: %d ",m_NumThreadsWaiting);
	if (m_Terminated)
		return TERMINATE;

  if(*result = m_TransactionsQueue.get())
  {
	  WlTraceMsg("slave recieved work to do. num waiting: %d ",m_NumThreadsWaiting);
    return ((CWLTransactionsObject*)*result)->GetObjType();
  }

	return NONE;
}
				
bool CWLQueue::insert(CWLTransactionsObject* pTransObj)
{
	if(!pTransObj)
		return false;

  RVMutexLock::LockGuard guard(m_NotEmptyMutex);    
	RVCollectable* itemInserted  = NULL;
  
  HtmlQueueItemType   objType = pTransObj->GetObjType();

  if ( objType != FRAME && objType != XML && m_pCm->GetWorkspace()->m_HttpCacheType == HTTP_CACHE_TYPE_NONE )
	{
		// Checking if item already in cache
		// if it is not then we add it else we ignore it
		const RVCollectableString* sKey = new RVCollectableString(pTransObj->GetObjUrl());
		if (!m_CacheTable.findValue(sKey)) 
    {
      CWLTransactionsObject* pCachedObj = new CWLTransactionsObject(*pTransObj);
			m_CacheTable.insertKeyAndValue((RVCollectable*)sKey, pCachedObj);
			WlTraceMsg("image inserted into Q");
		}
		else //cached items
    {
      pTransObj->SetCached(true);

			delete sKey;
			WlTraceMsg("cached image inserted into Q");

      if (!m_pCm->IsCallbackEnabled())//in probing client we enter it to Q
        return true;
		}
	}
	//insert to Q anyway
  itemInserted = m_TransactionsQueue.append(pTransObj);					

  // If we haven't failed to insert to queue then we finish with failure
	if(!itemInserted)
		return false;	

  // Chaim - Save in backup
//  if (m_UseBackup && pTransObj->GetObjType() == IMAGE){
  if (m_UseBackup){
    WlTraceMsg("Saving transaction: %s in backup queue for: %s", pTransObj->GetObjUrl(), m_BackupKey.data());
    RVSlistCollectablesQueue*  local_TransactionsQueue;

    // Use new copy operator
    CWLTransactionsObject* pCachedTransObj = new CWLTransactionsObject(*pTransObj, false);

    if ((local_TransactionsQueue = GetSavedTransactions (&m_BackupKey)) == NULL){

      // Create backup transaction queue
      WlTraceMsg("creating backup queue");
      local_TransactionsQueue = new RVSlistCollectablesQueue;
      itemInserted = local_TransactionsQueue->append (pCachedTransObj);
      RVCollectableString* temp_key = new RVCollectableString(m_BackupKey);
      m_SavedTransactions.insertKeyAndValue (temp_key, local_TransactionsQueue);
    }
    else {
      WlTraceMsg("Using existing queue");
      itemInserted = local_TransactionsQueue->append (pCachedTransObj);
      m_SavedTransactions.findValue (&m_BackupKey, local_TransactionsQueue);
    }
  }



	if(m_MasterWaiting)
	{
		WlTraceMsg("signaling to master");
		m_MasterSignal.signal();
	}
	else
	{
		WlTraceMsg("signaling to slave");
		m_SlaveSignal.signal();                                 
	}
	return true;
}


// Chaim
bool CWLQueue::UseBackup (RVCollectableString* key)
{
  RVMutexLock::LockGuard guard(m_ParseOnceMutex);    

  RVSlistCollectablesQueue*      local_TransactionsQueue;
  CWLTransactionsObject*        TransObj;


  if ((local_TransactionsQueue = GetSavedTransactions (key)) == NULL){
    return false;
  }

  RVSlistCollectablesIterator    iterator(*local_TransactionsQueue);

  // Turn off backup
  m_UseBackup = false;
  
  while ((TransObj =  (CWLTransactionsObject*) iterator()) != NULL){

    WlTraceMsg("Restoring transaction: %s for original url: %s", key->data(), TransObj->GetObjUrl());

    // Reset transaction object: clear subtransaction and event lists.
    TransObj->Reset();
    if (!insert(TransObj)){
      WlTraceMsg("Failed to insert object");
      return false;
    }
  }
  return true;
}

// Chaim 
void CWLQueue::SetBackup (bool sw, RVCollectableString* key)
{
  RVMutexLock::LockGuard guard(m_ParseOnceMutex);  
  
  m_UseBackup = sw;
  m_BackupKey = *key;
}

RVSlistCollectablesQueue* CWLQueue::GetSavedTransactions (RVCollectableString* key)
{
  return (RVSlistCollectablesQueue*) m_SavedTransactions.findValue (key);
}

bool CWLQueue::BackupExists (RVCollectableString* key)
{
  RVMutexLock::LockGuard guard(m_ParseOnceMutex);  

  bool ret;
  if (!m_SavedTransactions.entries()){
    return false;
  }

  ret = (GetSavedTransactions (key) != NULL);
  return ret;
}

void CWLQueue::ClearCache()
{
		m_CacheTable.clearAndDestroy ();
}
			
void CWLQueue::releaseAllThreads()
{
  RVMutexLock::LockGuard guard(m_NotEmptyMutex);    
	m_EndOfSession = true;
	m_SlaveSignal.signalAll();
}	

void CWLQueue::terminateAllThreads()
{
  RVMutexLock::LockGuard guard(m_NotEmptyMutex);    
	m_Terminated = true;
	m_SlaveSignal.signalAll();
	m_MasterSignal.signalAll();
}

void CWLQueue::clearAndDestroy()
{
	m_TransactionsQueue.clear();
	ClearCache();
}

CWLTransactionsObject* CWLQueue::GetCachedTransaction(RVCollectableString sKey)
{
	return (CWLTransactionsObject*)m_CacheTable.findValue(&sKey) ;
}
