/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <ace/OS.h>
#include "wlxmlcom.h"
#include "wlxmlfuncs.h"
#include <string.h>
#include "rvjsapi.h"

#include "xmlreplyprops.h"
#include "jsxmlreplynames.h"
#include "jsxmlprivate.h"
#include "jsXmlWrappers.h"
#include "xmlerrorreporter.h"
#include "tracedll.h"

JSBool
xmlelement_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp, IXMLDOMNodePtr& pNodePtr)
{
	TRY_COM {
		WlTraceMsg("Inside xmlelement_getProperty");
		IXMLDOMElementPtr pXMLElementInterfacePtr = pNodePtr;
		if ((pXMLElementInterfacePtr != NULL) && JSVAL_IS_STRING(id)) {
			JSString* str = JSVAL_TO_STRING(id);
			char* prop = JS_GetStringBytes(str);
			if (!strcmp(prop, WLJSNAME_PROP_TAGNAME)) {	
				_bstr_t tagName = pXMLElementInterfacePtr->GettagName ( );
				return WLXmlSetStringValue(cx, obj, prop,  (char*)tagName, vp);
			}
		}
		return JS_TRUE;
	}
	CATCH_COM(cx)
}

JSBool 
xmlelement_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp) 
{
	WlTraceMsg("Inside xmlelement_setProperty");
	return JS_TRUE;
}

JSBool
xmlelement_getAttribute(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval, IXMLDOMNodePtr& pNodePtr)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		WlTraceMsg("Inside xmlelement_getAttribute");
		if(argc != 1)
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "getAttribute", 1);
		if(!JSVAL_IS_STRING(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "getAttribute");
		_bstr_t name(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
		IXMLDOMElementPtr elementPtr  = pNodePtr;
		_bstr_t att = elementPtr->getAttribute(name );
		return WLXmlSetStringValue(cx, obj, NULL, (char*)att, rval);
	}
	CATCH_COM_METHOD(cx, "getAttribute")
}

JSBool
xmlelement_setAttribute(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval, IXMLDOMNodePtr& pNodePtr)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		WlTraceMsg("Inside xmlelement_setAttribute");
		if(argc != 2)
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "setAttribute", 2);
		if(!JSVAL_IS_STRING(argv[0]) || !JSVAL_IS_STRING(argv[1]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "setAttribute");
		_bstr_t name(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
		_variant_t value(JS_GetStringBytes(JSVAL_TO_STRING(argv[1])));
		IXMLDOMElementPtr elementPtr  = pNodePtr;
		HRESULT hr = elementPtr->setAttribute(name, value);		
		//doesnt return a value 
		return succeeded(cx, hr);
	}
	CATCH_COM_METHOD(cx,"setAttribute")
}

JSBool
xmlelement_removeAttribute(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval, IXMLDOMNodePtr& pNodePtr)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		WlTraceMsg("Inside xmlelement_removeAttribute");
		if(argc != 1)
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "removeAttribute", 1);
		if(!JSVAL_IS_STRING(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "removeAttribute");
		_bstr_t name(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
		IXMLDOMElementPtr elementPtr  = pNodePtr;
		HRESULT hr = elementPtr->removeAttribute(name );
		//doesnt return a value
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "removeAttribute")
}

JSBool
xmlelement_getAttributeNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval, IXMLDOMNodePtr& pNodePtr)
{	
  WlAssert(piHLErrorReporter);
	TRY_COM {
		WlTraceMsg("Inside xmlelement_getAttributeNode");
		if(argc != 1)
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "getAttributeNode", 1);
		if(!JSVAL_IS_STRING(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "getAttributeNode");
		_bstr_t name(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
		IXMLDOMElementPtr elementPtr  = pNodePtr;
		IXMLDOMNodePtr* pReturnNodePtr = new IXMLDOMNodePtr;
		*pReturnNodePtr = elementPtr->getAttributeNode(name);
		CWLXmlNodeWrapper* wrapper = new CWLXmlNodeWrapper(pReturnNodePtr);
		return WLXmlSetNodeInterfaceValue(cx, obj, wrapper, rval);
	}
	CATCH_COM_METHOD(cx, "getAttributeNode")
}


//this method sets a new attribute node as a child of the element - if exists an att node
//with the same name it replaces it. Returns NULL unless replacment took place.
JSBool
xmlelement_setAttributeNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval, IXMLDOMNodePtr& pNodePtr)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		WlTraceMsg("Inside xmlelement_setAttributeNode");
		if(argc != 1)
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "setAttributeNode", 1);
		if(!JSVAL_IS_OBJECT(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "setAttributeNode");
		//get the att node
		JSObject* attObj	= JSVAL_TO_OBJECT(argv[0]);
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, attObj);
		IXMLDOMAttributePtr attPtr =  *(((CWLXmlNodeWrapper*)(prv->GetObj()))->GetInterfacePtr());
		//set the node and return a NODE interface
		IXMLDOMElementPtr elementPtr  = pNodePtr;
		IXMLDOMNodePtr* pReturnNodePtr = new IXMLDOMNodePtr;
		*pReturnNodePtr = elementPtr->setAttributeNode(&(*attPtr));
		CWLXmlNodeWrapper* wrapper = new CWLXmlNodeWrapper(pReturnNodePtr);
		return WLXmlSetNodeInterfaceValue(cx, obj, wrapper, rval);
	}
	CATCH_COM_METHOD(cx, "setAttributeNode")
}

//Removes the specified attribute from this element and returns it
//if this attribute has a default value(dtd, schema) it is newly created with the default value
JSBool
xmlelement_removeAttributeNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval, IXMLDOMNodePtr& pNodePtr)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		WlTraceMsg("Inside xmlelement_removeAttributeNode");
		if(argc != 1)
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "removeAttributeNode", 1);
		if(!JSVAL_IS_OBJECT(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "removeAttributeNode");
		//get the att node to be removed
		JSObject* attObj	= JSVAL_TO_OBJECT(argv[0]);
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, attObj);
		IXMLDOMAttributePtr attPtr =  *(((CWLXmlNodeWrapper*)(prv->GetObj()))->GetInterfacePtr());
		//remove  the node and return a NODE interface
		IXMLDOMElementPtr elementPtr  = pNodePtr;
		IXMLDOMNodePtr* pReturnNodePtr = new IXMLDOMNodePtr;
		*pReturnNodePtr = elementPtr->removeAttributeNode(&(*attPtr));
		CWLXmlNodeWrapper* wrapper = new CWLXmlNodeWrapper(pReturnNodePtr);
		return WLXmlSetNodeInterfaceValue(cx, obj, wrapper, rval);
	}
	CATCH_COM_METHOD(cx, "removeAttributeNode")
}


//Returns a list of all descendant elements that match the supplied name.
//the list is automatically updated when elements are added/deleted etc.

JSBool
xmlelement_getElementsByTagName(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval, IXMLDOMNodePtr& pNodePtr)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		WlTraceMsg("Inside xmlelement_getElementsByTagName");
		if(argc != 1)
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "getElementsByTagName", 1);
		if(!JSVAL_IS_STRING(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "getElementsByTagName");
		//get the tagName string
		char* attName = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
		_bstr_t tagName(attName);
		//get the elements
		IXMLDOMElementPtr elementPtr  = pNodePtr;
		IXMLDOMNodeListPtr* pNodeList = new IXMLDOMNodeListPtr;
		*pNodeList = elementPtr->getElementsByTagName(tagName);
		CWLXmlNodeListWrapper* wrapper = new CWLXmlNodeListWrapper(pNodeList);
		return WLXmlSetNodeListInterfaceValue(cx, obj, wrapper, rval);
	}
	CATCH_COM_METHOD(cx, "getElementsByTagName")
}

JSBool
xmlelement_normalize(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval, IXMLDOMNodePtr& pNodePtr)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		WlTraceMsg("Inside xmlelement_normalize");
		if(argc)
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "normalize", 0);
		IXMLDOMElementPtr elementPtr  = pNodePtr;
		HRESULT hr = elementPtr->normalize();
		return succeeded(cx, hr);
	}
	CATCH_COM_METHOD(cx, "normalize")
}

