/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <ace/OS.h>
#include "wlxmlcom.h"
#include "wlxmlfuncs.h"
#include <string.h>
#include "rvjsapi.h"

#include "xmlreplyprops.h"
#include "jsxmlreplynames.h"
#include "jsxmlprivate.h"
#include "jsXmlWrappers.h"
#include "xmlerrorreporter.h"
#include "tracedll.h"


CWLXmlImplementationWrapper::CWLXmlImplementationWrapper(IXMLDOMImplementationPtr* pImplementationPtr) {
	m_pXMLImplementationPtr = pImplementationPtr;
}
	

IXMLDOMImplementationPtr* CWLXmlImplementationWrapper::GetInterfacePtr() {
	return m_pXMLImplementationPtr;
}

CWLXmlImplementationWrapper::~CWLXmlImplementationWrapper() {
		if(m_pXMLImplementationPtr)
			delete m_pXMLImplementationPtr;
}


static JSBool
xmlimplementation_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
	//no prperties to implementation
	WlTraceMsg("Inside xmlimlementation_getProperty");
	return JS_TRUE;
}

static JSBool 
xmlimplementation_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp) {
	//no prperties to implementation
	WlTraceMsg("Inside xmldocument_setProperty");
	return JS_TRUE;
}

static JSBool
xml_implementation_hasFeature(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) 
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		WlTraceMsg("Inside xml_implementation_hasFeature(%x, %x)", cx, obj);
		if(argc != 2)
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "hasFeature", 2);//?error
		if(!JSVAL_IS_STRING(argv[0]) || !JSVAL_IS_STRING(argv[1]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "hasFeature", 2); 
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
		if(!prv)
			return JS_TRUE;
		CWLXmlImplementationWrapper* impWrapper =  (CWLXmlImplementationWrapper*)prv->GetObj();
		if(!impWrapper)
			return JS_TRUE;
		IXMLDOMImplementationPtr pImp =  *(impWrapper->GetInterfacePtr());
		_bstr_t feature(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
		_bstr_t version(JS_GetStringBytes(JSVAL_TO_STRING(argv[1])));
		bool hasFeature = true;
		if(pImp->hasFeature(feature, version) == 0)
			hasFeature = false;
		return WLXmlSetBoolValue(cx, obj, NULL, hasFeature, rval);
	}
	CATCH_COM_METHOD(cx, "hasFeature")
}
	





static void
xmlimplementation_finalize(JSContext *cx, JSObject *obj)
{
	WlTraceMsg("Inside xmlimlementation_finalize(%x, %x)", cx, obj);
	WLXmlFinalize(cx, obj);
}																																																												

JSClass wl_XmlimplementationClass = {
	WLJSNAME_CLASS_XMLIMPLEMENTATION,
	JSCLASS_HAS_PRIVATE,	//ptr to msxml interface structure that info regarding the object it represents
	JS_PropertyStub,			//callback to C++
	JS_PropertyStub,			//--"--
	xmlimplementation_getProperty,
	xmlimplementation_setProperty,
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	xmlimplementation_finalize
};

static JSFunctionSpec implementation_methods[] = {
	//{wl_toString_str,		frame_toString,		0},
	{"hasFeature",	xml_implementation_hasFeature,	0},
	{0}
};

//constructor
static JSBool
WLXmlImplementation(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	return JS_TRUE;
}

JSObject* WLNewXmlImplementationObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewXmlImplementationObject(%x, %x)", cx, obj);
	// create an instance 
	JSObject* proto = NULL;
	JSObject* xmlimplementation = WLNewXmlObject(cx, &wl_XmlimplementationClass, proto, obj);
	if (!xmlimplementation)
		return NULL;
//	if (!JS_DefineFunctions(cx, xmlimplementation, implementation_methods))
//		return NULL;


	return xmlimplementation;
}


bool DllExport WLAddXmlImplementationClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddXmlImplementationClass(%x, %x)", cx, obj);
	if (!JS_InitClass(cx, obj, NULL, &wl_XmlimplementationClass, NULL, 1, NULL, implementation_methods, NULL, NULL))
		return false;

	return true;
}




