/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <ace/OS.h>
#include "wlxmlcom.h"
#include "wlxmlfuncs.h"
#include <string.h>
#include <RVCString.h>
#include "rvjsapi.h"

#include "xmlreplyprops.h"
#include "jsxmlreplynames.h"
#include "jsxmlprivate.h"
#include "jsxmlwrappers.h"
#ifdef _DEBUG
#include <iostream.h>
#endif

#include "xmlerrorreporter.h"
#include "tracedll.h"


CWLXmlNodeListWrapper :: CWLXmlNodeListWrapper(IXMLDOMNodeListPtr* pNodeListPtr){
	m_pNodeListPtr = pNodeListPtr;
}

IXMLDOMNodeListPtr* CWLXmlNodeListWrapper:: GetInterfacePtr(){
		return m_pNodeListPtr;
}

CWLXmlNodeListWrapper :: ~CWLXmlNodeListWrapper(){
		if(m_pNodeListPtr)
			delete m_pNodeListPtr;
}

static JSBool
xmlnodelist_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
	TRY_COM {
		WlTraceMsg("Inside xmlnodelist_getProperty");
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
		if (!prv)
			return JS_TRUE;
		IXMLDOMNodeListPtr pNodeListInterface = *(((CWLXmlNodeListWrapper*)(prv->GetObj()))->GetInterfacePtr());
		if ((pNodeListInterface != NULL) && JSVAL_IS_STRING(id)) {
			JSString* str = JSVAL_TO_STRING(id);
			char* prop = JS_GetStringBytes(str);
			if (!strcmp(prop, WLJSNAME_PROP_LENGTH)) {	
				long length;
				length = pNodeListInterface->Getlength();
				return WLXmlSetIntValue(cx, obj, NULL, (unsigned)length, vp);
			}

		}
		return JS_TRUE;
	}
	CATCH_COM(cx)
}

static JSBool
xmlnodelist_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp) {
	WlTraceMsg("Inside xmlnodelist_setProperty");
	return JS_TRUE;
}

static JSBool
xmlnodelist_getItem(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
		if (!prv)
			return JS_TRUE;
		if(argc != 1)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "item", 1);
		if(!JSVAL_IS_INT(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "item");
		CWLXmlNodeListWrapper* wrapper	= (CWLXmlNodeListWrapper*)(prv->GetObj());
		if(!wrapper)
			return JS_TRUE;//error
		IXMLDOMNodeListPtr listPtr				= *(wrapper->GetInterfacePtr());
		jsint index = JSVAL_TO_INT(argv[0]);
		IXMLDOMNodePtr* node = new IXMLDOMNodePtr;
		*node = listPtr->Getitem((long)index);
		CWLXmlNodeWrapper* nodeWrapper = new CWLXmlNodeWrapper(node);
		return WLXmlSetNodeInterfaceValue(cx, obj, nodeWrapper, rval);
	}
	CATCH_COM(cx)
}

static JSBool
xmlnodelist_nextNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
		if (!prv)
			return JS_TRUE;
		if(argc)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "nextNode", 0);
		CWLXmlNodeListWrapper* wrapper	= (CWLXmlNodeListWrapper*)(prv->GetObj());
		if(!wrapper)
			return JS_TRUE;//error
		IXMLDOMNodeListPtr listPtr				= *(wrapper->GetInterfacePtr());
		IXMLDOMNodePtr* node = new IXMLDOMNodePtr;
		*node = listPtr->nextNode();
		CWLXmlNodeWrapper* nodeWrapper = new CWLXmlNodeWrapper(node);
		return WLXmlSetNodeInterfaceValue(cx, obj, nodeWrapper, rval);	
	}
	CATCH_COM(cx)
}

static JSBool
xmlnodelist_reset(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
		if (!prv)
			return JS_TRUE;
		if(argc )//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "reset", 1);
		CWLXmlNodeListWrapper* wrapper	= (CWLXmlNodeListWrapper*)(prv->GetObj());
		if(!wrapper)
			return JS_TRUE;//error
		IXMLDOMNodeListPtr listPtr				= *(wrapper->GetInterfacePtr());
		HRESULT hr = listPtr->reset( );
		return succeeded(cx, hr);
	}
	CATCH_COM(cx)
}

static void
xmlnodelist_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside xmlnodelist_finalize(%x, %x)", cx, obj);
	WLXmlFinalize(cx, obj);
}					

JSClass wl_XmlnodelistClass = {
	WLJSNAME_CLASS_XML_NODELIST,
	JSCLASS_HAS_PRIVATE,	//ptr to msxml interface structure that info regarding the object it represents
	JS_PropertyStub,			//callback to C++
	JS_PropertyStub,			//--"--
	xmlnodelist_getProperty,
	xmlnodelist_setProperty,
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	xmlnodelist_finalize
};

static JSFunctionSpec nodeList_methods[] = {
	{"item", 						xmlnodelist_getItem,		 0},
	{"nextNode",				xmlnodelist_nextNode,		 0},
	{"reset",						xmlnodelist_reset,			 0},
	{0}
};


//constructor
static JSBool
WLNodeList(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	return JS_TRUE;
}

JSObject* WLNewXmlNodeListObject(JSContext *cx, JSObject *obj)
{
	WlTraceMsg("WLNewXmlNodeListObject(%x, %x)", cx, obj);
	// create an instance 
	JSObject* proto = NULL;
	JSObject* nodeList = WLNewXmlObject(cx, &wl_XmlnodelistClass, proto, obj);
	if (!nodeList)
		return NULL;

	// define it's methods
//	if (!JS_DefineFunctions(cx, nodeList, nodeList_methods))
//		return NULL;

	return nodeList;
}

bool DllExport WLAddXmlNodeListClass(JSContext *cx, JSObject *obj)
{
	WlTraceMsg("WLAddXmlNodeListClass(%x, %x)", cx, obj);
	if (!JS_InitClass(cx, obj, NULL, &wl_XmlnodelistClass, NULL, 1, NULL, nodeList_methods, NULL, NULL))
		return false;

	return true;
}
