/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// Log.h: interface for the CLog class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(__CMD_TOOLS_LOG_H__)
#define __CMD_TOOLS_LOG_H__

enum TimestampType{
		  etAbsolute = 0
		, etRelative = 1
		, etNone = 2 
};

enum MessageType{
		  eCommand = 0
		, eEvent 
		, eStats
		, eStart
		, eFinish
		, eMSG_User
		, eMSG_LG
};

extern const char* KTimeStampType[3];

/** Encapsulate log functionality. */
class CLog  
{
public:

	/** Initialize log object by name and features. */
	static void InitLog( 
		const char* filename, const TimestampType a_tsType = etAbsolute, 
		const bool a_overvrite = false, const bool a_console = false );

	/** Reset log object. */
	static void ResetLog();

	/** return global log object. */
//	static const CLog& GetLogger();

	/** Destructor. */
	virtual ~CLog();

	/** Write function. 
	@param - message and type of message. */
	static void Write( const char*, const MessageType a_eType = eStats );

private:
	/** Constructor. Called by InitLog(...) only */ 
	CLog( const char* filename, const TimestampType a_tsType, const bool a_overvrite, const bool a_console );
	
	///Log file name
	char* m_logFileName;
	//file 
	FILE* m_logFile;
	///Type of timestamp - may be absolute, relative, and absent
	TimestampType	m_tsType;
	///startup timestamp
	__int64 m_timeStamp;
	///console output flag
	bool m_consoleDump;
};

///
const size_t KMaxTimerEntryNameLen = 255;

/** The class story time in constructor and calculate and print duration in destructor. */
class CTimerEntry
{
public:
	///
	CTimerEntry(const char*);

	///
	~CTimerEntry( );

private:
	///
	char		m_strName[KMaxTimerEntryNameLen];
//	__int64 m_timestamp;
	unsigned int m_timerStart;
};

#define TIMERENTRY(name)	CTimerEntry	entr(name);

#endif // __CMD_TOOLS_LOG_H__
