/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#ifndef __JSClient_H
#define __JSClient_H
#include "tracedll.h"
#include "rvjsmachinethread.h"
#include "rvjsembed.h"
#include "loadgeneratorscript.h"
#include "loadreportinfo.h"
#include <hashtab.h>
#include <thr.h>

class DllExport iClientFile;
class DllExport cLEJSClientContext;
class DllExport cLEJSClientMachineThread;
class DllExport iLEJSClientThread;
class DllExport CRVCompileErrorInfo;
class DllExport iLEJSClientEmbedContext;

class DllExport iLEJSStatementHandle;
class DllExport iLEJSStatementInfo;
class DllExport iLEJSStatementCompileInfo;

class DllExport cLEJSFunctionHandle;
class DllExport cLEJSFunctionStatementInfo;
class DllExport cLEJSFunctionStatementCompileInfo;

class DllExport cLEJSScriptHandle;
class DllExport cLEJSScriptStatementInfo;
class DllExport cLEJSScriptStatementCompileInfo;

//class DllExport cLEJSScriptEvaluate;
class iLEProtocolJSClientEmbedContext;

template< typename rcRoot_ > class ecRVJSRoot;
typedef ecRVJSRoot< JSObject* > cRVJSRootObject;
//class DllExport cRVJSRootObject;
typedef ecRVJSRoot< jsval > cRVJSRootVal;
//class DllExport cRVJSRootVal;

class iClientFile: public iFile
{
public:
	explicit iClientFile() {}
	virtual ~iClientFile() {}

	void vPick();
	bool vDrop();

	virtual const char* fGetSource() const = 0;
};


class cLEJSClientContext: public iLEVCJSThread
{
public:
	explicit cLEJSClientContext();
	void fInit( CLoadGeneralThreadInstanceScript& );
	void fFinal();
	virtual ~cLEJSClientContext();	
	
private:
	CLoadGeneralThreadInstanceScript* mGeneralThreadInstanceScript;
	JSContext* mCx;

//iLEVCJSThread overrides

	CLoadGeneralThreadInstanceScript& vGetGeneralThreadInstanceScript();
	JSContext* vGetContext();

};

class cLEJSClientMachineThread: public iLEJSMachineThread, public cLEJSClientContext
{
public:
	explicit cLEJSClientMachineThread( CGeneralProcess&, cLEProtocolJSClientMachineThread& );
	virtual ~cLEJSClientMachineThread();	

	void fInit( CLoadGeneralThreadInstanceScript& );
	void fFinal();

  iLEProtocolMachineThread& GetProtocolMachineThread(WLProtocolIndex pProtocolIndex);

//iLEVCJSThread overrides
	iLEJSMachineThread& vGetMachineThread();

private:
  cLEProtocolJSClientMachineThread& m_ProtocolJSClientMachineThread;
};



class iLEJSClientThread
{
public:
	explicit iLEJSClientThread();
	void fInit( iLEVCJSThread& );
	void fFinal();
	virtual ~iLEJSClientThread();	

	iLEVCJSThread& fGetJSThread();

  CLoadGeneralThreadInstanceScript& fGetThreadInstanceScript();
	JSContext* fGetContext();

	iLEJSMachineThread& fGetMachineThread() { WlAssert(mJSThread); return mJSThread->fGetMachineThread(); }

private:
	iLEVCJSThread* mJSThread;
};

class CRVCompileErrorInfo {
public:
  uintN errorNumber;
  RVCString linebuf;
  RVCString filename;
  uintN currentLine;

  explicit CRVCompileErrorInfo();
  virtual ~CRVCompileErrorInfo();
};

class iLEJSClientEmbedContext: public iLEJSEmbedContext
{
	friend class cLEJSFunctionHandle;
  friend class iLEProtocolJSClientEmbedContext;
public:

	explicit iLEJSClientEmbedContext(iLEProtocolJSClientEmbedContext&);
//	void fInit();
//	void fFinal();
	virtual ~iLEJSClientEmbedContext();	

	JSObject* fSet( JSObject* rGlobal );
	void fClear();
	JSBool fCallFunction( JSObject* rScope, jsval rFuncVal, uintN rArgc, jsval rArgv[], jsval& oRval );

	bool IsJSActive();

	CLoadGeneralThreadInstanceScript& fGetThreadInstanceScript();
  iLEProtocolJSEmbedContext& GetProtocolJSEmbedContext(WLProtocolIndex pProtocolIndex);

	JSContext* fGetContext();

  bool Report( iLoadReportInfo& );

	virtual void vReporter( WLLogMessage::Severity, const char* rMessage ) = 0;

  CRVCompileErrorInfo mCompileErrorInfo;

  //void SetCharset (long pCharset) { mCharset = pCharset; }
  //long GetCharset () { return mCharset; }


private:
	JSObject* mJSGlobal;
	int mJSActiveCtr;
  iLEProtocolJSClientEmbedContext&  mProtocolJSClientEmbedContext;
	int mUserSectionActCtr;
	int mUserSectionActAntiCtr;

  //long mCharset; // Character set of CSJS code

	void fSetApply();
	
// iLEJSEmbedContext overrides
	iLEJSMachineThread& vGetMachineThread();
	iSession& vGetSession();
	bool vIsUserSection();

	JSContext* vCurrentJSContext();
	void reportApply( iReport& );
	void reportLog( iReport& );

	void vEnterControlMachineThread_iLEJSEmbedContext();
	void vExitControlMachineThread_iLEJSEmbedContext();

	virtual iLEJSClientThread& vGetThread() = 0;
		iLEJSClientThread& fGetThread() { return vGetThread(); } 

	iLEVCJSThread& fGetJSThread();

	void vEnterJava( const char* rClassDescriptionName, const char* rMemberName, tJavaCallType );
	void vExitJava( JSBool rOK, const char* rClassDescriptionName, const char* rMemberName, tJavaCallType );

	bool vIsJSGC();

public:
	class DllExport cUserSectionGuard {
	public:
		explicit cUserSectionGuard( iLEJSClientEmbedContext& );
		~cUserSectionGuard();

	private:
		iLEJSClientEmbedContext& mEmbedContext;

};
	friend class cUserSectionGuard;

	class DllExport cUserSectionOffGuard {
	public:
		explicit cUserSectionOffGuard( iLEJSClientEmbedContext& );
		~cUserSectionOffGuard();

	private:
		iLEJSClientEmbedContext& mEmbedContext;

	};
	friend class cUserSectionOffGuard;

};

typedef iLEJSClientEmbedContext::cUserSectionGuard cLEJSClientEmbedContextUserSectionGuard;
typedef iLEJSClientEmbedContext::cUserSectionOffGuard cLEJSClientEmbedContextUserSectionOffGuard;


// Common classes for cache
bool CacheReadOnly(iLEJSClientEmbedContext&);

class iLEJSStatement : public RVCollectable
{
public:
	explicit iLEJSStatement();

	virtual ~iLEJSStatement();

	void IncrementRef() { ++mRefN;}
	void DecrementRef() { --mRefN;}
	void IncrementUse() { ++mUseCnt;}
	int GetRef() { return mRefN; }
	int GetUse() { return mUseCnt; }

	virtual JSBool vAddRoot(JSContext* rCx) = 0;
	virtual JSBool vRemoveRoot(JSContext* rCx) = 0;

private:
	int					mRefN;
	int					mUseCnt;
};

class iLEJSStatementInfo : public RVCollectable
{
public:
	explicit iLEJSStatementInfo();
	virtual ~iLEJSStatementInfo();

	// Give a 'clone' of the statement info on the heap.
	// Loadengine will be resonsible for delete.
	// Httpload is allowed not to clone if in persistent storage
	virtual iLEJSStatementInfo* vCloneStatementInfo() = 0;
	iLEJSStatementInfo* fCloneStatementInfo() {
		WlAssert(mInit);
		return vCloneStatementInfo();
	}
	virtual iClientFile& vGiveClientFile() const = 0;
	iClientFile& fGiveClientFile() const {
		WlAssert(mInit);
		return vGiveClientFile();
	}

	// This function is called when load egnine wants to have his own copy of the
	// client file in the statement info

	virtual void vCloneClientFile() = 0;
	void fCloneClientFile(){
		WlAssert(mInit);
		vCloneClientFile();
	}

	// fInit must be called by user
	void fInit();
	virtual unsigned hash() const;
	int fGetSourceLength();
	const char* fGetSource();
	virtual const char* vGetIDString() = 0;

  virtual long vGetCharset() = 0;
  long fGetCharset(){
    WlAssert(mInit);
    return vGetCharset();
  }

private:
	int mSourceLength;
	unsigned mHashValue;
	virtual unsigned vCalcHashValue() = 0;
	bool mInit;
};

class iLEJSStatementCompileInfo
{
friend class cLEJSFunctionHandle;
friend class cLEJSScriptHandle;
public:
	explicit iLEJSStatementCompileInfo(JSObject* rScope):
	mScope(rScope)
	{}

	virtual ~iLEJSStatementCompileInfo() {}
protected:
	JSObject*			mScope;
};

class iLEJSStatementManager : public RVHashDictionary
{
	friend class cLEJSStatementManagerGuard;
public:
	explicit iLEJSStatementManager();
	virtual ~iLEJSStatementManager();

	iLEJSStatement* FindStatement(const iLEJSStatementInfo* pStatementInfo);
	iLEJSStatementInfo* AddStatement(iLEJSStatementInfo* pStatementInfo, iLEJSStatement* pStatment);
	void fFreezeCond(JSContext*, long);
	void fFinal(JSContext*);
	bool fIsFrozen();

private:
	RVMutexLock			mReadWriteLock;

	bool						mFrozen;

	void fFreeze(JSContext*, long);
};

class cLEJSStatementManagerGuard
{
public:
	explicit cLEJSStatementManagerGuard(iLEJSClientEmbedContext& rEmbedContext, iLEJSStatementManager& rStatementManager);
	virtual ~cLEJSStatementManagerGuard();

private :
	RVMutexLock::LockGuard* mGuard;
};

class iLEJSStatementHandle
{
public:
	explicit iLEJSStatementHandle(iLEJSStatementInfo&);
	virtual ~iLEJSStatementHandle();

	virtual bool fCompile(iLEJSClientEmbedContext&) = 0;

	void fFinal(iLEJSClientEmbedContext&);

protected:
	iLEJSStatement* GetStatement(iLEJSClientEmbedContext&, iLEJSStatementManager&);
  void fTraceStatement(const char*);

protected:
	iLEJSStatementInfo*
									mStatementInfo;
	iLEJSStatement* mStatement;
	bool						mOwnStatement;
	iLEJSStatementCompileInfo*
									mCompileInfo;

	void SetCompileInfo(iLEJSStatementCompileInfo* pCompileInfo) { mCompileInfo = pCompileInfo;}

	void fAddRoot(JSContext* rCx) { 
		mAddedRoot = true;
		mStatement->vAddRoot(rCx);
	}
	
	void fRemoveRoot(JSContext* rCx) { if (mAddedRoot) mStatement->vRemoveRoot(rCx);}

	void fBeforeExecute(iLEJSClientEmbedContext& rEmbedContext, iLEJSStatementManager& rStatementManager);
	void fAfterExecute(iLEJSClientEmbedContext& rEmbedContext, iLEJSStatementManager& rStatementManager);

private:
	bool mFinal;
	bool mAddedRoot;
	bool mCacheFrozen;
	
	// Create actual statment object including JS stuff
	virtual iLEJSStatement* CreateStatement(iLEJSClientEmbedContext& rEmbedContext) = 0;
};


// Function classes
class cLEJSFunctionStatement: public iLEJSStatement
{
friend class cLEJSFunctionHandle;
public:
	explicit cLEJSFunctionStatement();

	virtual ~cLEJSFunctionStatement();

	virtual JSBool vAddRoot(JSContext* rCx) { return JS_AddNamedRoot(rCx, &mFuncVal, "cLEJSFunctionHandle::mFuncVal");}
	virtual JSBool vRemoveRoot(JSContext* rCx) {	return JS_RemoveRoot(rCx, &mFuncVal); }

	jsval fGetFuncVal();
	const char* fGetToString( JSContext* );
	const char* fGetToString( iLEJSClientEmbedContext& );
private:
	jsval mFuncVal;
};


class cLEJSFunctionStatementInfo: public iLEJSStatementInfo
{
public:
	explicit cLEJSFunctionStatementInfo(const char* pName);
	virtual ~cLEJSFunctionStatementInfo();

	virtual const char* vGetIDString();
	virtual const char* vGetName();
	virtual bool isEqual(const RVCollectable* t) const;

private:
	RVCString mName;
	virtual unsigned vCalcHashValue();
};

class cLEJSFunctionStatementCompileInfo: public iLEJSStatementCompileInfo
{
friend class cLEJSFunctionHandle;
public:
  explicit cLEJSFunctionStatementCompileInfo(JSObject* rScope, uintN rNargs, jsval* rArgNames, uintN rLineNumber);
	virtual ~cLEJSFunctionStatementCompileInfo();
private:
	uintN					mNargs;
	jsval*			  mArgNames;
	uintN					mLineNumber;
};

class cLEJSFunctionHandle: public iLEJSStatementHandle
{
public:
	explicit cLEJSFunctionHandle(cLEJSFunctionStatementInfo&);
	virtual ~cLEJSFunctionHandle();

	int fGetSourceLength();
	const char* fGetSource();
	const char* fGetName();

	virtual bool fCompile(iLEJSClientEmbedContext&);

	bool fSet( iLEJSClientEmbedContext& rEmbedContext, cLEJSFunctionStatementCompileInfo& rFunctionCompileInfo);
	bool fSet( iLEJSClientEmbedContext&, jsval rFuncVal );
	bool fIsSet();
	JSBool fCall( iLEJSClientEmbedContext& rEmbedContext, jsval& oRval );
	jsval fGetFuncVal(iLEJSClientEmbedContext& rEmbedContext);
	const char* fGetToString( JSContext* );
	const char* fGetToString( iLEJSClientEmbedContext& );

protected:
	JSObject*	mFuncCloneObject;
	jsval			mFuncVal;

private:
	JSBool fCall( iLEJSClientEmbedContext& rEmbedContext, jsval rFuncVal, jsval& oRval );
	iLEJSStatement* CreateStatement(iLEJSClientEmbedContext& rEmbedContext);
};


// Script classes
class cLEJSScriptStatement: public iLEJSStatement
{
friend class cLEJSScriptHandle;
public:
	explicit cLEJSScriptStatement();
	virtual ~cLEJSScriptStatement();

	virtual JSBool vAddRoot(JSContext* rCx) { return JS_AddNamedRoot(rCx, &mRootObject, "cLEJSScriptStatement::mScriptObject");}
	virtual JSBool vRemoveRoot(JSContext* rCx) {	return JS_RemoveRoot(rCx, &mRootObject); }

	JSScript* fGetScript() { return mScript; }
	JSObject* fGetScriptObject() { return mRootObject; }

private:
	RVCString mCode;
	JSScript* mScript;
	JSObject* mRootObject;
};

class cLEJSScriptStatementInfo: public iLEJSStatementInfo
{
public:
	explicit cLEJSScriptStatementInfo();	
	virtual ~cLEJSScriptStatementInfo();

	virtual const char* vGetIDString();

	virtual bool isEqual(const RVCollectable* t) const;

private:
	virtual unsigned vCalcHashValue();
};

class cLEJSScriptStatementCompileInfo: public iLEJSStatementCompileInfo
{
friend class cLEJSScriptHandle;
public:
  explicit cLEJSScriptStatementCompileInfo(JSObject* rScope);
	virtual ~cLEJSScriptStatementCompileInfo();
};

class cLEJSScriptHandle: public iLEJSStatementHandle
{
public:
	explicit cLEJSScriptHandle(cLEJSScriptStatementInfo&);
	virtual ~cLEJSScriptHandle();

	int fGetSourceLength();
	const char* fGetSource();
	
	virtual bool fCompile(iLEJSClientEmbedContext&);

	JSBool fExecute( iLEJSClientEmbedContext& rEmbedContext, cLEJSScriptStatementCompileInfo& rScriptCompileInfo, jsval& oRval );

	// Two rules for using the following 2 functions:
	// 1. If you call fExecuteProlog, you must call fExecuteMain before the dtor
	// 2. You can;t call fExecuteMain without calling fExecuteProlog first
	JSBool fExecuteProlog( iLEJSClientEmbedContext& rEmbedContext, cLEJSScriptStatementCompileInfo& rScriptCompileInfo, jsval& oRval);
	JSBool fExecuteMain( iLEJSClientEmbedContext& rEmbedContext, jsval& oRval);

private:
	bool mExecPrologDone;
	bool mExecMainDone;
	iLEJSStatement* CreateStatement(iLEJSClientEmbedContext& rEmbedContext);
};






template< typename rcRoot_ >
class ecRVJSRoot
{
public:
	typedef rcRoot_ rcRoot;
	typedef ecRVJSRoot< rcRoot > jcRVJSRoot;

  explicit ecRVJSRoot(
	  JSContext* rJSContext,
	  rcRoot rRoot,
		const char* rName //= "ecRVJSRoot" 
		)
	:
	  mJSContext(rJSContext),
	  mRoot(rRoot)
	{
	  JS_AddNamedRoot( rJSContext, reinterpret_cast<void*>(&mRoot), rName );
	}

  
	~ecRVJSRoot()
	{
	  JS_RemoveRoot( mJSContext, reinterpret_cast<void*>(&mRoot) );
	}

	void fSet(
		rcRoot rRoot )
	{
		mRoot = rRoot;
	}

	rcRoot fGet() const
	{
		return mRoot;
	}

	rcRoot* fGetAddress()
	{
		return &mRoot;
	}

private:
  JSContext* mJSContext;
  rcRoot mRoot;
};


#endif //__JSClient_H

