/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_Runtime_H
#define __WL_Runtime_H
 
#include "platform.h"
#include "wlleprotocolcontainer.h"
#include "rvjsembed.h"
#include "rvjsclient.h"
#include "rvjsmachinethread.h"
#include "tvpsync.h"
#include <exception>

class cLEJSRuntimeEmbedContext : public iLEJSEmbedContext {
public:
  explicit cLEJSRuntimeEmbedContext(JSContext* p_Cx, iLEJSMachineThread* p_MachineThread):
    m_Cx(p_Cx),
    m_MachineThread(p_MachineThread)
  {
    fSignContext(m_Cx);
  };

  virtual ~cLEJSRuntimeEmbedContext() {};

  void reportApply( iReport& ) {;}
  void reportLog( iReport& ) {;}
  bool IsJSActive() { return true; }

private:
  iLEJSMachineThread& vGetMachineThread() { WlAssert(m_MachineThread); return *m_MachineThread;};
  void vEnterJava( const char* rClassDescriptionName, const char* rMemberName, tJavaCallType ) {;};
  void vExitJava( JSBool rOK, const char* rClassDescriptionName, const char* rMemberName, tJavaCallType ) {;};
  bool vIsJSGC() { return true; };
  JSContext* vCurrentJSContext() { WlAssert(m_Cx); return m_Cx;};
  iSession& vGetSession() {WlAssert(0); iSession* t = NULL; return *t;};
  iLEProtocolJSEmbedContext& GetProtocolJSEmbedContext(WLProtocolIndex pProtocolIndex) {
    WlAssert(0);
    void  *tmp;
    return (iLEProtocolJSEmbedContext&)tmp;
  }

  bool vIsUserSection() { return true;}
private:
  JSContext*          m_Cx;
  iLEJSMachineThread* m_MachineThread;
};

class CLEJSRuntimeMachineThread : public iLEJSAgendaMachineThread {
public:
  explicit CLEJSRuntimeMachineThread(CGeneralProcess& rProcess): iLEJSAgendaMachineThread(rProcess){};
  virtual ~CLEJSRuntimeMachineThread(){};
};

class DllExport CJSLoadRuntime: public nJs::cRuntime {
private:
  JSContext*                m_Cx;
  CLEJSRuntimeMachineThread m_MachineThread;   
  UINT                      m_RuntimeSize;
	iLEProtocolContainer      m_ProtocolJSRuntimeContainer;
	bool                      m_Terminate;
  RVThread                  m_RuntimeThread;
  // Events for controlling Runtime thread
  TVPManualEvent            m_RuntimeStartThreadEvent;
  TVPManualEvent            m_RuntimeStopThreadEvent;
  TVPManualEvent            m_RuntimeStopThreadDoneEvent;
public:
	CJSLoadRuntime(UINT pRuntimeSize, CGeneralProcess& pProcess);
	~CJSLoadRuntime();

	void Terminate();
private:
  void InitProtocols(iLEProtocolContainer& pProtocolProcessContainer);
  void ProtocolJSRuntimeAfterNewMainContext(JSContext* cx);
  void ProtocolJSRuntimeBeforeDestroyMainContext(JSContext* cx);
  void RuntimeThread();
  void ExceptionHandler(std::exception*);
public:
	iLEJSStatementManager			m_ScriptCache;
	iLEJSStatementManager			m_FunctionCache;
};

#endif
