/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#ifndef __WLLEProtocol_H
#define __WLLEProtocol_H

struct JSContext;
struct JSObject;//??
class IGeneralManagerScriptData;
class CLoadGeneralThreadInstanceScript;
class CWLJSThread;
class CLEVariablesData;
class iReport;
class IFileDictionary;
class IGeneralGlobalsRequestHandler;
class CAssociativeArray;
class iLEAgendaJSEmbedContext;
class CLEVariablesData;
class CWLJSJavaApplet;
class RVHashDictionary;
class cRVJVM;
class iSession;

#include "platform.h"
#include "generalprocess.h"
#include "rvjsreport.h"
#include "loadjsshell.h"
#include "wlstatpointtypeenum.h"
#include "wlleprotocolbasetypes.h"
#include "wlleprotocolembed.h"
#include <collectable.h>
#include <exception>

class DllExport iLEProtocolBase;
class DllExport iLEProtocolProcess;
class DllExport iLEProtocolSession;
class DllExport iLEProtocolJSRuntime;
class DllExport iLEProtocolAgendaMachineThread;
class DllExport iLEProtocolAgendaJSEmbedContext; 
class DllExport iLEProtocolAgendaInMix;
class DllExport iLEProtocolVCInMix;


//####### The declaration of the class iLEProtocolBase  #########

class iLEProtocolBase : public RVCollectable {
public:
  explicit iLEProtocolBase();
  virtual ~iLEProtocolBase();
	virtual iLEProtocolProcess& GetProtocolProcess() = 0;
};

//####### The declaration of the class iLEProtocolProcess  #########

class iLEProtocolProcess : public iLEProtocolBase {

private:
	WLProtocolIndex   m_ProtocolIndex;
  CGeneralProcess*  m_LEProcess;
 
protected:
	explicit iLEProtocolProcess();
public:
	virtual ~iLEProtocolProcess();

public:
//Callbacks that will be called from the LoadEngine module.
	virtual iLEProtocolSession& NewProtocolSession() = 0;
	virtual iLEProtocolAgendaMachineThread& NewProtocolAgendaMachineThread(WLProtocolThreadType) = 0;
  virtual iLEProtocolJSRuntime& NewProtocolJSRuntime() = 0;
	virtual bool ProcessInitialize() = 0;
	virtual bool ProcessInitializeJava(cRVJVM& jvm, JNIEnv* env) = 0;
	virtual bool ProcessFinalizeJava(cRVJVM& jvm, JNIEnv* env) = 0;
	virtual bool ProcessTermination() = 0;

  // @@Chaim - make pure virtual (this is for PMProbe only)
  virtual bool ProcessAfterStartLoad(int ReportResolutionSize) { return true; };

  // Called by LE
	void InitProtocol(WLProtocolIndex, CGeneralProcess*);//Set the protocol data member m_ProtocolIndex.
  WLProtocolIndex GetProtocolIndex() { return m_ProtocolIndex; };
  iLEProtocolProcess& GetProtocolProcess() { return *this;};
	iLEProtocolAgendaMachineThread& GetProtocolAgendaMachineThread(JSContext*);
  CGeneralProcess& GetLEProcess() { return *m_LEProcess;};

	bool AmIInScriptLevel(JSContext*);
	bool AmIInThreadLevel(JSContext*);
	iLEProtocolJSEmbedContext& GetProtocolJSEmbedContext(JSContext*);
	iLEProtocolAgendaJSEmbedContext& GetProtocolAgendaJSEmbedContext(JSContext*);
	iLEProtocolAgendaInMix& GetProtocolAgendaInMix(JSContext*);
	iLEProtocolVCInMix& GetProtocolVCInMix(JSContext*);
  iLEAgendaJSEmbedContext& GetAgendaJSEmbedContext(JSContext*);
  // Chaim - note that this is should be used with care as it searches the process' hash dict
  iLEProtocolMachineThread& GetCurrentProtocolMachineThread();

};




//####### The declaration of the class iLEProtocolSession  #########

class iLEProtocolSession : public iLEProtocolBase {
public:
	iLEProtocolProcess& m_iLEProtocolProcess;

protected:
	explicit iLEProtocolSession(iLEProtocolProcess&);
public:
	virtual ~iLEProtocolSession();

protected:
  iSession* m_Session;
public:
//Callbacks that will be called from the LoadEngine module.
	virtual iLEProtocolAgendaInMix& NewProtocolAgendaInMix() = 0;
	virtual bool SessionInitialize() = 0;
	virtual bool SessionTermination() = 0;

	iLEProtocolProcess& GetProtocolProcess();
public:
  void Init(iSession*);
};




//####### The declaration of the class iLEProtocolJSRuntime  #########

class iLEProtocolJSRuntime : public iLEProtocolBase {
public:
	iLEProtocolProcess& m_iLEProtocolProcess;

protected:
	explicit iLEProtocolJSRuntime(iLEProtocolProcess&);
public:
	virtual ~iLEProtocolJSRuntime();

public:
//Callbacks that will be called from the LoadEngine module.
	virtual bool JSRuntimeInitialize(JSRuntime*) = 0;
	virtual bool JSRuntimeAfterNewMainContext(JSContext*) = 0;
	virtual bool JSRuntimeBeforeDestroyMainContext(JSContext*) = 0;

	iLEProtocolProcess& GetProtocolProcess();
};


//####### The declaration of the class iLEProtocolAgendaJSEmbedContext   #########
class iLEProtocolAgendaJSEmbedContext : public iLEProtocolBase, public iLEProtocolJSEmbedContext {
protected:
  explicit iLEProtocolAgendaJSEmbedContext();
public:
  virtual ~iLEProtocolAgendaJSEmbedContext();  

  virtual RVCString& GetScriptName() = 0;
  virtual RVCString& GetScriptFileName() = 0;
};

//####### The declaration of the class iLEProtocolAgendaInMix  #########

class iLEProtocolAgendaInMix : public iLEProtocolAgendaJSEmbedContext {
public:
	iLEProtocolSession& m_iLEProtocolSession;

protected:
  IGeneralManagerScriptData* m_ScriptData;

protected:
	explicit iLEProtocolAgendaInMix(iLEProtocolSession&);
public:
	virtual ~iLEProtocolAgendaInMix();

public:
  void Init(IGeneralManagerScriptData*);

	virtual iLEProtocolVCInMix& NewProtocolVCInMix() = 0;
	virtual bool BeforeInitAgenda() = 0;
	virtual bool AfterTerminateAgenda() = 0;
	virtual bool JSInitializeForScript(JSContext*, JSObject*, JSObject*) = 0;
	virtual bool JSTerminateForScript(JSContext*, JSObject*, JSObject*) = 0;

public:
	iLEProtocolSession& GetProtocolSession();
	iLEProtocolProcess& GetProtocolProcess();


  // Exported from CGeneralLoadThreadInstanceScript

  // File ops.
  bool IncludeFile(const char*, bool );
  bool IncludeMonitorFile(const char*, bool);
  bool CopyFile(const char*, const char*);

  // Error handling
  void OnSevereError(); 

  // Java stuff
	bool IsJavaThreadWrapperCreationScriptExists();
	void SetJavaThreadWrapperCreationScript();

  // Env access and info
  RVCString& GetScriptName();
  RVCString& GetScriptFileName();
  const char* GetAgendaName();
	int GetNumScriptInMix();
	bool IsSpawn();
  bool InInitAgenda();
  IFileDictionary* GetFileDict();
	IGeneralGlobalsRequestHandler* GetGlobalsRequestHandler();

  // Misc.
  void AppendStringForEvaluation(const char*,CWLJSScript::WhenToRunScript);
  void SendMessage(CAssociativeArray&, int);

	bool IsJSActive();
  iLEJSEmbedContext& GetInternalLEJSEmbedContext();
};



//####### The declaration of the class iLEProtocolAgendaMachineThread  #########

class iLEProtocolAgendaMachineThread : public iLEProtocolBase, public iLEProtocolMachineThread {

protected:
	explicit iLEProtocolAgendaMachineThread(iLEProtocolProcess&, WLProtocolThreadType);
public:
	virtual ~iLEProtocolAgendaMachineThread();
  
protected:
  iLEJSAgendaMachineThread*         m_JSMachineThread;

public:
  void Init(iLEJSAgendaMachineThread*);

  virtual bool MachineThreadInitialize() = 0;
	virtual bool MachineThreadTermination() = 0;

	iLEProtocolProcess& GetProtocolProcess();

  // iLEJSAgendaMachineThread Stuff
  virtual iLEJSMachineThread& GetJSMachineThread();
};



//####### The declaration of the class iLEProtocolVCInMix  #########

class iLEProtocolVCInMix : public iLEProtocolAgendaJSEmbedContext, public iLEProtocolVCJSThread {
public:
	CWLJSThread* m_Parent;
	iLEProtocolAgendaInMix& m_iLEProtocolAgendaInMix;

protected:
  CLoadGeneralThreadInstanceScript* m_ThreadInstanceScript;

protected:
	explicit iLEProtocolVCInMix(iLEProtocolAgendaInMix&);
public:
	virtual ~iLEProtocolVCInMix();

public:
	virtual bool JSInitializeForClient(JSContext*, JSObject*, CLEVariablesData*) = 0;
	virtual bool OnThreadActivation() = 0;
	virtual bool BeforeInitClient() = 0;
	virtual bool AfterInitClient() = 0;
	virtual bool BeforeRound() = 0;
	virtual bool AfterRound() = 0;
	virtual bool BeforeTerminateClient() = 0;
	virtual bool AfterTerminateClient() = 0;
	virtual bool OnThreadTermination() =0;
        // @@Chaim - this should really be a pure virtual but it's only needed for protocol_httpload and maybe later
        // for httpload and I don't want to recompile everything.
        virtual bool OnThreadRestart() { return true; }
	virtual bool ExceptionHandler(std::exception& e) = 0;
	virtual bool BeforeSleep(long int) = 0;
	virtual bool AfterSleep(long int) = 0;

  // @@Chaim - make pure virtual (this is for httpload only)
  virtual bool SleepDisabled(long int recordedSleepTime) { return true; }
  virtual bool BeginTransaction(const char* transname, double time, const char* extraDataType, void* extraData, unsigned long len) { return true; }

  // Will be called only if the loadengine's EndTransaction is successful (an error there would
  // probably indicate that a non-existant transaction was being closed)
  virtual bool EndTransaction(const char* transname, double time, WLLogMessage::Severity severity, bool save, bool SendTimers, bool IsLog ) { return true; }

	JSContext* GetJSContext();
	JSObject* GetGlobalObject();
	CLEVariablesData& GetVariablesData();

	iLEProtocolAgendaInMix& GetProtocolAgendaInMix();
	iLEProtocolSession& GetProtocolSession();
	iLEProtocolProcess& GetProtocolProcess();

  iLEProtocolAgendaMachineThread& GetProtocolAgendaMachineThread();

public:
	void SetParent( CWLJSThread& );
  void Init(CLoadGeneralThreadInstanceScript*);

  // Exported from CGeneralLoadThreadInstanceScript

  // Java stuff
	void ToggleJavaDefaultStatistics(bool);
	bool IsDisabledJavaDefaultStatistics();

	void CreateApplet(
          JSContext*                  cx,
          const char*                 appletClassName,
					const char*                 appletCodeBase,
          const char*                 appletDocumentBase,
					RVHashDictionary*           appletParams,
          CWLJSJavaApplet*&           javaApplet);
  bool ClassAppletExists(
          JSContext*  cx,
          const char* appletClassName,
 					bool& classAppletExists);
  bool AddJavaStream(JSContext*  cx,
                     const char* streamName,
                     const char* codeBase,
						         RVCString&  javaStream);
	bool JavaStreamExists(const char* streamName);

  // Statistics
  void addMeasurement(const char*, double);
  void addTimer(const char*, double);
  void addCounter(const char*);
  void addAccumulator(const char*, double);
  void addLast(const char*, double, CWLStatPointTypeEnum::PointType);
	void resetTimer(const char*);
	void sendTimer(const char*);

	// Transaction
  void VerificationSetFailureReason(const char*, const char*, double, bool);
  void VerificationRecordEvent(const char*, const char*, double, WLLogMessage::Severity = WLLogMessage::Note, const char* = NULL, void* = NULL, unsigned long = 0, bool = true );
  bool VerificationBeginTransaction(const char*, double, const char*, void*, unsigned long);
  bool VerificationEndTransaction(const char*, double, WLLogMessage::Severity, bool, bool, bool);
  bool VerificationEndTransactionFailure(const char*, double, WLLogMessage::Severity, const char*, bool, bool, bool);
  int  VerificationGetTransactionDepth();
  const char* VerificationGetCurrentTransaction();

  // Misc. LE operations
  bool isAbort();
  void sleep(int);
	void reportApply(iReport&);
	void reportLog(iReport&);
	void reportApplyEvent(WLLogMessage::Severity, const char*);
	bool report(WLLogMessage::Severity, const char*);
	bool reportTime(WLLogMessage::Severity, const char*, double, WLLogMessage::Severity = WLLogMessage::Success);
	bool report(WLErrorException&);
	bool reportTime(WLErrorException&, double);
	bool reportMultiThread(iReportInfo&);
	bool reportMultiThread(WLLogMessage::Severity, const char* );
	bool reportMultiThread(WLErrorException&);
 	void endMultiThread();
  CLEVariablesData* GetCurrentLoadEngineVariablesData();

  // Env access and info
  RVCString& GetScriptName();
	RVCString& GetScriptFileName();
  bool IsJSGC();
  int GetNumScriptInMix();
  IFileDictionary* GetFileDict();
  unsigned long GetTotalRoundNum();
  unsigned long GetCurrentScriptRoundNum();
	unsigned long GetRoundNumScriptInSlave();
	unsigned long GetRoundNumMixInSlave();
  int GetLGThreadNum();
  WLLogMessage::Severity GetErrorSeverity();
  bool IsLoadServer();
  bool IsOneClient();
  bool IsRTAM();
	bool IsJSActive();

  // iLEProtocolJSEmbedContext stuff
  iLEJSEmbedContext& GetInternalLEJSEmbedContext();

	// iLEVCJSThread stuff
	iLEVCJSThread& vGetVCJSThread();
};

#endif

