/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
package com.radview.loadengine.appletloader;

import java.applet.Applet;
import java.util.HashMap;
import java.util.StringTokenizer;
import java.net.URL;
import java.net.MalformedURLException;

// Radview classes
import com.radview.util.log.Log;
import com.radview.util.rvexception.RVRunTimeException;
import com.radview.loadengine.JavaThreadWrapper;
import com.radview.loadengine.appletloader.system.*;
import com.radview.jrp.javaplayer.PlayerStub;


/**
 * public class wlapplet.WLAppletHandler
 *
 */
public class WLAppletHandler
{
	//private static WebLoadClassLoader __loader  = null;
	private static JarResources __appletResourcesManager = null;
	private static PlayerStub   __defaultPlayerStub = null;
	//private static ClassLoader  m_MainClassLoader = null;

	// per-thread frames tables + includes the javaThreadWrapper
	private static HashMap htFrames=new HashMap();

	// classLoader which is common to several VCs. one per codebase
	private static HashMap htClassLoaders=new HashMap();

	// #BEGIN: class static code
	static {
		try {

			//__loader = WebLoadClassLoader.getDefaultClassLoader();
			__appletResourcesManager = JarResources.getSystemJarResources();

		  // @@dudu : need to find out why this call cause "Illegal instruction" in debug.
			//   it cause the JVM to stay alive when destroy is at the end.   during the execution,
			//   everything works fine.
			__defaultPlayerStub = PlayerStub.getSystemPlayerStub();
		}
		catch(Exception e) {}
	}
	// #END: class static code


	public WLAppletHandler() {}

	private static AppletsFrame getFrame(int threadNum)
	{
		Integer threadID = new Integer(threadNum);

		AppletsFrame frame = (AppletsFrame)htFrames.get(threadID);
		if (frame==null) {
			frame = new AppletsFrame("thread#" + threadID + " - WebLOAD LG Frame.");
			frame.setSize(500,200);
			//  @@dudu : I don't know why :
			//          1. SetVisible (true + false) must be done at first time.  According to yahali,
			//                 it is done to set some values.
			//          2. I don't know why the lock should be done for both setVisible. When it was not
			//                 there it get stuck.   for example, when calling showAppletsViewer() it
			//                 does not happen.
			synchronized (htFrames) {
			  frame.setVisible(true);
			  frame.setVisible(false);
			  htFrames.put(threadID,frame);
			}
		}
		return frame;
	}

	private static WebLoadClassLoader
		getClassLoader(URL codeBase, String threadData)
			throws MalformedURLException
	{
		String codeBaseName = codeBase.toString();
	  WebLoadClassLoader loader = (WebLoadClassLoader)htClassLoaders.get(codeBaseName);
		if (loader==null) {
		  synchronized (htClassLoaders) {
				// need to check it again since other VC could have create this classloader
				loader = (WebLoadClassLoader)htClassLoaders.get(codeBaseName);
				if (loader == null) {
					// @@dudu : need to be checked
		  loader = new WebLoadClassLoader(new String[] {"."}, codeBase);
			loader.addClassFilesSearchPath(codeBaseName);
					// the classloader which is common to many VCs use one http client for its get.
					loader.setThreadData(threadData);
			htClassLoaders.put(codeBaseName, loader);
		}
		  }
		}
		return loader;
	}

	public static Applet createApplet(String  appletClassName,
																		String  fileClassPath,
																		String  appletDocumentBase,
																		String  params,
																		int 	  threadNum,
																		String  threadData)
	throws ClassNotFoundException,
		   MalformedURLException,
		   InstantiationException,
		   IllegalAccessException,
		   Throwable
	{
		//
		// load the applet and prepare for running it...
		if (Log.isLogging()) Log.logCategory.debug("createApplet("+appletClassName+")");

		Applet applet = null;

		try{
			URL codeBase = new URL(fileClassPath);
			URL docBase = new URL(appletDocumentBase);

			AppletsFrame frame = getFrame(threadNum);
			// @@dudu : why need to update the threadData again and again in the javaThreadWrapper ?
			frame.setJavaWrapper(threadNum, threadData);
			// @@dudu : thread data already have the javaThreadData.  why to have it saved twice ?
			//    threadData is the place to save everything need for the thread.
			//   neeed to change names : threadData is the c++ for the c++ context.  need to find new
			//   name for this "new threadData" mechanism.
			__defaultPlayerStub.createThreadData(threadNum, frame.getJavaThreadWrapper());

		  WebLoadClassLoader loader = getClassLoader(codeBase, threadData);

			Class appletClass = loader.loadClass(appletClassName);
			applet = (Applet)appletClass.newInstance();

			WebLoadAppletStub wlAppletStub = new WebLoadAppletStub(docBase, codeBase);

			wlAppletStub.addParameter("__wlThreadData", threadData);
			StringTokenizer st = new StringTokenizer(params, "\001");
			while (st.hasMoreTokens())
			{
				//System.out.println(st.nextToken());
				String key = st.nextToken();
				String value = st.nextToken();
				if (value.compareTo("$WLEmptyString")==0)
					value = "";
				wlAppletStub.addParameter(key, value);
			}

			applet.setStub(wlAppletStub);

			frame.add(applet);
		  int width = wlAppletStub.getIntParameter("width",100);
		  int height = wlAppletStub.getIntParameter("height",200);
			if (Log.isLogging()) Log.logCategory.debug("createApplet("+appletClassName+"): setting size("+width+","+height+")");
			applet.setSize(width,height);
		}
		catch(Throwable e) {
			if (Log.isLogging()) Log.logCategory.error("createApplet("+appletClassName+"): Exception occured");
			e.printStackTrace();
			//throw new RVException(e);
			throw e;
		}

                // @@dudu : why ?
		System.gc();

		if (Log.isLogging()) Log.logCategory.debug("createApplet("+appletClassName+"): Done...");
		return applet;
	}

	public static void initApplet(int threadNum, Applet applet)
		throws Throwable
	{
		String className = applet.getClass().getName();

		try {

			if (Log.isLogging()) Log.logCategory.debug("initApplet("+className+")");

			AppletsFrame frame = getFrame(threadNum);
			frame.getJavaThreadWrapper().SetTimer(className + ".init(Timer)");

		  if (Log.isLogging()) Log.logCategory.debug("initApplet("+className+"): before applet.init()");

			applet.init();

		  if (Log.isLogging()) Log.logCategory.debug("initApplet("+className+"): before SendTimer");

			frame.getJavaThreadWrapper().SendTimer(className + ".init(Timer)");

		  if (Log.isLogging()) Log.logCategory.debug("initApplet("+className+"): before addTree");

			//Register the applet in the player with the ThreadNum.
			//PlayerStub.getSystemPlayerStub().addTree(threadNum, applet); //@@Eyal: 06.01.02
			__defaultPlayerStub.addTree(threadNum, applet, frame.getJavaThreadWrapper());

		  if (Log.isLogging()) Log.logCategory.debug("initApplet("+className+"): before setActive");

		  //
			// set the active status on.
			WebLoadAppletStub wlAppletStub = (WebLoadAppletStub)applet.getAppletContext();
			wlAppletStub.setActiveStatus(true);

		  if (Log.isLogging()) Log.logCategory.debug("initApplet("+className+"): end AppletInit");

		}
		catch(Throwable e) {
			if (Log.isLogging()) Log.logCategory.error("initApplet("+className+"): Exception occured");
		  stopApplet(threadNum, applet);
			e.printStackTrace();
			//throw new RVException(e);
			throw e;
		}
	}

	public static void startApplet(int threadNum, Applet applet)
	{
		if (Log.isLogging()) {
			String className = applet.getClass().getName();
		  if (Log.isLogging()) Log.logCategory.debug("startApplet("+className+")");
		}
		applet.start();

		AppletsFrame frame = getFrame(threadNum);
		frame.refreshAppletsViewer();
	}

	public static void stopApplet(int threadNum, Applet applet) throws RVRunTimeException
	{
		String className = applet.getClass().getName();

		// Flush timers
		__defaultPlayerStub.flushTimers(threadNum);

		if (Log.isLogging()) Log.logCategory.debug("stopApplet("+className+")");

		WebLoadAppletStub appletStub = (WebLoadAppletStub)applet.getAppletContext();
		appletStub.setActiveStatus(false);
		if (Log.isLogging()) Log.logCategory.debug("stopApplet("+className+") calling stop");
		applet.stop();
		if (Log.isLogging()) Log.logCategory.debug("stopApplet("+className+") removing applet from frame");
		AppletsFrame frame = getFrame(threadNum);
		frame.remove(applet);
		frame.refreshAppletsViewer();
		if (Log.isLogging()) Log.logCategory.debug("stopApplet("+className+") calling destroy");
		applet.destroy();

		//Un-Register the applet from the player (with the ThreadNum??)
		//PlayerStub.getSystemPlayerStub().removeTree(threadNum, applet);
		__defaultPlayerStub.removeTree(threadNum, applet);
		if (Log.isLogging()) Log.logCategory.debug("stopApplet("+className+") Done...");
	}

	public static void showAppletsViewer(int threadNum)
	{
		AppletsFrame frame = getFrame(threadNum);
		frame.showAppletsViewer();
	}
/*
	public static void setJavaWrapper(int threadNum, JavaThreadWrapper javaThreadWrapper)
	{
		AppletsFrame frame = getFrame(threadNum);
		frame._javaThreadWrapper = javaThreadWrapper;
	}
*/
	public static void hideAppletsViewer(int threadNum)
	{
		AppletsFrame frame = getFrame(threadNum);
		frame.hideAppletsViewer();
	}

	public String toString()
	{
		return "WebLoad Applet Handler, Version 1.0 - in loadengine";
	}

}

