/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
package com.radview.loadengine.appletloader.system;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;

// Radview classes
import com.radview.util.log.Log;
import com.radview.loadengine.JavaThreadWrapper;

/**
 * Title:        Application Player
 * Description:  An application that plays events that are read from a file.
 * Copyright:    Copyright (c) 2001
 * Company:      Radview
 * @author Eyal Shalom
 * @version 1.0
 */


public class AppletsFrame extends Frame
{
  protected int width;
  protected int height;
  protected int xLocation = 0;
  protected int yLocation = 0;
	protected boolean shown = false;

	//
	// @@Yahali : temporary ...
	private JavaThreadWrapper _javaThreadWrapper = null;

	private static AppletsFrameListener myListener = null;

	static {
		myListener = new AppletsFrameListener();
	}

  public AppletsFrame(String title)
  {
    super(title);
    addWindowListener(myListener);
  }

	public void setJavaWrapper(int threadNum, String threadData)
	{
		if (_javaThreadWrapper == null) {
			_javaThreadWrapper = new JavaThreadWrapper(threadNum, threadData);
		} else {
			_javaThreadWrapper.setThreadData(threadData);
		}
	}

	public JavaThreadWrapper getJavaThreadWrapper() {
		return _javaThreadWrapper;
	}

	public void refreshAppletsViewer() {
		if (shown) {
			arrangeApplets();
			repaint();
		}
	}

	public void showAppletsViewer() {
		if (!shown) {
		  shown = true;
		  setVisible(true);
		}
		refreshAppletsViewer();
	}

	public void hideAppletsViewer() {
		if (shown) {
			shown = false;
			setVisible(false);
		}
	}

	boolean isAppletsViewerShown() {
		return shown;
	}

	public void arrangeApplets() {
		int cc = getComponentCount();
		int xPos = 0;
		int yPos = 30;
		int maxHeight = 0;
		int frameWidth = 0;
		int frameHeight = 0;
		for (int i=0;i<cc;i++)
		{
			Applet applet = (Applet)getComponent(i);
			if (Log.isLogging()) Log.logCategory.debug("AppletsFrame.arrangeApplets("+applet.getClass().getName()+"): setting location("+xPos+","+yPos+")");
			applet.setLocation(xPos,yPos);
			Dimension d = applet.getSize();
			xPos += d.width;
			frameWidth = Math.max(frameWidth, xPos);
			frameHeight = Math.max(frameHeight, yPos+d.height);
			maxHeight = Math.max(maxHeight,d.height);
			if (xPos>800) {
				yPos += maxHeight;
				xPos=0;
				maxHeight=0;
			}
		}
		setSize(frameWidth+20,frameHeight+20);
	}
	/*
  public void viewApplet(int width, int height)
  {
    this.width = width;
    this.height = height;


    if(stub != null) {
    applet.init();
    applet.start();
    }

    add("Center", applet);
    setSize(width, height);
    setLocation(getLocationPoint());
    setVisible(true);
  }

  public void disposeApplet()
  {
    hide();
    dispose();
    xLocation = 0;
    yLocation = 0;
    applet = null;
  }

  protected void createMenu()
  {
    MenuBar m = new MenuBar();
    super.setMenuBar(m);

    Menu fileMenu = new Menu("Applet");
    m.add(fileMenu);

    MenuItem mi = new MenuItem("Open Applet");
    mi.addActionListener(new MenuItemListener());
    fileMenu.add(mi);

    mi = new MenuItem("Restart Applet");
    mi.addActionListener(new MenuItemListener());
    fileMenu.add(mi);

    mi = new MenuItem("Stop Applet");
    mi.addActionListener(new MenuItemListener());
    fileMenu.add(mi);

    mi = new MenuItem("Exit");
    mi.addActionListener(new MenuItemListener());
    fileMenu.add(mi);
  }

  protected synchronized static Point getLocationPoint()
  {
    Point p = new Point(xLocation, yLocation);

    xLocation += width;
    if(xLocation >= Toolkit.getDefaultToolkit().getScreenSize().getWidth() - 100) {
      xLocation = 0;
      yLocation += (height + 20);
    }
    //if(yLocation += 200 > Toolkit.getDefaultToolkit().getScreenSize().getHeight()) {
    //  yLocation = 0;
    //}

    return p;
  }

  class MenuItemListener implements java.awt.event.ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      System.out.println(e.toString());
      MenuItem im = (MenuItem)e.getSource();

      if( im.getLabel().equals("Restart Applet") ) {
				try {
					Container c = applet.getParent();
					Class appletClass = applet.getClass();
					applet.stop();
					applet.destroy();

					c.remove(applet);
					applet = (Applet)appletClass.newInstance();
					c.add("Center", applet);
					applet.init();
					applet.start();
					c.setVisible(true);
				}
				catch(Exception exp) {
					exp.printStackTrace();
				}
				return;
      }

      if( im.getLabel().equals("Stop Applet") ) {
        applet.stop();
				applet.destroy();
				return;
      }

      if( im.getLabel().equals("Exit") ) {
        applet.stop();
				applet.destroy();
				System.exit(0);
      }
    }
  }
*/

}

class AppletsFrameListener extends WindowAdapter {
  public void windowClosing(WindowEvent e) {
		e.getWindow().hide();
  }
}

