/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
package com.radview.loadengine.appletloader.system;

import java.net.*;
import java.io.*;
import java.applet.*;
import java.util.Enumeration;
import sun.misc.CompoundEnumeration;

// Radview classes
import com.radview.util.log.Log;
import com.radview.httpload.*;
import com.radview.loadengine.appletloader.*;

/**
 * #public class wlapplet.wlsystem.WebLoadClassLoader
 *
 **/
public class WebLoadClassLoader extends URLClassLoader
{
	private JarResourcesStreamHandler __jarResourcesStreamHandler;

	private long 			    m_refID = 0;

	Object                m_refCountGuard = new Object();
	private static long 	m_refCount = 0;
	private JarResources	m_JarResources;
	private URL           m_CodeBase;
  private final char 	  classNameReplacementChar = '/';

	public String         m_ThreadData;

	public void setThreadData(String threadData) {
		m_ThreadData = threadData;
	}

	// #END: class static block
	protected long advanceRefCount()
	{
		synchronized (m_refCountGuard) {
		return ++m_refCount;
	}
	}

	public WebLoadClassLoader(String fileClassPathList[], URL codeBase)
	throws MalformedURLException
	{
		// @@dudu : need to check the classpath list issue and the parent issue
		super(FilePathToURLConverter.filePathListToURLList(fileClassPathList));

		__jarResourcesStreamHandler = new JarResourcesStreamHandler(this);

		m_JarResources = JarResources.getSystemJarResources();
		m_refID = advanceRefCount();
		m_CodeBase = codeBase;

		if (Log.isLogging()) Log.logCategory.debug("WebLoadClassLoader #" + m_refID);
	}

	public void addClassFilesSearchPath(String classPath)
	throws MalformedURLException
	{
		addURL( (new File(classPath)).toURL() );
	}

	public String toString()
	{
		return "com.radview.loadengine.appletloader.system.WebLoadClassLoader" + m_refID;
	}

	public Class findClass(String className)
		throws ClassNotFoundException
	{
		Class   result;
		byte[]  classBytes;

		if (Log.isLogging()) Log.logCategory.debug("ClassLoader:" + m_refID + ", findClass: " + className);
		//
		// Changing the class name from x.y.z to x/y/z path format
		String classPathName = formatClassName(className);
		//----- Try to load it from preferred source
		// Note loadStreamBytes() is an abstract method
		classBytes = loadStreamBytes(classPathName);
		if (classBytes == null) {
			//
			// Since the Class Loader is one per CodeBase, synchronizing on
			// "this" should be enough to make sure only one thread does the
			// http jni call.
			//synchronized (m_JarResources) {
			synchronized (this) {
				//
				// verify again, that the resource was not entered in yet.
				classBytes = loadStreamBytes(classPathName);
				if (classBytes == null) {
					URL classURL;
					try {
						classURL = new URL(m_CodeBase, classPathName);
					} catch (MalformedURLException ex) {
						ex.printStackTrace();
						ClassNotFoundException e = new ClassNotFoundException(ex.toString());
						throw e;
					}
					if (Log.isLogging()) Log.logCategory.debug("ClassLoader:" + m_refID + ", Findclass, getStream: " + classURL);
					byte[] classStream = HttpServices.jniGetStream(m_ThreadData, classURL.toString());
					if (Log.isLogging()) Log.logCategory.debug("classStream len: " + classStream.length);
					if (classStream.length > 0) {
						m_JarResources.addClassResource(classPathName, m_CodeBase, classStream);
						classBytes = classStream;
					}
				}
			}
			if (classBytes == null) {
				ClassNotFoundException e = new ClassNotFoundException();
				e.printStackTrace();
				throw e;
			}
		}

		//----- Define it (parse the class file)
		try {
			result = defineClass(className, classBytes, 0, classBytes.length);
		} catch (ClassFormatError e) {
			if (Log.isLogging()) Log.logCategory.debug("WebLoadClassLoader:" + m_refID + ", findClass: " + className + " Exception in defineClass.");
			e.printStackTrace();
			throw e;
		}

		if (result == null) {
			throw new ClassFormatError();
		}

		return result;
	}

	//  @@dudu : do I need here synch ?
  public URL findResource(String name) {
		if (Log.isLogging()) Log.logCategory.debug("ClassLoader:" + m_refID + ", findResource: " + name);

		if (name.equals(""))
			return null;
		else {
		  	if (loadStreamBytes(name) != null) {
					try {
			      return new URL("webload","JarResources",1000,name,__jarResourcesStreamHandler);
					}
					catch (java.net.MalformedURLException e) {
						if (Log.isLogging()) Log.logCategory.debug("ClassLoader:" + m_refID + ", findResource: " + name + " Exception : " + e);
						return null;
					}
		  	}
				else {
					URL url;
					try {
						url = new URL(m_CodeBase, name);
						return url;
					} catch (java.net.MalformedURLException e) {
						if (Log.isLogging()) Log.logCategory.debug("ClassLoader:" + m_refID + ", findResource: " + name + " Exception : " + e);
						return null;
					}
				}
		}

  }

	// @@dudu : need to be implemented
	//  @@dudu : do I need here synch ?
	public Enumeration findResources(String name) throws IOException {
		if (Log.isLogging()) Log.logCategory.debug("ClassLoader:" + m_refID + ", findResources: " + name);
	  return new CompoundEnumeration(new Enumeration[0]);
	}

  protected byte[] loadStreamBytes (String className)
	{
		//className = formatClassName(className);
		return (m_JarResources.getResource (className, m_CodeBase));
	}

	protected String formatClassName(String className)
	{
		return className.replace('.', classNameReplacementChar) + ".class";
	}
}


/**
 * class FilePathToURLConverter
 *
 **/
class FilePathToURLConverter
{
	static public URL[] filePathListToURLList(String filePathList[])
	throws MalformedURLException
	{
		if(filePathList.length > 0) {
			URL urlList[] = new URL[filePathList.length];

			// Create an array of URLs
			// from directory names specified in filePathList
			for(int i = 0; i < filePathList.length; i++) {
				urlList[i] = (new File(filePathList[i])).toURL();
			}

			return urlList;
		}
		return null;
	}

	static public URL filePathToURL(String filePath)
	throws MalformedURLException
	{
		if(filePath != null) {
			return (new File(filePath)).toURL();
		}
		return null;
	}
}
