/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <RVCTokenizer.h>

#include "generalprocess.h"
#include "wlleprotocol.h"
#include "tracedll.h"
#include "wldlls.h"
#include "loadenginevariablesdata.h"
#include "loadengineglobals.h"
#include "loadengineinifileproperties.h"
#include "loadengineselistener.h"
#include "platform.h"
#include "wljsruntime.h"
#include "tvtables.h"
#include "ProtocolAutoDiscovery.h"
#include "gt_unicode.h"
#define WLJSNAME_HTTP_CLIENT_TYPE       "ClientType"

//####### The implementation of the class CGeneralProcess   #########

CGeneralProcess* CGeneralProcess::msSingleton = NULL;

CGeneralProcess& CGeneralProcess::fsGetSingleton()
{
	WlAssert(msSingleton);
	return *msSingleton;
}

CGeneralProcess::CGeneralProcess() 
: 
	m_MasterPid(-1),
	m_LoadJSRuntime(NULL),
	mJVMManager(*this),
  m_InitPerformed(false)
{
	WlAssert( !msSingleton );
	msSingleton = this;
}

CGeneralProcess::~CGeneralProcess(){
}

bool CGeneralProcess::Init(RVCString& oDllName) {

  try {
		LE_SET_SE_TRANSLATOR();

		oDllName = "";

		m_PropIndex.RegisterLEProperties();//@@avidan1
		m_TempDirectory = CArrangeMachineTempFiles::GetTempDirectory();
		
		RVCString* DllNames[1000];//contain the dll's name that are in the ini file.
		int indx = 0;
		const char* constProtocols = CLEIniFileProperties::Protocols;
		RVCString delimiter(";");
		if(constProtocols) {
			RVCString protocols(constProtocols);
			RVCTokenizer next(protocols);   
			RVCString token;                // Will receive each token
		
			// Advance until the null string is returned:
			while (!(token=next(delimiter)).isNull()) {
				RVCString* nameDll = new RVCString(token);
				DllNames[indx] = nameDll;
				indx++;
			}
		} else {
			WlErrorMsg("Error 0 in LoadEngine in function CGeneralProcess::Init()");
			WlAssert(0);
		}
		WLProtocolIndex a_NumProtocols =indx;

		RVCString* DllNamesForLogMessage[1000];//contain the dll's name that are in the ini file.
		indx = 0;
		const char* constProtocolsNamesForLogMessage = CLEIniFileProperties::ProtocolsNames;
		RVCString delimiter2(";");
		if(constProtocolsNamesForLogMessage) {
			RVCString protocolsNamesForLogmessage(constProtocolsNamesForLogMessage);
			RVCTokenizer next2(protocolsNamesForLogmessage);   
			RVCString token2;                // Will receive each token
		
			// Advance until the null string is returned:
			while (!(token2=next2(delimiter2)).isNull()) {
				RVCString* nameDllForLogMessage = new RVCString(token2);
				DllNamesForLogMessage[indx] = nameDllForLogMessage;
				indx++;
			}
		} else {
			WlErrorMsg("Error 1 in LoadEngine in function CGeneralProcess::Init()");
			WlAssert(0);
		}
		
		if(indx != a_NumProtocols ){
			WlErrorMsg("Error :In the WebLoad.ini the number of 'Protocols_Names' should be same as 'Protocols'. ");
			WlAssert(0);
		}

		// Hryvach: added dll autodiscovey function
		CProtocolAutoDiscovery autoDiscovery;
		if( !autoDiscovery.discoverPath( CLEIniFileProperties::ProtocolExtensionsPath, "*.xml", true, DllNames, DllNamesForLogMessage, &indx, 1000 ) ){
			WlErrorMsg( "%s", autoDiscovery.getLastError() );
		}
		else
			a_NumProtocols = indx;

		m_ProtocolProcessContainer.InitProtocolContainer(a_NumProtocols);
		{for (int indx = 0;indx < a_NumProtocols;indx++) {
			WlTraceMsg("CGeneralProcess::Init DllNames[indx] %s", (LPCSTR)*DllNames[indx]);
			cDll* DLL = new cDll(*DllNames[indx], *DllNamesForLogMessage[indx]);
			if (!DLL) WlErrorMsg("CGeneralProcess::Init ERROR");
			m_DllProtocols.append(DLL);
		}}

		//the dll's names are not needed anymore
		{for( int indx =0;indx < a_NumProtocols; ++indx ) {
			delete DllNames[indx];
			delete DllNamesForLogMessage[indx];
		}}

		// is JS GC in execution context
		mIsJSGCInExecutionContext = !CLEIniFileProperties::GCThreadOnly;

		mJVMManager.fInit();

		if (!InitProtocols(a_NumProtocols, oDllName)){
			WlErrorMsg("Failed to init the following protocols: %s", oDllName.data());
			return false;
		}

		m_LoadJSRuntime = new CJSLoadRuntime(CLEIniFileProperties::RuntimeSize, *this );

		m_InitPerformed = true;
		return true;
	}

  catch (std::exception* e) {
		ExceptionHandler(e);
		delete e; // Only for catch of pointer 
		return false;
  }
  catch (std::exception& e) {
		ExceptionHandler(&e);
		return false;
  }
  catch (...) {
    WlErrorMsg("Internal Error : Unknown occurred in CGeneralProcess::Init");
		return false;
	}	
}

bool CGeneralProcess::InitProtocols(WLProtocolIndex p_NumProtocols, RVCString & oDllName) {

	bool ret(true);
	RVCString dllFuncEntryPoint("NewProtocolProcess");

	PFN_NewProtocolProcces pfn_NewProtocolProcces;

	for(int indx = 0 ;indx < p_NumProtocols;indx++) {//for each protocol
		cDll* DllToLoad = static_cast<cDll*>(m_DllProtocols[indx]);
			
		RVCString aErrorDescription;
		try {
			if(!DllToLoad->Load(aErrorDescription)) {//load the protocol dll 
				//WlAssertMsg(0)( aErrorDescription.data() );
			ret = false;
			break;
		}
		}
		catch (...) {
			WlErrorMsg("CGeneralProcess::InitProtocols exception in loading the dll %s", DllToLoad->GetNameDll());
			continue;
		}
	
		FARPROC f = (static_cast<cDll*>(m_DllProtocols[indx]))->fGetProcAddrss(dllFuncEntryPoint);//get the pointer of the dll function 'NewProtocolProcces'
		pfn_NewProtocolProcces = (PFN_NewProtocolProcces)f;
		if(pfn_NewProtocolProcces != NULL ) {
		 //activate the function 'NewProtocolProcces' and keep the returned pointer.
      m_ProtocolProcessContainer.InsertProtocolAtIndex(indx, *pfn_NewProtocolProcces());
		}
		else {
			if(oDllName.length() > 0)
				oDllName += ", ";
			oDllName += ((cDll*)m_DllProtocols[indx])->GetNameDllForLogMessage();
			char Msg[400];
			sprintf(Msg, "Error, Failed when trying to call the entry function of the protocol %s.",oDllName.data()); 
			WlErrorMsg(Msg);

			ret = false;
			break;
		}			
	}

  if (ret) ret = ProtocolProcessInit(oDllName);

  return ret;
}

int CGeneralProcess::GetMasterPid() {
	if(m_MasterPid == -1)
		WlAssert(0);
	return m_MasterPid;
}

void CGeneralProcess::SetMasterPid( int masterPid) {
	if( m_MasterPid != -1 )//it was already set
		WlAssert(0);
	m_MasterPid = masterPid;
}

WLProtocolIndex CGeneralProcess::GetNumProtocols() {
	return m_ProtocolProcessContainer.NumProtocols();
}

cDLLContainer& CGeneralProcess::GetDllProtocols(){
	return m_DllProtocols;
}

iLEProtocolContainer& CGeneralProcess::GetLEProtocolProcessContainer() {
	return m_ProtocolProcessContainer;
}

bool CGeneralProcess::ProtocolProcessInit(RVCString & oDllName){
	for (WLProtocolIndex indx = 0;indx < m_ProtocolProcessContainer.NumProtocols();indx++) {

    iLEProtocolProcess& CurProtocolProcess = static_cast<iLEProtocolProcess&>(m_ProtocolProcessContainer.GetProtocolAtIndex(indx));
    CurProtocolProcess.InitProtocol(indx, this);

    if (!CurProtocolProcess.ProcessInitialize()){

		  WlTraceMsg("Error 3 in function CGeneralProcess::ProtocolProcessInit");
		  if(! oDllName.contains(((cDll*)m_DllProtocols[indx])->GetNameDllForLogMessage()) ) {
			  if(oDllName.length() > 0) 
				  oDllName += ", ";
			  oDllName += ((cDll*)m_DllProtocols[indx])->GetNameDllForLogMessage();
		  }
		  return false;
		}
	}
  return true;
}

//Not used?
void CGeneralProcess::ProtocolProcessTermination(){
	WlTraceMsg("Calling ProtocolProcessTermination for all protocols");
  for (WLProtocolIndex indx = m_ProtocolProcessContainer.NumProtocols() - 1;indx >= 0;indx--){
    WlAssert(dynamic_cast<iLEProtocolProcess*>(&(m_ProtocolProcessContainer.GetProtocolAtIndex(indx))));
    if (!static_cast<iLEProtocolProcess&>(m_ProtocolProcessContainer.GetProtocolAtIndex(indx)).ProcessTermination()){
		  WlTraceMsg("Error 0 in function CGeneralProcess::ProtocolProcessTermination");
		}
  }
}  

void CGeneralProcess::ProcessInitializeJava(cRVJVM& rJvm, JNIEnv* rEnv){
	WlTraceMsg("Calling ProcessInitializeJava for all protocols");
  for (WLProtocolIndex indx = 0;indx < m_ProtocolProcessContainer.NumProtocols();indx ++){
    WlAssert(dynamic_cast<iLEProtocolProcess*>(&(m_ProtocolProcessContainer.GetProtocolAtIndex(indx))));
    if (!static_cast<iLEProtocolProcess&> (m_ProtocolProcessContainer.GetProtocolAtIndex(indx)).ProcessInitializeJava(rJvm, rEnv)){
		  WlTraceMsg("Error 0 in function CGeneralProcess::ProcessInitializeJava");
		}
  }
}  

void CGeneralProcess::ProcessFinalizeJava(cRVJVM& rJvm, JNIEnv* rEnv){
	WlTraceMsg("Calling ProcessFinalizeJava for all protocols");
  for (WLProtocolIndex indx = m_ProtocolProcessContainer.NumProtocols() - 1;indx >= 0;indx--){
    WlAssert(dynamic_cast<iLEProtocolProcess*>(&(m_ProtocolProcessContainer.GetProtocolAtIndex(indx))));
    if (!static_cast<iLEProtocolProcess&> (m_ProtocolProcessContainer.GetProtocolAtIndex(indx)).ProcessFinalizeJava(rJvm, rEnv)){
		  WlTraceMsg("Error 0 in function CGeneralProcess::ProcessFinalizeJava");
		}
  }
}  

void CGeneralProcess::ProtocolProcessAfterStartLoad(int ReportResolutionSize){
	WlTraceMsg("Calling ProcessAfterStartLoad for all protocols");
  for (WLProtocolIndex indx = m_ProtocolProcessContainer.NumProtocols() - 1;indx >= 0;indx--){
    WlAssert(dynamic_cast<iLEProtocolProcess*>(&(m_ProtocolProcessContainer.GetProtocolAtIndex(indx))));
    if (!static_cast<iLEProtocolProcess&> (m_ProtocolProcessContainer.GetProtocolAtIndex(indx)).ProcessAfterStartLoad(ReportResolutionSize)){
		  WlTraceMsg("Error 0 in function CGeneralProcess::ProcessAfterStartLoad");
		}
  }
}

//Not used?
void CGeneralProcess::Terminate() {
  ProtocolProcessTermination();

 // m_LoadJSRuntime->Terminate();
  
	if( m_LoadJSRuntime ) {
		delete m_LoadJSRuntime; m_LoadJSRuntime = 0;
	}
#ifdef _DEBUG
	TVSTerminate();
#endif
}

void CGeneralProcess::fThreadStart( 
	cLEJSMachineThreadKey& rKey,
	iLEJSMachineThread& rMThread )
{
	RVReadersWriterLock::WriteLockGuard aLockGuard( mThreadsDictLock );
	RVCollectable* aStatus = mThreadsDict.insertKeyAndValue( &rKey, &rMThread );
	WlAssert(aStatus); // already exist. either inserted twice, or terminated and restarted (thread pooling)
}


void CGeneralProcess::fThreadEnd(
	cLEJSMachineThreadKey& rKey )
{
	RVReadersWriterLock::WriteLockGuard aLockGuard( mThreadsDictLock );
	RVCollectable* aStatus = mThreadsDict.remove( &rKey );
	WlAssert( aStatus ); // not found
}
	

iLEJSMachineThread* CGeneralProcess::fGetifThread( 
	cLEJSMachineThreadKey& rKey )
{
	RVReadersWriterLock::ReadLockGuard aLockGuard( mThreadsDictLock );
	RVCollectable* aMThread_ = mThreadsDict.findValue( &rKey );
	iLEJSMachineThread* rtn;
	if( aMThread_ ) {
	WlAssert(dynamic_cast<iLEJSMachineThread*>(aMThread_)); // bad downcast
		rtn = static_cast<iLEJSMachineThread*>(aMThread_); 
	} else {
		rtn = NULL;
	}
	return rtn;
}

iLEJSMachineThread& CGeneralProcess::fGetThread( 
	cLEJSMachineThreadKey& rKey )
{
	iLEJSMachineThread* rtn = fGetifThread( rKey );
	WlAssert( rtn ); // not found
	return *rtn;
}

iLEJSMachineThread* CGeneralProcess::fGetifCurrentThread()
{
  	
  	cLEJSMachineThreadKey thrId( ::rvThreadId() );
	return fGetifThread( thrId );
}

iLEJSMachineThread& CGeneralProcess::fGetCurrentThread()
{
	iLEJSMachineThread* rtn = fGetifCurrentThread();
	WlAssert( rtn ); // not found
	return *rtn;
}

cRVJVMManager& CGeneralProcess::fGetJVMManager()
{
	return mJVMManager;
}


bool CGeneralProcess::fIsJSGCInExecutionContext()
{
	return mIsJSGCInExecutionContext;	
}

double CGeneralProcess::vGetCurrentTime()
{
	return TVPSysTime();
}

void CGeneralProcess::ExceptionHandler(std::exception* e){
	RVCString FullExcName ("");
	WLERRGetErrorMsg(e, FullExcName);
	RVCString StackTrace("");
	if (WLERRGetStackTrace(e, StackTrace) && StackTrace.length() > 0){
		WlErrorMsg("CGeneralProcess::Init - Exception %s occured\nTrace:\n%s", FullExcName, StackTrace);
		printf("CGeneralProcess::Init - Exception %s occured\nTrace:\n%s", FullExcName, StackTrace);
	}
	else {
		WlErrorMsg("CGeneralProcess::Init - Exception %s occured", FullExcName);
		printf("CGeneralProcess::Init - Exception %s occured", FullExcName);
	}
}


//####### The implementation of the class CPropIndex    #########


CPropIndex::CPropIndex() {}
CPropIndex::~CPropIndex() {}

void CPropIndex ::RegisterLEProperties ()
{
  CLEPropertyValue DefVal ;

	//DisableSleep
	m_IndexDisableSleep = 
		CLEVariablesData::RegisterProperty(
				WLJSNAME_LE_DISABLE_SLEEP,						//Name
				CLEPropertyValue::PT_BOOLEAN,					//Type
				DefVal.Set(false),										//Default Value,
				"",																		//List of values
				CLEPropertyValue::PL_GLOBALS,					//Start level
				CLEPropertyValue::PL_GLOBALS					//End level
				);

	//SleepRandomMin
	m_IndexSleepRandomMin = 
		CLEVariablesData::RegisterProperty(
				WLJSNAME_LE_SLEEP_RANDOM_MIN,										//Name
				CLEPropertyValue::PT_INT,												//Type
				DefVal.Set((long)DEFAULT_LE_SLEEP_RANDOM_MIN),	//Default Value,
				"",																							//List of values
				CLEPropertyValue::PL_GLOBALS,										//Start level
				CLEPropertyValue::PL_GLOBALS										//End level
				);

	//SleepRandomMax
	m_IndexSleepRandomMax = 
		CLEVariablesData::RegisterProperty(
				WLJSNAME_LE_SLEEP_RANDOM_MAX,										//Name
				CLEPropertyValue::PT_INT,												//Type
				DefVal.Set((long)DEFAULT_LE_SLEEP_RANDOM_MAX),	//Default Value,
				"",																							//List of values
				CLEPropertyValue::PL_GLOBALS,										//Start level
				CLEPropertyValue::PL_GLOBALS										//End level
				);

	//SleepDeviation
	m_IndexSleepDeviation = 
		CLEVariablesData::RegisterProperty(
				WLJSNAME_LE_SLEEP_DEVIATION,										//Name
				CLEPropertyValue::PT_INT,												//Type
				DefVal.Set((long)DEFAULT_LE_SLEEP_DEVIATION),		//Default Value,
				"",																							//List of values
				CLEPropertyValue::PL_GLOBALS,										//Start level
				CLEPropertyValue::PL_GLOBALS										//End level
				);
 
	//ExceptionEnabled
	m_IndexExceptionEnabled = 
		CLEVariablesData::RegisterProperty(
				WLJSNAME_LE_EXCEPTION_ENABLED,					//Name
				CLEPropertyValue::PT_BOOLEAN,					//Type
				DefVal.Set(false),								//Default Value,
				"",												//List of values
				CLEPropertyValue::PL_GLOBALS,					//Start level
				CLEPropertyValue::PL_GLOBALS					//End level
				);

	//ReportCSJSInternalErrors
  // default is true so messages for missing internal files will be displayed
	m_ReportCSJSInternalErrors = 
		CLEVariablesData::RegisterProperty(
				WLJSNAME_LE_REPORT_CSJS_INTERNAL_ERRORS,					//Name
				CLEPropertyValue::PT_BOOLEAN,					//Type
				DefVal.Set(true),								//Default Value,
				"",												//List of values
				CLEPropertyValue::PL_GLOBALS,					//Start level
				CLEPropertyValue::PL_GLOBALS					//End level
				);

	m_IndexTimeoutBeforeOperation = 
		CLEVariablesData::RegisterProperty(
				WLJSNAME_LE_TIMEOUT_BEFORE_OPERATION,			       //Name
				CLEPropertyValue::PT_INT,					           //Type
				DefVal.Set((long)DEFAULT_LE_TIMEOUT_BEFORE_OPERATION), //Default Value,
				"",												       //List of values
				CLEPropertyValue::PL_GLOBALS,					       //Start level
				CLEPropertyValue::PL_GLOBALS					       //End level
				);

	m_IndexSendClientStatistics = 
		CLEVariablesData::RegisterProperty(
				WLJSNAME_LE_SEND_CLIENT_STATISTICS,			       //Name
				CLEPropertyValue::PT_BOOLEAN,					//Type
				DefVal.Set(false),								//Default Value,
				"",												//List of values
				CLEPropertyValue::PL_GLOBALS,					//Start level
				CLEPropertyValue::PL_GLOBALS					//End level
				);

	m_IndexSendClientStatisticsFilter = 
		CLEVariablesData::RegisterProperty(
				WLJSNAME_LE_SEND_CLIENT_STATISTICS_FILTER,			       //Name
				CLEPropertyValue::PT_STRING,					           //Type
				DefVal.Set(""), //Default Value,
				"",												       //List of values
				CLEPropertyValue::PL_GLOBALS,					       //Start level
				CLEPropertyValue::PL_GLOBALS					       //End level
				);

	m_IndexJVMType = 
		CLEVariablesData::RegisterProperty(
				WLJSNAME_LE_JVM_TYPE,							       //Name
				CLEPropertyValue::PT_STRING,					           //Type
				DefVal.Set("Default"), //Default Value,
				"",												       //List of values
				CLEPropertyValue::PL_GLOBALS,					       //Start level
				CLEPropertyValue::PL_GLOBALS					       //End level
				);

	m_IndexCharEncoding = 
		CLEVariablesData::RegisterProperty(
				WLJSNAME_LE_CHAR_ENCODING,							       //Name
				CLEPropertyValue::PT_INT,                       //Type
				DefVal.Set ((long)CP_DEFAULT), \
				   "(long)0,\
					(long)874,\
					(long)932,\
					(long)936,\
					(long)949,\
					(long)950,\
					(long)1200,\
					(long)1250,\
					(long)1251,\
					(long)1252,\
					(long)1253,\
					(long)1254,\
					(long)1255,\
					(long)1256,\
					(long)1257,\
					(long)2022,\
					(long)10000,\
					(long)10008,\
					(long)20866,\
					(long)28593,\
					(long)40850,\
					(long)40852,\
					(long)40862,\
					(long)40866,\
					(long)40869,\
					(long)51932,\
					(long)51949,\
					(long)65000,\
					(long)65001",                                             //List of values
				CLEPropertyValue::PL_GLOBALS,                   //Start level
				CLEPropertyValue::PL_GLOBALS                    //End level
			); 
 
	//ClientType
 // m_VarIndex.ClientType = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_CLIENT_TYPE,          //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set ("Normal"),               //Default Value,
      "Normal,Thin,Thick,Custom",         //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_GLOBALS        //End level
     ); 
  
}



