/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// VariableProperties.cpp: implementation of the CGlobalVariableProperties class.
//
//////////////////////////////////////////////////////////////////////

#include "globalvariableproperties.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGlobalVariableProperties::CGlobalVariableProperties()
{
	varValue = NULL;
	varType[0] = 'U';
	varType[1] =  0;
}

CGlobalVariableProperties::CGlobalVariableProperties(const char *newType, const char *newValue)
{
	varValue = NULL;
	varType[0] = newType[0];
	varType[1] =  0;
	CreateValue(newValue);
}

CGlobalVariableProperties::~CGlobalVariableProperties()
{
	delete(varValue);
}


//////////////////////////////////////////////////////////////////////
// Methods
//////////////////////////////////////////////////////////////////////

void CGlobalVariableProperties::SetValue(GlobalsBool newBoolValue) 
{
	varType[0] = GLOBALS_BOOL; 

	char charBool[3];
	sprintf(charBool, "%d", newBoolValue);

	CreateValue(charBool);
}

void CGlobalVariableProperties::SetValue(double newNumberValue) 
{
	varType[0] = GLOBALS_NUMBER; 

	char charNumber[30];
	sprintf(charNumber, "%.12G", newNumberValue);

	CreateValue(charNumber);
}

void CGlobalVariableProperties::SetValue(const char* newStringValue) 
{
	varType[0] = GLOBALS_STRING; 
	
	CreateValue(newStringValue);
}

bool CGlobalVariableProperties::GetBoolValue(GlobalsBool& getBoolValue)
{
	if(IsBoolType()) {
		getBoolValue = atoi(varValue);
		return true;
	}
	return false;
}

bool CGlobalVariableProperties::GetNumberValue(double& getNumberValue)
{
	if(IsNumberType()) {
		getNumberValue = atof(varValue);
		return true;
	}
	return false;
}

bool CGlobalVariableProperties::GetStringValue(char*& getStringValue)
{
	if(IsStringType()) {
		getStringValue = varValue;
		return true;
	}
	return false;
}

bool CGlobalVariableProperties::IsBoolType(void)
{
	return (varType[0] == GLOBALS_BOOL); 
}

bool CGlobalVariableProperties::IsNumberType(void)
{
	return (varType[0] == GLOBALS_NUMBER); 
}

bool CGlobalVariableProperties::IsStringType(void)
{
	return (varType[0] == GLOBALS_STRING); 
}

inline void CGlobalVariableProperties::CreateValue(const char* value)
{
	delete varValue;

	varValue = new char[strlen(value) + 1];
	strcpy(varValue, value);
}

