/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <jni.h>
#include "rvjsapi.h"

#include "jsjavacore.h"

#include "tvenv.h"
#include "tracedll.h"
#include "wlerrorlib.h"
#include "loadexceptions.h"
#include "rvjsembed.h"
#include "rvjsmachinethread.h"
#include "generalprocess.h"
#include "loadreportinfo.h"
#include "pointertojava.h"
#include "loadengineinifileproperties.h"
#include "loadengineselistener.h"
#include "JarsAutoDiscovery.h"

#include <thr.h>
#include <thr.h>


#ifdef __TV_UNIX__
#include <dlfcn.h>
#include <errno.h>
#endif // __TV_UNIX__


cRVJVM::cRVJVM(
cRVJVMManager& rManager )
:
	mManager(rManager),
	mJVM(NULL),
	mJSJVM(NULL),
	mClassPath(),
	mJIT(),
	mJVMPath(),
  m_hJVMDLL(NULL)
{}

bool cRVJVM::fInit()
{
	mJVMStartThreadEvent.Reset();
	mJVMTerminateLiveConnectEvent.Reset();
	mJVMTerminateLiveConnectDoneEvent.Reset();
	mJVMDestroyJVMEvent.Reset();
  mJVMDestroyJVMDoneEvent.Reset();
	return true;
}

bool cRVJVM::fJSJFinal()
{
	bool rtn = true;

	if( mJVM ) {

		WlAssert(mJSJVM);

    WlTraceMsg("Entering cRVJVM::fJSJFinal");
		mJVMTerminateLiveConnectEvent.Set();
	  mJVMTerminateLiveConnectDoneEvent.Wait();
	  mJVMTerminateLiveConnectDoneEvent.Reset();
    WlTraceMsg("Exiting cRVJVM::fJSJFinal");
	}

	return rtn;
}

bool cRVJVM::fFinal()
{
	bool rtn = true;

  WlTraceMsg("Entering cRVJVM::fFinal");

	if( mJVM ) {

		WlAssert(mJSJVM);

		mJVMDestroyJVMEvent.Set();
		mJVMDestroyJVMDoneEvent.Wait();
		mJVMDestroyJVMDoneEvent.Reset();

		mJVMThread.join();
		//RVWaitStatus aWait = mJVMThread.join();
			//WlAssert( aWait == RV_THR_COMPLETED );
		}

  WlTraceMsg("Exiting cRVJVM::fFinal");

	WlAssert(!mJVM);
		WlAssert( !mJSJVM );

	return rtn;
}

cRVJVM::~cRVJVM()
{
// Ofir - Load/Free Library
	if(m_hJVMDLL) 
	{
#ifdef __TV_WINDOWS__
		FreeLibrary( m_hJVMDLL );
#elif defined __TV_UNIX__
		if(dlclose( m_hJVMDLL )) 
		{//return 0 on successs
			WlTraceMsg("Error in function cRVJVM::~cRVJVM()");
		}
#else
#error "not supported"
#endif
		m_hJVMDLL = NULL;
	}
}

bool cRVJVM::fCreateJVMInternal(
	JavaVM*& oJVM,
	JNIEnv*& oJEnv )
{

	const char* aJVMPath = NULL;
	if (CLEIniFileProperties::UseWLJVMIni) {
		aJVMPath = CLEIniFileProperties::WebLOADJVMPath;
	}

	if( !aJVMPath ) {
		#ifndef __TV_UNIX__
			aJVMPath = "jvm";
		#else
			aJVMPath = CLEIniFileProperties::LibJVMPath; // From env
		#endif
	}

#ifndef __TV_UNIX__
			m_hJVMDLL = LoadLibrary( aJVMPath );
#else
			// hJVMDLL = dlopen("/usr/java1.2/jre/lib/sparc/libjvm.so", RTLD_LAZY);
			m_hJVMDLL = dlopen( aJVMPath , RTLD_LAZY);
#endif

	if(!m_hJVMDLL) {
		WLErrorException e(LoadExceptionFrmFile, 300);
		// @@Chaim - not only was no error reported, but someone went to the trouble
    // of creating an exception object!
		return false;
	}

	tfsJNICreateJavaVM afsJNICreateJavaVM = NULL;
	#ifndef __TV_UNIX__
		afsJNICreateJavaVM = reinterpret_cast<tfsJNICreateJavaVM>( GetProcAddress(m_hJVMDLL,"JNI_CreateJavaVM") );
	#else
		void *jniCreateJVM = dlsym(m_hJVMDLL, "JNI_CreateJavaVM");
		afsJNICreateJavaVM = (tfsJNICreateJavaVM)( jniCreateJVM  );
	#endif

	if( !afsJNICreateJavaVM ) {
		WLErrorException e(LoadExceptionFrmFile, 300);
		// @@Chaim - not only was no error reported, but someone went to the trouble
    // of creating an exception object!
		return false;
	}

	JavaVMInitArgs aArgs;
	JavaVMOption aOptions[100];

  // Chaim - keep copy of options as RW strings.
  RVCString* optionsBuf = CLEIniFileProperties::OptionsBuf;
	int aOptionI( 0 );


	{ // INI JVM options
		bool aWhile(true);
		while(aWhile) {
			const char* aEnvValue = optionsBuf[aOptionI];
			if( strlen(aEnvValue) > 0) {

        optionsBuf[aOptionI] = aEnvValue;
        aOptions[aOptionI].optionString = const_cast<char*>(optionsBuf[aOptionI].data());
				++aOptionI;
			}
      else {
				aWhile = false;
			}
		}

	}


//		options[0].optionString = "-Djava.compiler=NONE"; //disable JIT

	RVCString aClassPath("");
	aClassPath += "-Djava.class.path=";
	aClassPath += msxClassPathSep;

	if( !mClassPath.isNull() ) {
		aClassPath += mClassPath;
		aClassPath += msxClassPathSep;
	}

	// Eyal - new code
	{
		const char* aPath = CLEIniFileProperties::WebLOADClasspath;
		if( aPath && strlen(aPath) > 0) {
			aClassPath += aPath;
			aClassPath += msxClassPathSep;
		}
	}

	{
		const char* aPath = CLEIniFileProperties::UserClasspath;
		if( aPath && strlen(aPath) > 0) {
			aClassPath += aPath;
			aClassPath += msxClassPathSep;
		}
	}

	{
		const char* aPath = getenv("CLASSPATH");
		if( aPath && strlen(aPath) > 0) {
			aClassPath += aPath;
			aClassPath += msxClassPathSep;
		}
	}

	//Hryvach: added AutiDiscovery of jars
	CJarsAutoDiscovery autoDiscoveryObj;
	autoDiscoveryObj.discoverPath( CLEIniFileProperties::JavaExtensionsPath, "*.jar", true, aClassPath, msxClassPathSep );

	aOptions[ aOptionI ].optionString = const_cast<char*>(aClassPath.data());
	++aOptionI;

	RVCString aCompiler;
	if( !mJIT.isNull() ) {
		aCompiler += "-Djava.compiler=";
		aCompiler += mJIT;
		aOptions[ aOptionI ].optionString = const_cast<char*>(aCompiler.data());
		++aOptionI;
	}

	aArgs.version = JNI_VERSION_1_2;
	aArgs.options = aOptions;
  aArgs.nOptions = aOptionI;
	aArgs.ignoreUnrecognized = JNI_TRUE;

        jint aStatus = -1;

        try {
          aStatus = afsJNICreateJavaVM( &oJVM, reinterpret_cast<void**>( &oJEnv ), &aArgs);
        }
        catch(...) {
#ifdef __TV_UNIX__
          WlErrorMsg("Failed To Create Java VM, Error Code: %d", errno );
#else
          WlErrorMsg("Failed To Create Java VM, Error Code: %d", GetLastError());
#endif //__TV_UNIX__
        }

	return 0 == aStatus;
}



bool fsAttachCurrentThread(
	JavaVM* rJVM,
	JNIEnv*& oJEnv )
{
	JavaVMAttachArgs aArgs;
	aArgs.version=JNI_VERSION_1_2;
	aArgs.name=NULL;
	aArgs.group=NULL;

	jint aStatus = rJVM->AttachCurrentThread( reinterpret_cast<void**>(&oJEnv), reinterpret_cast<void*>(&aArgs) );

	return( 0 == aStatus );
}


void cRVJVM::fJVMMainThread()
{
  //RVThreadSelf self = ::rvThread();
  //self.setPriority(self.getMaxPriority());

	try {
    //LE_SET_SE_TRANSLATOR(); @@Chaim - We suspect this causes problems in JVM
 
		
// JSJVM must be initialized and terminated in the same machine thread. liveconnect keeps the JNIEnv of the thread in which JSJVM initialized.
// also, the termination will fail if any of the threads that used Java through JS has not yet been unattached. thus we must unattach virtual clients and init-agenda before termination.
// so we must have a special thread for JSJVM that doesn't do any JS-Java work.

// only one thread is waiting here for JSJVM ready signal. other threads in need of JSJVM wait for that waiting thread.

	WlAssert(!mJVM);
	WlAssert(!mJSJVM);

	if (CLEIniFileProperties::JVMCreateWaitTime){
#ifdef __TV_UNIX__
    ::rvSleep(CLEIniFileProperties::JVMCreateWaitTime * 1000);
#else
    Sleep(CLEIniFileProperties::JVMCreateWaitTime * 1000);
#endif
	}
	
	JavaVM* aJVM  = NULL;
	JNIEnv* aJEnv = NULL;

	{
		bool aStatus = fCreateJVMInternal( aJVM, aJEnv );
		WlAssert(aStatus);
		mJVM = aJVM;
	}

	JSJavaVM* aJSJVM = RVJSJ_ConnectToJavaVMX0( aJVM, aJEnv );

	if( aJSJVM ) {
	} else {
		WlAssert(0);
	}
	mJSJVM = aJSJVM;
	mJNIEnv = aJEnv;

	fsCallInitFunc( *this, aJEnv );


// JVM is ready, waiting should be released
	mJVMStartThreadEvent.Set();

// waiting for signal to terminate LiveConnect

	mJVMTerminateLiveConnectEvent.Wait();
	mJVMTerminateLiveConnectEvent.Reset();

	WlAssert( mJVM == aJVM );
	WlAssert( mJSJVM == aJSJVM );

	JSJ_DisconnectFromJavaVM( aJSJVM );

// LiveConnect terminated, waiting should be released
	mJVMTerminateLiveConnectDoneEvent.Set();

// waiting for signal to terminate

	mJVMDestroyJVMEvent.Wait();
	mJVMDestroyJVMEvent.Reset();
	mJSJVM = NULL;

	{
    WlTraceMsg("Before DetachCurrentThread");
		jint aStatus = aJVM->DetachCurrentThread();
    WlTraceMsg("After DetachCurrentThread");
		WlAssert( 0 == aStatus );
	}


  if (CLEIniFileProperties::DestroyJVM) {
    WlTraceMsg("Before DestroyJavaVM");
	  jint aStatus = aJVM->DestroyJavaVM();
    WlTraceMsg("After DestroyJavaVM");
	  WlAssert( 0 == aStatus );
 	}
  mJVM = NULL;
	mJVMDestroyJVMDoneEvent.Set();

// JVM unattached, waiting should be released
// signaling by exit
	}
	catch (std::exception* e){
		ExceptionHandler(e);
	}
	catch (std::exception& e){
		ExceptionHandler(&e);
	}

}



bool cRVJVM::fCreateJVM( JNIEnv*& oJEnv )
{
	WlAssert(!mJVM);
	WlAssert(!mJSJVM);
	{
		RVCriticalSection::LockGuard aLockGuard(mJVMLock);
		// after the lock has to check again since, another thread
		// might have loaded & connected to the JVM.
		if( !mJVM ) {
			WlAssert(!mJSJVM);

 #if !defined(RV_THR_CANT_EXTRACT_TEMPLATE_ARGS_FROM_SIGNATURE)            
	    mJVMThread = rvMakeThreadFunction( *this, &cRVJVM::fJVMMainThread );
 #else
	    mJVMThread = rvMakeThreadFunctionM( cRVJVM, *this, void, &cRVJVM::fJVMMainThread );
 #endif
			mJVMThread.start();

			mJVMStartThreadEvent.Wait();
			mJVMStartThreadEvent.Reset();

		} else { // another
		}
	}

	bool rtn = mJVM && mJSJVM;

	if(rtn) {
		fsAttachCurrentThread( mJVM, oJEnv );
	}

	return rtn;
}



bool cRVJVM::fAttachCurrentThread(
	JNIEnv*& oJEnv )
{
	bool rtn;
	JNIEnv* aJEnv;

	if( mJVM ) {
		rtn = fsAttachCurrentThread( mJVM, aJEnv );
	} else {
		rtn = fCreateJVM( aJEnv ) ? true : false;
	}

	oJEnv = aJEnv;
	return ( rtn );
}

bool cRVJVM::fCreateJavaVM()
{
	bool rtn = true;
	JNIEnv* aJEnv;

	if( !mJVM ) {
		rtn = fCreateJVM( aJEnv ) ? true : false;
	}

	return ( rtn );
}

bool cRVJVM::fJSJAttachCurrentThread(
	JSJavaThreadState*& oJSJThreadState,
	JNIEnv* rJEnv )
{
	if( !mJSJVM ) {
		JNIEnv* aJEnv;
		bool aStatus = fCreateJVM( aJEnv );
		WlAssert( aStatus );
	}

	JSJavaThreadState* aJSJThreadState = NULL;

	JNIEnv* aJEnvX1 = 0;
	aJSJThreadState = JSJ_AttachCurrentThreadToJava( mJSJVM, "cRVJVM thread", &aJEnvX1 );
	WlAssert( rJEnv == aJEnvX1 );

	// status check:
	bool rtn;
	if( aJSJThreadState ) {
		oJSJThreadState = aJSJThreadState;
		rtn = true;
	} else {
		rtn = false;
	}

	return rtn;
}

bool cRVJVM::fDetachCurrentThread()
{
	WlAssert( mJVM );
	jint aStatus = mJVM->DetachCurrentThread();
	return 0 == aStatus;
}


bool cRVJVM::fJSJDetachCurrentThread(
	JSJavaThreadState* rJSJThreadState )
{
	bool rtn( false );

	WlAssert( mJSJVM );
	if( JSJ_DetachCurrentThreadFromJava( rJSJThreadState ) ) {
		if( fDetachCurrentThread() ) {
			rtn = true;
		}
	}

	return rtn;
}


void cRVJVM::fAddClassPath( const char* r )
{
	mClassPath += r;
	mClassPath += msxClassPathSep;
}

void cRVJVM::fSetClassPath( const char* r )
{
	mClassPath = r;
}

void cRVJVM::fSetJIT( const char* r )
{
	mJIT = r;
}

void cRVJVM::fSetJVMPath( const char* r )
{
	mJVMPath = r;
}

bool cRVJVM::fsCallInitFunc( cRVJVM& rJVM, JNIEnv* rJEnv )
{
  bool rtn = true;
  mManager.fGetProcess().ProcessInitializeJava(rJVM, rJEnv);
	return rtn;
}

bool cRVJVM::fsCallFinalFunc( cRVJVM& rJVM, JNIEnv* rJEnv )
{
  bool rtn = true;
  mManager.fGetProcess().ProcessFinalizeJava(rJVM, rJEnv);
	return rtn;
}

JavaVM* cRVJVM::fGetJVM()
{
	WlAssert(mJVM);
	return mJVM;
}

JNIEnv* cRVJVM::fGetEnv()
{
	WlAssert(mJNIEnv);
	return mJNIEnv;
}

cRVJVMManager& cRVJVM::fGetManager()
{
	return mManager;
}


void cRVJVM::ExceptionHandler(std::exception* e){
	RVCString FullExcName ("");
	WLERRGetErrorMsg(e, FullExcName);
	RVCString StackTrace("");
	if (WLERRGetStackTrace(e, StackTrace) && StackTrace.length() > 0){
	  WlErrorMsg("CLoadThreadInstance::loadLoop - Exception %s occured\nTrace:\n%s", FullExcName, StackTrace);
	  printf("CLoadThreadInstance::loadLoop - Exception %s occured\nTrace:\n%s", FullExcName, StackTrace);
	}
	else {
	  WlErrorMsg("CLoadThreadInstance::loadLoop - Exception %s occured", FullExcName);
	  printf("CLoadThreadInstance::loadLoop - Exception %s occured", FullExcName);
	}
}

const char* const cRVJVM::msxClassPathSep =
	#ifdef __TV_WINDOWS__
		";"
	#else
		":"
	#endif
	;







cRVJVMManager* cRVJVMManager::msSingleton = NULL;

cRVJVMManager& cRVJVMManager::fsGetSingleton()
{
	WlAssert( msSingleton );
	return *msSingleton;
}

cRVJVMManager::cRVJVMManager(
	CGeneralProcess& rProcess )
:
	mProcess(rProcess),
	mJVM(*this)
{
	WlAssert( !msSingleton );
	msSingleton = this;
}

cRVJVMManager::~cRVJVMManager()
{}

bool cRVJVMManager::fInit()
{
	JSJ_Init( &msxJSJCallbacks );
  RegisterLEEnterJava(fsEnterJava);
  RegisterLEExitJava(fsExitJava);

	return mJVM.fInit();
}

bool cRVJVMManager::fJSJFinal()
{
	return mJVM.fJSJFinal();
}

bool cRVJVMManager::fFinal()
{
	return mJVM.fFinal();
}

cRVJVM& cRVJVMManager::fGetJVM()
{
	return mJVM;
}

bool cRVJVMManager::fsJSInit( JSContext* rCx, JSObject* rJSObj )
{
	if( !JSJ_InitJSContext( rCx, rJSObj, NULL ) ) {
		return false;
	}
	// Natives
	return true;
}

CGeneralProcess& cRVJVMManager::fGetProcess()
{
	return mProcess;
}

JSContext* cRVJVMManager::fsJSJMapJSJThreadToJSContext(
	JSJavaThreadState* rJSJEnv,
#ifdef OJI
	void* rJavaAppletObj,
#endif
	JNIEnv* rJEnv,
	char** rError)
{
	JSContext* rtn;
	if( iLEJSMachineThread* aCurrentMThread = fsGetSingleton().fGetProcess().fGetifCurrentThread() ) {
		iLEJSEmbedContext* aEmbedContext = aCurrentMThread->fGetCurrentEmbedContext();
		WlAssert( aEmbedContext );
		rtn = aEmbedContext->fCurrentJSContext();
	} else {
	//@@
	//instance MachineThread
	//attach to java
//		add to container
//	create JSContext
	}

	return rtn;
}




JSJavaThreadState* cRVJVMManager::fsJSJMapJSContextToJSJThread(
	JSContext* rCx,
	char** oErr )
{
	// GC thread is possible too
	WlAssert(rCx);

// @@Chaim (inherited from Itaj): try to add JSJ_SetDefaultJSContextForJavaThread()

	iLEJSEmbedContext* aEmbedContext = iLEJSEmbedContext::fsGet( rCx );
	WlAssert(aEmbedContext);
  
	JSJavaThreadState* rtn = aEmbedContext->fGetMachineThread().fGetJSJThreadState();
	return rtn;
}


JSObject* cRVJVMManager::fsJSJMapJavaObjectToJSObject( JNIEnv* env, void* rJavaObject, char** rErr )
{
  jobject applet = (jobject)rJavaObject;
  jclass cls = env->GetObjectClass(applet);
  jmethodID midGetParam = env->GetMethodID(cls, "getParameter", "(Ljava/lang/String;)Ljava/lang/String;");

  //
  // Make sure mayscript is enabled for the applet.
	/*
  jstring mayscriptKey = env->NewStringUTF("mayscript");
  jstring mayscript = (jstring)env->CallObjectMethod(applet, midGetParam, mayscriptKey);
  env->DeleteLocalRef(mayscriptKey);
  if (mayscript==NULL){
    static const char* errmsg = "MAYSCRIPT is not enabled for this applet";
    *rErr = (char*)malloc(255);
    strcpy(*rErr, errmsg);
    return NULL;
  }
  */
  jstring key = env->NewStringUTF("CDOMWindow*");
  jstring jFramePtr = (jstring)env->CallObjectMethod(applet, midGetParam, key);
  CPointerToJava ptj(env, jFramePtr);
  env->DeleteLocalRef(key);

  JSObject* jsFramePtr = (JSObject*)ptj.getPointer();

	return jsFramePtr;
}

void cRVJVMManager::fsJSJErrorReporter( const char* rMsg )
{
	iLEJSMachineThread& aCurrentMThread = fsGetSingleton().fGetProcess().fGetCurrentThread();
	iLEJSEmbedContext* aEmbedContext = aCurrentMThread.fGetCurrentEmbedContext();
	WlAssert(aEmbedContext);
	cLoadReportInfoClient errRep( WLLogMessage::Error, rMsg, false, true, true, false, 0 );
  // @@Chaim (inherited from Itaj): find another way, when integrating with a version
  // of reportLog+Apply that receives iReportInfo&, it will be possible
  // to add EmbedContext::report()
	aEmbedContext->fReport( errRep );
}

JSBool cRVJVMManager::fsJSJCreateVM( SystemJavaVM** oJVM, JNIEnv** oJEnv, void* rArgs )
{
	// make sure this is never called, because don't know here who called.
	WlAssert(0);
	return JS_FALSE;
/*
	JSBool rtn;
	//not using args, the manager knows them
	JavaVM* aJVM = fsGetManager().fCreateJVM( *oJEnv );

	if( aJVM ) {
		*oJVM = reinterpret_cast<SystemJavaVM*>(aJVM);
		rtn = JS_TRUE;
	} else {
		rtn = JS_FALSE;
	}

	return rtn;
*/
}

JSBool cRVJVMManager::fsJSJDestroyVM( SystemJavaVM*, JNIEnv* )
{
	// JVM will be destroyed later
	return JS_TRUE;
}




JNIEnv* cRVJVMManager::fsJSJAttachCurrentThread(
	SystemJavaVM* rSystemJVM)
{
	// Called by JSJ_ConnectToJavaVM, to set main_thread
	// Try to do that (assert and return) with new JNI API (IsCurrentThreadAttached...)
	iRVJavaMachineThread& aCurrentMThread = fsGetSingleton().fGetProcess().fGetCurrentThread();
	JNIEnv* rtn = aCurrentMThread.fGetifJEnv(); // supposed to be attached already
	WlAssert(rtn);
	return rtn;

// Old code:
/*
	WlAssert( rSystemJVM );
  JavaVM* aJVM = reinterpret_cast<JavaVM*>(rSystemJVM);

  JNIEnv* aJEnv;
	if( !fsAttachCurrentThread( aJVM, aJEnv ) ) {
		aJEnv = NULL;
	}

  return aJEnv;
*/
}


JSBool cRVJVMManager::fsJSJDetachCurrentThread(
	SystemJavaVM* rJVM,
	JNIEnv* rJEnv)
{
	// JEnv will be detached later
	iRVJavaMachineThread& aCurrentMThread = fsGetSingleton().fGetProcess().fGetCurrentThread();
	WlAssert( aCurrentMThread.fIsJEnv() );
	return JS_TRUE;
}


SystemJavaVM* cRVJVMManager::fsJSJGetJVM(
	JNIEnv* rJEnv )
{
	// only one JVM
	return fsGetSingleton().mJVM.fGetJVM();
}


JSJCallbacks cRVJVMManager::msxJSJCallbacks = {
	fsJSJMapJSJThreadToJSContext, // map_jsj_thread_to_js_context
	fsJSJMapJSContextToJSJThread, // map_js_context_to_jsj_thread
	fsJSJMapJavaObjectToJSObject, // map_java_object_to_js_object
	NULL, // get_JSPrincipals_from_java_caller,
	NULL, // JSBool (*enter_js_from_java)(JNIEnv *jEnv, char **errp,  void **pNSIPrincipaArray, int numPrincipals, void *pNSISecurityContext,void* applet_obj);
	NULL, // JSBool (*enter_js_from_java)(JNIEnv *jEnv, char **errp);
	fsJSJErrorReporter, // error_print
	NULL,  // get_java_wrapper, This is needed when we use the latest jsjava.h file
	NULL, // unwrap_java_wrapper
	fsJSJCreateVM, // create_java_vm
	fsJSJDestroyVM, // destroy_java_vm
	fsJSJAttachCurrentThread, // attach_current_thread
	fsJSJDetachCurrentThread, // detach_current_thread
	fsJSJGetJVM // get_java_vm
};



iRVJSJSection* cRVJVMManager::fsEnterJava(
	JSContext* rCx,
	const char* rClassDescriptionName,
	const char* rMemberName,
	tJavaCallType rCallType )
{

	iLEJSEmbedContext* aEmbedCx = iLEJSEmbedContext::fsGet( rCx );
	WlAssert( aEmbedCx ); //@@ what about GC thread?
	return aEmbedCx->fEnterJava( rClassDescriptionName, rMemberName, rCallType );
}


void cRVJVMManager::fsExitJava(
	JSContext* rCx,
	JSBool rOK,
	const char* rClassDescriptionName,
	const char* rMemberName,
	tJavaCallType rCallType,
	iRVJSJSection* rJSJSection )
{
	iLEJSEmbedContext* aEmbedCx = iLEJSEmbedContext::fsGet( rCx );
	WlAssert( aEmbedCx ); //@@ what about GC thread?
	aEmbedCx->fExitJava( rOK, rClassDescriptionName, rMemberName, rCallType, rJSJSection );
}


/*
void LEexitJava(void* td_,JSBool ok,const char* class_description_name,const char* member_name, tJavaCallType rCallType ) {
	if(td_) {
		CLoadGeneratorThreadInstanceScript* aTD = reinterpret_cast<CLoadGeneratorThreadInstanceScript*>(td_);
		char methodName[256];
		aTD->ExitJava();
		if (ok) { //ok is the return value of the function : invoke_java_method succeeded
			if( (sJavaCallMethod == rCallType) && !aTD->IsDisabledJavaDefaultStatistics() ) {
				// Shahar - 26-11-02 - Patch 226.003
				// In order that we will have the correct timer name as was set in the
				// enter to java code the format it as we did early and only after we format 
				// it with the success...
				sprintf(methodName,"%s::%s (Timer)",class_description_name,member_name);
				aTD->sendTimer(methodName);
				sprintf(methodName,"%s::%s (success)",class_description_name,member_name);
			}
		} else {
			sprintf(methodName,"%s::%s (failure)",class_description_name,member_name);
		}
		if(!aTD->IsDisabledJavaDefaultStatistics()) {
			aTD->addCounter(methodName);
		}
	} else if( IGeneralManagerScriptData* aSD = getCurrentManagerScriptData() ) {
		aSD->ExitJava();
	} else {
		WlAssert(0);
	}
}
*/

