/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "jstestclass.h"

#include "loadreportinfo.h"
#include "codetest.h"
#include "rvjsembed.h"


cJSTestClassPrivate::cJSTestClassPrivate() {
}

cJSTestClassPrivate::~cJSTestClassPrivate() {
}


JSBool cJSTestClassPrivate::fInitialize(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	
	LoadReport( cx, WLLogMessage::Note, "JSTest Init" );
	
	return JS_TRUE;
}

void cJSTestClassPrivate::fFinalize(JSContext *cx, JSObject *obj)
{
//	LoadReport( cx, WLLogMessage::Note, "JSTest Finalize" );

}


JSBool Initialize(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	cJSTestClassPrivate* Prv = new cJSTestClassPrivate();
	JS_SetPrivate( cx, obj, reinterpret_cast<void*>(Prv) );

	return Prv->fInitialize( cx, obj, argc, argv, rval );
}

void ClassFinalize(JSContext *cx, JSObject *obj)
{
	cJSTestClassPrivate* Prv = reinterpret_cast<cJSTestClassPrivate*>( JS_GetPrivate( cx, obj ) );
	Prv->fFinalize( cx, obj );
	delete Prv;
}

JSBool ClassPropertyGet(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{

	//cJSTestClassPrivate* aPrv;
	if( !JS_GetPrivate( cx, obj ) ) {
		return JS_TRUE;
	}

	int aId;
	JSString* aX0;
	const char* aX1;
	if( JSVAL_IS_INT(id) ) {
		aId = JSVAL_TO_INT(id);
		WlTraceMsg( "X0" );
	} else if( JSVAL_IS_STRING(id) ) {
		aX0 = JSVAL_TO_STRING(id);
		aX1 = JS_GetStringBytes( aX0 );
		WlTraceMsg( "X1" );
		if( *vp == JSVAL_VOID ) {
			jsval v = INT_TO_JSVAL(4);
			JS_DefineUCProperty( cx, obj, JS_GetStringChars( aX0 ), JS_GetStringLength( aX0 ), v, NULL, NULL, JSPROP_ENUMERATE );
			*vp = v;
		} else {
			if( JSVAL_IS_INT( *vp ) ) {
				*vp = INT_TO_JSVAL( JSVAL_TO_INT( *vp ) +1 );
			}
		}
	}

	return JS_TRUE;
}

JSBool ClassPropertySet(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
	return JS_TRUE;
}

JSBool ClassEnumerate(JSContext *cx, JSObject *obj
//, JSIterateOp enum_op, jsval *statep, jsid *idp
)
{
	if( !JS_GetPrivate( cx, obj ) ) {
		return JS_TRUE;
	}
	
	JS_DefineProperty( cx, obj, "mX1", INT_TO_JSVAL( 1000 ), NULL, NULL, JSPROP_ENUMERATE );
	return JS_TRUE;
}


JSBool ClassResolve( JSContext *cx, JSObject *obj, jsval id, uintN flags, JSObject **objp)
{

	bool aIsQualified( 0 != ( flags & JSRESOLVE_QUALIFIED ) );
	bool aIsAssigning( 0 != ( flags & JSRESOLVE_ASSIGNING ) );
	
	int aId;
	JSString* aX0;
	const char* aX1;
	if( JSVAL_IS_INT(id) ) {
		aId = JSVAL_TO_INT(id);
		WlTraceMsg( "X0" );
	} else if( JSVAL_IS_STRING(id) ) {
		aX0 = JSVAL_TO_STRING(id);
		aX1 = JS_GetStringBytes( aX0 );
		WlTraceMsg( "X1" );
	}

	*objp = NULL;

	return JS_TRUE;
}


JSBool ClassResolveChain( JSContext *cx, JSObject *obj, jsval id, JSBool isProto, uintN flags, JSBool* resolved )
{

	if( !JS_GetPrivate( cx, obj ) ) {
		*resolved = JS_FALSE;
		return JS_TRUE;
	}


	bool aIsQualified( 0 != ( flags & JSRESOLVE_QUALIFIED ) );
	bool aIsAssigning( 0 != ( flags & JSRESOLVE_ASSIGNING ) );

	if( isProto ) {
	
		int aId;
		JSString* aX0;
		const char* aX1;
		if( JSVAL_IS_INT(id) ) {
			aId = JSVAL_TO_INT(id);
			*resolved = JS_FALSE;
			WlTraceMsg( "X0" );
		} else if( JSVAL_IS_STRING(id) ) {
			aX0 = JSVAL_TO_STRING(id);
			aX1 = JS_GetStringBytes( aX0 );
			JS_DefineUCProperty( cx, obj, JS_GetStringChars( aX0 ), JS_GetStringLength( aX0 ), INT_TO_JSVAL(25), NULL, NULL, JSPROP_ENUMERATE );
			*resolved = JS_TRUE;
			WlTraceMsg( "X1" );
		}

	} else {
		*resolved = JS_FALSE;
	}


	return JS_TRUE;
}




JSClass ClassType = {
  "cTest",
  JSCLASS_HAS_PRIVATE | JSCLASS_NEW_RESOLVE //| JSCLASS_NEW_ENUMERATE
	,
  JS_PropertyStub,	// add Property
  JS_PropertyStub,	// del Property
  ClassPropertyGet, // get Property
  ClassPropertySet, // set Property
  (JSEnumerateOp)ClassEnumerate,
	//JS_ResolveStub //
	(JSResolveOp)ClassResolve
	,
	JS_ConvertStub,
	ClassFinalize,

	0, 0, 0, 0, 0, 0, 0, 0, 0,
	ClassResolveChain
};

JSBool fsMethodEval(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) 
{
	JSString* aCode = JS_ValueToString( cx, argv[0] );
	JSString* aFileName = JS_ValueToString( cx, argv[1] );

	JSBool aStatus = JS_EvaluateUCScript( cx, obj, JS_GetStringChars( aCode ), JS_GetStringLength( aCode ),	JS_GetStringBytes( aFileName ), 1, rval );
	return aStatus;
}

JSBool ClassFX0(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) 
{
	iLEJSEmbedContext* aEmbedContext = iLEJSEmbedContext::fsGet(cx);
	aEmbedContext->fGetMachineThread().fGetJEnv();

	return JS_TRUE;
}

JSBool ClassFAssert(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) 
{
	JSBool aCond;
	JS_ValueToBoolean( cx, argv[0], &aCond );
	RVJS_Assert( aCond );
	
	return JS_TRUE;
}

static JSFunctionSpec ClassMethods[] = {
	{"fEval", fsMethodEval, 2 },
	{"fX0", ClassFX0, 0},
	{"fAssert", ClassFAssert, 1},
	{0}
};

static JSPropertySpec ClassProperties[] = {
	{"mX1", 0, JSPROP_ENUMERATE },
	{0}
};


JSBool ClassCtor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	return Initialize( cx, obj, argc, argv, rval );
}


bool AddTestJSClass(JSContext *cx, JSObject *obj)
{
  JSObject* aProto = JS_InitClass(cx, obj, NULL, &ClassType, ClassCtor, 0, ClassProperties, ClassMethods, NULL, NULL);

	JS_DefineProperty( cx, aProto, "mX0", INT_TO_JSVAL(21), NULL, NULL, JSPROP_ENUMERATE );

  return aProto;
}

