/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <crtdbg.h>
#include "tracedll.h"
#include "loadgenerator.h"
#include "loadgeneratorslave.h"
#include "loadthreadsmanager.h"
#include "loadengineglobals.h"
#include "tvenv.h"
#include "loadenginedecs.h"
#include "loadenginestatistics.h"
#include "loadengineinifileproperties.h"
#include "generalprocess.h"
//Galtech
#include "gt_unicode.h"

#include "tvtables.h"

#include "CommandLinePropertie.h"

extern CCommandLineProperties	g_cmdprop;

#ifdef __TV_UNIX__
#include <sys/types.h>
#include <sys/stat.h>
#endif


int LoadGeneratorMain(int argc, char *argv[], int ExtraProtocol)
{
	if (CLEIniFileProperties::initPerformed == false)
	{
		CLEIniFileProperties::Init();
	}

  SetExtraProtocol(ExtraProtocol);
	
	_CrtSetDbgFlag( 0 );
	try {
		WlTraceMsg("Starting LoadGeneratorMain(%s,%s,%s,%s)",argv[1], argv[2], argv[3], argv[4]);
		CLoadGenerator* lg = new CLoadGenerator(argv[1], argv[2], argv[3], argv[4]);
		WlTraceMsg("Starting LoadGeneratorMain(%s,%s,%s,%s)",argv[1], argv[2], argv[3], argv[4]);
		lg->MainLoop();
		try {
			//    For Performance, leave the delete in comments although it is "memory leak".
			//    The Stop process stuck when the Dtor was activated
#ifdef _DEBUG
			
			// Commented out again because we don't need the dtors to catch leaks anymore
//			 delete lg;
#endif
		} catch (...) {
			WlAssert(0);
			//#ifdef _DEBUG
			//throw;
			//#endif
		}
	} catch (...) {
		//WlTraceMsg("LoadGeneratorMain: Unhandled exception occurred");
#ifdef _DEBUG
		throw;
#endif
	}
	return 1;
}

int LoadGeneratorSlave(int argc, char *argv[], int ExtraProtocol)
{
	
	if (CLEIniFileProperties::initPerformed == false)
	{
		CLEIniFileProperties::Init();
	}

  SetExtraProtocol(ExtraProtocol);
	
	_CrtSetDbgFlag( 0 );
	
	
	try {
		WlTraceMsg("Starting LoadGeneratorSlave(%s,%s)",argv[1], argv[2]/*, argv[3]*/);
		CGeneralProcess* generalProcess = new CGeneralProcess();
		CLoadGeneratorSlave* lg = new CLoadGeneratorSlave(argv[1], argv[2], *generalProcess/*, argv[3]*/);
		lg->MainLoop();
		try {
#ifdef _DEBUG
			// Commented out again because we don't need the dtors to catch leaks anymore
	//		      delete lg;
			      delete generalProcess;
#endif
		} catch (...) {
			WlAssert(0);
			//#ifdef _DEBUG
			//throw;
			//#endif
		}
	} catch (...) {
		//WlTraceMsg("LoadGeneratorSlave: Unhandled exception occurred");
		WlAssert(0);
	}
	
	return 1;
}

int LoadGeneratorTest(int argc, char *argv[], int ExtraProtocol)
{
	if (CLEIniFileProperties::initPerformed == false)
	{
		CLEIniFileProperties::Init();
	}
	
	SetExtraProtocol(ExtraProtocol);
	
	_CrtSetDbgFlag( 0 );
	try {
		int reportInterval = 100000; //100 seconds by default

		//parse parameters list, except the firts one - scritp name
		for(size_t i = 2; i<argc; ++i) {
			if(argv[i][0] == '-')
			{
				switch(argv[i][1]) {
				case 'b':
					g_cmdprop.batchMode = true;
					break;
				case 'c':
					g_cmdprop.virtualClients = atoi(argv[i]+3);
					if (g_cmdprop.virtualClients == 0)
						g_cmdprop.virtualClients = 1;
					break;
				case 't':
					if( strcmp(KTimeStampType[etAbsolute], argv[i]+3) == 0 )
						g_cmdprop.timeStamp = etAbsolute;
					else if( strcmp(KTimeStampType[etRelative], argv[i]+3) == 0 )
						g_cmdprop.timeStamp = etRelative;
					else if( strcmp(KTimeStampType[etNone], argv[i]+3) == 0 )
						g_cmdprop.timeStamp = etNone;
					break;
				case 'a':
					g_cmdprop.appendLog = true;
					break;
				case 'm':
					g_cmdprop.messagesOn = true;
					break;
				case 'd':
					g_cmdprop.consoleDump = true;
					break;
				case 'u':
					g_cmdprop.logSuffix = true;
					break;
				case 's': 
					SetStatisticsFileName(argv[i]+3);
					break;
				case 'r':
					reportInterval = atoi(argv[i]+3);
					if (reportInterval == 0)
						reportInterval = 100000;
					break;
				default:
					cout << "unsupported option: " << argv[i] << endl;
				}
			}
			else
				cout << "unsupported option: " << argv[i] << endl;
		}

		if (g_cmdprop.batchMode){
			RVCString statname = GetTestLogFileName( argv[1]);
			CLog::InitLog( statname.data(), getCommandLineProperties().timeStamp, !getCommandLineProperties().appendLog, getCommandLineProperties().consoleDump );
			
			//dump command-line parameters
			char* buffer = new char[2000];
			int buffLen = 2000;
			for(i=0; i<argc; i++)
			{
				if ( buffLen < (strlen(argv[i]) + 100) ) {
					//reallocate buffer to fit long command-line parameter
					delete [] buffer;
					buffLen = strlen(argv[i]) + 100;
					buffer = new char[buffLen];
				}
					
				sprintf( buffer, "argv[%d] = '%s'", i, argv[i] );
				CLog::Write( buffer, eCommand );
			}
			delete [] buffer;
		}
		
		CLoadGenerator* lg = new CLoadGenerator(argv[1], reportInterval);
		{
			TIMERENTRY("lg->MainLoop()");
			lg->MainLoop();
		}
		try {
#ifdef _DEBUG
			// Commented out again because we don't need the dtors to catch leaks anymore
			//      delete lg;
#endif
		} catch (...) {
			WlAssert(0);
		}
	} catch (WLErrorException e) {
		cout << "Exception: " << e.GetErrorMsg() << endl;
		WlTraceMsg("Exception: %s",e.GetErrorMsg());
		//		WlAssert(0);
	} catch (...) {
		//WlTraceMsg("LoadGeneratorMain: Unhandled exception occurred");
		WlAssert(0);
	}
	return 1;
}
//
/*
WLInitGeneratorShell *pWLInitGeneratorShell = NULL;
void RegisterWLInitGeneratorShell(WLInitGeneratorShell *f)
{
pWLInitGeneratorShell = f;
}
*/


IThreadsManager *pCurrentThreadsManager = NULL;
void setCurrentThreadsManager(IThreadsManager *pTM)
{
	pCurrentThreadsManager = pTM;
}

IThreadsManager *getCurrentThreadsManager()
{
	return pCurrentThreadsManager;
}

void setCurrentManagerScriptData(IGeneralManagerScriptData* rMSD, JSContext* rCx ) {
	rMSD->fSignContext( rCx );	
}

IGeneralManagerScriptData* getCurrentManagerScriptData(JSContext* rCx) {
	return dynamic_cast<IGeneralManagerScriptData*>(iLEJSEmbedContext::fsGet(rCx));
}

static long MaxThreadsPerSlave = 0;
void SetMaxThreadsPerSlave(long maxthreadsperslave) {MaxThreadsPerSlave = maxthreadsperslave;}
long GetMaxThreadsPerSlave() {return MaxThreadsPerSlave;}

RVCString CArrangeMachineTempFiles::GetTempDirectory() {
	const char* tempDir;
	RVCString tempDirectory(TVPGetTempDirectory());
	TVEnvContainerClass tvpc;
	if(tempDirectory.isNull()) {
		TableHandle tableHandle=TVSCreateTable();
		TVEnvContainerClass tvpc;
		if(tempDir = tvpc.TVEGetEnv("TV_BIN")) {
			tempDirectory = tempDir;
		} else {
			WlTraceMsg("Error : in function CArrangeMachineTempFiles::GetTempDirectory()");
			WlAssert(0);
		}
	}
	return tempDirectory;
}

// the function return the full path with the file.
RVCString CArrangeMachineTempFiles::GetWlTempFullPath(JSContext* cx, TypeOfTheUserFile typeOfTheUserFile, const char* fileName) {
	char pathTemp[WL_MAX_PATH];
	int numScript;
	int masterPID;
	const char *tempDirectory = NULL;
	
	iLEAgendaJSEmbedContext* pAgendaEmbedContext = iLEAgendaJSEmbedContext::fGetAgendaEmbedContext(cx);
	numScript = pAgendaEmbedContext->GetNumScriptInMix();
	masterPID = pAgendaEmbedContext->fGetProcess().GetMasterPid();
	tempDirectory = pAgendaEmbedContext->fGetProcess().GetTempDirectory().data();
	
	switch(typeOfTheUserFile) {
	case WLECopiedFile :
#ifdef __TV_WINDOWS__
		sprintf(pathTemp,"%s\\wlTemp%d\\wlScripNumInMix%d\\%s",tempDirectory, masterPID, numScript, fileName);
#endif
#ifdef __TV_UNIX__
		sprintf(pathTemp,"%s/wlTemp%d/wlScripNumInMix%d/%s",tempDirectory, masterPID, numScript, fileName);
#endif
		break;
	case WLEOutputFile :
#ifdef __TV_WINDOWS__
		sprintf(pathTemp,"%s\\%s",tempDirectory, fileName);
#endif
#ifdef __TV_UNIX__
		sprintf(pathTemp,"%s/%s",tempDirectory, fileName);
#endif
		break;
	}
	
	RVCString tempFileName(fileName);
#ifdef __TV_WINDOWS__
	if(tempFileName.contains("\\")) { //the filename parameter is a path
		strcpy(pathTemp,fileName);
	}
#endif
#ifdef __TV_UNIX__
	if(tempFileName.contains("/")) { //the filename parameter is a path
		strcpy(pathTemp,fileName);
	}
#endif
	
	RVCString rwPath(pathTemp);
	return rwPath;
}


// the function return the full path without the file
RVCString CArrangeMachineTempFiles::GetWlTempFullPath(int masterPID, int numScriptInMix) {
	char pathTemp[WL_MAX_PATH];
	RVCString tempDirectory = GetTempDirectory();
	
	
#ifdef __TV_WINDOWS__
	sprintf(pathTemp,"%s\\wlTemp%d\\wlScripNumInMix%d",tempDirectory.data() , masterPID, numScriptInMix);
#endif
#ifdef __TV_UNIX__
	sprintf(pathTemp,"%s/wlTemp%d/wlScripNumInMix%d",tempDirectory.data() , masterPID, numScriptInMix);
#endif
	
	RVCString rwPath(pathTemp);
	return rwPath;
	
}


// return the root temp directory that is created by wl for this generator.
RVCString CArrangeMachineTempFiles::GetWlTempPath( int masterPid) {
	char pathTemp[WL_MAX_PATH];
	RVCString tempDirectory = GetTempDirectory();
	
	sprintf(pathTemp,"%s%swlTemp%d",tempDirectory.data() ,FILE_SEP_STR,masterPid);	
	
	RVCString rwPath(pathTemp);
	WlTraceMsg("GetWlTempPath returns %s", (char *)pathTemp);
	return rwPath;
}


void CArrangeMachineTempFiles::LECreateDirectory(const char* path) {
	if(!TVPCreateDirectory(path)) {
		
		char error[1000];
		sprintf(error,"%s%s","failed to create directory in function CArrangeMachineTempFiles::LECreateDirectory the path is : ",path);
		WlTraceMsg(error);
		
	}
	
}

//GALTECH: Get\SetMonitorCharset functions
#ifndef _ANSI_ONLY
//static int MonitorCharSet = CP_MS_LATIN1;
static int MonitorCharSet = CP_DEFAULT;
int GetMonitorCharSet()
{
	return MonitorCharSet;
}

void SetMonitorCharSet(int charset)
{
	MonitorCharSet = charset;
}
#endif

void SetExtraProtocol(int p_iAddProtocol){
  if (p_iAddProtocol == WLE_ADDPROTOCOL_HTTP)
		{
    CLEIniFileProperties::AppendProtocol("HTTP_PROTOCOL_ADDITIONAL_PROTOCOLS", "HTTP_PROTOCOL_ADDITIONAL_PROTOCOLS_NAMES");  
		}
		else
		{
    	WlTraceMsg("SetExtraProtocol:: NONE PROTOCOL DEFINED ::p_iAddProtocol = %d ",p_iAddProtocol);
		}
}

