/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "loadthreadsmanager.h"
#include "loadenginevariablesdata.h"
#include "loadengineglobals.h"
#include "tracedll.h"
#include "wlerrorlib.h"
#include "loadjsshell.h"
#include "loadjsdefs.h"
#include "loadenginedecs.h"
#include "generalprocess.h"
#include "rvjsapi.h"
#include "loadjspropertyobj.h"
#include "loadexceptions.h"

#define SYN_FILE_FRM_NAME           "jssyntaxchecks.frm"
#define SYN_ILLEGAL_VALUE           50
#define SYN_WRONG_TYPE              51
#define SYN_CASE_SENSITIVE_ERROR    52
#define SYN_WRONG_LEVEL             53
#define SYN_WORKING_MODE            54
#define SYN_VARIABLES_NOT_FOUND     55
#define SYN_ILLEGAL_DICT_TYPE       56

/****************************************************************/

////////////////////////////////////////////////////////////////////
//  the implementation of class CLEPropWorkSpace   
////////////////////////////////////////////////////////////////////

long CLEPropWorkSpace::m_JTimeoutBeforeOperation = -1;

CLEPropWorkSpace::CLEPropWorkSpace():
m_SendClientStatisticsFilter("") {}
CLEPropWorkSpace::~CLEPropWorkSpace() {}

void CLEPropWorkSpace::ReadData(CWLJSScript& pCWLJSScript) {
//	int indexSleep = pCWLJSScript.fGetProcess().m_PropIndex.m_indexSleep;
	//m_sleep = pCWLJSScript.GetVariablesData()->Get(indexSleep).m_Value.LongVal;
	int indexDisableSleep = pCWLJSScript.fGetProcess().m_PropIndex.m_IndexDisableSleep;
	m_DisableSleep = pCWLJSScript.GetVariablesData()->Get(indexDisableSleep).m_Value.BoolVal;
	int indexSleepRandomMin = pCWLJSScript.fGetProcess().m_PropIndex.m_IndexSleepRandomMin;
	m_SleepRandomMin = pCWLJSScript.GetVariablesData()->Get(indexSleepRandomMin).m_Value.LongVal;
	int indexSleepRandomMax = pCWLJSScript.fGetProcess().m_PropIndex.m_IndexSleepRandomMax;
	m_SleepRandomMax = pCWLJSScript.GetVariablesData()->Get(indexSleepRandomMax).m_Value.LongVal;
	int indexSleepDeviation = pCWLJSScript.fGetProcess().m_PropIndex.m_IndexSleepDeviation;
	m_SleepDeviation = pCWLJSScript.GetVariablesData()->Get(indexSleepDeviation).m_Value.LongVal;
	int indexExceptionEnabled = pCWLJSScript.fGetProcess().m_PropIndex.m_IndexExceptionEnabled;
	m_ExceptionEnabled = pCWLJSScript.GetVariablesData()->Get(indexExceptionEnabled).m_Value.BoolVal;
	int indexReportCSJSInternalErrors = pCWLJSScript.fGetProcess().m_PropIndex.m_ReportCSJSInternalErrors;
	m_ReportCSJSInternalErrors = pCWLJSScript.GetVariablesData()->Get(indexReportCSJSInternalErrors).m_Value.BoolVal;
	
        int indexTimeOut = pCWLJSScript.fGetProcess().m_PropIndex.m_IndexTimeoutBeforeOperation;
	m_JTimeoutBeforeOperation = pCWLJSScript.GetVariablesData()->Get(indexTimeOut).m_Value.LongVal;

  int indexSendClientStatistics = pCWLJSScript.fGetProcess().m_PropIndex.m_IndexSendClientStatistics;
	m_SendClientStatistics = pCWLJSScript.GetVariablesData()->Get(indexSendClientStatistics).m_Value.BoolVal;
  int indexSendClientStatisticsFilter = pCWLJSScript.fGetProcess().m_PropIndex.m_IndexSendClientStatisticsFilter;
	m_SendClientStatisticsFilter = pCWLJSScript.GetVariablesData()->Get(indexSendClientStatisticsFilter).m_Value.StringVal;
  if (m_SendClientStatistics && m_SendClientStatisticsFilter.length() > 0 && m_SendClientStatisticsFilter.data()[0] != '*'){
	prepareClientsStatisticsFilter();
  }
  int indexJVMType = pCWLJSScript.fGetProcess().m_PropIndex.m_IndexJVMType;
	m_JVMType = pCWLJSScript.GetVariablesData()->Get(indexJVMType).m_Value.StringVal;
  int indexCharEncoding = pCWLJSScript.fGetProcess().m_PropIndex.m_IndexCharEncoding;
	m_CharEncoding = pCWLJSScript.GetVariablesData()->Get(indexCharEncoding).m_Value.LongVal;
}


long CLEPropWorkSpace::GetTimeoutBeforeOperation() 
{
	return m_JTimeoutBeforeOperation;
}
void CLEPropWorkSpace::prepareClientsStatisticsFilter(){
	m_SendClientStatisticsFilter.prepend(";");	
	m_SendClientStatisticsFilter.append(";");
	// @@Chaim - tbd - collpase white spaces.

}
/*********************************************/
/*   Variables dictionary class declarations */
/*********************************************/
class CLEPropertyAttributes : public RVCollectable {
public:
  CLEPropertyAttributes() ;
  ~CLEPropertyAttributes() ;
  char*                              m_Name;               //Property name
  CLEPropertyValue::PropertyType     m_Type;               //Property type
  CLEPropertyValue                   m_DefaultValue;       //Property default value
  char*                              m_LegalListOfValues ; //List of valid strings/numbers
                                                           //separated by comma (i.e. "aaa,bbb,ccc")
  CLEPropertyValue::Level            m_StartLevel; //Starting level of property - Globals,Locals or Http
  CLEPropertyValue::Level            m_EndLevel;   //Ending level of property - Globals,Locals or Http
  int                                m_Index ;     //The index of the property in the variables array
} ;

class CLEPropertiesDictionary : public RVHashDictionary {
public:
  CLEPropertiesDictionary();
  ~CLEPropertiesDictionary();
  int RegisterProperty
    ( char*                             Name,
      CLEPropertyValue::PropertyType    Type,
      const CLEPropertyValue &          DefaultValue,
      char*                             LegalListOfValues,
      CLEPropertyValue::Level           StartLevel = CLEPropertyValue::PL_GLOBALS,
      CLEPropertyValue::Level           EndLevel = CLEPropertyValue::PL_PRIVATE);

	void RegisterObjectProperty
  ( char*                             Name,
    CLEPropertyValue::PropertyType    Type);


  int DictionaryLength() {return m_IndexCounter;};
private:
  int m_IndexCounter ; //Sequential numerator for generating unique index numbers
};

/****************************************/
/*      Static variables definitions    */
/****************************************/
//Static variables definitions
bool CLEVariablesData::m_DoSyntaxChecking = false;
static CLEPropertyValue::tVariablesArray DefaultValuesArray = NULL ;
static CLEPropertyValue::tVariablesArray DefaultValuesArrayForCopy = NULL ;


//This is the ONE and ONLY instance of the properties dictionary
CLEPropertiesDictionary LEVariablePropertiesDictionary;

/****************************************/
/*      CLEPropertyValue methods        */
/****************************************/

//constractor
CLEPropertyValue::CLEPropertyValue () {
  m_IsSet = true ; //Should be true as initial value so the destractor will free string values
  m_ValueType = PT_INT;
  m_Value.LongVal = 0 ;
}

//Copy constractor
CLEPropertyValue::CLEPropertyValue (const CLEPropertyValue & Other)
{
  m_IsSet = true ;

  m_ValueType = Other.m_ValueType ;
  switch(m_ValueType)
  {
      case PT_INT:
          m_Value.LongVal = Other.m_Value.LongVal;
          break;
      case PT_FLOAT:
          m_Value.FloatVal = Other.m_Value.FloatVal;
          break;
      case PT_STRING:
          m_Value.StringVal = strdup( Other.m_Value.StringVal) ;
          break;
      case PT_BOOLEAN:
      case PT_BOOLEAN_STR:
          m_Value.BoolVal = Other.m_Value.BoolVal;
          break;
  };
};

CLEPropertyValue::CLEPropertyValue( const long Value )
{
  m_IsSet = true ;
  m_ValueType = PT_INT ;
  m_Value.LongVal = Value;
};
CLEPropertyValue::CLEPropertyValue( const float Value )
{
  m_IsSet = true ;
  m_ValueType = PT_FLOAT ;
  m_Value.FloatVal = Value;
};
CLEPropertyValue::CLEPropertyValue( const char* Value)
{
  m_IsSet = true ;
  m_ValueType = PT_STRING ;
  m_Value.StringVal = strdup( Value);
};
CLEPropertyValue::CLEPropertyValue( const bool Value , PropertyType ValueType )
{
  m_IsSet = true ;
  m_ValueType = ValueType ;
  m_Value.BoolVal = Value;
};

//Destructor
CLEPropertyValue::~CLEPropertyValue()
{
  //If the value is a string we should free it.
  if ((m_ValueType == PT_STRING) && (m_IsSet))
      free (m_Value.StringVal) ;
};

CLEPropertyValue & CLEPropertyValue::operator=(const CLEPropertyValue & Other) {

  //If original value was a string and it was not copied from the default then
  //we should free it. If the string is the default then it means that the pointer is
  //to a string that was allocated and still pointed by the DefaultValuesArray and thus
  //should not be deallocated.
  if ((m_ValueType == PT_STRING) && (m_IsSet))
        free(  m_Value.StringVal );

  m_IsSet = true ;

  m_ValueType = Other.m_ValueType ;
  switch(m_ValueType)
  {
      case PT_INT:
          m_Value.LongVal = Other.m_Value.LongVal;
          break;
      case PT_FLOAT:
          m_Value.FloatVal = Other.m_Value.FloatVal;
          break;
      case PT_STRING:
          m_Value.StringVal = strdup( Other.m_Value.StringVal) ;
          break;
      case PT_BOOLEAN:
      case PT_BOOLEAN_STR:
          m_Value.BoolVal = Other.m_Value.BoolVal;
          break;
  };
  return (*this) ;

} ;

CLEPropertyValue & CLEPropertyValue::Set ( long Value)
{
  if ((m_ValueType == PT_STRING) && (m_IsSet))
    free (m_Value.StringVal) ;

  m_IsSet = true ;
  m_ValueType = PT_INT ;
  m_Value.LongVal = Value;
  return (*this) ;
};
CLEPropertyValue & CLEPropertyValue::Set ( float Value)
{
  if ((m_ValueType == PT_STRING) && (m_IsSet))
    free ( m_Value.StringVal ) ;

  m_IsSet = true ;
  m_ValueType = PT_FLOAT ;
  m_Value.FloatVal = Value;
  return (*this) ;
};
CLEPropertyValue & CLEPropertyValue::Set ( char* Value)
{
  if ((m_ValueType == PT_STRING) && (m_IsSet))
    free( m_Value.StringVal );

  m_IsSet = true ;
  m_ValueType = PT_STRING ;
  m_Value.StringVal = strdup( Value);
  return (*this) ;
};
CLEPropertyValue & CLEPropertyValue::Set ( bool Value, PropertyType PropType)
{
  if ((m_ValueType == PT_STRING) && (m_IsSet))
    free ( m_Value.StringVal );

  m_IsSet = true ;
  m_ValueType = PropType ;
  m_Value.BoolVal = Value;
  return (*this) ;
};

/****************************************/
/*  CLEPropertyAttributes methods       */
/****************************************/
CLEPropertyAttributes::CLEPropertyAttributes()
{
  m_Name = NULL ;
  m_LegalListOfValues = NULL ;
};

CLEPropertyAttributes::~CLEPropertyAttributes()
{
  if (m_Name) free (m_Name) ;
  if (m_LegalListOfValues) free (m_LegalListOfValues) ;
};

/****************************************/
/*  CLEPropertiesDictionary methods     */
/****************************************/

//Constructor
CLEPropertiesDictionary::CLEPropertiesDictionary()
{
  m_IndexCounter = 0 ;
};

//Constructor
CLEPropertiesDictionary::~CLEPropertiesDictionary()
{
  clearAndDestroy();
  //Delete default arrays
  delete [] DefaultValuesArray ;
  delete [] DefaultValuesArrayForCopy ;
};

int CLEPropertiesDictionary::RegisterProperty
  ( char*                             Name,
    CLEPropertyValue::PropertyType    Type,
    const CLEPropertyValue &          DefaultValue,
    char*                             LegalListOfValues,
    CLEPropertyValue::Level           StartLevel,
    CLEPropertyValue::Level           EndLevel)
{

  //Asserts when trying to register after the default arrays were created.
  //Once the default arrays were created no more registration is allowed.
  WlAssert (DefaultValuesArray==NULL) ;

	int rtn;

  RVCollectable* Key ;
  RVCollectableString* pPropName = new RVCollectableString(Name) ;

	if( RVCollectable* X0 = findValue( pPropName ) ) {
		WlAssert( dynamic_cast<CLEPropertyAttributes*>(X0) );
		CLEPropertyAttributes* pPropAttr = static_cast<CLEPropertyAttributes*>(X0);

		rtn = pPropAttr->m_Index;

	} else {
		
  CLEPropertyAttributes* pPropAttr = new CLEPropertyAttributes;

  pPropAttr->m_Name = strdup(Name) ;
  pPropAttr->m_Type = Type ;
  pPropAttr->m_DefaultValue = DefaultValue ;
  pPropAttr->m_LegalListOfValues = strdup(LegalListOfValues);
  pPropAttr->m_StartLevel = StartLevel;
  pPropAttr->m_EndLevel = EndLevel ;
  pPropAttr->m_Index = m_IndexCounter ;
		rtn = m_IndexCounter;
		++m_IndexCounter;

  //Insert to hash table
  Key = insertKeyAndValue ((RVCollectable*) pPropName,
                           (RVCollectable*) pPropAttr);

  //Duplicate property name in dictionary
  WlAssert (Key != NULL) ;

	}

  return (rtn) ;
};


void CLEPropertiesDictionary::RegisterObjectProperty
  ( char*                             Name,
    CLEPropertyValue::PropertyType    Type)
{

  //Asserts when trying to register after the default arrays were created.
  //Once the default arrays were created no more registration is allowed.
  WlAssert (DefaultValuesArray==NULL) ;

  RVCollectable* Key ;
  RVCollectableString* pPropName = new RVCollectableString(Name) ;
	if( RVCollectable* X0 = findValue( pPropName ) ) {
		WlAssert( dynamic_cast<CLEPropertyAttributes*>(X0) );
		CLEPropertyAttributes* pPropAttr = static_cast<CLEPropertyAttributes*>(X0);
	} else {
		
  CLEPropertyAttributes* pPropAttr = new CLEPropertyAttributes;

  pPropAttr->m_Name = strdup(Name) ;
  pPropAttr->m_Type = Type ;

  //Insert to hash table
  Key = insertKeyAndValue ((RVCollectable*) pPropName,
                           (RVCollectable*) pPropAttr);

  //Duplicate property name in dictionary
  WlAssert (Key != NULL) ;
	}

};


/****************************************/
/*      CLEVariablesData methods        */
/****************************************/
void CLEVariablesData::MessageReport (JSContext* Cx, int ErrorId, ...)
{
  const char* MsgFmt = WLERRGetErrorFormat(SYN_FILE_FRM_NAME, ErrorId);

  char errBuf[1000];
  memset(errBuf, 0, sizeof(errBuf));

  va_list ap;
  va_start(ap, ErrorId);
  try {
    vsprintf(errBuf, MsgFmt, ap);
  } catch (...) {
		WlAssert(0);
	}
  va_end(ap);

	LoadReport( Cx, WLLogMessage::Warning, errBuf );
};


void CLEVariablesData::CreateDefaultValuesArrays()
{
  int i ;
  int DictionaryLength = LEVariablePropertiesDictionary.DictionaryLength();
  size_t VariablesArrLen = DictionaryLength * sizeof(CLEPropertyValue) ;

  //This function should be called only once. If default array is already created raise assert.
  if (DefaultValuesArray)
    return;

  DefaultValuesArray = new CLEPropertyValue[DictionaryLength] ;
  DefaultValuesArrayForCopy = new CLEPropertyValue[DictionaryLength] ;
  //copy default values to the new default values array
  RVHashDictionaryIterator Itr(LEVariablePropertiesDictionary);
  CLEPropertyAttributes* pPropertyDefinition ;
  while (Itr()) {
    pPropertyDefinition = (CLEPropertyAttributes*)Itr.value() ;
		if( pPropertyDefinition->m_Type != CLEPropertyValue::PT_OBJECT )
    DefaultValuesArray[pPropertyDefinition->m_Index] = pPropertyDefinition->m_DefaultValue ;
  } ;
  //memcopy the default array to another default values array that will be used for the memcopy
  //when reseting the variables array. In this array the string values are pointing to
  //to the storage that was allocated in the original default array and thus the m_IsSet should be set
  //to false so it won't be freed in the destructors.
  memcpy (DefaultValuesArrayForCopy, DefaultValuesArray, VariablesArrLen);
  for (i=0; i < DictionaryLength; i++) DefaultValuesArrayForCopy[i].m_IsSet = false;

};

void CLEVariablesData::InitializeArrays()
{
  int DictionaryLength = LEVariablePropertiesDictionary.DictionaryLength();
  size_t VariablesArrLen = DictionaryLength * sizeof(CLEPropertyValue) ;
  m_ValuesArrayLength = DictionaryLength ;

  m_ValuesArray = new CLEPropertyValue[m_ValuesArrayLength] ;
  //memcopy the default array to the values array
  memcpy (m_ValuesArray, DefaultValuesArrayForCopy, VariablesArrLen);

};

// constructor
CLEVariablesData::CLEVariablesData(CLEVariablesData* Parent, CLEPropertyValue::Level Level)
{
  m_Parent = Parent;
  m_VariablesLevel = Level;
  m_ValuesArrayLength = 0 ;
  m_ValuesArray = NULL ;

  //Create default and variables array
  InitializeArrays();
};

//Copy constructor
CLEVariablesData::CLEVariablesData (const CLEVariablesData & other)
{
  int i ;

  m_ValuesArrayLength = 0 ;
  m_ValuesArray = NULL ;
  m_VariablesLevel = other.m_VariablesLevel ;
  m_Parent = other.m_Parent ;
  //Create default and variables array
  InitializeArrays();
  //Copy only the values that were set explicitly
  for (i=0 ; i < other.m_ValuesArrayLength ; i++)
      if (other.m_ValuesArray[i].m_IsSet)
          m_ValuesArray[i] = other.m_ValuesArray[i] ;
};


//Destructor
CLEVariablesData::~CLEVariablesData()
{
  //Deallocate variables array and default variables array
  delete [] m_ValuesArray ;

};

//Since the VariableDictionary class is not exported from the DLL,
//This wrapper function was created.
//Returns the index number of the new definition.
int CLEVariablesData::RegisterProperty
    ( char*                            Name,
      CLEPropertyValue::PropertyType   Type,
      const CLEPropertyValue &         DefaultValue,
      char*                            LegalListOfValues,
      CLEPropertyValue::Level          StartLevel,
      CLEPropertyValue::Level          EndLevel)
{
  return LEVariablePropertiesDictionary.RegisterProperty
          ( Name, Type, DefaultValue, LegalListOfValues, StartLevel, EndLevel) ;
};

void CLEVariablesData::RegisterObjectProperty
    ( char*                            Name,
      CLEPropertyValue::PropertyType   Type)
{
  LEVariablePropertiesDictionary.RegisterObjectProperty( Name, Type) ;
};


//Reset the variables array to its default values.
void CLEVariablesData::ResetVariablesArray ()
{
  int i ;

  //Deallocate strings that were set (not copied from default)
  for (i=0; i < m_ValuesArrayLength; i++) {
    if ((m_ValuesArray[i].m_ValueType == CLEPropertyValue::PT_STRING) && (m_ValuesArray[i].m_IsSet))
          free ( m_ValuesArray[i].m_Value.StringVal );
  };

  //memcopy the default array to the values array
  size_t VariablesArrLen = m_ValuesArrayLength * sizeof(CLEPropertyValue) ;
  memcpy (m_ValuesArray, DefaultValuesArrayForCopy, VariablesArrLen);
};


void CLEVariablesData::Set ( int Index, const CLEPropertyValue & Val)
{
  m_ValuesArray[Index] = Val ;
};

void CLEVariablesData::Set ( char* Name, const CLEPropertyValue & Val)
{
  int Index ;
  RVCollectableString Key(Name) ;
  CLEPropertyAttributes* Value = (CLEPropertyAttributes *) LEVariablePropertiesDictionary.findValue ((RVCollectable*) &Key);

  Index =  Value->m_Index;

  Set(Index, Val) ;
};

const CLEPropertyValue & CLEVariablesData::Get ( int Index, bool HierarchySearch) const
{
  if (!HierarchySearch)
    return m_ValuesArray[Index] ;
  else {
    //Hierarchy Search
    const CLEVariablesData* CurrVariables = this;
    //While not set and has a parent keep looking for a set variable.
    while ((!(CurrVariables->m_ValuesArray[Index].m_IsSet))
            && (CurrVariables->m_Parent))
      CurrVariables = CurrVariables->m_Parent;
    if (CurrVariables->m_ValuesArray[Index].m_IsSet)
      return CurrVariables->m_ValuesArray[Index] ;
    else
      //If nothing is set in the Hierarchy, return the default value of the current level.
      return m_ValuesArray[Index] ;
  };

};

const CLEPropertyValue & CLEVariablesData::Get ( char* Name, bool HierarchySearch) const
{
  int Index ;
  RVCollectableString Key(Name) ;
  CLEPropertyAttributes* Value = (CLEPropertyAttributes *) LEVariablePropertiesDictionary.findValue ((RVCollectable*) &Key);

  Index =  Value->m_Index;
  return Get (Index, HierarchySearch) ;
};

void CLEVariablesData::SetDoSyntaxChecking (JSContext* cx, bool Mode)
{
  m_DoSyntaxChecking = Mode ;
  if (Mode) {
		WLErrorException errExcp( SYN_FILE_FRM_NAME, SYN_WORKING_MODE );
		LoadReport(cx, errExcp );
	}
};

//This function converts the property value from the Java script type to the C++ type
//and sets the variables mechanism.
bool CLEVariablesData::JsValueToPropertyValue
          ( JSContext *Cx, jsval & JsValue,
            const CLEPropertyAttributes & PropAttr,
            CLEPropertyValue & PropVal )
{
  int st = 0 ; //0: success, 1: Illegal type, 2:JS conversion error/internal error
  char *StrValue ;

  //For performace reasons I assume that the PropVal variable is
  //set to the proper CLEPropertyValue::PropertyType.
  //I.E. for string property the PropVal.m_ValueType is already
  //PT_STRING. I can assume that because in the registration process
  //each property get its default value and thus set to the proper type.
  switch(PropAttr.m_Type) {
    //-------------------------
    // **** Integer values ****
    //-------------------------
   case CLEPropertyValue::PT_INT:
      int32 IntVal ;
      if (JS_ValueToInt32(Cx, JsValue, &IntVal) == JS_TRUE) {
         PropVal.m_Value.LongVal = (long)IntVal ;
         PropVal.m_IsSet = true ;
      }
      else
        st = 2 ; //JS conversion error
      break ;
    //-------------------------
    // **** Float values ****
    //-------------------------
    case CLEPropertyValue::PT_FLOAT:
      jsdouble DblValue ;
      if (JS_ValueToNumber(Cx, JsValue, &DblValue) == JS_TRUE) {
         PropVal.m_Value.FloatVal = (float)DblValue ;
         PropVal.m_IsSet = true ;
      }
      else
        st = 2 ; //JS conversion error
      break ;
    //-------------------------
    // **** Boolean values ****
    //-------------------------
    case CLEPropertyValue::PT_BOOLEAN:
    case CLEPropertyValue::PT_BOOLEAN_STR:
      StrValue = JS_GetStringBytes(JS_ValueToString(Cx, JsValue)) ;
      if (!strcmpi(StrValue, NO_STR_VALUE) || !strcmpi(StrValue, FALSE_STR_VALUE)) {
          PropVal.m_Value.BoolVal = false ;
          PropVal.m_IsSet = true ;
      }
      else if (!strcmpi(StrValue, YES_STR_VALUE) || !strcmpi(StrValue, TRUE_STR_VALUE)) {
          PropVal.m_Value.BoolVal = true ;
          PropVal.m_IsSet = true ;
      }
      else
        st = 1 ; //Illegal type
      break ;
    //-------------------------
    // **** String values ****
    //-------------------------
    case CLEPropertyValue::PT_STRING: {
      StrValue = JS_GetStringBytes(JS_ValueToString(Cx, JsValue)) ;

      //Delete old string value if exists
      if (PropVal.m_IsSet)
          free (PropVal.m_Value.StringVal) ;

      PropVal.m_Value.StringVal = strdup(StrValue) ;
      PropVal.m_IsSet = true ;
      //In syntax check mode check value against list of values
      if ((strlen(PropAttr.m_LegalListOfValues) > 0) &&
          (CLEVariablesData::m_DoSyntaxChecking )) {
        if (strstr( PropAttr.m_LegalListOfValues, StrValue) == NULL ) {
          //Value not found in list of values
          MessageReport (Cx, SYN_ILLEGAL_VALUE, StrValue, PropAttr.m_Name, PropAttr.m_LegalListOfValues) ;
        } ;
      };
      break ;
      } ;
    default :
      MessageReport (Cx, SYN_ILLEGAL_DICT_TYPE) ;
      st = 2 ; //internal error, Stop agenda
  } ;

  //if st==1 then it means that there was mismatch between the Java script type and
  //the expected property type.
  if ((st == 1) && CLEVariablesData::m_DoSyntaxChecking ) {
    //In syntax check mode display warning message
    char StrType[100];

    switch(PropAttr.m_Type ){
      case CLEPropertyValue::PT_FLOAT:
      case CLEPropertyValue::PT_INT:
        strcpy( StrType , "NUMBER");
        break;
      case CLEPropertyValue::PT_STRING:
        strcpy (StrType , "STRING") ;
        break ;
      case CLEPropertyValue::PT_BOOLEAN:
      case CLEPropertyValue::PT_BOOLEAN_STR:
        strcpy (StrType , "BOOLEAN (legal values are: \"Yes\"/\"No\" or true/false)") ;
        break;

      default:
        strcpy(StrType, "Severe error - unknown type");
    };
    MessageReport(Cx, SYN_WRONG_TYPE, PropAttr.m_Name, StrType) ;
  };

  if (st==2) //return false only for Js conversion error and internal error to stop the agenda
    return false ;
  else
    return true  ;

};


bool CLEVariablesData::PropertyValueToJsValue
      ( JSContext *Cx, jsval & JsValue,
        const CLEPropertyAttributes & PropAttr,
        const CLEPropertyValue & PropVal )
{
  bool St = true ;

  switch(PropVal.m_ValueType){
      case CLEPropertyValue::PT_INT:
        JsValue = INT_TO_JSVAL(PropVal.m_Value.LongVal) ;
        break ;
      case CLEPropertyValue::PT_FLOAT:
        JS_NewDoubleValue(Cx, PropVal.m_Value.FloatVal, &JsValue);
        break ;
      case CLEPropertyValue::PT_STRING:
    		JSString* Str ;
        Str = JS_NewStringCopyZ(Cx,PropVal.m_Value.StringVal);
        JsValue = STRING_TO_JSVAL(Str) ;
        break ;
      case CLEPropertyValue::PT_BOOLEAN:
        JsValue = BOOLEAN_TO_JSVAL(PropVal.m_Value.BoolVal) ;
        break ;
      case CLEPropertyValue::PT_BOOLEAN_STR:
    		JSString* BoolStr ;
        if (PropVal.m_Value.BoolVal)
            BoolStr = JS_NewStringCopyZ(Cx,YES_STR_VALUE);
        else
            BoolStr = JS_NewStringCopyZ(Cx,NO_STR_VALUE);
        JsValue = STRING_TO_JSVAL(BoolStr) ;
        break ;
      default:
        St=false;
  };

  return St;
};


//This function is registered in the Java script for the wlGlobals and wlLocals
JSBool CLEVariablesData::JsSet (JSContext* cx, JSObject* obj, jsval id, jsval* vp)
{

	// check that id is a string ???
	if (!JSVAL_IS_STRING(id))
		return JS_TRUE;

  //Get pointer to Variable array from the private data of the JS object.
  CLEVariablesData* pVariablesData = (CLEVariablesData*)JS_GetPrivate(cx, obj);
	if (!pVariablesData) {
    return JS_TRUE;
  };

  return pVariablesData->JsVarSet (cx, obj, id, vp) ;

};

//This function is registered in the Java script for the wlGlobals and wlLocals
JSBool CLEVariablesData::JsGet (JSContext* cx, JSObject* obj, jsval id, jsval* vp)
{
	if (!JSVAL_IS_STRING(id))
		return JS_TRUE;


	if( RVJS_IsInstanceOf(cx, *vp , GetPropertyObjectJSClass()) )
		return JS_TRUE;// *vp is a property object.

	//case *vp isn't an object that is hanged on the local js object.
	//Get pointer to Variable array from the private data of the JS object.
  CLEVariablesData* pVariablesData = (CLEVariablesData*)JS_GetPrivate(cx, obj);
	if (!pVariablesData) {
    return JS_TRUE;
  };

  return pVariablesData->JsVarGet ( cx, obj, id, vp) ;

};

CLEVariablesData::SyntaxCheckStatus CLEVariablesData::SyntaxCheck
          (JSContext* cx, JSObject* obj, jsval id, jsval* vp,
           bool SetMethod, CLEPropertyAttributes* & PropAttr)
{

  //  Convert id into name (char *)
	char* Name = JS_GetStringBytes(JSVAL_TO_STRING(id));

  //search this property in our dictionary and get its attributes. If not found, return.
  RVCollectableString Key(Name) ;
  PropAttr = (CLEPropertyAttributes*) LEVariablePropertiesDictionary.findValue ((RVCollectable*) &Key );
  if (!PropAttr) {
    if (m_DoSyntaxChecking ) {
      //In syntax check mode check if this property exists ignoring case sensitivity
      RVHashDictionaryIterator Itr(LEVariablePropertiesDictionary);
      RVCollectableString* pKeyName ;
      bool FoundKey = false;
      while (Itr() && !FoundKey) {
        pKeyName = (RVCollectableString*)Itr.key() ;
        if (!strcmpi(pKeyName->data(), Name))
              FoundKey = true ;
      };
      if (FoundKey)
        MessageReport(cx, SYN_CASE_SENSITIVE_ERROR, Name, pKeyName->data()) ;
    } ;
    //Property was not found
    return PropertyNotFound ;
  }

  // Check if the set/get is done in the right property level (i.e global,locals,http)
  // If the property is an object no checking is done
	if ( PropAttr->m_Type != CLEPropertyValue::PT_OBJECT ) {

  if ((m_VariablesLevel < PropAttr->m_StartLevel) ||
      (m_VariablesLevel > PropAttr->m_EndLevel)) {
    if (m_DoSyntaxChecking) {
        MessageReport (cx, SYN_WRONG_LEVEL, Name, JS_GetClass(cx,obj)->name) ;
    } ;
    *vp = JSVAL_NULL;
    return WrongLevel;
		}
	}

  return OK ;
};

JSBool CLEVariablesData::JsVarSet (JSContext* cx, JSObject* obj, jsval id, jsval* vp)
{
  CLEPropertyAttributes* PropAttr ;

	SyntaxCheckStatus st = SyntaxCheck (cx, obj, id, vp, true, PropAttr);

	switch (st) {
		case OK:
			if( PropAttr->m_Type == CLEPropertyValue::PT_OBJECT ) {
				// try to set an object property.
				WLErrorException e(LoadExceptionFrmFile, 330, JS_GetStringBytes(JSVAL_TO_STRING(id)));
				return LoadReport(cx, e);
			} else {
				// try to set a property

			//Set the variable if the type matches.
			if (JsValueToPropertyValue ( cx, *vp, *PropAttr, m_ValuesArray[PropAttr->m_Index] ) )
				return JS_TRUE ;
			else
				return JS_FALSE ;
			}
		case WrongLevel:
			return JS_TRUE;
		case PropertyNotFound: //this option must be impossible if(!JSVAL_IS_STRING(id))
			if( WLJSIsGlobalPerThreadObject( cx, obj ) )
			//if( m_VariablesLevel == 	CLEPropertyValue::PL_GLOBALS )
			{ //if this is a wlGlobals

				JSObject* proto = JS_GetPrototype( cx, obj );
				if(!proto) { //practically impossible, per-thread wlGlobals have a prototype
					return(JS_FALSE);
					}
				JSString* name = JSVAL_TO_STRING(id); //id is knows to be string
				//return JS_SetProperty( cx, proto, JS_GetStringBytes(name), vp );
				return JS_SetUCProperty( cx, proto,
					JS_GetStringChars(name),
					JS_GetStringLength(name), vp );
				}
			else {
				return(JS_TRUE);
				}
		}
		throw 0;
};

JSBool CLEVariablesData::JsVarGet (JSContext* cx, JSObject* obj, jsval id, jsval* vp)
{

  CLEPropertyAttributes* PropAttr;

	SyntaxCheckStatus st = SyntaxCheck (cx, obj, id, vp, false, PropAttr);//PropAttr is being sets in that call.(from the dictionary)

	switch (st) {
		case OK:
			if( PropAttr->m_Type == CLEPropertyValue::PT_OBJECT ) {
			//property is a object.
				RVCString propName = JS_GetStringBytes(JSVAL_TO_STRING(id));
				JSObject  *propertyObj = NewPropertyObject(cx, propName, this);
		 	  if (!JS_DefineProperty( cx, obj, propName.data() ,OBJECT_TO_JSVAL(propertyObj),NULL,NULL,JSPROP_ENUMERATE ))
					return false;
				*vp = OBJECT_TO_JSVAL(propertyObj);
			} else {

			//Get the variable and convert it to JS type.
			PropertyValueToJsValue ( cx, *vp, *PropAttr, m_ValuesArray[PropAttr->m_Index]  )  ;
			}
			return JS_TRUE;
		case WrongLevel:
			return JS_TRUE;
		case PropertyNotFound: //this option must be impossible if(!JSVAL_IS_STRING(id))
			if( WLJSIsGlobalPerThreadObject( cx, obj ) )
			//if( m_VariablesLevel == 	CLEPropertyValue::PL_GLOBALS )
			{ //if this is a wlGlobals
				JSObject* proto = JS_GetPrototype( cx, obj );
				if(!proto) { //practically impossible, per-thread wlGlobals have a prototype
					return(JS_FALSE);
					}
				JSString* name = JSVAL_TO_STRING(id); //id is knows to be string
				//return JS_GetProperty( cx, proto, JS_GetStringBytes(name), vp );
				return JS_GetUCProperty( cx, proto,
					JS_GetStringChars(name),
					JS_GetStringLength(name), vp );
				}
			else {
				return(JS_TRUE);
				}
		}

		throw 0;
};



//This function is registered in the Java script for the wlPropertyObject_class
JSBool CLEVariablesData::JsGetPropertyObj (JSContext* cx, JSObject* obj, jsval id, jsval* vp)
{
	if (!JSVAL_IS_STRING(id))
		return JS_TRUE;

	if( RVJS_IsInstanceOf(cx, *vp , GetPropertyObjectJSClass()) )
		return JS_TRUE;// *vp is a property object.

	//case *vp isn't an object that is hanged on the local js object.
	//Get pointer to Variable array from the private data of the JS object.
  CPropertyObjectPrivateData* pCPropertyObjectPrivateData = (CPropertyObjectPrivateData*)JS_GetPrivate(cx, obj);
	if (!pCPropertyObjectPrivateData) {
    return JS_TRUE;
  }
	CLEVariablesData* pVariablesData = pCPropertyObjectPrivateData->GetVariablesData();

	RVCString propObjName = pCPropertyObjectPrivateData->GetName();

  return pVariablesData->JsVarGetPropertyObj ( cx, obj, id, vp, propObjName ) ;
}

//This function is registered in the Java script for the wlPropertyObject_class
JSBool CLEVariablesData::JsSetPropertyObj (JSContext* cx, JSObject* obj, jsval id, jsval* vp)
{
	
	// check that id is a string ???
	if (!JSVAL_IS_STRING(id))
		return JS_TRUE;

	//Get pointer to Variable array from the private data of the JS object.
  CPropertyObjectPrivateData* pCPropertyObjectPrivateData = (CPropertyObjectPrivateData*)JS_GetPrivate(cx, obj);
	if (!pCPropertyObjectPrivateData) {
		return JS_TRUE;
  }
	CLEVariablesData* pVariablesData = pCPropertyObjectPrivateData->GetVariablesData();

	RVCString propObjName = pCPropertyObjectPrivateData->GetName();

  return pVariablesData->JsVarSetPropertyObj (cx, obj, id, vp, propObjName) ;
}

JSBool CLEVariablesData::JsVarGetPropertyObj (JSContext* cx, JSObject* obj, jsval id, jsval* vp, RVCString & objPropName )
{

  CLEPropertyAttributes* PropAttr;

	RVCString propName = JS_GetStringBytes(JSVAL_TO_STRING(id));
	RVCString fullPropName = MakeKeyToPropertiesDictionary ( objPropName , propName );
	jsval fullId = STRING_TO_JSVAL(JS_NewStringCopyZ(cx, fullPropName.data()));

	SyntaxCheckStatus st = SyntaxCheck (cx, obj, fullId, vp, false, PropAttr);//PropAttr is being sets in that call.(from the dictionary)

	switch (st) {
		case OK:
			if( PropAttr->m_Type == CLEPropertyValue::PT_OBJECT ) {
			//property is a object.
				JSObject  *propertyObj = NewPropertyObject(cx, fullPropName, this);
				*vp = OBJECT_TO_JSVAL(propertyObj);
		 	  if (!JS_DefineProperty( cx, obj, propName.data() ,OBJECT_TO_JSVAL(propertyObj),NULL,NULL,JSPROP_ENUMERATE))
					return false;
			} else {

			//Get the variable and convert it to JS type.
			PropertyValueToJsValue ( cx, *vp, *PropAttr, m_ValuesArray[PropAttr->m_Index]  )  ;
			}
			return JS_TRUE;
		case WrongLevel:
			return JS_TRUE;
		case PropertyNotFound:
			return JS_TRUE;
		}

		throw 0;
};

JSBool CLEVariablesData::JsVarSetPropertyObj (JSContext* cx, JSObject* obj, jsval id, jsval* vp, RVCString & objPropName )
{

  CLEPropertyAttributes* PropAttr;

	RVCString propName = JS_GetStringBytes(JSVAL_TO_STRING(id));
	RVCString fullPropName = MakeKeyToPropertiesDictionary ( objPropName , propName );
	jsval fullId = STRING_TO_JSVAL(JS_NewStringCopyZ(cx, fullPropName.data()));

	SyntaxCheckStatus st = SyntaxCheck (cx, obj, fullId, vp, false, PropAttr);//PropAttr is being sets in that call.(from the dictionary)

	switch (st) {
		case OK:
			if( PropAttr->m_Type == CLEPropertyValue::PT_OBJECT ) {
				// try to set an object property.
				WLErrorException e(LoadExceptionFrmFile, 330, JS_GetStringBytes(JSVAL_TO_STRING(id)));
				return LoadReport(cx, e);
			} else {
				// try to set a property

			//Set the variable if the type matches.
			if (JsValueToPropertyValue ( cx, *vp, *PropAttr, m_ValuesArray[PropAttr->m_Index] ) )
				return JS_TRUE ;
			else
				return JS_FALSE ;
			}
		case WrongLevel:
			return JS_TRUE;
		case PropertyNotFound: //this option must be impossible if(!JSVAL_IS_STRING(id))
			return JS_TRUE;
	}
		throw 0;
}


RVCString CLEVariablesData::MakeKeyToPropertiesDictionary ( const RVCString & objPropertyName,const RVCString & propertyName ) {
	RVCString fullPropertyName;
	fullPropertyName += objPropertyName;
	fullPropertyName +=".";
	fullPropertyName += propertyName;
	return fullPropertyName; 
}

