/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <stdio.h>
//#include <fstream.h>

#include "rvjsapi.h"
#include "tracedll.h"

#include "associativearray.h"
#include "loadexceptions.h"
#include "loadenginestatistics.h"
#include "loadengineglobals.h"
#include "loadgeneratorslave.h"
#include "loadthreadsmanager.h"
#include "loadthreadinstance.h"

#include "loadgeneratorscript.h"

#include "managerscriptdata.h"
#include "loadenginedecs.h"
#include "wljsjavaapplet.h"

#include "loadengineselistener.h"
#include "wlleprotocol.h"
#include "loadengineinifileproperties.h"
#include "loadenginevariablesdata.h"
#include "tvenv.h"
#include "rtamscript.h"
#include "wljsruntime.h"

#include "Log.h"

//////////////////////////////// CUserLockObject ///////////////////////////////

CUserLockObject::CUserLockObject():
	m_Count(0),
	m_Owner(-1) {
}

void	CUserLockObject::Lock(
	int ThreadNum,
	JSContext* cx)
{

	if( ThreadNum == m_Owner ) {
		++m_Count;
	}
	else {
		{
			CJSGCRequestSuspendResume GCRelease(cx);
			m_Lock.acquire();
		}
		m_Owner = ThreadNum;
		m_Count = 1;
	}
}

void CUserLockObject::Unlock(
	int ThreadNum,
	JSContext* cx )
{

	if( ( ThreadNum != m_Owner ) || ( 0 == m_Count ) ) {
		LoadReport( cx, WLLogMessage::Warning, "Calling Unlock() with no Lock()" );
		return;
	}

	--m_Count;
	if( 0 == m_Count ) {
		m_Owner = -1;
		m_Lock.release();
	}
}


bool CUserLockObject::CheckUnlock(
	int ThreadNum )
{
	if( ThreadNum == m_Owner ) {
		m_Count = 0;
		m_Owner = -1;
		m_Lock.release();
		return true;
	}
	return false;
}

iLEVCJSThread::iLEVCJSThread()
{}

iLEVCJSThread::~iLEVCJSThread()
{}


////////////////////////////////////////////////////////////////
// Moved from Thread Data
////////////////////////////////////////////////////////////////
CLoadGeneralThreadInstanceScript::CLoadGeneralThreadInstanceScript(CLoadGeneralThreadInstance* pI, CWLJSScript& rS):
  m_MyThreadInstance(pI),
	m_MyJSThread(rS, true ),
  m_RoundNum(0),
  m_JSThreadInitDone(false)
{
}

CLoadGeneralThreadInstanceScript::~CLoadGeneralThreadInstanceScript()
{
}

void CLoadGeneralThreadInstanceScript::vEnterJava( const char* rClassDescriptionName, const char* rMemberName, tJavaCallType rCallType )
{
	m_MyJSThread.EnterJava();
  
	if( (sJavaCallMethod == rCallType) && !m_MyJSThread.IsDisabledJavaDefaultStatistics()) {
		char aPositionName[256];
		sprintf(aPositionName, "%s::%s (Timer)", rClassDescriptionName, rMemberName );
		resetTimer(aPositionName);
	}
}

void CLoadGeneralThreadInstanceScript::vExitJava( JSBool rOK, const char* rClassDescriptionName, const char* rMemberName, tJavaCallType rCallType  )
{
	m_MyJSThread.ExitJava();

	if( (sJavaCallMethod == rCallType) && !m_MyJSThread.IsDisabledJavaDefaultStatistics()) {
		char aPositionName[256];
		if(rOK) { //ok is the return value of the function : invoke_java_method succeeded
  		sprintf(aPositionName, "%s::%s (Timer)", rClassDescriptionName, rMemberName );
			sendTimer(aPositionName);
			sprintf(aPositionName, "%s::%s (success)", rClassDescriptionName, rMemberName );
		} else {
			sprintf( aPositionName, "%s::%s (failure)", rClassDescriptionName, rMemberName );
		}
		addCounter(aPositionName);
	}
}

void CLoadGeneralThreadInstanceScript::reportApply(
	iReport& rReport ) {

	m_MyThreadInstance->reportApply( rReport );
	}	


void CLoadGeneralThreadInstanceScript::reportLog(
	iReport& rReport ) 
{
	m_MyThreadInstance->reportLog( rReport );
}	

void CLoadGeneralThreadInstanceScript::CreateApplet(JSContext*                cx,
                                                    const char*               appletClassName,
                        										        const char*               appletCodeBase,
                                                    const char*               appletDocumentBase,
                          											    RVHashDictionary*         appletParams,
                                                    CWLJSJavaApplet*&         javaApplet){
  m_MyThreadInstance->CreateApplet(cx,
                                   this,
                                   appletClassName,
                                   appletCodeBase,
                                   appletDocumentBase,
                                   appletParams,
                                   javaApplet);
}

bool CLoadGeneralThreadInstanceScript::ClassAppletExists(JSContext*  cx,
                                 const char* appletClassName,
                                 bool& classAppletExists){
  return m_MyThreadInstance->ClassAppletExists(cx, appletClassName, classAppletExists);
}

bool CLoadGeneralThreadInstanceScript::AddJavaStream(JSContext*  cx,
                             const char* streamName,
                             const char* codeBase,
                             RVCString& javaStream){
  return m_MyThreadInstance->AddJavaStream(cx, streamName, codeBase, javaStream);
}

bool CLoadGeneralThreadInstanceScript::JavaStreamExists(const char* streamName){
  return m_MyThreadInstance->JavaStreamExists(streamName);
}

void CLoadGeneralThreadInstanceScript::VerificationSetFailureReason(const char* reason, const char* description, double time, bool isEvent ){
  m_MyThreadInstance->VerificationSetFailureReason( reason,  description, time, isEvent );
}

void CLoadGeneralThreadInstanceScript::VerificationRecordEvent(const char* reason, const char* description, double time, WLLogMessage::Severity severity, const char* extraDataType, void* extraData, unsigned long len, bool IsLog ){
  m_MyThreadInstance->VerificationRecordEvent( reason,  description, time, severity , extraDataType , extraData, len, IsLog );
}

bool CLoadGeneralThreadInstanceScript::VerificationBeginTransaction(const char* transname, double time, const char* extraDataType, void* extraData, unsigned long len){
  return m_MyThreadInstance->VerificationBeginTransaction( transname, time, extraDataType, extraData, len);
}

bool CLoadGeneralThreadInstanceScript::VerificationEndTransaction(const char* transname, double time, WLLogMessage::Severity severity, bool save, bool SendTimers, bool IsLog ){
  return m_MyThreadInstance->VerificationEndTransaction( transname, time,  severity, save, SendTimers, IsLog );
}

bool CLoadGeneralThreadInstanceScript::VerificationEndTransactionFailure(const char* transname, double time, WLLogMessage::Severity severity,const char* reason, bool SendTimers, bool IsLog, bool isFailureEvent ){
  return m_MyThreadInstance->VerificationEndTransactionFailure( transname,  time, severity, reason, SendTimers, IsLog, isFailureEvent );
}

int  CLoadGeneralThreadInstanceScript::VerificationGetTransactionDepth(){
  return m_MyThreadInstance->VerificationGetTransactionDepth();
}

const char* CLoadGeneralThreadInstanceScript::VerificationGetCurrentTransaction(){
  return m_MyThreadInstance->VerificationGetCurrentTransaction();
}
void CLoadGeneralThreadInstanceScript::endMultiThread(){
  m_MyThreadInstance->endMultiThread();
}

void CLoadGeneralThreadInstanceScript::reportApplyEvent( WLLogMessage::Severity sev, const char* msg){
  m_MyThreadInstance->reportApplyEvent(sev, msg);
}

bool CLoadGeneralThreadInstanceScript::report( WLLogMessage::Severity sev, const char* msg){
  return m_MyThreadInstance->report(sev, msg);
}

bool CLoadGeneralThreadInstanceScript::reportTime( WLLogMessage::Severity sev, const char* msg, double Time, WLLogMessage::Severity ErrorInTransTree  ){
  return m_MyThreadInstance->reportTime(sev, msg, Time, ErrorInTransTree);
}

bool CLoadGeneralThreadInstanceScript::report( WLErrorException& exc){
  return m_MyThreadInstance->report(exc);
}

bool CLoadGeneralThreadInstanceScript::reportTime( WLErrorException& exc, double Time ){
  return m_MyThreadInstance->reportTime(exc, Time);
}

bool CLoadGeneralThreadInstanceScript::reportMultiThread( iReportInfo& info){
  return m_MyThreadInstance->reportMultiThread(info);
}

bool CLoadGeneralThreadInstanceScript::reportMultiThread( WLLogMessage::Severity sev, const char* msg){
  return m_MyThreadInstance->reportMultiThread(sev, msg);
}

bool CLoadGeneralThreadInstanceScript::reportMultiThread( WLErrorException& exc){
  return m_MyThreadInstance->reportMultiThread(exc);
}

CLEVariablesData* CLoadGeneralThreadInstanceScript::GetCurrentLoadEngineVariablesData(){
  return m_MyThreadInstance->GetCurrentLoadEngineVariablesData();
}

WLEThreadType CLoadGeneralThreadInstanceScript::GetThreadType(){
  return m_MyThreadInstance->GetThreadType();
}

unsigned long CLoadGeneralThreadInstanceScript::TotalRoundNum(){
  return m_MyThreadInstance->TotalRoundNum();
}

unsigned long CLoadGeneralThreadInstanceScript::CurrentScriptRoundNum(){
  return m_MyThreadInstance->CurrentScriptRoundNum();
}

unsigned long CLoadGeneralThreadInstanceScript::GetRoundNumScriptInSlave(){
  return m_MyThreadInstance->GetRoundNumScriptInSlave();
}

unsigned long CLoadGeneralThreadInstanceScript::GetRoundNumMixInSlave(){
  return m_MyThreadInstance->GetRoundNumMixInSlave();
}

void CLoadGeneralThreadInstanceScript::OnSevereError(){
  m_MyThreadInstance->OnSevereError();
}

IGeneralGlobalsRequestHandler* CLoadGeneralThreadInstanceScript::GetGlobalsRequestHandler(){
	return m_MyThreadInstance->GetGlobalsRequestHandler();
}

int CLoadGeneralThreadInstanceScript::GetLGThreadNum(){
  return m_MyThreadInstance->GetLGThreadNum();
}

WLLogMessage::Severity CLoadGeneralThreadInstanceScript::getErrorSeverity(){
  return m_MyThreadInstance->getErrorSeverity();
}

bool CLoadGeneralThreadInstanceScript::InScript(){
  return m_MyThreadInstance->InScript();
}

bool CLoadGeneralThreadInstanceScript::IsOneClient(){
  return m_MyThreadInstance->IsOneClient();
}

bool CLoadGeneralThreadInstanceScript::IsLoadServer(){
  return m_MyThreadInstance->IsLoadServer();
}

bool CLoadGeneralThreadInstanceScript::IsRTAM(){
  return m_MyThreadInstance->IsRTAM();
}

void CLoadGeneralThreadInstanceScript::UserLock(){
  m_MyThreadInstance->UserLock();
}

void CLoadGeneralThreadInstanceScript::UserUnlock(){
  m_MyThreadInstance->UserUnlock();
}

int CLoadGeneralThreadInstanceScript::GetAgendaNumRunning(){
  return m_MyThreadInstance->GetAgendaNumRunning();
}

const char* CLoadGeneralThreadInstanceScript::GetLoadGeneratorNo(){
  return m_MyThreadInstance->GetLoadGeneratorNo();
}

int CLoadGeneralThreadInstanceScript::GetErrorCounter(){
  return m_MyThreadInstance->GetErrorCounter();
}

int CLoadGeneralThreadInstanceScript::GetWarningCounter(){
  return m_MyThreadInstance->GetWarningCounter();
}

void CLoadGeneralThreadInstanceScript::IncrementErrorCounter(){
  m_MyThreadInstance->IncrementErrorCounter();
}

void CLoadGeneralThreadInstanceScript::IncrementWarningCounter(){
  m_MyThreadInstance->IncrementWarningCounter();
}

const char* CLoadGeneralThreadInstanceScript::GetAgendaName(){
   return m_MyThreadInstance->GetAgendaName();
}

CGeneralProcess& CLoadGeneralThreadInstanceScript::fGetProcess(){
  return m_MyThreadInstance->fGetProcess();
}

int CLoadGeneralThreadInstanceScript::GetSlaveThreadNum(){
  return m_MyThreadInstance->GetSlaveThreadNum();
}

SynchronizationPointStatus CLoadGeneralThreadInstanceScript::SynchronizationPoint(int timeoutmillis){
  return m_MyThreadInstance->SynchronizationPoint(timeoutmillis);
}

iLEJSMachineThread& CLoadGeneralThreadInstanceScript::vGetMachineThread()
{
	return m_MyThreadInstance->vGetMachineThread();
}

iSession& CLoadGeneralThreadInstanceScript::vGetSession()
{
  return m_MyThreadInstance->vGetSession();
}

void CLoadGeneralThreadInstanceScript::ToggleJavaDefaultStatistics( bool rValue ) {
	m_MyJSThread.ToggleJavaDefaultStatistics(rValue);
}

bool CLoadGeneralThreadInstanceScript::IsDisabledJavaDefaultStatistics() {
	return m_MyJSThread.IsDisabledJavaDefaultStatistics();
}

void CLoadGeneralThreadInstanceScript::EnterJava() {
	m_MyJSThread.EnterJava();
}

void CLoadGeneralThreadInstanceScript::ExitJava() {
	m_MyJSThread.ExitJava();
}

bool CLoadGeneralThreadInstanceScript::isAbort(bool CheckInScript) {
  return m_MyThreadInstance->isAbort(CheckInScript);
}

void CLoadGeneralThreadInstanceScript::sleep(int milisec) {

  m_MyThreadInstance->sleep(milisec);
}

bool CLoadGeneralThreadInstanceScript::BeforeSleep(long int Time){
	ProtocolBeforeSleep(Time);
  bool ret = vBeforeSleep(Time);
  return ret;
}

void CLoadGeneralThreadInstanceScript::AfterSleep(long int Time){
  vAfterSleep();
	ProtocolAfterSleep(Time);
}

void CLoadGeneralThreadInstanceScript::InitProtocols(CWLJSThread& parent, IGeneralManagerScriptData& pMSD, JSContext* cx, JSObject* obj, CLEVariablesData* pVariablesData){

  iLEProtocolContainer& aProtocolAgendaInMixContainer = pMSD.GetLEProtocolAgendaInMixContainer();

	WlTraceMsg("Calling NewProtocolVCInMix for all protocols");
  m_ProtocolVCInMixContainer.InitProtocolContainer(aProtocolAgendaInMixContainer.NumProtocols());
  for(WLProtocolIndex indx=0; indx< aProtocolAgendaInMixContainer.NumProtocols(); indx++) {
    if (aProtocolAgendaInMixContainer.ProtocolExists(indx)){
    WlAssert(dynamic_cast<iLEProtocolAgendaInMix*>(&(aProtocolAgendaInMixContainer.GetProtocolAtIndex(indx))));
    iLEProtocolVCInMix& aProtocolVCInMix = static_cast<iLEProtocolAgendaInMix&>(aProtocolAgendaInMixContainer.GetProtocolAtIndex(indx)).NewProtocolVCInMix();
    aProtocolVCInMix.SetParent(parent);
    aProtocolVCInMix.Init(this);
      if (!aProtocolVCInMix.JSInitializeForClient(cx, obj, pVariablesData)){
  		WlErrorMsg("Error 1 in function IGeneralManagerScriptData::NewProtocolVCInMix");
      WlAssert(0);
    }
    else {
      m_ProtocolVCInMixContainer.InsertProtocolAtIndex(indx, aProtocolVCInMix);
    }
  }
}
}

bool CLoadGeneralThreadInstanceScript::ProtocolOnThreadActivation(){

	WlTraceMsg("Calling ProtocolOnThreadActivation for all protocols");
	for(WLProtocolIndex indx=0; indx< m_ProtocolVCInMixContainer.NumProtocols(); indx++) {
    if (m_ProtocolVCInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolVCInMix*>(&(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolVCInMix&>(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx)).OnThreadActivation()){
	  		WlTraceMsg("Error 1 in function CLoadGeneralThreadInstanceScript::ProtocolOnThreadActivation");
        return false;
		  }
    }
  }
  return true;
}

bool CLoadGeneralThreadInstanceScript::ProtocolBeforeInitClient(){

	WlTraceMsg("Calling ProtocolBeforeInitClient for all protocols");
	for(WLProtocolIndex indx=0; indx< m_ProtocolVCInMixContainer.NumProtocols(); indx++) {
    if (m_ProtocolVCInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolVCInMix*>(&(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolVCInMix&>(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx)).BeforeInitClient()){
	  		WlTraceMsg("Error 1 in function CLoadGeneralThreadInstanceScript::ProtocolBeforeInitClient");
        return false;
		  }
    }
  }
  return true;
}

bool CLoadGeneralThreadInstanceScript::ProtocolAfterInitClient(){

	WlTraceMsg("Calling ProtocolBeforeInitClient for all protocols");
	for(WLProtocolIndex indx=0; indx< m_ProtocolVCInMixContainer.NumProtocols(); indx++) {
    if (m_ProtocolVCInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolVCInMix*>(&(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolVCInMix&>(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx)).AfterInitClient()){
	  		WlTraceMsg("Error 1 in function CLoadGeneralThreadInstanceScript::ProtocolBeforeInitClient");
        return false;
		  }
    }
  }
  return true;
}

bool CLoadGeneralThreadInstanceScript::ProtocolBeforeRound(){

	WlTraceMsg("Calling ProtocolBeforeRound for all protocols");
	for(WLProtocolIndex indx=0; indx< m_ProtocolVCInMixContainer.NumProtocols(); indx++) {
    if (m_ProtocolVCInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolVCInMix*>(&(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolVCInMix&>(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx)).BeforeRound()){
	  		WlTraceMsg("Error 1 in function CLoadGeneralThreadInstanceScript::ProtocolBeforeRound");
        return false;
      }
		}
  }
  return true;
}

bool CLoadGeneralThreadInstanceScript::ProtocolAfterRound(){

	WlTraceMsg("Calling ProtocolAfterRound for all protocols");
	for(WLProtocolIndex indx=0; indx< m_ProtocolVCInMixContainer.NumProtocols(); indx++) {
    if (m_ProtocolVCInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolVCInMix*>(&(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolVCInMix&>(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx)).AfterRound()){
	  		WlTraceMsg("Error 1 in function CLoadGeneralThreadInstanceScript::ProtocolAfterRound");
        return false;
      }
		}
  }
  return true;
}

bool CLoadGeneralThreadInstanceScript::ProtocolBeforeTerminateClient(){

	WlTraceMsg("Calling ProtocolBeforeTerminateClient for all protocols");
	for(WLProtocolIndex indx=0; indx< m_ProtocolVCInMixContainer.NumProtocols(); indx++) {
    if (m_ProtocolVCInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolVCInMix*>(&(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolVCInMix&>(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx)).BeforeTerminateClient()){
	  		WlTraceMsg("Error 1 in function CLoadGeneralThreadInstanceScript::ProtocolAfterTerminateClient");
        return false;
      }
		}
  }
  return true;
}

bool CLoadGeneralThreadInstanceScript::ProtocolAfterTerminateClient(){

	WlTraceMsg("Calling ProtocolAfterTerminateClient for all protocols");
  for (WLProtocolIndex indx = m_ProtocolVCInMixContainer.NumProtocols() - 1; indx >= 0; indx--){
    if (m_ProtocolVCInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolVCInMix*>(&(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolVCInMix&>(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx)).AfterTerminateClient()){
	  		WlTraceMsg("Error 1 in function CLoadGeneralThreadInstanceScript::ProtocolAfterTerminateClient");
        return false;
      }
		}
  }
  return true;
}

bool CLoadGeneralThreadInstanceScript::ProtocolOnThreadTermination(){

	WlTraceMsg("Calling ProtocolOnThreadTermination for all protocols");
	for(WLProtocolIndex indx=0; indx< m_ProtocolVCInMixContainer.NumProtocols(); indx++) {
    if (m_ProtocolVCInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolVCInMix*>(&(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolVCInMix&>(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx)).OnThreadTermination()){
			  WlTraceMsg("Error 1 in function CLoadGeneralThreadInstanceScript::ProtocolOnThreadTermination");
        return false;
      }
		}
  }
  return true;
}

bool CLoadGeneralThreadInstanceScript::ProtocolOnThreadRestart(){

	WlTraceMsg("Calling ProtocolOnThreadTermination for all protocols");
	for(WLProtocolIndex indx=0; indx< m_ProtocolVCInMixContainer.NumProtocols(); indx++) {
    if (m_ProtocolVCInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolVCInMix*>(&(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolVCInMix&>(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx)).OnThreadRestart()){
			  WlTraceMsg("Error 1 in function CLoadGeneralThreadInstanceScript::ProtocolOnThreadRestart");
        return false;
      }
		}
  }
  return true;
}

void CLoadGeneralThreadInstanceScript::ProtocolExceptionHandler(std::exception& e){

	WlTraceMsg("Calling ProtocolExceptionHandler for all protocols");
	for(WLProtocolIndex indx=0; indx< m_ProtocolVCInMixContainer.NumProtocols(); indx++) {
    if (m_ProtocolVCInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolVCInMix*>(&(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolVCInMix&>(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx)).ExceptionHandler(e)){
			  WlTraceMsg("Error 1 in function CLoadGeneralThreadInstanceScript::ProtocolExceptionHandler");
      }
		}
  }
}

void CLoadGeneralThreadInstanceScript::ProtocolBeforeSleep(long int sleepTime){

	WlTraceMsg("Calling ProtocolBeforeSleep for all protocols");
	for(WLProtocolIndex indx=0; indx< m_ProtocolVCInMixContainer.NumProtocols(); indx++) {
    if (m_ProtocolVCInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolVCInMix*>(&(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolVCInMix&>(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx)).BeforeSleep(sleepTime)){
			  WlTraceMsg("Error 1 in function CLoadGeneralThreadInstanceScript::ProtocolBeforeSleep");
      }
		}
  }
}

void CLoadGeneralThreadInstanceScript::ProtocolAfterSleep(long int sleepTime){

	WlTraceMsg("Calling ProtocolAfterSleep for all protocols");
	for(WLProtocolIndex indx=0; indx< m_ProtocolVCInMixContainer.NumProtocols(); indx++) {
    if (m_ProtocolVCInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolVCInMix*>(&(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolVCInMix&>(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx)).AfterSleep(sleepTime)){
			  WlTraceMsg("Error 1 in function CLoadGeneralThreadInstanceScript::ProtocolAfterSleep");
      }
		}
  }
}

void CLoadGeneralThreadInstanceScript::ProtocolSleepDisabled(long int recordedSleepTime){

	WlTraceMsg("Calling ProtocolSleepDisabled for all protocols");
	for(WLProtocolIndex indx=0; indx< m_ProtocolVCInMixContainer.NumProtocols(); indx++) {
    if (m_ProtocolVCInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolVCInMix*>(&(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolVCInMix&>(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx)).SleepDisabled(recordedSleepTime)){
			  WlTraceMsg("Error 1 in function CLoadGeneralThreadInstanceScript::ProtocolSleepDisabled");
      }
		}
  }
}

void CLoadGeneralThreadInstanceScript::ProtocolBeginTransaction(const char* transname, double time, const char* extraDataType, void* extraData, unsigned long len) {

	WlTraceMsg("Calling ProtocolBeginTransaction for all protocols");
	for(WLProtocolIndex indx=0; indx< m_ProtocolVCInMixContainer.NumProtocols(); indx++) {
    if (m_ProtocolVCInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolVCInMix*>(&(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolVCInMix&>(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx)).BeginTransaction(transname, time, extraDataType, extraData, len)){
			  WlTraceMsg("Error 1 in function CLoadGeneralThreadInstanceScript::ProtocolBeginTransaction");
      }
		}
  }
}

void CLoadGeneralThreadInstanceScript::ProtocolEndTransaction(const char* transname, double time, WLLogMessage::Severity severity, bool save, bool SendTimers, bool IsLog ) {

	WlTraceMsg("Calling ProtocolEndTransaction for all protocols");
	for(WLProtocolIndex indx=0; indx< m_ProtocolVCInMixContainer.NumProtocols(); indx++) {
    if (m_ProtocolVCInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolVCInMix*>(&(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolVCInMix&>(m_ProtocolVCInMixContainer.GetProtocolAtIndex(indx)).EndTransaction(transname, time, severity, save, SendTimers, IsLog)){
			  WlTraceMsg("Error 1 in function CLoadGeneralThreadInstanceScript::ProtocolEndTransaction");
      }
		}
  }
}

iLEProtocolJSEmbedContext& CLoadGeneralThreadInstanceScript::GetProtocolJSEmbedContext(WLProtocolIndex p_ProtocolIndex) {
  WlAssert(dynamic_cast<iLEProtocolJSEmbedContext*>(&(static_cast<iLEProtocolVCInMix&>(m_ProtocolVCInMixContainer.GetProtocolAtIndex(p_ProtocolIndex)))));
  return static_cast<iLEProtocolJSEmbedContext&>(static_cast<iLEProtocolVCInMix&>(m_ProtocolVCInMixContainer.GetProtocolAtIndex(p_ProtocolIndex)));
}

void CLoadGeneralThreadInstanceScript::fEnterCriticalSection(){
	m_MyThreadInstance->fEnterCriticalSection();
}

void CLoadGeneralThreadInstanceScript::fExitCriticalSection(){
	m_MyThreadInstance->fExitCriticalSection();
}

void CLoadGeneralThreadInstanceScript::vEnterControlMachineThread_iLEJSEmbedContext()
{
	m_MyThreadInstance->fEnterControlMachineThread();
}

void CLoadGeneralThreadInstanceScript::vExitControlMachineThread_iLEJSEmbedContext()
{
	m_MyThreadInstance->fExitControlMachineThread();
}

bool CLoadGeneralThreadInstanceScript::vIsUserSection() 
{
	return m_MyThreadInstance->vIsUserSection();
}

CLoadGeneralThreadInstanceScript& CLoadGeneralThreadInstanceScript::vGetGeneralThreadInstanceScript()
{
	return *this;
}

JSContext* CLoadGeneralThreadInstanceScript::vGetContext()
{
	return m_MyThreadInstance->vCurrentJSContext();
}

// Chaim - serves to distinguish ambiguouity between iLEAgendaJSEmbedContext and iLEVCJSThread
iLEJSMachineThread& CLoadGeneralThreadInstanceScript::fGetMachineThread(){
  return iLEAgendaJSEmbedContext::fGetMachineThread();
}

//////////////////////////////// CLoadThreadInstanceScript ///////////////////////////////

CLoadThreadInstanceScript::CLoadThreadInstanceScript(CLoadThreadsManagerScript* pMS, CLoadThreadInstance* pTI) :
  CLoadGeneralThreadInstanceScript(pTI, pMS->m_WLJSScript),
  m_MyThreadsManagerScript(pMS),
	m_ThreadActivationDone(false),
  m_InitClientDone(false),
  m_StatClient(&(pMS->m_AgendaStat), pTI->GetLGThreadNum() ),
	m_TransactionManager()
{
	m_TransactionManager.InitParent(this);
  m_StatisticsMutex = new RVCriticalSection;
}

CLoadThreadInstanceScript::~CLoadThreadInstanceScript() {
  delete m_StatisticsMutex;
}

bool CLoadThreadInstanceScript::InScript() {
	return m_MyJSThread.IsInRound();
}

bool CLoadThreadInstanceScript::Init() {
  if (!m_JSThreadInitDone) {
		CWLJSScript* CWLJSS = &(m_MyThreadsManagerScript->m_WLJSScript);
		CLoadGeneratorSlave* CLGS = m_MyThreadsManagerScript->GetLoadGenerator();
    if (!m_MyJSThread.Init(CLGS->fGetProcess().GetLoadRuntime(), this, m_MyThreadsManagerScript->m_ScriptData, m_MyThreadInstance->GetLGThreadNum()))
      return false;

    m_JSThreadInitDone = true;
  }

  return true;
}


bool CLoadThreadInstanceScript::ThreadActivation()
{
	if (m_ThreadActivationDone)
		return true;

	bool returnStatus = true;

	m_MyJSThread.SetJSContext();

	setCurrentThreadInstanceScript(this, m_MyJSThread.GetContext());

	{
		CJSGCRequestBeginEnd GCGuard(m_MyJSThread.GetContext());

    ProtocolOnThreadActivation();

	}

	RVCString temp;
	returnStatus = returnStatus && m_MyJSThread.Evaluate(temp = m_MyThreadsManagerScript->GetEvaluationStringBeforeThreadActivation()); //call callbacks vcinmix

	returnStatus = returnStatus && m_MyJSThread.Evaluate(m_MyThreadsManagerScript->GetEvaluationStringOnThreadActivation());

	m_ThreadActivationDone = true;
	return returnStatus;
}

bool CLoadThreadInstanceScript::InitClient() {
  m_InitClientDone = true;
  
  bool st = m_MyJSThread.InitClient(this);
  
	if( UserCheckUnlock() ) {
		if( st ) { // if aborted, not need warning
		}
	}
  
  return st;
}

void CLoadThreadInstanceScript::TerminateClient()
{
  if (m_InitClientDone) {
    m_InitClientDone = false;
    ((CLoadThreadInstance*) m_MyThreadInstance)->SetCurrentScript(this);

    ProtocolBeforeTerminateClient();

    bool st;
      // bug 15822
    if (static_cast<CLoadThreadInstance*>(m_MyThreadInstance)->m_SevereErrorOccurred) {
      st = m_MyJSThread.OnErrorTerminateClient();
      if (st){
        st = m_MyJSThread.TerminateClient();
      }
		}
    else {
      st = m_MyJSThread.TerminateClient();
		}
		if( UserCheckUnlock() ) {
			if( st ) { // if aborted, not need warning
			}
		}
    
		m_MyJSThread.Evaluate(m_MyThreadsManagerScript->GetEvaluationStringAfterTerminateClient());

    ProtocolAfterTerminateClient();

    ((CLoadThreadInstance*) m_MyThreadInstance)->SetCurrentScript(NULL);


  }
}

void CLoadThreadInstanceScript::OnScriptAbort()
{
  bool st = m_MyJSThread.OnScriptAbort();
	if( UserCheckUnlock() ) {
		if( st ) { // if aborted, not need warning
		}
	}
}

//
// running the script (one time)
// this function is called from a loop.
bool CLoadThreadInstanceScript::runJScript()
{
	TIMERENTRY("CLoadThreadInstanceScript::runJScript()");
	PURIFY_NEW_LEAKS(true,"CLoadThreadInstanceScript::runJScript - Before Round - Leaks");
	PURIFY_MEMORY(true,"CLoadThreadInstanceScript::runJScript - Before Round - Memory");

	cLEJSEmbedContextControlMachineThread aGuard( *(dynamic_cast<CLoadThreadInstanceScript*>(m_MyThreadInstance->GetCurrentScript())));

	uint32 aAtomTimeclipIndex;
	{
	  CJSGCRequestBeginEnd GCGuard(fCurrentJSContext());
		aAtomTimeclipIndex = RVJS_AtomTimeclipBegin( fCurrentJSContext() );
	}

  if (!m_InitClientDone)
    InitClient();

  ++m_RoundNum;
  static_cast<CLoadThreadInstance*>(m_MyThreadInstance)->IncrementRoundNum();

	bool ret = false;
  jsval result;
	if( m_MyJSThread.CallFunctionName( "UNDOC_BeforeRound" ,0,0,result ) ) {
		if( ! m_MyJSThread.Evaluate(m_MyThreadsManagerScript->GetEvaluationStringBeforeRound()) ) {//call calbacks
			return false;
		}
    SetMonitorCharSet(GetJSScript().m_PropWorkSpace.m_CharEncoding);

    if (!ProtocolBeforeRound()){
      return false;
    }
	  resetTimer("Round Time");
		ret = m_MyJSThread.ExecScript(m_RoundNum);
	}

	if( UserCheckUnlock() ) {
		if( ret ) { // if aborted, not need warning
		}
	}

  if (ret){
    sendTimer("Round Time");
    addLast("Successful Rounds", 1, CWLStatPointTypeEnum::Counter);
  }
  else if (isAbort(false)){
    addLast("Aborted Rounds", 1, CWLStatPointTypeEnum::Counter);
  }
  else {
    addLast("Failed Rounds", 1, CWLStatPointTypeEnum::Counter);
  }
  addLast("Rounds", 1, CWLStatPointTypeEnum::Counter);
	ret = ret && m_MyJSThread.Evaluate(m_MyThreadsManagerScript->GetEvaluationStringAfterRound());

  ProtocolAfterRound();
	fExitCriticalSection();

	fGetSession().fGetProcess().GetLoadRuntime().m_ScriptCache.fFreezeCond(fCurrentJSContext(), fGetSession().GetCompletedRounds());
	fGetSession().fGetProcess().GetLoadRuntime().m_FunctionCache.fFreezeCond(fCurrentJSContext(), fGetSession().GetCompletedRounds());

	ret = ret && m_MyJSThread.CallFunctionName( "UNDOC_AfterRound" ,0,0,result );

	{
	  CJSGCRequestBeginEnd GCGuard(fCurrentJSContext());
		RVJS_AtomTimeclipEnd( fCurrentJSContext(), aAtomTimeclipIndex );
	}


	PURIFY_NEW_LEAKS(true,"CLoadThreadInstanceScript::runJScript - After Round - Leaks");
	PURIFY_MEMORY(true,"CLoadThreadInstanceScript::runJScript - After Round - Memory");

	fGetSession().IncrementCompletedRounds();	

	return ret;
}

#define STAT_NAME_LEN 256
const char* CutStr(const char* source,char* dest,size_t size) {
  if (strlen(source) <= size) {
    return source;
	} else {
    strncpy(dest,source,size);
    dest[size] = '\0';
    return dest;
	}
}

bool CLoadThreadInstanceScript::vIsJSGC()
{
	return m_MyThreadInstance->vGetMachineThread().fGetProcess().fIsJSGCInExecutionContext();
}

void CLoadThreadInstanceScript::addLast(const char *name, double l, CWLStatPointTypeEnum::PointType type)
{
  if (!m_MyThreadsManagerScript->GetLoadGenerator()->IsSendStatisticalMesurement(type))
    return;

  RVCriticalSection::LockGuard lock(*m_StatisticsMutex);

  if (strlen(name) <= STAT_NAME_LEN)
    m_StatClient.AddLast(name, l, type);
  else {
    char CutName[STAT_NAME_LEN+1];
    m_StatClient.AddLast(CutStr(name,CutName,STAT_NAME_LEN), l, type);
  }
}

void CLoadThreadInstanceScript::sendTimer(const char *name)
{
  if (!m_MyThreadsManagerScript->GetLoadGenerator()->IsSendStatisticalMesurement(CWLStatPointTypeEnum::Timer))
    return;

  if (strlen(name) <= STAT_NAME_LEN)
    double timerVal = m_StatClient.SendTimer(name);
  else {
    char CutName[STAT_NAME_LEN+1];
    double timerVal = m_StatClient.SendTimer(CutStr(name,CutName,STAT_NAME_LEN));
  }
}

void CLoadThreadInstanceScript::resetTimer(const char *name)
{
  if (strlen(name) <= STAT_NAME_LEN)
    m_StatClient.ResetTimer(name);
  else {
    char CutName[STAT_NAME_LEN+1];
    m_StatClient.ResetTimer(CutStr(name,CutName,STAT_NAME_LEN));
  }
}

void CLoadThreadInstanceScript::UserLock() {
	m_MyThreadsManagerScript->UserLock( m_MyThreadInstance->GetSlaveThreadNum(), m_MyJSThread.GetContext() );
}
void CLoadThreadInstanceScript::UserUnlock() {
	m_MyThreadsManagerScript->UserUnlock( m_MyThreadInstance->GetSlaveThreadNum(), m_MyJSThread.GetContext() );
}

bool CLoadThreadInstanceScript::UserCheckUnlock() {
	return m_MyThreadsManagerScript->UserCheckUnlock( m_MyThreadInstance->GetSlaveThreadNum() );
}


IFileDictionary* CLoadThreadInstanceScript::GetFileDict() {
  return m_MyThreadsManagerScript->GetFileDict();
}

void CLoadThreadInstanceScript::CreateApplet(JSContext*           cx,
                                             const char*          appletClassName,
 															               const char*          appletCodeBase,
                                             const char*          appletDocumentBase,
															               RVHashDictionary*    appletParams,
                                             CWLJSJavaApplet*&    javaApplet)
{
	javaApplet = CWLAppletFactory::CreateJavaApplet(m_MyJSThread.GetContext(),
                                                  this,
                                                  javaApplet,
												                          appletClassName,
												                          appletCodeBase,
                                                  appletDocumentBase,
												                          appletParams );

}

bool CLoadThreadInstanceScript::ClassAppletExists(JSContext*  cx,
                                                  const char* appletClassName,
 															                    bool& classAppletExists)
{
  return CWLAppletFactory::ClassAppletExists(	m_MyJSThread.GetContext(),
	  											                    appletClassName,
  								                            classAppletExists);

}


//**************

SynchronizationPointStatus CLoadThreadInstanceScript::SynchronizationPoint(int timeoutmillis) {
  //
  // if I'm in script and IsAbort, I've been rescheduled.
	if (isAbort(false)) {
		return SynchronizationPointLoadChanged;
	}
  return m_MyThreadsManagerScript->SynchronizationPoint(timeoutmillis);
}

CLoadGeneratorSlave* CLoadThreadInstanceScript::GetLoadGeneratorSlave() {
	return ((CLoadThreadInstance*) m_MyThreadInstance)->GetLoadGeneratorSlave();
}

int CLoadThreadInstanceScript::GetNumScriptInMix() {
	return m_MyThreadsManagerScript->GetNumScriptInMix();
}

////////////////////////////////////////////////////////////////
// Moved from Thread Data
////////////////////////////////////////////////////////////////

void CLoadThreadInstanceScript::addMeasurement(const char *name, double l)
{
  addLast(name, l, CWLStatPointTypeEnum::Measurement);
}  

void CLoadThreadInstanceScript::addTimer(const char *name, double l)
{
  addLast(name, l, CWLStatPointTypeEnum::Timer);
}  

void CLoadThreadInstanceScript::addCounter(const char *name)
{
  addLast(name, 1, CWLStatPointTypeEnum::Counter);
}  

void CLoadThreadInstanceScript::addAccumulator(const char *name, double l)
{
  addLast(name, l, CWLStatPointTypeEnum::Accumulator);
}  

//*********
//////////////////////////////// CLoadThreadInstanceScriptContainer ///////////////////////////////
CLoadThreadInstanceScriptContainer::CLoadThreadInstanceScriptContainer(CLoadThreadsManagerScriptContainer* pSC) :
  m_ScriptContainer(pSC)
{
}

CLoadThreadInstanceScriptContainer::~CLoadThreadInstanceScriptContainer()
{
  clearAndDestroy();
}

bool
CLoadThreadInstanceScriptContainer::Init(CLoadThreadInstance* pThreadInstance)
{
  bool ok;
  RVHashDictionaryIterator i(*m_ScriptContainer);
  while (i()) {
    CLoadThreadsManagerScript *pManagerScript = (CLoadThreadsManagerScript*)i.value();
    RVCollectableString* scriptName = new RVCollectableString(pManagerScript->m_ScriptName);
    CLoadThreadInstanceScript* pTIS = new CLoadThreadInstanceScript(pManagerScript, pThreadInstance);
    ok = pTIS->Init();
    if (!ok) return false;
    insertKeyAndValue(scriptName, pTIS);
  }
  return true;
}

void
CLoadThreadInstanceScriptContainer::TerminateClient()
{
  RVHashDictionaryIterator i(*this);
  while (i()) {
    CLoadThreadInstanceScript *pInstanceScript = (CLoadThreadInstanceScript*)i.value();
    // AMDOCS - add guard to support doing http stuff in TerminateClient
    // @@Chaim - 19032
//  	cLEJSEmbedContextControlMachineThread aGuard( *pInstanceScript );
    pInstanceScript->TerminateClient();
  }
}

bool CLoadThreadInstanceScriptContainer::TerminateJSThreads() {

	RVHashDictionaryIterator i(*this);
	while (i()) {
		CLoadThreadInstanceScript *pInstanceScript = (CLoadThreadInstanceScript*)i.value();
    pInstanceScript->ProtocolOnThreadTermination();
		pInstanceScript->m_MyJSThread.Terminate();
	}
	return true;
}

bool CLoadThreadInstanceScriptContainer::RestartJSThreads() {

	RVHashDictionaryIterator i(*this);
	while (i()) {
		CLoadThreadInstanceScript *pInstanceScript = (CLoadThreadInstanceScript*)i.value();
    pInstanceScript->ProtocolOnThreadRestart();
	}
	return true;
}

CLoadThreadInstanceScript*
CLoadThreadInstanceScriptContainer::SelectScript(CLoadThreadInstance* pTi,CLoadThreadInstanceScript* Previous, unsigned long& RoundNumScriptInSlave, unsigned long& RoundNumMixInSlave )
{
  RVCollectableString name = m_ScriptContainer->SelectThreadsManagerScript(pTi,Previous, RoundNumScriptInSlave, RoundNumMixInSlave );
  CLoadThreadInstanceScript* pTIS = (CLoadThreadInstanceScript*)findValue(&name);

  return pTIS;
}

void
CLoadThreadInstanceScriptContainer::ReleaseThreadsManagerScript(CLoadThreadInstanceScript* Current)
{
  m_ScriptContainer->ReleaseThreadsManagerScript(Current);
}


//////////////////////////////// CLoadThreadsManagerScript ///////////////////////////////

CLoadThreadsManagerScript::CLoadThreadsManagerScript(CLoadThreadsManagerScriptContainer *sc, const char* scriptName,int numScriptInMix) :
	m_WLJSScript(numScriptInMix),
  m_MyContainer(sc),
	m_ScriptData(),
  m_ScriptName(scriptName),
  m_ScriptFileName(""),
  m_ScriptCode(""),

  m_RoundsCounter(0),
  m_AgendaStat(scriptName),
  m_StatClient(&m_AgendaStat, 1000000),
  m_InitAgendaDone(false),
  m_SevereErrorOccurred(false),
	m_SynchronizationPointTreated(0),
  m_RuntimeTotal(0),
  m_RunningTotal(0),
  m_InputFileDict(&m_ScriptData),
	m_numScriptInMix (numScriptInMix)
{
		//m_SynchronizationPointMutex = new RVMutexLock;
	m_SynchronizationPointCondition = new RVCondition(*sc->m_MyLoadThreadsManager->m_SynchronizationPointMutex);
	m_SynchronizationPointFinishEvent = new RVCondition(*sc->m_MyLoadThreadsManager->m_SynchronizationPointMutex);

	m_ScriptData.InitParent(this);

  m_StatStartEvent.Reset();
  m_StatPackEvent.Reset();
  m_StatGarbageEvent.Reset();

  m_AgendaStat.Init(this);

  //
  // create the running thread functors for statistics:
  #if !defined(RV_THR_CANT_EXTRACT_TEMPLATE_ARGS_FROM_SIGNATURE)
    m_StatPackThread = rvMakeThreadFunction(
                        *this,                                        // instance
                        &CLoadThreadsManagerScript::StatPackFunc      // member function
                      );
    m_StatGarbageThread = rvMakeThreadFunction(
                        *this,                                        // instance
                        &CLoadThreadsManagerScript::StatGarbageFunc   // member function
                      );

  #else
    m_StatPackThread = rvMakeThreadFunctionM(
                        CLoadThreadsManagerScript                     // class
                        *this,                                        // instance
                        void,                                         // return value
                        &CLoadThreadsManagerScript::StatPackFunc      // member function
                      );
    m_StatGarbageThread = rvMakeThreadFunctionM(
                        CLoadThreadsManagerScript                     // class
                        *this,                                        // instance
                        void,                                         // return value
                        &CLoadThreadsManagerScript::StatGarbageFunc   // member function
                      );
  #endif
}

CLoadThreadsManagerScript::~CLoadThreadsManagerScript()
{
  delete m_SynchronizationPointCondition;
  delete m_SynchronizationPointFinishEvent;
}

IGeneralManagerScriptData& CLoadThreadsManagerScript::GetScriptData() {
	return m_ScriptData;
	}


void CLoadThreadsManagerScript::LoadScript(
	const char* scriptName,
  const char* scriptFileName,
  const char* ScriptCode,
  double weight) {

	RVCString rwAgendaName;

	CLoadGeneratorSlave* LGS = GetLoadGenerator();
 	TVEnvContainerClass tvpc;
 	const char* agendaName = tvpc.TVPGetPathName(scriptFileName);
 	if(agendaName) {
 		rwAgendaName = agendaName;
 	}
   else {
 		throw WLErrorException(LoadExceptionFrmFile, 31, scriptName);
 	}

 	if (!m_WLJSScript.Init(LGS->fGetProcess().GetLoadRuntime(), GetScriptData(), LGS->GetLEProtocolSessionContainer(), rwAgendaName.data() ) ) {
    throw WLErrorException(LoadExceptionFrmFile, 33, scriptName,m_WLJSScript.GetError());
 	}

  //WlTraceMsg("CLoadThreadsManagerScript::LoadScript(scriptName=%s, scriptFileName=%s, ScriptCode=??, weight=%lg)",scriptName, scriptFileName, weight);
  m_ScriptFileName = scriptFileName;
  if (ScriptCode == NULL)
    m_ScriptCode = "";
  else
    m_ScriptCode = ScriptCode;
  m_Weight = weight;

  if (!m_WLJSScript.LoadScript(m_ScriptFileName, m_ScriptCode.length(), m_ScriptCode.data())) {
    throw WLErrorException(true, LoadExceptionFrmFile, 34, m_WLJSScript.GetError());
  }

  //
  // create the statistics Pack and Garbage collector tasks.
  m_StatPackThread.start();
  m_StatGarbageThread.start();

  // wait for creation events.
  m_StatPackEvent.Wait();
  m_StatPackEvent.Reset();
  m_StatGarbageEvent.Wait();
  m_StatGarbageEvent.Reset();
}

bool CLoadThreadsManagerScript::IncludeMonitorFile(
	const char* filename,
	bool execute) {
  char lowcaseFileName[255];
  (void)strcpy(lowcaseFileName,filename);
#ifdef __TV_UNIX__
  (void)TVPStrlwr(lowcaseFileName);
#endif // __TV_UNIX__
	const char* fullPath = NULL;
	TVEnvContainerClass tvpc;
	const char* tv_Include = tvpc.TVEGetEnv("WL_DEFAULT_INCLUDE");
	if(!tv_Include) {//tvpc.TVEGetEnv return null
		WlErrorMsg("Error : in LoadEngine in function CLoadThreadsManagerScript::IncludeMonitorFile");
		WlAssert(0);
	}
	RVCString rwTVInclude = tv_Include;
  	tvpc.reset();
	if( (fullPath = tvpc.TVPPathFileExist(rwTVInclude.data(), lowcaseFileName) ) != NULL) {//the file exist in WL_DEFAULT_INCLUDE
		RVCString path(fullPath);
		bool ok( true );
		RVCString code;
		FILE* File = fopen( path.data() , "r");
		if( File ) {
			char ch;
			bool more( true );
			do {//read the file
				if( fread( &ch, 1, 1, File ) ) {
					code += ch;
				} else {
					more = false;
					if(feof( File ))//It returns 0 if the current position is not end of file.
						ok = true;
					else
						ok = false;
				}
			} while( more );

			if( ! ok ) {//the file wasn't read succesfully
				LoadReport(m_WLJSScript.GetContext(), WLLogMessage::Error, "Failed to read an include file");
			}

			RVCString internalFilename("internal - ");
			internalFilename += lowcaseFileName;
			return m_WLJSScript.IncludeFile(internalFilename.data(), code.length(), code, execute);
		}
	}
	CAssociativeArray Message;

	Message.AddCouple("$INCLUDEMONITORFILE","1");
	Message.AddCouple("$SOURCEFILENAME",filename);
	Message.AddCouple("$AGENDAPATH" ,m_ScriptFileName);
	CLoadGeneratorSlave* lg = GetLoadGenerator();
	lg->SendMessage(Message, 2);
	CAssociativeArray* AssoBuffer = lg->GetMessage();

	if (strcmp(AssoBuffer->GetValueOfParameter("$INCLUDEFILESTATUS"),"OK") == 0 ) {
		const char* code = AssoBuffer->GetValueOfParameter("$DATA");
		return m_WLJSScript.IncludeFile(filename,strlen(code), code, execute);
	}
	else {
		return LoadReport( m_WLJSScript.GetContext(), WLLogMessage::Error, AssoBuffer->GetValueOfParameter("$ERRORMESSAGE") );
	}

}

void CLoadThreadsManagerScript::AppendStringForEvaluation(const char* str,CWLJSScript::WhenToRunScript whenToRunScript){
		m_WLJSScript.AppendStringForEvaluation( str, whenToRunScript );
}

RVCString& CLoadThreadsManagerScript::GetEvaluationStringBeforeInitClient()
{
	return	m_WLJSScript.GetEvaluationStringBeforeInitClient();
}
RVCString& CLoadThreadsManagerScript::GetEvaluationStringAfterInitAgenda()
{
	return	m_WLJSScript.GetEvaluationStringAfterInitAgenda();
}
/*
RVCString&	CLoadThreadsManagerScript::GetEvaluationStringOnThreadTermination()
{
	return	m_WLJSScript.GetEvaluationStringOnThreadTermination();
}
*/
RVCString&	CLoadThreadsManagerScript::GetEvaluationStringBeforeThreadActivation()
{
	return	m_WLJSScript.GetEvaluationStringBeforeThreadActivation();
}
RVCString&	CLoadThreadsManagerScript::GetEvaluationStringOnThreadActivation()
{
	return	m_WLJSScript.GetEvaluationStringOnThreadActivation();
}
RVCString&	CLoadThreadsManagerScript::GetEvaluationStringAfterTerminateClient()
{
	return	m_WLJSScript.GetEvaluationStringAfterTerminateClient();
}
RVCString&	CLoadThreadsManagerScript::GetEvaluationStringBeforeRound()
{
	return	m_WLJSScript.GetEvaluationStringBeforeRound();
}
RVCString&	CLoadThreadsManagerScript::GetEvaluationStringAfterRound()
{
	return	m_WLJSScript.GetEvaluationStringAfterRound();
}
RVCString&	CLoadThreadsManagerScript::GetEvaluationStringAfterTerminateAgenda()
{
	return	m_WLJSScript.GetEvaluationStringAfterTerminateAgenda();
}

RVCString& CLoadThreadsManagerScript::GetScriptFileName(){
	return m_ScriptFileName;
}

RVCString& CLoadThreadsManagerScript::GetScriptName(){
	return m_ScriptName;
}


bool CLoadThreadsManagerScript::CopyMonitorFile(
	const char* source,
	const char* target) {

  CAssociativeArray Message;

  Message.AddCouple("$COPYMONITORFILE","1");
  Message.AddCouple("$SOURCEFILENAME",source);
  Message.AddCouple("$TARGETFILENAME",target);
	Message.AddCouple("$AGENDAPATH" ,m_ScriptFileName);

	CLoadGeneratorSlave* lg = GetLoadGenerator();
	lg->SendMessage(Message, 2);
	CAssociativeArray* AssoBuffer = NULL;
	AssoBuffer = lg->GetMessage();

  if (strcmp(AssoBuffer->GetValueOfParameter("$COPYFILESTATUS"),"OK") == 0 ) {
    return true;
  }
  else {
		return LoadReport( m_WLJSScript.GetContext(), WLLogMessage::Error, AssoBuffer->GetValueOfParameter("$ERRORMESSAGE") );
  }
}

void CLoadThreadsManagerScript::AddNumRunning()
{
  double value = m_RunningTotal * 1.0;
  m_StatClient.AddLast("Load Size", value , CWLStatPointTypeEnum::LoadSize);
}

#define SECOND 1.0
double CLoadThreadsManagerScript::CalculateCurrentWeight(bool PreviosIsCurrentScript)
{
  //
  // here we calculate how close do we get to the weight if the next script is chosen.
  //WlTraceMsg("m_RoundsCounter=%d, m_MyContainer->m_RoundsCounter=%d, m_Weight=%lg",m_RoundsCounter, m_MyContainer->m_RoundsCounter, m_Weight);
//  return ((m_RoundsCounter+1.0)/(m_MyContainer->m_RoundsCounter+1.0))*100.0 - m_Weight;

// by users
  // if the the thread was running the same script, there is no change in the script distribution. Other I have to
  // add one thread running the script.
  if (m_MyContainer->m_MyLoadThreadsManager->m_MixNumRunning == 0) return 0;
  int AdditionalThread;
  if (PreviosIsCurrentScript)
    AdditionalThread = 0;
  else
    AdditionalThread = 1;
  return ((m_RuntimeTotal+SECOND*(AdditionalThread+m_RunningTotal))/(m_MyContainer->m_RuntimeTotal+SECOND*m_MyContainer->m_MyLoadThreadsManager->m_MixNumRunning))* 100.0 - m_Weight;
}

void CLoadThreadsManagerScript::StatPackFunc()
{
  try {
    TIMERENTRY("CLoadThreadsManagerScript::StatPackFunc()");
    CTraceMsg::RegisterThread ();
    WlTraceMsg("Starting CLoadThreadsManagerScript::StatPackFunc");
    RVThreadSelf self = ::rvThread();
    //self.setPriority(self.getMaxPriority());
#ifndef __TV_linux__
    self.setPriority(self.getMaxPriority()-1);
#endif
    //TVPSysThread::setPriority(TVPSysThread::Critical);
    m_StatPackEvent.Set(); // sign start of the stat thread
    m_StatStartEvent.Wait(); // wait for the statustucs to start.

    if (!m_MyContainer->m_MyLoadThreadsManager->IsTerminate()) {
      WlTraceMsg("GlobalStatPackFunc: doPackWork");
      m_AgendaStat.doPackWork();
    }

    m_StatPackEvent.Set(); // sign start of the stat thread
    WlTraceMsg("CLoadThreadsManagerScript::StatPackFunc: Done");
  } catch (...) {
    m_StatPackEvent.Set();
    WlTraceMsg("Fatal Error in CLoadThreadsManagerScript::StatPackFunc");
    //  Since yahali said there is a race problem he cannot find.  We are not sending any message
    //    when terminating.
    if (!m_MyContainer->m_MyLoadThreadsManager->IsTerminate()) {
      CAssociativeArray	ErrorMessage;
      ErrorMessage.AddCouple("$GENERATORLOGMESSAGE","Fatal Error in CLoadThreadsManagerScript::StatPackFunc");
      ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
      ErrorMessage.AddCouple("$TIME", TVPSysTime()-m_AgendaStat.GetBeginSystem());
      GetLoadGenerator()->SendMessage(ErrorMessage, 2);
      #ifdef _DEBUG
      throw;
      #endif
    }
  }
}

void CLoadThreadsManagerScript::StatGarbageFunc()
{
  try {LE_SET_SE_TRANSLATOR();
    TIMERENTRY("CLoadThreadsManagerScript::StatGarbageFunc()");

    WlTraceMsg("Starting CLoadThreadsManagerScript::StatGarbageFunc");
    RVThreadSelf self = ::rvThread();
    //self.setPriority(self.getMaxPriority());
#ifndef __TV_linux__
    self.setPriority(self.getMaxPriority()-1);
#endif
    //TVPSysThread::setPriority(TVPSysThread::Critical);
    m_StatGarbageEvent.Set(); // sign start of the stat thread
    m_StatStartEvent.Wait(); // wait for the statustucs to start.

    if (!m_MyContainer->m_MyLoadThreadsManager->IsTerminate()) {
      WlTraceMsg("GlobalStatGarbageFunc: doGarbageWork");
      m_AgendaStat.doGarbageWork();
    }

    m_StatGarbageEvent.Set(); // sign start of the stat thread
    WlTraceMsg("CLoadThreadsManagerScript::StatGarbageFunc: Done");

	}	catch (std::exception* e){
		ExceptionHandler(e);
	}	catch (std::exception& e){
		ExceptionHandler(&e);
  } catch (...) {
    m_StatGarbageEvent.Set();
    WlTraceMsg("Fatal Error in CLoadThreadsManagerScript::StatGarbageFunc");
    //  Since yahali said there is a race problem he cannot find.  We are not sending any message
    //    when terminating.
    if (!m_MyContainer->m_MyLoadThreadsManager->IsTerminate()) {
      CAssociativeArray	ErrorMessage;
      ErrorMessage.AddCouple("$GENERATORLOGMESSAGE","Fatal Error in CLoadThreadsManagerScript::StatGarbageFunc");
      ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
      ErrorMessage.AddCouple("$TIME", TVPSysTime()-m_AgendaStat.GetBeginSystem());
      GetLoadGenerator()->SendMessage(ErrorMessage, 2);
      #ifdef _DEBUG
      throw;
      #endif
    }
  }
}

void CLoadThreadsManagerScript::ExceptionHandler(std::exception* e){

  // Check if WLErrorException and if reported
  WLErrorException* aException;

  if (aException = dynamic_cast<WLErrorException*>(e)){
    if (aException->GetReported()){
      return; // Already reported, do nothing
    }
  }

	RVCString FullExcName ("");
	WLERRGetErrorMsg(e, FullExcName);
	RVCString StackTrace("");
	if (WLERRGetStackTrace(e, StackTrace) && StackTrace.length() > 0){
	  WlErrorMsg("CLoadThreadInstance::loadLoop - Exception %s occured\nTrace:\n%s", FullExcName, StackTrace);
	  printf("CLoadThreadInstance::loadLoop - Exception %s occured\nTrace:\n%s", FullExcName, StackTrace);
	}
	else {
	  WlErrorMsg("CLoadThreadInstance::loadLoop - Exception %s occured", FullExcName);
	  printf("CLoadThreadInstance::loadLoop - Exception %s occured", FullExcName);
	}

  //  Since yahali said there is a race problem he cannot find.  We are not sending any message
  //    when terminating.
  if (!m_MyContainer->m_MyLoadThreadsManager->IsTerminate()) {
    CAssociativeArray	ErrorMessage;
    ErrorMessage.AddCouple("$GENERATORLOGMESSAGE","Fatal Error in CLoadThreadsManagerScript::StatGarbageFunc");
    ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
    ErrorMessage.AddCouple("$TIME", TVPSysTime()-m_AgendaStat.GetBeginSystem());
    GetLoadGenerator()->SendMessage(ErrorMessage, 2);
  }
}

void
CLoadThreadsManagerScript::WaitForTerminateToEnd()
{
  m_AgendaStat.WaitForTerminateToEnd();

  WlTraceMsg("CLoadThreadsManagerScript::Terminate - Wait for pack to terminate");
  if (m_StatPackEvent.Wait(CLEIniFileProperties::StatPackTerminateTimeout) == RV_THR_TIMEOUT) {
    WlTraceMsg("CLoadThreadsManagerScript::Terminate - ***** Brutal Terminate of stat pack thread after timeout");
    m_StatPackThread.terminate();
  }

  WlTraceMsg("CLoadThreadsManagerScript::Terminate - Wait for garbage to terminate");
  if (m_StatGarbageEvent.Wait(CLEIniFileProperties::StatGarbageTerminateTimeout) == RV_THR_TIMEOUT) {
    WlTraceMsg("CLoadThreadsManagerScript::Terminate - ***** Brutal Terminate of stat garbage thread after timeout");
    m_StatGarbageThread.terminate();
  }

}

void
CLoadThreadsManagerScript::RequestForTerminate()
{
  m_StatStartEvent.Set();
  //
  // Terminate the agenda statistics tasks and
  // wait for stat threads to die before termination.
  WlTraceMsg("CLoadThreadsManagerScript::Terminate - agendaStat.Terminate()");
  m_AgendaStat.Terminate();
  m_WLJSScript.Terminate();
  
  if ( m_StatPackThread.getExecutionState() != RV_THR_INITIAL &&
       m_StatPackThread.join(8000) == RV_THR_TIMEOUT ) 
  {
    try 
    {
      m_StatPackThread.terminate();
    }
    catch (...) {}
  }
  
  if ( m_StatGarbageThread.getExecutionState() != RV_THR_INITIAL &&
       m_StatGarbageThread.join(8000) == RV_THR_TIMEOUT ) 
  {
    try 
    {
      m_StatGarbageThread.terminate();
    }
    catch (...) {}
  }

}

void CLoadThreadsManagerScript::OnSevereError() {
  m_AgendaStat.TerminationInProgress();
	m_SevereErrorOccurred = true;
}

bool CLoadThreadsManagerScript::InitAgenda(const char* InitializationScript)
{
	cLEJSEmbedContextControlMachineThread aGuard( m_ScriptData );
  m_InitAgendaDone = true;
	CLoadGeneratorSlave* LGS = GetLoadGenerator();

	// Set version number by calling RVScriptVersion() if generated by AAT
	m_WLJSScript.InitVersionNumber();
  m_ScriptData.ProtocolBeforeInitAgenda();
  if (!m_WLJSScript.InitAgenda(InitializationScript ) ) {
    WLErrorException e(LoadExceptionFrmFile, 60, m_WLJSScript.GetError());
    GetThreadsManager()->reportLogCurrent(e.GetErrorSeverity(),e.GetErrorMsg());
    m_SevereErrorOccurred = true;
    return false;
  }

  return true;
}

void CLoadThreadsManagerScript::TerminateAgenda()
{
	cLEJSEmbedContextControlMachineThread aGuard( m_ScriptData );
  if (m_InitAgendaDone) {
    m_InitAgendaDone = false;
    if (m_SevereErrorOccurred) {
      // bug 15822
      if (!m_WLJSScript.OnErrorTerminateAgenda(m_ScriptData)) {
        WLErrorException e(LoadExceptionFrmFile, 62, m_WLJSScript.GetError());
        GetThreadsManager()->reportLogCurrent(e.GetErrorSeverity(),e.GetErrorMsg());
      }
      else{
        if (!m_WLJSScript.TerminateAgenda(m_ScriptData)) {
          WLErrorException e(LoadExceptionFrmFile, 61, m_WLJSScript.GetError());
          GetThreadsManager()->reportLogCurrent(e.GetErrorSeverity(),e.GetErrorMsg());
        }
      }
    }
    else {
      if (!m_WLJSScript.TerminateAgenda(m_ScriptData)) {
        WLErrorException e(LoadExceptionFrmFile, 61, m_WLJSScript.GetError());
        GetThreadsManager()->reportLogCurrent(e.GetErrorSeverity(),e.GetErrorMsg());
      }
    }

  }
}

void
CLoadThreadsManagerScript::StartLoad(double startTime, int sliceSize)
{
  m_DelayTime = GetLoadGenerator()->fGetTimeFromBegin();
  m_AgendaStat.Start(startTime,
                     sliceSize,
                     m_WLJSScript.m_PropWorkSpace.m_SendClientStatistics,
                     m_WLJSScript.m_PropWorkSpace.m_SendClientStatisticsFilter.data());
  
  m_LoadSizeManager.Start(startTime, sliceSize);
  //
  // release the Load-size statistics thread.
  m_StatStartEvent.Set();

}

CLoadGeneratorSlave* //there are 2 api to get that pointer (because historical reasons)
CLoadThreadsManagerScript::GetLoadGenerator()
{
  return m_MyContainer->m_MyLoadThreadsManager->m_pMyLoadGeneratorSlave;
}

CLoadGeneratorSlave* //there are 2 api to get that pointer (because historical reasons)
CLoadThreadsManagerScript::GetLoadGeneratorSlave()
{
  return m_MyContainer->m_MyLoadThreadsManager->m_pMyLoadGeneratorSlave;
}

CLoadThreadsManager* CLoadThreadsManagerScript::GetThreadsManager()
{
  return m_MyContainer->GetThreadsManager();
}

IGeneralGlobalsRequestHandler* CLoadThreadsManagerScript::GetGlobalsRequestHandler()
{
	return (IGeneralGlobalsRequestHandler*)m_MyContainer->m_MyLoadThreadsManager->m_ThreadsManagerGlobalsHandler;
}

void CLoadThreadsManagerScript::ReleaseSynchronizationPoints() {
	//RVMutexLock::LockGuard lock(*m_MyContainer->m_MyLoadThreadsManager->m_SynchronizationPointMutex);
	if (m_SynchronizationPointTreated != 0 && m_MyContainer->m_MyLoadThreadsManager->m_EventIsTerminating==false) {
		m_SynchronizationPointStatus = SynchronizationPointLoadChanged;
		m_SynchronizationPointCondition->signalAll();
    m_MyContainer->m_MyLoadThreadsManager->m_EventIsTerminating = true;
		while (m_SynchronizationPointTreated != 0 && m_MyContainer->m_MyLoadThreadsManager->m_EventIsTerminating == true)
      m_SynchronizationPointFinishEvent->wait(10);
    m_MyContainer->m_MyLoadThreadsManager->m_EventIsTerminating = false;
	}
}

SynchronizationPointStatus CLoadThreadsManagerScript::SynchronizationPoint(int timeoutmillis) {
	RVMutexLock::LockGuard lock(*m_MyContainer->m_MyLoadThreadsManager->m_SynchronizationPointMutex);

  //
  // If the event is in termination, wait for the finish event outside.
  if (m_MyContainer->m_MyLoadThreadsManager->m_EventIsTerminating==true) {
		SynchronizationPointStatus status = m_SynchronizationPointStatus;
		while (m_SynchronizationPointTreated != 0 && m_MyContainer->m_MyLoadThreadsManager->m_EventIsTerminating == true)
      m_SynchronizationPointFinishEvent->wait(10);

    m_MyContainer->m_MyLoadThreadsManager->m_EventIsTerminating = false;
    return status;
  }

	m_SynchronizationPointTreated++;
  //
  // if I'm the last one, Signal everybody to exit.
	if (m_SynchronizationPointTreated == m_MyContainer->m_MyLoadThreadsManager->m_NumRunning) {
		m_SynchronizationPointTreated--;
		m_SynchronizationPointStatus = SynchronizationPointOK;
		m_SynchronizationPointCondition->signalAll();
    m_MyContainer->m_MyLoadThreadsManager->m_EventIsTerminating = true;
		while (m_SynchronizationPointTreated != 0 && m_MyContainer->m_MyLoadThreadsManager->m_EventIsTerminating == true)
      m_SynchronizationPointFinishEvent->wait(10);
    m_MyContainer->m_MyLoadThreadsManager->m_EventIsTerminating = false;
		m_SynchronizationPointStatus = SynchronizationPointLoadChanged;
		return SynchronizationPointOK;
	}
	else {
		RVWaitStatus WaitStatus = RV_THR_SIGNALED;
		if (timeoutmillis == -1)
			m_SynchronizationPointCondition->wait();
		else
			WaitStatus = m_SynchronizationPointCondition->wait(timeoutmillis);

    //
    // if I'm had timeout, and not yet terminating, Signal everybody to exit.
		if (WaitStatus == RV_THR_TIMEOUT && m_MyContainer->m_MyLoadThreadsManager->m_EventIsTerminating==false) {
			m_SynchronizationPointTreated--;
			m_SynchronizationPointStatus = SynchronizationPointTimeout;
		  m_SynchronizationPointCondition->signalAll();
      m_MyContainer->m_MyLoadThreadsManager->m_EventIsTerminating = true;
		  while (m_SynchronizationPointTreated != 0 && m_MyContainer->m_MyLoadThreadsManager->m_EventIsTerminating == true)
        m_SynchronizationPointFinishEvent->wait(10);
      m_MyContainer->m_MyLoadThreadsManager->m_EventIsTerminating = false;
		  m_SynchronizationPointStatus = SynchronizationPointLoadChanged;
			return SynchronizationPointTimeout;
		}
		else {
			SynchronizationPointStatus status = m_SynchronizationPointStatus;
			m_SynchronizationPointTreated--;
      //
      // if I'm the last one, signal the terminator.
      if (m_SynchronizationPointTreated == 0)
        m_SynchronizationPointFinishEvent->signalAll();
      //
      // Wait for the everybody to finish.
      else
        m_SynchronizationPointFinishEvent->wait();

      m_MyContainer->m_MyLoadThreadsManager->m_EventIsTerminating = false;
      return status;
		}
	}
	return SynchronizationPointOK;
}


void CLoadThreadsManagerScript::UserLock(
	int ThreadNum,
	JSContext* cx) {
	m_UserLockObj.Lock(ThreadNum, cx);
}

void CLoadThreadsManagerScript::UserUnlock(
	int ThreadNum,
	JSContext* cx) {
	m_UserLockObj.Unlock(ThreadNum, cx);
}


bool CLoadThreadsManagerScript::UserCheckUnlock(
	int ThreadNum) {
	return m_UserLockObj.CheckUnlock(ThreadNum);
}

void CLoadThreadsManagerScript::EnterJava()
{
	WlAssert(m_WLJSScript);
	m_WLJSScript.EnterJava();
}

void CLoadThreadsManagerScript::ExitJava()
{
	WlAssert(m_WLJSScript);
	m_WLJSScript.ExitJava();
}

void CLoadThreadsManagerScript::SetJavaThreadWrapperCreationScript()
{
	WlAssert(m_WLJSScript);
	m_WLJSScript.SetJavaThreadWrapperCreationScript();
}

bool CLoadThreadsManagerScript::IsJavaThreadWrapperCreationScriptExists()
{
	WlAssert(m_WLJSScript);
	return m_WLJSScript.IsJavaThreadWrapperCreationScriptExists();
}


//////////////////////////////// CLoadThreadsManagerContainer ///////////////////////////////

/*
CLoadThreadsManagerScriptContainer::CLoadThreadsManagerScriptContainer(CLoadThreadsManager* pTM) :
  m_MyLoadThreadsManager(pTM),
  m_RoundsCounter(0),
  m_StatisticsOn(true),
  m_Terminate(false),
  m_StatisticsFrequency(10),
  m_RuntimeTotal(0)
{
  m_StatStartEvent.Reset();
  Init();
}
*/

CLoadThreadsManagerScriptContainer::CLoadThreadsManagerScriptContainer() :
  m_RoundsCounter(0),
  m_StatisticsOn(true),
  m_Terminate(false),
  m_StatisticsFrequency(10),
  m_RuntimeTotal(0)
{
  m_StatStartEvent.Reset();
  Init();
}

CLoadThreadsManagerScriptContainer::~CLoadThreadsManagerScriptContainer()
{
  clearAndDestroy();
}


void CLoadThreadsManagerScriptContainer::InitParent(CLoadThreadsManager* pTM) {
	 m_MyLoadThreadsManager = pTM;
}

void CLoadThreadsManagerScriptContainer::Init()
{
  //
  // create the running thread functor:
  #if !defined(RV_THR_CANT_EXTRACT_TEMPLATE_ARGS_FROM_SIGNATURE)
    m_StatLoadSizeThread = rvMakeThreadFunction(
                        *this,                                                // instance
                        &CLoadThreadsManagerScriptContainer::StatLoadSizeFunc // member function
                      );
//    m_JSGCThread = rvMakeThreadFunction(
//                        *this,                                                      // instance
//                        &CLoadThreadsManagerScriptContainer::JSGarbageCollectorFunc // member function
//                      );
  #else
    m_StatLoadSizeThread = rvMakeThreadFunctionM(
                        CLoadThreadsManagerScriptContainer                    // class
                        *this,                                                // instance
                        void,                                                 // return value
                        &CLoadThreadsManagerScriptContainer::StatLoadSizeFunc // member function
                      );
//    m_JSGCThread = rvMakeThreadFunctionM(
//                        CLoadThreadsManagerScriptContainer                          // class
//                        *this,                                                      // instance
//                        void,                                                       // return value
//                        &CLoadThreadsManagerScriptContainer::JSGarbageCollectorFunc // member function
//                      );
  #endif

  //m_StatLoadSizeThread.start();
//  m_JSGCThread.start();
}

void
CLoadThreadsManagerScriptContainer::StartLoad(double startTime, int sliceSize)
{
  RVHashDictionaryIterator i(*this);
  while (i()) {
    CLoadThreadsManagerScript *pManagerScript = (CLoadThreadsManagerScript*)i.value();
    pManagerScript->StartLoad(startTime, sliceSize);
  }
  //
  // release the Load-size statistics thread.
  m_StatStartEvent.Set();

//  m_TimerTotal.reset();
}





bool CLoadThreadsManagerScriptContainer::LoadScript(
	const char* scriptName,
	const char* scriptFileName,
	const char* ScriptCode,
	double weight,
	const char* InitializationScript,
	int numAgendaInMix)
{
 CTraceMsg::RegisterAgenda (scriptFileName);
CLoadThreadsManagerScript *pManagerScript = new CLoadThreadsManagerScript(this, scriptName, numAgendaInMix);
  pManagerScript->LoadScript(scriptName, scriptFileName, ScriptCode, weight);
  insertKeyAndValue(new RVCollectableString(scriptName), pManagerScript);

  bool Rtn = pManagerScript->InitAgenda(InitializationScript);

	return Rtn;
}




void
CLoadThreadsManagerScriptContainer::StatLoadSizeFunc()
{
  try {
    TIMERENTRY("CLoadThreadsManagerScriptContainer::StatLoadSizeFunc");
    CTraceMsg::RegisterThread ();
    WlTraceMsg("Starting CLoadThreadsManagerScriptContainer::StatLoadSizeFunc");
    RVThreadSelf self = ::rvThread();
    self.setPriority(self.getMaxPriority());
    //self.setPriority(self.getMaxPriority()-1);
    //TVPSysThread::setPriority(TVPSysThread::Critical);
    m_StatStartEvent.Wait(); // wait for the statustucs to start.

    while (!m_MyLoadThreadsManager->IsTerminate()) {
      try {
        //RVCriticalSection::LockGuard lock(m_RoundsMutex);
        //statClient.AddLast("Load Size",m_MyLoadThreadsManager->m_NumRunning, CWLStatPointTypeEnum::LoadSize);
        RVHashDictionaryIterator i(*this);
        while (i()) {
          CLoadThreadsManagerScript *pManagerScript = (CLoadThreadsManagerScript*)i.value();
          pManagerScript->AddNumRunning(/*m_MyLoadThreadsManager->m_NumRunning*/);
        }
      } catch (...) {
        WlTraceMsg("CLoadThreadsManagerScriptContainer::StatLoadSizeFunc: Fatal error while adding statistics ");
        #ifdef _DEBUG
        throw;
        #endif
      }
      ::rvSleep(100);
    }
    WlTraceMsg("CLoadThreadsManagerScriptContainer::StatLoadSizeFunc : Done");
  } catch (...) {
    WlTraceMsg("Fatal Error in CLoadThreadsManagerScriptContainer::StatLoadSizeFunc");
    CAssociativeArray	ErrorMessage;
    ErrorMessage.AddCouple("$GENERATORLOGMESSAGE","Fatal Error in CLoadThreadsManagerScriptContainer::StatLoadSizeFunc");
    ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
    ErrorMessage.AddCouple("$TIME",    m_MyLoadThreadsManager->m_pMyLoadGeneratorSlave->fGetTimeFromBegin() );
    GetLoadGenerator()->SendMessage(ErrorMessage, 2);
    #ifdef _DEBUG
    throw;
    #endif
  }

}

const char*
CLoadThreadsManagerScriptContainer::SelectThreadsManagerScript(CLoadThreadInstance* currentThreadInstance, CLoadThreadInstanceScript* Previous,
	unsigned long& RoundNumScriptInSlave, unsigned long& RoundNumMixInSlave )
{
  CLoadThreadsManagerScript *pNextManagerScript = NULL;

  RVCriticalSection::LockGuard lock(m_RoundsMutex);
/*
  RVHashDictionaryIterator i(*this);
  while (i()) {
    CLoadThreadsManagerScript *pManagerScript = (CLoadThreadsManagerScript*)i.value();
    double curWeight = pManagerScript->CalculateCurrentWeight();
    //WlTraceMsg("Script (%s), weight %f", pManagerScript->m_ScriptName.data(), curWeight);
    if (curWeight<minWeight) {
      minWeight = curWeight;
      pNextManagerScript = pManagerScript;
    }
  }
*/
// by users

  int tmp_numRunning = m_MyLoadThreadsManager->m_NumRunning;
  if ( tmp_numRunning == 0 )  // if someone stoped the generator
    throw new CWLGeneratorTerminatedException();

  double diff = m_TimerTotal.get();
  m_TimerTotal.reset();

  RVHashDictionaryIterator i(*this);
  m_RuntimeTotal = 0;
  while (i()) {
    CLoadThreadsManagerScript *pManagerScript = (CLoadThreadsManagerScript*)i.value();
    pManagerScript->m_RuntimeTotal += pManagerScript->m_RunningTotal * diff;
    m_RuntimeTotal += pManagerScript->m_RuntimeTotal;
  }

  CLoadThreadsManagerScript *pPreviosManagerScript = NULL;
  if (Previous) {
    pPreviosManagerScript = Previous->m_MyThreadsManagerScript;
    double actualWeight = ((pPreviosManagerScript->m_RunningTotal * 100.0)/tmp_numRunning);
    if (actualWeight <= pPreviosManagerScript->m_Weight)
      pNextManagerScript = pPreviosManagerScript;
  }

  if (pNextManagerScript == NULL) {
    double minWeight=101;
    i.reset();
    while (i()) {
      CLoadThreadsManagerScript *pManagerScript = (CLoadThreadsManagerScript*)i.value();
      double actualWeight = ((pManagerScript->m_RunningTotal* 100.0)/tmp_numRunning);
      if (pManagerScript == pPreviosManagerScript || actualWeight < pManagerScript->m_Weight) {
        double curWeight = pManagerScript->CalculateCurrentWeight((pPreviosManagerScript == pManagerScript));
        if (curWeight<minWeight) {
          minWeight = curWeight;
          pNextManagerScript = pManagerScript;
        }
      }
    }
  }
  #ifdef _DEBUG
    if (pNextManagerScript==NULL) {
      GetThreadsManager()->reportLogCurrent(WLLogMessage::Fatal, "Next Script is NULL");
      throw;
    }
  #endif
  RoundNumMixInSlave = ++(pNextManagerScript->m_RoundsCounter);
  RoundNumScriptInSlave = ++m_RoundsCounter;

  //WlTraceMsg("Selected script (%s)", pNextManagerScript->m_ScriptName.data());

  if (Previous) Previous->m_MyThreadsManagerScript->m_RunningTotal--;
  pNextManagerScript->m_RunningTotal++;

  //
  // Added for the load-size manager
  // Yahali 11-Apr-2000
  if (Previous) {
    if (Previous->m_MyThreadsManagerScript != pNextManagerScript) {
      double now = TVPSysTime();
      Previous->m_MyThreadsManagerScript->m_LoadSizeManager.RemoveThread(now);
      pNextManagerScript->m_LoadSizeManager.AddThread(now);
    }
  }
  else {
    pNextManagerScript->m_LoadSizeManager.AddThread(TVPSysTime());
  }

  if (currentThreadInstance->m_FirstTimeSelectingScript) {
    m_MyLoadThreadsManager->m_MixNumRunning++;
    currentThreadInstance->m_FirstTimeSelectingScript = false;
  }

  return pNextManagerScript->m_ScriptName.data();
}

void
CLoadThreadsManagerScriptContainer::ReleaseThreadsManagerScript(CLoadThreadInstanceScript* Current)
{
  RVCriticalSection::LockGuard lock(m_RoundsMutex);
// by users
  double diff = m_TimerTotal.get();
  m_TimerTotal.reset();

  RVHashDictionaryIterator i(*this);
  m_RuntimeTotal = 0;
  while (i()) {
    CLoadThreadsManagerScript *pManagerScript = (CLoadThreadsManagerScript*)i.value();
    pManagerScript->m_RuntimeTotal += pManagerScript->m_RunningTotal * diff;
    m_RuntimeTotal += pManagerScript->m_RuntimeTotal;
  }
  if (Current) {
    Current->m_MyThreadsManagerScript->m_RunningTotal--;
    Current->m_MyThreadsManagerScript->m_LoadSizeManager.RemoveThread(TVPSysTime());
  }
}

CLoadThreadsManagerScript*
CLoadThreadsManagerScriptContainer::GetThreadsManagerScript(const char* name) const
{
  RVCString key(name);
  CLoadThreadsManagerScript* pMS = (CLoadThreadsManagerScript*)findValue((RVCollectable*)&key);
  return pMS;
}

CLoadGeneratorSlave*
CLoadThreadsManagerScriptContainer::GetLoadGenerator()
{
  return m_MyLoadThreadsManager->m_pMyLoadGeneratorSlave;
}

void
CLoadThreadsManagerScriptContainer::Terminate()
{
  //
  // Activate terminate functions.
//  MessageBox(0,"Terminate","terminate",MB_ICONERROR||MB_OK);
  TerminateAgenda();
  m_Terminate = true;
  //
  // Terminate all script managers.
  RVHashDictionaryIterator terminateRequestIterator(*this);
  while (terminateRequestIterator()) {
    CLoadThreadsManagerScript *pManagerScript = (CLoadThreadsManagerScript*)terminateRequestIterator.value();
    pManagerScript->RequestForTerminate();
  }

  RVHashDictionaryIterator waitForTerminateIterator(*this);
  while (waitForTerminateIterator()) {
    CLoadThreadsManagerScript *pManagerScript = (CLoadThreadsManagerScript*)waitForTerminateIterator.value();
    pManagerScript->WaitForTerminateToEnd();
  }

  m_StatStartEvent.Set();
  
  if ( m_StatLoadSizeThread.getExecutionState() != RV_THR_INITIAL &&
       m_StatLoadSizeThread.join(8000) == RV_THR_TIMEOUT ) 
  {
    try 
    {
      m_StatLoadSizeThread.terminate();
    }
    catch (...) {}
  }
  
}

void
CLoadThreadsManagerScriptContainer::OnSevereError() {
  RVHashDictionaryIterator i(*this);
  while (i()) {
    CLoadThreadsManagerScript *pManagerScript = (CLoadThreadsManagerScript*)i.value();
    pManagerScript->OnSevereError();
  }
}


void
CLoadThreadsManagerScriptContainer::TerminateAgenda()
{
  RVHashDictionaryIterator i(*this);
  while (i()) {
    CLoadThreadsManagerScript *pManagerScript = (CLoadThreadsManagerScript*)i.value();
    pManagerScript->TerminateAgenda();
  }
}

void CLoadThreadsManagerScriptContainer::ReleaseSynchronizationPoints() {
  RVHashDictionaryIterator i(*this);
  while (i()) {
    CLoadThreadsManagerScript *pManagerScript = (CLoadThreadsManagerScript*)i.value();
    pManagerScript->ReleaseSynchronizationPoints();
  }
}

void CLoadThreadsManagerScriptContainer::InputFileData(CAssociativeArray* AssoArrayCommand) {
  const char* ScriptName = AssoArrayCommand->GetValueOfParameter("$SCRIPTNAME");
  RVCollectableString key(ScriptName);
  CLoadThreadsManagerScript *pManagerScript = (CLoadThreadsManagerScript*)findValue(&key);
  pManagerScript->FillInputFileQueue(AssoArrayCommand);
}
/*
void CLoadThreadsManagerScriptContainer::InputFileRelease() {
  RVHashDictionaryIterator i(*this);
  while (i()) {
    CLoadThreadsManagerScript *pManagerScript = (CLoadThreadsManagerScript*)i.value();
    pManagerScript->InputFileRelease();
  }
}
*/
void CLoadThreadsManagerScriptContainer::InputFileTerminate() {
  RVHashDictionaryIterator i(*this);
  while (i()) {
    CLoadThreadsManagerScript *pManagerScript = (CLoadThreadsManagerScript*)i.value();
    pManagerScript->InputFileTerminate();
  }
}

//////////////////////////////// CRTAMThreadInstanceScript ///////////////////////////////
CRTAMThreadInstanceScript::CRTAMThreadInstanceScript(CRTAMThreadInstance* pTI, iLoadSingleClient* pISC):
  CLoadGeneralThreadInstanceScript(pTI, *pISC->m_jsScript),
  m_LoadSingleClient(pISC)
{
}
  
CRTAMThreadInstanceScript::~CRTAMThreadInstanceScript(){
}

bool CRTAMThreadInstanceScript::Init()
{
  if (!m_JSThreadInitDone) {
    if (!m_MyJSThread.Init(
            m_LoadSingleClient->fGetProcess().GetLoadRuntime(),
            this,
            m_LoadSingleClient->mManagerScriptData,
            m_MyThreadInstance->GetLGThreadNum()))
      return false;

    m_JSThreadInitDone = true;
  }

  return true;
}

void CRTAMThreadInstanceScript::addLast(const char *name, double l, CWLStatPointTypeEnum::PointType type) {
  m_LoadSingleClient->fThreadAddLast(name, l, type);
};

bool CRTAMThreadInstanceScript::vIsJSGC()
{
  return true;
}

void CRTAMThreadInstanceScript::addMeasurement(const char *name, double l) {
  addLast(name, l, CWLStatPointTypeEnum::Measurement);
}

void CRTAMThreadInstanceScript::addTimer(const char *name, double l) {
  addLast(name, l, CWLStatPointTypeEnum::Timer);
}

void CRTAMThreadInstanceScript::addCounter(const char *name) {
  addLast(name, 1, CWLStatPointTypeEnum::Counter);
}

void CRTAMThreadInstanceScript::addAccumulator(const char *name, double l) {
  addLast(name, l, CWLStatPointTypeEnum::Accumulator);
}

void CRTAMThreadInstanceScript::resetTimer(const char *name) {
  m_LoadSingleClient->fThreadResetTimer(name);
}

void CRTAMThreadInstanceScript::sendTimer(const char *name) {
  m_LoadSingleClient->fThreadSendTimer(name);
}

RVCString& CRTAMThreadInstanceScript::GetScriptName(){
  return m_LoadSingleClient->mManagerScriptData.ScriptName();
}

RVCString& CRTAMThreadInstanceScript::GetScriptFileName(){
  return m_LoadSingleClient->mManagerScriptData.ScriptFileName();
}

IFileDictionary* CRTAMThreadInstanceScript::GetFileDict() {
  return m_MyThreadInstance->GetFileDict();
}

// For iLEAgendaJSEmbedContext
CWLJSScript& CRTAMThreadInstanceScript::GetJSScript() {
  WlAssert(m_LoadSingleClient);
  return *m_LoadSingleClient->m_jsScript;
}

bool CRTAMThreadInstanceScript::vBeforeSleep(long int Time) {
  return m_LoadSingleClient->fBeforeSleep( Time);
}

void CRTAMThreadInstanceScript::vAfterSleep() {
  m_LoadSingleClient->fAfterSleep();
}
