/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
//#pragma warning( disable : 4290 )

//#include <hashdict.h>
#include <RVCString.h>
//#include <collstr.h>
//#include <collint.h>
//#include <ordered.h>


#if defined(__TV_solaris__)                                   
#include <pthread.h>
#include <unistd.h>                                          
#endif

#include "platform.h"
#include "tracedll.h"
#include "tvpsync.h"
#include "tvenv.h"
#include "socketlib.h"

#include "loadengineglobals.h"
#include "loadexceptions.h"
#include "associativearray.h"
#include "loadthreadinstance.h"
#include "loadthreadsmanager.h"
#include "loadgeneratorslaveproxy.h"
#include "loadgeneratorslave.h"
#include "loadgenerator.h"
#include "loadagendamanager.h"
#include "threadglobalshandler.h"
#include "loadenginedecs.h"
#include "loadengineinifileproperties.h"


#include "wljsruntime.h"
#include "wlleprotocol.h"
#include "tvenv.h"
#include <RVCTokenizer.h>


#include "loadengineselistener.h"
#include "Log.h"

CLoadGeneratorSlave::CLoadGeneratorSlave( 
	const char* ProxyAddress,
                                          const char* SlaveNo,
	CGeneralProcess& rProcess)
:
	iSession(rProcess),
	iLEJSAgendaMachineThread(rProcess),
//  m_MessagesCommunicator(this, SlaveNo)
  m_MessagesCommunicator(SlaveNo),
	m_MonitorAddress(""),
	m_MonitorAddressInit(false),
  m_ThreadPoolOn(false),
  mGCThread(*this)
{
  m_MyProxy = NULL;
  m_ProxyAddress = ProxyAddress;
  m_MyThreadsManager = NULL;
  m_StatisticsFrequency = 10;
  m_StartTime        = 0;
  m_Terminated       = false; 
  m_StopMeDone       = false; 
  m_SendingLogMessages = true;
  m_LogMessagesCounter = 0;
  m_LogMessagesMax = CLEIniFileProperties::LogMessagesMax;
  m_SendingTransactions = true;
  m_TransactionsCounter = 0;
  m_TransactionsMax = CLEIniFileProperties::TransactionMax;
  m_StatisticalMesurementsMask = 0;
  m_SlaveBaseThreadId = 0;

	m_MasterPID = -1;
  m_GeneratorType   = Regular;

	m_MessagesCommunicator.InitParent(this);

  WLERRGetErrorFormat(LoadExceptionFrmFile, 10);
  WLERRGetErrorFormat(JSErrorsFrmFile, 10000);
  WLERRGetErrorFormat(JSSyntaxChecks, 50);

  m_MessagesCommunicator.Init();

}

//
// StandAlone Slave.
//InProcess
CLoadGeneratorSlave::CLoadGeneratorSlave( 
	CLoadGeneratorSlaveProxy* myProxy,
  const char* SlaveNo ,
	CGeneralProcess& rProcess) 
:
	iSession(rProcess),
	iLEJSAgendaMachineThread(rProcess),
  m_MyProxy(myProxy),
//  m_MessagesCommunicator(this, SlaveNo)
	m_MonitorAddress(""),
	m_MonitorAddressInit(false),
  mGCThread( *this ),
	m_MessagesCommunicator(SlaveNo),
  m_MustLogBaseSeverity(WLLogMessage::Fatal)
{
  m_MyThreadsManager = NULL;
  m_StatisticsFrequency = 10;
  m_StartTime       = 0;
  m_Terminated       = false;
  m_StopMeDone       = false; 
  m_SendingLogMessages = true;
  m_LogMessagesCounter = 0;
  m_LogMessagesMax = CLEIniFileProperties::LogMessagesMax;
  m_SendingTransactions = true;
  m_TransactionsCounter = 0;
  m_TransactionsMax = CLEIniFileProperties::TransactionMax;
  m_StatisticalMesurementsMask = 0;
  m_SlaveBaseThreadId = 0;

  m_GeneratorType = InProcess;

	m_MasterPID = -1;

	m_MessagesCommunicator.InitParent(this);

  #if !defined(RV_THR_CANT_EXTRACT_TEMPLATE_ARGS_FROM_SIGNATURE)            
    m_MainLoopThread = rvMakeThreadFunction(
                        *this,                                // instance
                        &CLoadGeneratorSlave::MainLoop        // member function
                      );
  #else
    m_MainLoopThread = rvMakeThreadFunctionM( 
                        CLoadGeneratorSlave                   // class
                        *this,                                // instance
                        void,                                 // return value
                        &CLoadGeneratorSlave::MainLoop        // member function
                      );
  #endif

}

CLoadGeneratorSlave::~CLoadGeneratorSlave()
{
}

CAssociativeArray* CLoadGeneratorSlave::GetMessage() 
{ 
  switch (m_GeneratorType) {
    case Regular:
    case InProcess:
      return (CAssociativeArray*)m_ControlMessageQueue.waitEvent(); 

    case StandAlone:
      //!!ToDo: return LGSlaveStandAlone(this);

    default:
      WlAssert(NULL);
      return NULL;
  }
  return NULL;
}

void CLoadGeneratorSlave::Start()
{

  // Make sure to start the InProcess thread only in InProcess mode .....:)
  if( m_GeneratorType == InProcess )
    m_MainLoopThread.start();

}

void CLoadGeneratorSlave::Terminate() {

  iSession::fFinal();     // in iSession
 
  WlTraceMsg("CLoadGeneratorSlave::Terminate - Wait for JavaScript GC to terminate");
  
  // Tell JSGC thread to terminate (destroying its own context)
	mGCThread.fTerminate();

  if( m_JSGCThread.isValid() ) {
		if (m_JSGCThread.join(CLEIniFileProperties::JSGCTerminateTimeout) == RV_THR_TIMEOUT) {
	    WlTraceMsg("CLoadGeneratorSlave::Terminate - ***** Brutal Terminate of JavaScript GC thread after timeout");
		  m_JSGCThread.terminate();
		}
	}

#ifdef _DEBUG
  // Commented out again because we don't need the dtors to catch leaks anymore

  /*
  // Chaim - delete causes exception in slave
  if (m_MyThreadsManager && m_GeneratorType == InProcess) delete m_MyThreadsManager;

  iLEJSMachineThread::fFinal();

  fGetProcess().Terminate();
  */

#endif
}

void CLoadGeneratorSlave::SendMessage( CAssociativeArray& Message, int priority )
{
  switch (m_GeneratorType) {
    case Regular:
      //Message.AddCouple("$FROM",WhoAmI());
      m_MessagesCommunicator.SendMessage(Message, m_ProxyAddress, "", priority);
      break;
    case InProcess:
      m_MyProxy->MessageHandler(Message);
      break;
    case StandAlone:
      if (Message.KeyExists("$AGENDASTATUS")) 
        cout << "Status: " << Message["$AGENDASTATUS"] << " NumRunning: " << Message["$AGENDARUNNING"] << endl;
      break;
    default:
      break;
  }
}

void  CLoadGeneratorSlave::SendMessageToAddress( CAssociativeArray& Message, const char* netAddress, int priority)
{
	switch (m_GeneratorType) {
	case Regular: 
		m_MessagesCommunicator.SendMessage(Message, netAddress, "", priority);
		break;
	case InProcess:
		(m_MyProxy->GetAgendaManager()->GetLoadGenerator())->SendMessageToAddress(Message, netAddress, priority);
		break;
	case StandAlone:
		if (Message.KeyExists("$AGENDASTATUS")) 
			cout << "Status: " << Message["$AGENDASTATUS"] << " NumRunning: " << Message["$AGENDARUNNING"] << endl;
		break;
	default:
		break;
	}
}

void CLoadGeneratorSlave::MainLoop()
{
	TIMERENTRY("CLoadGeneratorSlave::MainLoop");
	CTraceMsg::RegisterThread ();

#ifdef __TV_WINDOWS__  
  HANDLE hThread = GetCurrentThread();
  SetThreadPriority(hThread,15);
  //TVPSysThread::setPriority(TVPSysThread::Critical);
#endif
#ifdef __TV_UNIX__
  sched_param  sc;
#ifdef __TV_solaris__
  sc.sched_priority = 127;
#else
  sc.sched_priority = 127;
#endif
  pthread_t th = pthread_self();

  pthread_setschedparam(th,SCHED_OTHER,&sc);
#endif
  

  // Message to indicate that the slave was created successfully
	// It was added in order that in case of spawning the loadgeneratorproxy will get 
	// the status of the creation of the new process.
	if(m_GeneratorType != InProcess) {
		CAssociativeArray SlaveCreatedSuccessfully;
		SlaveCreatedSuccessfully.AddCouple("$CREATESLAVESTATUS", "CREATEDSUCCESSFULLY");
		SendMessage(SlaveCreatedSuccessfully, 1);
	}
  //
  // Init Messages.
  CAssociativeArray InitMessage;
  InitMessage.AddCouple("$SLAVEREADY",1L);
  InitMessage.AddCouple("$SLAVEADDRESS",WhoAmI());
  SendMessage(InitMessage, 2);

	CAssociativeArray *AssoArrayCmd = GetMessage();
	m_MonitorGlobalVariablesAddress = AssoArrayCmd->GetValueOfParameter("$MONITORGLOBALSNAME");
	InitMonitorAddress(AssoArrayCmd->GetValueOfParameter("$MONITORNAME"));
	m_MasterPID = atoi(AssoArrayCmd->GetValueOfParameter("$LGMASTERPID"));
	m_MonitorUID = AssoArrayCmd->GetValueOfParameter("$MONITORUID");
	if( NULL == m_MonitorUID.data() )
		m_MonitorUID = "";
	if( NULL == m_MonitorUID.data() )
		m_MonitorUID = "";

	/*GALTECH*/
#ifndef _ANSI_ONLY
	if ( AssoArrayCmd->KeyExists( "$MONITORCHARSET" ) )
		SetMonitorCharSet(AssoArrayCmd->GetLongValueOfParameter("$MONITORCHARSET"));
#endif

  m_ThreadPoolOn = CLEIniFileProperties::ThreadPoolOn;
  
  // Check value of threadpool flag in message only on Linux
#ifdef __TV_linux__ 
    m_ThreadPoolOn = (bool)AssoArrayCmd->GetLongValueOfParameter("$THREADPOOLON", 0);
#endif
  
	delete AssoArrayCmd;
	AssoArrayCmd = NULL;

	fGetProcess().SetMasterPid(m_MasterPID);

  RVCString a_ErrDll;
  if (!fGetProcess().Init(a_ErrDll)){

		CAssociativeArray ErrorMessage;
		char Msg[400];
		sprintf(Msg,"Failed to initialize the following WebLOAD protocols: %s. These protocols will not be available in the Agenda.", a_ErrDll.data());
		ErrorMessage.AddCouple("$GENERATORLOGMESSAGE", Msg);
		ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Warning);
		ErrorMessage.AddCouple("$TIME", fGetTimeFromBegin());
		SendMessage(ErrorMessage, 2);
  }

  iSession::fInit();

  RVJS_SetAtomTimeclipState( fGetProcess().GetLoadRuntime().fGet(), (CLEIniFileProperties::UseAtomClip? JS_TRUE : JS_FALSE));

  iLEJSAgendaMachineThread::fInit(THREAD_TYPE_NONE);

 // Start the MainLoop
  while ( true )
  {
	  try { LE_SET_SE_TRANSLATOR();


	if (AssoArrayCmd)
	{
		delete AssoArrayCmd;
		AssoArrayCmd = NULL;
	}

      AssoArrayCmd = GetMessage();
      CAssociativeArray& AssoArrayCommand = *AssoArrayCmd;

      WlTraceMsg("CLoadGeneratorSlave::MainLoop - Received command (%s)",AssoArrayCommand.ToString());
      
      if ( AssoArrayCommand.KeyExists( "$CREATE" ) )
        {
          CLog::Write("slave lg $CREATE", eCommand );
          m_LoadGeneratorNo = AssoArrayCommand.GetValueOfParameter("$LOADGENERATORNO");
          m_CurrentRequestId = AssoArrayCommand.GetLongValueOfParameter("$REQUESTID");
          m_SlaveBaseThreadId = AssoArrayCommand.GetLongValueOfParameter("$SLAVEBASEID");

          // Handle the Load-Agenda message first.
          CAssociativeArray LoadMessage;
          LoadMessage.FromString(AssoArrayCommand["$LOADAGENDAMESSAGE"]);
					LoadMessage.AddCouple("$REQUESTID", m_CurrentRequestId);
          if ( LoadMessage.KeyExists( "$LOADAGENDA" ) )
            LoadThreadsManager( false, LoadMessage);
          else if ( AssoArrayCommand.KeyExists( "$LOADCLIENTAGENDA" ) )
            LoadThreadsManager( true, LoadMessage);
          else 
            ;// !Error
          if (m_ThreadPoolOn) {
            m_MyThreadsManager->LoadNumThreads(GetMaxThreadsPerSlave());
          }
        }
      else if ( AssoArrayCommand.KeyExists( "$CREATE&START" ) ) 
        {
          CLog::Write("slave lg $CREATE&START", eCommand );
          m_LoadGeneratorNo = AssoArrayCommand.GetValueOfParameter("$LOADGENERATORNO");
          m_CurrentRequestId = AssoArrayCommand.GetLongValueOfParameter("$REQUESTID");

					//  get the starttime.  In general the start time
					//  is given in the startsession message. But becuase
					//  the will start after the session was started
					//  need to get the starttime earlier.  For example,
					//  the InitAgenda can send InfoMessages which has
					//  time.
          //
					//  Get the StartSession message
          CAssociativeArray *StartMessage = new CAssociativeArray;
          StartMessage->FromString(AssoArrayCommand["$STARTSESSIONMESSAGE"]);
          m_StartTime = StartMessage->GetDoubleValueOfParameter("$STARTTIME");

          // Handle the Load-Agenda message first.
          CAssociativeArray LoadMessage;
          LoadMessage.FromString(AssoArrayCommand["$LOADAGENDAMESSAGE"]);
					LoadMessage.AddCouple("$REQUESTID", m_CurrentRequestId);
          if( LoadMessage.KeyExists( "$LOADAGENDA" ) ) {
            LoadThreadsManager( false, LoadMessage);
          } else if( AssoArrayCommand.KeyExists( "$LOADCLIENTAGENDA" ) ) {
            LoadThreadsManager( true, LoadMessage);
          } else {
            WlAssert(0);
					}
          //
          // Put the StartSession and SetNumRunning messages in the queue.
          StartMessage->AddCouple("$REQUESTID", m_CurrentRequestId);
          StartMessage->AddCouple("$SLAVEBASEID", AssoArrayCommand.GetLongValueOfParameter("$SLAVEBASEID"));
          PutMessage(StartMessage);
          CAssociativeArray *RunMessage = new CAssociativeArray;
          RunMessage->FromString(AssoArrayCommand["$SETNUMRUNNINGMESSAGE"]);
          RunMessage->AddCouple("$REQUESTID", m_CurrentRequestId);
          PutMessage(RunMessage);
        }
      else if ( AssoArrayCommand.KeyExists( "$STARTSESSION" ) )
        {
          CLog::Write("slave lg $STARTSESSION", eCommand );
          m_CurrentRequestId = AssoArrayCommand.GetLongValueOfParameter("$REQUESTID");
          m_StatisticsFrequency = AssoArrayCommand.GetLongValueOfParameter("$REPORTRESOLUTION");
					if (m_StartTime == 0)
						m_StartTime = AssoArrayCommand.GetDoubleValueOfParameter("$STARTTIME");
          m_SlaveBaseThreadId = AssoArrayCommand.GetLongValueOfParameter("$SLAVEBASEID");
	
          m_MyThreadsManager->StartLoad(m_StartTime, m_StatisticsFrequency);
          fGetProcess().ProtocolProcessAfterStartLoad(m_StatisticsFrequency);

          ReportLoadGeneratorSlaveStatus("$STARTSESSIONDONE", 1);
          //
          // New Parameters for transactions, log-messages, and timer sending instructions.
          if ( AssoArrayCommand.KeyExists( "$DONTSENDTIMERS" ) )
            m_StatisticalMesurementsMask |= (int)CWLStatPointTypeEnum::Timer;
          if ( AssoArrayCommand.KeyExists( "$DONTSENDTRANSACTIONTIMERS" ) )
            m_StatisticalMesurementsMask |= (int)CWLStatPointTypeEnum::TransactionTimer;
          if ( AssoArrayCommand.KeyExists( "$LOGMESSAGESMAX" ) )
            m_LogMessagesMax = AssoArrayCommand.GetLongValueOfParameter("$LOGMESSAGESMAX");
					if ( AssoArrayCommand.KeyExists( "$ALWAYSLOGERRORS" ) )
						m_MustLogBaseSeverity = WLLogMessage::Error;
          if ( AssoArrayCommand.KeyExists( "$TRANSACTIONSMAX" ) ) {
            m_TransactionsMax = AssoArrayCommand.GetLongValueOfParameter("$TRANSACTIONSMAX");
            if (m_TransactionsMax==0)
              m_SendingTransactions = false;
          }
        }
           
      else if ( AssoArrayCommand.KeyExists( "$LOADAGENDA" ) )
        {
          CLog::Write("slave lg $LOADAGENDA", eCommand );
          m_CurrentRequestId = AssoArrayCommand.GetLongValueOfParameter("$REQUESTID");
          LoadThreadsManager( false, AssoArrayCommand );
          if (m_ThreadPoolOn) {
            m_MyThreadsManager->LoadNumThreads(GetMaxThreadsPerSlave());
          }
        }

      else if ( AssoArrayCommand.KeyExists( "$LOADCLIENTAGENDA" ) )
        {
          CLog::Write("slave lg $LOADCLIENTAGENDA", eCommand );
          m_CurrentRequestId = AssoArrayCommand.GetLongValueOfParameter("$REQUESTID");
          LoadThreadsManager( true, AssoArrayCommand );
        }
      
      else if ( AssoArrayCommand.KeyExists( "$SETNUMRUNNING" ) )
        {
          CLog::Write("slave lg $SETNUMRUNNING", eCommand );
          m_CurrentRequestId = AssoArrayCommand.GetLongValueOfParameter("$REQUESTID");
          int numThreads = AssoArrayCommand.GetLongValueOfParameter("$SETNUMRUNNING");
          if (!(m_MyThreadsManager->setNumActive(numThreads))) {
            ReportLoadGeneratorSlaveStatus("$SETNUMRUNNINGDONE", 0L);
          } else {
            ReportLoadGeneratorSlaveStatus("$SETNUMRUNNINGDONE", 1L);
          }
        }

			else if( AssoArrayCommand.KeyExists("$GLOBALSNAME") )
			{
				CLog::Write("slave lg $GLOBALSNAME", eCommand );
				if ( AssoArrayCommand.KeyExists( "$THREADID" ) ) {
					CLoadThreadInstance *thread = m_MyThreadsManager->GetThread(AssoArrayCommand.GetLongValueOfParameter("$THREADID"));
					thread->GetGlobalsRequestHandler()->HandleReplay(AssoArrayCommand);
				}
				else 
					m_MyThreadsManager->m_ThreadsManagerGlobalsHandler->HandleReplay(AssoArrayCommand);
      }

      else if ( AssoArrayCommand.KeyExists( "$TERMINATE" ) )
        {
          CLog::Write("slave lg $TERMINATE", eCommand );
          WlTraceMsg("CLoadGeneratorSlave::MainLoop(%s) - Recieved exit command", WhoAmI());

          if (m_Terminated)
            continue;
          
					PURIFY_NEW_LEAKS(false,"CLoadGeneratorSlave::MainLoop() - End - Leaks");
					PURIFY_MEMORY(false,"CLoadGeneratorSlave::MainLoop() - End - memory");
          
          m_Terminated = true;
          m_CurrentRequestId = AssoArrayCommand.GetLongValueOfParameter("$REQUESTID");

          if (m_MyThreadsManager){
             WlTraceMsg("CLoadGeneratorSlave::MainLoop(%s) - calling stop agenda", WhoAmI());
             m_MyThreadsManager->StopAgenda();
    		  }


          Terminate();

          ReportLoadGeneratorSlaveStatus("$STOPSLAVEDONE", 1, 5);
          
      //    WlTraceMsg("CLoadGeneratorSlave::MainLoop(%s) - Before stopping communicator", WhoAmI());
          m_MessagesCommunicator.Terminate();


          if (AssoArrayCmd)
            delete AssoArrayCmd;
          

				//	WlTraceMsg("CLoadGeneratorSlave::MainLoop(%s) - Exit done");
					break;

        }

      else if ( AssoArrayCommand.KeyExists( "$STOPSENDINGLOGMESSAGES" ) )
        {
          CLog::Write("slave lg $STOPSENDINGLOGMESSAGES", eCommand );
          m_SendingLogMessages = false;
        }
      
      else if ( AssoArrayCommand.KeyExists( "$STOPSENDINGTRANSACTIONS" ) )
        {
          CLog::Write("slave lg $STOPSENDINGTRANSACTIONS", eCommand );
          m_SendingTransactions = false;
        }

/*    else if ( AssoArrayCommand.KeyExists( "$INPUTFILE" ) )
      {
        m_MyThreadsManager->InputFileData(&AssoArrayCommand);
      } */

    else
        {
          WlTraceMsg("Illegal command (%s)",AssoArrayCmd->ToString());
          //SendMessage( WhoAmI(), "Wrong command: " + AssoArrayString 
        }


      } catch (std::exception& e) {
				ExceptionHandler(&e);

      } catch (std::exception* e) {
				ExceptionHandler(e);
				delete e; // Only for catch of pointer 

      } catch (...) {
        try {
          WLErrorException e("LoadGenerator::MainLoop - Exception not handled with command (%s)",WLLogMessage::Fatal, AssoArrayCmd->ToString());
          WlErrorMsg(e.GetErrorMsg());
          CAssociativeArray ErrorMessage;
          ErrorMessage.AddCouple("$GENERATORLOGMESSAGE",e.GetErrorMsg());
          ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
          ErrorMessage.AddCouple("$TIME", fGetTimeFromBegin());
          SendMessage(ErrorMessage, 2);
          StopMe();
        } catch (...) {
        WlAssert(0);
				}
        #ifdef _DEBUG
        throw;
        #endif
      }
  }

  // @@Chaim
  WlTraceMsg("Terminating LoadRuntime");
  fGetProcess().GetLoadRuntime().Terminate();



}

void CLoadGeneratorSlave::ExceptionHandler(std::exception* e){
	RVCString FullExcName ("");
	WLERRGetErrorMsg(e, FullExcName);

  // Check if WLErrorException and if reported
  WLErrorException* aException;

  if (aException = dynamic_cast<WLErrorException*>(e)){
    if (aException->GetReported()){
      StopMe(); // Already reported, just stop
      return;
    }
  }
  
	RVCString StackTrace("");
	if (WLERRGetStackTrace(e, StackTrace) && StackTrace.length() > 0){
	  WlErrorMsg("CLoadGeneratorSlave(%s)::MainLoop - Exception %s occured\nTrace:\n%s", m_MessagesCommunicator.WhoAmI() ,FullExcName, StackTrace);
	  printf("CLoadGeneratorSlave(%s)::MainLoop - Exception %s occured\nTrace:\n%s", m_MessagesCommunicator.WhoAmI() ,FullExcName, StackTrace);
	}
	else {
	  WlErrorMsg("CLoadGeneratorSlave(%s)::MainLoop - Exception %s occured", m_MessagesCommunicator.WhoAmI() ,FullExcName);
	  printf("CLoadGeneratorSlave(%s)::MainLoop - Exception %s occured", m_MessagesCommunicator.WhoAmI() ,FullExcName);
	}
  CAssociativeArray ErrorMessage;
  ErrorMessage.AddCouple("$GENERATORLOGMESSAGE",FullExcName);
  ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
  ErrorMessage.AddCouple("$TIME", fGetTimeFromBegin());
  SendMessage(ErrorMessage, 2);

  StopMe();
}

double CLoadGeneratorSlave::vGetBeginTime() 
{
  return m_StartTime;
}


const char* CLoadGeneratorSlave::WhoAmI()
{
  return m_MessagesCommunicator.WhoAmI();
}


void CLoadGeneratorSlave::StopMe()
{
	WlTraceMsg("CLoadGeneratorSlave::StopMe invoked");

  if (m_StopMeDone || m_Terminated)
    return;

  m_StopMeDone = true;
  CAssociativeArray *StopMessage = new CAssociativeArray;
  StopMessage->AddCouple("$TERMINATE", 1L);
  StopMessage->AddCouple("$REQUESTID", -1L);
  PutMessage(StopMessage);
	ForceKillLoadGenerator();
}

void CLoadGeneratorSlave::ReportLoadGeneratorSlaveStatus( const char* MessageName, long status, long priority )
{
  CAssociativeArray ReportMessage;
  ReportMessage.AddCouple(MessageName, 1L);
  ReportMessage.AddCouple("$RETVALUE", status);
  ReportMessage.AddCouple("$REQUESTID", m_CurrentRequestId);
	ReportMessage.AddCouple("$STATISTICSFREQUENCY", (long)m_StatisticsFrequency);
  SendMessage(ReportMessage, priority);
}

void CLoadGeneratorSlave::LoadThreadsManager( bool oneClient, CAssociativeArray& AssoArrayCommand)
{
	m_LoadGeneratorNo = AssoArrayCommand.GetValueOfParameter("$LOADGENERATORNO", m_LoadGeneratorNo.data());
  const char* ThreadsManagerName = AssoArrayCommand.GetValueOfParameter("$AGENDANAME");
	const char* ThreadsManagerRealName = AssoArrayCommand.GetValueOfParameter("$AGENDAPATH");

  if ( m_MyThreadsManager == NULL )
    m_MyThreadsManager = new CLoadThreadsManager(ThreadsManagerName);

	m_MyThreadsManager->Init(ThreadsManagerName, ThreadsManagerRealName, this, oneClient);

  const char* GeneralInitScript = AssoArrayCommand.GetValueOfParameter("$INITSCRIPT","");
  SetMaxThreadsPerSlave(AssoArrayCommand.GetLongValueOfParameter("$MAXTHREADSPERSLAVE"));

  int numScripts = AssoArrayCommand.GetLongValueOfParameter("$NUMSCRIPTS");
  for (int i=1; i<=numScripts; i++) {
    const char* scriptName = AssoArrayCommand.GetValueOfParameterArray("$SCRIPTNAME",i);
    const char* scriptFile = AssoArrayCommand.GetValueOfParameterArray("$SCRIPTFILE", i);
    const char* scriptData = AssoArrayCommand.GetValueOfParameterArray("$SCRIPTDATA", i, NULL);
    double weight = AssoArrayCommand.GetDoubleValueOfParameterArray("$SCRIPTWEIGHT",i);

    const char* ScriptInitScript = AssoArrayCommand.GetValueOfParameterArray("$SCRIPTINITSCRIPT", i, "");
		RVCString InitScript;

    if (CLEIniFileProperties::IgnoreInitScript){
      InitScript = "{}";
    }
    else{
		  InitScript += "{";
		  InitScript += GeneralInitScript;
		  InitScript += ";}\n";
		  InitScript += "{";
		  InitScript += ScriptInitScript;
		  InitScript += ";}";
    }

    if (!m_MyThreadsManager->LoadScript(scriptName, scriptFile, scriptData, weight, InitScript.data(), i )) {
      ReportLoadGeneratorSlaveStatus("$LOADAGENDADONE", 0L);
      StopMe();
      return;
		}

		PURIFY_NEW_LEAKS(false,"CLoadGeneratorSlave::LoadThreadsManager - Start - Leaks");
		PURIFY_MEMORY(false,"CLoadGeneratorSlave::LoadThreadsManager - Start - memory");
  }
  //
  // Chaim - create the running thread functors for JSCG 
  m_JSGCTerminateEvent.Reset();
  
  ReportLoadGeneratorSlaveStatus("$LOADAGENDADONE", 1L);
  #if !defined(RV_THR_CANT_EXTRACT_TEMPLATE_ARGS_FROM_SIGNATURE)
    m_JSGCThread = rvMakeThreadFunction(
                        *this,                                        // instance
                        &CLoadGeneratorSlave::JSGCFunc          // member function
                      );
  #else
    m_JSGCThread = rvMakeThreadFunction(
                        CWLJSScript                                   // class
                        *this,                                        // instance
                        void,                                         // return value
                        &CLoadGeneratorSlave::JSGCFunc          // member function
                      );
  #endif
    
  mGCThread.fPreInit(fGetProcess().GetLoadRuntime());
  m_JSGCThread.start();
}

//
// Functions for eliminating sending Messages, Transactions 
// and Statistical Measurements.
bool CLoadGeneratorSlave::IsSendLogMessage()
{
  return m_SendingLogMessages;
}

void CLoadGeneratorSlave::SetSendLogMessage(bool send)
{
  m_SendingLogMessages = send;
}

void CLoadGeneratorSlave::AddLogMessage()
{
  m_LogMessagesCounter++;
  if (m_LogMessagesMax != -1 && m_LogMessagesCounter >= m_LogMessagesMax)
    m_SendingLogMessages = false;
}

bool CLoadGeneratorSlave::IsSendTransactions()
{
  return m_SendingTransactions;
}

void CLoadGeneratorSlave::AddTransactions(int count)
{
  m_TransactionsCounter += count;
  if (m_TransactionsCounter >= m_TransactionsMax)
    m_SendingTransactions = false;
}

bool CLoadGeneratorSlave::IsSendStatisticalMesurement(CWLStatPointTypeEnum::PointType type)
{
  return (m_StatisticalMesurementsMask & (int)type) == 0;
}

void CLoadGeneratorSlave::ForceKillLoadGenerator() {
	CAssociativeArray killMsg;
	char localHostName[100] = "localhost";

    CSocketLib::GetLocalHostName(localHostName, 100);
	CTestTalkAddress sendTo("theTml", localHostName);

	killMsg.AddCouple("$KILLLOADGENERATOR", "");
	killMsg.AddCouple("$GENERATORUID", m_MonitorUID.data());

	SendMessageToAddress(killMsg, sendTo.Address(), 1);
}

bool CLoadGeneratorSlave::AddJavaStream(JSContext*   cx,
                                        const char*  streamName,
                                        const char*  codeBase,
											                  RVCString&   javaStream)
{
  bool ret;// = JavaStreamExists(streamName);
  //if ( !ret )
	ret = m_JavaStreamTable.AddJavaStream(cx,
                                        streamName,
                                        codeBase,
                                        javaStream.data(),
                                        javaStream.length());
  return ret;
}

bool CLoadGeneratorSlave::JavaStreamExists(const char* streamName)
{
	return m_JavaStreamTable.JavaStreamExists(streamName);
}

void CLoadGeneratorSlave::JSGCFunc() {
	TIMERENTRY("CLoadGeneratorSlave::JSGCFunc")
	CTraceMsg::RegisterThread ();
  TVPManualEvent::SetGuard GCGuard(m_JSGCTerminateEvent);

  try {
    RVThreadSelf self = ::rvThread();
    self.setPriority(self.getMaxPriority());
    //self.setPriority(self.getMaxPriority()-1);
    //TVPSysThread::setPriority(TVPSysThread::High);
    mGCThread.fExecute();
  } catch(...) {
    WlTraceMsg("Fatal Error in CLoadThreadsManagerScript::JSGCFunc");
    CAssociativeArray	ErrorMessage;
    ErrorMessage.AddCouple("$GENERATORLOGMESSAGE","Fatal Error in CLoadThreadsManagerScript::JSGCFunc");
    ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
    SendMessage(ErrorMessage, 2);
  }

  WlTraceMsg("CLoadThreadsManagerScript::JSGCFunc - Done");
}

/////////////////////////////////////////////////////////////////////////////////////////
//
//////////////////////// CLoadGeneratorSlaveCommunicator Implementation /////////////////
//
/////////////////////////////////////////////////////////////////////////////////////////

CLoadGeneratorSlaveCommunicator::CLoadGeneratorSlaveCommunicator(const char* name) :
  CNetMessageHandler("LGSlaveCommunicator")
{
  m_SenderName.append(name);
}

CLoadGeneratorSlaveCommunicator::~CLoadGeneratorSlaveCommunicator()
{
}

void CLoadGeneratorSlaveCommunicator::InitParent(CLoadGeneratorSlave *pLG) {
	  m_pMyLoadGeneratorSlave = pLG;
}

void CLoadGeneratorSlaveCommunicator::MessageHandler( CAssociativeArray& AssoArrayCommand )
{
	try
	{
    if ( AssoArrayCommand.KeyExists( "$PING" ) )
      {
        CAssociativeArray PingReply;
        PingReply.AddCouple("$PINGREPLY", "1");
				m_pMyLoadGeneratorSlave->SendMessage(PingReply, 1);
				return;
			}
    else if ( AssoArrayCommand.KeyExists( "$INPUTFILE" ) )
      {
        m_pMyLoadGeneratorSlave->GetThreadsManager()->InputFileData(&AssoArrayCommand);
        return;
      }
	//
    // put in my Load Generator's queue.
    CAssociativeArray* pAssociativeArray = new CAssociativeArray;
    pAssociativeArray->FromString(AssoArrayCommand.ToString());
    m_pMyLoadGeneratorSlave->PutMessage(pAssociativeArray);

  } catch (WLErrorException e) {
//    WlTraceMsg("CNetKernelProxy::MainLoop : Exception: %s",e.GetErrorMsg());
		WlAssert(0);
		} catch (...) {
    WlTraceMsg("CNetKernelProxy::MainLoop : Unknown exception");
    printf("CNetKernelProxy::MainLoop : Unknown exception");
    #ifdef _DEBUG
    throw;
    #endif
  }
}

void CLoadGeneratorSlaveCommunicator::SendMessageErrorHandler( CAssociativeArray& Message , const char* SendTo, const char* ErrorMsg, bool status)
{

	WlTraceMsg("CLoadGeneratorSlaveCommunicator::SendMessageErrorHandler invoked");

	if (CLEIniFileProperties::EnableStopgeneratorOnNetErrors) {
		m_pMyLoadGeneratorSlave->StopMe();
	}
}

