/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
//#pragma warning( disable : 4290 )

#include <ordered.h>
#include <RVCString.h>

#include "platform.h"
#include "tvenv.h"
#include "tracedll.h"
#include "associativearray.h"
#include "loadengineverificationclasses.h"
#include "loadagendamanager.h"
#include "loadthreadsmanager.h"
#include "loadgenerator.h"
#include "loadgeneratorslave.h"
#include "loadgeneratorslaveproxy.h"
#include "loadenginedecs.h"
#include "socketlib.h"
#include "loadengineglobals.h"

#include "generalprocess.h"
#include "loadengineinifileproperties.h"

#include "nvsnames.h"

//////////////////////// CLoadGeneratorSlaveProxy Implementation /////////////////

CLoadGeneratorSlaveProxy::CLoadGeneratorSlaveProxy(CLoadAgendaManager *pLAM, int slaveId, bool inProcess) :
  m_pMyLoadAgendaManager(pLAM),
  m_SlaveId(slaveId),
  m_InProcess(inProcess),
	m_TimeoutOccurred(false),
  CNetMessageHandler("LGSlaveProxy")
{
  m_pMyLoadGeneratorSlave = NULL;
  m_NumRunningRequested = 0;
  m_NumRunning = 0;
  m_CreateRequestId = -1;
  m_SlaveOK = true;
  m_SlaveStartTime = -1;
  m_Terminated = false;
  m_StopSlaveCalled = false;
  m_SlaveAddress = "";

  char temp[20];
  sprintf(temp, "#%s#%d", m_pMyLoadAgendaManager->m_pMyLoadGenerator->GetLoadGeneratorNo(),m_SlaveId);
  m_SlaveName = temp;
  m_SenderName.append(m_SlaveName);
	m_StatisticsFrequency = -1;
}

CLoadGeneratorSlaveProxy::~CLoadGeneratorSlaveProxy()
{
  CGeneralProcess* aSaveProcessPtr;
  if (m_InProcess) aSaveProcessPtr = &m_pMyLoadGeneratorSlave->fGetProcess();


  if (m_pMyLoadGeneratorSlave) {
    delete m_pMyLoadGeneratorSlave;
    m_pMyLoadGeneratorSlave = NULL;
  }
  else {
    Terminate();
  }

  if (m_InProcess) delete aSaveProcessPtr; // Chaim - for main process, delete here (slave processes are deleted in main)
}

bool CLoadGeneratorSlaveProxy::CreateSlave(long requestId, int slaveBaseThreadId, int numRunning)
{
  m_CreateRequestId = requestId;
  m_SlaveBaseThreadId = slaveBaseThreadId;
  m_NumRunningRequested = numRunning;
  if (m_InProcess) {
	//m_SlaveId
		CGeneralProcess* generalProcess = new CGeneralProcess();
    m_pMyLoadGeneratorSlave = new CLoadGeneratorSlave(this, m_SlaveName, *generalProcess);
		m_pMyLoadGeneratorSlave->InitMonitorAddress(m_pMyLoadAgendaManager->m_pMyLoadGenerator->m_MonitorName.data());
    m_pMyLoadGeneratorSlave->Start(); // Activate the MainLoop thread (After the
                                      // new has finished)
  } else {
    //
    // Initialize Communicator.
    Init();
    //
    // Create a process with the Slave.
    m_SlaveOK = CreateLoadGenerator();
  }

  return m_SlaveOK;
}

void CLoadGeneratorSlaveProxy::StartSession(long requestId, CAssociativeArray* paramsArray)
{
  if (!m_SlaveOK) {
    m_pMyLoadAgendaManager->m_CommandsManager.SlaveDone(requestId, false);
    return;
  }
  paramsArray->AddCouple("$REQUESTID", requestId);
  paramsArray->AddCouple("$SLAVEBASEID", (long)m_SlaveBaseThreadId);
  SendMessageToSlave(paramsArray);
}

bool CLoadGeneratorSlaveProxy::SetSlaveNumRunning(long requestId, int numRunning)
{
  if (!m_SlaveOK) {
    m_pMyLoadAgendaManager->m_CommandsManager.SlaveDone(requestId, false);
    return false;
  }
  m_NumRunningRequested = numRunning;
  CAssociativeArray *Message = new CAssociativeArray;
  Message->AddCouple("$REQUESTID", requestId);
  Message->AddCouple("$SETNUMRUNNING", (long)numRunning);
  SendMessageToSlave(Message);
  return true;
}

void CLoadGeneratorSlaveProxy::StopSlave(long requestId)
{
  if (!m_SlaveOK || m_Terminated) {
    m_pMyLoadAgendaManager->m_CommandsManager.SlaveDone(requestId, true);
    return;
  }

  m_TerminateRequestId = requestId;
  m_StopSlaveCalled = true;
  CAssociativeArray *Message = new CAssociativeArray;
  Message->AddCouple("$TERMINATE", 1L);
  Message->AddCouple("$REQUESTID", requestId);
  SendMessageToSlave(Message);
}

void CLoadGeneratorSlaveProxy::SendMessageToSlave( CAssociativeArray* Message , int priority)
{
  if (m_InProcess) {
     if ( Message->KeyExists( "$INPUTFILE" ) ) {
        m_pMyLoadGeneratorSlave->GetThreadsManager()->InputFileData(Message);
        delete Message;
        return;
     }
    m_pMyLoadGeneratorSlave->PutMessage(Message);
  } else {
    if (m_SlaveAddress.length() > 0)
      SendMessage(*Message, m_SlaveAddress, "", 1);
    delete Message;
  }
}

CNetHandlerMessage* CLoadGeneratorSlaveProxy::SenderWaitMessage()
{
	RVCollectable* pMessage;
	TVPPriorityQueue::WaitReturnType ret;

	while (1) {
		ret = waitEvent(pMessage, (m_TimeoutOccurred ? CLEIniFileProperties::LoadGeneratorSlaveTimeOut * 2 : CLEIniFileProperties::LoadGeneratorSlaveTimeOut));
		switch (ret)
		{
			case TVPPriorityQueue::Success :
				return (CNetHandlerMessage*)pMessage;
			case TVPPriorityQueue::TimedOut :
			{
				if (m_TimeoutOccurred) {
					CAssociativeArray ErrorMessage;
					ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
					ErrorMessage.AddCouple("$AGENDALOGMESSAGE", "Slave is not responding.");
					ErrorMessage.AddCouple("$AGENDANAME", m_pMyLoadAgendaManager->m_Name.data());
					ErrorMessage.AddCouple("$TIME", m_pMyLoadAgendaManager->m_pMyLoadGenerator->GetTimeFromBeginning());
					m_Terminated = true;
					m_SlaveOK = false;
					m_pMyLoadAgendaManager->reportLogMessageString(ErrorMessage);
					m_pMyLoadAgendaManager->SlaveFatalError();
				}
				else {
					m_TimeoutOccurred = true;
					CAssociativeArray* PingMessage = new CAssociativeArray();
					PingMessage->AddCouple("$PING",1L);
					SendMessageToSlave(PingMessage);
				}
				break;
			}
			default:
				return NULL;
		}
	}
}

void CLoadGeneratorSlaveProxy::MessageHandler( CAssociativeArray& AssoArrayCommand )
{
  WlTraceMsg("MessageHandler - CLoadGeneratorSlaveProxy - MessageHandler");
	m_TimeoutOccurred = false;
	try
	{
		if ( AssoArrayCommand.KeyExists( "$CREATESLAVESTATUS" ) )
			{
				const char* createSlaveStatus = AssoArrayCommand.GetValueOfParameter("$CREATESLAVESTATUS");
				if( strcmp( createSlaveStatus, "CREATEDSUCCESSFULLY") == 0 ) {
					m_StatusCreateSlave = true;
		      WlTraceMsg("The theTml succedded to create slave");
				} else {
					m_StatusCreateSlave = false;
		      //the theTml failed to create slave
		      CAssociativeArray ErrorMessage;
			    ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
		      #ifndef __TV_UNIX__
			      ErrorMessage.AddCouple("$AGENDALOGMESSAGE", (char*)GetLastError());
		      #else
			      ErrorMessage.AddCouple("$AGENDALOGMESSAGE", "??????");
		      #endif
			    ErrorMessage.AddCouple("$AGENDANAME", m_pMyLoadAgendaManager->m_Name.data());
			    ErrorMessage.AddCouple("$TIME", m_pMyLoadAgendaManager->m_pMyLoadGenerator->GetTimeFromBeginning());
			    m_Terminated = true;
		      m_SlaveOK = false;
			    m_pMyLoadAgendaManager->reportLogMessageString(ErrorMessage);
			    if (m_CreateRequestId >=0)
				    m_pMyLoadAgendaManager->m_CommandsManager.SlaveDone(m_CreateRequestId, m_SlaveOK);
			    else
				    m_pMyLoadAgendaManager->SlaveFatalError();
				}
				//m_CreateSlaveDoneEvent.Set();
			}

		else if ( AssoArrayCommand.KeyExists( "$SLAVEREADY" ) )
      {
    //
        // Get the slave address.
        m_SlaveAddress = AssoArrayCommand.GetValueOfParameter("$SLAVEADDRESS","");
        //WlTraceMsg("Slave %d, Received SLAVEREADY Mesasge : %s ", m_SlaveId, AssoArrayCommand.ToString());

				CAssociativeArray *Message = new CAssociativeArray;
				const char *globalsAddress = m_pMyLoadAgendaManager->m_pMyLoadGenerator->m_MonitorGlobalVariablesAddress.data();
				Message->AddCouple("$MONITORGLOBALSNAME", globalsAddress);
				Message->AddCouple("$MONITORNAME", m_pMyLoadAgendaManager->m_pMyLoadGenerator->m_MonitorName.data());
				Message->AddCouple("$LGMASTERPID", (long)(GetAgendaManager()->GetLoadGenerator()->GetMasterPid()));
				Message->AddCouple("$MONITORUID", GetAgendaManager()->GetLoadGenerator()->GetMonitorUID().data());
				/*GALTECH*/
#ifndef _ANSI_ONLY
				Message->AddCouple("$MONITORCHARSET", (long)GetMonitorCharSet());
#endif
        Message->AddCouple("$THREADPOOLON", (long) GetAgendaManager()->GetLoadGenerator()->m_ThreadPoolOn);
				SendMessageToSlave(Message);

        if (m_StopSlaveCalled) {
          Message = new CAssociativeArray;
          Message->AddCouple("$TERMINATE", 1L);
          Message->AddCouple("$REQUESTID", m_TerminateRequestId);
          SendMessageToSlave(Message);
          return;
        }
        //
        // Send the Initialization message to the Slave
        Message = new CAssociativeArray;
        Message->AddCouple("$REQUESTID", m_CreateRequestId);
        Message->AddCouple("$LOADGENERATORNO", m_pMyLoadAgendaManager->m_pMyLoadGenerator->GetLoadGeneratorNo());
        if (m_InProcess && m_NumRunningRequested == 0) {
          Message->FromString(m_pMyLoadAgendaManager->m_LoadAgendaMessageString);
          //WlTraceMsg("Slave %d, LOADTHREADSMANAGER Mesasge : %s ", m_SlaveId, Message->ToString());
        } else if (m_NumRunningRequested == 0){
          Message->AddCouple("$CREATE", 1L);
          Message->AddCouple("$SLAVEBASEID", (long)m_SlaveBaseThreadId);
          Message->AddCouple("$LOADAGENDAMESSAGE", m_pMyLoadAgendaManager->m_LoadAgendaMessageString);
          //WlTraceMsg("Slave %d, CREATE&START Mesasge : %s ", m_SlaveId, Message->ToString());
        } else {
          Message->AddCouple("$CREATE&START", 1L);
          Message->AddCouple("$SLAVEBASEID", (long)m_SlaveBaseThreadId);
          Message->AddCouple("$LOADAGENDAMESSAGE", m_pMyLoadAgendaManager->m_LoadAgendaMessageString);
          Message->AddCouple("$STARTSESSIONMESSAGE", m_pMyLoadAgendaManager->m_StartSessionCommand);
          CAssociativeArray NumRunningMessage;
          NumRunningMessage.AddCouple("$SETNUMRUNNING", (long)m_NumRunningRequested);
          Message->AddCouple("$SETNUMRUNNINGMESSAGE", NumRunningMessage.ToString());
          //WlTraceMsg("Slave %d, CREATE&START Mesasge : %s ", m_SlaveId, Message->ToString());
        }
        SendMessageToSlave(Message);
      }

    else if ( AssoArrayCommand.KeyExists( "$LOADAGENDADONE" ) )
      {
        //WlTraceMsg("Slave %d, Received LOADAGENDADONE Mesasge : %s ", m_SlaveId, AssoArrayCommand.ToString());
        m_SlaveOK = (AssoArrayCommand.GetLongValueOfParameter("$RETVALUE")==1);
        long requestId = AssoArrayCommand.GetLongValueOfParameter("$REQUESTID");
        if (!m_SlaveOK || m_NumRunningRequested == 0) {
          WlTraceMsg("Slave %d, LOADAGENDADONE : SlaveDone %d",m_SlaveId, requestId);
          m_pMyLoadAgendaManager->m_CommandsManager.SlaveDone(requestId, m_SlaveOK);
        } else if (m_InProcess) {
          CAssociativeArray *Message = new CAssociativeArray;
          Message->FromString(m_pMyLoadAgendaManager->m_StartSessionCommand);
          StartSession(requestId, Message);
        }
      }
    else if ( AssoArrayCommand.KeyExists( "$STARTSESSIONDONE" ) )
      {
        //WlTraceMsg("Slave %d, STARTSESSIONDONE Message : %s",m_SlaveId, AssoArrayCommand.ToString());
				m_StatisticsFrequency = (int)AssoArrayCommand.GetLongValueOfParameter("$STATISTICSFREQUENCY");
        m_SlaveOK = (AssoArrayCommand.GetLongValueOfParameter("$RETVALUE")==1);
        long requestId = AssoArrayCommand.GetLongValueOfParameter("$REQUESTID");
        m_SlaveStartTime = m_pMyLoadAgendaManager->m_pMyLoadGenerator->GetTimeFromBeginning();
        if (!m_SlaveOK || m_NumRunningRequested == 0) {
          m_pMyLoadAgendaManager->m_CommandsManager.SlaveDone(requestId, m_SlaveOK);
        } else if (m_InProcess) {
          SetSlaveNumRunning(requestId, m_NumRunningRequested);
        }
        m_pMyLoadAgendaManager->m_SlaveCountManager.AddSlave(m_SlaveStartTime);
      }
    else if ( AssoArrayCommand.KeyExists( "$SETNUMRUNNINGDONE" ) )
       {
        WlTraceMsg("Slave %d, SETNUMRUNNINGDONE Message : %s",m_SlaveId, AssoArrayCommand.ToString());
        m_SlaveOK = (AssoArrayCommand.GetLongValueOfParameter("$RETVALUE")==1);
        long requestId = AssoArrayCommand.GetLongValueOfParameter("$REQUESTID");
        if (m_SlaveOK)
          m_NumRunning = m_NumRunningRequested;
        m_pMyLoadAgendaManager->m_CommandsManager.SlaveDone(requestId, m_SlaveOK);
      }
    else if ( AssoArrayCommand.KeyExists( "$STOPSLAVEDONE" ) )
      {
        WlTraceMsg("Slave %d, STOPSLAVEDONE Message : %s",m_SlaveId, AssoArrayCommand.ToString());
        long requestId = AssoArrayCommand.GetLongValueOfParameter("$REQUESTID");
        m_Terminated = true;
        if (requestId>=0)
          m_pMyLoadAgendaManager->m_CommandsManager.SlaveDone(requestId, true);
        else if (m_StopSlaveCalled)
          m_pMyLoadAgendaManager->m_CommandsManager.SlaveDone(m_TerminateRequestId, true);
        else
          m_pMyLoadAgendaManager->SlaveFatalError();
      }
    else if ( AssoArrayCommand.KeyExists( "$STATISTICSMANAGERDATA" ) )
      {
				double sliceTime = AssoArrayCommand.GetDoubleValueOfParameter("$TIME");;
				//double timeTheStatisticWasSent = m_pMyLoadAgendaManager->GetLoadGenerator()->fGetBeginTime();
				double stopAllTime = m_pMyLoadAgendaManager->GetLoadGenerator()->GetStopAllTime();
				if( (sliceTime > stopAllTime + m_StatisticsFrequency ) && (stopAllTime != -1) ) {//the user press the stop all button.
					return;// Don't send statistics.
				}
				// get the time of the statistics transaction
				double time = AssoArrayCommand.GetDoubleValueOfParameter("$TIME");
				WlTraceMsg("sliceTime is %f, stopAllTime is %f, statistic frequenty is %f", sliceTime, stopAllTime, m_StatisticsFrequency);
        WlTraceMsg("Slave %d, Time: %f, STATISTICSMANAGERDATA Message : %s",m_SlaveId, time, AssoArrayCommand.ToString());
				if ( (m_SlaveStartTime >= 0) && (time >= m_SlaveStartTime) )
          m_pMyLoadAgendaManager->m_AgendaStatisticsManager.NewStatistics(AssoArrayCommand);
      }
    else if ( AssoArrayCommand.KeyExists( "$AGENDALOGMESSAGE" ) )
      {
        //WlTraceMsg("Slave %d, AGENDALOGMESSAGE Message : %s",m_SlaveId, AssoArrayCommand.ToString());
        if (!m_pMyLoadAgendaManager->reportLogMessageString(AssoArrayCommand))
        {
          //m_SlaveOK = false;
          m_pMyLoadAgendaManager->SlaveFatalError();
        }
      }
    else if ( AssoArrayCommand.KeyExists( "$GENERATORLOGMESSAGE" ) )
      {
        //WlTraceMsg("GENERATORLOGMESSAGE Message : %s",AssoArrayCommand.ToString());
        if (!m_pMyLoadAgendaManager->reportLoadGeneratorMessage(AssoArrayCommand))
        {
          //m_SlaveOK = false;
          m_pMyLoadAgendaManager->SlaveFatalError();
        }
      }
    else if ( AssoArrayCommand.KeyExists( "$TRANSACTIONTREE" ) )
      {
        if (m_pMyLoadAgendaManager->m_pMyLoadGenerator->IsSendTransactions()) {
          long NumTransactions = AssoArrayCommand.GetLongValueOfParameter("NumTransactions");
          m_pMyLoadAgendaManager->m_pMyLoadGenerator->AddTransactions(NumTransactions);

          if (m_pMyLoadAgendaManager->m_pMyLoadGenerator->m_GeneratorType == CLoadGenerator::StandAlone)
            ;//PrintTransactionTree(AssoArrayCommand.ToString());
          else
            m_pMyLoadAgendaManager->m_pMyLoadGenerator->SendMessage(AssoArrayCommand, 4);
        }
      }
    else if ( AssoArrayCommand.KeyExists( "$STANDALONEEVENT" ) )
      {
        if (m_pMyLoadAgendaManager->m_pMyLoadGenerator->IsSendTransactions()) {
          m_pMyLoadAgendaManager->m_pMyLoadGenerator->AddTransactions(1);

          if (m_pMyLoadAgendaManager->m_pMyLoadGenerator->m_GeneratorType == CLoadGenerator::StandAlone)
            cout << AssoArrayCommand.ToString() << endl;
          else
            m_pMyLoadAgendaManager->m_pMyLoadGenerator->SendMessage(AssoArrayCommand, 4);
        }
      }
		else if(AssoArrayCommand.KeyExists("$GLOBALSNAME"))
			{
				m_pMyLoadAgendaManager->GetGlobalsHandler()->MessageHandler(AssoArrayCommand, this);
			}
    else if ( AssoArrayCommand.KeyExists( "$COPYMONITORFILE" ) )
      {
        CAssociativeArray *Message = new CAssociativeArray;
        RVCString errormessage;
        bool st = m_pMyLoadAgendaManager->CopyMonitorFile(AssoArrayCommand.GetValueOfParameter("$SOURCEFILENAME"),
																													AssoArrayCommand.GetValueOfParameter("$TARGETFILENAME"),
																													AssoArrayCommand.GetValueOfParameter("$AGENDAPATH"),
																													errormessage);
        if (st) {
          Message->AddCouple("$COPYFILESTATUS", "OK");
        }
        else {
          Message->AddCouple("$COPYFILESTATUS", "ERROR");
          Message->AddCouple("$ERRORMESSAGE", errormessage.data());
        }
        SendMessageToSlave(Message);
      }
    else if ( AssoArrayCommand.KeyExists( "$INCLUDEMONITORFILE" ) )
      {
        CAssociativeArray *Message = new CAssociativeArray;
        RVCString errormessage;
        const char* data;
        bool st = m_pMyLoadAgendaManager->IncludeMonitorFile(AssoArrayCommand.GetValueOfParameter("$SOURCEFILENAME"),
																													   &data,
																														 AssoArrayCommand.GetValueOfParameter("$AGENDAPATH"),
																														 errormessage);
        if (st) {
          Message->AddCouple("$INCLUDEFILESTATUS", "OK");
          Message->AddCouple("$DATA", data);
        }
        else {
          Message->AddCouple("$INCLUDEFILESTATUS", "ERROR");
          Message->AddCouple("$ERRORMESSAGE", errormessage.data());
        }
        SendMessageToSlave(Message);
      }
    else if (AssoArrayCommand.KeyExists( "$OUTPUTFILE" ))
      {
        CAssociativeArray *Message = new CAssociativeArray;
				Message->FromString(AssoArrayCommand.ToString());
        m_pMyLoadAgendaManager->OutputFilePutMessage(Message);
      }

    else if (AssoArrayCommand.KeyExists( "$INPUTFILE" ))
      {
        CAssociativeArray* chunk = m_pMyLoadAgendaManager->InputFileActions(AssoArrayCommand);
       if (chunk != NULL)
          SendMessageToSlave(chunk,5);
      }

    else if ( AssoArrayCommand.KeyExists( "$PINGREPLY" ) )
      {
        //WlTraceMsg("PINGREPLY Message : %s",AssoArrayCommand.ToString());
				// Do nothing
			}
    else if ( AssoArrayCommand.KeyExists( "$PROTOCOLMESSAGE" ) )
      {
         m_pMyLoadAgendaManager->m_pMyLoadGenerator->SendMessage(AssoArrayCommand, 4);
      }
    else if ( AssoArrayCommand.KeyExists( "$" ) )
      {
      }
    else
      {
        //WlTraceMsg("Illegal command (%s)",AssoArrayCommand.ToString());
      }

  } catch (WLErrorException e) {
    //WlTraceMsg("CNetKernelProxy::MainLoop : Exception: %s",e.GetErrorMsg());
		WlAssert(0);
  } catch (...) {
    //WlTraceMsg("CNetKernelProxy::MainLoop : Unknown exception");
    #ifdef _DEBUG
    throw;
    #endif
  }
}

void CLoadGeneratorSlaveProxy::SendMessageErrorHandler( CAssociativeArray& Message , const char* SendTo, const char* ErrorMsg, bool status)
{
  if (!m_SlaveOK || m_Terminated)
		return;
  CAssociativeArray ErrorMessage;
  ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
  ErrorMessage.AddCouple("$AGENDALOGMESSAGE", "Error while sending message to slave.");
  ErrorMessage.AddCouple("$AGENDANAME", m_pMyLoadAgendaManager->m_Name.data());
  ErrorMessage.AddCouple("$TIME", m_pMyLoadAgendaManager->m_pMyLoadGenerator->GetTimeFromBeginning());
  m_Terminated = true;
  m_SlaveOK = false;
  WlTraceMsg("CLoadGeneratorSlaveProxy::SendMessageErrorHandler create message : %s",ErrorMessage.ToString());
	m_pMyLoadAgendaManager->reportLogMessageString(ErrorMessage);
  //
  // the request id is used here when sending message with Request ID.   The only relevant
  //  situation is STOP.
  long requestId = Message.GetLongValueOfParameter("$REQUESTID",-1);
  WlTraceMsg("CNetKernelProxy::SendMessageErrorHandler : SlaveDone");
  if (requestId >=0)
    m_pMyLoadAgendaManager->m_CommandsManager.SlaveDone(requestId, m_SlaveOK);
  else
    m_pMyLoadAgendaManager->SlaveFatalError();

}

bool CLoadGeneratorSlaveProxy::CreateLoadGenerator()
{
	CAssociativeArray Msg;

	char localHostName[100] = "localhost";

    CSocketLib::GetLocalHostName(localHostName, 100);
	CTestTalkAddress sendTo("theTml", localHostName);
	Msg.AddCouple("$CREATELOADGENERATORSLAVE", 1L);
	Msg.AddCouple("$MASTERADDRESS", WhoAmI());
	Msg.AddCouple("$SLAVENAME",m_SlaveName.data());
	Msg.AddCouple("$MASTERTTGROUPUID",GetAgendaManager()->GetLoadGenerator()->GetMonitorUID() );
 	try{
		WlTraceMsg("Send the message : %s. to the theTml",Msg.ToString());
		SendMessage(Msg, sendTo.Address(), "");
	}
	catch(...)
	{
		TraceSendText("Enhandled exeption in SendMessage($CREATELOADGENERATORSLAVE)");
	}

  return true;
}

bool CLoadGeneratorSlaveProxy::StopSendingLogMessages()
{
  if (!m_SlaveOK)
    return false;
  CAssociativeArray *Message = new CAssociativeArray;
  Message->AddCouple("$STOPSENDINGLOGMESSAGES", "1");
  SendMessageToSlave(Message);
  return true;
}

bool CLoadGeneratorSlaveProxy::StopSendingTransactions()
{
  if (!m_SlaveOK)
    return false;
  CAssociativeArray *Message = new CAssociativeArray;
  Message->AddCouple("$STOPSENDINGTRANSACTIONS", "1");
  SendMessageToSlave(Message);
  return true;
}

