/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
//#include <rw/strstrea.h>
#include <RVCTokenizer.h>

#include "tvdebug.h"
#include "tracedll.h"
#include "loadgeneratorscript.h"
#include "associativearray.h"
#include "loadgenerator.h"
#include "loadthreadsmanager.h"
#include "loadinputfile.h"
#include "loadagendamanager.h"
#include "loadexceptions.h"
#include "loadengineglobals.h"
#include <time.h>
#include "loadjsdefs.h"
#include "loadengineinifileproperties.h"
//Galtech
#include "gt_unicode.h"

#include <slistcolque.h>
#include "tvpsync.h"

#ifdef __TV_UNIX__
#define MIN min
#ifdef __TV_linux__
#define max(v1, v2) ((v1) > (v2) ? (v1) : (v2))
#define min(v1, v2) ((v1) < (v2) ? (v1) : (v2))
#endif
#else
#define MIN __min
#endif


CWLInputLine::CWLInputLine(const char* line, long lineNumber, long fileRound) :
  RVCollectableString(line), m_LineNumber(lineNumber), m_FileRound(fileRound)
{
}

CWLInputLine::~CWLInputLine()
{
}

const char* CWLInputLine::GetLine()
{
  return data();
}

long CWLInputLine::GetLineNumber()
{
  return m_LineNumber;
}

long CWLInputLine::GetFileRound()
{
  return m_FileRound;
}



class DllExport TVPEventQueue1
{
private:
  bool              m_Active;
  bool              m_Released;
	RVMutexLock       *m_Touch;
  RVCondition       *m_Condition;
  RVSlistCollectablesQueue *m_Q;

public:
  typedef enum {
    Success,
    TimedOut,
    Released,
    Failed
  } WaitReturnType;

public:
	TVPEventQueue1();
	virtual ~TVPEventQueue1();

  void Init();
  void Terminate();

	RVCollectable* waitEvent();
	WaitReturnType waitEvent(RVCollectable*& event, long waitTime);
  void putEvents(CAssociativeArray* AssoArray);

	RVCollectable* get();
	bool isEmpty() const;
	int entries() {return m_Q->entries();};
};
TVPEventQueue1::TVPEventQueue1()
{
  m_Q = new RVSlistCollectablesQueue;
  m_Touch = new RVMutexLock;
  m_Condition  = new RVCondition(*m_Touch);
  Init();
}

TVPEventQueue1::~TVPEventQueue1()
{
  Terminate();
  delete m_Q;
  delete m_Touch;
  delete m_Condition;
}

void TVPEventQueue1::Init()
{
  RVMutexLock::LockGuard lock(*m_Touch);
  m_Q->clearAndDestroy();
  m_Released = false;
  m_Active = true;
}

void TVPEventQueue1::Terminate()
{
  RVMutexLock::LockGuard lock(*m_Touch);
  m_Active = false;
  m_Released = true;
  m_Q->clearAndDestroy();
	m_Condition->signalAll();
}

void TVPEventQueue1::putEvents(CAssociativeArray* AssoArray) {
  RVMutexLock::LockGuard lock(*m_Touch);
  if (!m_Active || m_Released)
		return;
  int num = AssoArray->GetLongValueOfParameter("$LINESAMOUNT");
  for (int i=0; i < num; i++) {
    CWLInputLine* line = new CWLInputLine(AssoArray->GetValueOfParameterArray("$LINE",i),
                                          AssoArray->GetLongValueOfParameterArray("$LINENUM",i),
                                          AssoArray->GetLongValueOfParameterArray("$FILEROUND",i));
	  m_Q->append((RVCollectable *)line);
  }
	m_Condition->signal();
}

RVCollectable* TVPEventQueue1::waitEvent()
{
  RVMutexLock::LockGuard lock(*m_Touch);

	//
	// wait for the message
  while (m_Active && !m_Released && m_Q->isEmpty()) {
  	m_Condition->wait();
  }

  if (!m_Active || (m_Released && m_Q->isEmpty()))
		return NULL;

	RVCollectable* e = (RVCollectable *)m_Q->get();
	return e;
}

TVPEventQueue1::WaitReturnType TVPEventQueue1::waitEvent(RVCollectable*& event, long waitTime)
{
  RVMutexLock::LockGuard lock(*m_Touch);
  event = NULL;

	//
	// wait for the message
  while (m_Active && !m_Released && m_Q->isEmpty()) {
  	RVWaitStatus status = m_Condition->wait(waitTime);
    if (status == RV_THR_TIMEOUT)
      return TimedOut;
  }

  if (!m_Active || (m_Released && m_Q->isEmpty())) {
    return Released;
  }

	event = (RVCollectable *)m_Q->get();
	return Success;
}

RVCollectable* TVPEventQueue1::get()
{
  RVMutexLock::LockGuard lock(*m_Touch);
  if (!m_Active)
		return NULL;

  return m_Q->get();
}

bool TVPEventQueue1::isEmpty() const
{
  RVMutexLock::LockGuard lock(*m_Touch);
  return m_Q->isEmpty();
}


CAssociativeArray* CWLInputFileHandler::InputFileActions(CAssociativeArray& AssoArray) {

  const char* ScriptName = AssoArray.GetValueOfParameter("$SCRIPTNAME");
	RVCollectableString key(ScriptName);
	CWLFileDictionary1* FileDict;
	{
		RVCriticalSection::ReadLockGuard lock(creationMutex);
		FileDict = (CWLFileDictionary1*)findValue(&key);
	}
	if (FileDict == NULL) {
		RVCriticalSection::WriteLockGuard lock(creationMutex);
		FileDict = new CWLFileDictionary1(this,ScriptName);
		insertKeyAndValue(new RVCollectableString(ScriptName),FileDict);
	}
	if (AssoArray.KeyExists("$OPEN")) {
		if( -1 == FileDict->AddFile(AssoArray.GetValueOfParameter("$FILENAME"),AssoArray.KeyExists("$RANDOM"))) {
			WLErrorException e(LoadExceptionFrmFile, 310, WLJSNAME_FILE_OPEN, AssoArray.GetValueOfParameter("$FILENAME"));
			const char* ErrorMessageStr =  strdup(e.GetErrorMsg());
			CAssociativeArray ErrorMessage;
			ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
			ErrorMessage.AddCouple("$AGENDALOGMESSAGE", ErrorMessageStr);
			ErrorMessage.AddCouple("$AGENDANAME", m_LoadAgendaManager->Name());
			ErrorMessage.AddCouple("$TIME", m_LoadAgendaManager->GetLoadGenerator()->GetTimeFromBeginning());
			m_LoadAgendaManager->reportLogMessageString(ErrorMessage);
			WlTraceMsg("call stopMe %s",ErrorMessage.ToString());
			m_LoadAgendaManager->GetLoadGenerator()->StopMe();
		}
	}
	else if (AssoArray.KeyExists("$GETLINES")) {
		CAssociativeArray* chunk = (CAssociativeArray*)FileDict->GetNextItem(AssoArray.GetValueOfParameter("$FILENAME"));
    if (chunk == NULL) {
			WLErrorException e(LoadExceptionFrmFile, 310, WLJSNAME_FILE_GET_LINE, AssoArray.GetValueOfParameter("$FILENAME"));
			const char* ErrorMessageStr =  strdup(e.GetErrorMsg());
			CAssociativeArray ErrorMessage;
			ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
			ErrorMessage.AddCouple("$AGENDALOGMESSAGE", ErrorMessageStr);
			ErrorMessage.AddCouple("$AGENDANAME", m_LoadAgendaManager->Name());
			ErrorMessage.AddCouple("$TIME", m_LoadAgendaManager->GetLoadGenerator()->GetTimeFromBeginning());
			m_LoadAgendaManager->reportLogMessageString(ErrorMessage);
			WlTraceMsg("call stopMe %s",ErrorMessage.ToString());
			m_LoadAgendaManager->GetLoadGenerator()->StopMe();
		}
    else
      return chunk;
	}
  return NULL;
}

void CWLInputFileHandler::Terminate() {
  RVCriticalSection::LockGuard lock(creationMutex);
  RVHashDictionaryIterator i(*this);
  while (i()) {
    CWLFileDictionary1* FileDict = (CWLFileDictionary1*)i.value();
		FileDict->Terminate();
	}
  clearAndDestroy();
}

#ifdef __TV_WINDOWS__
unsigned GlobalFunction(void* sq)
#else
unsigned GlobalFunction(void *sq)
#endif
{
  ((CWLStoreQueue *) sq)->QueueMainLoop();
//  delete (CWLStoreQueue *) sq;
  return 0;
}

CWLStoreQueue::CWLStoreQueue(const char* filename, bool random,CWLFileDictionary1* filedict, long MaxThreadsPerSlave)
{
  m_FileDict = filedict;
  m_Random = random;
  valid = 1;
  m_Terminate = false;
  file_name= filename;
  linenum = 0;
  fileround = 1;
  //delimiters.append();
  if ((input = fopen(filename, "r")) == NULL) {
    valid = 0;

    // QueueStart will no be invoked if we got to this lines so
    // Set the event and make m_TerminateEvent.Wait to be released
    // imidyatly inside the Terminate method.
    m_TerminateEvent.Set();

    return;
  }
		
  fseek( input, 0L, SEEK_END);       // seek to the end of the file
  if( ftell( input ) == 0 ) {        // taking its length.
	//empty file
		fclose(input);
	   valid = 0;

	  // QueueStart will no be invoked if we got to this lines so
		// Set the event and make m_TerminateEvent.Wait to be released
		// imidyatly inside the Terminate method.
		m_TerminateEvent.Set();

	   return;		
	}

	fseek( input, 0L, SEEK_SET);


  // For random
  if (m_Random) {
    srand( (unsigned)time( NULL ) );   // set the random
    fseek( input, 0L, SEEK_END);       // seek to the end of the file
    m_lFileSize = ftell( input );        // taking its length.
    if (m_lFileSize <= 0) m_lFileSize = 1; // don't want it to be zero.
  }
  m_TerminateEvent.Reset();

  m_LinesPerChunk = MIN(MaxThreadsPerSlave,CLEIniFileProperties::MaxItemsPerChunk);

  QueueStart();
}

CWLStoreQueue::~CWLStoreQueue()
{
	if (input) {
		fclose(input);
		ClearQueue();
	}
}

void CWLStoreQueue::Terminate()
{
  m_Terminate = true;
  event.Set();
  m_TerminateEvent.Wait();
}

int CWLStoreQueue::FillTheQueue()
{
  if (!input) {
    //sergei 3/24/01 - debugPrint(0,("Error while opening the file\n"));
    return (-1);
  }

  int        EndOfLine = 0;
	char       line[102400]; // 100K, old value was 2K
	//Galtech
#ifndef _ANSI_ONLY
	char       utfline[102400]; // 100K, old value was 2K
#endif
  int        num;
  RVCString  buffer;
  //Random vars
  long       lRandPos,temp;

	num = entries();
  for (int i = num; i < m_FileDict->ChunksInQueue(); i++) {
		CAssociativeArray* chunk = new CAssociativeArray();
    WlTraceMsg("created");
    chunk->AddCouple("$INPUTFILE","1");
    chunk->AddCouple("$FILENAME",file_name);
    chunk->AddCouple("$SCRIPTNAME",m_FileDict->ScriptName());
    chunk->AddCouple( "$LINESAMOUNT", (long)m_LinesPerChunk );
		for (int j = 0; j < m_LinesPerChunk; j++) {
			if (m_Random) {
				// there is double read. First seek to the rand record.
				// And, read this record to the end. We'll use the next record
				// whether it is in next in the file or the first one.
#ifdef __TV_linux__
				lRandPos = (lRandPos | rand()) % m_lFileSize;
#else
				temp = rand();
				lRandPos = temp << 16;
				lRandPos = (lRandPos | rand()) % m_lFileSize;
#endif // __TV_linux__
				fseek( input, lRandPos, SEEK_SET);
				fgets(line, sizeof(line), input);
			}

			while (!EndOfLine) {

				if (!fgets(line, sizeof(line), input)){
					fseek(input, 0L, 0);
					linenum = 0;
					fileround ++;
					continue;
				}
				//Galtech
				char *pLine = line;
#ifndef _ANSI_ONLY
				ANSIToUTF8(GetMonitorCharSet(),	line, -1, utfline, sizeof(utfline));
				pLine = utfline;
#endif
				int lineSize = strlen(pLine);
/*
				if (line[lineSize-2] == '\\') {
					buffer.append(line, lineSize - 2);
					buffer.append("\n");
					continue;
				}
*/
				linenum ++;
				if (pLine[lineSize-1] == '\n')
					buffer.append(pLine, lineSize - 1);
				else
					buffer.append(pLine, lineSize );
				EndOfLine = 1;
				//
				// insert the line into the chunk
				chunk->AddArrayCouple( "$LINE", buffer, j );
        if (m_Random) {
				  chunk->AddArrayCouple( "$LINENUM", -1L, j );
				  chunk->AddArrayCouple( "$FILEROUND", -1L, j );
        }
        else {
				  chunk->AddArrayCouple( "$LINENUM", (long)linenum, j );
				  chunk->AddArrayCouple( "$FILEROUND", (long)fileround, j );
        }
				buffer.remove(0);
			}
			EndOfLine = 0;
		}
		RVCriticalSection::LockGuard lock(mutex);
    insert(chunk);
  }
  return (0);
}

int CWLStoreQueue::CreateThreadForReading()
{
  int ret = filler_thr.create(GlobalFunction, (void *) this);
  if (ret != 0) {
    //sergei 3/24/01 - debugPrint(0,("thr_create system call Failed!!!\n"));
    return (-1);
  }
  else
    return (0);
}

void CWLStoreQueue::QueueMainLoop()
{
	
	CTraceMsg::RegisterThread ();

  //#ifdef __TV_WINDOWS__
  RVThreadSelf self = ::rvThread();
  self.setPriority(self.getMaxPriority());

  WlTraceMsg("CWLStoreQueue::QueueMainLoop - starting");
  //TVPSysThread::setPriority(TVPSysThread::High);
  //#endif
  TVPManualEvent::SetGuard GCGuard(m_TerminateEvent);
  while (1) {
    event.Wait();
    event.Reset();
    if (m_Terminate)
      return;
    int ret = FillTheQueue();
    if (ret == -1)
      return;
  }

  WlTraceMsg("CWLStoreQueue::QueueMainLoop - done");
}

CAssociativeArray* CWLStoreQueue::GetNextItem()
{
  RVCriticalSection::LockGuard lock(mutex);
  CAssociativeArray* chunk = (CAssociativeArray *)get();

  if ((int)entries() <= m_FileDict->ChunksInQueue()) {
    event.Set();
  }

  return chunk;
}

int CWLStoreQueue::QueueStart()
{
  int ret = FillTheQueue();
  if (ret == -1) {
    return (-1);
  }

  ret = CreateThreadForReading();
  if (ret == -1) {
    return (-1);
  }
  return (0);
}

void CWLStoreQueue::ClearQueue()
{
  RVCriticalSection::LockGuard lock(mutex);
  clearAndDestroy();
}



CFileDictKey::CFileDictKey(const char *str): RVCollectableString(str) {}

CFileDictKey::CFileDictKey(const RVCString& str): RVCollectableString(str) {}

bool CFileDictKey::isEqual(const RVCollectable* a) const {
  return (RVCString::compareTo(((RVCollectableString*)a)->data(), RVCString::ignoreCase) == 0);
}


CWLFileDictionary1::CWLFileDictionary1(CWLInputFileHandler* fh,const char* scriptname) :
  m_FileHandler(fh)
{
  m_ScriptName = strdup(scriptname);
}

CWLFileDictionary1::~CWLFileDictionary1()
{
}

const char* CWLFileDictionary1::ScriptName() {
  return m_ScriptName;
}

int CWLFileDictionary1::ChunksInQueue() {
  return m_FileHandler->ChunksInQueue();
}

void CWLFileDictionary1::Terminate() {
  RVHashDictionaryIterator hdi((RVHashDictionary &) *this);

  while (hdi())
    ((CWLStoreQueue *) hdi.value())->Terminate();
  clearAndDestroy();
}


int CWLFileDictionary1::isValid()
{
  RVHashDictionaryIterator hdi((RVHashDictionary &) *this);

  while (hdi())
    if (!((CWLStoreQueue *) hdi.value())->isValid())
      return 0;
  return 1;
}


int CWLFileDictionary1::AddFile(const char* filename, bool random)
{
  CFileDictKey key(filename);
  RVCriticalSection::LockGuard lock(CreationMutex);
  CWLStoreQueue* queue = (CWLStoreQueue *)findValue((RVCollectable *) &key);
  if (queue == NULL) {
    RVCollectableString* ret;
    CFileDictKey* target;
    queue  = new CWLStoreQueue(filename, random, this, m_FileHandler->GetLoadAgendaManager()->GetMaxThreadsPerSlave());
    target = new CFileDictKey(filename);
    ret = (RVCollectableString *)insertKeyAndValue((RVCollectable *) target,
                                                   (RVCollectable *) queue);
    if (!queue->isValid()) // don't move this line backward !!! queue must be
                           // in the hash
      return -1;

    if (ret == NULL)
      return (-1);
    else
      return (0);
  }
  else
    return (0);
}

CAssociativeArray* CWLFileDictionary1::GetNextItem(const char* filename)
{
  CFileDictKey key(filename);
  CWLStoreQueue* queue = (CWLStoreQueue *)findValue((RVCollectable *) &key);
  if (queue == NULL) {
    RVCriticalSection::LockGuard lock(CreationMutex);
    queue = (CWLStoreQueue *)findValue((RVCollectable *) &key);
    if (queue == NULL) {
      RVCollectableString* target, *ret;
      queue  = new CWLStoreQueue(filename, false, this, m_FileHandler->GetLoadAgendaManager()->GetMaxThreadsPerSlave());
      target = new CFileDictKey(filename);
      ret = (RVCollectableString *)insertKeyAndValue((RVCollectable *) target,
                                                     (RVCollectable *) queue);
      if (ret == NULL)
        return NULL;
    }
  }

  if (!queue->isValid())
    return NULL;

  CAssociativeArray* chunk;
  while (!(chunk = queue->GetNextItem()))
    ::rvSleep(200);
//  WlTraceMsg("New Line=(%s)", (const char*)nextline);
  return chunk;
}

class CFileQueue : public TVPEventQueue1 {
public :
	int m_ChunksAsked;
	int m_ChunkReceived;
	RVTRecursiveLock<RVMutexLock> m_GetMoreLines;

public:
	CFileQueue() : m_ChunksAsked(0),m_ChunkReceived(0) {};
	virtual ~CFileQueue() {};
};




CWLFileDictionary::CWLFileDictionary(
	IGeneralManagerScriptData* MSD ):
	m_ManagerScriptData(MSD),
	m_Terminate(false),
  m_MinItems(0)
{
  m_MinItems = MIN(GetMaxThreadsPerSlave(),CLEIniFileProperties::MaxItemsPerChunk);
};

CWLFileDictionary::~CWLFileDictionary() {};

bool CWLFileDictionary::AddFile(const char* filename, bool random)
{

  CFileDictKey key(filename);
  RVCriticalSection::LockGuard lock(CreationMutex);
	if (m_Terminate) return false;
  CFileQueue* queue = (CFileQueue *)findValue((RVCollectable *) &key);
  if (queue == NULL) {
    CFileDictKey* target;
    queue  = new CFileQueue();
    target = new CFileDictKey(filename);
    insertKeyAndValue((RVCollectable *) target,
                      (RVCollectable *) queue);
		// send message to the LoadagendaManager for opening the global file
		CAssociativeArray Message;
		Message.AddCouple("$INPUTFILE","1");
		Message.AddCouple("$SCRIPTNAME",m_ManagerScriptData->ScriptName());
		Message.AddCouple("$FILENAME",filename);
		Message.AddCouple("$OPEN","1");
		if (random)
			Message.AddCouple("$RANDOM","1");
		else
			Message.AddCouple("$SEQUENTIAL","1");
		m_ManagerScriptData->SendMessage(Message,4);
  }
	return true;
}

CWLInputLine* CWLFileDictionary::GetNextLine(const char* filename, iLEAgendaJSEmbedContext* embedContext)
{
	CFileQueue* queue;

	CFileDictKey key(filename);
	queue = (CFileQueue *)findValue((RVCollectable *) &key);

	if (queue == NULL) {
		RVCriticalSection::LockGuard lock(CreationMutex);
		if (m_Terminate)
			return NULL;
		queue = (CFileQueue *)findValue((RVCollectable *) &key);
		if (queue == NULL) {
			CFileDictKey* target;
			queue  = new CFileQueue();
			target = new CFileDictKey(filename);
			insertKeyAndValue((RVCollectable *) target,
												(RVCollectable *) queue);
		}
	}


	if(queue->entries() <= m_MinItems) {
		RVTRecursiveLock<RVMutexLock>::TryLockGuard lock(queue->m_GetMoreLines);
		if(lock.isAcquired()) {
			if (queue->m_ChunksAsked == queue->m_ChunkReceived) {
				CAssociativeArray Message;
				Message.AddCouple("$INPUTFILE","1");
				Message.AddCouple("$SCRIPTNAME",m_ManagerScriptData->ScriptName());
				Message.AddCouple("$FILENAME",filename);
				Message.AddCouple("$GETLINES","1");

				m_ManagerScriptData->SendMessage(Message,4);
				queue->m_ChunksAsked++;
				WlTraceMsg("ask");
      }
		}
  }


	RVCollectable* nextline = NULL;
  while (nextline == NULL) {
	  queue->waitEvent(nextline, 1000);
    if (embedContext && embedContext->isAbort()) return NULL;
  }


  return static_cast<CWLInputLine*>(nextline);

}

void CWLFileDictionary::FillTheQueue(CAssociativeArray* AssoArray) {
	const char* filename = AssoArray->GetValueOfParameter("$FILENAME");
  CFileDictKey key(filename);
  CFileQueue* queue = (CFileQueue *)findValue((RVCollectable *) &key);
  {
		RVTRecursiveLock<RVMutexLock>::LockGuard lock(queue->m_GetMoreLines);
	  queue->m_ChunkReceived++;
		WlTraceMsg("receive");
  }
  queue->putEvents(AssoArray);
}
/*
void CWLFileDictionary::Release() {
  RVCriticalSection::LockGuard lock(CreationMutex);
	m_Terminate = true;
  RVHashDictionaryIterator i(*this);
  while (i()) {
    CFileQueue* queue = (CFileQueue*)i.value();
		queue->release();
	}
}
*/
void CWLFileDictionary::Terminate() {

  RVCriticalSection::LockGuard lock(CreationMutex);
	m_Terminate = true;
  RVHashDictionaryIterator i(*this);
  while (i()) {
    CFileQueue* queue = (CFileQueue*)i.value();
		queue->Terminate();
	}

}

