/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#include "loadjsgcthread.h"


#include "wljsobjects.h"
#include "wljsruntime.h"
#include "loadjsshell.h"
#include "loadengineinifileproperties.h"
#include <thr.h>
#include "loadenginedecs.h"
#include "loadgeneratorscript.h"
#include "tracedll.h"
#include "wlleprotocolbasetypes.h"
#include "wlerrorlib.h"
#include "loadengineselistener.h"


cLEJSGCThread::cLEJSGCThread(
	CLoadGeneratorSlave& rLoadGeneratorSlave )
:
	iLEJSAgendaMachineThread( rLoadGeneratorSlave.fGetProcess() ),
	iLEJSEmbedContext(),
	mLoadGeneratorSlave(rLoadGeneratorSlave),
	mShouldExecute(false),
  mGCCx(0)
{}

cLEJSGCThread::~cLEJSGCThread()
{}


void cLEJSGCThread::fPreInit( CJSLoadRuntime& rLoadRuntime )
{
	WlAssert(!mGCCx);
  mGCCx = RVJS_NewContext(rLoadRuntime.fGet(), 1024);
	WlAssert(mGCCx);

	fSignContext( mGCCx );

//	CJSGCRequestBeginEnd Guard( mGCCx );

}

void cLEJSGCThread::fExecute()
{
	mShouldExecute = true;
	WlAssert(mGCCx); // didnt call JSGarbageCollectorInit() 

	JS_SetContextThread( mGCCx );

	iLEJSAgendaMachineThread::fInit(THREAD_TYPE_GC);

	try { LE_SET_SE_TRANSLATOR();

    // Initial sleep to wait for end of init agenda
		::rvSleep(CLEIniFileProperties::JSGCInitialSleep);

    int aGCCount=0;
    int aMaybeGCCount=0;
		int aTimeSlice = 0;
		while( mShouldExecute ) {
			if( aTimeSlice == CLEIniFileProperties::JSGCInterval) {
				aTimeSlice = 0;
				{

					{
						uint32 aNentries = RVJS_GetAtomTableNentries( mGCCx );
						char aBuf[100];
						sprintf( aBuf, "AtomTable nentries before GC: %d", (int) aNentries );

						//WlErrorMsg( aBuf );
						printf( "%s\n", aBuf );
					}

          WlTraceMsg("cLEJSGCThread::fExecute - calling JS_GC (%d)",++aGCCount);
					RVTRecursiveLock<RVMutexLock>::WriteLockGuard aGuard( mJSGCLock );
          CJSGCRequestBeginEnd Guard( mGCCx, UseOldStyle );
					JS_GC(mGCCx);
          WlTraceMsg("cLEJSGCThread::fExecute - calling JS_GC, after GC");

					{
						uint32 aNentries = RVJS_GetAtomTableNentries( mGCCx );
						char aBuf[100];
						sprintf( aBuf, "AtomTable nentries after GC: %d", (int) aNentries );

						//WlErrorMsg( aBuf );
						printf( "%s\n", aBuf );
					}
				}
      }	
      else {
        WlTraceMsg("cLEJSGCThread::fExecute - calling JS_MaybeGC (%d)",++aMaybeGCCount);
				RVTRecursiveLock<RVMutexLock>::WriteLockGuard aGuard( mJSGCLock );
        CJSGCRequestBeginEnd Guard( mGCCx, UseOldStyle );
				JS_MaybeGC(mGCCx);
        WlTraceMsg("cLEJSGCThread::fExecute - calling JS_MaybeGC, after GC");
			}

      // Chaim - don't wait for end of slice to exit
      if (!mShouldExecute) break;
			::rvSleep(CLEIniFileProperties::JSGCSleep);
			++aTimeSlice;
		}

    // Will perform GC on last time
    JS_DestroyContext(mGCCx);
  // Chaim - moved here so it will disconnect from Java after final GC
  	iLEJSAgendaMachineThread::fFinal();
  }
	catch (std::exception* e){
		ExceptionHandler(e);
	}
	catch (std::exception& e){
		ExceptionHandler(&e);
	}

}

void cLEJSGCThread::fTerminate()
{
	mShouldExecute = false;
}

iSession& cLEJSGCThread::vGetSession()
{
	return mLoadGeneratorSlave;
}

void cLEJSGCThread::reportApply( iReport& )
{
	WlAssert(0);
}

void cLEJSGCThread::reportLog( iReport& )
{
	WlAssert(0);
}

bool cLEJSGCThread::IsJSActive()
{
	return false; // never evaluating code
}

JSContext* cLEJSGCThread::	vCurrentJSContext()
{
	return mGCCx;
}


iLEJSMachineThread& cLEJSGCThread::vGetMachineThread()
{
	return *this;
}

void cLEJSGCThread::vEnterJava( const char* rClassDescriptionName, const char* rMemberName, tJavaCallType )
{} // don't care about that, never active anyway

void cLEJSGCThread::vExitJava( JSBool rOK, const char* rClassDescriptionName, const char* rMemberName, tJavaCallType )
{}

bool cLEJSGCThread::vIsJSGC()
{
	return true;
}

// Chaim - don't care what it returns.
iLEProtocolJSEmbedContext& cLEJSGCThread::GetProtocolJSEmbedContext(WLProtocolIndex pProtocolIndex) { 
  WlAssert(0);
  iLEProtocolJSEmbedContext* tmp=NULL;
  return *tmp;
}

void cLEJSGCThread::ExceptionHandler(std::exception* e){
	RVCString FullExcName ("");
	WLERRGetErrorMsg(e, FullExcName);
	RVCString StackTrace("");
	if (WLERRGetStackTrace(e, StackTrace) && StackTrace.length() > 0){
	  WlErrorMsg("CLoadThreadInstance::loadLoop - Exception %s occured\nTrace:\n%s", FullExcName, StackTrace);
	  printf("CLoadThreadInstance::loadLoop - Exception %s occured\nTrace:\n%s", FullExcName, StackTrace);
	}
	else {
	  WlErrorMsg("CLoadThreadInstance::loadLoop - Exception %s occured", FullExcName);
	  printf("CLoadThreadInstance::loadLoop - Exception %s occured", FullExcName);
	}
}
