/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************
 *	File name : jsGlobals.cpp
 *
 *	Author : Zion Menahem
 *
 *	Description :
 *
 *	History :
 *
 *	Modifications :
 *		Who				When				What
 *		---				----				----
 *
******************************************************************/

/** include files **/
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
//#include <windows.h>
#include <fstream.h>

#include "rvjsapi.h"

#include "wlerrorlib.h"
#include "platform.h"
#include "loadengineglobals.h"
#include "loadthreadsmanager.h"
#include "wljsobjects.h"
#include "loadjsdefs.h"
#include "tracedll.h"
#include "wlerrorlib.h"
#include "loadexceptions.h"
#include "loadenginevariablesdata.h"
#include "loadengineinifileproperties.h"
#include "loadreportinfo.h"
#include "loadenginedecs.h"
#include "jsjavacore.h"
#include "generalprocess.h"
#include "tvenv.h"
#include "gt_unicode.h"
#include "gt_jshelper.h"

//#include <time.h>

/** local definitions **/

/** default settings **/

/** external functions **/

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/

/** classes definitions **/

/** public functions **/

/** private functions **/




/******************************************************************************\

   Global functions and properties

\******************************************************************************/

#ifdef _DEBUG
#ifdef __TV_UNIX__
#include <sys/timeb.h>
#include <time.h>
#endif // __TV_UNIX__


static JSBool
global_printf(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  JSString* str;
	*rval = JSVAL_VOID;

	if (argc > 0) {
		str = JS_ValueToString(cx, argv[0]);
		printf("%s\n", JS_GetStringBytes(str));
		*rval = argv[0];
	}

	return JS_TRUE;
}


static JSBool
global_write(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  if (argc > 0) {
		JSString* str = JS_ValueToString(cx, argv[0]);
		// call print function with str->bytes
//		WlTraceMsg(JS_GetStringBytes(str));
		*rval = STRING_TO_JSVAL(str);
	}

	return JS_TRUE;
}


static JSBool
global_getTime(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  DWORD prevTime = 0;
	DWORD delta;

	if (argc > 0 && JSVAL_IS_INT(argv[0]))
		prevTime = (DWORD)JSVAL_TO_INT(argv[0]);

#ifdef __TV_UNIX__
	struct timeb tbf;
	ftime( &tbf );
	delta = 1000*tbf.time + tbf.millitm - prevTime;
#else
	delta = GetTickCount() - prevTime;
#endif // __TV_UNIX__
	*rval = INT_TO_JSVAL((int)delta);

	return JS_TRUE;
}

#endif


static JSBool
global_getOperatingSystem(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  *rval = STRING_TO_JSVAL(JS_NewStringCopyZ(cx, TVPGetOSInfo()));

  return JS_TRUE;
}

static JSBool
global_getWorkingSetSize(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  *rval = INT_TO_JSVAL(TVPGetWorkingSetSize());

  return JS_TRUE;
}

static JSBool
global_getProtocolNames(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  *rval = STRING_TO_JSVAL(ANSINewStringCopyZ(cx, CLEIniFileProperties::ProtocolsNames.data()));

  return JS_TRUE;
}

static JSBool
global_getWLTempPath(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

	iLEAgendaJSEmbedContext* pAgendaEmbedContext = iLEAgendaJSEmbedContext::fGetAgendaEmbedContext(cx);
	int masterPID = pAgendaEmbedContext->fGetProcess().GetMasterPid();
	int numScript = pAgendaEmbedContext->GetNumScriptInMix();

  *rval = STRING_TO_JSVAL(ANSINewStringCopyZ(cx, CArrangeMachineTempFiles::GetWlTempFullPath(masterPID, numScript)));

  return JS_TRUE;
}

static JSBool global_IsAbort(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	iLEAgendaJSEmbedContext* pAgendaEmbedContext = iLEAgendaJSEmbedContext::fGetAgendaEmbedContext(cx);

	if (!pAgendaEmbedContext->InInitAgenda()){
		if (pAgendaEmbedContext->isAbort()){
			*rval = JSVAL_TRUE;
		}
		 else {
			*rval = JSVAL_FALSE;
		}
	}
	return JS_TRUE;
}

static JSBool
global_Sleep(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  try {
  YieldRequest(cx);

  int32 time1;
	*rval = JSVAL_VOID;

  if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_SLEEP_FUNC, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  if (JS_ValueToInt32(cx, argv[0], &time1) == JS_FALSE) {
    return JS_FALSE;
		}

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
		CWLJSScript& WLJSS = tis->GetJSScript();
    if(WLJSS.m_PropWorkSpace.m_DisableSleep){
      tis->ProtocolSleepDisabled(time1);
			return JS_TRUE;
    }

		if((WLJSS.m_PropWorkSpace.m_SleepRandomMin != DEFAULT_LE_SLEEP_RANDOM_MIN )
			&&(WLJSS.m_PropWorkSpace.m_SleepRandomMax != DEFAULT_LE_SLEEP_RANDOM_MAX ) ) {

			int randomNumber = rand(); //randomNumber is a random number between 0 to RAND_MAX

			time1 = WLJSS.m_PropWorkSpace.m_SleepRandomMin + 
				(long)( ( randomNumber * (WLJSS.m_PropWorkSpace.m_SleepRandomMax -
												WLJSS.m_PropWorkSpace.m_SleepRandomMin + 1) ) /RAND_MAX );
		}
		else {
			if (WLJSS.m_PropWorkSpace.m_SleepDeviation != -1 ) {
				long MaxLimit = time1 + ((WLJSS.m_PropWorkSpace.m_SleepDeviation * time1)/100);
				long MinLimit = time1 - ((WLJSS.m_PropWorkSpace.m_SleepDeviation * time1)/100);
				
				int randomNumber = rand(); //randomNumber is a random number between 0 to RAND_MAX
				time1 = MinLimit + ( (randomNumber * (MaxLimit-MinLimit + 1))/RAND_MAX );
			}
		}		

		if( tis->BeforeSleep( time1 ) ) {
			{
				CJSGCRequestSuspendResume GCGuard(cx);
				tis->sleep(time1);
			}
			tis->AfterSleep(time1);
		}
    if (tis->isAbort()) return JS_FALSE;
  }

  } catch (...) {
    throw;
  }

	return JS_TRUE;
}

static JSBool
global_SleepAlways(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  try {
    YieldRequest(cx);
    
    int32 time1;
    *rval = JSVAL_VOID;
    
    if (argc != 1) {
      WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_SLEEP_FUNC, 1);
      return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }
    
    if (JS_ValueToInt32(cx, argv[0], &time1) == JS_FALSE) {
      return JS_FALSE;
		  }
    
    CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);
    
    if(tis) {
      
      if( tis->BeforeSleep( time1 ) ) {
        {
          CJSGCRequestSuspendResume GCGuard(cx);
          tis->sleep(time1);
        }
        tis->AfterSleep(time1);
      }
      if (tis->isAbort()) return JS_FALSE;
    }
    
  } catch (...) {
    throw;
  }
  
  return JS_TRUE;
}

static JSBool
global_DisableJavaDefaultStatistics(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	
	// Chaim - no statistics in InitAgenda so this isn't needed in InitAgenda either
	if (argc != 0) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_DISABLE_JAVA_DEFAULT_STATISTICS_FUNC, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
	
  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
		tis->ToggleJavaDefaultStatistics( false );
    if (tis->isAbort()) return JS_FALSE;
  }
	return JS_TRUE;
}

static JSBool
global_EnableJavaDefaultStatistics(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	
	if (argc != 0) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_ENABLE_JAVA_DEFAULT_STATISTICS_FUNC, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
	
  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
		tis->ToggleJavaDefaultStatistics( true );
    if (tis->isAbort()) return JS_FALSE;
  }
	return JS_TRUE;
}


static JSBool
global_SetTimer(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_SET_TIMER_FUNC, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

	char* TimerName = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));
	if (strlen(TimerName) == 0) return JS_TRUE;

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
		tis->resetTimer(TimerName);
    if (tis->isAbort()) return JS_FALSE;
  }

	return JS_TRUE;
}

static JSBool
global_SendTimer(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_SEND_TIMER_FUNC, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

	char* TimerName = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));
	if (strlen(TimerName) == 0) return JS_TRUE;

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
		tis->sendTimer(TimerName);
    if (tis->isAbort()) return JS_FALSE;
  }

	return JS_TRUE;
}

static JSBool
global_SendCounter(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_SEND_COUNTER_FUNC, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

	char* CounterName = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));
	if (strlen(CounterName) == 0) return JS_TRUE;

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
		tis->addCounter(CounterName);
    if (tis->isAbort()) return JS_FALSE;
  }

	return JS_TRUE;
}

static JSBool
global_SendMeasurement(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  if (argc != 2) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_SEND_MEASUREMENT_FUNC, 2);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

	char* MeasurementName = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));
	if (strlen(MeasurementName) == 0) return JS_TRUE;

  jsdouble measurement;
  if (JS_ValueToNumber(cx, argv[1], &measurement) == JS_FALSE) {
		WLErrorException e(LoadExceptionFrmFile, 260, WLJSNAME_SEND_MEASUREMENT_FUNC, 2 );
    return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
		}

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
		tis->addMeasurement(MeasurementName,measurement);
    if (tis->isAbort()) return JS_FALSE;
  }

	return JS_TRUE;
}

static JSBool global_Lock(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

	iLEAgendaJSEmbedContext* pAgendaEmbedContext = iLEAgendaJSEmbedContext::fGetAgendaEmbedContext(cx);
  if (pAgendaEmbedContext->InInitAgenda()) {
    WLErrorException e(LoadExceptionFrmFile, 231, WLJSNAME_LOCK);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

	pAgendaEmbedContext->UserLock();
  if (pAgendaEmbedContext->isAbort()) return JS_FALSE;

	return JS_TRUE;
}

static JSBool global_Unlock(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

	iLEAgendaJSEmbedContext* pAgendaEmbedContext = iLEAgendaJSEmbedContext::fGetAgendaEmbedContext(cx);
  if (pAgendaEmbedContext->InInitAgenda()) {
    WLErrorException e(LoadExceptionFrmFile, 231, WLJSNAME_LOCK);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

	pAgendaEmbedContext->UserUnlock();
  if (pAgendaEmbedContext->isAbort()) return JS_FALSE;

	return JS_TRUE;
}


static JSBool global_GetRoundNumScriptInSlave(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    if (tis->isAbort()) return JS_FALSE;
		*rval = INT_TO_JSVAL( tis->GetRoundNumScriptInSlave() );
  }
	
	return JS_TRUE;
}

static JSBool global_GetRoundNumMixInSlave(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    if (tis->isAbort()) return JS_FALSE;
		*rval = INT_TO_JSVAL( tis->GetRoundNumMixInSlave() );
  }
	
	return JS_TRUE;
}

static JSBool global_GetThreadNumInSlave(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    if (tis->isAbort()) return JS_FALSE;
		*rval = INT_TO_JSVAL( tis->GetSlaveThreadNum() );
  }
	
	return JS_TRUE;
}

static JSBool global_GetThreadType(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    if (tis->isAbort()) return JS_FALSE;
		*rval = INT_TO_JSVAL( WL_ENUM_TO_JSINT( tis->GetThreadType() ) );
  }
	
	return JS_TRUE;
}


static JSBool
global_VerificationBeginTransaction(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD != NULL) {
	  if (pMSD->InInitAgenda()) {
      WLErrorException e(LoadExceptionFrmFile, 231, WLJSNAME_VERIFICATION_BEGIN_TRANSACTION);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }
  }

  if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_VERIFICATION_BEGIN_TRANSACTION, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

	const char* transName = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));

  if (!transName || strlen(transName)==0) {
    WLErrorException e(LoadExceptionFrmFile, 225, WLJSNAME_VERIFICATION_BEGIN_TRANSACTION, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
      tis->VerificationBeginTransaction(transName, tis->fGetProcess().fGetCurrentTime(), "UserDefined", NULL, 0);
    tis->ProtocolBeginTransaction(transName, tis->fGetProcess().fGetCurrentTime(), "UserDefined", NULL, 0);
    if (tis->isAbort()) return JS_FALSE;
  }

	return JS_TRUE;
}


static JSBool
global_VerificationBeginTransactionEx(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD != NULL) {
	  if (pMSD->InInitAgenda()) {
      WLErrorException e(LoadExceptionFrmFile, 231, WLJSNAME_VERIFICATION_BEGIN_TRANSACTION_EX);
			return LoadReport( cx, e ); //is the same as : 	return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }
  }

  if (argc != 2) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_VERIFICATION_BEGIN_TRANSACTION_EX, 2);
		return LoadReport( cx, e ); //is the same as : 	return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

	const char* transName = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));

  if (!transName || strlen(transName)==0) {
    WLErrorException e(LoadExceptionFrmFile, 225, WLJSNAME_VERIFICATION_BEGIN_TRANSACTION_EX, 1);
		return LoadReport( cx, e ); //is the same as : 	return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
		jsdouble val;
		if( JS_ValueToNumber( cx, argv[1], &val ) ) {
			tis->VerificationBeginTransaction(transName,(double)val, "UserDefined", NULL, 0);
			tis->ProtocolBeginTransaction(transName,(double)val, "UserDefined", NULL, 0);
		}
		else {
			WLErrorException e(LoadExceptionFrmFile, 260, WLJSNAME_VERIFICATION_BEGIN_TRANSACTION_EX, 2);
			LoadReport(cx,e);
		}
		if (tis->isAbort()) return JS_FALSE;
	}
	return JS_TRUE;
}


static JSBool
global_VerificationEndTransaction(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  WLLogMessage::Severity severity = WLLogMessage::Success;
  bool save = false;

  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD != NULL) {
	  if (pMSD->InInitAgenda()) {
      WLErrorException e(LoadExceptionFrmFile, 231, WLJSNAME_VERIFICATION_END_TRANSACTION);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	  }
  }

  if ((argc < 1) || (argc > 3)) {
    WLErrorException e(LoadExceptionFrmFile, 221, WLJSNAME_VERIFICATION_END_TRANSACTION, 1, 3);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

	const char* transname = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));

	if (argc >= 2) {
		if (!JSVAL_IS_INT(argv[1])) {
      WLErrorException e(LoadExceptionFrmFile, 226, WLJSNAME_VERIFICATION_END_TRANSACTION, 2);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }
    else {
      int sev = JSVAL_TO_INT(argv[1]);
      if (sev != WLLogMessage::Success && sev != WLLogMessage::Note && sev != WLLogMessage::Warning && sev != WLLogMessage::Error && sev != WLLogMessage::Fatal) {
        WLErrorException e(LoadExceptionFrmFile, 226, WLJSNAME_VERIFICATION_END_TRANSACTION, 2);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
      }
      severity = (WLLogMessage::Severity)sev;
    }
	}

	if (argc == 3) {
    const char* str = JS_GetStringBytes(JS_ValueToString(cx, argv[2]));
    if (!strcmpi(str, NO_STR_VALUE) || !strcmpi(str, FALSE_STR_VALUE))
      save = false;
    else if (!strcmpi(str, YES_STR_VALUE) || !strcmpi(str, TRUE_STR_VALUE))
      save = true;
    else {
      WLErrorException e(LoadExceptionFrmFile, 224, WLJSNAME_VERIFICATION_END_TRANSACTION, 3);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }
	}

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    if (tis->isAbort())
      return JS_FALSE;
    if (!tis->VerificationEndTransaction(transname, tis->fGetProcess().fGetCurrentTime(), severity, save, true, true ))
      return JS_FALSE;
    tis->ProtocolEndTransaction(transname, tis->fGetProcess().fGetCurrentTime(), severity, save, true, true );
  }

	return JS_TRUE;
}

static JSBool
global_VerificationEndTransactionEx(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  WLLogMessage::Severity severity;
	bool save;
	double Time;
	bool SendTimers;
	bool IsLog;

  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD != NULL) {
	  if (pMSD->InInitAgenda()) {
      WLErrorException e(LoadExceptionFrmFile, 231, WLJSNAME_VERIFICATION_END_TRANSACTION_EX);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	  }
  }

  if (argc != 6) { 
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_VERIFICATION_END_TRANSACTION_EX, 6);
		return LoadReport( cx, e ); 
  }

	const char* transname = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));

	int32 sev;
	if (JS_ValueToInt32(cx, argv[1], &sev) == JS_FALSE) {
		return JS_FALSE;
	}
  if (sev != WLLogMessage::Success && sev != WLLogMessage::Note && sev != WLLogMessage::Warning && sev != WLLogMessage::Error && sev != WLLogMessage::Fatal) {
		WLErrorException e(LoadExceptionFrmFile, 226, WLJSNAME_VERIFICATION_END_TRANSACTION_EX, 2);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
  severity = (WLLogMessage::Severity)sev;
	
  const char* str = JS_GetStringBytes(JS_ValueToString(cx, argv[2]));
  if (!strcmpi(str, NO_STR_VALUE) || !strcmpi(str, FALSE_STR_VALUE)) {
		save = false;
	} else if (!strcmpi(str, YES_STR_VALUE) || !strcmpi(str, TRUE_STR_VALUE)) {
		save = true;
	} else {
		WLErrorException e(LoadExceptionFrmFile, 224, WLJSNAME_VERIFICATION_END_TRANSACTION_EX, 3);
		return LoadReport( cx, e ); 
	}

  
	jsdouble val;
	if( JS_ValueToNumber( cx, argv[3], &val ) ) {
		Time = (double)val;
	}	else {
		WLErrorException e(LoadExceptionFrmFile, 260, WLJSNAME_VERIFICATION_END_TRANSACTION_EX, 4);
		LoadReport(cx,e);
	}
	
	const char* str2 = JS_GetStringBytes(JS_ValueToString(cx, argv[4]));
  if (!strcmpi(str2, NO_STR_VALUE) || !strcmpi(str2, FALSE_STR_VALUE))
		SendTimers = false;
  else if (!strcmpi(str2, YES_STR_VALUE) || !strcmpi(str2, TRUE_STR_VALUE))
		SendTimers = true;
  else {
		WLErrorException e(LoadExceptionFrmFile, 224, WLJSNAME_VERIFICATION_END_TRANSACTION_EX, 5);
		return LoadReport( cx, e ); 
  }

	const char* str3 = JS_GetStringBytes(JS_ValueToString(cx, argv[5]));
  if (!strcmpi(str3, NO_STR_VALUE) || !strcmpi(str3, FALSE_STR_VALUE))
		IsLog = false;
  else if (!strcmpi(str3, YES_STR_VALUE) || !strcmpi(str3, TRUE_STR_VALUE))
		IsLog = true;
  else {
		WLErrorException e(LoadExceptionFrmFile, 224, WLJSNAME_VERIFICATION_END_TRANSACTION_EX, 6);
		return LoadReport( cx, e ); 
	}

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
		if (tis->isAbort())
      return JS_FALSE;
    if (!tis->VerificationEndTransaction(transname, Time, severity, save, SendTimers, IsLog ))
      return JS_FALSE;
  }

	return JS_TRUE;
}


static JSBool
global_VerificationRecordEvent(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

  YieldRequest(cx);

  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD != NULL) {
	  if (pMSD->InInitAgenda()) {
      WLErrorException e(LoadExceptionFrmFile, 231, WLJSNAME_VERIFICATION_RECORD_EVENT);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	  }
  }
  if (argc != 1 && argc != 2) {
    WLErrorException e(LoadExceptionFrmFile, 221, WLJSNAME_VERIFICATION_RECORD_EVENT, 1, 2);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

	const char* event = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));
  const char* description = " ";
  if (argc == 2)
	  description = JS_GetStringBytes(JS_ValueToString(cx, argv[1]));

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    tis->VerificationRecordEvent(event, description, tis->fGetProcess().fGetCurrentTime() );
	  if (tis->isAbort()) return JS_FALSE;
  }
  return JS_TRUE;
}

static JSBool
global_VerificationRecordEventEx(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

  YieldRequest(cx);

  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD != NULL) {
	  if (pMSD->InInitAgenda()) {
      WLErrorException e(LoadExceptionFrmFile, 231, WLJSNAME_VERIFICATION_RECORD_EVENT_EX);
			return LoadReport(cx,e);
	  }
  }
  if (argc != 7) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_VERIFICATION_RECORD_EVENT_EX,7);
		return LoadReport(cx,e);
  }

	const char* event;
  const char* description;
	double Time;
  WLLogMessage::Severity severity = WLLogMessage :: Note;
	const char* extraDataType;
	void * extraData = NULL;
	unsigned long len;
	bool IsLog;

	event= JS_GetStringBytes(JS_ValueToString(cx, argv[0]));

	description = JS_GetStringBytes(JS_ValueToString(cx, argv[1]));

	jsdouble val;
	if( JS_ValueToNumber( cx, argv[2], &val ) ) 
		Time = (double)val;
	else {
		WLErrorException e(LoadExceptionFrmFile, 260, WLJSNAME_VERIFICATION_RECORD_EVENT_EX, 3);
		LoadReport(cx,e);
	}

	int32 sev;
	if (JS_ValueToInt32(cx, argv[3], &sev) == JS_FALSE)
		return JS_FALSE;
  if (sev != WLLogMessage::Success && sev != WLLogMessage::Note && sev != WLLogMessage::Warning && sev != WLLogMessage::Error && sev != WLLogMessage::Fatal ) {
		WLErrorException e(LoadExceptionFrmFile, 226, WLJSNAME_VERIFICATION_RECORD_EVENT_EX, 4);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
  severity = (WLLogMessage::Severity)sev;


	extraDataType = JS_GetStringBytes(JS_ValueToString(cx, argv[4]));//to be changed
	extraData = JS_GetStringBytes(JS_ValueToString(cx, argv[5]));//to be changed
	
	//Galtech
//	len = (unsigned long)JS_GetStringLength(JS_ValueToString(cx, argv[5]));
	len = strlen((char*)extraData);
	 
	const char* str3 = JS_GetStringBytes(JS_ValueToString(cx, argv[6]));//to be changed
  if (!strcmpi(str3, NO_STR_VALUE) || !strcmpi(str3, FALSE_STR_VALUE))
		IsLog = false;
  else if (!strcmpi(str3, YES_STR_VALUE) || !strcmpi(str3, TRUE_STR_VALUE))
    IsLog = true;
  else {
		WLErrorException e(LoadExceptionFrmFile, 224, WLJSNAME_VERIFICATION_RECORD_EVENT_EX, 7);
	  return LoadReport( cx, e ); 
	}


  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
		tis->VerificationRecordEvent(event, description, Time,severity,extraDataType,extraData,len,true);
	  if (tis->isAbort()) return JS_FALSE;
  }
  return JS_TRUE;
}


static JSBool
global_VerificationSetFailure(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD != NULL) {
	  if (pMSD->InInitAgenda()) {
      WLErrorException e(LoadExceptionFrmFile, 231, WLJSNAME_VERIFICATION_SET_FAILURE_REASON);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	  }
  }
  if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_VERIFICATION_SET_FAILURE_REASON, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

	const char* reason = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    tis->VerificationSetFailureReason(reason, NULL, tis->fGetProcess().fGetCurrentTime(), true );
	  if (tis->isAbort()) return JS_FALSE;
  }
  return JS_TRUE;
}

static JSBool
global_VerificationSetFailureEx(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);
	const char* reason;
	const char* description;
	double Time;
	bool IsEvent;

  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD != NULL) {
	  if (pMSD->InInitAgenda()) {
      WLErrorException e(LoadExceptionFrmFile, 231, WLJSNAME_VERIFICATION_SET_FAILURE_REASON_EX);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	  }
  }
  if (argc != 4) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_VERIFICATION_SET_FAILURE_REASON_EX, 4);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

	reason = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));
	description = JS_GetStringBytes(JS_ValueToString(cx,argv[1]));
	jsdouble val;
	if( JS_ValueToNumber( cx, argv[2], &val ) ) 
			Time = (double)val;
	else {
		WLErrorException e(LoadExceptionFrmFile, 260, WLJSNAME_VERIFICATION_SET_FAILURE_REASON_EX, 3);
		LoadReport(cx,e);
	}
	const char * str = JS_GetStringBytes(JS_ValueToString(cx,argv[3]));
  if (!strcmpi(str, NO_STR_VALUE) || !strcmpi(str, FALSE_STR_VALUE))
    IsEvent = false;
  else if (!strcmpi(str, YES_STR_VALUE) || !strcmpi(str, TRUE_STR_VALUE)) {
    IsEvent = true;
	} 
	else {
    WLErrorException e(LoadExceptionFrmFile, 224, WLJSNAME_VERIFICATION_SET_FAILURE_REASON_EX, 4);
	  return LoadReport( cx, e ); 
	}

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    tis->VerificationSetFailureReason(reason, description, Time, IsEvent );
	  if (tis->isAbort()) return JS_FALSE;
  }
  return JS_TRUE;
}

static JSBool
global_VerificationEndTransactionFailureEx(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

	const char* transName;
	double time;
	WLLogMessage::Severity  severity;
	const char* reason;
	bool sendTimers;
	bool isLog;
	bool isFailureEvent;	

  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD != NULL) {
	  if (pMSD->InInitAgenda()) {
      WLErrorException e(LoadExceptionFrmFile, 231, WLJSNAME_VERIFICATION_END_TRANSACTION_FAILURE_EX);
			return LoadReport(cx,e);
	  }
  }
  if (argc != 7) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_VERIFICATION_END_TRANSACTION_FAILURE_EX, 7);
		return LoadReport(cx,e);
  }

	transName = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));

	jsdouble val;
	if( JS_ValueToNumber( cx, argv[1], &val ) ) 
			time = (double)val;
	else {
		WLErrorException e(LoadExceptionFrmFile, 260, WLJSNAME_VERIFICATION_END_TRANSACTION_FAILURE_EX, 2);
		LoadReport(cx,e);
	}
	
	int32 sev;
	if (JS_ValueToInt32(cx, argv[2], &sev) == JS_FALSE)
		return JS_FALSE;
  if (sev != WLLogMessage::Success && sev != WLLogMessage::Note && sev != WLLogMessage::Warning && sev != WLLogMessage::Error && sev != WLLogMessage::Fatal) {
		WLErrorException e(LoadExceptionFrmFile, 226, WLJSNAME_VERIFICATION_END_TRANSACTION_FAILURE_EX, 3);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
  severity = (WLLogMessage::Severity)sev;

	reason = JS_GetStringBytes(JS_ValueToString(cx, argv[3]));

	const char * str = JS_GetStringBytes(JS_ValueToString(cx,argv[4]));
  if (!strcmpi(str, NO_STR_VALUE) || !strcmpi(str, FALSE_STR_VALUE))
    sendTimers = false;
  else if (!strcmpi(str, YES_STR_VALUE) || !strcmpi(str, TRUE_STR_VALUE))
    sendTimers = true;
  else {
    WLErrorException e(LoadExceptionFrmFile, 224, WLJSNAME_VERIFICATION_END_TRANSACTION_FAILURE_EX, 5);
	  return LoadReport( cx, e ); 
	}

 	const char * str1 = JS_GetStringBytes(JS_ValueToString(cx,argv[5]));
  if (!strcmpi(str1, NO_STR_VALUE) || !strcmpi(str1, FALSE_STR_VALUE))
    isLog = false;
  else if (!strcmpi(str1, YES_STR_VALUE) || !strcmpi(str1, TRUE_STR_VALUE))
    isLog = true;
  else {
    WLErrorException e(LoadExceptionFrmFile, 224, WLJSNAME_VERIFICATION_END_TRANSACTION_FAILURE_EX, 6);
	  return LoadReport( cx, e ); 
	}

	const char * str2 = JS_GetStringBytes(JS_ValueToString(cx,argv[6]));
  if (!strcmpi(str2, NO_STR_VALUE) || !strcmpi(str2, FALSE_STR_VALUE))
    isFailureEvent = false;
  else if (!strcmpi(str2, YES_STR_VALUE) || !strcmpi(str2, TRUE_STR_VALUE))
    isFailureEvent = true;
  else {
    WLErrorException e(LoadExceptionFrmFile, 224, WLJSNAME_VERIFICATION_END_TRANSACTION_FAILURE_EX, 7);
	  return LoadReport( cx, e ); 
	}


  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    tis->VerificationEndTransactionFailure(transName, time, severity ,reason, sendTimers , isLog, isFailureEvent );
	  if (tis->isAbort()) return JS_FALSE;
  }
  return JS_TRUE;
}


static JSBool
global_VerificationGetCurrentTransaction(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD != NULL) {
	  if (pMSD->InInitAgenda()) {
      WLErrorException e(LoadExceptionFrmFile, 231, WLJSNAME_VERIFICATION_GET_CURRENT_TRANSACTION);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	  }
  }
  if (argc != 0) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_VERIFICATION_GET_CURRENT_TRANSACTION, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    const char* transName = tis->VerificationGetCurrentTransaction();
	  if (tis->isAbort()) return JS_FALSE;
    if (transName)
      *rval = STRING_TO_JSVAL(JS_NewStringCopyZ(cx, transName));
  }
  return JS_TRUE;
}

static JSBool
global_VerificationGetTransactionDepth(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD != NULL) {
	  if (pMSD->InInitAgenda()) {
      WLErrorException e(LoadExceptionFrmFile, 231, WLJSNAME_VERIFICATION_GET_TRANSACTION_DEPTH);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	  }
  }
  if (argc != 0) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_VERIFICATION_GET_TRANSACTION_DEPTH, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    *rval = INT_TO_JSVAL(tis->VerificationGetTransactionDepth());
	  if (tis->isAbort()) return JS_FALSE;
  }
  return JS_TRUE;
}


static JSBool
global_SynchornizationPoint(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD != NULL) {
	  if (pMSD->InInitAgenda()) {
      WLErrorException e(LoadExceptionFrmFile, 231, WLJSNAME_SYNCHRONIZATION_POINT);
			*rval = INT_TO_JSVAL(3);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	  }
  }

  if (argc > 1) {
    WLErrorException e(LoadExceptionFrmFile, 221, WLJSNAME_SYNCHRONIZATION_POINT, 0, 1);
		*rval = INT_TO_JSVAL(3);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

	int32 timeoutmillis;
	if (argc == 1) {
    if (JS_ValueToInt32(cx, argv[0], &timeoutmillis) == JS_FALSE)
      return JS_FALSE;
		if (timeoutmillis < 0) timeoutmillis = 0;
	}
	else
		timeoutmillis = -1;

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(!tis) {
		RVCString Message;
    Message = Message + WLJSNAME_SYNCHRONIZATION_POINT + " : Internal error : ti is null.";
		*rval = INT_TO_JSVAL(3);
		return LoadReport( cx, WLLogMessage::Error, Message.data() );
	}
  else {
    if (tis->isAbort()) return JS_FALSE;
		SynchronizationPointStatus status;
		{
			CJSGCRequestSuspendResume GCGuard(cx);
			status = tis->SynchronizationPoint(timeoutmillis);
		}
    if (tis->isAbort()) return JS_FALSE;

		switch( status ) {
			case SynchronizationPointLoadChanged:
		    *rval = INT_TO_JSVAL(1);
				return JS_TRUE;
			case SynchronizationPointTimeout :
		    *rval = INT_TO_JSVAL(2);
				return JS_TRUE;
			case SynchronizationPointOK :
		    *rval = INT_TO_JSVAL(0);
				return JS_TRUE;
		}
  }
	return JS_TRUE;
}

// this is undocumented
JSBool global_Report(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {
  YieldRequest(cx);

	WLLogMessage::Severity severity;
	{
		int32 severityX0;

		if( argc < 2 ) {
			return LoadReport( cx, WLLogMessage::Fatal, "Report() takes 2 params");
			}

		if( !JS_ValueToInt32( cx, argv[0], &severityX0 ) ) {
			return LoadReport( cx, WLLogMessage::Fatal, "Report must have numeric first parameter" );
			}

		severity = (WLLogMessage::Severity)severityX0;
/*
		switch( severityX0 ) {
		case 0:
			severity = WLLogMessage::Note;
			break;
		case 1:
			severity = WLLogMessage::Warning;
			break;
		case 2:
			severity = WLLogMessage::Error;
			break;
		case 3:
			severity = WLLogMessage::Fatal;
			break;
		default:
			return LoadReport( cx, WLLogMessage::Fatal, "Report must have numeric first parameter 0..3" );
		}
*/
	}

  const char* message = JS_GetStringBytes(JS_ValueToString(cx, argv[1]));
	WlAssert( message );

	
	bool HasTime = (argc>=3);
	double Time;
	if( HasTime ) {
		jsdouble t;
		if( JS_ValueToNumber( cx, argv[2], &t ) ) {
			Time = t;
		} else {
			return LoadReport( cx, WLLogMessage::Fatal, "Report must have numeric third parameter" );
		}
	}
	

	bool MT = false;	
/*
	if( argc > 2 ) {
		int32 X0;
		MT = JS_ValueToInt32( cx, argv[2], &X0 );
		if( MT ) {
			MT = (X0 != 0);
			}
		}
*/
  bool Rtn;
  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
		if( MT ) {
			Rtn = tis->reportMultiThread( severity, message );
		}	else {
			if( HasTime ) {
				Rtn = tis->reportTime( severity, message, Time );
			} else {
				Rtn = tis->report( severity, message );
			}
		}
	} else {
		Rtn = LoadReport( cx, severity, message );
	}
	return Rtn;
}

JSBool global_Note(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {
  YieldRequest(cx);

  if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_INFO_MESSAGE, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    // JS_ReportError(cx, e.GetErrorMsg());
		// return JS_FALSE;
  }

  const char* message = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    if (tis->isAbort()) {
			return JS_FALSE;
		}
	}

	cLoadReportInfoAgenda errRep(WLLogMessage::Note, message);
	return LoadReportNoExn( cx, errRep );
}


//*-----------------------------------------------------------------------------
//| MODULE | loadengine
//|-----------------------------------------------------------------------------
//| CLASS  | n/a global function
//|-----------------------------------------------------------------------------
//| FUN    | global_Trace
//|-----------------------------------------------------------------------------
//| RETVAL | only JS_TRUE                                                                   
//|-----------------------------------------------------------------------------
//| PARAMS | 
//|        | 1)   IN   JSContext *cx    pointer to the JS context  
//|        | 2)   IN   JSObject *obj    pointer to the JS object     
//|        | 3)   IN   uintN argc       counts parameters
//|        | 4)   IN   jsval *argv      array of pointers to the parameters
//|        | 5)   OUT  jsval *rval      return value                                                              
//|-----------------------------------------------------------------------------
//| DESCR  |   This function prints message from the agenda to the trace file.                                                                   
//|-----------------------------------------------------------------------------
//| NOTES  |  It doesn't consider this flags TRACED_MODULES or NOT_TRACED_MODULES
//|        |  *** Greg 
//*-----------------------------------------------------------------------------
JSBool global_Trace(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {
  YieldRequest(cx);

  if(!WlTrace) return JS_TRUE;

  if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_TRACE_MESSAGE, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
   }

  const char* message = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    if (tis->isAbort()) {
			return JS_TRUE;
		}
	}

	CTraceMsg TrcObj=0; // statically defined object.

	if(message == NULL) return JS_TRUE; 

	// Checks that the trace file exists.
	if (WlTrace)
	{
		TrcObj.PreDefVals (__MODULE__,__FILE__,__LINE__);
		TrcObj.TraceMsgNoFilter("****Trace from Agenda:  %s",message);
	}
	return JS_TRUE;
}

JSBool global_Warning(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {
  YieldRequest(cx);

  if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_WARNING_MESSAGE, 1);
    return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  const char* message = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    if (tis->isAbort()) {
			return JS_FALSE;
		}
	}

	cLoadReportInfoAgenda loadRepInf(WLLogMessage::Warning, message);
	return LoadReportNoExn( cx, loadRepInf );
	
}

JSBool global_Error(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {
  YieldRequest(cx);

  if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_ERROR_MESSAGE, 1);
    return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  const char* message = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    if (tis->isAbort()) {
			return JS_FALSE;
		}
	}

	cLoadReportInfoAgenda loadRepInf(WLLogMessage::Error, message);
	return LoadReportNoExn( cx, loadRepInf );
}

JSBool global_Severe(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {
  YieldRequest(cx);

  if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_SEVERE_ERROR_MESSAGE, 1);
    return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  const char* message = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));

  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    if (tis->isAbort()) {
			return JS_FALSE;
		}
	}

	cLoadReportInfoAgenda errRep(WLLogMessage::Fatal, message);
	return LoadReportNoExn( cx, errRep );
}

JSBool global_StopTest(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) 
{
  YieldRequest(cx);
  RVCString message;

  if (argc != 0) {
    if (argc == 1) {
      message = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));
    }
    else {
      WLErrorException e(LoadExceptionFrmFile, 221, WLJSNAME_STOPTEST, 0, 1);
      return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }
  }
  else {
    WLErrorException e(LoadExceptionFrmFile, 3, WLJSNAME_STOPTEST);
    message = e.GetErrorMsg();
  }


  
	iLEAgendaJSEmbedContext* pAgendaEmbedContext = iLEAgendaJSEmbedContext::fGetAgendaEmbedContext(cx);
  if(!pAgendaEmbedContext->isAbort()) {
	  LoadReport( cx, WLLogMessage::Note, message.data() );
    pAgendaEmbedContext->OnSevereError();
	}
  
  return JS_FALSE;
}

static JSBool global_CopyMonitorFile(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {

  YieldRequest(cx);

  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD == NULL) {
    WLErrorException e(LoadExceptionFrmFile, 230, WLJSNAME_COPY_MONITOR_FILE);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
  if (!(pMSD->InInitAgenda())) {
    WLErrorException e(LoadExceptionFrmFile, 230, WLJSNAME_COPY_MONITOR_FILE);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }



	if( (argc!=1) && (argc!=2) ) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_COPY_MONITOR_FILE, 2);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
		}

	RVCString source = ANSIGetStringBytes(JS_ValueToString(cx, argv[0]));
	
	char targetFileName[1000];
	char fullPath[WL_MAX_PATH];
	const char* target;
	switch( argc ) {
	case 1:
		{
			TVEnvContainerClass tvpc;
			strcpy( targetFileName, tvpc.TVPGetPathName(source));
		  tvpc.reset();
			const char* extention = tvpc.TVPGetPathExt(source);
			if (strcmp(extention,"")) {
				strcat(targetFileName,".");
				strcat(targetFileName,extention);
			}		
			strcpy(fullPath,CArrangeMachineTempFiles::GetWlTempFullPath(cx ,WLECopiedFile, targetFileName).data());
			target = fullPath;
		}
		break;
	case 2:
		{
			RVCString argumentTarget = ANSIGetStringBytes(JS_ValueToString(cx, argv[1]));
			strcpy(fullPath, CArrangeMachineTempFiles::GetWlTempFullPath(cx ,WLECopiedFile, argumentTarget).data());
			target = fullPath;
		break;
		}
	default:
		WlAssert(0);
		};


  RVCString ErrorMessage;
	{
	CJSGCRequestSuspendResume GCGuard(cx);
  
	if( !pMSD->CopyFile(ToUTF8(source.data()), ToUTF8(target) ) ) {
		return JS_FALSE; // error reported by callee
		}
	}

 	*rval = STRING_TO_JSVAL(ANSINewStringCopyZ(cx,targetFileName));

	return JS_TRUE;
}

//@@@@ new code
static JSBool global_CopyMonitorJavaFile(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) 
{
	JSBool retStatus;
	
	retStatus = global_CopyMonitorFile(cx, obj, argc, argv, rval);

	if(retStatus == JS_FALSE) {
		return JS_FALSE;
	}
	
//Galtech
	RVCString javaFilePath = ANSIGetStringBytes(JS_ValueToString(cx, *rval));

	TVEnvContainerClass tvpc;
	RVCString   extention    = tvpc.TVPGetPathExt(javaFilePath);


	if( (extention == "jar") || (extention == "class") ) {
		iLEJSEmbedContext* aEmbedContext = iLEJSEmbedContext::fsGet( cx );
		cRVJVM& aJVM = aEmbedContext->fGetMachineThread().fGetJVM();
		aJVM.fAddClassPath(javaFilePath);
	}

	return JS_TRUE;
}

static JSBool
global_Include(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  bool execute;

  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD == NULL) {
    WLErrorException e(LoadExceptionFrmFile, 230, WLJSNAME_INCLUDE);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
  if (!(pMSD->InInitAgenda())) {
    WLErrorException e(LoadExceptionFrmFile, 230, WLJSNAME_INCLUDE);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  if ((argc != 1) && (argc != 2)) {
    WLErrorException e(LoadExceptionFrmFile, 221, WLJSNAME_INCLUDE, 1,2);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  const char* source = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));

  if (argc == 2) {
		if (!JSVAL_IS_INT(argv[1])) {
      WLErrorException e(LoadExceptionFrmFile, 249, WLJSNAME_INCLUDE, 2);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }
    else {
      int executeint = JSVAL_TO_INT(argv[1]);
      if ((executeint != 0) && (executeint != 1)) {
        WLErrorException e(LoadExceptionFrmFile, 249, WLJSNAME_INCLUDE, 2);
				return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
      }
      execute = (JSVAL_TO_INT(argv[1]) == 1);
    }
  }
  else {
    execute = false;
		}

  RVCString ErrorMessage;
	CJSGCRequestSuspendResume GCGuard(cx);
  bool status = pMSD->IncludeFile(source, execute);
  if (!status) {
		return JS_FALSE;
  }
  else
    return JS_TRUE;

}

static JSBool
global_CreateJavaThreadWrapper(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	YieldRequest(cx);

  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD == NULL) {
    WLErrorException e(LoadExceptionFrmFile, 230, WLJSNAME_CREATE_JAVA_THREAD_WRAPPER);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
  if (!(pMSD->InInitAgenda())) {
    WLErrorException e(LoadExceptionFrmFile, 230, WLJSNAME_CREATE_JAVA_THREAD_WRAPPER);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  if (argc != 0) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_CREATE_JAVA_THREAD_WRAPPER, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
	
	if(! pMSD->IsJavaThreadWrapperCreationScriptExists() ) {
		pMSD->SetJavaThreadWrapperCreationScript();
		const char* script = "javaThreadWrapper = new Packages.JavaThreadWrapper(this)";
		pMSD->AppendStringForEvaluation(script,CWLJSScript::WLBeforeThreadActivation);
	}
	
	return JS_TRUE;
}



static JSBool
global_EvaluateScript(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);
	CWLJSScript::WhenToRunScript whenToRunScript;
  int when = 0;

  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD == NULL) {
    WLErrorException e(LoadExceptionFrmFile, 230, WLJSNAME_EVALUATE_SCRIPT);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
  if (!(pMSD->InInitAgenda())) {
    WLErrorException e(LoadExceptionFrmFile, 230, WLJSNAME_EVALUATE_SCRIPT);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  if (argc != 2) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_EVALUATE_SCRIPT, 2);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  const char* script = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));

	if (!JSVAL_IS_INT(argv[1])) {
		WLErrorException e(LoadExceptionFrmFile, 255, WLJSNAME_EVALUATE_SCRIPT, 2);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	} 
	else {
    whenToRunScript = (CWLJSScript::WhenToRunScript )JSVAL_TO_INT(argv[1]);
    if (whenToRunScript < CWLJSScript::WLAfterInitAgenda || 
				whenToRunScript > CWLJSScript::WLAfterTerminateAgenda) {
      WLErrorException e(LoadExceptionFrmFile, 255, WLJSNAME_EVALUATE_SCRIPT, 2);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
		}				
	}

	pMSD->AppendStringForEvaluation(script,whenToRunScript);
  return JS_TRUE;
}

static JSBool
global_GCYield(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);
  return JS_TRUE;
}

static JSBool
global_GC(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);
	JS_GC(cx);
  return JS_TRUE;
}


static JSBool
global_LoadSize(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);
  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    *rval = INT_TO_JSVAL(tis->GetAgendaNumRunning());
	}
  return JS_TRUE;
}

static JSBool global_WLJSAtomicIntCounter(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	int ArgList[ 5 ];
	for( uintN ArgI = 0; ArgI < argc; ++ArgI ) {
		int32 Arg;
		if( !JS_ValueToInt32( cx, argv[ArgI], &Arg ) ) {
			return LoadReport( cx, WLLogMessage::Fatal, "AtomicIntCounter parameter must be integer." );
		}
		ArgList[ ArgI ] = Arg;
	}

	intN Counter = RVJS_AtomicIntCounter( ArgList );
	*rval = INT_TO_JSVAL( Counter );
	return JS_TRUE;
}

static JSBool
global_LoadGeneratorNo(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);
  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    *rval = STRING_TO_JSVAL(JS_NewStringCopyZ(cx,tis->GetLoadGeneratorNo()));
  }
  else {
    IThreadsManager *pTM = getCurrentThreadsManager();
    if (pTM) *rval = STRING_TO_JSVAL(JS_NewStringCopyZ(cx,pTM->GetLoadGeneratorNo()));
  }
  return JS_TRUE;
}

static JSBool
global_ErrorCounter(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);
  WlAssert(dynamic_cast<iLEAgendaJSEmbedContext*>(iLEJSEmbedContext::fsGet(cx)));
  iLEAgendaJSEmbedContext* aAgendaEmbedContext= static_cast<iLEAgendaJSEmbedContext*>(iLEJSEmbedContext::fsGet(cx));

  *rval = INT_TO_JSVAL(aAgendaEmbedContext->GetErrorCounter());

  return JS_TRUE;
}

static JSBool
global_WarningCounter(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);
  iLEAgendaJSEmbedContext* aAgendaEmbedContext= static_cast<iLEAgendaJSEmbedContext*>(iLEJSEmbedContext::fsGet(cx));

  *rval = INT_TO_JSVAL(aAgendaEmbedContext->GetWarningCounter());

  return JS_TRUE;
}

static JSBool
global_SpawningFactor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);
	*rval = INT_TO_JSVAL(GetMaxThreadsPerSlave());
  return JS_TRUE;
}

static JSBool
global_AgendaName(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);
  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
    *rval = STRING_TO_JSVAL(JS_NewStringCopyZ(cx,tis->GetAgendaName()));
  }
  else {
    IThreadsManager *pTM = getCurrentThreadsManager();
    if (pTM) *rval = STRING_TO_JSVAL(JS_NewStringCopyZ(cx,pTM->GetAgendaName()));
  }
  return JS_TRUE;
}

static JSBool global_ThreadUniqueId(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);
  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
		char UniqueId[100];
		sprintf(UniqueId,"%s.%d.%d",tis->GetAgendaName(),tis->GetLGThreadNum(),tis->TotalRoundNum());
    *rval = STRING_TO_JSVAL(JS_NewStringCopyZ(cx,UniqueId));
	}
  else {
    IThreadsManager *pTM = getCurrentThreadsManager();
    if (pTM) *rval = STRING_TO_JSVAL(JS_NewStringCopyZ(cx,pTM->GetAgendaName()));
  }
  return JS_TRUE;
}




static JSBool
global_SyntaxCheckMode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {

  YieldRequest(cx);
  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD == NULL) {
    WLErrorException e(LoadExceptionFrmFile, 230, WLJSNAME_SYNTAX_CHECK_MODE);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
  if (!(pMSD->InInitAgenda())) {
    WLErrorException e(LoadExceptionFrmFile, 230, WLJSNAME_SYNTAX_CHECK_MODE);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_SYNTAX_CHECK_MODE, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  bool mode;
  if (!JSVAL_IS_BOOLEAN(argv[0])) {
    WLErrorException e(LoadExceptionFrmFile, 224, WLJSNAME_SYNTAX_CHECK_MODE, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
  else
    mode = (JSVAL_TO_BOOLEAN(argv[0])==JS_TRUE);

  //Set global variables
  CLEVariablesData::SetDoSyntaxChecking (cx, mode);

  return JS_TRUE;
}

bool  gTraceModeFirstCall = true;

static JSBool
global_TraceMode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {

  YieldRequest(cx);
//  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
//  if (pMSD == NULL) {
//    WLErrorException e(LoadExceptionFrmFile, 230, WLJSNAME_TRACE_MODE);
//		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
//  }

//  if (!(pMSD->InInitAgenda())) {
//    WLErrorException e(LoadExceptionFrmFile, 230, WLJSNAME_TRACE_MODE);
//		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
//  }

  if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_TRACE_MODE, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  bool mode;
  if (!JSVAL_IS_BOOLEAN(argv[0])) {
    WLErrorException e(LoadExceptionFrmFile, 224, WLJSNAME_TRACE_MODE, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
  else
    mode = (JSVAL_TO_BOOLEAN(argv[0])==JS_TRUE);

  //Change trace mode and make it trace into file.
  CTraceMsg::SetTrace (mode, true) ;

  //Display a warning message that the trace is ON
  IThreadsManager *pTM = getCurrentThreadsManager();
  if (mode && pTM && gTraceModeFirstCall) {
    gTraceModeFirstCall = false;
    WLErrorException e(LoadExceptionFrmFile, 256);
    return LoadReport( cx, e ) ? JS_TRUE : JS_FALSE;
  }
  return JS_TRUE;
}

static JSBool
global_getScope(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {
  YieldRequest(cx);
	JSObject* scope;
  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
		scope = tis->m_MyJSThread.GetObject();
	}
  else {
		IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
		scope = pMSD->GetJSScript().GetObject();
  }
	*rval = OBJECT_TO_JSVAL(scope);
  return JS_TRUE;
}

#define MAX_READ_SIZE		    	8192
static JSBool global_ReadFile(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval){

  if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_READ_FILE, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

//Galtech
  RVCString argumentFileName = ANSIGetStringBytes(JS_ValueToString(cx, argv[0]));

	const char* filename;
	char fullPath[WL_MAX_PATH];
	strcpy(fullPath, CArrangeMachineTempFiles::GetWlTempFullPath(cx ,WLECopiedFile, argumentFileName).data());
	filename = fullPath;


  RVCString data;
  char dataChunk[MAX_READ_SIZE];
  int counter = 0;
  long pos = 0;

#if defined(__PPC750__) || defined(__PPC405__) || defined(__TV_linux__)
  ifstream fd;
  fd.open(filename,ios::in | ios::binary);
#else
  int mode;
  mode = ios::in | ios::nocreate | ios::binary;
  ifstream fd(filename,mode);
#endif


  //if ( fd.tellg() < 0 )
  pos = fd.tellg();
  if ( pos < 0 )
  {
    jsval v = STRING_TO_JSVAL(JS_NewStringCopyZ(cx,""));
	  *rval = v;
    return JS_TRUE;
  }
#if !defined(__TV_solaris__)
  if (fd.is_open())
#else
  if (fd)
#endif
  {
    while(fd.read(dataChunk,MAX_READ_SIZE))
    {
      pos = fd.tellg();
      data.append(dataChunk,pos-counter*MAX_READ_SIZE);
      counter++;
    }
    pos = fd.tellg();
    data.append(dataChunk,pos-counter*MAX_READ_SIZE);

    fd.close();
  }

  //Galtech
#ifndef _ANSI_ONLY
  int nLen = ANSIToUTF8(GetMonitorCharSet(), data, -1, NULL, 0);
  char *utfdata = new char[nLen + 1];
  ANSIToUTF8(GetMonitorCharSet(), data, -1, utfdata, nLen);

  jsval v = STRING_TO_JSVAL(JS_NewStringCopyN(cx,utfdata,strlen(utfdata)));
	delete [] utfdata;
#else
  jsval v = STRING_TO_JSVAL(JS_NewStringCopyN(cx,data.data(),data.length()));
#endif

	*rval = v;

	return JS_TRUE;
}

CWLStatGenMeasurementDictionary* getCurrentStatGenMeasurementDictionary(JSContext *cx){
  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
	if (pMSD){
		CLoadManagerScriptData* pLMSD;
		if ((pLMSD = dynamic_cast<CLoadManagerScriptData*> (pMSD))){
			CLoadThreadsManagerScript* pLTMS = pLMSD->GetLoadThreadsManagerScript();
			if (pLTMS){
				return pLTMS->getCurrentStatGenMeasurementDictionary();
			}
		}
	}
	else {
	  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);
		if (tis){
			CLoadThreadInstanceScript* ltis = dynamic_cast<CLoadThreadInstanceScript*>(tis);
			if (ltis){
				CLoadThreadsManagerScript* pLTMS = ltis->m_MyThreadsManagerScript;
				if (pLTMS){
					return pLTMS->getCurrentStatGenMeasurementDictionary();
				}
			}
		}
	}
	return NULL;
}

static JSBool global_getStatistic(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval){
  YieldRequest(cx); 

  if (argc != 2) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_GET_STATISTIC, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

	if (!JSVAL_IS_INT(argv[1])) {
    WLErrorException e(LoadExceptionFrmFile, 226, WLJSNAME_GET_STATISTIC, 2);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
  else {
    int type = JSVAL_TO_INT(argv[1]);

		CWLStatGenMeasurementDictionary* aDic = getCurrentStatGenMeasurementDictionary(cx);

		if (aDic){
			const char* name = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));

			switch (type){
			case WLJSNAME_SESSION_AVERAGE_VALUE:
				*rval = DOUBLE_TO_JSVAL(aDic->GetAverage(name));
				break;
			case WLJSNAME_SESSION_MINIMUM_VALUE:
				*rval = DOUBLE_TO_JSVAL(aDic->GetMinimum(name));
				break;
			case WLJSNAME_SESSION_MAXIMUM_VALUE:
				*rval = DOUBLE_TO_JSVAL(aDic->GetMaximum(name));
				break;
			case WLJSNAME_SESSION_COUNT_VALUE:
				*rval = DOUBLE_TO_JSVAL(aDic->GetCount(name));
				break;
			case WLJSNAME_LAST_SLICE_AVERAGE_VALUE:
				*rval = DOUBLE_TO_JSVAL(aDic->GetLastsAverage(name));
				break;
			case WLJSNAME_LAST_SLICE_MINIMUM_VALUE:
				*rval = DOUBLE_TO_JSVAL(aDic->GetLastsMinimum(name));
				break;
			case WLJSNAME_LAST_SLICE_MAXIMUM_VALUE:
				*rval = DOUBLE_TO_JSVAL(aDic->GetLastsMaximum(name));
				break;
			case WLJSNAME_LAST_SLICE_COUNT_VALUE:
				*rval = DOUBLE_TO_JSVAL(aDic->GetLastsCount(name));
				break;
			}
		}
		else{
			*rval = INT_TO_JSVAL(-1);
		}
	}

	return JS_TRUE;
}

static JSBool
global_FileExist(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if ((argc != 1))
	{
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_FILE_EXIST_FUNC, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}
		
	if (!JSVAL_IS_STRING(argv[0])) {
    WLErrorException e(LoadExceptionFrmFile, 222, WLJSNAME_FILE_EXIST_FUNC, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}
	const char* fileName = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
	TVEnvContainerClass tvpc;
	int rv = (int) tvpc.TVPFileExist(fileName);

	*rval = INT_TO_JSVAL(rv);

	return JS_TRUE;
}

static JSBool
global_Mkdir(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if ((argc != 1))
	{
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_MKDIR_FUNC, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}
		
	if (!JSVAL_IS_STRING(argv[0])) {
    WLErrorException e(LoadExceptionFrmFile, 222, WLJSNAME_MKDIR_FUNC, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

  const char* dirName = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
  int rv = (int) TVPCreateDirectory(dirName);
	*rval = INT_TO_JSVAL(rv);
	return JS_TRUE;
}

static JSBool
global_GetEnv(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if ((argc != 1))
	{
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_GET_ENV_FUNC, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}
		
	if (!JSVAL_IS_STRING(argv[0])) {
    WLErrorException e(LoadExceptionFrmFile, 222, WLJSNAME_GET_ENV_FUNC, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

  const char* envVar = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
  const char* str = getenv(envVar);
	if (!str)
	  str = "";
	JSString *jsStrVal = JS_NewStringCopyN(cx, str, strlen(str));
	*rval = STRING_TO_JSVAL(jsStrVal);

	return JS_TRUE;
}

static JSBool
global_GetCwd(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 0) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_GET_CWD_FUNC, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

	TVEnvContainerClass tvpc;
	const char* str = tvpc.TVPGetCwd();
	if (!str)
	  str = "";
	JSString *jsStrVal = JS_NewStringCopyN(cx, str, strlen(str));
	*rval = STRING_TO_JSVAL(jsStrVal);

	return JS_TRUE;
}

static JSBool global_JSHashTableMeters(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	JSBool rtn = JS_TRUE;
	uint32 aMeterN(30);
	uint32 aMeters[30];
	const char** aMeterNames = RVJS_HashTableMeterNames();

	RVJS_HashTableMeter( cx, 0, &aMeterN, aMeters );

	RVCString aMsg("");
	{for( int aI = 0; aI < aMeterN; ++aI ) {
		char aBuf[100];
		sprintf( aBuf, "%d", (int)aMeters[aI] );
		aMsg += aBuf;
		aMsg += ", ";
	}}

	JSString* aStr = JS_NewStringCopyN(cx, aMsg.data(), aMsg.length());
	WlAssert(aStr);
	*rval = STRING_TO_JSVAL(aStr);
		
	return rtn;
}


void JSStateInfoObject( JSContext* rCx, JSObject* rObj, RVCString& o )
{
	char aBuffer[200];
	sprintf( aBuffer, "%p", rObj );

	o += aBuffer;

	JSClass* aJSClass = JS_GetClass( rCx, rObj );
	void* aPrivate = JS_GetPrivate( rCx, rObj );

	o += aPrivate ? ":" : ":^";

	o += aJSClass->name;
}

static JSBool global_JSStateInfo(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	JSBool rtn = JS_TRUE;
	RVCString aMsg("");

	JSStateInfoObject( cx, obj, aMsg );

	aMsg += "( ";
	{for( int aI = 0; aI < argc; ++aI ) {
		JSStateInfoObject( cx, (JSObject*)argv[aI], aMsg );
		aMsg += " ";
	}}
	aMsg += ")";
		
	aMsg += "[ ";
	JSObject* aScopeChain[100];
	RVJS_StateInfo( cx, aScopeChain );
	{for( int aI = 0; aScopeChain[aI]; ++aI ) {
		JSStateInfoObject( cx, (JSObject*)aScopeChain[aI], aMsg );
		aMsg += " ";
	}}
	aMsg += "]";
		


	JSString* aStr = JS_NewStringCopyN(cx, aMsg.data(), aMsg.length());
	WlAssert(aStr);
	*rval = STRING_TO_JSVAL(aStr);
		
	return rtn;
}




static JSBool
global_EnterCriticalSection(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {

  YieldRequest(cx);
	
  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
		tis->fEnterCriticalSection();
//	  LoadReport( cx, WLLogMessage::Note, "Entering critical section" );
  }
	return JS_TRUE;
}

	

static JSBool
global_ExitCriticalSection(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {

  YieldRequest(cx);
	
  CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);

  if(tis) {
		tis->fExitCriticalSection();
//	  LoadReport( cx, WLLogMessage::Note, "Exiting critical section" );
    if (tis->isAbort()) return JS_FALSE;
  }
	return JS_TRUE;
}

static JSBool global_setDebuggerOn(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  JSD_SetDebuggerStatus(JS_TRUE);
	return JS_TRUE;
}
static JSBool global_setDebuggerOff(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  JSD_SetDebuggerStatus(JS_FALSE);
	return JS_TRUE;
}

static JSFunctionSpec global_methods[] = {
#ifdef _DEBUG
	{"printf",		global_printf,			1},
	{"write",			global_write,				1},
	{"getTime",		global_getTime,			1},
#endif
	{"setDebuggerOn",		global_setDebuggerOn,			1},
	{"setDebuggerOff",		global_setDebuggerOff,			1},
  {WLJSNAME_SLEEP_FUNC,				                    global_Sleep,					                1},
  {WLJSNAME_SLEEP_ALWAYS_FUNC,				            global_SleepAlways, 	                1},
  {WLJSNAME_SET_TIMER_FUNC,		                    global_SetTimer,      			          1},
	{WLJSNAME_DISABLE_JAVA_DEFAULT_STATISTICS_FUNC,			global_DisableJavaDefaultStatistics,	0},
	{WLJSNAME_ENABLE_JAVA_DEFAULT_STATISTICS_FUNC,			global_EnableJavaDefaultStatistics,		0},
	{WLJSNAME_IS_ABORT,																	global_IsAbort,												0},
	{WLJSNAME_SEND_TIMER_FUNC,	                    global_SendTimer,      			          1},
	{WLJSNAME_SEND_COUNTER_FUNC,	                  global_SendCounter,    			          1},
	{WLJSNAME_SEND_MEASUREMENT_FUNC,	              global_SendMeasurement,			          2},
	{WLJSNAME_VERIFICATION_END_TRANSACTION_FAILURE_EX,	global_VerificationEndTransactionFailureEx,	7},
	{WLJSNAME_VERIFICATION_END_TRANSACTION_EX,					global_VerificationEndTransactionEx,				6},
	{WLJSNAME_VERIFICATION_BEGIN_TRANSACTION_EX,				global_VerificationBeginTransactionEx,			2},
	{WLJSNAME_VERIFICATION_RECORD_EVENT_EX,							global_VerificationRecordEventEx,						8},
	{WLJSNAME_VERIFICATION_SET_FAILURE_REASON_EX,				global_VerificationSetFailureEx,			4},
  {WLJSNAME_VERIFICATION_BEGIN_TRANSACTION,       global_VerificationBeginTransaction,  1},
  {WLJSNAME_VERIFICATION_END_TRANSACTION,         global_VerificationEndTransaction,    3},
  {WLJSNAME_VERIFICATION_RECORD_EVENT,            global_VerificationRecordEvent,       2},
  {WLJSNAME_VERIFICATION_SET_FAILURE_REASON,      global_VerificationSetFailure,        2},
  {WLJSNAME_VERIFICATION_GET_CURRENT_TRANSACTION, global_VerificationGetCurrentTransaction, 0},
  {WLJSNAME_VERIFICATION_GET_TRANSACTION_DEPTH,   global_VerificationGetTransactionDepth, 0},
  {WLJSNAME_REPORT,																global_Report,      	                3},
  {WLJSNAME_INFO_MESSAGE,		                      global_Note,      	                  1},
  {WLJSNAME_TRACE_MESSAGE,		                    global_Trace,      	                  1},
	{WLJSNAME_WARNING_MESSAGE,		                  global_Warning,      	                1},
	{WLJSNAME_ERROR_MESSAGE,		                    global_Error,      	                  1},
	{WLJSNAME_SEVERE_ERROR_MESSAGE,	                global_Severe,      	                1},
	{WLJSNAME_INCLUDE,	                            global_Include,      	                1},
	{WLJSNAME_COPY_MONITOR_FILE,                    global_CopyMonitorFile,               2},
	{WLJSNAME_CREATE_JAVA_THREAD_WRAPPER,								global_CreateJavaThreadWrapper,				0},
	{WLJSNAME_COPY_MONITOR_JAVA_FILE,               global_CopyMonitorJavaFile,           2}, // @@@@ new code
	{WLJSNAME_EVALUATE_SCRIPT,                      global_EvaluateScript,                2},
	{WLJSNAME_GC_YIELD,	                            global_GCYield,      	                0},
	{"JSGC",	                            global_GC,      	                0},
	{WLJSNAME_LOAD_GENERATOR_NO,	                  global_LoadGeneratorNo,               0},
	{WLJSNAME_ERROR_COUNTER,												global_ErrorCounter,									0},
	{WLJSNAME_WARNING_COUNTER,											global_WarningCounter,								0},
	{WLJSNAME_SPAWNING_FACTOR,											global_SpawningFactor,								0},
	{WLJSNAME_AGENDA_NAME,                          global_AgendaName,                    0},
  {WLJSNAME_LOAD_SIZE,                            global_LoadSize,                      0},
  {WLJSNAME_UNDOC_LOAD_SIZE,                      global_LoadSize,                      0},
  {WLJSNAME_GET_STATISTIC,                        global_getStatistic,                  0},
  {WLJSNAME_WLJS_ATOMIC_INT_COUNTER,              global_WLJSAtomicIntCounter,          1},
	{WLJSNAME_SYNCHRONIZATION_POINT,                global_SynchornizationPoint,          0},
	{WLJSNAME_TRACE_MODE,                           global_TraceMode,                     1},
	{WLJSNAME_SYNTAX_CHECK_MODE,                    global_SyntaxCheckMode,               1},
	{WLJSNAME_READ_FILE,                            global_ReadFile,                      1},
	{WLJSNAME_UNIQUE_ID,                            global_ThreadUniqueId,                0},
	{WLJSNAME_LOCK,																	global_Lock,													0},
	{WLJSNAME_UNLOCK,		                            global_Unlock,												0},
	{WLJSNAME_ROUND_NUM_SCRIPT_IN_SLAVE,						global_GetRoundNumScriptInSlave,			0},
	{WLJSNAME_ROUND_NUM_MIX_IN_SLAVE,								global_GetRoundNumMixInSlave,					0},
	{WLJSNAME_THREAD_NUM_IN_SLAVE,									global_GetThreadNumInSlave,						0},
	{WLJSNAME_GET_THREAD_TYPE,											global_GetThreadType,									0},
  {WLJSNAME_STOPTEST,		                          global_StopTest,											1},
  {WLJSNAME_GET_SCOPE,		                        global_getScope,											0},
  {WLJSNAME_GET_OPERATING_SYSTEM,                 global_getOperatingSystem,						0},
  {WLJSNAME_GET_WORKINGSETSIZE,                   global_getWorkingSetSize,						  0},
  {WLJSNAME_GET_TEMP_PATH     ,                   global_getWLTempPath,   						  0},
  {WLJSNAME_GET_PROTOCOL_NAMES,                   global_getProtocolNames, 						  0},
	{WLJSNAME_FILE_EXIST_FUNC,									    global_FileExist,											1},
	{WLJSNAME_MKDIR_FUNC,				                    global_Mkdir,											    1},
	{WLJSNAME_GET_ENV_FUNC,                         global_GetEnv,											  1},
  {WLJSNAME_GET_CWD_FUNC,                         global_GetCwd,											  0},
  {WLJSNAME_ENTER_CRITICAL_SECTION,               global_EnterCriticalSection,  			  0},
  {WLJSNAME_EXIT_CRITICAL_SECTION,                global_ExitCriticalSection,					  0},
  {"UNDOC_JSHashTableMeters", global_JSHashTableMeters, 0},
	{0}
};

static JSFunctionSpec global_methods_proto[] = {
  {"UNDOC_JSStateInfo", global_JSStateInfo, 0},
	{0}
};

bool WLJSAddGlobalFunctions(JSContext* cx, JSObject* obj, JSObject* proto )
{
	{for (int i = 0; global_methods[i].name; i++) {
		if (!JS_DefineFunction(cx, obj, global_methods[i].name, global_methods[i].call,
									global_methods[i].nargs, global_methods[i].flags)) {
			return false;
		}
	}}

	{for (int i = 0; global_methods_proto[i].name; i++) {
		if (!JS_DefineFunction(cx, proto, global_methods_proto[i].name, global_methods_proto[i].call,
									global_methods_proto[i].nargs, global_methods_proto[i].flags)) {
			return false;
		}
	}}


	return true;
}

