/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : jsRand.cpp
 *
 *	Author : Zion Menahem
 *
 *	Description : 
 *
 *	History : 
 *
 *	Modifications : 
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/

/** include files **/
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <RVCString.h>

#include "rvjsapi.h"

#ifdef __TV_UNIX__
#include "jsdbgapi.h"
#else
//#include "jsdbgapi.h"
#endif

#include "loadjsdefs.h"
#include "wljsobjects.h"
#include "loadthreadsmanager.h"
#include "wlerrorlib.h"
#include "loadexceptions.h"
#include "loadengineglobals.h"
#include "loadreportinfo.h"

/** local definitions **/
//#define _round(x) (int)(x-floor(x) < 0.5 ? floor(x) : ceil(x))

/** default settings **/

/** external functions **/

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/

/** classes definitions **/

/** public functions **/

/** private functions **/



/*********************************************************************\

	Rand Object

\*********************************************************************/

JSClass js_RandClass = {
    WLJSNAME_RAND_CLASS,
    0,
    JS_PropertyStub,	// add Property
		JS_PropertyStub,	// del Property
		JS_PropertyStub,	// get Property
		JS_PropertyStub,	// set Property
    JS_EnumerateStub,	
		JS_ResolveStub,
		JS_ConvertStub,
		JS_FinalizeStub
};

static JSBool
rand_Seed(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_RAND_OBJECT"."WLJSNAME_RAND_SEED, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
    
  int32 seed;
  if (JS_ValueToInt32(cx, argv[0], &seed) == JS_FALSE) {
		WLErrorException e(LoadExceptionFrmFile, 223, WLJSNAME_RAND_OBJECT"."WLJSNAME_RAND_SEED, 1 );
    return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
		}

	srand((unsigned)seed);

	return JS_TRUE;
}

static JSBool
rand_Num(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  if ( (argc != 0) && (argc != 1) ) {
    WLErrorException e(LoadExceptionFrmFile, 221, WLJSNAME_RAND_OBJECT"."WLJSNAME_RAND_NUM, 0, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  if (argc > 0) {
    int32 seed;
    if (JS_ValueToInt32(cx, argv[0], &seed) == JS_FALSE) {
			WLErrorException e(LoadExceptionFrmFile, 223, WLJSNAME_RAND_OBJECT"."WLJSNAME_RAND_NUM, 1 );
			return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
			}
		

	  srand((unsigned)seed);
  }

  // rand() ranges change between OS (the manual says 15bit random number).
	*rval = INT_TO_JSVAL((rand()&0x7fff));

	return JS_TRUE;
}

static JSBool
rand_Range(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  if ( (argc != 2) && (argc != 3) ) {
    WLErrorException e(LoadExceptionFrmFile, 221, WLJSNAME_RAND_OBJECT"."WLJSNAME_RAND_RANGE, 2, 3);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

  int32 start;
  int32 end;

  if (JS_ValueToInt32(cx, argv[0], &start) == JS_FALSE) {
		WLErrorException e(LoadExceptionFrmFile, 223, WLJSNAME_RAND_OBJECT"."WLJSNAME_RAND_RANGE, 1 );
		return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
		}

  if (JS_ValueToInt32(cx, argv[1], &end) == JS_FALSE) {
		WLErrorException e(LoadExceptionFrmFile, 223, WLJSNAME_RAND_OBJECT"."WLJSNAME_RAND_RANGE, 2 );
		return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
		}

	if (start > end) {
		int32 temp = start;
		start = end;
		end = temp;
	}

  if (argc == 3) {
    int32 seed;
    if (JS_ValueToInt32(cx, argv[2], &seed) == JS_FALSE)
      return JS_FALSE;

	  srand((unsigned)seed);
  }

	// create a mapping from (0:RAND_MAX) to (start:end)
//my function  int new_rand = (int)floor(((double)rand() / (double)(RAND_MAX+1)) * (double)(end-start+1)) + start;
//	int new_rand = _round((double)(end-start) / (double)RAND_MAX * (double)rand() + (double)start);
  int new_rand = (rand() % (end-start+1)) + start; // no matter what range rand() returns
  *rval = INT_TO_JSVAL(new_rand);

	return JS_TRUE;
}

static JSBool
rand_Select(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  if (argc == 0) {
    WLErrorException e(LoadExceptionFrmFile, 227, WLJSNAME_RAND_OBJECT"."WLJSNAME_RAND_SELECT);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

  // create a mapping from (0:RAND_MAX) to (0:argc-1)
// my function  int indx = (int)floor(((double)rand() / (double)(RAND_MAX+1)) * (double)argc);
  int indx = (rand() % argc); // no matter what range rand() returns

//	int indx = _round((double)(argc-1) / (double)RAND_MAX * (double)rand());
	*rval = argv[indx];

	return JS_TRUE;
}

static JSBool
rand_valueOf(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  return rand_Num(cx, obj, argc, argv, rval);
}

static JSBool
rand_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  *rval = JSVAL_VOID;
	return JS_TRUE;
}

static JSFunctionSpec rand_methods[] = {
	{WLJSNAME_TO_STRING,			rand_toString,	0},
	{WLJSNAME_VALUE_OF,				rand_valueOf,		1},
	{WLJSNAME_RAND_SEED,			rand_Seed,			1},
	{WLJSNAME_RAND_NUM,				rand_Num,				1},
	{WLJSNAME_RAND_RANGE,			rand_Range,			3},
	{WLJSNAME_RAND_SELECT,		rand_Select,		9},
	{0}
};

JSObject*
js_AddRandObject(JSContext *cx, JSObject *obj)
{
	JSObject* wlRand = JS_DefineObject(cx, obj, WLJSNAME_RAND_OBJECT, &js_RandClass, NULL, JSPROP_PERMANENT);
	if (!wlRand)
		return NULL;

	// define it's methods
	if (!JS_DefineFunctions(cx, wlRand, rand_methods))
		return NULL;

	return wlRand;
}

