/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "tracedll.h"
#include "send2tml.h"
#include "wlerrorlib.h"
#include "loadengineselistener.h"
#include "tvpnumconversions.h"
#include "associativearray.h"
#include "loadjsshell.h"
#include "loadenginestatistics.h"
#include "loadengineglobals.h"
#include "loadgeneratorslave.h"
#include "loadthreadinstance.h"
#include "loadthreadsmanager.h"
#include "loadexceptions.h"
#include "loadreportinfo.h"
#include "rtamscript.h"
#include "loadengineinifileproperties.h"

#include "wlleprotocol.h"

#include <time.h>

#include "loadgeneratorscript.h"
#include "Log.h"

//////////////////////////////////////////////////////////////
//
//  The implementation of the class CLoadThreadInstance
//
//////////////////////////////////////////////////////////////

CLoadThreadInstance::CLoadThreadInstance(CLoadThreadsManager  *agenda, int id, CGeneralProcess& rProcess )
:
	eiRVInheritArc< CLoadThreadInstance, RVCollectableInt >(id),
iLEJSAgendaMachineThread( rProcess ),
m_ThreadsManager(agenda),
m_CurrentScript(NULL),
m_PreviousScript(NULL),
m_SingleTest(false),
m_FirstTimeSelectingScript(true),
m_jsj_thread(NULL),
m_ScriptContainer(&(agenda->GetScriptManager())),
m_GlobalsRequestHandler(NULL),
m_InitDone(false),
m_SevereErrorOccurred(false)
{
  m_LogMutex = new RVCriticalSection;
  m_ErrorSeverity = WLLogMessage::Success;
  //jsThread.Init(&(agenda->jsScript), value());
}

CLoadThreadInstance::~CLoadThreadInstance()
{
  delete m_LogMutex;
  delete m_GlobalsRequestHandler;

}

int CLoadThreadInstance::GetLGThreadNum() 
{ 
  return GetSlaveThreadNum() + m_ThreadsManager->m_pMyLoadGeneratorSlave->m_SlaveBaseThreadId; 
}


unsigned long CLoadThreadInstance::GetRoundNumScriptInSlave() {
  return m_RoundNumScriptInSlave;
}

unsigned long CLoadThreadInstance::GetRoundNumMixInSlave() {
  return m_RoundNumMixInSlave;
}



bool CLoadThreadInstance::Init() 
{
  WlAssert (!m_InitDone);
  m_InitDone = true;

  WlTraceMsg("CLoadThreadInstance::Init(). Thread num : %d, started",value());
  m_GlobalsRequestHandler = new CThreadGlobalsRequestHandler(this); 
  bool ok = m_ScriptContainer.Init(this);
  if (!ok) return false;

	//initialize 'm_ProtocolsMachineThread'

	// @@Chaim - need this?
	CLoadThreadsManager* CLTM = GetThreadsManager();
	CLoadGeneratorSlave* CLGS = CLTM->m_pMyLoadGeneratorSlave;

  WlTraceMsg("CLoadThreadInstance::Init(). Thread num : %d, Done.",value());

  return true;
}



WLEThreadType CLoadThreadInstance::GetThreadType() {
  if( IsOneClient() ) {
    return WLEThreadTypePC;
  } else {
    return WLEThreadTypeLE;
  }
}


int CLoadThreadInstance::run(RVCString& errMessage)
{
  //  WlTraceMsg("CLoadThreadInstance::run - Running thread %d", value());
  //
  // create the running thread functor:
#if !defined(RV_THR_CANT_EXTRACT_TEMPLATE_ARGS_FROM_SIGNATURE)            
  m_WorkThread = rvMakeThreadFunction(
    *this,                                // instance
    &CLoadThreadInstance::mainLoop        // member function
    );
#else
  m_WorkThread = rvMakeThreadFunctionM( 
    CLoadThreadInstance                   // class
    *this,                                // instance
    void,                                 // return value
    &CLoadThreadInstance::mainLoop        // member function
    );
#endif
  
  m_WorkThread.start();
  return 0;
  //return sys_thr.create(GlobalThreadMainLoop, this, 0);
}

void CLoadThreadInstance::stop(){
  if ( m_WorkThread.getExecutionState() != RV_THR_INITIAL &&
       m_WorkThread.join(8000) == RV_THR_TIMEOUT ) 
  {
    try 
    {
      m_WorkThread.terminate();
    }
    catch (...) {}
  }
}


#define WLE_THREAD_RUN 1
#define WLE_THREAD_TERMINATE 2
#define WLE_THREAD_PAUSE 3

void CLoadThreadInstance::mainLoop()
{
	TIMERENTRY("CLoadThreadInstance::mainLoop");
	//call the protocol callback for each machine thread
	CLoadThreadsManager* CLTM = GetThreadsManager();
	CLoadGeneratorSlave* CLGS = CLTM->m_pMyLoadGeneratorSlave;
	iLEJSAgendaMachineThread::fInit(THREAD_TYPE_VC);

  m_ThreadSelf = ::rvThread();

  WlTraceMsg("CLoadThreadInstance::mainLoop - thread %d started", value());

	srand(value() + (unsigned)time( NULL ) );
	rand();//in order to skip the first random number (because it happened 
	       // that all first numbers of each thread was close )
	
  if(!WlTrace) {
    try {
      mainLoopFunc();
			WlTraceMsg("after mainLoopFunc");
    } catch (...) {
      WlTraceMsg("Fatal error in CLoadThreadInstance::mainLoop(thread=%d)", value());
      #ifdef _DEBUG
      throw;
      #endif
    }
  }
  else {
    mainLoopFunc();
    WlTraceMsg("after mainLoopFunc");
  }

  
  WlTraceMsg("CLoadThreadInstance::mainLoop - thread %d Terminated", value());

}

void CLoadThreadInstance::mainLoopFunc()
{
  int err;
  int status = WLE_THREAD_RUN;

  //
  // setting the current thread instance for use
  //setCurrentThreadInstance((CLoadThreadInstance *)this);
  TVPSafeCounter::IncrementGuard AliveThread(m_ThreadsManager->m_AliveThreadsCounter);
  
  //RVThreadSelf self = ::rvThread();
  //self.setPriority(self.getPriority()-1);
  //m_ScriptContainer.Init(this);
  
  while (1) {
    WlTraceMsg("In Main loop. Thread num : %d - numRunning: %d",value(),m_ThreadsManager->m_NumRunning);
    if (value() < m_ThreadsManager->m_NumRunning) {

			// @@Chaim - try moving this to closer to activation of load loop
			//					 I'm not even sure it can be run here
      if (!m_InitDone) {
        static RVCriticalSection* InitMutex = new RVCriticalSection;
        RVCriticalSection::LockGuard lock(*InitMutex);
        if (!Init()) {
          report(WLLogMessage::Fatal, "Internal Error : Fatal error occurred while initializing thread."); 
          return;
        }
      }
      WlTraceMsg("In Main loop. Thread num : %d - Starts working",value());
      status = WLE_THREAD_RUN;
      m_ThreadsManager->IncrementTreatedThreads();
      WlTraceMsg("In Main loop. Thread num : %d - Starts working, Before WaitToStart",value());
      m_ThreadsManager->WaitToStart();
      if (!isAbort(false)) {
        //TVPSysThread::setPriority(TVPSysThread::Minimal);
        WlTraceMsg("In Main loop. Thread num : %d - Entering LoadLoop",value());
        err = loadLoop();
        //TVPSysThread::setPriority(TVPSysThread::Normal);
        WlTraceMsg("In Main loop. Thread num : %d - After loadLoop. Err=%d",value(),err);
        if (err == -1)
          while (!isAbort(false)) {
            WlTraceMsg("Waiting to abort. Thread %d",value());
            sleep(20000);
          }
      }
      else
        WlTraceMsg("In Main loop. Thread num : %d - IsAbort on",value());
      
    }
    else if (m_ThreadsManager->IsTerminate()){
      WlTraceMsg("In Main loop. Thread num : %d - Terminate",value());
      // m_ThreadsManager->IncrementTreatedThreads(); // done after terminations
      status = WLE_THREAD_TERMINATE;
      break;
    }
    else{

      if (status != WLE_THREAD_PAUSE){
        WlTraceMsg("In Main loop. Thread num : %d - pause",value());
        m_ThreadsManager->IncrementTreatedThreads();
        WlTraceMsg("In Main loop. Thread num : %d - pause, after IncrementTreatedThreads()",value());
        status = WLE_THREAD_PAUSE;
      }
      if (value() >= m_ThreadsManager->m_NumRunning) {
        WlTraceMsg("In Main loop. Thread num : %d - pause, WaitToStart",value());
        //m_ThreadsManager->WaitToStart();
//        m_ThreadsManager->m_ThreadsStartEvent.Wait(2000);
        m_ThreadsManager->m_ThreadsStartEvent.Wait();
      }

      WlTraceMsg("In Main loop. Thread num : %d - pause, WaitToReStart",value());

      m_ThreadsManager->WaitToRestart();
      WlTraceMsg("In Main loop. Thread num : %d - pause, After WaitToReStart",value());

  	if(! m_ScriptContainer.RestartJSThreads() ) {
	  	WlTraceMsg("Error in the call to function 'RestartJSThreads' while calling from the function 'CLoadThreadInstance::mainLoop'");
    }
    }
  }

	if(! m_ScriptContainer.TerminateJSThreads() ) {
		WlTraceMsg("Error in the call to function 'TerminateJSThreads' while calling from the function 'CLoadThreadInstance::mainLoop'");
  }
  
	Terminate();
  iLEJSMachineThread::fFinal();
	m_ThreadsManager->IncrementTreatedThreads(); //@@Chaim (inherited from Itaj): move finalization insdide mainLoopFunc()
}

bool CLoadThreadInstance::DoNextCommand()
{
  m_PreviousScript = m_CurrentScript;
  m_CurrentScript = m_ScriptContainer.SelectScript(this,m_PreviousScript, m_RoundNumScriptInSlave, m_RoundNumMixInSlave );
  m_ErrorSeverity = WLLogMessage::Success;

    bool rtn;

  {
		// @@ChaimTBD - need to do this just once for all scripts in start of main loop - later
		bool runStatus = m_CurrentScript->ThreadActivation();
  	cLEJSEmbedContextControlMachineThread aGuard( *m_CurrentScript );
    runStatus = runStatus && m_CurrentScript->runJScript(); 

    bool AfterRoundStatus = true;

    if (!runStatus){
      m_CurrentScript->m_TransactionManager.SendFailureForOpenTransactions(true, true );
      m_CurrentScript->OnScriptAbort();
    }
    else {
      m_CurrentScript->m_TransactionManager.SendFailureForOpenTransactions(false, true);
    }

// @@Chaim
//    m_ThreadSelf.yield();

    rtn = runStatus;
  }
  
  return rtn;
}

int CLoadThreadInstance::loadLoop()
{
  int err = 0;
  
  WlTraceMsg("CLoadThreadInstance::loadLoop() - starting");
  
  try { LE_SET_SE_TRANSLATOR();
			
		err = loadLoopFunc();
  } 
  catch (CWLGeneratorTerminatedException *) {
		err = -1;
  }
  catch (std::exception* e) {
		ExceptionHandler(e);
    err = -1;
		delete e; // Only for catch of pointer 
  }
  catch (std::exception& e) {
		ExceptionHandler(&e);
    err = -1;
  }
  catch (...) {
    report(WLLogMessage::Fatal, "Internal Error : Fatal error occurred while performing Load");
    err = -1;
#ifdef _DEBUG
    throw;
#endif
  }
  
  m_ScriptContainer.ReleaseThreadsManagerScript(m_CurrentScript);
  
  m_ScriptContainer.TerminateClient();

  // AMDOCS - moved from before TerminateClient to support http stuff in TerminateClient
  m_CurrentScript = NULL;
  
  return err;
}

void CLoadThreadInstance::ExceptionHandler(std::exception* e){
	try {
		if (m_CurrentScript)
			m_CurrentScript->ProtocolExceptionHandler(*e);

  // Check if WLErrorException and if reported
  WLErrorException* aException;

  if (aException = dynamic_cast<WLErrorException*>(e)){
    if (aException->GetReported()){
      return; // Already reported, do nothing
    }
  }

	RVCString FullExcName ("");
	WLERRGetErrorMsg(e, FullExcName);
	RVCString StackTrace("");
	if (WLERRGetStackTrace(e, StackTrace) && StackTrace.length() > 0){
	  WlErrorMsg("CLoadThreadInstance::loadLoop - Exception %s occured\nTrace:\n%s", FullExcName, StackTrace);
	  printf("CLoadThreadInstance::loadLoop - Exception %s occured\nTrace:\n%s", FullExcName, StackTrace);
	}
	else {
	  WlErrorMsg("CLoadThreadInstance::loadLoop - Exception %s occured", FullExcName);
	  printf("CLoadThreadInstance::loadLoop - Exception %s occured", FullExcName);
	}
  report(WLLogMessage::Fatal, FullExcName.data());
}
	catch (...){
		// If anything failed, just do StopMe()
		m_ThreadsManager->GetLoadGeneratorSlave()->StopMe();
	}
}

int CLoadThreadInstance::loadLoopFunc()
{
  int err = 0;
  
  while (!isAbort(false)) {
    if (m_ThreadsManager->m_ErrorFlag.Get()){
      err = -1;
      break;
    }
    
    DoNextCommand();
    
    if (isAbort(false)){
      err = -1;
      break;
    }
  }
  
  return err;
}

bool CLoadThreadInstance::internalIsAbort() {
	if (inCriticalSection())
		return false;

  if ((value() >= m_ThreadsManager->m_NumRunning) || (m_ThreadsManager->IsTerminate())) {
    WlTraceMsg("internalIsAbort. value %d m_NumRunning %d terminate %d", value(), m_ThreadsManager->m_NumRunning, m_ThreadsManager->IsTerminate());
    return true;
  }  
  
  return false;
}

bool CLoadThreadInstance::IsJSActive() {
  WlAssert(m_CurrentScript);
  return m_CurrentScript->m_MyJSThread.IsActive();
}

JSContext* CLoadThreadInstance::vCurrentJSContext() {
  WlAssert(m_CurrentScript);
  return m_CurrentScript->m_MyJSThread.GetContext();
}


bool CLoadThreadInstance::InScript()
{
  if (m_CurrentScript==NULL)
    return false;
  else
    return m_CurrentScript->InScript();
}



bool CLoadThreadInstance::IsLoadServer() {
  return m_ThreadsManager->IsLoadServer();
}

bool CLoadThreadInstance::IsOneClient() {
  return m_ThreadsManager->IsOneClient();
}

void CLoadThreadInstance::OnSevereError()
{
	m_SevereErrorOccurred = true;
	m_CurrentScript->m_MyThreadsManagerScript->OnSevereError();
  m_ThreadsManager->OnSevereError();
}

void CLoadThreadInstance::ToggleJavaDefaultStatistics( bool rValue )
{
	if (m_CurrentScript)
		m_CurrentScript->ToggleJavaDefaultStatistics( rValue );
}

bool CLoadThreadInstance::IsDisabledJavaDefaultStatistics()
{
	return m_CurrentScript->IsDisabledJavaDefaultStatistics();
}

iLEJSMachineThread& CLoadThreadInstance::vGetMachineThread()
{
	return *this;
}

IGeneralThreadTransactionManager& CLoadThreadInstance::GetTransactionManager() {
	return m_CurrentScript->m_TransactionManager;
}

void CLoadThreadInstance::endMultiThread() {
  RVCriticalSection::LockGuard lock(*m_LogMutex);
  m_CurrentScript->m_MyJSThread.SetJSContext();
}

void CLoadThreadInstance::reportApply(
	iReport& rReport ) 
{
	
	WLLogMessage::Severity aSeverity = rReport.fGetSeverity();

  if (m_ErrorSeverity < aSeverity)
    m_ErrorSeverity = aSeverity;
  
  if ( aSeverity == WLLogMessage::Fatal) {
    OnSevereError();
	}
}


void CLoadThreadInstance::reportLog(
	iReport& rReport ) 
{
	WLLogMessage::Severity aSeverity( rReport.fGetSeverity() );
	const char* aMessage = rReport.fGetMessage();
//	bool aHasTime( rReport.fHasTime() );
//	double aTime( rReport.fGetTime() );

  m_ThreadsManager->reportLog( rReport, GetLGThreadNum() );

  if (m_CurrentScript) {
		reportApplyEvent( aSeverity, aMessage );
	}
}

bool CLoadThreadInstance::reportMultiThread(
	iReportInfo& r )
{
  RVCriticalSection::LockGuard lock(*m_LogMutex);
  m_CurrentScript->m_MyJSThread.SetJSContext();

  return LoadReport( *m_CurrentScript, vCurrentJSContext(), r );
};



SynchronizationPointStatus CLoadThreadInstance::SynchronizationPoint(int timeoutmillis) {
  return m_CurrentScript->SynchronizationPoint(timeoutmillis);
}

int CLoadThreadInstance::GetAgendaNumRunning() {
  return m_ThreadsManager->GetAgendaNumRunning();
}

const char* CLoadThreadInstance::GetLoadGeneratorNo() {
  return m_ThreadsManager->GetLoadGeneratorNo();
}

int CLoadThreadInstance::GetErrorCounter() {
  return m_ThreadsManager->GetErrorCounter();
}

int CLoadThreadInstance::GetWarningCounter() {
  return m_ThreadsManager->GetWarningCounter();
}

void CLoadThreadInstance::IncrementErrorCounter() {
  m_ThreadsManager->IncrementErrorCounter();
}

void CLoadThreadInstance::IncrementWarningCounter() {
  m_ThreadsManager->IncrementWarningCounter();
}

CLoadGeneratorSlave* CLoadThreadInstance::GetLoadGeneratorSlave() {
	return m_ThreadsManager->GetLoadGeneratorSlave();
}

IGeneralGlobalsRequestHandler* CLoadThreadInstance::GetGlobalsRequestHandler() {
  return static_cast<IGeneralGlobalsRequestHandler*>(m_GlobalsRequestHandler);
} 

JSObject* CLoadThreadInstance::GetGlobalObject() {
	return m_CurrentScript->m_MyJSThread.GetObject();
} 


const char* CLoadThreadInstance::GetAgendaName() {
  return m_ThreadsManager->GetAgendaName();
}

void CLoadThreadInstance::CreateApplet(JSContext*           cx,
                                       CLoadGeneralThreadInstanceScript*
                                                            TIS,
                                       const char*          appletClassName,
                                       const char*          appletCodeBase,
                                       const char*          appletDocumentBase,
                                       RVHashDictionary*    appletParams,
                                       CWLJSJavaApplet*&    javaApplet)
{
	
	cLEJSEmbedContextControlMachineThread aGuard( *m_CurrentScript ); //@@Chaim (inherited from Itaj): i'll have to use AppletsEmbedContext
  ((CLoadThreadInstanceScript*)GetCurrentScript())->CreateApplet(
                                   cx,
                                   appletClassName,
                                   appletCodeBase,
                                   appletDocumentBase,
                                   appletParams,
                                   javaApplet);
}

bool CLoadThreadInstance::ClassAppletExists(JSContext*  cx,
                                            const char* appletClassName,
 							                              bool& classAppletExists)
{
	cLEJSEmbedContextControlMachineThread aGuard( *m_CurrentScript ); //@@Chaim (inherited from Itaj): i'll have to use AppletsEmbedContext
  return ((CLoadThreadInstanceScript*)GetCurrentScript())->ClassAppletExists(
                                               cx,
                                               appletClassName, 
                                               classAppletExists);
}

bool CLoadThreadInstance::AddJavaStream(JSContext*  cx,
                                        const char* streamName,
                                        const char* codeBase,
						        		                RVCString&  javaStream)
{
	cLEJSEmbedContextControlMachineThread aGuard( *m_CurrentScript ); //@@Chaim (inherited from Itaj): i'll have to use AppletsEmbedContext
 	return GetLoadGeneratorSlave()->AddJavaStream(cx, streamName, codeBase, javaStream);
}

bool CLoadThreadInstance::JavaStreamExists(const char* streamName)
{
	cLEJSEmbedContextControlMachineThread aGuard( *m_CurrentScript ); //@@Chaim (inherited from Itaj): i'll have to use AppletsEmbedContext
	return GetLoadGeneratorSlave()->JavaStreamExists(streamName);
}

CWLJSScript& CLoadThreadInstance::GetJSScript() {
	return GetCurrentScript()->GetJSScript();
}

CGeneralProcess& CLoadThreadInstance::fGetProcess() {
	return GetLoadGeneratorSlave()->fGetProcess();
}

iSession& CLoadThreadInstance::vGetSession() {
	return *GetLoadGeneratorSlave();
}

int CLoadThreadInstance::GetNumScriptInMix() {
	return m_CurrentScript->GetNumScriptInMix();
}

void CLoadThreadInstance::SignNewJSContext( JSContext* cx)
{
  return; // do nothing in load generator
}

bool CLoadThreadInstance::vIsUserSection()
{
	return GetCurrentScript()->m_MyJSThread.IsUserSectionActive();
}

