/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "managerscriptdata.h"
#include "loadgeneratorscript.h"
#include "loadthreadsmanager.h"
#include "wlleprotocol.h"

IGeneralManagerScriptData::IGeneralManagerScriptData() {}

IGeneralManagerScriptData::~IGeneralManagerScriptData() 
{
	m_IncludeFiles.clearAndDestroy();
}

void IGeneralManagerScriptData::reportApply(
	iReport& rReport ) 
{
	GetJSScript().reportApply( rReport );
}


bool IGeneralManagerScriptData::IncludeFile(
	const char* FileNameBytes, 
	bool Execute ) {

	RVCollectableString* FileName = new RVCollectableString(FileNameBytes);
	FileName->toLower();

	bool rtn;

	RVCollectable* temp =	m_IncludeFiles.insert( FileName );
	if( temp == static_cast<RVCollectable*>(FileName) ) { // comparing pointers to instances at the same anscestor level
		rtn = vIncludeFile_IGeneralManagerScriptData( FileNameBytes, Execute );
	} else {
		// delete FileName; // Chaim - this shouldn't be needed; FileName is inserted into 
                        // m_IncludeFiles and deleted later.
		rtn = true;
	}

	return rtn;
}

iLEProtocolJSEmbedContext& IGeneralManagerScriptData::GetProtocolJSEmbedContext(WLProtocolIndex p_ProtocolIndex){

  WlAssert(dynamic_cast<iLEProtocolAgendaInMix*>(&(m_ProtocolAgendaInMixContainer.GetProtocolAtIndex(p_ProtocolIndex))));
  iLEProtocolAgendaInMix& aProtocolAgenda = static_cast<iLEProtocolAgendaInMix&>(m_ProtocolAgendaInMixContainer.GetProtocolAtIndex(p_ProtocolIndex));
  return aProtocolAgenda;

}

void IGeneralManagerScriptData::ProtocolBeforeInitAgenda(){
	WlTraceMsg("Calling BeforeInitAgenda for all protocols");
	for(WLProtocolIndex indx=0; indx< m_ProtocolAgendaInMixContainer.NumProtocols(); indx++) {
    if (m_ProtocolAgendaInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolAgendaInMix*>(&(m_ProtocolAgendaInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolAgendaInMix&>(m_ProtocolAgendaInMixContainer.GetProtocolAtIndex(indx)).BeforeInitAgenda()){
			  WlTraceMsg("Error 1 in function IGeneralManagerScriptData::BeforeInitAgenda");
		  }
    }
  }
}

void IGeneralManagerScriptData::ProtocolAfterTerminateAgenda(){

	WlTraceMsg("Calling ProtocolAfterTerminateAgenda for all protocols");
  for (WLProtocolIndex indx = m_ProtocolAgendaInMixContainer.NumProtocols() - 1; indx >= 0; indx--){
    if (m_ProtocolAgendaInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolAgendaInMix*>(&(m_ProtocolAgendaInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolAgendaInMix&>(m_ProtocolAgendaInMixContainer.GetProtocolAtIndex(indx)).AfterTerminateAgenda()){
			  WlTraceMsg("Error 1 in function IGeneralManagerScriptData::ProtocolAfterTerminateAgenda");
		  }
    }
  }
}

void IGeneralManagerScriptData::ProtocolJSInitializeForScript(JSContext* cx, JSObject* AgendaGlobal, JSObject* GeneralGlobal, iLEProtocolContainer& pProtocolSessionContainer){
	WlTraceMsg("Calling JSInitializeForScript for all protocols");
  for (WLProtocolIndex indx = 0; indx < m_ProtocolAgendaInMixContainer.NumProtocols(); indx++){
    if (pProtocolSessionContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolAgendaInMix*>(&(m_ProtocolAgendaInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolAgendaInMix&>(m_ProtocolAgendaInMixContainer.GetProtocolAtIndex(indx)).JSInitializeForScript(
                                                                                      cx,
                                                                                      AgendaGlobal,
                                                                                      GeneralGlobal)){
        WlAssert(0);
      }
    }
  }
}

void IGeneralManagerScriptData::ProtocolJSTerminateForScript(JSContext* cx, JSObject* AgendaGlobal, JSObject* GeneralGlobal){
	WlTraceMsg("Calling JSTerminateForScript for all protocols");
  for (WLProtocolIndex indx = 0; indx < m_ProtocolAgendaInMixContainer.NumProtocols(); indx++){
    if (m_ProtocolAgendaInMixContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolAgendaInMix*>(&(m_ProtocolAgendaInMixContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolAgendaInMix&>(m_ProtocolAgendaInMixContainer.GetProtocolAtIndex(indx)).JSTerminateForScript(
                                                                                      cx,
                                                                                      AgendaGlobal,
                                                                                      GeneralGlobal)){
        WlAssert(0);
      }
    }
  }
}

void IGeneralManagerScriptData::InsertProtocolAgendaInMix(iLEProtocolContainer& pProtocolSessionContainer){
	WlTraceMsg("Calling NewProtocolAgendaInMix for all protocols");
  m_ProtocolAgendaInMixContainer.InitProtocolContainer(pProtocolSessionContainer.NumProtocols());
	for(WLProtocolIndex indx=0; indx< m_ProtocolAgendaInMixContainer.NumProtocols(); indx++) {
    if (!m_ProtocolAgendaInMixContainer.ProtocolExists(indx)){
      if (pProtocolSessionContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolSession*>(&(pProtocolSessionContainer.GetProtocolAtIndex(indx))));
      iLEProtocolAgendaInMix& ProtocolAgendaInMix = static_cast<iLEProtocolSession&>(pProtocolSessionContainer.GetProtocolAtIndex(indx)).NewProtocolAgendaInMix();
      ProtocolAgendaInMix.Init(this);
      m_ProtocolAgendaInMixContainer.InsertProtocolAtIndex(indx, ProtocolAgendaInMix);
    }
  }
}
}

bool IGeneralManagerScriptData::vIsUserSection()
{
	return GetJSScript().IsUserSectionActive();
}


CLoadManagerScriptData::~CLoadManagerScriptData() {
}


void CLoadManagerScriptData::InitParent(CLoadThreadsManagerScript* ManagerScript) {
		m_ManagerScript = ManagerScript;
}

CWLJSScript& CLoadManagerScriptData::GetJSScript() {
	return m_ManagerScript->GetJSScript();
	}

CWLJSBaseObject& CLoadManagerScriptData::GetJSBaseObject() {
	return m_ManagerScript->GetJSScript();
	}

CGeneralProcess& CLoadManagerScriptData::fGetProcess(){
	return m_ManagerScript->GetLoadGeneratorSlave()->fGetProcess();
}

iSession& CLoadManagerScriptData::vGetSession(){
	return *m_ManagerScript->GetLoadGeneratorSlave();
}


bool CLoadManagerScriptData::vIncludeFile_IGeneralManagerScriptData(
	const char* FileName, 
	bool Execute ) {

	return m_ManagerScript->IncludeMonitorFile( FileName, Execute );
	}


bool CLoadManagerScriptData::CopyFile(
	const char* Source, 
	const char* Target ) {

	return m_ManagerScript->CopyMonitorFile( Source, Target );
	}


void CLoadManagerScriptData::AppendStringForEvaluation(
	const char* Str,CWLJSScript::WhenToRunScript whenToRunScript ) {

	m_ManagerScript->AppendStringForEvaluation( Str, whenToRunScript );
	}



bool CLoadManagerScriptData::InInitAgenda() {
	return m_ManagerScript->InInitAgenda();
	}


bool CLoadManagerScriptData::IsSpawn() {
	return true;
	}


IFileDictionary* CLoadManagerScriptData::GetFileDict() {
	return m_ManagerScript->GetFileDict();
	}

IGeneralGlobalsRequestHandler* CLoadManagerScriptData::GetGlobalsRequestHandler() {
	return m_ManagerScript->GetGlobalsRequestHandler();
	}


RVCString& CLoadManagerScriptData::ScriptName() {
	return m_ManagerScript->ScriptName();
	}


RVCString& CLoadManagerScriptData::ScriptFileName() {
	return m_ManagerScript->ScriptFileName();
	}


void CLoadManagerScriptData::SendMessage(
	CAssociativeArray& Message, 
	int Priority ) {
	m_ManagerScript->GetLoadGenerator()->SendMessage( Message, Priority );
	}






void CLoadManagerScriptData::reportLog(
	iReport& rReport ) 
{
	m_ManagerScript->GetThreadsManager()->reportLog( rReport );
}


void CLoadManagerScriptData::OnSevereError() {
  m_ManagerScript->GetThreadsManager()->OnSevereError();
}


iLEJSMachineThread& CLoadManagerScriptData::vGetMachineThread()
{
	return *(m_ManagerScript->GetLoadGeneratorSlave());
}

void CLoadManagerScriptData::vEnterJava( const char* rClassDescriptionName, const char* rMemberName, tJavaCallType rCallType )
{
	WlAssert(m_ManagerScript);
	m_ManagerScript->EnterJava();
}

void CLoadManagerScriptData::vExitJava( JSBool rOK, const char* rClassDescriptionName, const char* rMemberName, tJavaCallType rCallType )
{
	WlAssert(m_ManagerScript);
	m_ManagerScript->ExitJava();
}

bool CLoadManagerScriptData::vIsJSGC()
{
	return fGetMachineThread().fGetProcess().fIsJSGCInExecutionContext();
}

void CLoadManagerScriptData::SetJavaThreadWrapperCreationScript()
{
	WlAssert(m_ManagerScript);
	m_ManagerScript->SetJavaThreadWrapperCreationScript();
}

bool CLoadManagerScriptData::IsJavaThreadWrapperCreationScriptExists()
{
	WlAssert(m_ManagerScript);
	return m_ManagerScript->IsJavaThreadWrapperCreationScriptExists();
}

int CLoadManagerScriptData::GetNumScriptInMix() {
	return m_ManagerScript->GetNumScriptInMix();
}

int CLoadManagerScriptData::GetErrorCounter(){
	return m_ManagerScript->GetLoadGenerator()->GetErrorCounter();
}

int CLoadManagerScriptData::GetWarningCounter(){
	return m_ManagerScript->GetLoadGenerator()->GetWarningCounter();
}

void CLoadManagerScriptData::IncrementErrorCounter(){
	m_ManagerScript->GetLoadGenerator()->IncrementErrorCounter();
}

void CLoadManagerScriptData::IncrementWarningCounter(){
	m_ManagerScript->GetLoadGenerator()->IncrementWarningCounter();
}
