/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "rvjsembed.h"

#include "tracedll.h"
#include "loadreportinfo.h"
#include "loadexceptions.h"
#include "loadjserrorfuncs.h"
#include "rvjsmachinethread.h"
#include "loadthreadinstance.h"
#include <typeinfo>

iActionReportInfo::~iActionReportInfo()
{}



iLEJSEmbedContext::iLEJSEmbedContext()
{}

iLEJSEmbedContext::~iLEJSEmbedContext()
{}


iLEJSEmbedContext* iLEJSEmbedContext::fsGet( JSContext* rCx )
{
	iJSEmbedContext* aX0 = iJSEmbedContext::fsGet(rCx);

	WlAssert( aX0 ? ( 0 != dynamic_cast<iLEJSEmbedContext*>(aX0) ): true );
	iLEJSEmbedContext* rtn = static_cast<iLEJSEmbedContext*>(aX0);

	return rtn;
}

bool iLEJSEmbedContext::fIsControlMachineThread()
{
  //@@Chaim (inherited from Itaj): maybe save state
  return( this == (fGetMachineThread().fGetCurrentEmbedContext()) );
}


void iLEJSEmbedContext::fReport(
	iReportInfo& rInfo )
{
	LoadReport( *this, fCurrentJSContext(), rInfo );
}

iRVJSJSection* iLEJSEmbedContext::fEnterJava( const char* rClassDescriptionName, const char* rMemberName, tJavaCallType rCallType )
{
	iRVJSJSection* rtn = NULL; //@@ vGetMachineThread().fSetCurrentEmbedContext( this );
	vEnterJava( rClassDescriptionName, rMemberName, rCallType );
	return rtn;
}

void iLEJSEmbedContext::fExitJava( JSBool rOK, const char* rClassDescriptionName, const char* rMemberName, tJavaCallType rCallType, iRVJSJSection* rJSJSection )
{
	vExitJava( rOK, rClassDescriptionName, rMemberName, rCallType );
	//@@ vGetMachineThread().fSetCurrentEmbedContext( static_cast<iLEJSEmbedContext*>(rJSJSection) );
}

void iLEJSEmbedContext::vJSExecReportCommit( iJSExecReportCommit& rCommit, iReportInfo& rInfo_ )
{
	if( iLoadReportInfo* aInfo = dynamic_cast<iLoadReportInfo*>(&rInfo_) ) {

		cLoadReport aReport( rCommit.fGetThreadState(), *aInfo );
		reportLog( aReport );
		if( aInfo->fIsApply() ) {
		reportApply( aReport );
		}

	} else if( cInternalReportInfo* aInfo = dynamic_cast<cInternalReportInfo*>(&rInfo_) ) {


		cJSInternalReport aReport( *this, rCommit.fGetThreadState(), *aInfo );
		reportLog( aReport );
		if(	CLoadGeneralThreadInstanceScript  * aThreadInstanceScript = dynamic_cast<CLoadGeneralThreadInstanceScript  *>(this) ) {
			aThreadInstanceScript->m_MyThreadInstance->VerificationSetFailureReason("JavaScript Engine Error",NULL,TVPSysTime(), true);
		}
		reportApply( aReport );

	} else {
		WlErrorMsg("Bad cast, object is: %s", typeid(rInfo_).name()); 
		WlAssert(0); 
	}

}


void iLEJSEmbedContext::vHardReportCommit( iReportInfo& rInfo_ )
{
	if( iLoadReportInfo* aInfo = dynamic_cast<iLoadReportInfo*>(&rInfo_) ) {

		cLoadReportHard aReport( *aInfo );
		reportLog( aReport );
		if( aInfo->fIsApply() ) {
		reportApply( aReport );
		}

	} else if( iActionReportInfo* aInfo = dynamic_cast<iActionReportInfo*>(&rInfo_) ) {

		aInfo->vApplyHard( *this );

	} else {
		WlErrorMsg("Bad cast, object is: %s", typeid(rInfo_).name()); 
		WlAssert(0); 
	}

}




/*
void iLEJSEmbedContext::fActionReportCommit( iActionReportInfo& )
{}
*/


void iLEJSEmbedContext::fEnterControlMachineThread()
{
	//@@Chaim (inherited from Itaj) improve: can postpone until first use
	vEnterControlMachineThread_iLEJSEmbedContext();
}

void iLEJSEmbedContext::fExitControlMachineThread()
{
	vExitControlMachineThread_iLEJSEmbedContext();
}



iLEJSEmbedContext::cControlMachineThread::cControlMachineThread()
:
	mTop(NULL)
{}

iLEJSEmbedContext::cControlMachineThread::cControlMachineThread(
	iLEJSEmbedContext& rTop)
:
	mTop(&rTop)
{
	fBegin();
}

iLEJSEmbedContext::cControlMachineThread::cControlMachineThread(
	iLEJSEmbedContext& rTop,
	bool rDoBegin )
:
	mTop(&rTop)
{
	if(rDoBegin) { 
		fBegin();
	}
}

iLEJSEmbedContext::cControlMachineThread::~cControlMachineThread()
{
	if(mTop) {
		fEnd();
	}
}


void iLEJSEmbedContext::cControlMachineThread::fBegin(
	iLEJSEmbedContext& rTop)
{
	WlAssert(!mTop);
	mTop = &rTop;
	fBegin();
}

void iLEJSEmbedContext::cControlMachineThread::fBegin()
{
	WlAssert(mTop);
	mPrev = mTop->vGetMachineThread().fSetCurrentEmbedContext( mTop );
	if( mPrev != mTop ) {
		if( mPrev ) {
			mPrev->fExitControlMachineThread();
		}
		mTop->fEnterControlMachineThread();
	}
}

void iLEJSEmbedContext::cControlMachineThread::fEnd()
{
	WlAssert(mTop);
	if( mPrev != mTop ) {
		mTop->fExitControlMachineThread();
		mTop->vGetMachineThread().fSetCurrentEmbedContext( mPrev ); // mPrev may be NULL
		if( mPrev ) {
			mPrev->fEnterControlMachineThread(); 
		}
	}
	mTop = NULL;
}




cLEJSEmbedContextBeginEndRequest::cLEJSEmbedContextBeginEndRequest( 
	iLEJSEmbedContext& rEmbedContext )
:
	CJSGCRequestBeginEnd( rEmbedContext.fCurrentJSContext() )
{}

cLEJSEmbedContextSuspendResumeRequest::cLEJSEmbedContextSuspendResumeRequest( 
	iLEJSEmbedContext& rEmbedContext )
:
	CJSGCRequestSuspendResume( rEmbedContext.fCurrentJSContext() )
{}





