/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// SlaveCountManager.cpp: implementation of the CSlaveCountManager class.
//
//////////////////////////////////////////////////////////////////////

#include "slavecountmanager.h"
#include "tracedll.h"

class CSlaveCountListItem :  public RVCollectable
{
private:
  double m_Time;
  long   m_Value; 
	  
public:
  CSlaveCountListItem(double time, long value);
  ~CSlaveCountListItem();
  
  virtual bool isEqual(const RVCollectable* o) const;
  virtual int compareTo(const  RVCollectable* o) const;
  virtual char *nameOf() const;
  
  double GetTime() const { return m_Time; }
  long GetSlaveCount() const { return m_Value; }
  void SetSlaveCount(long newcount) { m_Value = newcount; }
};

CSlaveCountListItem::CSlaveCountListItem(double time, long value)
{
  m_Time = time;
  m_Value = value;
}

CSlaveCountListItem::~CSlaveCountListItem()
{
}

bool CSlaveCountListItem::isEqual(const RVCollectable* o) const
{
  return( ((CSlaveCountListItem*)o)->m_Time == m_Time );
}

int CSlaveCountListItem::compareTo(const  RVCollectable* o) const
{
  if( ((CSlaveCountListItem *)o)->m_Time == m_Time )
    return 0;

  if( ((CSlaveCountListItem *)o)->m_Time < m_Time )
    return 1;		
  else
    return -1;
}

char *CSlaveCountListItem::nameOf() const
{
  return ("SlaveCountListItem");
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSlaveCountManager::CSlaveCountManager()
{
  m_NumSlaves = 0;
}

CSlaveCountManager::~CSlaveCountManager()
{
  m_SlaveCountList.clearAndDestroy();
}

void CSlaveCountManager::Insert( double time, long newSlaveCount )
{
	// protect load size list when inserting a new one item
	RVReadersWriterLock::WriteLockGuard writer(m_ListLock);

  CSlaveCountListItem *item = (CSlaveCountListItem *)m_SlaveCountList.last();
  if (item && item->GetTime() == time)
    item->SetSlaveCount(newSlaveCount);
  else
    m_SlaveCountList.insert(new CSlaveCountListItem(time, newSlaveCount));
}

void CSlaveCountManager::AddSlave( double time)
{
	// protect load size list when inserting a new one item
	RVReadersWriterLock::WriteLockGuard writer(m_ListLock);

  m_NumSlaves++;
  CSlaveCountListItem *item = (CSlaveCountListItem *)m_SlaveCountList.last();
  if (item && item->GetTime() == time)
    item->SetSlaveCount(m_NumSlaves);
  else
    m_SlaveCountList.insert(new CSlaveCountListItem(time, m_NumSlaves));
}

long CSlaveCountManager::GetSlaveCount( double time )
{
  // protect load size list while reading
  RVReadersWriterLock::ReadLockGuard reader(m_ListLock);

  for( size_t i = m_SlaveCountList.entries(); i > 0; i-- )
  {
    if( ((CSlaveCountListItem *)(m_SlaveCountList[i-1]))->GetTime() <= time )
      return ((CSlaveCountListItem *)(m_SlaveCountList[i-1]))->GetSlaveCount();
  }
  return -1;
}

