/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wldlls.h"
#include "tracedll.h"

#if defined __TV_UNIX__
#include <dlfcn.h>
#endif


//######  The implementation of class cDll  #######

cDll::cDll(RVCString& NameDll, RVCString& NameDllForLogMessage ) : m_Hlib(NULL), m_NameDll(NameDll) 
															, m_NameDllForLogMessage(NameDllForLogMessage) {}

cDll::~cDll() {
	if(m_Hlib) {
#ifdef __TV_WINDOWS__
		FreeLibrary( m_Hlib );
#elif defined __TV_UNIX__
		if(dlclose( m_Hlib )) {//return 0 on successs
			WlTraceMsg("Error in function cDll::~cDll()");
		}
#else
#error "not supported"
#endif
	}
}

bool cDll::Load(RVCString & oErrorDescription) {
#ifdef __TV_WINDOWS__
	SetLastError(0);
	m_Hlib = LoadLibrary(m_NameDll);
#endif
#ifdef __TV_UNIX__
	m_Hlib = dlopen(m_NameDll, RTLD_LAZY);
#endif
	if(m_Hlib)
		return true;
	else {
#ifdef __TV_WINDOWS__
			WlErrorMsg("cDll::Load loading %s failed %x. %d", m_NameDll, GetLastError(), GetLastError());
			/*LPVOID lpMsgBuf;
			FormatMessage( 
				FORMAT_MESSAGE_ALLOCATE_BUFFER | 
				FORMAT_MESSAGE_FROM_SYSTEM | 
				FORMAT_MESSAGE_IGNORE_INSERTS,
				NULL,
				GetLastError(),
				MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
				(LPTSTR) &lpMsgBuf,
				0,
				NULL );
			oErrorDescription = "The dll \'";
			oErrorDescription += GetNameDllForLogMessage();
			oErrorDescription += "\' failed to be loaded. The reason:";
			oErrorDescription += (LPCTSTR)lpMsgBuf;
			// Free the buffer.
			LocalFree( lpMsgBuf );*/
#elif defined(  __TV_UNIX__ )
			oErrorDescription = "The dynamic library \'";
			oErrorDescription += GetNameDllForLogMessage();
			oErrorDescription += "\' failed to be loaded. The reason:";
			oErrorDescription += dlerror();
#else
#error "bad switch"
#endif
		return false;
	}
}
	
FARPROC cDll::fGetProcAddrss(RVCString& funcName) {
	FARPROC addr = NULL;

#ifdef __TV_WINDOWS__
	addr = GetProcAddress(m_Hlib,funcName.data());
#elif defined( __TV_UNIX__ )
	void* addrX1 = dlsym(m_Hlib,funcName.data());
addr = (FARPROC)addrX1;
#elif
#error "no support"
#endif

	if (addr == NULL) {
		WlTraceMsg("Error in function cDll::fGetProcAddrss");
	}
	return addr;
}

