/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wljsruntime.h"
#include "wlleprotocol.h"
#include "loadengineselistener.h"
#include "loadengineinifileproperties.h"
#include "rvjsgeneral.h"
#include "rvcsjscompilerintervention.h"
#include "Log.h"

//class CJSLoadRuntime
CJSLoadRuntime::CJSLoadRuntime(UINT pRuntimeSize, CGeneralProcess& pProcess):
	m_Terminate(false),
  m_MachineThread(pProcess),
  m_RuntimeSize(pRuntimeSize)
{
  m_RuntimeStartThreadEvent.Reset();
  m_RuntimeStopThreadEvent.Reset();
  m_RuntimeStopThreadDoneEvent.Reset();

  // Start runtime thread
  m_RuntimeThread = rvMakeThreadFunction( *this, &CJSLoadRuntime::RuntimeThread );
	m_RuntimeThread.start();

  m_RuntimeStartThreadEvent.Wait();
	m_RuntimeStartThreadEvent.Reset();
}

CJSLoadRuntime::~CJSLoadRuntime()
{
  Terminate();
}

void CJSLoadRuntime::InitProtocols(iLEProtocolContainer& pProtocolProcessContainer){
	WlTraceMsg("Calling JSRuntimeInitialize for all protocols");
  m_ProtocolJSRuntimeContainer.InitProtocolContainer(pProtocolProcessContainer.NumProtocols());
  for (WLProtocolIndex indx = 0; indx < pProtocolProcessContainer.NumProtocols(); indx++){
    iLEProtocolJSRuntime& aProtocolJSRuntime = static_cast<iLEProtocolProcess&>(pProtocolProcessContainer.GetProtocolAtIndex(indx)).NewProtocolJSRuntime();
    if (aProtocolJSRuntime.JSRuntimeInitialize(fGet())){
      m_ProtocolJSRuntimeContainer.InsertProtocolAtIndex(indx, aProtocolJSRuntime);
    }
    else{
      WlAssert(0);
    }
  }
}

void CJSLoadRuntime::ProtocolJSRuntimeAfterNewMainContext(JSContext* cx){
	WlTraceMsg("Calling JSRuntimeAfterNewMainContext for all protocols");
  for (WLProtocolIndex indx = 0; indx < m_ProtocolJSRuntimeContainer.NumProtocols(); indx++){
    WlAssert(dynamic_cast<iLEProtocolJSRuntime*>(&(m_ProtocolJSRuntimeContainer.GetProtocolAtIndex(indx))));
    if (!static_cast<iLEProtocolJSRuntime&>(m_ProtocolJSRuntimeContainer.GetProtocolAtIndex(indx)).JSRuntimeAfterNewMainContext(cx)){
      WlAssert(0);
    }
  }
}

void CJSLoadRuntime::ProtocolJSRuntimeBeforeDestroyMainContext(JSContext* cx){
	WlTraceMsg("Calling JSRuntimeBeforeDestroyMainContext for all protocols");
  for (WLProtocolIndex indx = m_ProtocolJSRuntimeContainer.NumProtocols() - 1; indx >= 0; indx--){
    WlAssert(dynamic_cast<iLEProtocolJSRuntime*>(&(m_ProtocolJSRuntimeContainer.GetProtocolAtIndex(indx))));
    if (!static_cast<iLEProtocolJSRuntime&>(m_ProtocolJSRuntimeContainer.GetProtocolAtIndex(indx)).JSRuntimeBeforeDestroyMainContext(cx)){
      WlAssert(0);
    }
  }
}

void CJSLoadRuntime::Terminate()
{
	m_Terminate = true;
  // Stop runtime thread
  m_RuntimeStopThreadEvent.Set();
  m_RuntimeStopThreadDoneEvent.Wait();
  m_RuntimeStopThreadDoneEvent.Reset();
  if ( m_RuntimeThread.getExecutionState() != RV_THR_INITIAL &&
       m_RuntimeThread.join(8000) == RV_THR_TIMEOUT ) 
  {
    try 
    {
      m_RuntimeThread.terminate();
    }
    catch (...) {}
  }
}

JSBool fsJSFPCallback( JSContext* rCx, const char* filename ){

	if (!CLEIniFileProperties::DisplayUserAgendaPositionInErrors)
		return JS_TRUE; // If mechanism turned on, then all FP's aee user files
	
	// @@Chaim - Eventually, all files will be handled as iFiles (in load generator
	//					 AND WebFT) and then we'll also keep a flag telling if it's an internal file
	//					 or not. For now, we just check the file name. In the loadgenerator, we
	//					 just add a prefix "internal - " and in WebFT, we can just check if the
	//					 file name contains the default include path.
	const char* prefix = "internal - ";
	if (!strncmp(filename, prefix, strlen (prefix)))
		return JS_FALSE;
	else {
		iFile* aFile = nJs::ZTSEncriptionToFile(filename);
		if (!aFile)
			return JS_TRUE;
		
		TVEnvContainerClass tvpc;
		const char* path = CLEIniFileProperties::WebLOADDefaultIncludePath.data();
		const char* decryptedFilename = aFile->fGetName();
		if (!strnicmp(decryptedFilename, path, strlen(path)))
			return JS_FALSE;
		else
			return JS_TRUE;
	}
}

uint32 fsThreadIdCallback()
{
	RVThreadId rtn = ::rvThreadId();	
	return rtn;
}

// The runtime is created in a separate thread. This is because the shutdown of the runtime is
// a very delicate process and one of the many requirements is that we have a working embed
// context during the final GC and destroy context. This requires a machine thread, which we
// must get rid of before disconnecting from the JVM. For all these reason's, we don't
// want to use the process' main thread because that would mean the load engine would control
// the main process' machine thread and that would restrict WebFT users. Hence, a new thread.
void CJSLoadRuntime::RuntimeThread(){

	try{ LE_SET_SE_TRANSLATOR();
	TIMERENTRY("CJSLoadRuntime::RuntimeThread()");


  // Create LE machine thread of type NONE (not a client, just an admin thread)
  m_MachineThread.fInit(THREAD_TYPE_NONE);

	cRuntime::fInit( m_RuntimeSize );

	RVJS_SetThreadIdCallbackRT( fGet(), fsThreadIdCallback );
  RVJS_SetReservedKeywordsCallbackRT(fGet(), fsReservedKeywordsCallback);

  InitProtocols(m_MachineThread.fGetProcess().GetLEProtocolProcessContainer());

	cLEJSRuntimeEmbedContext* a_MainEmbedContext;
  m_Cx = RVJS_NewContext(fGet(), 1024);
	{	CJSGCRequestBeginEnd aGuard( m_Cx , UseOldStyle );


		RVJS_SetReportErrorFilenameCallbackRT(fGet(), fsJSFPCallback);
    RVJS_SetCompileErrorCallbackRT(fGet(), fsCompileErrorCallback);
    RVJS_SetReferenceErrorCallbackRT(fGet(), fsReferenceErrorCallback);
    RVJS_SetExtraWhiteSpaceCallbackRT(fGet(), fsExtraWhiteSpaceCallback);
		ProtocolJSRuntimeAfterNewMainContext(m_Cx);

		// Thread creation done
		m_RuntimeStartThreadEvent.Set();

		{	CJSGCRequestSuspendResume aGuard( m_Cx , UseOldStyle );
		// Wait for shutdown
		  m_RuntimeStopThreadEvent.Wait();
		  m_RuntimeStopThreadEvent.Reset();
		}

		a_MainEmbedContext = new cLEJSRuntimeEmbedContext(m_Cx, &m_MachineThread);

		ProtocolJSRuntimeBeforeDestroyMainContext(m_Cx);
  
 		m_ScriptCache.fFinal(m_Cx);
 		m_FunctionCache.fFinal(m_Cx);

		JS_GC(m_Cx);
	}
  JS_DestroyContext(m_Cx);

  m_MachineThread.fFinal();

  m_MachineThread.fGetProcess().fGetJVMManager().fJSJFinal();
	m_MachineThread.fGetProcess().fGetJVMManager().fFinal();
 
  delete a_MainEmbedContext;

//	fFinal();

  // Shutdown done
  m_RuntimeStopThreadDoneEvent.Set();
	}
	catch (std::exception* e){
		ExceptionHandler(e);
	}
	catch (std::exception& e){
		ExceptionHandler(&e);
	}
}

void CJSLoadRuntime::ExceptionHandler(std::exception* e){
	RVCString FullExcName ("");
	WLERRGetErrorMsg(e, FullExcName);
	RVCString StackTrace("");
	if (WLERRGetStackTrace(e, StackTrace) && StackTrace.length() > 0){
	  WlErrorMsg("CLoadThreadInstance::loadLoop - Exception %s occured\nTrace:\n%s", FullExcName, StackTrace);
	}
	else {
	  WlErrorMsg("CLoadThreadInstance::loadLoop - Exception %s occured", FullExcName);
	}
}
