/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
//  File WLLWProtocol.cpp

#include "wlleprotocol.h"
#include "loadengineglobals.h"

#include "managerscriptdata.h"
#include "loadgeneratorslave.h"
#include "loadthreadinstance.h"
#include "isession.h"
#include "rvjsclient.h"

	
//


//####### The implementation of the class iLEProtocolBase  #########

iLEProtocolBase::iLEProtocolBase() {}
iLEProtocolBase::~iLEProtocolBase() {}


//####### The implementation of the class iLEProtocolProcess  #########

iLEProtocolProcess::iLEProtocolProcess() {}
iLEProtocolProcess::~iLEProtocolProcess() {}

void iLEProtocolProcess::InitProtocol(WLProtocolIndex pIndex, CGeneralProcess* pLEProcess) {
	m_ProtocolIndex = pIndex;
  m_LEProcess = pLEProcess;
}

bool iLEProtocolProcess::AmIInScriptLevel(JSContext* cx) 
{
	bool rtn;
	if( getCurrentManagerScriptData(cx) ) {
		rtn = true;
	} else {
		rtn = false;
	}
	return( rtn );
}

bool iLEProtocolProcess::AmIInThreadLevel(JSContext* cx) 
{
	bool rtn;
	if( getCurrentThreadInstanceScript(cx) ) {
		rtn = true;
	} else {
		rtn = false;
	}
	return( rtn );
}

iLEAgendaJSEmbedContext& iLEProtocolProcess::GetAgendaJSEmbedContext(JSContext* cx){
  WlAssert(dynamic_cast<iLEAgendaJSEmbedContext*>(iLEJSEmbedContext::fsGet(cx)));
  return *static_cast<iLEAgendaJSEmbedContext*>(iLEJSEmbedContext::fsGet(cx));
}

iLEProtocolJSEmbedContext& iLEProtocolProcess::GetProtocolJSEmbedContext(JSContext* cx){
  iLEJSEmbedContext* aEmbedContext = iLEJSEmbedContext::fsGet(cx);
  return aEmbedContext->GetProtocolJSEmbedContext(m_ProtocolIndex);
}

iLEProtocolAgendaJSEmbedContext& iLEProtocolProcess::GetProtocolAgendaJSEmbedContext(JSContext* cx){
  WlAssert(dynamic_cast<iLEProtocolAgendaJSEmbedContext*>(&(GetAgendaJSEmbedContext(cx).GetProtocolJSEmbedContext(m_ProtocolIndex))));
  return static_cast<iLEProtocolAgendaJSEmbedContext&>(GetAgendaJSEmbedContext(cx).GetProtocolJSEmbedContext(m_ProtocolIndex));
}

iLEProtocolAgendaInMix& iLEProtocolProcess::GetProtocolAgendaInMix(JSContext* cx){
	if (AmIInScriptLevel(cx)){
  WlAssert(dynamic_cast<iLEProtocolAgendaInMix*>(&(GetProtocolAgendaJSEmbedContext(cx))));
  return static_cast<iLEProtocolAgendaInMix&>(GetProtocolAgendaJSEmbedContext(cx));
}
	else {
		if (!AmIInThreadLevel(cx))
			WlAssert(0);
		return GetProtocolVCInMix(cx).GetProtocolAgendaInMix();
	}
}

iLEProtocolVCInMix& iLEProtocolProcess::GetProtocolVCInMix(JSContext* cx){
  WlAssert(dynamic_cast<iLEProtocolVCInMix*>(&(GetProtocolAgendaJSEmbedContext(cx))));
  return static_cast<iLEProtocolVCInMix&>(GetProtocolAgendaJSEmbedContext(cx));
}

iLEProtocolAgendaMachineThread& iLEProtocolProcess::GetProtocolAgendaMachineThread(JSContext* cx) {
  WlAssert(dynamic_cast<iLEProtocolAgendaMachineThread*>(&(GetAgendaJSEmbedContext(cx))));
  return static_cast<iLEProtocolAgendaMachineThread&>(GetAgendaJSEmbedContext(cx).fGetMachineThread().GetProtocolMachineThread(m_ProtocolIndex));
}

iLEProtocolMachineThread& iLEProtocolProcess::GetCurrentProtocolMachineThread(){
  return GetProtocolProcess().GetLEProcess().fGetCurrentThread().GetProtocolMachineThread(GetProtocolProcess().GetProtocolIndex());
}


//####### The implementation of the class iLEProtocolSession  #########

iLEProtocolSession::iLEProtocolSession(iLEProtocolProcess& pFather)
:
    m_iLEProtocolProcess(pFather),
    m_Session(NULL)
{}

iLEProtocolSession::~iLEProtocolSession() {}

iLEProtocolProcess& iLEProtocolSession::GetProtocolProcess() {
	return m_iLEProtocolProcess;
}

void iLEProtocolSession::Init(iSession* pSession){
  m_Session = pSession;
}



//####### The implementation of the class iLEProtocolJSRuntime  #########

iLEProtocolJSRuntime::iLEProtocolJSRuntime(iLEProtocolProcess& pFather)
	:m_iLEProtocolProcess(pFather) {}

iLEProtocolJSRuntime::~iLEProtocolJSRuntime() {}

iLEProtocolProcess& iLEProtocolJSRuntime::GetProtocolProcess() {
	return m_iLEProtocolProcess;
}



//####### The implementation of the class iLEProtocolAgendaJSEmbedContext   #########

iLEProtocolAgendaJSEmbedContext::iLEProtocolAgendaJSEmbedContext(){
}

iLEProtocolAgendaJSEmbedContext:: ~iLEProtocolAgendaJSEmbedContext(){
}

//####### The implementation of the class iLEProtocolAgendaInMix  #########

iLEProtocolAgendaInMix::iLEProtocolAgendaInMix(iLEProtocolSession& pFather) 
	:m_iLEProtocolSession(pFather) {}

iLEProtocolAgendaInMix::~iLEProtocolAgendaInMix() {}

iLEProtocolSession& iLEProtocolAgendaInMix::GetProtocolSession() {
	return m_iLEProtocolSession;
}
iLEProtocolProcess& iLEProtocolAgendaInMix::GetProtocolProcess() {
	return m_iLEProtocolSession.m_iLEProtocolProcess;
}

void iLEProtocolAgendaInMix::Init(IGeneralManagerScriptData* pScriptData){
  m_ScriptData = pScriptData;
}

// Exported from CGeneralLoadThreadInstanceScript

// File ops.
bool iLEProtocolAgendaInMix::IncludeFile(const char* pFileNameBytes, bool pExecute){
	WlAssert(m_ScriptData);
  return m_ScriptData->IncludeFile(pFileNameBytes, pExecute);
}

bool iLEProtocolAgendaInMix::IncludeMonitorFile(const char* pFileName, bool pExecute){
	WlAssert(m_ScriptData);
  return m_ScriptData->vIncludeFile_IGeneralManagerScriptData(pFileName, pExecute);
}

bool iLEProtocolAgendaInMix::CopyFile(const char* pSource, const char* pTarget){
	WlAssert(m_ScriptData);
  return m_ScriptData->CopyFile(pSource, pTarget);
}

// Error handline
void iLEProtocolAgendaInMix::OnSevereError(){
	WlAssert(m_ScriptData);
  m_ScriptData->OnSevereError();
}

// Java stuff
bool iLEProtocolAgendaInMix::IsJavaThreadWrapperCreationScriptExists(){
	WlAssert(m_ScriptData);
  return m_ScriptData->IsJavaThreadWrapperCreationScriptExists();
}

void iLEProtocolAgendaInMix::SetJavaThreadWrapperCreationScript(){
	WlAssert(m_ScriptData);
  m_ScriptData->SetJavaThreadWrapperCreationScript();
}

// Env access and info
RVCString& iLEProtocolAgendaInMix::GetScriptName(){
	WlAssert(m_ScriptData);
  return m_ScriptData->ScriptName();
}

RVCString& iLEProtocolAgendaInMix::GetScriptFileName(){
	WlAssert(m_ScriptData);
  return m_ScriptData->ScriptFileName();
}

int iLEProtocolAgendaInMix::GetNumScriptInMix(){
	WlAssert(m_ScriptData);
  return m_ScriptData->GetNumScriptInMix();
}

bool iLEProtocolAgendaInMix::IsSpawn(){
	WlAssert(m_ScriptData);
  return m_ScriptData->IsSpawn();
}

bool iLEProtocolAgendaInMix::InInitAgenda(){
	WlAssert(m_ScriptData);
  return m_ScriptData->InInitAgenda();
}

IFileDictionary* iLEProtocolAgendaInMix::GetFileDict(){
	WlAssert(m_ScriptData);
  return m_ScriptData->GetFileDict();
}

IGeneralGlobalsRequestHandler* iLEProtocolAgendaInMix::GetGlobalsRequestHandler(){
	WlAssert(m_ScriptData);
  return m_ScriptData->GetGlobalsRequestHandler();
}

// Misc.
void iLEProtocolAgendaInMix::AppendStringForEvaluation(const char* pStr,CWLJSScript::WhenToRunScript pWhenToRunScript ){
	WlAssert(m_ScriptData);
  m_ScriptData->AppendStringForEvaluation(pStr, pWhenToRunScript);
}

void iLEProtocolAgendaInMix::SendMessage(CAssociativeArray& pMessage, int pPriority){
	WlAssert(m_ScriptData);
  m_ScriptData->SendMessage(pMessage, pPriority);
}

bool iLEProtocolAgendaInMix::IsJSActive(){
	WlAssert(m_ScriptData);
  return m_ScriptData->IsJSActive();
}

iLEJSEmbedContext& iLEProtocolAgendaInMix::GetInternalLEJSEmbedContext(){
	WlAssert(m_ScriptData);
  return *m_ScriptData;
}


//####### The implementation of the class iLEProtocolAgendaMachineThread  #########

iLEProtocolAgendaMachineThread::iLEProtocolAgendaMachineThread(iLEProtocolProcess& pFather, WLProtocolThreadType pProtocolThreadType) :
iLEProtocolMachineThread(pFather, pProtocolThreadType)
{
}

iLEProtocolAgendaMachineThread::~iLEProtocolAgendaMachineThread(){
}

void iLEProtocolAgendaMachineThread::Init(iLEJSAgendaMachineThread* rThread){
  m_JSMachineThread = rThread;
}

iLEProtocolProcess& iLEProtocolAgendaMachineThread::GetProtocolProcess(){
  return m_iLEProtocolProcess;
}

iLEJSMachineThread& iLEProtocolAgendaMachineThread::GetJSMachineThread(){
  WlAssert(m_JSMachineThread);
  return *m_JSMachineThread;
}


//####### The implementation of the class iLEProtocolVCInMix  #########

iLEProtocolVCInMix::iLEProtocolVCInMix(iLEProtocolAgendaInMix& pFather) 
	:m_iLEProtocolAgendaInMix(pFather){}

iLEProtocolVCInMix::~iLEProtocolVCInMix() {}

void iLEProtocolVCInMix::Init(CLoadGeneralThreadInstanceScript* pThreadInstanceScript){
  m_ThreadInstanceScript = pThreadInstanceScript;
}

JSContext* iLEProtocolVCInMix::GetJSContext() {
	return( m_Parent->GetContext() );
}

JSObject* iLEProtocolVCInMix::GetGlobalObject()
{
	return m_Parent->GetObject();
}

CLEVariablesData& iLEProtocolVCInMix::GetVariablesData()
{
	CLEVariablesData* rtn = m_Parent->GetLoadEngineVariablesData();
	WlAssert(rtn);
	return *rtn;
}



iLEProtocolAgendaInMix& iLEProtocolVCInMix::GetProtocolAgendaInMix() {
	return m_iLEProtocolAgendaInMix;
}

iLEProtocolSession& iLEProtocolVCInMix::GetProtocolSession() {
	return m_iLEProtocolAgendaInMix.m_iLEProtocolSession;
}

iLEProtocolProcess& iLEProtocolVCInMix::GetProtocolProcess() {
	return m_iLEProtocolAgendaInMix.m_iLEProtocolSession.m_iLEProtocolProcess;
}

iLEProtocolAgendaMachineThread& iLEProtocolVCInMix::GetProtocolAgendaMachineThread(){
	WlAssert(m_ThreadInstanceScript);
  WlAssert(dynamic_cast<iLEProtocolAgendaMachineThread*>(&(m_ThreadInstanceScript->fGetMachineThread().GetProtocolMachineThread(GetProtocolProcess().GetProtocolIndex()))));
  return static_cast<iLEProtocolAgendaMachineThread&>(m_ThreadInstanceScript->fGetMachineThread().GetProtocolMachineThread(GetProtocolProcess().GetProtocolIndex()));
}

void iLEProtocolVCInMix::SetParent(CWLJSThread& pParent) {
	m_Parent = &pParent;
}


// Exported from CGeneralLoadThreadInstanceScript

// Java stuff
void iLEProtocolVCInMix::ToggleJavaDefaultStatistics(bool pValue){
	WlAssert(m_ThreadInstanceScript);
  m_ThreadInstanceScript->ToggleJavaDefaultStatistics(pValue);
}

bool iLEProtocolVCInMix::IsDisabledJavaDefaultStatistics(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->IsDisabledJavaDefaultStatistics();
}

void iLEProtocolVCInMix::CreateApplet(JSContext*                  cx,
                                      const char*                 appletClassName,
										                  const char*                 appletCodeBase,
                                      const char*                 appletDocumentBase,
										                  RVHashDictionary*           appletParams,
                                      CWLJSJavaApplet*&           javaApplet){
	WlAssert(m_ThreadInstanceScript);
  m_ThreadInstanceScript->CreateApplet(cx,
                                       appletClassName,
                                       appletCodeBase,
                                       appletDocumentBase,
                                       appletParams,
                                       javaApplet);
}

bool iLEProtocolVCInMix::ClassAppletExists(
          JSContext*  cx,
          const char* appletClassName,
          bool& classAppletExists){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->ClassAppletExists(cx, appletClassName, classAppletExists);
}

bool iLEProtocolVCInMix::AddJavaStream(JSContext*  cx,
                                       const char* streamName,
                                       const char* codeBase,
                                       RVCString&  javaStream){
	WlAssert(m_ThreadInstanceScript);
   return m_ThreadInstanceScript->AddJavaStream(cx, streamName, codeBase, javaStream);
}

bool iLEProtocolVCInMix::JavaStreamExists(const char* streamName)
{
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->JavaStreamExists(streamName);
}

// Transaction
void iLEProtocolVCInMix::VerificationSetFailureReason(const char* pReason, const char* pDescription, double pTime, bool pIsEvent){
	WlAssert(m_ThreadInstanceScript);
  m_ThreadInstanceScript->VerificationSetFailureReason(pReason, pDescription, pTime, pIsEvent);
}

void iLEProtocolVCInMix::VerificationRecordEvent(const char* pReason, const char* pDescription, double pTime, WLLogMessage::Severity pSeverity, const char* pExtraDataType, void* pExtraData, unsigned long pLen, bool pIsLog){
	WlAssert(m_ThreadInstanceScript);
  m_ThreadInstanceScript->VerificationRecordEvent(pReason, pDescription, pTime, pSeverity, pExtraDataType, pExtraData, pLen, pIsLog);
}

bool iLEProtocolVCInMix::VerificationBeginTransaction(const char* pTransname, double pTime, const char* pExtraDataType, void* pExtraData, unsigned long pLen){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->VerificationBeginTransaction(pTransname, pTime, pExtraDataType, pExtraData, pLen);
}

bool iLEProtocolVCInMix::VerificationEndTransaction(const char* pTransname, double pTime, WLLogMessage::Severity pSeverity, bool pSave, bool pSendTimers, bool pIsLog ){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->VerificationEndTransaction(pTransname, pTime, pSeverity, pSave, pSendTimers, pIsLog);
}

bool iLEProtocolVCInMix::VerificationEndTransactionFailure(const char* pTransname, double pTime, WLLogMessage::Severity pSeverity,const char* pReason, bool pSendTimers, bool pIsLog, bool pIsFailureEvent ){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->VerificationEndTransactionFailure(pTransname, pTime, pSeverity, pReason, pSendTimers, pIsLog, pIsFailureEvent);
}

int  iLEProtocolVCInMix::VerificationGetTransactionDepth(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->VerificationGetTransactionDepth();
}

const char* iLEProtocolVCInMix::VerificationGetCurrentTransaction(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->VerificationGetCurrentTransaction();
}

// Statistics
void iLEProtocolVCInMix::addMeasurement(const char* name, double l){
	WlAssert(m_ThreadInstanceScript);
  m_ThreadInstanceScript->addMeasurement(name, l);
}

void iLEProtocolVCInMix::addTimer(const char* name, double l){
	WlAssert(m_ThreadInstanceScript);
  m_ThreadInstanceScript->addTimer(name, l);
}

void iLEProtocolVCInMix::addCounter(const char* name){
	WlAssert(m_ThreadInstanceScript);
  m_ThreadInstanceScript->addCounter(name);
}

void iLEProtocolVCInMix::addAccumulator(const char* name, double l){
	WlAssert(m_ThreadInstanceScript);
  m_ThreadInstanceScript->addAccumulator(name, l);
}

void iLEProtocolVCInMix::addLast(const char* name, double l, CWLStatPointTypeEnum::PointType type){
	WlAssert(m_ThreadInstanceScript);
  m_ThreadInstanceScript->addLast(name, l, type);
}

void iLEProtocolVCInMix::resetTimer(const char* name){
	WlAssert(m_ThreadInstanceScript);
  m_ThreadInstanceScript->resetTimer(name);
}

void iLEProtocolVCInMix::sendTimer(const char* name){
	WlAssert(m_ThreadInstanceScript);
  m_ThreadInstanceScript->sendTimer(name);
}

// Misc. LE operations
bool iLEProtocolVCInMix::isAbort(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->isAbort();
}

void iLEProtocolVCInMix::sleep(int pMilisec){
	WlAssert(m_ThreadInstanceScript);
  m_ThreadInstanceScript->sleep(pMilisec);
}

void iLEProtocolVCInMix::reportApply(iReport& pReport){
	WlAssert(m_ThreadInstanceScript);
  m_ThreadInstanceScript->reportApply(pReport);
}

void iLEProtocolVCInMix::reportLog(iReport& pReport){
	WlAssert(m_ThreadInstanceScript);
  m_ThreadInstanceScript->reportLog(pReport);
}

bool iLEProtocolVCInMix::report(WLLogMessage::Severity pSeverity, const char* pMessage){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->report(pSeverity, pMessage);
}

void iLEProtocolVCInMix::reportApplyEvent(WLLogMessage::Severity pSeverity, const char* pMessage){
	WlAssert(m_ThreadInstanceScript);
  m_ThreadInstanceScript->reportApplyEvent(pSeverity, pMessage);
}

bool iLEProtocolVCInMix::reportTime(WLLogMessage::Severity pSeverity, const char* pMessage, double pTime, WLLogMessage::Severity pErrorInTransTree){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->reportTime(pSeverity, pMessage, pTime, pErrorInTransTree);
}

bool iLEProtocolVCInMix::report(WLErrorException& pException){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->report(pException);
}

bool iLEProtocolVCInMix::reportTime(WLErrorException& pException, double pTime){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->reportTime(pException, pTime);
}

bool iLEProtocolVCInMix::reportMultiThread( iReportInfo& pReportInfo){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->reportMultiThread(pReportInfo);
}

bool iLEProtocolVCInMix::reportMultiThread( WLLogMessage::Severity pSeverity, const char* pDescription){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->reportMultiThread(pSeverity,pDescription);
}

bool iLEProtocolVCInMix::reportMultiThread( WLErrorException& pException){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->reportMultiThread(pException);
}

void iLEProtocolVCInMix::endMultiThread(){
	WlAssert(m_ThreadInstanceScript);
  m_ThreadInstanceScript->endMultiThread();
}

CLEVariablesData* iLEProtocolVCInMix::GetCurrentLoadEngineVariablesData(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->GetCurrentLoadEngineVariablesData();
}

// Env access and info
RVCString& iLEProtocolVCInMix::GetScriptName(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->GetScriptName();
}

RVCString& iLEProtocolVCInMix::GetScriptFileName(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->GetScriptFileName();
}

bool iLEProtocolVCInMix::IsJSGC(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->vIsJSGC();
}

int iLEProtocolVCInMix::GetNumScriptInMix(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->GetNumScriptInMix();
}

IFileDictionary* iLEProtocolVCInMix::GetFileDict(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->GetFileDict();
}

unsigned long iLEProtocolVCInMix::GetTotalRoundNum(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->TotalRoundNum();
}

unsigned long iLEProtocolVCInMix::GetCurrentScriptRoundNum(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->CurrentScriptRoundNum();
}

unsigned long iLEProtocolVCInMix::GetRoundNumScriptInSlave(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->GetRoundNumScriptInSlave();
}

unsigned long iLEProtocolVCInMix::GetRoundNumMixInSlave(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->GetRoundNumMixInSlave();
}

int iLEProtocolVCInMix::GetLGThreadNum(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->GetLGThreadNum();
}

WLLogMessage::Severity iLEProtocolVCInMix::GetErrorSeverity(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->getErrorSeverity();
}

bool iLEProtocolVCInMix::IsLoadServer(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->IsLoadServer();
}

bool iLEProtocolVCInMix::IsOneClient(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->IsOneClient();
}

bool iLEProtocolVCInMix::IsRTAM(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->IsRTAM();
}

bool iLEProtocolVCInMix::IsJSActive(){
	WlAssert(m_ThreadInstanceScript);
  return m_ThreadInstanceScript->IsJSActive();
}

iLEJSEmbedContext& iLEProtocolVCInMix::GetInternalLEJSEmbedContext(){
	WlAssert(m_ThreadInstanceScript);
  return *m_ThreadInstanceScript;
}

iLEVCJSThread& iLEProtocolVCInMix::vGetVCJSThread(){
	WlAssert(m_ThreadInstanceScript);
  return *m_ThreadInstanceScript;
}
