/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#include "njsdxml.h"
#include <memory>
#include "pcui.h"

namespace nPCUi {

cXMLStream::cXMLStream( 
	const char* FileName )
:
	iXMLStream(),
	mFile( FileName )
{}

cXMLStream::~cXMLStream()
{}

void cXMLStream::fPush( 
	char Char )
{
	mFile << Char;
}


void cXMLStreamStub::fPush( 
	const char* String )
{}


cPropertyNameToString::cPropertyNameToString(
	iProperty& Property )
{
	if( Property.fIsNameString() ) {
		mName = Property.fGetNameString();
	} else if( Property.fIsNameInt() ) {
		char Buffer[10];
		sprintf( Buffer, "[%d]", (int)Property.fGetNameInt() );
		mName = Buffer;
	} else {
		mName = "~~";
	}
}

const char* cPropertyNameToString::fGetName() const
{
	return mName.data();
}

cPropertyXMLElement::cPropertyXMLElement(
	iXMLStream& Stream,
	iProperty& Property,
	cLevel ParentLevel )
:
	mStringName(cPropertyNameToString(Property).fGetName()),
	mXMLElement( Stream, mStringName.data(), 0, ParentLevel )
{
	fAttribute( Stream, "Enumerated", Property.fIsEnumerate() );	
	fAttribute( Stream, "ReadOnly", Property.fIsReadOnly() );	
	fAttribute( Stream, "Permanent", Property.fIsPermanent() );	
	fAttribute( Stream, "Alias", Property.fIsAlias() );	
	fAttribute( Stream, "Argument", Property.fIsArgument() );	
	fAttribute( Stream, "Variable", Property.fIsVariable() );	
	fAttribute( Stream, "Hinted", Property.fIsHinted() );	

	{
		char Buffer[10];
		sprintf( Buffer, "[%d]", (int)Property.fGetSlotIndex() );
		cXMLElement( Stream, "SlotIndex", Buffer );
	}

	{
		std::auto_ptr<iValue> Value(Property.fGetValue());
		cValueXMLElement( Stream, Value.get(), "Value", mXMLElement.fGetLevel() );
	}
}

void cPropertyXMLElement::fAttribute(
	iXMLStream& Stream,
	const char* Name,
	bool State )
{
	cXMLElement( Stream, Name, ( State ? "true" : "false" ) );
}



cValueXMLElement::cValueXMLElement(
	iXMLStream& Stream,
	iValue* pValue,
	const char* Name,
	cLevel ParentLevel )
:
	cXMLElementRoot( Stream, Name, (pValue ? pValue->fToString() : 0 ), ParentLevel )
{
	if( pValue ) {
		iValue& Value = *pValue;

		{
			std::string Type("");

			if( Value.fIsObject() ) {
				Type += "Object_";
			}
			if( Value.fIsNull() ) {
				Type += "null_";
			}
			if( Value.fIsFunction() ) {
				Type += "Func_";
			}
			if( Value.fIsNative() ) {
				Type += "Nat_";
			}
			if( Value.fIsPrimitive() ) {
				Type += "Primitive_";
			}
			if( Value.fIsVoid() ) {
				Type += "void_";
			}
			if( Value.fIsBoolean() ) {
				Type += "bool_";
			}
			if( Value.fIsInt() ) {
				Type += "int_";
			}
			if( Value.fIsDouble() ) {
				Type += "float_";
			}
			if( Value.fIsNumber() ) {
				Type += "num_";
			}
			if( Value.fIsString() ) {
				Type += "str_";
			}

			cXMLElement( Stream, "Type", Type.data() );
		}

		cXMLElement( Stream, "Class", Value.fGetClassName() );

		if( 0 < fGetLevel() ) {
			cValueXMLElement( Stream, (std::auto_ptr<iValue>(Value.fGetProto())).get(), "Proto", fGetLevel() );
			cValueXMLElement( Stream, (std::auto_ptr<iValue>(Value.fGetParent())).get(), "Parent", fGetLevel() );
			cValueXMLElement( Stream, (std::auto_ptr<iValue>(Value.fGetCtor())).get(), "Ctor", fGetLevel() );
		}

		if( Value.fIsObject() ) { // propterties

			char Buffer[10] = "-";
			sprintf( Buffer, "%d", (int)Value.fGetPropertiesCount() );
			cXMLElement X0( Stream, "Properties", Buffer );

			if( 0 < fGetLevel() ) {
				cPropertiesIter PropIter( Value );
				iProperty* Prop;
				while( Prop = PropIter.fGetNext() ) {
					{
						cPropertyXMLElement( Stream, *Prop, fGetLevel() );
					}
					delete Prop;
				}
			}

		}
	}
}

cStackFrameXMLElement::cStackFrameXMLElement(
	iXMLStream& Stream,
	iStackFrame& StackFrame,
	cLevel ParentLevel )
:
	cXMLElementRoot( Stream, "SF", 0, ParentLevel )
{
	{
		cScriptFile* File = dynamic_cast< cScriptFile* >( StackFrame.fGetFile() );
		if( File ) {
			cXMLElement( Stream, "FileName", File->fGetName() );
		} else {
			cXMLElement( Stream, "ScriptName", StackFrame.fGetName() );
		}
	}

	char Buffer[10];

	sprintf( Buffer, "%d", (int)StackFrame.fGetBaseLine() );
	cXMLElement( Stream, "BaseLine", Buffer );

	sprintf( Buffer, "%d", (int)StackFrame.fGetLineExtent() );
	cXMLElement( Stream, "LineExtent", Buffer );

	cXMLElement( Stream, "Function", StackFrame.fGetFunctionName() );

	sprintf( Buffer, "%d", (int)StackFrame.fGetCurrentLineNumber() );
	cXMLElement( Stream, "CurrentLine", Buffer );

	{
		cLevel Level( fGetLevel() );
		//Level = 1; //force no level
		cValueXMLElement( Stream, (std::auto_ptr<iValue>(StackFrame.fGetCallObject())).get(), "CallObject", Level );
		cValueXMLElement( Stream, (std::auto_ptr<iValue>(StackFrame.fGetThisValue())).get(), "this", Level );
		cValueXMLElement( Stream, (std::auto_ptr<iValue>(StackFrame.fGetScopeChain())).get(), "ScopeChain", Level );

//		std::string Script( "new String(\"boo\")" );
//		std::string Script( "a" );
		std::string Script( "arguments" );
//		cValueXMLElement( Stream, (std::auto_ptr<iValue>(StackFrame.fEvaluateScript( Script.data(), Script.length() ) ) ).get(), "eval", Level );
	}
}


cThreadStateXMLElement::cThreadStateXMLElement(
	iXMLStream& Stream,
	iDMThreadState& ThreadState,
	cLevel ParentLevel )
:
	cXMLElementRoot( Stream, "THS", 0, ParentLevel )
{
	{

		char Buffer[10];
		sprintf( Buffer, "%d", (int)ThreadState.fGetStackFramesCount() );

		cXMLElement X0( Stream, "StackFrames", Buffer );
		cIteratableStackFrame StackFrameIter( ThreadState );
		while( StackFrameIter.fIsValid() ) {
			cStackFrameXMLElement( Stream, StackFrameIter, fGetLevel() );
			StackFrameIter.fNext();
		}
	}
	{
		std::auto_ptr<iValue> Exception( ThreadState.fGetException() );
		if( Exception.get() ) {
//			cValueXMLElement( Exception, "exp", Stream, fGetLevel() );
		}
	}
}


}

