/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#include "xmlstream.h"


iXMLStream::iXMLStream()
{}

iXMLStream::~iXMLStream()
{}

void iXMLStream::fPushXML( 
	const char* String )
{
	int i=0;
	while( String[i] ) {
		fPush( String[i] );
		++i;
	}
}

void iXMLStream::fPushName( 
	const char* String )
{
	int i=0;
	while( char c = String[i] ) {

		bool ok(true);
		if( ( ( c >= '0' ) && ( c <= '9' ) ) || ( ( c >= 'a' ) && ( c <= 'z' ) ) || ( ( c >= 'A' ) && ( c <= 'Z' ) ) ) {
			fPush( c );
		} else {
			fPushXML( "_" );
		}
		++i;
	}

}

void iXMLStream::fPushData( 
	const char* String )
{
	int i=0;
	while( String[i] ) {
		switch( String[i] ) {
		case '\"':
			fPushXML( "&quot;" );
			break;
		case '\'':
			fPushXML( "&apos;" );
			break;
		case '&':
			fPushXML( "&amp;" );
			break;
		case '<':
			fPushXML( "&lt;" );
			break;
		case '>':
			fPushXML( "&gt;" );
			break;
		default:
			fPush( String[i] );
			break;
		}
		++i;
	}
}

cXMLElement::cXMLElement(
	iXMLStream& Stream,
	const char* Name,
	const char* Value )
:
	mStream(Stream),
	mName( Name )
{
	mStream.fPushXML( "<" );
	mStream.fPushName( mName );
	if( Value ) {
		mStream.fPushXML( " X0=\"" );
		mStream.fPushData( Value );
		mStream.fPushXML( "\"" );
	}
	mStream.fPushXML( ">" );
}

cXMLElement::~cXMLElement()
{
	mStream.fPushXML( "</" );
	mStream.fPushName( mName );
	mStream.fPushXML( ">" );
}


iXMLStream& cXMLElement::fGetStream()
{
	return mStream;
}


cXMLElementRoot::cXMLElementRoot(
	iXMLStream& Stream,
	const char* Name,
	const char* Value,
	cLevel ParentLevel )
:
	cXMLElement( Stream, Name, Value ),
	mLevel(ParentLevel-1)
{
}


cXMLElementRoot::cLevel cXMLElementRoot::fGetLevel()
{
	return mLevel;
}
