/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "rvhtmlattributefinder.h"
#include "rvhtmlparseutils.h"
#include <collstr.h>
#include <collint.h>
#include <ctype.h>

CHTMLAttributeFinder::CHTMLAttributeFinder()// : m_HashTable(RVCString::hash)
{
  
}

void CHTMLAttributeFinder::FindAttributes(RVCString& p_sBuff, size_t& p_nBuffPtr, RVHashDictionary& po_oValueDict, RVHashDictionary& po_oOffsetDict)
{
	size_t length = p_sBuff.length(), indx;
	size_t attributeIndex, delimiterIndex = RV_NPOS,
		endAttributeIndex, equalSignIndex;
	char c;
	RVCString attributeName, attributeValue;

  po_oValueDict.clearAndDestroy();
  po_oOffsetDict.clearAndDestroy();

	static	RVCRegexp sp("[>\\t\\r\\n ]"); // oooo
  //@@Skip to the start attribute
  //htmlSkipNonWhiteSpaces(p_sBuff, length, p_nBuffPtr);
	htmlSkipWhiteSpaces(p_sBuff, length, p_nBuffPtr);
	while(RV_NPOS == delimiterIndex)
	{
		attributeIndex = RV_NPOS;
		endAttributeIndex = RV_NPOS;
		equalSignIndex = RV_NPOS;
		
		for(indx = p_nBuffPtr; indx < length; ++indx){
			c = p_sBuff(indx);
			switch(c) {
				case '=':
					if (attributeIndex != RV_NPOS) {
						if (endAttributeIndex == RV_NPOS)
							endAttributeIndex = indx;
						equalSignIndex = indx;
					} 
					else
					{ //! Illegal char
						size_t i = p_sBuff.index(sp, indx);
						if (i == RV_NPOS)
							break;  // Blahhh. something defenitly wrong. Break the process.
						if (p_sBuff(i) == '>') {
							delimiterIndex = i;
							break;
						}
						indx = i;
					}
						
					break;
						
				case '>':
					if (attributeIndex != RV_NPOS && endAttributeIndex == RV_NPOS)
						endAttributeIndex = indx;
					delimiterIndex = indx;
					break;
					
				case ' ':
				case '\t':
				case '\n':
				case '\r':
        case '"':
					if (attributeIndex != RV_NPOS && endAttributeIndex == RV_NPOS)
						endAttributeIndex = indx;
					continue;
					
				default:
					if (isalnum(c) || c == '-') {
						if (attributeIndex == RV_NPOS)
							attributeIndex = indx;
						else if (endAttributeIndex != RV_NPOS)
							break;
						} else { //! Illegal char
						size_t i = p_sBuff.index(sp, indx);
						if (i == RV_NPOS)
							break; // again something is incomplete, so get out
						if (p_sBuff(i) == '>') {
							delimiterIndex = i;
							break;
						}
						indx = i;
					}
					continue;
			}
      break;
    }
    
    if(attributeIndex != RV_NPOS && endAttributeIndex != RV_NPOS)
    {
      attributeName  = p_sBuff(attributeIndex, endAttributeIndex - attributeIndex);
      
      if( equalSignIndex == RV_NPOS )  // no value !!
      {
        attributeValue = "";
        p_nBuffPtr = endAttributeIndex;
	RVCollectableString csAttrName(attributeName);
        if (!po_oValueDict.contains(&csAttrName))
        {
          RVCollectableString* psName1 = new RVCollectableString(attributeName);
          RVCollectableString* psName2 = new RVCollectableString(attributeName);
          RVCollectableString* psValue = new RVCollectableString(attributeValue);
          RVCollectableInt* pnOffset = new RVCollectableInt(-1);        
          psName1->toLower();
          psName2->toLower();          
          po_oValueDict.insertKeyAndValue(psName1, psValue);
          po_oOffsetDict.insertKeyAndValue(psName2, pnOffset);
        }
      }
      else
      {
        size_t endAttributsPtr = 0, startAttributsPtr = 0;
        strVal attVal = htmlGetStringValue(p_sBuff, attributeValue, equalSignIndex,
          startAttributsPtr, endAttributsPtr, length);
        if(attVal == HTML_STRING_NOT_COMPLETE)
          break; // incomplete string. Break the whole process
        else// attVal == HTML_STRING_COMPLETE
        {
          p_nBuffPtr = endAttributsPtr;
          //@@p_content->SetAttribute( attributeName.data(), attributeValue, true, true );
	  RVCollectableString csAttrName(attributeName);
          if (!po_oValueDict.contains(&csAttrName))
          {
            RVCollectableString* psName1 = new RVCollectableString(attributeName);
            RVCollectableString* psName2 = new RVCollectableString(attributeName);
            RVCollectableString* psValue = new RVCollectableString(attributeValue);
            RVCollectableInt* pnOffset = new RVCollectableInt(startAttributsPtr);   
            psName1->toLower();
            psName2->toLower();
            po_oValueDict.insertKeyAndValue(psName1, psValue);
            po_oOffsetDict.insertKeyAndValue(psName2, pnOffset);
          }
        }
      }
    } 
    else
      break; // no valid attribute. Do not continue.
  }
  
  return;
  
}
