/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "rvhtmloffsetparser.h"
#include "rvhtmlattributefinder.h"
#include "rvhtmldiettransactiondata.h"
#include "rvhtmldietattributeobject.h"
#include "tracedll.h"
#include <collstr.h>
#include <ordered.h>
#include <collint.h>



CHTMLOffsetParser::CHTMLOffsetParser(const char* p_sBaseUrl) : m_nBuffPos(0) , m_pDietVirtualQueue(NULL)
{
  RVCString sAttributesArray[][6] = 
  {
	  { "BASE",   "href", "", "", "", ""},
    { "BODY",   "background", "", "", "", ""},
    { "TABLE" , "background", "", "", "", ""},
    { "TD"    , "background", "", "", "", ""},
    { "TH"    , "background", "", "", "", ""},
    { "APPLET", "code", "codebase", "src" , "archive", "data"}, 
    { "OBJECT", "code", "codebase", "classid" , "archive"}, 
    { "EMBED" , "src", "", "", "", ""},
    { "FRAME",  "src", "", "", "", ""},
    { "IFRAME", "src", "", "", "", ""},
    { "IMG"   , "src" , "lowsrc", "dynsrc", "", ""},
    { "SCRIPT", "src", "", "", "", ""},
    { "LINK"  , "href", "rel", "", "", ""},
    { "INPUT" , "src", "", "", "", ""},
    { "XML"   , "src", "", "", "", ""},
    { "TEST" , "", "", "", "", ""},
// @@Chaim - rolling back ParseCSS fix for IMPORT ""
//    { "STYLE" , "type", "", "", "", ""},
	  { "?IMPORT" , "implementation" , "", "", "", ""},
    { "", "", "", "", "", ""}
  };

  RVCString sUrlBase;
  if (p_sBaseUrl)
  {
    sUrlBase.append(p_sBaseUrl);
  }
  m_oUrlBase.setBase(sUrlBase);
  m_eReadState = START_APPEND;
  int nX = 0;
  int nY = 0;
  while (sAttributesArray[nX][0] != "")
  {
    RVCollectableString* psElementName = new RVCollectableString(sAttributesArray[nX][0]);
    RVOrdered* pAttributesOrdered = new RVOrdered;
    nY++;
    while (sAttributesArray[nX][nY] != "")
    {
      pAttributesOrdered->insert(new RVCollectableString(sAttributesArray[nX][nY]));
      nY++;
    }
    nY = 0;
    nX++;
    if (pAttributesOrdered->entries() > 0)
    {
      m_oElementAttributesHash.insertKeyAndValue(psElementName, pAttributesOrdered);
    } else
    {
      delete psElementName;
      pAttributesOrdered->clearAndDestroy();
      delete pAttributesOrdered;
    }
  }
}

CHTMLOffsetParser::~CHTMLOffsetParser()
{
  RVHashDictionaryIterator iter(m_oElementAttributesHash);
  while ( iter() )
  {
    RVCollectable* pKey = iter.key();
    if (pKey)
    {
      delete pKey;
    } 
    RVOrdered* pOrderedValue = dynamic_cast<RVOrdered*>(iter.value());
    if (pOrderedValue)
    {
      pOrderedValue->clearAndDestroy();
      delete pOrderedValue;
    }    
  }
  m_oOrderedTrans.clearAndDestroy();//leaks
}

void CHTMLOffsetParser::SetDietVirtualQueue(CRVHtmlDietVirtualQueue* p_pDietQueue)
{
  m_pDietVirtualQueue = p_pDietQueue;
}
CRVHtmlDietVirtualQueue* CHTMLOffsetParser::GetDietVirtualQueue()
{
  return m_pDietVirtualQueue;
}

bool CHTMLOffsetParser::isWhiteSpace(char& c)
{
  if(c == ' ' || c == '\n' || c == '\r' || c == '\t' || c == '~' || c=='>' || c=='<')
    return true;
  
  return false;
}


RVCString CHTMLOffsetParser::findTagName()
{
  RVCString sTagName = "";
  while (m_sBuffer.length() > m_nBuffPos && m_sBuffer(m_nBuffPos) != '<')
  {
    m_nBuffPos++;
  }
  while (m_sBuffer.length() > m_nBuffPos && isWhiteSpace(m_sBuffer(m_nBuffPos)))
  {
    m_nBuffPos++;
  }
  while (m_sBuffer.length() > m_nBuffPos && !isWhiteSpace(m_sBuffer(m_nBuffPos)))
  {
    sTagName.append(m_sBuffer(m_nBuffPos));
    m_nBuffPos++;
  }
  //  cout<<"findTagName: "<<RVCString(m_sBuffer(m_nBuffPos, 10)).data()<<"\t Found :"<<sTagName<<endl;
  return sTagName;
}

void CHTMLOffsetParser::ParseHTML(const char* p_sHTMLData, size_t p_nSize)
{
	m_nBuffPos = 0;
	m_sBuffer.remove(0);
  m_sBuffer.append(p_sHTMLData);
  wlOverwriteScriptWithBlanks(m_sBuffer);
 	wlOverwriteTagWithBlanks(m_sBuffer,"<!--","-->");
 	
  
  size_t  lt = m_nBuffPos;
  while(1)
  {
    //cout <<"Parser buffer position = "<<m_nBuffPos<<"\t "<<RVCString(m_sBuffer(m_nBuffPos, 10)).data()<<endl;
    RVCollectableString sTagName = findTagName();
    sTagName.toUpper();
    
    if (m_oElementAttributesHash.find(&sTagName))
    {
// @@Chaim - rolling back ParseCSS fix for IMPORT ""
//      if (!sTagName.compareTo("STYLE")){
//        ParseCSS(p_sHTMLData);
//      }
//      else {
      //cout<<">>Parsing \t"<<sTagName<<endl;
      parseHTMLElement(sTagName);
//      }
    } else
    {
      //      cout<<">>Skip \t"<<sTagName<<endl;
    }
    if (m_nBuffPos >= m_sBuffer.length())
    {
      return;
    }

  }
}




void CHTMLOffsetParser::parseHTMLElement(RVCString p_sTagName)
{
//  cout<<"parseHTMLElement: "<<p_sTagName<<" - "<<RVCString(m_sBuffer(m_nBuffPos, 10)).data()<<endl;

  if ((m_sBuffer)(m_nBuffPos) == '/')
    return;
  
  size_t tagEndIndex = m_sBuffer.index('>', m_nBuffPos);
  
  if(tagEndIndex == RV_NPOS)
    return;
  
  size_t buffPtr = m_nBuffPos, attValueStart = RV_NPOS;
  RVCString attName, attValue, id, name;
  CHTMLAttributeFinder attrFinder;
  RVHashDictionary attrValuesDict, attrOffsetsDict;
  attrFinder.FindAttributes(m_sBuffer, m_nBuffPos, attrValuesDict, attrOffsetsDict);
  
  CRVHTMLDietTransactionsObject* pTransObject = NULL;
  RVCollectable* pAttributes = NULL;
  RVCollectableString csTagName(p_sTagName);
  m_oElementAttributesHash.findKeyAndValue(&csTagName, pAttributes);
  if (pAttributes)
  {
    RVOrdered* pAttributesOrdered = (RVOrdered*)(pAttributes);
    if (pAttributesOrdered->entries() > 0)
    {
      for (unsigned int i = 0 ; i < pAttributesOrdered->entries() ; i++)
      {
        RVCollectable* pColAttrName = NULL;
        RVCollectable* pColOffset = NULL;
        RVCollectableString* psAttrValue = NULL;
        RVCollectableInt* pnAttrOffset = NULL;
        RVCollectableString* psAttributeName = dynamic_cast<RVCollectableString*>(pAttributesOrdered->at(i));
        if (psAttributeName)
        {
          if (attrValuesDict.findKeyAndValue(psAttributeName, pColAttrName))
          {
            psAttrValue = dynamic_cast<RVCollectableString*>(pColAttrName);
          }
          if (attrOffsetsDict.findKeyAndValue(psAttributeName, pColOffset))
          {
            pnAttrOffset = dynamic_cast<RVCollectableInt*>(pColOffset);
          }          
          if (psAttrValue && pnAttrOffset)
          {
            if (!pTransObject)
            {
              pTransObject = new CRVHTMLDietTransactionsObject();
            }
            pTransObject->SetTagName(p_sTagName);
          
            RVCString sFullUrl;
            m_oUrlBase.calcUrl(*psAttrValue, sFullUrl);
            CRVHTMLDietAttributeObject* pAttributeObj = new CRVHTMLDietAttributeObject(*psAttributeName, sFullUrl, *psAttrValue, pnAttrOffset->value());
            if (pAttributeObj)
            {
              pTransObject->InsertAttributeObject(pAttributeObj);
            }
          }
        }
      }
    }
  }
  m_nBuffPos = tagEndIndex;
  if (pTransObject)
  {
    InsertObjectToQueue(pTransObject);
  }
	else if (csTagName.compareTo("FRAME") == 0 || csTagName.compareTo("IFRAME")== 0 )
	{
		//It was IFRAME or FRAME without src attribute.
    pTransObject = new CRVHTMLDietTransactionsObject();
		pTransObject->SetTagName(csTagName);

    RVCString tmp1 = "src";
    RVCString tmp2 = "";
    RVCString tmp3 = "";
    //CRVHTMLDietAttributeObject* pAttributeObj = new CRVHTMLDietAttributeObject(RVCString("src"), RVCString(""), RVCString(""), -1);
    CRVHTMLDietAttributeObject* pAttributeObj = new CRVHTMLDietAttributeObject(tmp1, tmp2, tmp3, -1);
    if (pAttributeObj)
    {
			pTransObject->InsertAttributeObject(pAttributeObj);
    }
		InsertObjectToQueue(pTransObject);
	}
  //now clean up
  attrValuesDict.clearAndDestroy();
  attrOffsetsDict.clearAndDestroy();
}



// @@Chaim - rolling back ParseCSS fix for IMPORT ""
// I think I can leave the changes in place as long as ParseCSS isn't called
void CHTMLOffsetParser::ParseCSS(const RVCString& p_sData)
{
	m_nBuffPos = 0;
	m_sBuffer.remove(0);

  RVCString StyleBuff;
  size_t BuffIndex = 0;
  size_t beginUrl, beginUrl1, beginUrl2, beginUrlLength, endUrl, beginComment, endComment, beginImport, endImport;
  RVCString url, beforeUrl, attribute;
  enum styleType {COMMENT, IMPORT, URL};
  styleType state;
  
  StyleBuff = p_sData;
  
  while(1) {
		/*! currently css parsing has only one purpose:
    *  find external objects that we need to bring automatically (GET method)
    *  only two types of such objects known to me (Alex): 
    *  1) @import("url string");
    *  2) url("url string")
    *  C-style comments should be omitted with no doubt
    */
    beginComment = StyleBuff.index("/*", BuffIndex);
    beginImport  = StyleBuff.index("@import", BuffIndex, RVCString::ignoreCase);
    beginUrl1     = StyleBuff.index("url(", BuffIndex, RVCString::ignoreCase);
    beginUrl2     = StyleBuff.index("\"", BuffIndex, RVCString::ignoreCase);

    if (beginUrl1 != RV_NPOS){
      beginUrl = beginUrl1;
      beginUrlLength = 4;
    }
    else {
      beginUrl = beginUrl2;
      beginUrlLength = 1;
    }
    
    if (beginComment != RV_NPOS 
      && (beginImport == RV_NPOS  || beginComment < beginImport)
      && (beginUrl    == RV_NPOS  || beginComment < beginUrl))
      state = COMMENT;
    else if (beginImport != RV_NPOS && beginUrl != RV_NPOS
      && beginImport < beginUrl)
      state = IMPORT;
    else if (beginUrl != RV_NPOS)
      state = URL;
    else
    {
      return;
    }
     
    
    if(state == URL) {
      beginUrl += beginUrlLength;
      if (beginUrl1 != RV_NPOS){
      if ((endUrl = StyleBuff.index(")", beginUrl)) == RV_NPOS)
      {
        //! no closing ")"
        return;
      }
      }
      else {
        if ((endUrl = StyleBuff.index("\"", beginUrl)) == RV_NPOS)
        {
          //! no closing ")"
          return;
        }
      }
      beforeUrl = StyleBuff(0, beginUrl);
      int beginProp, endProp;
      if ((beginProp = endProp = beforeUrl.last(':')) !=RV_NPOS) {
        while(beginProp >= 0) {
          switch (beforeUrl[beginProp])
          {
          case ' ': case '\n': case '\r': case '\t': case ';': case '{':
            break;
          default: --beginProp; continue;
          }	
          ++beginProp; break;
        }
        
        if(beginProp < 0)
            beginProp = 0;

        attribute = beforeUrl(beginProp, endProp - beginProp);
      }
      url = StyleBuff(beginUrl, endUrl - beginUrl);
      switch (StyleBuff(beginUrl)) {
      case '"':
        ++beginUrl;
        url = url.strip(RVCString::both, '"');
        break;
      case '\'':
        ++beginUrl;
        url = url.strip(RVCString::both, '\'');
        break;
        
      }
      
      if (!url.isNull() && url(0) != '#')
      {
        CRVHTMLDietTransactionsObject* pTransObj = NULL;
        pTransObj = new CRVHTMLDietTransactionsObject();
        pTransObj->SetTagName("IMG");
        RVCString sFullUrl;
        m_oUrlBase.calcUrl(url, sFullUrl);
        RVCString sAttrName("src");
        CRVHTMLDietAttributeObject* pAttributeObj = new CRVHTMLDietAttributeObject(sAttrName, sFullUrl, url, m_nBuffPos + beginUrl);
        pTransObj->InsertAttributeObject(pAttributeObj);
        InsertObjectToQueue(pTransObj);
      }
      BuffIndex = endUrl + 1;
      
    } else if(state == IMPORT) {
      beginUrl += beginUrlLength;
      if (beginUrl1 != RV_NPOS){
        if ((endUrl = StyleBuff.index(")", beginUrl)) == RV_NPOS)
        {
      //! no closing ")"
          return;
        }
      }
      else {
        if ((endUrl = StyleBuff.index("\"", beginUrl)) == RV_NPOS)
      {
          //! no closing ")"
        return;
      }
      }
      if ((endImport = StyleBuff.index(";", beginImport)) == RV_NPOS)
      //! no ';' after @import
      {
        return;
      }
      if (beginUrl > endImport || endUrl > endImport || beginUrl > endUrl) {
        BuffIndex = endImport + 1; //! something bad, point after ";"
        continue; // 
      }
      url = StyleBuff(beginUrl, endUrl - beginUrl);
      switch (StyleBuff(beginUrl)) {
      case '"':
        ++beginUrl;
        url = url.strip(RVCString::both, '"');
        break;
      case '\'':
        ++beginUrl;
        url = url.strip(RVCString::both, '\'');
        break;
      }
      if (!url.isNull())
      {
        CRVHTMLDietTransactionsObject* pTransObj = new CRVHTMLDietTransactionsObject();
        pTransObj->SetTagName("IMG");
        RVCString sFullUrl;
        m_oUrlBase.calcUrl(url, sFullUrl);
        RVCString sAttrName("src");
        CRVHTMLDietAttributeObject* pAttributeObj = new CRVHTMLDietAttributeObject(sAttrName, sFullUrl, url, m_nBuffPos + beginUrl);
        pTransObj->InsertAttributeObject(pAttributeObj);
        InsertObjectToQueue(pTransObj);
      }
      BuffIndex = endImport + 1;
      
    } else { //! if(state == COMMENT)  
      endComment   = StyleBuff.index("*/", BuffIndex);
      //IE and Mozilla also accept inversed comments, */ comment........./*
      if (endComment == RV_NPOS)
      {
        //script error in css, we return
        return;
      }
      else if (endComment > beginComment)
      {
        BuffIndex = endComment + 2;
    }
      else  if (endComment < beginComment)
      {
        BuffIndex = beginComment + 2;
      }
      //endComment = StyleBuff.index("*/", beginComment);
      //if (endComment == RV_NPOS)
      //  BuffIndex = beginComment;
      //else
      //  BuffIndex = endComment + 2;
    }
  }
}


void CHTMLOffsetParser::InsertObjectToQueue(CRVHTMLDietTransactionsObject* p_pTransactionsObject)
{
  if (!p_pTransactionsObject)
  {
    return;
  }
  if (m_pDietVirtualQueue)
  {
    m_pDietVirtualQueue->insert(p_pTransactionsObject);
  }
  m_oOrderedTrans.insert(p_pTransactionsObject);
}


void CHTMLOffsetParser::PrintAll()
{
  int nLength = m_oOrderedTrans.entries();
  cout<<"Queue contains "<<nLength<<" objects"<<endl;
  for (int i = 0 ; i < nLength ; i++)
  {
    RVCollectable* pTrans = m_oOrderedTrans.at(i);
    CRVHTMLDietTransactionsObject* pTransObj = dynamic_cast<CRVHTMLDietTransactionsObject*>(pTrans);
    cout<< "===== NUMBER: "<<i << "\t ===== TagName: "<<pTransObj->GetTagName()<<endl;
    cout<< "=== First Attr Name "<<pTransObj->GetFirstAttributeObject()->GetAttributeName()<<endl;
    pTransObj->PrintAll();
  }  
}


