/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlbuffpart.h"

CWLBuffPart::~CWLBuffPart()
{
	if( privateBuffer )
		free(privateBuffer);
}

void CWLBuffPart::resetPart()
{
	if( privateBuffer )
		free(privateBuffer);
	originBuffer = NULL;
	startLocation = 0;
	buffLength = 0;
	privateBuffer = NULL;
}

void CWLBuffPart::setPart(const char *buff, int start, int len)
{
	if( privateBuffer ) {
		free(privateBuffer);
		privateBuffer = NULL;
	}
	originBuffer = buff;
	startLocation = start;
	buffLength = len;
}


void CWLBuffPart::updatePart(const char *newBuff)
{
	if( privateBuffer )
		free(privateBuffer);

	privateBuffer = strdup(newBuff);
	startLocation = 0;
	buffLength = strlen(privateBuffer);
}

void CWLBuffPart::getPart(const char **part, int *l)
{
	if( privateBuffer )
		*part = privateBuffer;
	else
		*part = originBuffer+startLocation;
	
	*l = buffLength;
}


const char *CWLBuffPart::getPart()
{
	if( ! privateBuffer )
	{
		privateBuffer = (char *)malloc(buffLength + 1);
		strncpy(privateBuffer,originBuffer+startLocation,buffLength);
		privateBuffer[buffLength] = '\0';
	}

	return privateBuffer;
}


int CWLBuffPart::compare(const char *str, int l, bool p_bCaseInsensitive)
{
	const char *part;
	int res;

	typedef int (*str_comparator)(const char*, const char*, size_t);
	str_comparator comparator = &strncmp;
	if (p_bCaseInsensitive)
		comparator = &strnicmp;
	
	if( privateBuffer )
		part = privateBuffer;
	else
		part = originBuffer+startLocation;

	if( buffLength < l ) //! part is shorter than string
	{
        res = comparator(part,str,buffLength);
		if( res == 0 ) //! equal
			return -1;  //! part is less than str
		else
			return res;
	}
	else if( buffLength > l )
	{
        res = comparator(part,str,l);
		if( res == 0 ) //! equal
			return 1;  //! part is greater than str
		else
			return res;
	}
	else
    {
        return comparator(part,str,l);
    }
}


int CWLBuffPart::compare(const char *str, bool p_bCaseInsensitive)
{
	int l = strlen(str);
	return compare(str,l, p_bCaseInsensitive);
}

int CWLBuffPart::compare(RVCString& cstr, bool p_bCaseInsensitive)
{
	int l = cstr.length();
	return compare(cstr.data(),l, p_bCaseInsensitive);
}

int CWLBuffPart::compare(CWLBuffPart& part, bool p_bCaseInsensitive)
{
	const char *str;
	int   str_len;

	part.getPart(&str, &str_len);

	return compare(str, str_len, p_bCaseInsensitive);
}


