/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlurlbase.h"
#include "rvhtmlparseutils.h"



urlBase::urlBase() :
m_Parsed(false),
m_subDirCount (0)
{
}

urlBase::urlBase( const RVCString& p_sNewBase ) :
m_Parsed(false),
m_subDirCount (0)
{
	setBase( p_sNewBase );
}

int urlBase::setBase( const RVCString& newBase_ )
{
	size_t hashPos = newBase_.first('#');
	if(hashPos != RV_NPOS)
    m_OrigBase = newBase_(0,hashPos);
  else
    m_OrigBase = newBase_;
	
  //! Take out all cr,lf and blanks from both sides (inside blanks are url-encoded)
  stripUrl (m_OrigBase);
	
	
	size_t firstQMark = m_OrigBase.first('?');
	if(firstQMark != RV_NPOS)
    m_base = m_OrigBase(0,firstQMark);
  else
    m_base = m_OrigBase;
	
  m_Parsed = false;
	
  return 1;
}


int urlBase::parse()
{
  if (m_Parsed)
    return 1;
	
  m_Parsed = true;
	
  RVCString newBase(m_base.data());
	
  if(newBase.index("http",0,RVCString::ignoreCase) == RV_NPOS) //see that it is not <base target =....> 
		return -1 ;
  //base = newBase_.data();
  m_subDirCount = 0;
  m_host.remove(0);
  m_path.remove(0);
  size_t indx = newBase.index("http://",0,RVCString::ignoreCase);
  size_t indxs = newBase.index("https://",0,RVCString::ignoreCase);
  if(indx == RV_NPOS && indxs == RV_NPOS) // buggy ??? != 0 check better ??? oooo
		return -1;
  m_protocol= (indxs == RV_NPOS ? "http:":"https:");
  newBase.remove(0, newBase.index("//") + 2);
  indx = newBase.index("/");
  if(indx == RV_NPOS)
  {
		m_host.append(newBase,newBase.length());
		m_path = "/";
		return 1;
  }
  m_host = newBase(0,indx);
  newBase.remove(0,indx);
  while((indx = newBase.index("/")) != RV_NPOS)
  {
    ++m_subDirCount;
    m_path.append(newBase, indx + 1);
    newBase.remove(0,indx + 1);
  }
  if(m_path == "") 
    m_path.append("/");
  m_file = newBase(0,newBase.length());
  newBase.remove(0);
  return 0;
}

void urlBase::calcUrl(const RVCString& srcUrl, RVCString& destUrl, bool bCalcEmpty/*false*/, bool bUnEscapeURL/*true*/)
{   
  // 
  //yossi - when srcUrl="" there`s a difference between IE to Mozilla. IE performs recalc 
  // to base, and performs navigate, Mozilla does not do nothing. In this fix here we do like IE.
  //
  // Alon: 24-11-2004 change getProtocol interface to return (out parameter) protocol length 
  // instead of the string it self
  //
	
  if (!bCalcEmpty && srcUrl.isNull())
    return;
  
	parse();
	
	destUrl = srcUrl;
	
	//! In any case of the url we remove
	//! the blanks from both sides and also inside cr and lf
  stripUrl (destUrl, bUnEscapeURL);
	
  if (destUrl.length()>1)
  {
		if (destUrl(0) == '/' && destUrl(1) == '/') //! "//" replaced to protocol://
			destUrl.prepend(m_protocol);
		
		if (destUrl(0) == '\\' && destUrl(1) == '\\') //! "\\" replaced to file://
    { 
			destUrl.remove(0,2);
			destUrl.prepend("file://");
		}
  }	
	
	//RVCString sProtocol;
  int nProtocolLen = -1;
	switch(getProtocol(destUrl, nProtocolLen)) { //! check for http/s at the start of relUrl
	case HTTP_protcl: case HTTPS_protcl:
		
    //if (destUrl(sProtocol.length()) == '/' && destUrl(sProtocol.length()+1) == '/')
    if (destUrl(nProtocolLen) == '/' && destUrl(nProtocolLen+1) == '/')
    {
      //Sergei fix for bug 12990 for the case of http://hostname:/path1/file.htm
      //I will just remove second :
      /*size_t secondSemicolPos = destUrl.index(":", sProtocol.length() + 1);
      if (secondSemicolPos != RV_NPOS && destUrl(secondSemicolPos + 1) == '/')
      {
			destUrl.remove(secondSemicolPos,1);
      }
			
			 CDOMUrl temp(destUrl);
			 temp.AnalyzeHref();
      destUrl = temp.Href_();*/
      //@@Sergei!!! Very important three lines above were inserted as bug fix and I removed
      //them in order to separate urlbase from rvhtmldom
			return;
    }	else 
    {
			//destUrl.remove(0, sProtocol.length());
			destUrl.remove(0, nProtocolLen);
    }
		break;
	case FILE_protcl:
		//if (destUrl(sProtocol.length()+1) != '/') {
		//	destUrl.remove(0, sProtocol.length());
		  if (destUrl(nProtocolLen+1) != '/') {
			  destUrl.remove(0, nProtocolLen);
			if (destUrl(0) == '/')
				destUrl.prepend("file://");
			else
				destUrl.prepend("file:///");
		}
		return;
		
	case JAVASCRIPT_protcl: case MAILTO_protcl: case FTP_protcl: case OTHER_protcl:
		return;
	}
	
  size_t tempPos;
  if (destUrl.length() >= 2) {
		// Remove all "./" from dest URL
		if (destUrl(0, 2) == "./")
			destUrl.remove(0, 2);
		
    tempPos = RV_NPOS;
		while ( (tempPos = destUrl.index("/./")) != RV_NPOS)
		{
			destUrl.remove(tempPos + 1,2);
		}
  }
	
  
  //Bug 14502 and 16454 
	tempPos = RV_NPOS;
  while ( (tempPos = destUrl.index('\\')) != RV_NPOS)
  {
    destUrl[tempPos] = '/';
  }
  size_t uUrlLength = destUrl.length();
	if((uUrlLength>0) && (destUrl(0)=='/'))      //! start from host
	{
		destUrl.prepend(m_host);
		destUrl.prepend("//");
		destUrl.prepend(m_protocol);
  }
	else if ((uUrlLength>0) && (destUrl(0)=='#' || destUrl(0)=='?') )	// both cases should append the QS to url after the filename
     destUrl.prepend(m_base);
    //destUrl.prepend(m_OrigBase);//if there is already QS in the previous navigations it causes problems
	else
  {
    destUrl.prepend(m_path);
    destUrl.prepend(m_host);
    destUrl.prepend("//");
    destUrl.prepend(m_protocol);
	}
	
  size_t posPathStart = destUrl.index(m_host);
  posPathStart += m_host.length();
  
  tempPos = RV_NPOS;
  while ( (tempPos = destUrl.index("../", posPathStart)) != RV_NPOS)
  {
    //if pass as parameter to CGI Etc. 
    size_t posQueryStringStart = destUrl.index('?');
    if (tempPos>posQueryStringStart)
      break;

    // if not in start of path - ignore
    //if (tempPos>0)
    //  break;


    if (tempPos == (posPathStart +1)) // the URL is http://hostname/../path/file.htm - just remove "../"
      destUrl.remove(posPathStart, 3);
    else // the URL is http://hostname/subpath1/../subpath/file.html - remove "subpath1/../"
    {
      destUrl.remove(tempPos, 3);
      size_t posPrevPath = RVCString(destUrl(0,tempPos - 1)).last('/');
      destUrl.remove(posPrevPath + 1, tempPos - posPrevPath - 1);
    }
  }    
  
  //*****************************************
  //Now we have to take case of all "../" in the URL
  /*
	if(destUrl(0,3) =="../")
	{
		RVCString tempPath(m_path);
		int subDirPtr=0;
		while(destUrl(0,3) == "../") //! how far should we go back in the sub dir hierarchy
		{
		++subDirPtr;
		destUrl.remove(0,3);
		}
		
		 if (subDirPtr+1>=m_subDirCount)
		 tempPath="/";
		 else
		 {
		 int counter = 0;  //!  counts "../"
		 int indx = 0 ;
		 while(m_subDirCount-subDirPtr-1>counter)
		 {
		 indx = tempPath.index("/",indx +1);
		 ++counter;
		 }
		 tempPath.remove(indx+1);
		 }
		 destUrl.prepend(tempPath);
		 destUrl.prepend(m_host);
		 destUrl.prepend("//");
		 destUrl.prepend(m_protocol);
		 return;
		 }
  */
  //********************************************
  
	
}



