/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// FileAutoDiscovery.cpp: implementation of the CFileAutoDiscovery class.
//
//////////////////////////////////////////////////////////////////////

#include "FileAutoDiscovery.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFileAutoDiscovery::CFileAutoDiscovery()
{
}

CFileAutoDiscovery::~CFileAutoDiscovery()
{

}


bool CFileAutoDiscovery::handleFile( const char* filePath )
{
	printf( "\nFile parsed: %s\n", filePath );

	CWLErrorInfo errorInfo;
#ifdef USE_MSXML
	CRVXMLDocument XMLDoc(MSXMLParser);
#else
	CRVXMLDocument XMLDoc;
#endif //USE_MSXML

	// load XML file
	XMLDoc.load( filePath, errorInfo );
	if( errorInfo.GetErrorId() )
		return false;

	// parse XML file
	CRVXMLElement rootElement = XMLDoc.get_documentElement( errorInfo );
	if( errorInfo.GetErrorId() )
		return false;

	if( rootElement.is_empty( errorInfo ) )
		return true;

	RVCString rootName = rootElement.get_name( errorInfo );
	if( errorInfo.GetErrorId() )
		return false;

	CRVXMLElementList children = rootElement.get_childElements( errorInfo );
	if( errorInfo.GetErrorId() )
		return false;

	RVCString NodeName;
	RVCString DllName;
	RVCString DllDisplayName;
	RVCString DllVersion;
	RVCString DllVendor;
	for( int i = 0; i < children.entries(); i++ ){
		NodeName = "";
		RVCString NodeName = children[i].get_name( errorInfo );
		if( errorInfo.GetErrorId() || 0 != strcmp( "Dll", NodeName.data() ) )
			continue;

		// get Dll info
		DllName = "";
		DllDisplayName = "";
		DllVersion = "";
		DllVendor = "";
		if( !getDllInfo( &(children[i]), DllName, DllDisplayName, DllVersion, DllVendor ) )
			continue;

		// handle Dll parameters
		printf( "\n%s | %s | %s | %s\n", DllName, DllDisplayName, DllVersion, DllVendor );
	}

	return true;
}


const char* CFileAutoDiscovery::getLastError()
{
	return m_LastErrorDescription.data();
}


bool CFileAutoDiscovery::discoverPath( const char* path, const char* spec, bool lookInSubDirs )
{
	return FindFile( path, spec, lookInSubDirs, this );
}


bool CFileAutoDiscovery::getDllInfo( CRVXMLElement* pDllNode, RVCString& DllName, RVCString& DllDisplayName, RVCString& DllVersion, RVCString& DllVendor )
{
	CWLErrorInfo errorInfo;

	DllName = pDllNode->get_value( errorInfo );
		if( errorInfo.GetErrorId() )
			return false;

	CRVXMLElementList children = pDllNode->get_childElements( errorInfo );
	if( errorInfo.GetErrorId() )
		return false;

	for( int i = 0; i < children.entries(); i++ ){
		RVCString DllShowName = children[i].get_name( errorInfo );
		if( errorInfo.GetErrorId() )
			return false;

		if( 0 == strcmp( "FileName", DllShowName.data() ) ){
			DllName = children[i].get_value( errorInfo );
			if( errorInfo.GetErrorId() )
				return false;
			continue;
		}

		if( 0 == strcmp( "DisplayName", DllShowName.data() ) ){
			DllDisplayName = children[i].get_value( errorInfo );
			if( errorInfo.GetErrorId() )
				return false;
			continue;
		}

		if( 0 == strcmp( "Version", DllShowName.data() ) ){
			DllVersion = children[i].get_value( errorInfo );
			if( errorInfo.GetErrorId() )
				return false;
			continue;
		}

		if( 0 == strcmp( "Vendor", DllShowName.data() ) ){
			DllVendor = children[i].get_value( errorInfo );
			if( errorInfo.GetErrorId() )
				return false;
		}
	}

	return true;
}
