
/*
 * JavaScript API.
 */
#include "jsstddef.h"
#include <ctype.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "jstypes.h"
#include "jsarena.h" /* Added by JSIFY */
#include "jsutil.h" /* Added by JSIFY */
#include "jsclist.h"
#include "jsdhash.h"
#include "jsprf.h"
#include "jsapi.h"
#include "jsarray.h"
#include "jsatom.h"
#include "jsbool.h"
#include "jscntxt.h"
#include "jsconfig.h"
#include "jsdate.h"
#include "jsemit.h"
#include "jsexn.h"
#include "jsfun.h"
#include "jsgc.h"
#include "jsinterp.h"
#include "jslock.h"
#include "jsmath.h"
#include "jsnum.h"
#include "jsobj.h"
#include "jsopcode.h"
#include "jsparse.h"
#include "jsregexp.h"
#include "jsscan.h"
#include "jsscope.h"
#include "jsscript.h"
#include "jsstr.h"

#if JS_HAS_FILE_OBJECT
#include "jsfile.h"
#endif

#include "rvjsapi.h"

JS_PUBLIC_API(JSBool) RVJS_IsInstanceOf( JSContext* cx, jsval val, JSClass* clasp )
{
	if( JSVAL_IS_OBJECT( val ) ) {
		if( !JSVAL_IS_NULL( val ) ) {
		if( clasp == JS_GetClass( cx, JSVAL_TO_OBJECT( val ) ) ) {
			return JS_TRUE;
			}
		}
	}
	return JS_FALSE;
}

JS_PUBLIC_API(JSBool) RVJS_IsFunction( JSContext* cx, jsval val )
{
	return RVJS_IsInstanceOf( cx, val, &js_FunctionClass );
}

JS_PUBLIC_API(int) RVJS_AtomicIntCounter( int* ArgList )
{
//	return PR_RVJSAtomicIntCounter( ArgList );
	return 0;
}


JS_PUBLIC_API(JSBool) RVJS_DefaultValue( JSContext* cx, jsval v, jsval* vp )
{
	JSBool rtn = JS_TRUE;

	if( JSVAL_IS_OBJECT( v ) ) {
		if( JSVAL_IS_NULL( v ) ) {
			*vp = v;
		} else {
			rtn = js_TryValueOf( cx, JSVAL_TO_OBJECT( v ), JSTYPE_VOID, vp );
		}
	} else {
		*vp = v;
	}

	return rtn;
}


JS_PUBLIC_API(void*) RVJS_GetPrivate( JSContext* cx, JSObject* obj )
{
	void* rtn = NULL;
	if( OBJ_GET_CLASS(cx, obj)->flags & JSCLASS_HAS_PRIVATE ) {
		rtn = JS_GetPrivate( cx, obj );
	}
	return rtn;
}

JS_PUBLIC_API(JSBool) RVJS_DefinePropertyId(JSContext *cx, JSObject *obj, jsval id, jsval value, JSPropertyOp getter, JSPropertyOp setter, uintN attrs)
{
	JSBool rtn = JS_FALSE;
	if( JSVAL_IS_STRING( id ) ) {
		JSString* idStr = JSVAL_TO_STRING( id );
		rtn = JS_DefineUCProperty( cx, obj, JS_GetStringChars( idStr ), JS_GetStringLength( idStr ), value, getter, setter, attrs );
	} else if( JSVAL_IS_INT( id ) ) {
		JS_ASSERT( 0 );
	}
	return rtn;
}



#define JSVAL_INT_MAX_STRING "1073741823"
JS_PUBLIC_API(JSBool) RVJS_DefineProperty(JSContext *cx, JSObject *obj, jsval id, jsval value, JSPropertyOp getter, JSPropertyOp setter, uintN attrs)
{

	{
        if (!JSVAL_IS_INT(id)) {                                              \
            JSAtom *_atom = (JSAtom *)id;                                     \
            JSString *_str = ATOM_TO_STRING(_atom);                           \
            const jschar *_cp = _str->chars;                                  \
            JSBool _negative = (*_cp == '-');                                 \
            if (_negative) _cp++;                                             \
            if (JS7_ISDEC(*_cp) &&                                            \
                _str->length - _negative <= sizeof(JSVAL_INT_MAX_STRING) - 1) \
            {                                                                 \
                jsuint _index = JS7_UNDEC(*_cp++);                            \
                jsuint _oldIndex = 0;                                         \
                jsuint _c = 0;                                                \
                if (_index != 0) {                                            \
                    while (JS7_ISDEC(*_cp)) {                                 \
                        _oldIndex = _index;                                   \
                        _c = JS7_UNDEC(*_cp);                                 \
                        _index = 10 * _index + _c;                            \
                        _cp++;                                                \
                    }                                                         \
                }                                                             \
                if (*_cp == 0 &&                                              \
                    (_oldIndex < (JSVAL_INT_MAX / 10) ||                      \
                     (_oldIndex == (JSVAL_INT_MAX / 10) &&                    \
                      _c <= (JSVAL_INT_MAX % 10)))) {                         \
                    if (_negative) _index = 0 - _index;                       \
                    id = INT_TO_JSVAL((jsint)_index);                         \
                }                                                             \
            } else {                                                          \
            }                                                                 \
        }                                                                     \
	}


    return OBJ_DEFINE_PROPERTY(cx, obj, id, value, getter, setter, attrs, NULL);
}


JS_PUBLIC_API(JSBool) RVJS_ReportUncaughtException( JSContext *cx )
{
	JSBool rtn = js_ReportUncaughtException(cx);
	return rtn;
}

JS_PUBLIC_API(void) RVJS_Assert( JSBool cond )
{
	JS_ASSERT( cond );
}


#ifdef JS_PARANOID_REQUEST
JS_PUBLIC_API(JSContext *)
RVJS_NewContextThreadSafe(JSRuntime *rt, size_t stackChunkSize, JSBool isCheckRequest )
{
    JSContext* rtn = js_NewContext(rt, stackChunkSize);
		if( rtn ) {
			rtn->isCheckRequest = isCheckRequest;
		}
	return rtn;
}

JS_PUBLIC_API(JSContext *)	
RVJS_NewContext(JSRuntime *rt, size_t stackChunkSize )
{
	return RVJS_NewContextThreadSafe( rt, stackChunkSize, JS_TRUE );
}

#endif

JS_PUBLIC_API(JSReportErrorFilenameCallback)
RVJS_SetReportErrorFilenameCallbackRT(JSRuntime *rt, JSReportErrorFilenameCallback cb)
{
    JSReportErrorFilenameCallback oldcb;

    oldcb = rt->reportErrorFilenameCallback;
    rt->reportErrorFilenameCallback = cb;
    return oldcb;
}

JS_PUBLIC_API(JSCompileErrorCallback)
RVJS_SetCompileErrorCallbackRT(JSRuntime *rt, JSCompileErrorCallback cb)
{
    JSCompileErrorCallback oldcb;

    oldcb = rt->compileErrorCallback;
    rt->compileErrorCallback = cb;
    return oldcb;
}

JS_PUBLIC_API(JSReferenceErrorCallback)
RVJS_SetReferenceErrorCallbackRT(JSRuntime *rt, JSReferenceErrorCallback cb)
{
    JSReferenceErrorCallback oldcb;

    oldcb = rt->referenceErrorCallback;
    rt->referenceErrorCallback = cb;
    return oldcb;
}

JS_PUBLIC_API(JSThreadIdCallback)
RVJS_SetThreadIdCallbackRT(JSRuntime *rt, JSThreadIdCallback cb)
{
    JSThreadIdCallback oldcb = NULL;
#ifdef DEBUG
    oldcb = rt->threadIdCallback;
    rt->threadIdCallback = cb;
#endif
    return oldcb;
}

JS_PUBLIC_API(JSExtraWhiteSpaceCallback)
RVJS_SetExtraWhiteSpaceCallbackRT(JSRuntime *rt, JSExtraWhiteSpaceCallback cb)
{
    JSExtraWhiteSpaceCallback oldcb;

    oldcb = rt->extraWhiteSpaceCallback;
    rt->extraWhiteSpaceCallback = cb;
    return oldcb;
}

JS_PUBLIC_API(JSReservedKeywordsCallback)
RVJS_SetReservedKeywordsCallbackRT(JSRuntime *rt, JSReservedKeywordsCallback cb)
{
    JSReservedKeywordsCallback oldcb;

    oldcb = rt->reservedKeywordsCallback;
    rt->reservedKeywordsCallback = cb;
    return oldcb;
}

JS_PUBLIC_API(uint32) RVJS_GetAtomTableNentries( JSContext *cx )
{
		uint32 rtn = cx->runtime->atomState.table->nentries;
		return rtn;
}


JS_PUBLIC_API(const char**) RVJS_HashTableMeterNames()
{
	static const char* sNames[] = { "elem", "add", "rawadd", "remove", "lookup", "step", "grow", "shrink" };
	return sNames;
}


JS_PUBLIC_API(void) RVJS_HashTableMeter( 
	JSContext* rCx, 
	uint32 rTable, 
	uint32* roSize, 
	uint32* oMeters )
{
	
	JSHashTable* aTable = NULL;
	int aI = 0;		

	switch( rTable ) {
	case 0:
		aTable = rCx->runtime->atomState.table;
		break;
	default:
		JS_ASSERT(0);
	}
	
	oMeters[aI++] = aTable->nentries;
#ifdef HASHMETER
	oMeters[aI++] = aTable->nentriesMax;
	oMeters[aI++] = aTable->nadds;
	oMeters[aI++] = aTable->nrawadds;
	oMeters[aI++] = aTable->nremoves;
	oMeters[aI++] = aTable->nlookups;
	oMeters[aI++] = aTable->nsteps;
	oMeters[aI++] = aTable->ngrows;
	oMeters[aI++] = aTable->nshrinks;
#endif		
	*roSize = aI;
}


extern JS_PUBLIC_API(void) RVJS_StateInfo( JSContext* rCx, JSObject** oScopeChain )
{
	int aI = 0;
	JSObject* aObj;

	for( aObj = rCx->fp->scopeChain; aObj; aObj = JS_GetParent( rCx, aObj ) ) {
		oScopeChain[ aI ] = aObj;
		++aI;
	}

	oScopeChain[ aI ] = NULL;
}

// @@Chaim - needs to be called from BeginEndRequest - for now I'm wrapping it in htmlparse
// but the entire code should probably be moved there as only the code in if (JSVAL_IS_STRING(v))
// requires BeginEndRequest.
JS_PUBLIC_API(JSBool) RVJS_JSVAL_To_Int( JSContext* cx, jsval v, int *pVal ){
  JSBool bRet = JS_TRUE;
  if (JSVAL_IS_INT(v)){
    *pVal = JSVAL_TO_INT(v);
  }
  else {
    if (JSVAL_IS_STRING(v)){
    JSString* timeoutValue = JS_ValueToString(cx, v);
    char* timeoutValueString = JS_GetStringBytes(timeoutValue);

    // Note that when string is not a valid number, 0 will be returned
    *pVal = atoi(timeoutValueString);

    // Check if the string was really an integer
    // If the rsult of atoi != 0 then it was a valid string; or if the result is zero
    // but the string was "0"
    if (!(*pVal) && strcmp(timeoutValueString, "0")){
      bRet = JS_FALSE;
    }

    // Check for case when the value is so large, it overflows to
    // negative
    if (*pVal < 0){
      *pVal = JSVAL_INT_MAX;
    }
    }
    else {
      bRet = JS_FALSE;
    }

  }
  return bRet;
}


JS_PUBLIC_API(JSBool) RVJS_IsCallOpNext( JSContext* cx ){
  return cx->RVIsCallOpNext;
}
