/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// WLStatGenMeasurement.cpp

#include <math.h>
#include <thr.h>

#include "tracedll.h"
#include "wllightstring.h"
#include "tvpnumconversions.h"
#include "wlstatisticsgenerator.h"
#include "wlstatmanagerglobals.h"
 

//////////////////////////////////////////////////////////////////
//                  `         CWLStatGenMeasurement
//////////////////////////////////////////////////////////////////

CWLStatGenMeasurement::CWLStatGenMeasurement(  CWLStatPointTypeEnum::PointType type):typeName(type)
{
 Reset();
}

void CWLStatGenMeasurement::Reset()
{
	array[tminimum] = -1; 
	array[tmaximum] = -1; 
	array[tcount] = 0; 
	array[tsum] = 0;
  array[tsumSqr] = 0;
	array[tstddev] = 0;
	array[taverage]=-1;

	array[tlastmin] = -1; 
	array[tlastmax] = -1; 
	array[tlastcount] = 0; 
	array[tlastsum] = 0;
  array[tlastsumSqr] = 0;
	array[tlastaverage]= -1;
}

void CWLStatGenMeasurement::Print()
{
	cout << " Type " << GetType();
  cout << " minimum = " << array[tminimum] << " maximum = " << array[tmaximum];
  cout << " count = " << array[tcount] << " sum = " << array[tsum] << endl;

	cout << "lasts: minimum = " << array[tlastmin] << " maximum = " << array[tlastmax];
  cout << " count = " << array[tlastcount] << " sum = " << array[tlastsum] << endl;

}

void CWLStatGenMeasurement::AddLast(double l)
{
  WlTraceMsg(" ----------------- ");	
  WlTraceMsg(" CWLStatGenMeasurement::AddLast ( %d )",(int)l);
  
  if (l < array[tminimum]  ||  array[tminimum] == -1)
    array[tminimum] = l;
  if (l > array[tmaximum] || array[tmaximum]==-1 )
    array[tmaximum] = l;

  array[tcount]+=1;
  array[tsum] += l;
  array[tsumSqr] += ( l * l);

  WlTraceMsg(" tsum = %d    tsumSqr = %d\n",(int)array[tsum], (int)array[tsumSqr]);
  WlTraceMsg(" ----------------- ");	
	
	if (l < array[tlastmin] || array[tlastmin] == -1)
		array[tlastmin] = l;
	if (l > array[tlastmax] || array[tlastmax]== -1)
		array[tlastmax] = l;

	array[tlastcount]+=1;
	array[tlastsum] += l;
  array[tlastsumSqr] += ( l * l );
}

// sum 2 measurment
void CWLStatGenMeasurement::AddValues(double min, double max, int cnt, double s, double sSqr,
                                 double l_min, double l_max, int l_cnt, double l_sum, double l_sumSqr, AddType type)
{
  WlTraceMsg(" ----------------- "); 
  WlTraceMsg(" CWLStatGenMeasurement::AddValues (sum = %d  sSqr = %d)\n", (int)s, (int)sSqr);

	if (cnt == 0)
		return;

  if (min < array[tminimum] || array[tminimum] == -1)
		array[tminimum] = min;
	if (max > array[tmaximum])
		array[tmaximum] = max;
	array[tcount] += cnt;
	array[tsum] += s;
  array[tsumSqr] += sSqr;
  
  WlTraceMsg(" After Add, the values are :");
  WlTraceMsg(" array[tsum] = %d    array[tsumSqr] = %d\n",(int)array[tsum], (int)array[tsumSqr]);

	if (l_cnt == 0)
		return;
	if (l_min < array[tlastmin] || array[tlastmin] == -1)
		array[tlastmin] = l_min;
	if (l_max > array[tlastmax])
		array[tlastmax] = l_max;
	array[tlastcount] += l_cnt;
	array[tlastsum] += l_sum;
  array[tlastsumSqr] += l_sumSqr;
}

void CWLStatGenMeasurement::SetValues(double min, double max, int cnt, double s, double sSqr,
                                      double l_min, double l_max, int l_cnt, double l_sum, double l_sumSqr, 
                                        CWLStatPointTypeEnum::PointType type)
{
  WlTraceMsg(" ----------------- "); 
  WlTraceMsg(" CWLStatGenMeasurement::SetValues "); 
	array[tminimum] = min;
	array[tmaximum] = max;
	array[tcount] = cnt;
	array[tsum] = s;
  array[tsumSqr] = sSqr;
	array[tlastmin] = l_min;
	array[tlastmax] = l_max;
	array[tlastcount] = l_cnt;
	array[tlastsum] = l_sum;
  array[tlastsumSqr] = l_sumSqr;
  typeName=type;

  WlTraceMsg(" After Set, the values are :");
  WlTraceMsg(" array[tsum] = %d    array[tsumSqr] = %d\n",(int)array[tsum], (int)array[tsumSqr]);
  WlTraceMsg(" ----------------- "); 
}

void CWLStatGenMeasurement::ResetLasts()
{
	array[tlastmin] = -1;
	array[tlastmax] = -1;
	array[tlastcount] = 0;
	array[tlastsum] = 0;
  array[tlastsumSqr] = 0;
	array[tlastaverage]=-1;
}

bool CWLStatGenMeasurement::IsResetLasts()
{
	bool  answer;
	answer =  ((array[tlastmin] == -1) && (	array[tlastmax] == -1) && (array[tlastcount] == 0)
					&& (array[tlastsum] == 0) && (array[tlastsumSqr] == 0));
 return answer;
}

void CWLStatGenMeasurement::AppendDump(RVCString &str, const char* name)
{
  //char buf[2048];
	char c=1;

  RVCString num;

  WlTraceMsg(" ----------------- ");
  WlTraceMsg(" AppendDump NAME = %s ",name);
  WlTraceMsg(" array[tsum] = %d    array[tsumSqr] = %d\n", (int)array[tsum], (int)array[tsumSqr]);

  str.append(name);
  str.append(c);
  str.append(CWLStatPointTypeEnum::GetTypeName(typeName));
  str.append(c);
  TVPDoubleToString(array[tminimum], num, 3);
  str.append(num);
  str.append(c);
  TVPDoubleToString(array[tmaximum], num, 3);
  str.append(num);
  str.append(c);
  TVPIntToString((int)(array[tcount]), num);
  str.append(num);
  str.append(c);
  TVPDoubleToString(array[tsum], num, 3);
  str.append(num);
  str.append(c);
  TVPDoubleToString(array[tsumSqr], num, 3);
  str.append(num);
  str.append(c);
  TVPDoubleToString(GetAverage(), num, 3);
  str.append(num);
  str.append(c);

  TVPDoubleToString(array[tlastmin], num, 3);
  str.append(num);
  str.append(c);
  TVPDoubleToString(array[tlastmax], num, 3);
  str.append(num);
  str.append(c);
  TVPIntToString((int)(array[tlastcount]), num);
  str.append(num);
  str.append(c);
  TVPDoubleToString(array[tlastsum], num, 3);
  str.append(num);
  str.append(c);
  TVPDoubleToString(array[tlastsumSqr], num, 3);
  str.append(num);
  str.append(c);
  TVPDoubleToString(GetLastsAverage(), num, 3);
  str.append(num);
  str.append(c);
  WlTraceMsg(" String = %s ",str);
  WlTraceMsg(" ----------------- ");
}


void CWLStatGenMeasurement::DumpLasts(RVCString &str, const char* name)
{
	char c=1;

  RVCString num;

  str.append(name);
  str.append(c);
  str.append(CWLStatPointTypeEnum::GetTypeName(typeName));
  str.append(c);
  TVPDoubleToString(array[tlastmin], num, 3);
  str.append(num);
  str.append(c);
  TVPDoubleToString(array[tlastmax], num, 3);
  str.append(num);
  str.append(c);
  TVPIntToString((int)(array[tlastcount]), num);
  str.append(num);
  str.append(c);
  TVPDoubleToString(array[tlastsum], num, 3);
  str.append(num);
  str.append(c);
  TVPDoubleToString(array[tlastsumSqr], num, 3);
  str.append(num);
  str.append(c);
}

