/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_MESSAGEHANDLERIMP_H
#define __WL_MESSAGEHANDLERIMP_H

#include <collint.h>
#include <ordered.h>

#include "netmsg.h"
#include "platform.h"
#include "tvpsync.h"
#include "tvpeventqueue.h"
#include "tvppriorityqueue.h"
#include "associativearray.h"
#include "testtalkclientobj.h"
#include "netkernelmessagehandler.h"

class CNetMessageHandlerImp;

class CNetHandlerMessage : public RVCollectableInt
{
  friend CNetMessageHandlerImp;
public:
  typedef enum {Wait, NoWait, NoResponse, FileCopy, InfoRequest} SendType;
	
private:
  RVCString m_Message;
  RVCString m_Recipient;
  RVCString m_ErrorMsg;
  long      m_Id;
  int       m_Priority;
  CNetHandlerMessage::SendType m_SendType;

public:                                   /*new parameter*/
  CNetHandlerMessage(const char* Message, size_t MessageLen, const char* Recipient, const char* ErrorMsg)
    : m_Recipient(Recipient), m_Message(Message, MessageLen), m_ErrorMsg(ErrorMsg) {}
  ~CNetHandlerMessage () {}

  
  const char* GetMessage()       {return m_Message.data();}
	size_t      GetMessageLength() {return m_Message.length();}
  const char* GetRecipient()		 {return m_Recipient;}
  const char* GetErrorMsg()      {return m_ErrorMsg;}
  long  GetId() { return m_Id; }
  CNetHandlerMessage::SendType GetSendType() { return m_SendType; }
};

class CNetMessageTTObject : public TestTalkClientObject
{
private:
  CNetMessageHandlerImp* m_MyNetMessageHandler;

public:
  CNetMessageTTObject() {}
  virtual ~CNetMessageTTObject() {}

  void SetMessageHandler(CNetMessageHandlerImp* nmh);

  virtual bool NewMessageEvent(TypedMessageObject* tmo, short qtype);
};

class CNetMessageHandlerImp
{
  friend class CNetMessageHandler;

private:
  CNetMessageHandler*   m_Owner;
	INetMessagesListener** m_MessageListeneres;

  long                  id;
  long                  m_MessageCounter;
  RVCString             m_ErrorMsg;
  RVCString             m_BaseName;
	RVCString             m_LastSendError;

  bool                  m_Initialized;
	bool                  m_UseCallbacksModel;
  double                m_LastRecievedMessageTime;
  unsigned short        m_TestTalkPort;
 
  CNetMessageTTObject   m_TestTalkClient;
  TVPPriorityQueue      m_MessagesQueue;
  TVPEventQueue         m_SentMessages;

	RVThread              *m_SenderThread;
	RVThread              *m_RecieverThread;

private:
  TVPManualEvent        m_SenderStartEvent;
  TVPManualEvent        m_SenderDoneEvent;
  TVPManualEvent        m_RecieverStartEvent;
  TVPManualEvent        m_RecieverDoneEvent;

private:
	void HandleSendMessageError(CNetHandlerMessage* message, bool status = false);

public:
	CNetMessageHandlerImp(CNetMessageHandler* owner);
	~CNetMessageHandlerImp();

	void UseCallbacksModel(bool callbacksModel); // default is true
  bool Init(unsigned short testtalkport=0);
  void Terminate();
  //
  // MainLoop functions:
	void StartSender();
	void SenderMainLoop();
	void StartReciever();
  void HandleEvent(const char* AssoArrayString);
  void HandleAck(TestTalkMessage* ttm);

	void AddMessagesListener(INetMessagesListener* listener);
	void RemoveMessagesListener(INetMessagesListener* listener);

  const char* WhoAmI()           { return m_TestTalkClient.NETWhoAmI(); }
	const char* GetLocalHostName() { return m_TestTalkClient.NETLocalHostName(); }
  bool IsSendQueueEmpty()        { return (m_MessagesQueue.isEmpty() != FALSE); }

  void SendMessage( CNetHandlerMessage::SendType sendtype, CAssociativeArray& Message , const char* SendTo, const char* ErrorMsg, int priority=1);

	//
	// Next methods should be used in no callback methods model
	bool ReadNextMessage(/*[OUT]*/CAssociativeArray& message);

	//
	// Return NULL if none
  const char* GetLastErrorMessage();  
};

#endif //__WL_MESSAGEHANDLERIMP_H

