/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_NETKERNELMESSAGEHANDLER_H
#define __WL_NETKERNELMESSAGEHANDLER_H

#include <RVCString.h>

#include "associativearray.h"
#include "tvppriorityqueue.h"
#include "tvpsync.h"
#include "netmessageslistener.h"


class CNetMessageHandlerImp;
class CNetHandlerMessage;
class INetMessagesListener;


class DllExport CNetMessageHandler
{
private:
	CNetMessageHandlerImp *self;
	
public:
	RVCString             m_SenderName; 
	
	enum CopyFileCompletionState{ CopyFileSucceed, OpenFileFailed, 
															  CloseFileFailed, ReadFileFailed, 
																WriteFileFailed, CopyFileFailed };
public:
	CNetMessageHandler(const char* basename);
	~CNetMessageHandler(void);
	

	//
	// NOTES:
	// 1. This method must be called before Init method
	// 2. The default model is callbacks model
	void UseCallbacksModel(bool callbacksModel); 

	bool Init(unsigned short testtalkport = 0);
	void Terminate(void);
	bool IsActive();
	
	void KillProcess(const char* hostName, const char* processUID, long waitBeforeKill = 0);

	void AddMessagesListener(INetMessagesListener* listener);
	void RemoveMessagesListener(INetMessagesListener* listener);
	
	const char* WhoAmI(void);
	const char* GetLocalHostName(void);

	// This method gets a complete testtalk address and returns only
	// the host name with or with out the net port.
	// Example: testtalkAddress = Monitor275Sender1@10.0.1.58:9000
	//          if hostAndPort = true   HostName = 10.0.1.58:9000
	//          if hostAndPort = false  HostName = 10.0.1.58
	bool ParseHostName(/*[OUT]*/RVCString& hostName,
										 /*[IN]*/const char* testtalkAddress, 
										 /*[IN]*/bool hostAndPort = true);

	double			GetLastRecievedMessageTime(void);
	void				SetLastRecievedMessageTime(double time); 
	
	bool IsSendQueueEmpty(void);
	RVCollectable* waitEvent(void);
	TVPPriorityQueue::WaitReturnType waitEvent(RVCollectable*& event, long waitTime);
	
	// Copies a file from local machine to a specified remote machine
	// and file name.
	// The method sends a request to copy the file and doesn't wait for
	// the copy to finish. Instead when the copy is done, CopyFileDone()
	// method is called and passes information about the copy completion status.
	// Incase of a fatal error (socket error, etc') CopyFileDone() is called 
	// and then SendMessageErrorHandler(). 
	void CopyFile(const char* localFilePath, 
								const char* remoteFilePath, 
								const char* remoteHostName, 
								const char* errorMsg = "CopyFile Failed", int priority = 1);

	void SendMessage(CAssociativeArray& message,
									 const char* sendTo, 
									 const char* errorMsg, int priority = 1);

	void SendMessageNoresponse(CAssociativeArray& message,
														 const char* sendTo, 
														 const char* errorMsg, int priority = 1);

	void SendMessageNowait(CAssociativeArray& message,
												 const char* sendTo,
												 const char* errorMsg, int priority = 1);

	virtual CNetHandlerMessage* SenderWaitMessage();

	// This method is called when a copy file operation is done.
	// The method should be implemented by a sub class of this class.
	virtual void CopyFileDone(CNetMessageHandler::CopyFileCompletionState copyStatus,
														const char* copyStatusFileName,
														const char* localFileName, 
														const char* remoteFileName, 
														const char* remoteHostAddress);

	virtual void SendMessageErrorHandler(CAssociativeArray& message,
																			 const char* sendTo, 
																			 const char* errorMsg, 
																			 bool status);

	virtual void SendMessageSucceded(const char* message,
																	 const char* sendTo);

	virtual void MessageHandler(CAssociativeArray& assoArrayCommand);

	//
	// Next methods should be used in no callbacks model
	bool ReadNextMessage(/*[OUT]*/CAssociativeArray& message);

  const char* GetLastErrorMessage();
};

#endif //__WL_NETKERNELMESSAGEHANDLER_H

