/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "messagehandlerimp.h"
#include "netkernelmessagehandler.h"
#include "nvsnames.h"
#include <RVCTokenizer.h>

///////////////////  CNetMessageHandler implementation ////////////////////

CNetMessageHandler::CNetMessageHandler(const char* basename) :
  m_SenderName(basename)
{
  self = new CNetMessageHandlerImp(this);
}

CNetMessageHandler::~CNetMessageHandler()
{
  delete self;
  self = NULL;
}

bool CNetMessageHandler::IsSendQueueEmpty() 
{ 
  return (self->m_MessagesQueue.isEmpty() != FALSE); 
}

RVCollectable* CNetMessageHandler::waitEvent()
{
  return self->m_MessagesQueue.waitEvent();
}

TVPPriorityQueue::WaitReturnType CNetMessageHandler::waitEvent(RVCollectable*& event, long waitTime)
{
  return self->m_MessagesQueue.waitEvent(event, waitTime);
}

const char* CNetMessageHandler::WhoAmI()
{ 
  return self->m_TestTalkClient.NETWhoAmI(); 
}

const char* CNetMessageHandler::GetLocalHostName(void)
{
	return self->m_TestTalkClient.NETLocalHostName();
}

bool CNetMessageHandler::ParseHostName(/*[OUT]*/RVCString& hostName,
																			 /*[IN]*/const char* testtalkAddress, 
 		 																   /*[IN]*/bool hostAndPort)														 
{
	CTestTalkAddress address(testtalkAddress);

	if(hostAndPort) {
		hostName = address.HostName();
		return(hostName.length() > 0);
	}

	RVCTokenizer next(address.HostName());
  hostName = next(":");
	return(hostName.length() > 0);
}

double CNetMessageHandler::GetLastRecievedMessageTime()
{
  return self->m_LastRecievedMessageTime;
}

void CNetMessageHandler::SetLastRecievedMessageTime(double time)
{
  self->m_LastRecievedMessageTime = time;
}

void CNetMessageHandler::UseCallbacksModel(bool callbacksModel)
{
	self->UseCallbacksModel(callbacksModel);
}

bool CNetMessageHandler::Init(unsigned short testtalkport)
{
  return self->Init(testtalkport);
}

void CNetMessageHandler::Terminate()
{
  if( self ) {
    self->Terminate();
  }
}

bool CNetMessageHandler::IsActive()
{
  if( self ) {
		return self->m_Initialized;
	}

	return false;
}

void CNetMessageHandler::KillProcess(const char* hostName, const char* processUID, long waitBeforeKill)
{
	CAssociativeArray message;

	message.AddCouple("$KILLPROCESS", "");
	message.AddCouple("$PROCESSUID", processUID);
	message.AddCouple("$WAITBEFOREKILL", waitBeforeKill);

	SendMessage(message, hostName, "KillProcess Failed");
}

void CNetMessageHandler::CopyFile( const char* LocalFileName, const char* RemoteFileName, const char* RemoteHostAddress, const char* ErrorMsg, int priority)
{
	CAssociativeArray message;

	message.AddCouple("$LOCALFILE", LocalFileName);
	message.AddCouple("$REMOTEFILE", RemoteFileName);

	self->SendMessage(CNetHandlerMessage::FileCopy, message, RemoteHostAddress, ErrorMsg, priority);
}

void CNetMessageHandler::SendMessage( CAssociativeArray& Message , const char* SendTo, const char* ErrorMsg, int priority)
{
  self->SendMessage(CNetHandlerMessage::Wait, Message , SendTo, ErrorMsg, priority);
}

void CNetMessageHandler::SendMessageNoresponse( CAssociativeArray& Message , const char* SendTo, const char* ErrorMsg, int priority)
{
  self->SendMessage(CNetHandlerMessage::NoResponse, Message , SendTo, ErrorMsg, priority);
}

void CNetMessageHandler::SendMessageNowait( CAssociativeArray& Message , const char* SendTo, const char* ErrorMsg, int priority)
{
  self->SendMessage(CNetHandlerMessage::NoWait, Message , SendTo, ErrorMsg, priority);
}

CNetHandlerMessage* CNetMessageHandler::SenderWaitMessage()
{
  //WlTraceMsg("CNetMessageHandler::SenderWaitMessage(%s) - Started",m_TestTalkClient.NETWhoAmI());
  return (CNetHandlerMessage*)waitEvent();
}

void CNetMessageHandler::CopyFileDone(CNetMessageHandler::CopyFileCompletionState copyStatus,
																			const char* copyStatusFileName,
																			const char* LocalFileName, 
																			const char* RemoteFileName, 
																			const char* RemoteHostAddress)
{
}

void CNetMessageHandler::SendMessageSucceded( const char* Message , const char* SendTo)
{
}

void CNetMessageHandler::SendMessageErrorHandler( CAssociativeArray& Message , const char* SendTo, const char* ErrorMsg, bool status)
{
}

void CNetMessageHandler::MessageHandler( CAssociativeArray& AssoArrayCommand )
{
	if ( AssoArrayCommand.KeyExists( "$EXIT" ) )
  {
  }
}

void CNetMessageHandler::AddMessagesListener(INetMessagesListener* listener)
{
	self->AddMessagesListener(listener);
}

void CNetMessageHandler::RemoveMessagesListener(INetMessagesListener *listener)
{
	self->RemoveMessagesListener(listener);
}

bool CNetMessageHandler::ReadNextMessage(/*[OUT]*/CAssociativeArray& message)
{
	return self->ReadNextMessage(message);
}

const char* CNetMessageHandler::GetLastErrorMessage()
{
	return self->GetLastErrorMessage();
}

