/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <collint.h>
#include <hashdict.h> 
#include <thr.h>

#include "tracedll.h"
#include "testtalkdefs.h"
#include "netevent.h"
#include "testtalkclientobjimp.h"
#include "testtalkclientcommunicator.h"

class CClientsCollection : public RVHashDictionary {
  public:
    CClientsCollection() {}
    ~CClientsCollection() {}

    void Insert(HWND hWnd, CTestTalkClientCommunicator* pTestTalkClientCommunicator) 
    {
      RVCollectableInt *key = new RVCollectableInt((int)hWnd);
      insertKeyAndValue(key, pTestTalkClientCommunicator);
    }

    CTestTalkClientCommunicator* Get(HWND hWnd) 
    {
      RVCollectableInt key((int)hWnd);
      CTestTalkClientCommunicator* pTTCC = (CTestTalkClientCommunicator*)findValue(&key);
      return pTTCC;
    }

    void Remove(HWND hWnd) 
    {
      RVCollectableInt key((int)hWnd);
      remove(&key);
    }
};

static CClientsCollection clientsCollection;

void TestTalkClearClientsTable()
{
  clientsCollection.clearAndDestroy();
}


/****************** CTestTalkClientCommunicator Functions ******************/

CTestTalkClientCommunicator::CTestTalkClientCommunicator(CTestTalkClientObject& ttcc) : 
  m_TestTalkClientObject(ttcc), m_Valid(false)
{
  m_Terminate = false;
  #if !defined(RV_THR_CANT_EXTRACT_TEMPLATE_ARGS_FROM_SIGNATURE)            
    m_ReaderThread = rvMakeThreadFunction(
                        *this,                                      // instance
                        &CTestTalkClientCommunicator::StartReader     // member function
                      );
  #else
    m_ReaderThread = rvMakeThreadFunctionM( 
                        CTestTalkClientCommunicator                   // class
                        *this,                                      // instance
                        void,                                       // return value
                        &CTestTalkClientCommunicator::StartReader     // member function
                      );
  #endif
}

CTestTalkClientCommunicator::~CTestTalkClientCommunicator()
{
  //EnterProcedure("CTestTalkClientCommunicator::~CTestTalkClientCommunicator");
  try {
    Terminate();
  } catch (...) {
  }
}

void CTestTalkClientCommunicator::Terminate(bool waitForThreads)
{
  bool terminate;
  {
    //RVCriticalSection::LockGuard Lock(m_Touch);
    RVTRecursiveLock<RVMutexLock>::LockGuard Lock(m_Touch);
    terminate = m_Terminate;
    if (!terminate) {
      WlTraceMsg("CTestTalkClientCommunicator::Terminate (%s,%d)", m_TestTalkClientObject.m_NetClientData.clientName, m_StreamSocket.m_Socket); //@@eyal: trace
      m_Terminate = true;
      //m_TestTalkClientObject.m_CTestTalkClientCommunicator = NULL;
      m_TestTalkClientObject.NETTerminate();
      m_StreamSocket.CloseSocket();
    }
  }
  if (!terminate) {
    if (waitForThreads) {
      m_ReaderThread.join(5000);
    }
  }
}

void CTestTalkClientCommunicator::StartReader()
{
	CTraceMsg::RegisterThread ();

  RVThreadSelf self = ::rvThread();
  self.setPriority(self.getMaxPriority());

  try {
	  doReading();
  } catch (...) {
  }
  Terminate(false);
  WlTraceMsg("CTestTalkClientCommunicator::StartReader - Done (%s,%d)", m_TestTalkClientObject.m_NetClientData.clientName, m_StreamSocket.m_Socket); //@@eyal: trace
}

int CTestTalkClientCommunicator::doReading()
{
	void* msg;
	int len;
	short type, s_t;

	while (1) {
		int st = m_StreamSocket.WaitTypedMessage(type, s_t, msg, len);
		if (st >= 0) {
      TypedMessageObject* tmo = new TypedMessageObject(msg, len, type, s_t);
      m_TestTalkClientObject.NewMessageEvent(tmo, s_t);
			m_StreamSocket.FreeMessage(msg);
		}
		else
		{
			//
			// treat the error situation here.
			WlTraceMsg("CTestTalkClientCommunicator::doReading - Error reading from socket, client (%s)", m_TestTalkClientObject.m_NetClientData.clientName); //@@eyal: trace
      Terminate(false);
      m_TestTalkClientObject.NETTerminate();
			return -1;
		}
	}
}

bool CTestTalkClientCommunicator::sendMessage(void* msg, int len, short type, short subtype)
{
  if (m_Terminate)
    return false;

  int st = m_StreamSocket.SendTypedMessage(type, subtype, msg, len);
	if (st < 0)
	{
		//
		// treat the error situation here.
		WlTraceMsg("CTestTalkClientCommunicator::sendMessage - Error sending message to socket, client (%s)", m_TestTalkClientObject.m_NetClientData.clientName); //@@eyal: trace
		return false;
	}
  return true;
}

bool CTestTalkClientCommunicator::Start()
{
  m_StreamSocket.SetSocketStyle();
	m_StreamSocket.CreateSocket();
	m_StreamSocket.SetBlockingMode(0);

  //
	// Connect.
	int connst = m_StreamSocket.Connect(m_TestTalkClientObject.m_TestTalkClientsPort);
	if (connst < 0)
	{
		//
		// treat the error situation here.
		WlTraceMsg("CTestTalkClientCommunicator::Start - error %d occurred while trying to connect to port %d.", m_StreamSocket.GetLastError(), m_TestTalkClientObject.m_TestTalkClientsPort);

		char localhost[1024];
		CSocketLib::GetLocalHostName(localhost, sizeof(localhost));

		sockaddr_in    myPeerSocketAddress;
	  CSocketLib::GetAddress(localhost, m_TestTalkClientObject.m_TestTalkClientsPort, myPeerSocketAddress);

		WlTraceMsg("CTestTalkClientCommunicator::Start - failed, host=%s", localhost);

		return false;
	}

  //
  // Sending the startup message to testtalk.
  if (!sendMessage((void*)&(m_TestTalkClientObject.m_NetClientData), sizeof(NetClientData), NV_CLIENT_STARTED, 0))
	{
		WlTraceMsg("CTestTalkClientCommunicator::Start - sendMessage failed.");
    return false;
	}

  m_ReaderThread.start();

  m_Valid = true;
  return true;
}

