/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// TestTalkNetConnection.cpp : implementation file
//

#include <fstream.h>
#include <RVCTokenizer.h>

#include "tracedll.h"
#include "wlerrorlib.h"
#include "tvenv.h"
#include "tvpsync.h"
#include "testtalkdefs.h"
#include "testtalkclass.h"
#include "testtalktoolkit.h"
#include "testtalkeventserver.h"
#include "testtalknetconnection.h"
#include "supportinformationgenerator.h"

#define TESTTALK_SENDMESSAGE_PING_TIMEOUT_SECONDS   120 // 2 Minutes :)
#define TESTTALK_CURRENT_VERSION    (0x7F04)

#ifdef __TV_WINDOWS__
static const char* GetErrorString(int nError) {
	const char* szTemp;
	switch (nError){
	case WSAEINTR:
		szTemp = "(WSAEINTR): Interrupted system call"; break;
	case WSAEBADF:
		szTemp = "(WSAEBADF): Bad file number"; break;
	case WSAEACCES:
		szTemp = "(WSAEACCES): Permission denied"; break;
	case WSAEFAULT:
		szTemp = "(WSAEFAULT): Bad address passed"; break;
	case WSAEINVAL:
		szTemp = "(WSAEINVAL): Invalid parameter passed"; break;
	case WSAEMFILE:
		szTemp = "(WSAEMFILE): Too many open files"; break;
	case WSAEWOULDBLOCK:
		szTemp = "(WSAEWOULDBLOCK): Operation would block"; break;
	case WSAEINPROGRESS:
		szTemp = "(WSAEINPROGRESS): Operation is now in progress"; break;
	case WSAEALREADY:
		szTemp = "(WSAEALREADY): Operation is already in progress"; break;
	case WSAENOTSOCK:
		szTemp = "(WSAENOTSOCK): Socket operation on non-socket"; break;
	case WSAEDESTADDRREQ:
		szTemp = "(WSAEDESTADDRREQ): Destination address required"; break;
	case WSAEMSGSIZE:
		szTemp = "(WSAEMSGSIZE): Message is too long"; break;
	case WSAEPROTOTYPE:
		szTemp = "(WSAEPROTOTYPE): The protocol is of the wrong type for the socket"; break;
	case WSAENOPROTOOPT:
		szTemp = "(WSAENOPROTOOPT): The requested protocol is not available"; break;
	case WSAEPROTONOSUPPORT:
		szTemp = "(WSAEPROTONOSUPPORT): The requested protocol is not supported"; break;
	case WSAESOCKTNOSUPPORT:
		szTemp = "(WSAESOCKTNOSUPPORT): The specified socket type is not supported"; break;
	case WSAEOPNOTSUPP:
		szTemp = "(WSAEOPNOTSUPP): The specified operation is not supported"; break;
	case WSAEPFNOSUPPORT:
		szTemp = "(WSAEPFNOSUPPORT): The specified protocol family is not supported"; break;
	case WSAEAFNOSUPPORT:
		szTemp = "(WSAEAFNOSUPPORT): The specified address family is not supported"; break;
	case WSAEADDRINUSE:
		szTemp = "(WSAEADDRINUSE): The specified address is already in use"; break;
	case WSAEADDRNOTAVAIL:
		szTemp = "(WSAEADDRNOTAVAIL): The requested address is unassignable"; break;
	case WSAENETDOWN:
		szTemp = "(WSAENETDOWN): The network appears to be down"; break;
	case WSAENETUNREACH:
		szTemp = "(WSAENETUNREACH): The network is unreachable"; break;
	case WSAENETRESET:
		szTemp = "(WSAENETRESET): The network dropped the connection on reset"; break;
	case WSAECONNABORTED:
		szTemp = "(WSAECONNABORTED): Software caused a connection abort"; break;
	case WSAECONNRESET:
		szTemp = "(WSAECONNRESET): Connection was reset by peer"; break;
	case WSAENOBUFS:
		szTemp = "(WSAENOBUFS): Out of buffer space"; break;
	case WSAEISCONN:
		szTemp = "(WSAEISCONN): Socket is already connected"; break;
	case WSAENOTCONN:
		szTemp = "(WSAENOTCONN): Socket is not presently connected"; break;
	case WSAESHUTDOWN:
		szTemp = "(WSAESHUTDOWN): Can't send data because socket is shut down"; break;
	case WSAETOOMANYREFS:
		szTemp = "(WSAETOOMANYREFS): Too many references, unable to splice"; break;
	case WSAETIMEDOUT:
		szTemp = "(WSAETIMEDOUT): The connection timed out"; break;
	case WSAECONNREFUSED:
		szTemp = "(WSAECONNREFUSED): The connection was refused"; break;
	case WSAELOOP:
		szTemp = "(WSAELOOP): Too many symbolic link levels"; break;
	case WSAENAMETOOLONG:
		szTemp = "(WSAENAMETOOLONG): File name is too long"; break;
	case WSAEHOSTDOWN:
		szTemp = "(WSAEHOSTDOWN): The host appears to be down"; break;
	case WSAEHOSTUNREACH:
		szTemp = "(WSAEHOSTUNREACH): The host is unreachable"; break;
	case WSAENOTEMPTY:
		szTemp = "(WSAENOTEMPTY): The directory is not empty"; break;
	case WSAEPROCLIM:
		szTemp = "(WSAEPROCLIM): There are too many processes"; break;
	case WSAEUSERS:
		szTemp = "(WSAEUSERS): There are too many users"; break;
	case WSAEDQUOT:
		szTemp = "(WSAEDQUOT): The disk quota is exceeded"; break;
	case WSAESTALE:
		szTemp = "(WSAESTALE): Bad NFS file handle"; break;
	case WSAEREMOTE:
		szTemp = "(WSAEREMOTE): There are too many levels of remote in the path"; break;
	case WSAEDISCON:
		szTemp = "(WSAEDISCO): Disconnect"; break;
	case WSASYSNOTREADY:
		szTemp = "(WSASYSNOTREADY): Network sub-system is not ready or unusable"; break;
	case WSAVERNOTSUPPORTED:
		szTemp = "(WSAVERNOTSUPPORTED): The requested version is not supported"; break;
	case WSANOTINITIALISED:
		szTemp = "(WSANOTINITIALISED): Socket system is not initialized"; break;
	case WSAHOST_NOT_FOUND:
		szTemp = "(WSAHOST_NOT_FOUND): The host was not found"; break;
	case WSATRY_AGAIN:
		szTemp = "(WSATRY_AGAIN): Non-authoritative host not found"; break;
	case WSANO_RECOVERY:
		szTemp = "(WSANO_RECOVERY): Non-recoverable error"; break;
	case WSANO_DATA:
		szTemp = "(WSANO_DATA): No information of the requested type was available"; break;
	default:
		szTemp = "(???): Unknown error or no error";
   }
   return szTemp;
}

#else

#include <errno.h>


static const char* GetErrorString(int nError) {
	const char* szTemp;
	switch (nError){
	case EINTR:
		szTemp = "(EINTR): Interrupted system call"; break;
	case EBADF:
		szTemp = "(EBADF): Bad file number"; break;
	case EACCES:
		szTemp = "(EACCES): Permission denied"; break;
	case EFAULT:
		szTemp = "(EFAULT): Bad address passed"; break;
	case EINVAL:
		szTemp = "(EINVAL): Invalid parameter passed"; break;
	case EMFILE:
		szTemp = "(EMFILE): Too many open files"; break;
	case EWOULDBLOCK:
		szTemp = "(EWOULDBLOCK): Operation would block"; break;
	case EINPROGRESS:
		szTemp = "(EINPROGRESS): Operation is now in progress"; break;
	case EALREADY:
		szTemp = "(EALREADY): Operation is already in progress"; break;
	case ENOTSOCK:
		szTemp = "(ENOTSOCK): Socket operation on non-socket"; break;
	case EDESTADDRREQ:
		szTemp = "(EDESTADDRREQ): Destination address required"; break;
	case EMSGSIZE:
		szTemp = "(EMSGSIZE): Message is too long"; break;
	case EPROTOTYPE:
		szTemp = "(EPROTOTYPE): The protocol is of the wrong type for the socket"; break;
	case ENOPROTOOPT:
		szTemp = "(ENOPROTOOPT): The requested protocol is not available"; break;
	case EPROTONOSUPPORT:
		szTemp = "(EPROTONOSUPPORT): The requested protocol is not supported"; break;
	case ESOCKTNOSUPPORT:
		szTemp = "(ESOCKTNOSUPPORT): The specified socket type is not supported"; break;
	case EOPNOTSUPP:
		szTemp = "(EOPNOTSUPP): The specified operation is not supported"; break;
	case EPFNOSUPPORT:
		szTemp = "(EPFNOSUPPORT): The specified protocol family is not supported"; break;
	case EAFNOSUPPORT:
		szTemp = "(EAFNOSUPPORT): The specified address family is not supported"; break;
	case EADDRINUSE:
		szTemp = "(EADDRINUSE): The specified address is already in use"; break;
	case EADDRNOTAVAIL:
		szTemp = "(EADDRNOTAVAIL): The requested address is unassignable"; break;
	case ENETDOWN:
		szTemp = "(ENETDOWN): The network appears to be down"; break;
	case ENETUNREACH:
		szTemp = "(ENETUNREACH): The network is unreachable"; break;
	case ENETRESET:
		szTemp = "(ENETRESET): The network dropped the connection on reset"; break;
	case ECONNABORTED:
		szTemp = "(ECONNABORTED): Software caused a connection abort"; break;
	case ECONNRESET:
		szTemp = "(ECONNRESET): Connection was reset by peer"; break;
	case ENOBUFS:
		szTemp = "(ENOBUFS): Out of buffer space"; break;
	case EISCONN:
		szTemp = "(EISCONN): Socket is already connected"; break;
	case ENOTCONN:
		szTemp = "(ENOTCONN): Socket is not presently connected"; break;
	case ESHUTDOWN:
		szTemp = "(ESHUTDOWN): Can't send data because socket is shut down"; break;
	case ETOOMANYREFS:
		szTemp = "(ETOOMANYREFS): Too many references, unable to splice"; break;
	case ETIMEDOUT:
		szTemp = "(ETIMEDOUT): The connection timed out"; break;
	case ECONNREFUSED:
		szTemp = "(ECONNREFUSED): The connection was refused"; break;
	case ELOOP:
		szTemp = "(ELOOP): Too many symbolic link levels"; break;
	case ENAMETOOLONG:
		szTemp = "(ENAMETOOLONG): File name is too long"; break;
	case EHOSTDOWN:
		szTemp = "(EHOSTDOWN): The host appears to be down"; break;
	case EHOSTUNREACH:
		szTemp = "(EHOSTUNREACH): The host is unreachable"; break;
	case ENOTEMPTY:
		szTemp = "(ENOTEMPTY): The directory is not empty"; break;
		//      case EPROCLIM:
		//szTemp = "(EPROCLIM): There are too many processes"; break;
	case EUSERS:
		szTemp = "(EUSERS): There are too many users"; break;
	case EDQUOT:
		szTemp = "(EDQUOT): The disk quota is exceeded"; break;
	case ESTALE:
		szTemp = "(ESTALE): Bad NFS file handle"; break;
	case EREMOTE:
		szTemp = "(EREMOTE): There are too many levels of remote in the path"; break;
		/*      case EDISCON:
		szTemp = "(EDISCO): Disconnect"; break;
		case SYSNOTREADY:
		szTemp = "(SYSNOTREADY): Network sub-system is not ready or unusable"; break;
		case VERNOTSUPPORTED:
		szTemp = "(VERNOTSUPPORTED): The requested version is not supported"; break;
		case NOTINITIALISED:
		szTemp = "(NOTINITIALISED): Socket system is not initialized"; break;
		*/
	case HOST_NOT_FOUND:
		szTemp = "(HOST_NOT_FOUND): The host was not found"; break;
	case TRY_AGAIN:
		szTemp = "(TRY_AGAIN): Non-authoritative host not found"; break;
	case NO_RECOVERY:
		szTemp = "(NO_RECOVERY): Non-recoverable error"; break;
		//      case NO_DATA:
		//szTemp = "(NO_DATA): No information of the requested type was available"; break;
	default:
		szTemp = "(???): Unknown error or no error";
   }
   return szTemp;
}

#endif

/****************** CTestTalkHostConnection Functions ******************/

/****************** CTestTalkHostConnection Functions ******************/

CTestTalkHostConnection::CTestTalkHostConnection(const char* host, CTestTalkClass *ttMain) :
m_Name(host), m_TestTalkMain(ttMain)
{
	//EnterProcedure("CTestTalkHostConnection::CTestTalkHostConnection");
	////sergei 3/24/01 - debugPrint(3,("CTestTalkHostConnection(host=%s)\n",host));
	//
	// registering the Host-event.
	int op = signOn((FUNCP)&CTestTalkHostConnection::newMessage);
	m_TestTalkMain->m_EventServer->registerHostEvent(host, *this, op);

	Init();
	m_WriterThread.start();
}

CTestTalkHostConnection::CTestTalkHostConnection(SOCKET s, CTestTalkClass *ttMain) :
m_Name("",128), m_StreamSocket(s), m_TestTalkMain(ttMain)
{
	//EnterProcedure("CTestTalkHostConnection::CTestTalkHostConnection");
	//
	// add the openning message & the first message to the
	Init();
	m_ReaderThread.start();
}

CTestTalkHostConnection::~CTestTalkHostConnection()
{
	EnterProcedure("CTestTalkHostConnection::~CTestTalkHostConnection");
	try {
		Terminate();
		//delete m_ThreadsMutex;
		delete m_MsgsQueue;
		delete m_SendFilesQueue;
		delete m_ReceiveFilesQueue;
		delete m_FileCopyHandler;
	} catch (...) {
	}
}

void CTestTalkHostConnection::Init()
{
	m_IsStarted = false;
	m_Terminate = false;
	m_FileWriterThreadRunning = false;
	m_FileReaderThreadRunning = false;
	//m_ThreadsMutex = new RVCriticalSection;
	m_MsgsQueue         = new TVPEventQueue;
	m_SendFilesQueue    = new TVPEventQueue;
	m_ReceiveFilesQueue = new TVPEventQueue;
	m_ConnectionDone.Reset();

	m_FileCopyHandler     = new CFileTransferHandler(this);

#if !defined(RV_THR_CANT_EXTRACT_TEMPLATE_ARGS_FROM_SIGNATURE)
	m_ReaderThread = rvMakeThreadFunction(
		*this,                                      // instance
		&CTestTalkHostConnection::StartReader       // member function
		);
	m_WriterThread = rvMakeThreadFunction(
		*this,                                      // instance
		&CTestTalkHostConnection::StartWriter       // member function
		);
	//
	//FileCopy Threads:
	m_FileReaderThread = rvMakeThreadFunction(
		*this,                                        // instance
		&CTestTalkHostConnection::StartFileReader     // member function
		);
	m_FileWriterThread = rvMakeThreadFunction(
		*this,                                        // instance
		&CTestTalkHostConnection::StartFileWriter     // member function
		);
#else
	m_ReaderThread = rvMakeThreadFunctionM(
		CTestTalkHostConnection                   // class
		*this,                                      // instance
		void,                                       // return value
		&CTestTalkHostConnection::StartReader     // member function
		);
	m_WriterThread = rvMakeThreadFunctionM(
		CTestTalkHostConnection                   // class
		*this,                                      // instance
		void,                                       // return value
		&CTestTalkHostConnection::StartWriter     // member function
		);
	//
	//FileCopy Threads:
	m_ReaderThread = rvMakeThreadFunctionM(
		CTestTalkHostConnection                   // class
		*this,                                      // instance
		void,                                       // return value
		&CTestTalkHostConnection::StartFileReader     // member function
		);
	m_WriterThread = rvMakeThreadFunctionM(
		CTestTalkHostConnection                   // class
		*this,                                      // instance
		void,                                       // return value
		&CTestTalkHostConnection::StartFileWriter     // member function
		);
#endif
}

void CTestTalkHostConnection::Terminate(bool waitForThreads)
{
	try {
		bool terminate;
		{
			RVCriticalSection::LockGuard Lock(m_Touch);
			terminate = m_Terminate;
			if (!terminate) {
				WlTraceMsg("CTestTalkHostConnection::Terminate(%s,%d)", m_Name.data(), m_StreamSocket.m_Socket); //@@eyal: trace
				m_ConnectionDone.Set();
				m_FileCopyHandler->Terminate();
				m_TestTalkMain->m_EventServer->unRegisterClient(*this);
				emptyMessagesQueue();
				emptyFileMessagesQueue();
				m_StreamSocket.CloseSocket();
				m_Terminate = true;
				m_MsgsQueue->Terminate();
				if(m_FileReaderThread.getExecutionState() != RV_THR_INITIAL)
					m_FileReaderThread.join(5000);
				if(m_FileWriterThread.getExecutionState() != RV_THR_INITIAL)
					m_FileWriterThread.join(5000);
			}
		}
		if (!terminate) {
			if (m_TestTalkMain)
				m_TestTalkMain->m_EventServer->removeHost(*this);
			if (waitForThreads) {
				m_ReaderThread.join(5000);
				m_WriterThread.join(5000);
			}
		}
	}
	catch(...) {}
}

void CTestTalkHostConnection::StartReader()
{
	RVThreadSelf self = ::rvThread();
	self.setPriority(self.getMaxPriority());

	try {
		doReading();
	} catch (...) {
		WlTraceMsg("CTestTalkHostConnection::StartReader - Done(%s,%d)", m_Name.data(), m_StreamSocket.m_Socket);
	}
	Terminate(false);
	WlTraceMsg("CTestTalkHostConnection::StartReader - Done(%s,%d)", m_Name.data(), m_StreamSocket.m_Socket);
}

void CTestTalkHostConnection::StartWriter()
{
	RVThreadSelf self = ::rvThread();
	self.setPriority(self.getMaxPriority());

	try {
		doWriting();
	} catch (...) {
	}
	Terminate(false);
	WlTraceMsg("CTestTalkHostConnection::StartWriter - Done(%s,%d)", m_Name.data(), m_StreamSocket.m_Socket); //@@eyal: trace
}

void CTestTalkHostConnection::StartFileReader()
{
	RVThreadSelf self = ::rvThread();
	self.setPriority(self.getMaxPriority());

	try {
		doFileReading();
	} catch (...) {
		WlTraceMsg("CTestTalkHostConnection::StartFileReader - unknow exception thrown.\n");
	}

	WlTraceMsg("CTestTalkHostConnection::StartFileReader - Done, closing thread.\n");
}

void CTestTalkHostConnection::StartFileWriter()
{
	RVThreadSelf self = ::rvThread();
	self.setPriority(self.getMaxPriority());

	try {
		doFileWriting();
	} catch (...) {
		WlTraceMsg("CTestTalkHostConnection::StartFileWriter - unknow exception thrown.\n");
	}

	WlTraceMsg("CTestTalkHostConnection::StartFileWriter - Done, closing thread.\n");
}

void CTestTalkHostConnection::emptyMessagesQueue()
{
	//WlTraceMsg("CTestTalkHostConnection::emptyMessagesQueue");
	try {
		while (!m_MsgsQueue->isEmpty())
		{
			TestTalkHostEvent *he = (TestTalkHostEvent *)m_MsgsQueue->waitEvent();
			int type = he->getType();
			if (type == NV_SENDMESSAGE || type == NV_SENDMESSAGE_NOWAIT)
			{
				//
				// The queue is determined according to the type, regular
				// message to the control queue and the nowait to the dataQ.
				QType queue;
				if(type == NV_SENDMESSAGE_NOWAIT)
					queue = AckQ;
				else //if (type==NV_SENDMESSAGE)
					queue = ControlQ;
				//
				// if sending fails - return error.
				const char* error = "Connection to host broke.";
				TestTalkClientEvent& ce =
					* new TestTalkClientEvent(he->getMessageId(),
					he->getFromHost(),
					he->getFromClient(),
					he->getToHost(),
					he->getToClient(),
					queue,
					NV_SENDMESSAGE_STATUS_FAIL,
					FALSE,
					(void*)error,
					strlen(error)+1);

				m_TestTalkMain->m_EventServer->dispatchClientEvent(ce);
				delete he;
			}
		}
	} catch (...) {
	}
}

void CTestTalkHostConnection::emptyFileMessagesQueue()
{
	//WlTraceMsg("CTestTalkHostConnection::emptyFileMessagesQueue");
	try {
		while (!m_SendFilesQueue->isEmpty()) {
			TestTalkHostEvent* hostEvent = (TestTalkHostEvent*)m_SendFilesQueue->waitEvent();
			//
			// if sending fails - return error.
			const char* error = "Connection to host broke.";
			TestTalkClientEvent* clientEvent =
				new TestTalkClientEvent(hostEvent->getMessageId(),
				hostEvent->getFromHost(),
				hostEvent->getFromClient(),
				hostEvent->getToHost(),
				hostEvent->getToClient(),
				AckQ,
				NV_SENDMESSAGE_STATUS_FAIL,
				FALSE,
				(void*)error,
				strlen(error) + 1);

			sendMessageToClient(clientEvent);
			delete hostEvent;

		}
	}
	catch (...) {
	}
}

int CTestTalkHostConnection::newMessage(BaseEvent& be)
{
	//WlTraceMsg("CTestTalkHostConnection::newMessage");

	TestTalkHostEvent* tmo = (TestTalkHostEvent*)&be;

	if(tmo->getType() != NV_FILECOPY) {
		m_MsgsQueue->putEvent(tmo);
	}
	else { // Type = NV_FILECOPY
		putFileMessage(tmo);
	}
	return TRUE;
}

int CTestTalkHostConnection::sendMessageToClient(TestTalkClientEvent* clientEvent)
{
	return m_TestTalkMain->m_EventServer->dispatchClientEvent(*clientEvent);
}

int CTestTalkHostConnection::doneReadingMessage(void* msg,
												int   len,
												short type,
												short subtype)
{
	//
	// check the message-type and act accordingly.
	switch (type)
	{
	case NV_SENDMESSAGE:
	case NV_SENDMESSAGE_NOWAIT:
	case NV_SENDMESSAGE_NORESPONSE:
		{
			//
			// dispatch a client-event.
			TestTalkClientEvent& e = * new TestTalkClientEvent(msg, len, type, subtype);
			//WlTraceMsg("received %s\n",e.printOnString());

			RVCString fh = e.getFromHost();
			RVCString fc = e.getFromClient();
			RVCString th = e.getToHost();
			RVCString tc = e.getToClient();
			long msgId   = e.getMessageId();
			short st = m_TestTalkMain->m_EventServer->dispatchClientEvent(e);
			//
			//return sending status.
			if (type != NV_SENDMESSAGE_NORESPONSE) {
				//
				// The queue is determined according to the type, regular
				// message to the control queue and the nowait to the dataQ.
				QType queue;
				if(type == NV_SENDMESSAGE_NOWAIT)
					queue = AckQ;
				else //if (type==NV_SENDMESSAGE)
					queue = ControlQ;

				TestTalkHostEvent& hevent =
					* new TestTalkHostEvent(msgId, fh.data(), fc.data(), th.data(), tc.data(),
					queue,
					(short)(st?NV_SENDMESSAGE_STATUS_SUCCESS:NV_SENDMESSAGE_STATUS_FAIL),
					(short)0,
					(void *)"",
					(long)1);
				//WlTraceMsg("sending confirmation %s\n",hevent.printOnString());
				newMessage(hevent);
			}
		}
		break;

	case NV_FILE_TRANSFER:
		{
			if(m_FileReaderThread.getExecutionState() != RV_THR_INITIAL)
				m_FileReaderThread.join();
			m_ReceiveFilesQueue->putEvent(new TestTalkHostEvent(msg, len, type, subtype));
			m_FileReaderThread.start();
		}
		break;

	case NV_FILECOPY_EVENT:
		{
			//TestTalkMessage event(msg, len, type, subtype);
			m_FileCopyHandler->HandleFileEvent(msg, len, type, subtype);
		}
		break;

	case NV_SENDMESSAGE_STATUS_SUCCESS:
	case NV_SENDMESSAGE_STATUS_FAIL:
		{
			//
			// dispatch a client-event.
			TestTalkClientEvent& e = * new TestTalkClientEvent(msg, len, type, subtype);
			//WlTraceMsg("received confirmation %s\n", e.printOnString());
			short st = m_TestTalkMain->m_EventServer->dispatchClientEvent(e);
		}
		break;

	case NV_SENDMESSAGE_PING:
		{
			//WlTraceMsg("Ping Arrived from host %s",m_Name);
		}
		break;
	}

	return TRUE;
}

int CTestTalkHostConnection::doFileReading()
{
	if(!m_IsStarted) {
		WlTraceMsg("CTestTalkHostConnection::doFileReading - waiting for connection to end");
		m_ConnectionDone.Wait();
		if(!m_IsStarted) {
			return FALSE;
		}
	}

	m_FileReaderThreadRunning = true;
	CFileTransferHandler::CompletionState status;

	while(m_FileReaderThreadRunning && !m_ReceiveFilesQueue->isEmpty()) {

		RVCollectable* event = m_ReceiveFilesQueue->get();

		if(!event)
			return FALSE;

		TestTalkHostEvent* fileEvent = (TestTalkHostEvent*)event;
		const char* dstFilePath = (char*)fileEvent->getData();

		status = m_FileCopyHandler->ReceiveFile(dstFilePath, fileEvent);
		delete fileEvent;

		m_FileReaderThreadRunning = false;
	}

	return TRUE;
}

//@@
int CTestTalkHostConnection::doFileWriting()
{
	if(!m_IsStarted) {
		WlTraceMsg("CTestTalkHostConnection::doFileWriting - waiting for connection to end");
		m_ConnectionDone.Wait();
		if(!m_IsStarted) {
			return FALSE;
		}
	}

	CAssociativeArray completionState;
	CFileTransferHandler::CompletionInfo* status;

	while(m_FileWriterThreadRunning) {

		TestTalkHostEvent* fileEvent = (TestTalkHostEvent*)m_SendFilesQueue->get();

		if(!fileEvent)
			return FALSE;

		CAssociativeArray fileNamesArray;

		fileNamesArray.FromString((char*)fileEvent->getData());
		const char* srcFilePath = fileNamesArray.GetValueOfParameter("$LOCALFILE");
		const	char* dstFilePath = fileNamesArray.GetValueOfParameter("$REMOTEFILE");

		status = m_FileCopyHandler->SendFile(srcFilePath, dstFilePath, fileEvent);

		char*  buffer;
		size_t bufferLen;
		completionState.AddCouple("$FILECOPYCOMPLETIONSTATE", (long)status->m_CompletionState);
		completionState.AddCouple("$FILENAME", status->m_FileName);
		completionState.ToString(buffer, bufferLen);

		sendMessageToClient(TestTalkToolKit::CreateReturnClientEvent(fileEvent,
												buffer, bufferLen, NV_SENDMESSAGE_STATUS_SUCCESS, AckQ));

		delete status;
		delete fileEvent;

		// Next block: the thread checks if it can go down.
		// The critical section is shared between this method and PutFileMessage
		{
			RVCriticalSection::LockGuard lock(m_FileWriterThreadSync);

			if(m_SendFilesQueue->isEmpty())
				m_FileWriterThreadRunning = false;
			else
				m_FileWriterThreadRunning = true;
		}
	}
	return TRUE;
}

void CTestTalkHostConnection::putFileMessage(TestTalkHostEvent* hostEvent)
{
	RVCriticalSection::LockGuard lock(m_FileWriterThreadSync);

	if(!m_FileWriterThreadRunning) {
		if(m_FileWriterThread.getExecutionState() != RV_THR_INITIAL)
			m_FileWriterThread.join();

		m_FileWriterThreadRunning = true;
		m_SendFilesQueue->putEvent(hostEvent);
		m_FileWriterThread.start();
	}
	else {
		m_SendFilesQueue->putEvent(hostEvent);
	}
}

int CTestTalkHostConnection::doReading()
{
	void* msg;
	int len;
	short type, s_t;

	m_StreamSocket.SetBlockingMode(0);

	if (!m_IsStarted)
	{
		//
		// first read the connected host's name
		int st = m_StreamSocket.WaitTypedMessage(type, s_t, msg, len);
		((char*)msg)[len-1] = 0;
		m_Name = (char*)msg;
		m_StreamSocket.FreeMessage(msg);
		if (s_t != TESTTALK_CURRENT_VERSION) {
			WlTraceMsg("CTestTalkHostConnection::doReading - Error receiving connection from host %s, TestTalk version there is Obsolete.",m_Name.data());
			m_ConnectionDone.Set();
			return -1;
		}
		//
		// registering the Host-event.
		int op = signOn((FUNCP)&CTestTalkHostConnection::newMessage);
		m_TestTalkMain->m_EventServer->registerHostEvent(m_Name, *this, op);
		WlTraceMsg("CTestTalkHostConnection::doReading - Received a connection from host: %s",(const char*)m_Name);
		//
		// create the writed thread:
		m_IsStarted=true;
		m_ConnectionDone.Set();
		m_WriterThread.start();
	}
	while (1) {
		int st = m_StreamSocket.WaitTypedMessage(type, s_t, msg, len);
		if (s_t != TESTTALK_CURRENT_VERSION) {
			WlTraceMsg("CTestTalkHostConnection::doReading - Socket error while reading from host %s", m_Name.data());
			if (type == NV_SENDMESSAGE_STATUS_SUCCESS || type == NV_SENDMESSAGE_STATUS_FAIL)
				doneReadingMessage(msg, len, NV_SENDMESSAGE_STATUS_FAIL, s_t);
			m_StreamSocket.FreeMessage(msg);
			return -1;
		}
		else if (st >= 0) {
			m_LastMessageTime = TVPSysTime();
			doneReadingMessage(msg, len, type, s_t);
			m_StreamSocket.FreeMessage(msg);
		}
		else
		{
			//
			// treat the error situation here.
			int err = m_StreamSocket.GetLastError();
			WlTraceMsg("CTestTalkHostConnection::doReading - Socket error while reading from %s (Error: %s, No: %d).",(const char*)m_Name, GetErrorString(err), err);
			return -1;
		}
	}
}

int CTestTalkHostConnection::doWriting()
{
	if (!m_IsStarted) {
		if (!doConnect())
		{
			//
			// treat the error situation here.
			m_ConnectionDone.Set();
			emptyMessagesQueue();
			emptyFileMessagesQueue();
			return FALSE;
		}

		m_IsStarted = true;
		m_ConnectionDone.Set();
		m_ReaderThread.start();
	}

	int pingTimeout = TESTTALK_SENDMESSAGE_PING_TIMEOUT_SECONDS;
	TVEnvContainerClass tvpc;
	if(tvpc.TVEGetEnv("TESTTALK_INTERNAL_PING_TIMEOUT_SECONDS") != NULL) {
		tvpc.reset();
		pingTimeout = atoi(tvpc.TVEGetEnv("TESTTALK_INTERNAL_PING_TIMEOUT_SECONDS"));
	}

	while (1) {
		//
		// Waiting for a message to send, using the timeout in order to check if we need to send ping.
		// The timeout is in seconds, and checking every 10th of that time.
		RVCollectable *event;
		TVPEventQueue::WaitReturnType ret = m_MsgsQueue->waitEvent(event, pingTimeout*100);
		TestTalkHostEvent *hevent = (TestTalkHostEvent *)event;  //: public TestTalkMessage


		switch (ret) {
		case TVPEventQueue::Success :
			{
				if (!hevent)
					return FALSE;

				int st = m_StreamSocket.SendTypedMessage(
					hevent->getType(),
					TESTTALK_CURRENT_VERSION,
					hevent->getPackedBuf(),
					hevent->getPackedBufLen());

				delete hevent;
				if (st < 0)
				{
					//
					// treat the error situation here.
					int err = m_StreamSocket.GetLastError();
					WlTraceMsg("CTestTalkHostConnection::doWriting - Socket error while writing to %s (Error: %s, No: %d).",(const char*)m_Name, GetErrorString(err), err);
					return FALSE;
				}
				m_LastMessageTime = TVPSysTime();
			}
			break;

		case TVPEventQueue::TimedOut :
			{
				WlTraceMsg("CTestTalkHostConnection::doWriting, Checking for Ping to host %s", m_Name);
				if (TVPSysTime() - m_LastMessageTime > pingTimeout) {
					WlTraceMsg("Sending Ping to host %s", m_Name);
					int st = m_StreamSocket.SendTypedMessage(
						NV_SENDMESSAGE_PING,
						TESTTALK_CURRENT_VERSION,
						(void*)"",
						1);
					m_LastMessageTime = TVPSysTime();
				}
			}
			break;

		default:
			return FALSE;
		}
	}
}

int CTestTalkHostConnection::doConnect()
{
	m_StreamSocket.SetSocketStyle();
	m_StreamSocket.CreateSocket();
	m_StreamSocket.SetBlockingMode(0);
	//
	// Connect:
	RVCString hostname;
	short port;
	size_t seperator = m_Name.index(":");
	if (seperator == RV_NPOS) {
		hostname = m_Name;
		port = m_TestTalkMain->m_tcpNetPort;
	} else {
		RVCTokenizer tok(m_Name);
		hostname = tok(":");
		RVCString cport = tok(":");
		port = atoi((const char*)cport);
	}
	WlTraceMsg("Connecting to %s:%d.\n",(const char*)hostname, (int)port);
	int connst = m_StreamSocket.Connect(hostname, port);
	//
	// check the connection status.
	if (connst < 0)
	{
		//
		// treat the error situation here.
		int err = m_StreamSocket.GetLastError();
		WlTraceMsg("Error connecting to %s (Error: %s, No: %d).\n",(const char*)m_Name, GetErrorString(err), err);
		return FALSE;
	}
	//
	// now, send the local-host name to the connected computer.
	char fullHostName[1024];
	sprintf(fullHostName,"%s:%d", m_TestTalkMain->m_LocalHostAddress.data(), (long)m_TestTalkMain->m_tcpNetPort);
	int st = m_StreamSocket.SendTypedMessage(
		NV_HOST_CONNECTION,
		TESTTALK_CURRENT_VERSION,
		(void*)(&fullHostName),
		strlen(fullHostName)+1);
	if (st<0)
	{
		//
		// treat the error situation here.
		WlTraceMsg("Error sending host-name message to host %s.\n", (const char*)m_Name);
		return FALSE;
	}

	WlTraceMsg("Connection to %s:%d Succeeded.\n",(const char*)hostname, (int)port);

	return TRUE;
}

/****************** CTestTalkClientConnection Functions ******************/

CTestTalkClientConnection::CTestTalkClientConnection(SOCKET s, CTestTalkClass *ttMain) :
m_Name("",128), m_StreamSocket(s), m_TestTalkMain(ttMain)
{
	//EnterProcedure("CTestTalkClientConnection::CTestTalkClientConnection");
	//
	// add the openning message & the first message to the
	Init();
	m_ReaderThread.start();
}

CTestTalkClientConnection::~CTestTalkClientConnection()
{
	EnterProcedure("CTestTalkClientConnection::~CTestTalkClientConnection");
	try {
		Terminate();
		delete m_MsgsQueue;
	} catch (...) {
	}
}

void CTestTalkClientConnection::Init()
{
	m_IsStarted = false;
	m_Terminate = false;
	//m_ThreadsMutex = new RVCriticalSection;
	m_MsgsQueue = new TVPEventQueue;

#if !defined(RV_THR_CANT_EXTRACT_TEMPLATE_ARGS_FROM_SIGNATURE)
	m_ReaderThread = rvMakeThreadFunction(
		*this,                                      // instance
		&CTestTalkClientConnection::StartReader     // member function
		);
	m_WriterThread = rvMakeThreadFunction(
		*this,                                      // instance
		&CTestTalkClientConnection::StartWriter     // member function
		);
#else
	m_ReaderThread = rvMakeThreadFunctionM(
		CTestTalkClientConnection                   // class
		*this,                                      // instance
		void,                                       // return value
		&CTestTalkClientConnection::StartReader     // member function
		);
	m_WriterThread = rvMakeThreadFunctionM(
		CTestTalkClientConnection                   // class
		*this,                                      // instance
		void,                                       // return value
		&CTestTalkClientConnection::StartWriter     // member function
		);
#endif
}

void CTestTalkClientConnection::Terminate(bool waitForThreads)
{
	bool terminate;
	{
		RVCriticalSection::LockGuard Lock(m_Touch);
		terminate = m_Terminate;
		if (!terminate) {
			WlTraceMsg("CTestTalkClientConnection::Terminate (%s,%d)", m_Name.data(), m_StreamSocket.m_Socket);
			m_StreamSocket.CloseSocket();
			m_Terminate = true;
			m_MsgsQueue->Terminate();
		}
	}
	if (!terminate) {
		if (m_TestTalkMain) {
			m_TestTalkMain->m_EventServer->removeClient(*clientInst);
		}
		if (waitForThreads) {
			m_ReaderThread.join(5000);
			m_WriterThread.join(5000);
		}
	}
}

void CTestTalkClientConnection::StartReader()
{
	RVThreadSelf self = ::rvThread();
	self.setPriority(self.getMaxPriority());

	try {
		doReading();
	} catch (...) {
	}
	Terminate(false);
	WlTraceMsg("CTestTalkClientConnection::StartReader - Done (%s,%d)", m_Name.data(), m_StreamSocket.m_Socket);
}

void CTestTalkClientConnection::StartWriter()
{
	RVThreadSelf self = ::rvThread();
	self.setPriority(self.getMaxPriority());

	try {
		doWriting();
	} catch (...) {
	}
	Terminate(false);
	WlTraceMsg("CTestTalkClientConnection::StartWriter - Done (%s,%d)", m_Name.data(), m_StreamSocket.m_Socket);
}


int CTestTalkClientConnection::newMessage(BaseEvent& be)
{
	//EnterProcedure("CTestTalkClientConnection::newMessage");

	TestTalkClientEvent* tmo = (TestTalkClientEvent*)&be;
	m_MsgsQueue->putEvent(tmo);

	return TRUE;
}

int CTestTalkClientConnection::doReading()
{
	void* msg;
	int len;
	short type, s_t;

	m_StreamSocket.SetBlockingMode(0);

	if (!m_IsStarted)
	{
		//
		// first read the connected client's details
		int st = m_StreamSocket.WaitTypedMessage(type, s_t, msg, len);
		NetClientData *ncd = (NetClientData*)msg;
		//
		// Sending back the NetClientData with host address.
		strncpy(ncd->localHostName, m_TestTalkMain->m_LocalHostAddress.data(), sizeof(ncd->localHostName));
		ncd->TestTalkPort = m_TestTalkMain->m_tcpNetPort;
		st = m_StreamSocket.SendTypedMessage(type, ControlQ, msg, len);
		//
		// creating the associated client-instance.
		clientInst = new TestTalkClientInstance(ncd->clientName, this, m_TestTalkMain);
		m_StreamSocket.FreeMessage(msg);
		//
		// create the writed thread:
		m_IsStarted=true;
		m_WriterThread.start();
	}

	while (1) {
		int st = m_StreamSocket.WaitTypedMessage(type, s_t, msg, len);
		if (st >= 0) {
			clientInst->doneReadingMessage(msg, len, type, s_t);
			m_StreamSocket.FreeMessage(msg);
		}
		else
		{
			//
			// treat the error situation here.
			WlTraceMsg("CTestTalkClientConnection::doReading - Error reading from socket\n"); //@@eyal: trace
			return -1;
		}
	}
}

int CTestTalkClientConnection::doWriting()
{
	while (1) {
		TestTalkClientEvent *cevent = (TestTalkClientEvent*)m_MsgsQueue->waitEvent();
		if (!cevent)
			return FALSE;

		int st = m_StreamSocket.SendTypedMessage(
			cevent->getType(),
			cevent->getQtype(),
			cevent->getPackedBuf(),
			cevent->getPackedBufLen());

		delete cevent;
		if (st < 0)
		{
			//
			// treat the error situation here.
			WlTraceMsg("CTestTalkClientConnection::doWriting - Error writing to socket\n"); //@@eyal: trace
			return FALSE;
		}
	}
}

