/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "testtalkmain.h"
#include <unistd.h>
#include <stdio.h>
#include "tvenv.h"
#include "tvpsync.h"
#include "tracedll.h"

#include <sys/resource.h>


#include <sys/wait.h>

CTestTalkMain *mainObj = NULL;

void terminateTesttalk(int sig)
{
  if(mainObj)
  {
    mainObj->Terminate();
    delete mainObj;
  }
  exit(0);
}

#ifdef __TV_linux__
void waitForChild(int sig)
{
  if ( SIGCHLD == sig ) {
    (void)waitpid(-1,NULL,WNOHANG);
  }
}
#endif // __TV_linux__


main(int argc, char *argv[])
{
  //initialize Environment
  TVEnvContainerClass tvpc;
  char thisFileName[MAX_PATH];
  memset(thisFileName,0,sizeof(thisFileName));
  (void)realpath(argv[0],thisFileName);
  const char* pathDir = tvpc.TVPGetPathDirectory(thisFileName);
  if( !TVEInit(pathDir))
  {
    TVPFatalErrorLog("tvenv.dll: Init Terminated with error: (%s)",TVEError());
    return FALSE;
  }

  //  Note that CTraceMsg::Init must be called AFTER the call to TVEInit.
  CTraceMsg::Init ("TestTalk") ;
  // create signal handler
  const char *SignalMask = tvpc.TVEGetEnv(ENV_SIGNAL_MASK);
  TVPSignalHandler *hnd = new TVPSignalHandler(SignalMask?SignalMask:"");
  hnd->TVPSigSet(SIGINT,terminateTesttalk);  // terminate on SIGINT
#ifdef __TV_linux__
  hnd->TVPSigSet(SIGCHLD,waitForChild); 
#endif // __TV_linux__

#if !defined (__TV_aix__)
  // raise the number of file descriptor.
  struct rlimit rlp;
  getrlimit(RLIMIT_NOFILE, &rlp);
  rlp.rlim_cur = rlp.rlim_max;
  setrlimit(RLIMIT_NOFILE, &rlp);
#endif

  mainObj = new CTestTalkMain();

  if( ! mainObj->Init() )
  {
    const char *ttClientPortStr = tvpc.TVEGetEnv("TESTTALK_CLIENT_PORT");
    if (!ttClientPortStr)
	ttClientPortStr = "9010";
    int ttClientPort = atoi(ttClientPortStr);
    const char *ttNetworkPortStr = tvpc.TVEGetEnv("TESTTALK_NETWORK_PORT");
    if (!ttNetworkPortStr)
	ttNetworkPortStr = "9000";
    int ttNetworkPort = atoi(ttNetworkPortStr);
    printf("\nTestTalk cannot start. Ports (%d and/or %d) are busy.\n",
	ttClientPort,ttNetworkPort); 
    printf("Check that testtalk/httpload is not already running and no other process are running on the required ports.\n");
    printf("Verify that the required ports are not already being used by another process\n");
    printf("by typing the command /bin/ps -elf | grep <port number>\n");
    printf("and looking in the /etc/services file.\n");
    exit(-1);
  }
  else
    while (1)
      sleep(3600);

}

