/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// AssociativeArray.cpp : Implementation of CAssociativeArrayCOM
#include "stdafx.h"
#include <comdef.h>
#include <atlbase.h>
#include "associativearrayobject.h"
#include "associativearraycom.h"
#include "ErrorLibLoader.h"

/////////////////////////////////////////////////////////////////////////////
// CAssociativeArrayCOM

STDMETHODIMP CAssociativeArrayCOM::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IAssociativeArray
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (::ATL::InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}


STDMETHODIMP CAssociativeArrayCOM::ToString(BSTR *Out_Val)
{

  try {
  	_bstr_t  tmp_bstr(m_AssociativeArray.ToString());
    *Out_Val = tmp_bstr.copy();
  }

  catch (std::exception& e) {
    MakeErrorInfo(e);
    return E_FAIL;
  }

  catch (std::exception* e) {
    MakeErrorInfo(*e);
    return E_FAIL;
  }

	return S_OK;
}

STDMETHODIMP CAssociativeArrayCOM::FromString(BSTR AssoString)
{

  LPOLESTR temp_str_w = (LPOLESTR)AssoString;
  USES_CONVERSION;
	const char *temp_str = OLE2A(temp_str_w);

  try {
    m_AssociativeArray.FromString(temp_str);
  }

  catch (std::exception& e) {
    MakeErrorInfo(e);
    return E_FAIL;
  }

  catch (std::exception* e) {
    MakeErrorInfo(*e);
    return E_FAIL;
  }

  return S_OK;
}

STDMETHODIMP CAssociativeArrayCOM::GetValueOfParameter(BSTR Key,/*[out,retval]*/ BSTR *Out_Val)
{

  LPOLESTR temp_str_w = (LPOLESTR)Key;
	USES_CONVERSION;
  const char *temp_str = OLE2A(temp_str_w);

  try {
    const char* tmp_str1 =  m_AssociativeArray.GetValueOfParameter(temp_str);
    _bstr_t  tmp_bstr(tmp_str1);
    *Out_Val = tmp_bstr.copy();
  }

  catch (std::exception& e) {
    MakeErrorInfo(e);
    return E_FAIL;
  }

  catch (std::exception* e) {
    MakeErrorInfo(*e);
    return E_FAIL;
  }
	return S_OK;
}



STDMETHODIMP CAssociativeArrayCOM::GetLongValueOfParameter(BSTR Key, long *Out_Val)
{
	
  LPOLESTR temp_str_w = (LPOLESTR)Key;
  USES_CONVERSION;
  const char *temp_str = OLE2A(temp_str_w);

  try {
      *Out_Val =  m_AssociativeArray.GetLongValueOfParameter(temp_str);
  }

  catch (std::exception& e) {
    MakeErrorInfo(e);
    return E_FAIL;
  }

  catch (std::exception* e) {
    MakeErrorInfo(*e);
    return E_FAIL;
  }

	return S_OK;
}

STDMETHODIMP CAssociativeArrayCOM::GetDoubleValueOfParameter(BSTR Key, double *Out_Val)
{
	
  LPOLESTR temp_str_w = (LPOLESTR)Key;
	USES_CONVERSION;
	const char *temp_str = OLE2A(temp_str_w);

  try {
    *Out_Val =  m_AssociativeArray.GetDoubleValueOfParameter(temp_str);
  }

  catch (std::exception& e) {
    MakeErrorInfo(e);
    return E_FAIL;
  }

  catch (std::exception* e) {
    MakeErrorInfo(*e);
    return E_FAIL;
  }
	return S_OK;
}

STDMETHODIMP CAssociativeArrayCOM::GetValueOf2Parameter(BSTR Key, BSTR DefaultValue, BSTR *Out_Val)
{

  LPOLESTR temp_str_w1 = (LPOLESTR)Key;
	USES_CONVERSION;
  const char *temp_str1 = OLE2A(temp_str_w1);

  LPOLESTR temp_str_w2 = (LPOLESTR)DefaultValue;
  const char *temp_str2 = OLE2A(temp_str_w2);

  try {
    const char* tmp_str3 =  m_AssociativeArray.GetValueOfParameter(temp_str1,temp_str2);
  
   	_bstr_t  tmp_bstr(tmp_str3);
    *Out_Val = tmp_bstr.copy();
  }

  catch (std::exception& e) {
    MakeErrorInfo(e);
    return E_FAIL;
  }

  catch (std::exception* e) {
    MakeErrorInfo(*e);
    return E_FAIL;
  }

	return S_OK;
}

STDMETHODIMP CAssociativeArrayCOM::GetLongValueOf2Parameter(BSTR Key, long DefaultValue, long *Out_Val)
{
  LPOLESTR temp_str_w = (LPOLESTR)Key;
  USES_CONVERSION;
  const char *temp_str = OLE2A(temp_str_w);

  try {
    *Out_Val =  m_AssociativeArray.GetLongValueOfParameter(temp_str,DefaultValue);
  }

  catch (std::exception& e) {
    MakeErrorInfo(e);
    return E_FAIL;
  }

  catch (std::exception* e) {
    MakeErrorInfo(*e);
    return E_FAIL;
  }

  return S_OK;
}

STDMETHODIMP CAssociativeArrayCOM::GetDoubleValueOf2Parameter(BSTR Key, double DefaultValue, double *Out_Val)
{
  LPOLESTR temp_str_w = (LPOLESTR)Key;

  USES_CONVERSION;
  const char *temp_str = OLE2A(temp_str_w);

  try {
    *Out_Val =  m_AssociativeArray.GetDoubleValueOfParameter(temp_str,DefaultValue);
  }

  catch (std::exception& e) {
    MakeErrorInfo(e);
    return E_FAIL;
  }

  catch (std::exception* e) {
    MakeErrorInfo(*e);
    return E_FAIL;
  }
	
  
  return S_OK;
}

STDMETHODIMP CAssociativeArrayCOM::AddCouple_Char(BSTR Key, BSTR Value)
{
  LPOLESTR temp_str_w1 = (LPOLESTR)Key;
	USES_CONVERSION;
  const char *temp_str1 = OLE2A(temp_str_w1);

  LPOLESTR temp_str_w2 = (LPOLESTR)Value;
  const char *temp_str2 = OLE2A(temp_str_w2);

  m_AssociativeArray.AddCouple(temp_str1,temp_str2);

	return S_OK;
}

STDMETHODIMP CAssociativeArrayCOM::AddCouple_Double(BSTR Key, double Value)
{
  LPOLESTR temp_str_w1 = (LPOLESTR)Key;
	USES_CONVERSION;
  const char *temp_str1 = OLE2A(temp_str_w1);

  m_AssociativeArray.AddCouple(temp_str1,Value);

	return S_OK;
}

STDMETHODIMP CAssociativeArrayCOM::AddCouple_Long(BSTR Key, long  Value)
{
  LPOLESTR temp_str_w1 = (LPOLESTR)Key;
	USES_CONVERSION;
  const char *temp_str1 = OLE2A(temp_str_w1);

  m_AssociativeArray.AddCouple(temp_str1,Value);

	return S_OK;
}


STDMETHODIMP CAssociativeArrayCOM::AddArrayCouple_Char(BSTR Key, BSTR Value,int index)
{
  LPOLESTR temp_str_w1 = (LPOLESTR)Key;
	USES_CONVERSION;
  const char *temp_str1 = OLE2A(temp_str_w1);

  LPOLESTR temp_str_w2 = (LPOLESTR)Value;
  const char *temp_str2 = OLE2A(temp_str_w2);

  m_AssociativeArray.AddArrayCouple(temp_str1,temp_str2,index);

	return S_OK;
}

STDMETHODIMP CAssociativeArrayCOM::AddArrayCouple_Double(BSTR Key, double Value,int index)
{
  LPOLESTR temp_str_w1 = (LPOLESTR)Key;
	USES_CONVERSION;
  const char *temp_str1 = OLE2A(temp_str_w1);

  m_AssociativeArray.AddArrayCouple(temp_str1,Value,index);

	return S_OK;
}

STDMETHODIMP CAssociativeArrayCOM::AddArrayCouple_Long(BSTR Key, long  Value, int index)
{
  LPOLESTR temp_str_w1 = (LPOLESTR)Key;
	USES_CONVERSION;
  const char *temp_str1 = OLE2A(temp_str_w1);

  m_AssociativeArray.AddArrayCouple(temp_str1,Value,index);

	return S_OK;
}

STDMETHODIMP CAssociativeArrayCOM::GetValueOf2ParameterArray(BSTR Key, int index, BSTR *Out_Val)
{
  LPOLESTR temp_str_w = (LPOLESTR)Key;
	USES_CONVERSION;
  const char *temp_str = OLE2A(temp_str_w);

  try {
    const char* tmp_str1 =  m_AssociativeArray.GetValueOfParameterArray(temp_str,index);
  	_bstr_t  tmp_bstr(tmp_str1);
    *Out_Val = tmp_bstr.copy();
  }

  catch (std::exception& e) {
    MakeErrorInfo(e);
    return E_FAIL;
  }

  catch (std::exception* e) {
    MakeErrorInfo(*e);
    return E_FAIL;
  }

	return S_OK;
}

STDMETHODIMP CAssociativeArrayCOM::GetLongValueOf2ParameterArray(BSTR Key, int index, long *Out_Val)
{
  LPOLESTR temp_str_w = (LPOLESTR)Key;
	USES_CONVERSION;
  const char *temp_str = OLE2A(temp_str_w);

  try {
    *Out_Val =  m_AssociativeArray.GetLongValueOfParameterArray(temp_str,index);
  }

  catch (std::exception& e) {
    MakeErrorInfo(e);
    return E_FAIL;
  }

  catch (std::exception* e) {
    MakeErrorInfo(*e);
    return E_FAIL;
  }

  return S_OK;
}

STDMETHODIMP CAssociativeArrayCOM::GetDoubleValueOf2ParameterArray(BSTR Key, int index, double *Out_Val)
{
  LPOLESTR temp_str_w = (LPOLESTR)Key;
	USES_CONVERSION;
  const char *temp_str = OLE2A(temp_str_w);

  *Out_Val =  m_AssociativeArray.GetDoubleValueOfParameterArray(temp_str,index);

	return S_OK;
}


STDMETHODIMP CAssociativeArrayCOM::GetValueOf3ParameterArray(BSTR Key, int index, BSTR DefaultValue, BSTR *Out_Val)
{
  LPOLESTR temp_str_w1 = (LPOLESTR)Key;
	USES_CONVERSION;
  const char *temp_str1 = OLE2A(temp_str_w1);

  LPOLESTR temp_str_w2 = (LPOLESTR)DefaultValue;
  const char *temp_str2 = OLE2A(temp_str_w2);

  try {

    const char* tmp_str3 =  m_AssociativeArray.GetValueOfParameterArray(temp_str1,index,temp_str2);
    _bstr_t  tmp_bstr(tmp_str3);
    *Out_Val = tmp_bstr.copy();
  }

  catch (std::exception& e) {
    MakeErrorInfo(e);
    return E_FAIL;
  }

  catch (std::exception* e) {
    MakeErrorInfo(*e);
    return E_FAIL;
  }

  return S_OK;
}



STDMETHODIMP CAssociativeArrayCOM::GetLongValueOf3ParameterArray(BSTR Key, int index, long DefaultValue,  long *Out_Val)
{
  LPOLESTR temp_str_w = (LPOLESTR)Key;
	USES_CONVERSION;
  const char *temp_str = OLE2A(temp_str_w);

  try {
    *Out_Val =  m_AssociativeArray.GetLongValueOfParameterArray(temp_str,index,DefaultValue);
  }

  catch (std::exception& e) {
    MakeErrorInfo(e);
    return E_FAIL;
  }

  catch (std::exception* e) {
    MakeErrorInfo(*e);
    return E_FAIL;
  }

	return S_OK;
}

STDMETHODIMP CAssociativeArrayCOM::GetDoubleValueOf3ParameterArray(BSTR Key, int index, double DefaultValue,double *Out_Val)
{
  LPOLESTR temp_str_w = (LPOLESTR)Key;
	USES_CONVERSION;
  const char *temp_str = OLE2A(temp_str_w);

  try {
    *Out_Val =  m_AssociativeArray.GetDoubleValueOfParameterArray(temp_str,index,DefaultValue);
  }

  catch (std::exception& e) {
    MakeErrorInfo(e);
    return E_FAIL;
  }

  catch (std::exception* e) {
    MakeErrorInfo(*e);
    return E_FAIL;
  }

return S_OK;
}

STDMETHODIMP CAssociativeArrayCOM::KeyExists(BSTR Key, VARIANT *Out_Val)
{

  LPOLESTR temp_str_w = (LPOLESTR)Key;
	USES_CONVERSION;
  const char *temp_str = OLE2A(temp_str_w);

	  try {
      bool result  = m_AssociativeArray.KeyExists(temp_str);
    


  _variant_t tmp_variant;
  VariantInit(&tmp_variant);

  tmp_variant.vt = VT_BOOL;
  tmp_variant.boolVal = result;

  *Out_Val = tmp_variant.Detach();
  }

  catch (std::exception& e) {
    MakeErrorInfo(e);
    return E_FAIL;
  }

  catch (std::exception* e) {
    MakeErrorInfo(*e);
    return E_FAIL;
  }

	return S_OK;
}

STDMETHODIMP CAssociativeArrayCOM::Reset()
{
	m_AssociativeArray.Reset();

	return S_OK;
}


void CAssociativeArrayCOM::MakeErrorInfo(std::exception &e){

  ICreateErrorInfo *pcerrinfo;
  IErrorInfo *perrinfo;
  HRESULT hr;

  hr = CreateErrorInfo(&pcerrinfo);

  hr = pcerrinfo->QueryInterface(IID_IErrorInfo, (LPVOID FAR*) &perrinfo);

	RVCString tmp_Msg("");
	//WLERRGetErrorMsg(&e, tmp_Msg);
    dllLoader.WLERRGetErrorMsg(&e, tmp_Msg);

  const char *tmp_info =  tmp_Msg.data();

  _bstr_t  tmp_bstr(tmp_info);

   pcerrinfo->SetDescription(tmp_bstr);

   if (SUCCEEDED(hr)){
           SetErrorInfo(0, perrinfo);
           perrinfo->Release();
   }
   pcerrinfo->Release();

}
