/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// errdef.h -- error mechanism definitions
//
//   who          when          what
//  Iris        28-11-93      Created
//
//
#ifndef __WLERRORLIB_H
#define __WLERRORLIB_H

#include <RVCString.h>
#include <exception>
//#include "platform.h"

typedef enum {
	  Success,
	  Note,
	  Warning,
	  Error,
    Fatal	
  } WLSeverity;

class iWLException : public std::exception {
public:
	iWLException();
	~iWLException();
	
	virtual const char* what() const; // Override standard default behavior
	virtual const char* GetErrorMsg() const = 0;

	virtual bool StackTrace(RVCString& outBuf, unsigned skip = 0, const char* fmt = 0);
private:
//	CSymEngine	m_SymEngine;
	RVCString		m_StackTrace;
};

class WLErrorException : public iWLException {
private:
  RVCString errorMsg;
  WLSeverity severity;

public:
  WLErrorException(const char* msg, WLSeverity sev, ...);
  WLErrorException(const char* errFileName, int fmtId, ...);
  // @@ofer: the format of copy ctor should be X(const X&) but since WebRM 4.6 uses the same dlls we cannot modify the interface
  // in 6.0 we should use X(const X&) for ALL platforms
#ifdef __TV_UNIX__
  WLErrorException(const WLErrorException& errorException);
#else
  WLErrorException(WLErrorException& errorException);
#endif // __TV_UNIX__
  virtual ~WLErrorException();

  virtual const char* GetErrorMsg() const;
  WLSeverity GetErrorSeverity() const;
};

class CAssociativeArrayException : public WLErrorException
{
public:
	CAssociativeArrayException(WLErrorException& error) : WLErrorException(error) {}

	~CAssociativeArrayException() {}
};

class CIllegalAssoSyntaxException : public CAssociativeArrayException
{
public:
	CIllegalAssoSyntaxException( const char* AssoString ) : 
	  CAssociativeArrayException( WLErrorException("loadengine.frm", 10, AssoString) )	{}
	  ~CIllegalAssoSyntaxException() {}
};


class CAssoKeyDoesNotExistException : public CAssociativeArrayException
{
public:
	CAssoKeyDoesNotExistException( const char* key, const char* AssoString ) : 
	  CAssociativeArrayException( WLErrorException("loadengine.frm", 11, key, AssoString) )	{}
	  ~CAssoKeyDoesNotExistException() {}
};
#endif // __WLERRORLIB_H

