/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// SupportInformationGenerator.h: interface for the CSupportInformationGenerator class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SUPPORTINFORMATION_H__71A36C33_3479_11D4_87EC_0004AC15A422__INCLUDED_)
#define AFX_SUPPORTINFORMATION_H__71A36C33_3479_11D4_87EC_0004AC15A422__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifdef __TV_WINDOWS__
#include <ace/OS.h>
#include <windows.h>
#include <io.h>
#include <direct.h>
#endif

#include <stdio.h>
#include <errno.h>
#include <fstream.h>
#include <iostream.h>
//#include <rw/strstrea.h>

#include "associativearray.h"

#define FILE_NAME_LENGTH 30
#define COMMAND_LINE_SIZE (_MAX_PATH + 30)

#define BUFFER 4096
#define DIV 1024 // 000


class DllExport CSupportInformationGenerator
{
public:
	CSupportInformationGenerator(void);
	virtual ~CSupportInformationGenerator(void);

	// Creates general system information.
	// HW profile, memory status etc' are flushed into file: _SysInfoFileName.
	// IP configuration is flushed into file: _IpConfigFileName.
	bool  GenerateSystemInfo(/*[IN]*/char* _IpConfigFileName,
													 /*[IN]*/char* _SysInfoFileName);

	// Creates an information DB and inserts it into infoDB.
	// This method uses GenerateSystemInfo() method but it deletes
	// the two file that this method created.
	// Keys = { $HOSTNAME, $SYSTEMREPORT, $IPCONFIG, $TVCTIDLL, $WEBLOADINI }
	bool  CreateInformationDB(/*[OUT]*/CAssociativeArray& infoDB);

	// Gets and information DB and save it into reportFile.
	bool  GenerateReportFile(/*[OUT]*/ofstream& reportFile,
													 /*[IN]*/CAssociativeArray& infoDB);

  // Adds a buffer of a specified file to infoDB under the specified key.
	bool  AddFileToDB(/*[IN]*/const char* key,
										/*[IN]*/const char* _FilePath,
										/*[OUT]*/CAssociativeArray& infoDB);

	bool  GetWLDirectoryPath(char* string, int strLength);

protected:
#if defined(__TV_WINDOWS__)
	// BEGIN: Methods for windows platform only
	void GetWindowsSystemInfo(/*[OUT]*/ostream& out);
	void GetHWProfile(/*[OUT]*/ostream& out);
	BOOL DisplaySystemVersion(/*[OUT]*/ostream& out);
	void GetMemoryStatus(/*[OUT]*/ostream& out);
	void GetDiskInfo(/*[OUT]*/ostream& out);
	void GetServicesList(/*[OUT]*/ostream& out);

	/* this method generates the information created by all
	   the methods before */
	bool GenerateWindowsInfo(/*[IN]*/const char* _SysInfoFileName);
	// END.
#endif

	bool GenerateIPConfigInfo(/*[IN]*/const char* _IpConfigFileName);
	bool CheckGenerateError(int status);
};

#endif // !defined(AFX_SUPPORTINFORMATION_H__71A36C33_3479_11D4_87EC_0004AC15A422__INCLUDED_)

