/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
//**********************************************************************
// Name : nvsmsg.cpp
// Purpose : implement NetView messages to tml.
//
// Creation: Yahali, 16-Nov-93
// Updates :  Who          When         What
//
//**********************************************************************/
#ifdef __TV_WINDOWS__
//#include "stdafx.h"
#endif
#ifndef __TV_UNIX__
#include <crtdbg.h>
#endif

#include <string.h>
#include <stdlib.h>
#include "tvdebug.h"
#include "nvsmsg.h"
#include "msghdr.h"


//**********************************************************************
// Name : NVSSendMsg
// Purpose : send a message to a givven queue.
//
// Input : hData     - a global Memory handler of the message.
//         queue     - the queue(HWND) to send the message to.
//         msgType  - the type of the message being sent.
//
// Output :
//
// Return values: Fail/Success.
//
// Creation: Yahali, 16-Nov-93
// Updates :  Who          When         What
//
//**********************************************************************/
Bool WINAPI NVSSendMsg( HCLIENTDATA hData,
                        CLIENTQUEUE queue,
                        MSGTYPE msgType)
{
	EnterProcedure("NVSSendMsg");
  if (!queue)
    return False;
	//assert(queue);
	//debugInfo(("Sending Message '%s' type %d to Queue: %d (%s)\n",getMsgByHandle(hData),msgType,queue,(isWait?"Wait":"NoWait")));
	//
	// Send the msg to CLIENTQUEUE.

#ifdef __TV_WINDOWS__
	COPYDATASTRUCT c;
	if (!hData)
	{
		c.cbData=0;
		c.lpData="";
		hData = &c;
	}

	hData->dwData=msgType;
  return((Bool)( SendMessage((HWND)queue,
                        WM_COPYDATA,
                        0,
                        (LPARAM)hData)));
#endif
#ifdef __TV_UNIX__
// UNIXCODE
  return True;
#endif
}

//**********************************************************************
// Name : NVSPostMsg
// Purpose : send a message to a givven queue.
//
// Input : hData     - a global Memory handler of the message.
//         queue     - the queue(HWND) to send the message to.
//         msgType  - the type of the message being sent.
//
// Output :
//
// Return values: Fail/Success.
//
// Creation: Yahali, 16-Nov-93
// Updates :  Who          When         What
//
//**********************************************************************/
Bool WINAPI NVSPostMsg( MSGTYPE msgType,
                 CLIENTQUEUE queue)
{
	EnterProcedure("");
//  assert(hData);
  //debugInfo(("Sending Message '%s' type %d to Queue: %d (%s)\n",getMsgByHandle(hData),msgType,queue,(isWait?"Wait":"NoWait")));
  //
  // Send the msg to CLIENTQUEUE.
#ifdef __TV_WINDOWS__
  return((Bool)( PostMessage((HWND)queue,
                             msgType,
                             0,
                             0)));
#endif
#ifdef __TV_UNIX__
// UNIXCODE
  return True;
#endif
}

//**********************************************************************
// Name : NVSWaitMsg
// Purpose : wait for the givven message on the givven Q.
//
// Input : HWND hWnd - the window to wait for message at.
//         UNIT wMsg - the message type to wait for.
//
// Output : the message passed through.
//
// Return values: C "output:".
//
// Creation: Yahali, 21-Nov-93
// Updates :  Who          When         What
//
//**********************************************************************/
HCLIENTDATA WINAPI NVSWaitMsg(HWND hWnd,UINT wMsg)
{
#ifdef __TV_WINDOWS__
  MSG msg;

  assert(hWnd);

  while (GetMessage (&msg, hWnd, 0, 0)) {
    if (wMsg == 0 || msg.message == WM_COPYDATA) {
      return (HCLIENTDATA)(msg.lParam);
    }
  }
#endif
#ifdef __TV_UNIX__
// UNIXCODE
#endif
  return NULL;    // Will never get here (only when stop arrives).
}

//**********************************************************************
// Name : NVSFetchData
// Purpose : fetch the givven message on the givven Q.
//
// Input : HWND hWnd - the window to wait for message at.
//         UNIT wMsg - the message type to wait for.
//
// Output : the message passed through.
//
// Return values: C "output:".
//
// Creation: Yahali, 28-Nov-93
// Updates :  Who          When         What
//
//**********************************************************************/
HCLIENTDATA WINAPI NVSFetchData(HWND hWnd,UINT wMsg)
{
#ifdef __TV_WINDOWS__
  MSG msg;

  assert(hWnd);

  while(PeekMessage (&msg, hWnd, 0, 0, PM_NOYIELD|PM_REMOVE)) {
    if (msg.message == WM_COPYDATA) {
      return (HCLIENTDATA)(msg.lParam);
    }
  }
#endif
#ifdef __TV_UNIX__
// UNIXCODE
#endif
  return NULL;
}

int WINAPI NVSPeekMsg(HWND hWnd,UINT wMsg)
{
#ifdef __TV_WINDOWS__
  MSG msg;

  assert(hWnd);

  while(PeekMessage (&msg, hWnd, 0, 0, PM_NOYIELD|PM_NOREMOVE)) {
    if (msg.message == WM_COPYDATA)
      return TRUE;
    else
      PeekMessage (&msg, hWnd, 0, 0, PM_NOYIELD|PM_REMOVE);
  }
#endif
#ifdef __TV_UNIX__
// UNIXCODE
#endif
  return NULL;
}

//**********************************************************************
// Name : NVSAlloc
// Purpose : Allocate global memory at the requested length.
//
// Input : lenToAlloc - the required length of global memory.
//
// Output :
//
// Return values: the Global memoty handler.
//
// Creation: Yahali, 9-Nov-93
// Updates :  Who          When         What
//
//**********************************************************************/
HCLIENTDATA WINAPI NVSAlloc(unsigned int lenToAlloc)
{
  // Set flag to zero
  assert(lenToAlloc > 0);
  HCLIENTDATA h = new COPYDATASTRUCT;
	h->cbData = lenToAlloc;
	h->lpData = malloc(lenToAlloc);

  return h;
}

//**********************************************************************
// Name : NVSGetData
// Purpose : get a char pointer to the data in global memory.
//
// Input : hData - a handler to a global memory.
//
// Output :
//
// Return values: a pointer to the global menory.
//
// Creation: Yahali, 9-Nov-93
// Updates :  Who          When         What
//
//**********************************************************************/
char* WINAPI NVSGetData(HCLIENTDATA hData)
{
  assert(hData!=NULL);
  return((char *)(hData->lpData));
}

//**********************************************************************
// Name : NVSReleaseData
// Purpose : Unlock the global memory previously Locked.
//
// Input : hData - a Global memory Handler.
//
// Output :
//
// Return values: Fail/Success
//
// Creation: Yahali, 9-Nov-93
// Updates :  Who          When         What
//
//**********************************************************************/
Bool WINAPI NVSReleaseData(HCLIENTDATA hData)
{
  assert(hData);
  return True;
}

//**********************************************************************
// Name : NVSFreeData
// Purpose : Free the global memory previously Locked.
//
// Input : hData - a Global memory Handler.
//
// Output :
//
// Return values: Fail/Success
//
// Creation: Yahali, 9-Nov-93
// Updates :  Who          When         What
//
//**********************************************************************/
Bool WINAPI NVSFreeData(HCLIENTDATA hData)
{
  try {
    assert(hData);
	  free(hData->lpData);
	  delete hData;
	  return True;
  }  catch (...) {}
  return False;
}

HCLIENTDATA WINAPI NVSPackData(void* data, long len, short type, short subtype)
{
	EnterProcedure("NVSPackData");
	int l = len + sizeof(MessageHeader);
  HCLIENTDATA hData = NVSAlloc(l);
  if (!hData)
    return FALSE;
  char* p = NVSGetData(hData);
  MessageHeader mh;
  SSMMakeMessageHeader(len,type,subtype,&mh);
  memcpy((void*)p,(void*)&mh,sizeof(MessageHeader));
  p+=sizeof(MessageHeader);
  memcpy((void*)p, data, len);
  NVSReleaseData(hData);

  return hData;
}

void WINAPI NVSUnPackData(HCLIENTDATA hData, void*& data, long& len, short& type, short& subtype)
{
	EnterProcedure("NVSUnPackData");
	char* p = NVSGetData(hData);
	MessageHeader *mh = (MessageHeader *)p;
	data = (void*)(p+sizeof(MessageHeader));
	len = SSMGetMessageHeaderLen(*mh);
	type = SSMGetMessageHeaderType(*mh);
	subtype = SSMGetMessageHeaderSubtype(*mh);

	return;
}

