/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "socketlib.h"
#include "tvenv.h"
#include <RVCString.h>
#include <string.h>

#ifdef __TV_UNIX__
#include <string.h>
#include <errno.h>
extern int errno;

#endif

bool CSocketLib::GetHostByName(const char *host, struct in_addr& sin_addr)
{
#ifdef __TV_WINDOWS__
  struct hostent      *result;
  unsigned long inetAddr = inet_addr(host);
  if (inetAddr != INADDR_NONE)  {
    memcpy((void*)&sin_addr, (void*)&inetAddr, sizeof(inetAddr));
    return true;
  }
  if ((result = gethostbyname(host)) != NULL)
  {
    memcpy((void*)&sin_addr, result->h_addr, result->h_length);
    return true;
  }
  
#elif (defined __TV_solaris__)
#define INADDR_NONE -1

  struct hostent   result;
  int    thd_errno;
  char   buffer[MAXHOSTNAMELEN];

  unsigned long inetAddr = inet_addr(host);
  if (inetAddr != INADDR_NONE)  {
    memcpy((void*)&sin_addr, (void*)&inetAddr, sizeof(inetAddr));
    return true;
  }
  if (gethostbyname_r(host, &result, buffer, MAXHOSTNAMELEN, &thd_errno) != NULL) {
    memcpy((void*)&sin_addr, result.h_addr, result.h_length);
    return true;
  }
  
#elif (defined __TV_aix__)
  struct hostent   result;
  struct hostent_data hd;

  unsigned long inetAddr = inet_addr(host);
  if (inetAddr != INADDR_NONE)  {
    memcpy((void*)&sin_addr, (void*)&inetAddr, sizeof(inetAddr));
    return true;
  }

  if (gethostbyname_r(host, &result, &hd) == 0) {
    memcpy((void*)&sin_addr, result.h_addr, result.h_length);
    return true;
  }
  
#elif (defined __TV_linux__)
  struct hostent      *result;
  unsigned long inetAddr = inet_addr(host);
  if (inetAddr != INADDR_NONE)  {
    memcpy((void*)&sin_addr, (void*)&inetAddr, sizeof(inetAddr));
    return true;
  }
  if ((result = gethostbyname(host)) != NULL)
  {
    memcpy((void*)&sin_addr, result->h_addr, result->h_length);
    return true;
  }
#endif

  return false;
}

bool CSocketLib::GetAddress(const char *host, u_short port, struct sockaddr_in& address)
{
  //
  // load the server structure.
  memset((char*)&address, 0, sizeof(address)); // set to 0x00 all bytes.
  address.sin_family = AF_INET;
  address.sin_port = htons(port);
  if (!GetHostByName(host, address.sin_addr))
    return false;
  else
    return true;
}

#ifdef __TV_UNIX__
#ifndef __TV_linux__
#if (defined _cplusplus) || (defined __cplusplus)
extern "C" {
#endif
extern int gethostname(char *name, int namelen);
#if (defined _cplusplus) || (defined __cplusplus)
}
#endif /* __cplusplus */
#endif /* __TV_linux__ */
#endif /* __TV_UNIX__*/

bool CSocketLib::GetLocalHostName(char* szName, int nNameLength) 
{
//This function returns the name of the computer on which this program is running. The
//  szName paramter will be set to be the name.
//The return value is either WINSOCK_ERROR or WINSOCK_NOERROR.  You can call the
//  HostInfoManager's GetLastError() to get the actual error value.
//szName is a pointer to pre-allocated buffer of minimum size of nNameLength
//nNameLength should be at least N_MAX_HOST_NAME.

	 // @@@@Eyal - new code
   TVEnvContainerClass tvpc;
   const char* ip = tvpc.TVEGetEnv("TESTTALK_IP_ADDRESS");
	 
	 if(ip != NULL) {
		 RVCString localHostIP = ip;

		 if( (localHostIP.length() > 0) && (localHostIP != "localhost" ) ) {
				if(localHostIP.length() >= nNameLength) {
					return false;
				}
				in_addr sin_addr;
				if( !GetHostByName(localHostIP.data(), sin_addr) ) {
					return false;
				}
				strcpy(szName, localHostIP.data());
				return true;
		 }
	 }
   
	 if(gethostname(szName, nNameLength) != 0) {  //sockets call
      return false;
   }
   return true;
}

bool CSocketLib::GetLocalHostName(char* szName, int nNameLength, bool resolveHostName)
// this method is identical to GetLocalHostName(szName, nNameLength)
// except that it is checking the flag  resolveHostName  first:
// If resolveHostName == false TESTTALK_IP_ADDRESS key doesn't 
//   exists in webload.ini or it exists but has no value or 
//   'localhost' value the method returns false else it calls the
//   first GetLocalHostName().
// If resolveHostName == true it acts exactly as GetLocalHostName(szName, nNameLength)
{
	TVEnvContainerClass tvpc;
	const char* ip = tvpc.TVEGetEnv("TESTTALK_IP_ADDRESS");

	if(resolveHostName == false) {
		if(ip == NULL) {
			return false;
		}
		else {
			RVCString localHostIP = ip;

			if( (localHostIP.length() == 0) || (localHostIP == "localhost" ) ) {
				return false;
			}
		}
	}
	
	return GetLocalHostName(szName, nNameLength);
}


int CSocketLib::GetLastError() 
{
#ifdef __TV_WINDOWS__
  return WSAGetLastError();
#else
  return errno;
#endif
}

