/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <windows.h>
#include <fstream.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include "tvtables.h"
#include "platform.h"
#include "tvver.h"
#include "tvenv.h"
#include "resource.h"
#include <assert.h>
#include <io.h>
#include <stdio.h>
#include <stdlib.h>


HINSTANCE  hDllInstance;
TableHandle table;

BOOL WINAPI DllMain( HINSTANCE hinstDll,
					DWORD fdwRreason,
					LPVOID plvReserved)
{
    BOOL retval = TRUE;
    static HANDLE hDllMutex=NULL ;
    char fname[MAX_PATH];
    TVEnvContainerClass tvpc;
	
    if (hDllMutex == NULL)
		hDllMutex = CreateMutex(NULL,FALSE,"TVVerDllEntryMutex");
	
    switch (fdwRreason){
	case DLL_PROCESS_ATTACH:
		{
			hDllInstance = hinstDll;
			table = TVSCreateTable();
			
			
			// search for ini file in windows directory
			char thisFileName[MAX_PATH];
			char thisFileDir[MAX_PATH];
			if (!GetModuleFileName(hinstDll, thisFileName, sizeof(thisFileName))) {
				assert(0);
				retval = FALSE;
				break;
			}
			strcpy(thisFileDir, tvpc.TVPGetPathDirectory(thisFileName));
			
			//read from registry lang name, load file XXXX\tvcti.dll
			
			LONG  ret;
			HKEY  Key = NULL;
			
			char * Path = "SOFTWARE\\RadView\\";
			unsigned long length = 256 ;
			char  pLang[256];
			strcpy (pLang, "English");
			ret = RegOpenKeyEx(HKEY_LOCAL_MACHINE, Path, 0, KEY_READ,&Key);
			if ( ret == ERROR_SUCCESS )
			{
				ret = RegQueryValueEx(Key, "ResourceLang" ,NULL,NULL,(LPBYTE)pLang,&length);
				RegCloseKey(Key);
			}	
			strcat (thisFileDir, "\\");
			strcat (thisFileDir, pLang);
			char strFile[MAX_PATH];
			strcpy(strFile, thisFileDir);
			strcat(strFile, "\\tvcti.dll");
			if ( (_access(thisFileDir, 0) == -1)  || (_access(strFile, 0) == -1))  //directory or file doesn't exist
				strcpy(thisFileDir, tvpc.TVPGetPathDirectory(thisFileName));
			
			const char* inifile = tvpc.TVPFindfileNoTVEnv("tvcti.dll", thisFileDir, "HOME", "$TV_BIN", NULL);
			if (!inifile) {
				
				TVPFatalErrorLog("tvver.dll: tvcti.dll not found");
				retval = FALSE;
				break;
			}
			strcpy(fname,inifile);
			DWORD mutexRes = WaitForSingleObject(hDllMutex, 10000);
			TVSLoad(fname,table,NULL);				 
			
			if (!TVEInit(tvpc.TVPGetPathDirectory( (const char *)thisFileName )))
			{
				TVPFatalErrorLog("tvenv.dll: Init Terminated with error: (%s)",TVEError());
				retval = FALSE;
				break;
			}

			if (mutexRes == WAIT_OBJECT_0)
				ReleaseMutex(hDllMutex);
		}
        break;
		
	case DLL_PROCESS_DETACH:
        //TVSDestroyTable(table);
		break;
    }
//	ReleaseMutex(hDllMutex);
    return retval;
}





const char * WINAPI TVVersionInfo(const char* key)
{
	return TVSGetValue(table,key);
}

const char*  WINAPI  TVVProductName()
{
	return TVVersionInfo("PRODUCT_NAME");
}

const char*  WINAPI  TVVProductNameDisplay()
{
	return TVVersionInfo("PRODUCT_NAME_DISPLAY");
}

const char*  WINAPI  TVVProductsList()
{
	const char* retVal = TVVersionInfo("PRODUCTS_LIST");
	
	return (retVal == NULL? "":retVal);
}

const char*  WINAPI  TVVProductFileName()
{
	return TVVersionInfo("PRODUCT_FILE_NAME");
}

const char*  WINAPI  TVVProductVersion()
{
	return TVVersionInfo("PRODUCT_VERSION");
}

const char*  WINAPI  TVVProductVersionName()
{
	return TVVersionInfo("PRODUCT_VERSION_NAME");
}

const char*  WINAPI  TVVProductMessage()
{
	return TVVersionInfo("PRODUCT_MESSAGE");
}

const char*  WINAPI  TVVProductErrorMessage()
{
	return TVVersionInfo("PRODUCT_ERROR_MESSAGE");
}

const char*  WINAPI  TVVProductQuery()
{
	return TVVersionInfo("PRODUCT_QUERY");
}

const char*  WINAPI  TVVEditorTitle()
{
	return  TVVersionInfo("EDITOR_TITLE");
}

const char*  WINAPI  TVVProductIniFile()
{
	return  TVVersionInfo("PRODUCT_INI_FILE");
}

const char*  WINAPI  TVVProductIniBackup()
{
	return  TVVersionInfo("PRODUCT_INI_BACKUP");
}

const char*  WINAPI  TVVProductHelpFile()
{
	return  TVVersionInfo("PRODUCT_HELP_FILE");
}

const char*  WINAPI  TVVProductFrmFile()
{
	return  TVVersionInfo("PRODUCT_FRM_FILE");
}

HBITMAP  WINAPI  TVVProductCover()
{
	return LoadBitmap(hDllInstance, TVSGetValue(table,"PRODUCT_BMP"));
}

int     WINAPI   TVVProductCoverX()
{
	return  atoi(TVVersionInfo("BMP_X_SIZE"));
}

int     WINAPI   TVVProductCoverY()
{
	return  atoi(TVVersionInfo("BMP_Y_SIZE"));
}

int    WINAPI  TVVProductIconNumber()
{
	return  atoi(TVVersionInfo("PRODUCT_ICON_NUMBER"));
}

int      WINAPI  TVVProductType()
{
	return  atoi(TVVersionInfo("PRODUCT_TYPE"));
}

HICON  WINAPI  TVVProductIcon()
{
	return LoadIcon(hDllInstance, TVVersionInfo("PRODUCT_ICON"));
}

HBITMAP  WINAPI  TVVProductIconBmp()
{
	return LoadBitmap(hDllInstance, TVVersionInfo("PRODUCT_ICON"));
}

BOOL  WINAPI  AboutDlgProc(HWND hDlg, UINT message, UINT wParam,
						   LONG lParam)
{
	static char** arr;
	static char about[64];
	HDC hDC,hDC2;
	HBITMAP	hBmp,hBmp2;
	(void) lParam;
	
	switch (message)
	{
	case WM_INITDIALOG:
		{
			arr = (char**)lParam;
			//    strcpy(about,"About ");
			
			const char* charAbout = TVSGetValue(table,"ABOUT");
			if (charAbout == NULL)
				return FALSE;
			
			//strcpy (about, TVSGetValue(table,"ABOUT"));
			strcpy (about, charAbout);
			strcat(about," ");
			
			if (!strcmp(arr[0],"CentralConsole"))
				strcat(about, TVSGetValue(table,"WEBLOADNAME"));
			else
				if( !strcmp(arr[0],"WebFT") )
					strcat(about, TVSGetValue(table,"WEBFTNAME"));
				else
					strcat(about,arr[0]);
				strcat(about," ...");
				SendMessage(hDlg,WM_SETTEXT,0,(LPARAM)about);
				
				//     SetDlgItemText(hDlg,IDC_PRODUCT_NAME, TVSGetValue(table,"PRODUCT_NAME"));
				SetDlgItemText(hDlg,IDC_PRODUCT_NAME, TVSGetValue(table,"PRODUCT_NAME_DISPLAY"));
				
				if( !strcmp(arr[0],"WebFT") )
					SetDlgItemText(hDlg,IDC_VERSION_NUM, TVSGetValue(table,"WEBFT_PRODUCT_VERSION_NAME"));
				else
					SetDlgItemText(hDlg,IDC_VERSION_NUM, TVSGetValue(table,"WEBLOAD_PRODUCT_VERSION_NAME"));
				
				// for ASG: ///////////////////////////// (Gal 12.4.05)
				LONG  ret;
				HKEY  Key = NULL;
				char * Path = "SOFTWARE\\RadView\\";
				unsigned long length = 256 ;
				char  pLang[256];
				strcpy (pLang, "English");
				ret = RegOpenKeyEx(HKEY_LOCAL_MACHINE, Path, 0, KEY_READ,&Key);
				if ( ret == ERROR_SUCCESS )
				{
					ret = RegQueryValueEx(Key, "ResourceLang" ,NULL,NULL,(LPBYTE)pLang,&length);
					RegCloseKey(Key);
				}	
				
				if (strcmp(pLang, "en-AU") == 0) 
				{
					// change the names of the applications:
					if (strcmp(arr[0], "Agenda Authoring Tool") == 0)
						arr[0] = "ASG-IntelliTest Agenda Authoring Tool";
					else if (strcmp(arr[0], "CentralConsole") == 0)
						arr[0] = "ASG-IntelliTest Load";
					else if (arr[0] == "TestTalk")
						arr[0] = "TestTalk";
					else if (strcmp(arr[0], "WebFT") == 0)
						arr[0] = "ASG-IntelliTest Functional";
					else if (strcmp(arr[0], "Visual AAT") == 0)
						arr[0] = "ASG-IntelliTest Visual AAT";
				}
				else
				{
					if (!strcmp(arr[0],"CentralConsole"))
						arr[0] = (char *)(TVSGetValue(table,"WEBLOADNAME"));
					else if( !strcmp(arr[0],"WebFT") )
						arr[0] = (char *)(TVSGetValue(table,"WEBFTNAME"));
					//else 
					//SetDlgItemText(hDlg,IDC_APP_NAME,arr[0]);
				}
				/////////////////////////////////////////////////
				
				SetDlgItemText(hDlg,IDC_APP_NAME,arr[0]);
				SetDlgItemText(hDlg,IDC_COPYRIGHT_NETSCAPE,  TVSGetValue(table,"COPYRIGHT_SSL")); 
				SetDlgItemText(hDlg,IDC_COPYRIGHT_COMPUWARE,  TVSGetValue(table,"COPYRIGHT_COMPUWARE")); 
				SetDlgItemText(hDlg,IDC_COPYRIGHT_RADVIEW, TVSGetValue(table,"COPYRIGHT_ABOUT"));
				SetDlgItemText(hDlg,IDC_COPYRIGHT_PLATINUM, TVSGetValue(table,"COPYRIGHT_ABOUT_PLATINUM"));
				SetDlgItemText(hDlg,IDC_RIGHTS, TVSGetValue(table,"ALL_RIGHTS")); 
				SetDlgItemText(hDlg,IDOK, TVSGetValue(table,"OK")); 
				UpdateWindow(hDlg);
				return TRUE;
		}
	case WM_PAINT:
		hDC = GetDC(hDlg);
		hDC2 = CreateCompatibleDC(hDC);
		hBmp = LoadBitmap(hDllInstance,arr[1]);
		hBmp2 = (HBITMAP)SelectObject(hDC2, hBmp);
		BitBlt(hDC, 10, 10, 32, 32, hDC2, 0, 0, SRCCOPY);
		SelectObject(hDC2, hBmp2);
		DeleteDC(hDC2);
		DeleteObject(hBmp);
		ReleaseDC(hDlg, hDC);
		return FALSE;
		
	case WM_CLOSE:
		EndDialog(hDlg,1);
		return TRUE;
		
	case WM_COMMAND:
		switch (wParam){
		case IDOK:
			EndDialog(hDlg,1) ;
			break;
		}
		return TRUE;
  }
  return FALSE;
}


int  WINAPI  TVVAboutDlg(const char* appName, const char* appBmp, HWND parent)
{
	// for ASG: ///////////////////////////// (Gal 12.4.05)
	/*LONG  ret;
	HKEY  Key = NULL;
	char * Path = "SOFTWARE\\RadView\\";
	unsigned long length = 256 ;
	char  pLang[256];
	strcpy (pLang, "English");
	ret = RegOpenKeyEx(HKEY_LOCAL_MACHINE, Path, 0, KEY_READ,&Key);
	if ( ret == ERROR_SUCCESS )
	{
	ret = RegQueryValueEx(Key, "ResourceLang" ,NULL,NULL,(LPBYTE)pLang,&length);
	RegCloseKey(Key);
	}	
	
	 if (strcmp(pLang, "en-AU") == 0) 
	 {
	 // change the names of the applications:
	 if (strcmp(appName, "Agenda Authoring Tool") == 0)
	 appName = "ASG-IntelliTest Agenda Authoring Tool";
	 else if (strcmp(appName, "CentralConsole") == 0)
	 appName = "ASG-IntelliTest Load";
	 else if (appName == "TestTalk")
	 appName = "TestTalk";
	 else if (strcmp(appName, "WebFT") == 0)
	 appName = "ASG-IntelliTest Functional";
	 else if (strcmp(appName, "Visual AAT") == 0)
	 appName = "ASG-IntelliTest Visual AAT";
	 }
	 else
	 {
	 if (!strcmp(appName,"CentralConsole"))
	 appName = TVSGetValue(table,"WEBLOADNAME");
	 else if( !strcmp(appName,"WebFT") )
	 appName = TVSGetValue(table,"WEBFTNAME");
	 //else 
	 //SetDlgItemText(hDlg,IDC_APP_NAME,arr[0]);
}*/
	/////////////////////////////////////////////////
	
	char* dlgResource = "DIALOG";
	DLGPROC DlgProc = (DLGPROC)MakeProcInstance((FARPROC)AboutDlgProc,hDllInstance);
	char*  arr[2] ;
	arr[0] = (char*) appName;
	arr[1] = (char*) appBmp;
	if (!strcmp(appName,"Editor"))
		arr[1] = (char*)TVVersionInfo("PRODUCT_ICON");
	if (!strcmp(appName,"CentralConsole") && TVSGetValue(table,"COPYRIGHT_SSL"))
	{
		dlgResource = "DIALOG2";
	}
	return DialogBoxParam(hDllInstance,dlgResource,parent,
		DlgProc,(LPARAM) arr);
}


bool WINAPI FindFile( const char* findPath, const char* spec, bool lookInSubdirectories, CFindFileFunctor* pObj )
{
#ifdef WIN32
	_finddata_t _data;
	char subDirPath[MAX_PATH] = { '\0' };
	char fullSpec[MAX_PATH] = { '\0' };
	char printPath[MAX_PATH] = { '\0' };
	long hndl = -1;
	
	// look in current directory
	_snprintf( fullSpec, (MAX_PATH-1), "%s\\%s", findPath, spec );
	hndl = _findfirst( fullSpec , &_data );
	if( -1 != hndl ){
		while( true ){
			if( _A_SUBDIR != _data.attrib ){
				_snprintf( printPath, (MAX_PATH-1), "%s\\%s\n", findPath, _data.name );
				// handle found files here
				if( !pObj->handleFile( printPath ) ){
					_findclose( hndl );
					return false;
				}
			}
			
			if( -1 == _findnext( hndl, &_data ) )
				break;
		}
		
		_findclose( hndl );
	}
	
	// look in subdirectories
	if( !lookInSubdirectories )
		return true;
	
	_snprintf( fullSpec, (MAX_PATH-1), "%s\\%s", findPath, "*" );
	hndl = _findfirst( fullSpec , &_data );
	if( -1 == hndl )
		return true;
	
	while( true ){
		if( ( 0 != (_A_SUBDIR & _data.attrib) ) &&
			(0 != strcmp( ".", _data.name )) &&
			(0 != strcmp( "..", _data.name )) ){
			_snprintf( subDirPath, (MAX_PATH-1), "%s\\%s", findPath, _data.name );
			if( !FindFile( subDirPath, spec, lookInSubdirectories, pObj ) ){
				_findclose( hndl );
				return false;
			}
		}
		
		if( -1 == _findnext( hndl, &_data ) )
			break;
	}
	
	_findclose( hndl );
	
	return true;
#endif // WIN32
}


//////////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////////
TVEnvContainerClass::TVEnvContainerClass() {
	m_TVPResolvePathVarNoTVEnv_result = NULL;
	reset();
}

//////////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////////
TVEnvContainerClass::~TVEnvContainerClass() {
	if (m_TVPResolvePathVarNoTVEnv_result) {
		free(m_TVPResolvePathVarNoTVEnv_result);
		m_TVPResolvePathVarNoTVEnv_result = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////////
void TVEnvContainerClass::reset() {
	if (m_TVPResolvePathVarNoTVEnv_result) {
		free(m_TVPResolvePathVarNoTVEnv_result);
		m_TVPResolvePathVarNoTVEnv_result = NULL;
	}
	m_searchInPath_result[0] = '\0';
	m_TVPGetUserDir_result[0] = '\0';
	m_TVPGetCwd_buf[0] = '\0';
	m_TVPSplitPath_dirname[0] = '\0';
	m_TVPSplitPath_filename[0] = '\0';
	m_TVPSplitPath_extname[0] = '\0';
	m_TVPMakePath_pathname[0] = '\0';
	m_TVERuntimeResolve_retVal[0] = '\0';
	m_TVPGetTempName_name[0] = '\0';
	m_TVPGetMyDocuments_result[0] = '\0';
	
}