/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/* Copyright  1993-1995, Radview Software LTD. All Rights
   Reserved. */
#include <stdio.h>
#include <string.h>

#include <assert.h>
#include <ctype.h>
#include <stdlib.h>

#include "tvdebug.h"
#include "platform.h"
#include <thr.h>
#include "tvenv.h"

#include <sys/stat.h>
#include <fcntl.h>


#ifdef __TV_UNIX__
#include <dir.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#define WL_MAX_PATH 300
#endif
#ifdef __TV_WINDOWS__
#include <hashdict.h>
#include <collstr.h>
#include <lzexpand.h>
#include <direct.h>
#include <dos.h>
#include <shlobj.h>

#define MAX_DRIVE 64
#include <io.h>
#endif


#ifdef __TV_UNIX__
long filelength(int fd)
{
  long pos = lseek(fd, 0, SEEK_CUR);
  long end = lseek(fd, 0, SEEK_END);
  lseek(fd, pos, SEEK_SET);
  return end;
}
#endif

int WINAPI TVPChdir(const char* newDir)
{
#ifdef __TV_WINDOWS__
  char e[_MAX_EXT];
  char n[_MAX_FNAME];
  char d[_MAX_DRIVE];
  char dir[_MAX_DIR];
  _splitpath(newDir,d,dir,n,e);

  char cdrive = tolower(d[0]);
  if (cdrive >= 'a')
  {
    int drive = cdrive - 'a' + 1;
    _chdrive(drive);
  }

  TVEnvContainerClass tvpc;
  const char* temp = tvpc.TVPMakePath(dir,n,e);
  char newpath[MAX_PATH];
  _fullpath((char*)&newpath,temp,MAX_PATH);
  int status = chdir(newpath);
#endif
#ifdef __TV_UNIX__
  int status = chdir(newDir);
#endif

  return (status==0);
}

const char* WINAPI TVEnvContainerClass::TVPAdjustPathLen(const char* oldpath, char* newpath, int len)
{
  assert(len>0);
  if (strlen(oldpath)<(unsigned)len)
  {
    strcpy(newpath,oldpath);
    return newpath;
  }

  char *d,*f,*e;
  //
  // spliting the file-name to name and extension
  TVPSplitPath(oldpath,&d,&f,&e);
  int pathlen = len - strlen(f) - strlen(e) - 5;
  if (pathlen<0)
    return NULL;

  strncpy(newpath,oldpath,pathlen);
  newpath[pathlen] = '\0';
  strcat(newpath,"...");
  strcat(newpath,"\\");
  strcat(newpath,f);
  strcat(newpath,".");
  strcat(newpath,e);

  return newpath;
}

int WINAPI TVEnvContainerClass::TVPFileExist(const char* fileName)
{
  struct stat statbuf;

  if(stat(fileName, &statbuf) == 0)
    return 1;
  else
    return 0;
}

const char* TVEnvContainerClass::TVPPathFileExist(const char* dir, const char* fname)
{
  assert(fname);
  if (dir == NULL)
    return NULL;
  //
  // Build the full path name = path/filename
  const char *path = TVPConcatDirAndName(dir,fname);
  //
  // Check if the file fileName exists
  if (TVPFileExist(path))
    return path;
  else
    return NULL;
}

const char* TVEnvContainerClass::TVPResolvePathVarNoTVEnv(const char* pathvar)
{
  const char* path = NULL;

  assert (pathvar != NULL);
  if (strlen(pathvar)==0)
    return NULL;

  if (m_TVPResolvePathVarNoTVEnv_result)
  {
    free(m_TVPResolvePathVarNoTVEnv_result);
    m_TVPResolvePathVarNoTVEnv_result=NULL;
  }

  if (!strcmp(pathvar,"PATH")) {  /* Dos & Unix variable PATH */
    path = getenv("PATH");

  } else if (!strcmp(pathvar,"HOME")) {  /* Unix variable HOME */
    //
    // Check if the file fileName exists within user-dir
    path = TVPGetUserDir();

  } 
  else if (!strcmp(pathvar,"MYDOCUMENTS")) {  /* Windows variable My Documents */
      path = TVPGetMyDocuments();

  } 
  else if (*pathvar == '$') {    /* $env_var - Unix & Windows */
    const char* env = pathvar+1;
    path = getenv(env);

  } else {                       /* path to look in */
    //
    // Check if the file exists
    path = pathvar;
  }

  if (path && strlen(path)) {
    m_TVPResolvePathVarNoTVEnv_result = (char*)malloc(strlen(path)+1);
    strcpy (m_TVPResolvePathVarNoTVEnv_result,path);
    return m_TVPResolvePathVarNoTVEnv_result;
  } else
    return NULL;
}

const char* WINAPI TVEnvContainerClass::TVPResolveFilePathNoTVEnv(const char* fileName, const char* pathvar)
{
  char *s, *p;
  const char* path = NULL;

  assert(fileName);
  assert(pathvar);

  //
  // resolve the path-variable name.
  p = (char *)TVPResolvePathVarNoTVEnv(pathvar);
  if (p==NULL)
    return NULL;

  s = __STRTOK(p,PATH_SEP_STR);
  while(s) {
    if( s != NULL) {
      //
      // Check if s is a valid directory.
      if (TVPIsClosedFileADirectory(s)) {
        path = TVPConcatDirAndName(s,fileName);
        return path;
      }
    }
    s = __STRTOK(NULL,PATH_SEP_STR);
  }

  return NULL;
}

const char* TVEnvContainerClass::searchInPath(const char* fileName, const char* Path)
{
  char *s, *p;
  const char *fname = fileName;
  const char* path = NULL;

  if (!Path)
    return NULL;

  p = new char[strlen(Path)+1];
  strcpy(p,Path);

  s = __STRTOK(p,PATH_SEP_STR);
  while(s) {
    if( s != NULL) {
      //
      // Check if the file fileName exists
      if (TVPIsClosedFileADirectory(s)) {
        path = TVPPathFileExist(s,fname);
        if (path) {
          strcpy(m_searchInPath_result,path);
          break;
        }
      }
    }
    s = __STRTOK(NULL,PATH_SEP_STR);
  }
  delete [] p;
  if (path)
    return m_searchInPath_result;
  else
    return NULL;
}

const char* WINAPI TVEnvContainerClass::TVPSearchPath(const char* fileName)
{
  return searchInPath(fileName, getenv("PATH"));
}

const char* WINAPI TVEnvContainerClass::TVPGetUserDir()
{
#ifdef __TV_UNIX__
  const char* ud = getenv("HOME");
  if (ud) {
    strcpy(m_TVPGetUserDir_result,ud);
    return m_TVPGetUserDir_result;
  }
#endif
#ifdef __TV_WINDOWS__
  UINT i;
  i = GetWindowsDirectory(m_TVPGetUserDir_result,sizeof(m_TVPGetUserDir_result));
  if (i)
    return m_TVPGetUserDir_result;
#endif
  return NULL;
}


const char* WINAPI TVEnvContainerClass::TVPGetMyDocuments()
{
#ifdef __TV_UNIX__
  const char* ud = getenv("MYDOCUMENTS");
  if (ud) {
    strcpy(m_TVPGetMyDocuments_result,ud);
    return m_TVPGetMyDocuments_result;
  }
#endif
#ifdef __TV_WINDOWS__
// Get My documents dir from the registry
  LONG retCode;
  unsigned long len;
  HKEY hKeyPath = NULL;
  const char * sKeyPath = "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders";
  const char *  sKeyName = "Personal";
  
  retCode = RegOpenKeyEx(HKEY_CURRENT_USER, sKeyPath, 0, KEY_READ,&hKeyPath);
  if ( retCode == ERROR_SUCCESS )
  {
    len = MAX_PATH - 1 ;
    retCode = RegQueryValueEx(hKeyPath, sKeyName, NULL, NULL, (unsigned char*)m_TVPGetMyDocuments_result, &len);
    RegCloseKey(hKeyPath);
   if ( retCode == ERROR_SUCCESS )
      return m_TVPGetMyDocuments_result;
  
  }

#endif
  return NULL;
}


static RVMutexLock gIniFileMutex;
const char* WINAPI TVEnvContainerClass::TVPFindfileNoTVEnv(const char* fileName,const char* where,...)
{
  RVMutexLock::LockGuard guard(gIniFileMutex);
  va_list ap;
  int firstPath = 1;
  const char* path = NULL;

  va_start(ap,where);

  while (firstPath ||
         (((where = (const char*) va_arg(ap,const char*)) != NULL) &&
          (*where != '\0'))) {

    firstPath = 0;
    //
    // search for the file in the path of where.
    if ((path = searchInPath(fileName,TVPResolvePathVarNoTVEnv(where))) != NULL){
      return path;
	}

  } // the while loop
  va_end(ap);

  return NULL;
}

int WINAPI TVPIsOpenFileADirectory(FILE* yy)
{
  struct stat statbuf;
  if (fstat(fileno(yy), &statbuf) != -1) {
    if (statbuf.st_mode & S_IFDIR) {
      return 1;
    }
  }
  return 0;
}

int WINAPI TVPIsClosedFileADirectory(const char* fileName)
{
  if (strlen(fileName)==0)
    return FALSE;

  struct stat statbuf;
  if (stat(fileName, &statbuf) == 0)
    if ((statbuf.st_mode & S_IFDIR)!=0)
      return TRUE;

  return FALSE;
}


/********************** TVP GET CWD ***********************/
/* On success the function returns the full path name of  */
/* the current working directory.                         */
/* On failure, the function returns "".                   */
/**********************************************************/

const char* WINAPI TVEnvContainerClass::TVPGetCwd()
{
  if (getcwd(m_TVPGetCwd_buf,sizeof(m_TVPGetCwd_buf)) == NULL)
    return NULL;
  return m_TVPGetCwd_buf;
}


/******************* TVP GET PATH DIR *********************/
/* On success the function returns the given path's       */
/* directory.                                             */
/* On failure the function returns "".                    */
/**********************************************************/
const char* WINAPI TVEnvContainerClass::TVPGetPathDirectory(const char* path)
{
  char* dir = NULL;
  TVPSplitPath(path,&dir,NULL,NULL);
  return dir;
}

/****************** TVP GET PATH NAME *********************/
/* On success the function returns the given path's file. */
/* On failure the function returns "".                    */
/**********************************************************/

const char* WINAPI TVEnvContainerClass::TVPGetPathName(const char* path)
{
  char* name;
  TVPSplitPath(path,NULL,&name,NULL);
  return name;
}

/******************* TVP GET PATH EXT *********************/
/* On success the function returns the given path's       */
/* extention.                                             */
/* On failure the function returns "".                    */
/**********************************************************/
const char* WINAPI TVEnvContainerClass::TVPGetPathExt(const char* path)
{
  char* ext;
  TVPSplitPath(path,NULL,NULL,&ext);
  return ext;
}

void WINAPI TVPGetCurrentDir(char *path,int len)
{
  // dos,windows unix
  getcwd(path,len);
}

BOOL WINAPI TVPAbsPath(const char *path,char *newpath,int len)
{
#ifdef __TV_WINDOWS__
  // dos,windows
  if (!_fullpath(newpath,path, len))
    return FALSE;
#endif
#ifdef __TV_UNIX__
  (void) len;
  if (!realpath((char*) path,newpath))
    return FALSE;
#endif
  return TRUE;
}

BOOL WINAPI TVPCopyFile(const char *FromFileName, const char *ToFileName)
{
  //
  // request to copy file to itself is denied
  if (!strcmpi(FromFileName,ToFileName))
    return FALSE;

#ifdef __TV_WINDOWS__
  return CopyFile(FromFileName,ToFileName,FALSE);
#endif
  /*
#ifdef __TV_WINDOWS__
  HFILE hFrom,hTo;
  OFSTRUCT FromOpenBuff,ToOpenBuff;
  long status;

  hFrom = LZOpenFile((char*)FromFileName,&FromOpenBuff,OF_READ);
  // FromFileName doesn't exist - nothing to copy
  if (hFrom < 0)
    return FALSE;

  hTo = LZOpenFile((char*)ToFileName,&ToOpenBuff,OF_CREATE);
  if (hTo < 0)
    {
      //EDDisplayError(EDERR_CANNOT_COPY_FILE,EDINTERR_CANNOT_LZOPEN);
      LZClose(hFrom);
      return FALSE;
    }
  status = LZCopy(hFrom,hTo);
  if (status < 0)
    {
      //EDDisplayError(EDERR_CANNOT_COPY_FILE,EDINTERR_CANNOT_LZCOPY);
      LZClose(hFrom);
      LZClose(hTo);
      return FALSE;
    }
  LZClose(hFrom);
  LZClose(hTo);
  return TRUE;
#endif
  */

#ifdef __TV_UNIX__
  TVEnvContainerClass tvpc;
  char *cmd;
  int ret;
  //
  // Check if FromFileName exists
  if (tvpc.TVPFileExist(FromFileName)){
    //
    // Allocate a space for command diff buffer
    cmd = new char[strlen(FromFileName) + strlen(ToFileName) + 20];
    sprintf(cmd,"/bin/cp %s %s > /dev/null",FromFileName,ToFileName);
    ret = system(cmd); //execute the command "cp file1 file2"
    //
    // Free the command buffer
    delete [] cmd;
    return (ret==0);
  }
  return FALSE; //File FromFileName don't exist
#endif
}

void WINAPI TVEnvContainerClass::TVPSplitPath(const char* path, char** dir, char** fname, char** ext)
{
  m_TVPSplitPath_dirname[0]  = '\0';
  m_TVPSplitPath_filename[0] = '\0';
  m_TVPSplitPath_extname[0]  = '\0';

  assert(path);
  assert(strlen(path) <= MAX_PATH);


  if (TVPIsClosedFileADirectory(path)) {
    strncpy(m_TVPSplitPath_dirname,path,MAX_PATH);
  } else {
#ifdef __TV_WINDOWS__

    char e[_MAX_EXT];
    char drive[_MAX_DRIVE];
    char direct[_MAX_DIR];
    _splitpath(path,drive,direct,m_TVPSplitPath_filename,e);
    _makepath(m_TVPSplitPath_dirname,drive,direct,"","");
    if (e[0] == '.')
      strcpy(m_TVPSplitPath_extname,(char*)e+1);
    if ( strlen(m_TVPSplitPath_dirname) > 0 && m_TVPSplitPath_dirname[strlen(m_TVPSplitPath_dirname)-1]==FILE_SEP_CHAR)
      m_TVPSplitPath_dirname[strlen(m_TVPSplitPath_dirname)-1] = '\0';

#endif

#ifdef __TV_UNIX__
    char temp[MAX_PATH];
    char *s,*sw,*e;
    //
    // copy the given path to a temporary area.
    strncpy(temp,path,MAX_PATH);

    //
    // extract the directory name.
    s = strrchr(temp,FILE_SEP_CHAR);
    sw = strrchr(temp,'\\'); // indeed search for WINDOWS seperator char
    if (s && sw) {
       if ( sw > s )
         s = sw;
    }
    else if (sw) {
	s = sw;
    }

    if (s) {
      *s = '\0';
      s++;
      strcpy(m_TVPSplitPath_dirname,temp);
    } else {
      m_TVPSplitPath_dirname[0] = '\0';
      s = temp;
    }

    //
    // extract the filename and extention:
    e = strrchr(s,'.');
    if (e) {
      *e = '\0';
      e++;
      strcpy(m_TVPSplitPath_extname,e);
    } else {
      m_TVPSplitPath_extname[0] = '\0';
    }
    strcpy(m_TVPSplitPath_filename,s);
#endif
  }

  if (dir)
    *dir = m_TVPSplitPath_dirname;
  if (fname)
    *fname = m_TVPSplitPath_filename;
  if (ext)
    *ext = m_TVPSplitPath_extname;

}

const char* WINAPI TVEnvContainerClass::TVPMakePath(const char* dir, const char* fname, const char* ext)
{

  assert(dir);
  assert(fname);
  assert(ext);

#ifdef __TV_WINDOWS__

  _makepath(m_TVPMakePath_pathname,"",dir,fname,ext);

#endif

#ifdef __TV_UNIX__

  //
  // Build the full path name = path/filename
  int l;
  if ((l = strlen(dir)) > 0)
    {
      strcpy(m_TVPMakePath_pathname,dir);
      if (m_TVPMakePath_pathname[strlen(m_TVPMakePath_pathname)-1] != FILE_SEP_CHAR)
        strcat(m_TVPMakePath_pathname,FILE_SEP_STR);
    }

  strcat(m_TVPMakePath_pathname,fname);
  if ((l = strlen(ext)) > 0)
    {
      strcat(m_TVPMakePath_pathname,".");
      strcat(m_TVPMakePath_pathname,ext);
    }
#endif

  return m_TVPMakePath_pathname;
}

//
// this function gets a dir and filename, and concatenates them to one path.
const char* WINAPI TVEnvContainerClass::TVPConcatDirAndName(const char* dir, const char* fname)
{
  assert(dir != NULL);
  assert(fname != NULL);
  char *f,*e;

  //
  // spliting the file-name to name and extension
  TVPSplitPath(fname,NULL,&f,&e);
  //
  // now the TVPMakePath function will do the job.
  return TVPMakePath(dir,f,e);
}

//
// this function gets a dir and filename, and concatenates them to one path.
const char* WINAPI TVEnvContainerClass::TVPReplacePathExt(const char* path, const char* newext)
{
  assert(path != NULL);
  assert(newext != NULL);
  char *d,*f,*e;

  //
  // spliting the file-name to name and extension
  TVPSplitPath(path,&d,&f,&e);
  //
  // now the TVPMakePath function will do the job.
  return TVPMakePath(d,f,newext);
}




bool WINAPI TVPCreateDirectory(const char* path) {
#ifdef __TV_WINDOWS__
	if(!CreateDirectory(path,NULL)) {
		return false;
	}
#endif

#ifdef __TV_UNIX__
	if(mkdir(path,0777)) {//returms 0 on succes
		return false;
	}
#endif
	return true;
}

// ((((((((((((((((((    BE CAREFUL    ))))))))))))))))))))
//BE CAREFUL THIS FUNCTION REMOVE THE DIRECTORY WITH THE SUB-DIRECTORIES
// ((((((((((((((((((    BE CAREFUL    ))))))))))))))))))))
void WINAPI TVPRemoveAllDirectories(const char* a_path)
{
  TVEnvContainerClass tvpc;
	RVCString path(a_path);
#ifdef __TV_WINDOWS__
	RVCString targetDir(path.data());
	if( !path.contains("wlTemp") && !path.contains("applogs")) {
		assert(0);//trying to remove not an wl directory while calling the 
							//function.
	}
	
  WIN32_FIND_DATA FindFileData ; // data buffer
  HANDLE ret;
  RVCString tmpFile;
  RVCString targetFile = "*.*";
  RVCString targetPatern = tvpc.TVPConcatDirAndName (targetDir  , targetFile );
  ret = FindFirstFile(targetPatern , &FindFileData);

  if ( ret == INVALID_HANDLE_VALUE )
    return;

  RVHashTable delFileList;
  tmpFile = FindFileData.cFileName;
  if ((tmpFile != "." ) && ( tmpFile != ".." ))
  {
    RVCollectableString *pFileName = new RVCollectableString( tmpFile );
    delFileList.insert( pFileName );
  }

  while ( FindNextFile(ret, &FindFileData))
  {
    tmpFile = FindFileData.cFileName;
    if ((tmpFile != "." ) && ( tmpFile != ".." ))
    {
      RVCollectableString *pFileName = new RVCollectableString( tmpFile.data());
      delFileList.insert( pFileName );
    }
  }


  RVCollectableString *pNextFileName;
  RVHashTableIterator iter( delFileList );
  while ( (pNextFileName = (RVCollectableString *)iter() ) != NULL )
  {

    RVCString ext = tvpc.TVPGetPathExt( pNextFileName->data() );
    RVCString targetFile = tvpc.TVPConcatDirAndName (targetDir , pNextFileName->data() );
    int retDeleteFile = DeleteFile( targetFile );
		if(!retDeleteFile && (ext == "") ) {//it may be a directory
		TVPRemoveAllDirectories(targetFile);
		}
  }

  delFileList.clearAndDestroy();
	FindClose(ret);	
	if ( RemoveDirectory( targetDir.data() ) == 0 )
  {
    DWORD errDebug = GetLastError();
  }
#endif
#ifdef __TV_UNIX__
	RVCString targetDir(path.data());
	if( !path.contains("/tmp") ) {
		assert(0);//trying to remove not an wl directory while calling the 
							//function.
	}
	char command[WL_MAX_PATH+8];
	sprintf(command,"/bin/rm -fr %s",path.data());
	if(system (command) == -1) {
//		WlTraceMsg("Error 0 in function CArrangeMachineTempFiles::RemoveWlTempDirectory");
	}
#endif
	return;
}


const char* WINAPI TVPGetTempDirectory() {//the function return "" if temp dir is not exist.
	const char* tempDirectory = NULL;
#ifdef __TV_WINDOWS__
	tempDirectory = getenv("TEMP");
#endif
#ifdef __TV_UNIX__
		DIR* dir = opendir("/tmp");
		if(dir) {
			closedir(dir);
		tempDirectory = "/tmp"; 
		}
#endif
	if(! tempDirectory ) {//still Null
		tempDirectory = "";
	}
	return tempDirectory;
}



const char* TVEnvContainerClass::TVPResolvePathVar(const char* pathvar)
{
  const char* path = NULL;
  assert (pathvar != NULL);

  if (*pathvar == '#') {    /* #tvenv_var - Unix & Windows */
    const char* tvenv = pathvar+1;
    path = TVEGetEnv(tvenv);

  } else {
    //
    // resolve other options.
    path = TVPResolvePathVarNoTVEnv(pathvar);
  }

  return path;
}

const char* WINAPI TVEnvContainerClass::TVPResolveFilePath(const char* fileName, const char* pathvar)
{
  char *s, *p;
  const char* path = NULL;

  assert(fileName);
  assert(pathvar);

  //
  // resolve the path-variable name.
  p = (char *)TVPResolvePathVar(pathvar);
  if (p==NULL)
    return NULL;

  s = __STRTOK(p,PATH_SEP_STR);
  while(s) {
    if( s != NULL) {
      //
      // Check if s is a valid directory.
      if (TVPIsClosedFileADirectory(s)) {
        path = TVPConcatDirAndName(s,fileName);
        return path;
      }
    }
    s = __STRTOK(NULL,PATH_SEP_STR);
  }

  return NULL;
}

const char* WINAPI TVEnvContainerClass::TVPFindfile(const char* fileName,const char* where,...)
{
  RVMutexLock::LockGuard guard(gIniFileMutex);
  va_list ap;
  int firstPath = 1;
  const char* path = NULL;

  va_start(ap,where);

  while (firstPath ||
         (((where = (const char*) va_arg(ap,const char*)) != NULL) &&
          (*where != '\0'))) {

    firstPath = 0;

    //
    // search for the file in the path of where.
    if ((path = searchInPath(fileName,TVPResolvePathVar(where))) != NULL)
		{
				return path;
		}

  } // the while loop
  va_end(ap);

  return NULL;
}


const char* WINAPI TVEnvContainerClass::TVPGetFullName(const char* Old,char* New,int len)
{
  const char* newpath;
  const char* ret = NULL;


  const char* pathdir = TVPGetPathDirectory(Old);
  if (strlen(pathdir) > 0)
  {
    char temp[MAX_PATH];
    if (TVPAbsPath(pathdir,temp,MAX_PATH))
    {
      strcpy(New,TVPMakePath(temp,TVPGetPathName(Old),TVPGetPathExt(Old)));
      ret = (const char*)New;
    }
    else
      ret = NULL;
  }
  else if ((newpath = TVPFindfile(Old,".","#INCLUDE_PATH",NULL)) != NULL)
    if (strlen(newpath)<=len)
    {
      TVPAbsPath(newpath,New,len);
      ret = (const char*)New;
    }

  //sergei 3/24/01 - debugPrint(3,("TVPGetFullName(%s,%s,%d) returns %s\n",Old,New,len,ret));
  return ret;

  /*
  if (strlen(TVPGetPathDirectory(Old)) > 0)
    if (TVPAbsPath(Old,New,len))
      return (const char*)New;

  if ((newpath = TVPFindfile(Old,".","#INCLUDE_PATH",NULL)) != NULL)
    if (strlen(newpath)<=(unsigned)len)
      {
        strcpy(New,newpath);
        return (const char*)New;
      }

  return NULL;
  */
}


const char* WINAPI TVEnvContainerClass::TVPGetTempName(const char *pathvar)
{
  char *c;
  static char name[MAX_PATH];

  char* path = (char*)TVPResolvePathVar(pathvar);
  //
  // get the temp-name in the requested directory.
  // !! read the tempname documentation to check where the temp-name
  //    can be givven.
  c = tempnam(path,"tmp");
  if (c != NULL)
    {
      strcpy(name,c);
      free(c);
      c = name;
    }
#ifdef __TV_WINDOWS__
  int f = open(c,O_CREAT,_S_IREAD | _S_IWRITE);
  close(f);
#endif

  return c;
}
