/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/* Copyright  1993-1995, Radview Software LTD. All Rights Reserved. */
#include <stdio.h>
#include <string.h>

#include <assert.h>
#include <ctype.h>
#include <stdlib.h>

#include "tvdebug.h"
#include "platform.h"
#include "tvpfiles.h"

#include <sys/stat.h>


#ifdef __TV_UNIX__
#include <dir.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#define WL_MAX_PATH 300
#endif
#ifdef __TV_WINDOWS__
#include <hashdict.h>
#include <collstr.h>
#include <lzexpand.h>
#include <direct.h>
#include <dos.h>
#define MAX_DRIVE 64
#endif


#ifdef __TV_UNIX__
long filelength(int fd)
{
  long pos = lseek(fd, 0, SEEK_CUR);
  long end = lseek(fd, 0, SEEK_END);
  lseek(fd, pos, SEEK_SET);
  return end;
}
#endif

int WINAPI TVPChdir(const char* newDir)
{
#ifdef __TV_WINDOWS__
  char e[_MAX_EXT];
  char n[_MAX_FNAME];
  char d[_MAX_DRIVE];
  char dir[_MAX_DIR];
  _splitpath(newDir,d,dir,n,e);

  char cdrive = tolower(d[0]);
  if (cdrive >= 'a')
  {
    int drive = cdrive - 'a' + 1;
    _chdrive(drive);
  }

  const char* temp = TVPMakePath(dir,n,e);
  char newpath[MAX_PATH];
  _fullpath((char*)&newpath,temp,MAX_PATH);
  int status = chdir(newpath);
#endif
#ifdef __TV_UNIX__
  int status = chdir(newDir);
#endif

  return (status==0);
}

const char* WINAPI TVPAdjustPathLen(const char* oldpath, char* newpath, int len)
{
  assert(len>0);
  if (strlen(oldpath)<(unsigned)len)
  {
    strcpy(newpath,oldpath);
    return newpath;
  }

  char *d,*f,*e;
  //
  // spliting the file-name to name and extension
  TVPSplitPath(oldpath,&d,&f,&e);
  int pathlen = len - strlen(f) - strlen(e) - 5;
  if (pathlen<0)
    return NULL;

  strncpy(newpath,oldpath,pathlen);
  newpath[pathlen] = '\0';
  strcat(newpath,"...");
  strcat(newpath,"\\");
  strcat(newpath,f);
  strcat(newpath,".");
  strcat(newpath,e);

  return newpath;
}

int WINAPI TVPFileExist(const char* fileName)
{
  struct stat statbuf;

  if(stat(fileName, &statbuf) == 0)
    return 1;
  else
    return 0;
}

const char* TVPPathFileExist(const char* dir, const char* fname)
{
  assert(fname);
  if (dir == NULL)
    return NULL;
  //
  // Build the full path name = path/filename
  const char *path = TVPConcatDirAndName(dir,fname);
  //
  // Check if the file fileName exists
  if (TVPFileExist(path))
    return path;
  else
    return NULL;
}

const char* TVPResolvePathVarNoTVEnv(const char* pathvar)
{
  static char *result=NULL;
  const char* path = NULL;

  assert (pathvar != NULL);
  if (strlen(pathvar)==0)
    return NULL;

  if (result)
  {
    free(result);
    result=NULL;
  }

  if (!strcmp(pathvar,"PATH")) {  /* Dos & Unix variable PATH */
    path = getenv("PATH");

  } else if (!strcmp(pathvar,"HOME")) {  /* Unix variable HOME */
    //
    // Check if the file fileName exists within user-dir
    path = TVPGetUserDir();

  } else if (*pathvar == '$') {    /* $env_var - Unix & Windows */
    const char* env = pathvar+1;
    path = getenv(env);

  } else {                       /* path to look in */
    //
    // Check if the file exists
    path = pathvar;
  }

  if (path && strlen(path)) {
    result = (char*)malloc(strlen(path)+1);
    strcpy (result,path);
    return result;
  } else
    return NULL;
}

const char* WINAPI TVPResolveFilePathNoTVEnv(const char* fileName, const char* pathvar)
{
  char *s, *p;
  const char* path = NULL;

  assert(fileName);
  assert(pathvar);

  //
  // resolve the path-variable name.
  p = (char *)TVPResolvePathVarNoTVEnv(pathvar);
  if (p==NULL)
    return NULL;

  s = __STRTOK(p,PATH_SEP_STR);
  while(s) {
    if( s != NULL) {
      //
      // Check if s is a valid directory.
      if (TVPIsClosedFileADirectory(s)) {
        path = TVPConcatDirAndName(s,fileName);
        return path;
      }
    }
    s = __STRTOK(NULL,PATH_SEP_STR);
  }

  return NULL;
}

const char* searchInPath(const char* fileName, const char* Path)
{
  static char result[MAX_PATH];
  char *s, *p;
  const char *fname = fileName;
  const char* path = NULL;

  if (!Path)
    return NULL;

  p = new char[strlen(Path)+1];
  strcpy(p,Path);

  s = __STRTOK(p,PATH_SEP_STR);
  while(s) {
    if( s != NULL) {
      //
      // Check if the file fileName exists
      if (TVPIsClosedFileADirectory(s)) {
        path = TVPPathFileExist(s,fname);
        if (path) {
          strcpy(result,path);
          break;
        }
      }
    }
    s = __STRTOK(NULL,PATH_SEP_STR);
  }
  delete [] p;
  if (path)
    return result;
  else
    return NULL;
}

const char* WINAPI TVPSearchPath(const char* fileName)
{
  return searchInPath(fileName, getenv("PATH"));
}

const char* WINAPI TVPGetUserDir()
{
  static char result[MAX_PATH];
#ifdef __TV_UNIX__
  const char* ud = getenv("HOME");
  if (ud) {
    strcpy(result,ud);
    return result;
  }
  else
    return NULL;
#endif
#ifdef __TV_WINDOWS__
      UINT i;
      i = GetWindowsDirectory(result,MAX_PATH);
      if (i)
        return result;
      else
        return NULL;
#endif
}

const char* WINAPI TVPFindfileNoTVEnv(const char* fileName,const char* where,...)
{
  va_list ap;
  int firstPath = 1;
  const char* path = NULL;

  va_start(ap,where);

  while (firstPath ||
         (((where = (const char*) va_arg(ap,const char*)) != NULL) &&
          (*where != '\0'))) {

    firstPath = 0;
    //
    // search for the file in the path of where.
    if ((path = searchInPath(fileName,TVPResolvePathVarNoTVEnv(where))) != NULL)
      return path;

  } // the while loop
  va_end(ap);

  return NULL;
}

int WINAPI TVPIsOpenFileADirectory(FILE* yy)
{
  struct stat statbuf;
  if (fstat(fileno(yy), &statbuf) != -1) {
    if (statbuf.st_mode & S_IFDIR) {
      return 1;
    }
  }
  return 0;
}

int WINAPI TVPIsClosedFileADirectory(const char* fileName)
{
  if (strlen(fileName)==0)
    return FALSE;

  struct stat statbuf;
  if (stat(fileName, &statbuf) == 0)
    if ((statbuf.st_mode & S_IFDIR)!=0)
      return TRUE;

  return FALSE;
}


/********************** TVP GET CWD ***********************/
/* On success the function returns the full path name of  */
/* the current working directory.                         */
/* On failure, the function returns "".                   */
/**********************************************************/

const char* WINAPI TVPGetCwd()
{
  static char buf[MAX_PATH];

  if (getcwd(buf,MAX_PATH) == NULL)
    return NULL;
  return buf;
}


/******************* TVP GET PATH DIR *********************/
/* On success the function returns the given path's       */
/* directory.                                             */
/* On failure the function returns "".                    */
/**********************************************************/
const char* WINAPI TVPGetPathDirectory(const char* path)
{
  char* dir;
  TVPSplitPath(path,&dir,NULL,NULL);
  return dir;
}

/****************** TVP GET PATH NAME *********************/
/* On success the function returns the given path's file. */
/* On failure the function returns "".                    */
/**********************************************************/

const char* WINAPI TVPGetPathName(const char* path)
{
  char* name;
  TVPSplitPath(path,NULL,&name,NULL);
  return name;
}

/******************* TVP GET PATH EXT *********************/
/* On success the function returns the given path's       */
/* extention.                                             */
/* On failure the function returns "".                    */
/**********************************************************/
const char* WINAPI TVPGetPathExt(const char* path)
{
  char* ext;
  TVPSplitPath(path,NULL,NULL,&ext);
  return ext;
}

void WINAPI TVPGetCurrentDir(char *path,int len)
{
  // dos,windows unix
  getcwd(path,len);
}

BOOL WINAPI TVPAbsPath(const char *path,char *newpath,int len)
{
#ifdef __TV_WINDOWS__
  // dos,windows
  if (!_fullpath(newpath,path, len))
    return FALSE;
#endif
#ifdef __TV_UNIX__
  (void) len;
  if (!realpath((char*) path,newpath))
    return FALSE;
#endif
  return TRUE;
}

BOOL WINAPI TVPCopyFile(const char *FromFileName, const char *ToFileName)
{
  //
  // request to copy file to itself is denied
  if (!strcmpi(FromFileName,ToFileName))
    return FALSE;

#ifdef __TV_WINDOWS__
  return CopyFile(FromFileName,ToFileName,FALSE);
#endif
  /*
#ifdef __TV_WINDOWS__
  HFILE hFrom,hTo;
  OFSTRUCT FromOpenBuff,ToOpenBuff;
  long status;

  hFrom = LZOpenFile((char*)FromFileName,&FromOpenBuff,OF_READ);
  // FromFileName doesn't exist - nothing to copy
  if (hFrom < 0)
    return FALSE;

  hTo = LZOpenFile((char*)ToFileName,&ToOpenBuff,OF_CREATE);
  if (hTo < 0)
    {
      //EDDisplayError(EDERR_CANNOT_COPY_FILE,EDINTERR_CANNOT_LZOPEN);
      LZClose(hFrom);
      return FALSE;
    }
  status = LZCopy(hFrom,hTo);
  if (status < 0)
    {
      //EDDisplayError(EDERR_CANNOT_COPY_FILE,EDINTERR_CANNOT_LZCOPY);
      LZClose(hFrom);
      LZClose(hTo);
      return FALSE;
    }
  LZClose(hFrom);
  LZClose(hTo);
  return TRUE;
#endif
  */

#ifdef __TV_UNIX__
  char *cmd;
  int ret;
  //
  // Check if FromFileName exists
  if (TVPFileExist(FromFileName)){
    //
    // Allocate a space for command diff buffer
    cmd = new char[strlen(FromFileName) + strlen(ToFileName) + 20];
    sprintf(cmd,"/bin/cp %s %s > /dev/null",FromFileName,ToFileName);
    ret = system(cmd); //execute the command "cp file1 file2"
    //
    // Free the command buffer
    delete [] cmd;
    return (ret==0);
  }
  return FALSE; //File FromFileName don't exist
#endif
}

void WINAPI TVPSplitPath(const char* path, char** dir, char** fname, char** ext)
{
  static char dirname[MAX_PATH];
  static char filename[MAX_PATH];
  static char extname[MAX_PATH];

  dirname[0]  = '\0';
  filename[0] = '\0';
  extname[0]  = '\0';

  assert(path);
  assert(strlen(path) <= MAX_PATH);


  if (TVPIsClosedFileADirectory(path)) {
    strncpy(dirname,path,MAX_PATH);
    filename[0] = '\0';
    extname[0] = '\0';
  } else {
#ifdef __TV_WINDOWS__

    char e[_MAX_EXT];
    char drive[_MAX_DRIVE];
    char direct[_MAX_DIR];
    _splitpath(path,drive,direct,filename,e);
    _makepath(dirname,drive,direct,"","");
    if (e[0] == '.')
      strcpy(extname,(char*)e+1);
    if (dirname[strlen(dirname)-1]==FILE_SEP_CHAR)
      dirname[strlen(dirname)-1] = '\0';

#endif

#ifdef __TV_UNIX__
    char temp[MAX_PATH];
    char *s,*sw,*e;
    //
    // copy the given path to a temporary area.
    strncpy(temp,path,MAX_PATH);

    //
    // extract the directory name.
    s = strrchr(temp,FILE_SEP_CHAR);
    sw = strrchr(temp,'\\'); // indeed search for WINDOWS seperator char
    if (s && sw) {
       if ( sw > s )
         s = sw;
    }
    else if (sw) {
	s = sw;
    }

    if (s) {
      *s = '\0';
      s++;
      strcpy(dirname,temp);
    } else {
      dirname[0] = '\0';
      s = temp;
    }

    //
    // extract the filename and extention:
    e = strrchr(s,'.');
    if (e) {
      *e = '\0';
      e++;
      strcpy(extname,e);
    } else {
      extname[0] = '\0';
    }
    strcpy(filename,s);
#endif
  }

  if (dir)
    *dir = dirname;
  if (fname)
    *fname = filename;
  if (ext)
    *ext = extname;

}

const char* WINAPI TVPMakePath(const char* dir, const char* fname, const char* ext)
{
  static char pathname[MAX_PATH];

  assert(dir);
  assert(fname);
  assert(ext);

#ifdef __TV_WINDOWS__

  _makepath(pathname,"",dir,fname,ext);

#endif

#ifdef __TV_UNIX__

  //
  // Build the full path name = path/filename
  int l;
  if ((l = strlen(dir)) > 0)
    {
      strcpy(pathname,dir);
      if (pathname[strlen(pathname)-1] != FILE_SEP_CHAR)
        strcat(pathname,FILE_SEP_STR);
    }

  strcat(pathname,fname);
  if ((l = strlen(ext)) > 0)
    {
      strcat(pathname,".");
      strcat(pathname,ext);
    }
#endif

  return pathname;
}

//
// this function gets a dir and filename, and concatenates them to one path.
const char* WINAPI TVPConcatDirAndName(const char* dir, const char* fname)
{
  assert(dir != NULL);
  assert(fname != NULL);
  char *f,*e;

  //
  // spliting the file-name to name and extension
  TVPSplitPath(fname,NULL,&f,&e);
  //
  // now the TVPMakePath function will do the job.
  return TVPMakePath(dir,f,e);
}

//
// this function gets a dir and filename, and concatenates them to one path.
const char* WINAPI TVPReplacePathExt(const char* path, const char* newext)
{
  assert(path != NULL);
  assert(newext != NULL);
  char *d,*f,*e;

  //
  // spliting the file-name to name and extension
  TVPSplitPath(path,&d,&f,&e);
  //
  // now the TVPMakePath function will do the job.
  return TVPMakePath(d,f,newext);
}




bool WINAPI TVPCreateDirectory(const char* path) {
#ifdef __TV_WINDOWS__
	if(!CreateDirectory(path,NULL)) {
		return false;
	}
#endif

#ifdef __TV_UNIX__
	if(mkdir(path,0777)) {//returms 0 on succes
		return false;
	}
#endif
	return true;
}

// ((((((((((((((((((    BE CAREFUL    ))))))))))))))))))))
//BE CAREFUL THIS FUNCTION REMOVE THE DIRECTORY WITH THE SUB-DIRECTORIES
// ((((((((((((((((((    BE CAREFUL    ))))))))))))))))))))
void WINAPI TVPRemoveAllDirectories(const char* a_path) {
	RVCString path(a_path);
#ifdef __TV_WINDOWS__
	RVCString targetDir(path.data());
	if( !path.contains("wlTemp") ) {
		assert(0);//trying to remove not an wl directory while calling the 
							//function.
	}
	
  WIN32_FIND_DATA FindFileData ; // data buffer
  HANDLE ret;
  RVCString tmpFile;
  RVCString targetFile = "*.*";
  RVCString targetPatern = TVPConcatDirAndName (targetDir  , targetFile );
  ret = FindFirstFile(targetPatern , &FindFileData);

  if ( ret == INVALID_HANDLE_VALUE )
    return;

  RVHashTable delFileList;
  tmpFile = FindFileData.cFileName;
  if ((tmpFile != "." ) && ( tmpFile != ".." ))
  {
    RVCollectableString *pFileName = new RVCollectableString( tmpFile );
    delFileList.insert( pFileName );
  }

  while ( FindNextFile(ret, &FindFileData))
  {
    tmpFile = FindFileData.cFileName;
    if ((tmpFile != "." ) && ( tmpFile != ".." ))
    {
      RVCollectableString *pFileName = new RVCollectableString( tmpFile.data());
      delFileList.insert( pFileName );
    }
  }


  RVCollectableString *pNextFileName;
  RVHashTableIterator iter( delFileList );
  while ( (pNextFileName = (RVCollectableString *)iter() ) != NULL )
  {

    RVCString ext = TVPGetPathExt( pNextFileName->data() );
    RVCString targetFile = TVPConcatDirAndName (targetDir , pNextFileName->data() );
    int retDeleteFile = DeleteFile( targetFile );
		if(!retDeleteFile && (ext == "") ) {//it may be a directory
		TVPRemoveAllDirectories(targetFile);
		}
  }

  delFileList.clearAndDestroy();
	FindClose(ret);	
	if ( RemoveDirectory( targetDir.data() ) == 0 )
  {
    DWORD errDebug = GetLastError();
  }
#endif
#ifdef __TV_UNIX__
	RVCString targetDir(path.data());
	if( !path.contains("/tmp") ) {
		assert(0);//trying to remove not an wl directory while calling the 
							//function.
	}
	char command[WL_MAX_PATH+8];
	sprintf(command,"/bin/rm -fr %s",path.data());
	if(system (command) == -1) {
//		WlTraceMsg("Error 0 in function CArrangeMachineTempFiles::RemoveWlTempDirectory");
	}
#endif
	return;
}


const char* WINAPI TVPGetTempDirectory() {//the function return "" if temp dir is not exist.
	const char* tempDirectory = NULL;
#ifdef __TV_WINDOWS__
	tempDirectory = getenv("TEMP");
#endif
#ifdef __TV_UNIX__
		DIR* dir = opendir("/tmp");
		if(dir) {
			closedir(dir);
		tempDirectory = "/tmp"; 
		}
#endif
	if(! tempDirectory ) {//still Null
		tempDirectory = "";
	}
	return tempDirectory;
}

