/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "tvpsync.h"

#ifdef __GNUG__
#if (__GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 91))
#define UseSignalCall
#endif
#endif // __GNUG__

#ifdef __s390__
#ifndef UseSignalCall
#define UseSignalCall
#endif // UseSignalCall
#endif // __s390__


//////////////////////// TVPSysThread implementation. /////////////////////////

TVPSysThread::TVPSysThread()
{
  m_Thread = new RVThread;
}

TVPSysThread::~TVPSysThread()
{
  try {
    //delete m_Thread;
  } catch (...) {}
}

#define THREAD_STACK 16000
int TVPSysThread::create(ThrFunc *start_addr, void *arg, int suspend_flag)
{
  // create a thread with a global function that takes 1 argument
  *m_Thread = rvMakeThreadFunctionGA1( unsigned,     // return type
                                       start_addr,   // function
                                       void*,        // argument type
                                       arg           // argument
                                      );
  //m_Thread
  m_Thread->start();

//  unsigned thr_id;
//  int init_state = (suspend_flag) ? CREATE_SUSPENDED : 0;

//  thr = (HANDLE) _beginthreadex(NULL, THREAD_STACK, start_addr, arg,
//        init_state, &thr_id);
  return 0;
}


BOOL TVPSysThread::setPriority(int hPriority)
{
	return TRUE;//SetThreadPriority(thr, hPriority);
}

bool TVPSysThread::setPriority(PriorityLevel priorityLevel)
{
#ifdef __TV_WINDOWS__
  HANDLE hThread = GetCurrentThread();
  switch (priorityLevel)
  {
    case Minimal:
      SetThreadPriority(hThread, THREAD_PRIORITY_IDLE);
      break;
    case Low:
      SetThreadPriority(hThread, THREAD_PRIORITY_LOWEST);
      break;
    case BelowNormal:
      SetThreadPriority(hThread, THREAD_PRIORITY_BELOW_NORMAL);
      break;
    case Normal:
      SetThreadPriority(hThread, THREAD_PRIORITY_NORMAL);
      break;
    case AboveNormal:
      SetThreadPriority(hThread, THREAD_PRIORITY_ABOVE_NORMAL);
      break;
    case High:
      SetThreadPriority(hThread, THREAD_PRIORITY_HIGHEST);
      break;
    case Critical:
      //SetThreadPriority(hThread, THREAD_PRIORITY_HIGHEST);
      SetThreadPriority(hThread, THREAD_PRIORITY_TIME_CRITICAL);
      break;
  }
#endif
  return true;
}

/*
int TVPSysThread::suspend()
{
  return TRUE;//SuspendThread(thr);
}

int TVPSysThread::resume()
{
  return TRUE;//ResumeThread(thr);
}

void TVPSysThread::end(unsigned retval)
{
  //_endthreadex(retval);
}

unsigned long TVPSysThread::getId()
{
  return (unsigned long) this;
}
*/


//////////////////////// TVPMutex implementation. /////////////////////////

TVPMutex::TVPMutex()
{
  mutex = new RVCriticalSection;
}


TVPMutex::~TVPMutex()
{
  delete mutex;
}


int TVPMutex::Lock()
{
  mutex->acquire();
  return TRUE;
}

int TVPMutex::Unlock()
{
  mutex->release();
  return TRUE;
}


//////////////////////// TVPSafeFlag implementation. /////////////////////////

TVPSafeFlag::TVPSafeFlag(bool value)
{
  m_Mutex = new RVCriticalSection;
  m_FlagValue = value;
}

TVPSafeFlag::~TVPSafeFlag()
{
  delete m_Mutex;
}

bool TVPSafeFlag::Set(bool newVal)
{
  RVCriticalSection::LockGuard lock(*m_Mutex);
  bool oldVal = m_FlagValue;
  m_FlagValue = newVal;
  return oldVal;
}

bool TVPSafeFlag::Get()
{
  RVCriticalSection::LockGuard lock(*m_Mutex);
  return m_FlagValue;
}




//////////////////////// TVPSafeCounter implementation. /////////////////////////

TVPSafeCounter::TVPSafeCounter(long initValue)
{
  m_Mutex = new RVCriticalSection;
  m_Counter = initValue;
}


TVPSafeCounter::~TVPSafeCounter()
{
  delete m_Mutex;
}

long TVPSafeCounter::Increment()
{
  RVCriticalSection::LockGuard lock(*m_Mutex);
  m_Counter++;
  return m_Counter;
}


long TVPSafeCounter::Decrement()
{
  RVCriticalSection::LockGuard lock(*m_Mutex);
  m_Counter--;
  return m_Counter;
}

long TVPSafeCounter::Get()
{
  RVCriticalSection::LockGuard lock(*m_Mutex);
  return m_Counter;
}

long TVPSafeCounter::Reset()
{
  RVCriticalSection::LockGuard lock(*m_Mutex);
  m_Counter=0;
  return m_Counter;
}

TVPSafeCounter::IncrementGuard::IncrementGuard(TVPSafeCounter& sc) :
  m_MySC(sc)
{
  m_MySC.Increment();
}

TVPSafeCounter::IncrementGuard::~IncrementGuard()
{
  m_MySC.Decrement();
}


//////////////////////// TVPManualEvent implementation. /////////////////////////

TVPManualEvent::TVPManualEvent()
{
  m_Mutex = new RVMutexLock;
  m_Condition  = new RVCondition(*m_Mutex);
  m_Flag = false;
}

TVPManualEvent::~TVPManualEvent()
{
  delete m_Mutex;
  delete m_Condition;
}

RVWaitStatus TVPManualEvent::Wait()
{
  RVMutexLock::LockGuard lock(*m_Mutex);
  while (!m_Flag)
    m_Condition->wait();

  return RV_THR_COMPLETED;
}

RVWaitStatus TVPManualEvent::Wait(long waitTime)
{
  RVMutexLock::LockGuard lock(*m_Mutex);
  while (!m_Flag) {
    RVWaitStatus status = m_Condition->wait(waitTime);
    if (status == RV_THR_TIMEOUT)
      return RV_THR_TIMEOUT;
  }

  return RV_THR_COMPLETED;
}

bool TVPManualEvent::Set()
{
  RVMutexLock::LockGuard lock(*m_Mutex);
  m_Flag = true;
  m_Condition->signalAll();
  return true;
}

bool TVPManualEvent::Reset()
{
  RVMutexLock::LockGuard lock(*m_Mutex);
  m_Flag = false;
  return true;
}


//////////////////////// TVPManualEvent::SetGuard implementation. /////////////////////////

TVPManualEvent::SetGuard::SetGuard(TVPManualEvent& me) :
  m_MyME(me)
{
  m_MyME.Reset();
}

TVPManualEvent::SetGuard::~SetGuard()
{
  m_MyME.Set();
}



#ifdef __TV_WINDOWS__

/*
TVPManualEvent::TVPManualEvent()
{
  event = CreateEvent(NULL, TRUE, FALSE, NULL);
  m_Flag = false;
}


TVPManualEvent::~TVPManualEvent()
{
  CloseHandle(event);
}

TVPManualEvent::WaitType TVPManualEvent::Wait(long waitTime)
{
  DWORD wait = waitTime;
  DWORD ret = WaitForSingleObject(event, wait);

  switch (ret) {
    case WAIT_OBJECT_0: return Success;
    case WAIT_TIMEOUT: return TimedOut;
    case WAIT_FAILED: return Failed;
    case WAIT_ABANDONED: return Success;
    default:
      return Failed;
  }
}


bool TVPManualEvent::Set()
{
  m_Flag = true;
  if (SetEvent(event)!=0)
    return true;
  else
    return false;
}


bool TVPManualEvent::Reset()
{
  m_Flag = false;
  if (ResetEvent(event)!=0)
    return true;
  else
    return false;
}


bool TVPManualEvent::Pulse()
{
  if (PulseEvent(event)!=0)
    return true;
  else
    return false;
}
*/

TVPTimerObject::TVPTimerObject()
{
  ftime(&mytime);
}


void TVPTimerObject::reset()
{
  ftime(&mytime);
}


double TVPTimerObject::get()
{
  struct timeb t;

  ftime (&t);
  double ret = (t.time - mytime.time) +
               (double) ((t.millitm - mytime.millitm) / 1000.0);
               return ret;
}


#else // of ifdef __TV_WINDOWS__



TVPTimerObject::TVPTimerObject()
{
  gettimeofday(&mytimeval, NULL);
}


void TVPTimerObject::reset()
{
  gettimeofday(&mytimeval, NULL);
}


double TVPTimerObject::get()
{
  struct timeval timeval_b;

  gettimeofday(&timeval_b, NULL);
  double ret = (timeval_b.tv_sec - mytimeval.tv_sec) +
               (double) (timeval_b.tv_usec - mytimeval.tv_usec) / 1000000.0;
  return ret;
}
#endif // of ifdef __TV_WINDOWS__

#ifdef __TV_UNIX__
TVPSignalHandler::TVPSignalHandler()
{
  int sig;
  for(sig = 1 ; sig <= MAX_SIG ; sig++)
    mask[sig-1] = SIG_DEFAULT;
}

TVPSignalHandler::TVPSignalHandler(const char *disp)
{
  int sig;
  for(sig = 0 ; sig < MAX_SIG ; sig++)
    if( sig < strlen(disp) )  // use info from the mask
      switch (*(disp+sig))
      {
        case 'i':   // IGNORE
        case 'I':
#ifdef UseSignalCall
          (void)signal(sig+1,SIG_IGN);
#else
          sigignore(sig+1);
#endif //UseSignalCall
          mask[sig] = SIG_IGNORE;
          break;

        case 'd':   // DEFAULT
        case 'D':
          break;

        case 'r':   // REPORT
        case 'R':
          signal(sig+1,reportSignal);
          mask[sig] = SIG_REPORT;
          break;
      }
    else
      mask[sig] = SIG_DEFAULT;
}

TVPSignalHandler::~TVPSignalHandler()
{
  int sig;
  for(sig = 1 ; sig <= MAX_SIG ; sig++) {
#ifdef UseSignalCall
    (void)signal(sig,SIG_DFL);
#else
    sigset(sig,SIG_DFL);
#endif //UseSignalCall
  }

}


void TVPSignalHandler::TVPSigSet(int sig,  void (*disp)(int))
{
#ifdef UseSignalCall
    (void)signal(sig,disp);
#else
  sigset(sig,disp);
#endif //UseSignalCall
  mask[sig] = SIG_HANDLER;
}

void TVPSignalHandler::TVPSigReset(int sig)
{
#ifdef UseSignalCall
    (void)signal(sig,SIG_DFL);
#else
  sigset(sig,SIG_DFL);
#endif //UseSignalCall
  mask[sig] = SIG_DEFAULT;
}

void reportSignal(int sig)
{
  fprintf(stderr,"Recievied signal: %d\n",sig);
}

#endif // __TV_UNIX__

