/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// WLLightString.cpp: implementation of the CWLLightString class.
//
//////////////////////////////////////////////////////////////////////

#include "wllightstring.h"
#ifdef __TV_UNIX__
#include <ctype.h>
#endif

//////////////////////////////////////////////////////////////////////
// CWLLightString Functions
//////////////////////////////////////////////////////////////////////

CWLLightString::CWLLightString() :
  m_theString(NULL)
{
  setValue(NULL, false);
}

CWLLightString::CWLLightString(const char* theString, bool own) :
  m_theString(NULL)
{
  setValue(theString, own);
}

CWLLightString::~CWLLightString()
{
  if (m_theString && m_ownString ) {
    delete[] m_theString;
  }
}

const char* CWLLightString::data()
{
  return m_theString;
}

unsigned CWLLightString::length()
{
  return m_length;
}

void CWLLightString::setValue(const char* newValue, bool own)
{
  if (m_theString && m_ownString )
    delete[] m_theString;

  if (newValue) {
    m_ownString = own;
    if (m_ownString) {
      m_theString = new char[strlen(newValue)+1];
      strcpy(m_theString, newValue);
    } else {
      m_theString = (char*)newValue;
    }
    m_length = strlen(m_theString);
  } else {
    m_theString = "";
    m_length = 0;
    m_ownString = false;
  }
}

//////////////////////////////////////////////////////////////////////
// CWLCollectableString Functions
//////////////////////////////////////////////////////////////////////

RVDEFINE_COLLECTABLE(CWLCollectableString, 0x0001)

inline static void mash(unsigned& hash, unsigned chars)
{
  hash = (chars ^
       ((hash << 5) |
        (hash >> (8*sizeof(unsigned) - 5))));
}

CWLCollectableString::CWLCollectableString() :
  m_IgnoreCase(false)
{
}

CWLCollectableString::CWLCollectableString(const char* theString, bool own) :
  CWLLightString(theString, own), m_IgnoreCase(false)
{
}

CWLCollectableString::~CWLCollectableString()
{
}

int CWLCollectableString::compareTo(const RVCollectable* c) const
{
  CWLCollectableString *cs = (CWLCollectableString*)c;
  if (m_IgnoreCase) {
    return strcmp(m_theString, cs->data());
  } else {
    return stricmp(m_theString, cs->data());
  }
}

unsigned CWLCollectableString::hash() const
{
#ifdef __TV_WINDOWS__
  if (m_IgnoreCase) {
    unsigned hv = (unsigned)m_length;    // Mix in the string length.
    unsigned i  = hv;
    const unsigned char* p = (const unsigned char*) m_theString;
    while (i--) {
      mash(hv, toupper(*p));
      ++p;
    }
    return hv;
  } else {
    unsigned hv       = (unsigned)m_length; // Mix in the string length.
    unsigned i        = m_length/sizeof(unsigned);
    const unsigned* p = (const unsigned*)m_theString;

    while (i--)
      mash(hv, *p++);                     // XOR in the characters.

    // XOR in any remaining characters:
    if ((i = m_length%sizeof(unsigned)) != 0)
    {
      register const char* c = (const char*)p;
      while (i--)
        mash(hv, (unsigned) *c++);
    }
    return hv;
  }
#else
  unsigned hv = (unsigned)m_length;    // Mix in the string length.
  unsigned i  = hv;
  const unsigned char* p = (const unsigned char*) m_theString;
  unsigned int u;
  while (i--) {
    u = *p;
    if (m_IgnoreCase) {
      u = toupper(*p);
    }
    mash(hv, u);
    ++p;
  }
  return hv;
#endif // __TV_WINDOWS__
}

bool CWLCollectableString::isEqual(const RVCollectable* c) const
{
  if( c->isA() != CWLCollectableString::isA() ) return FALSE;
  return (compareTo(c) == 0);
}


void CWLCollectableString::SetIgnoreCase(bool ignore)
{
  m_IgnoreCase = ignore;
}

bool CWLCollectableString::GetIgnoreCase()
{
  return m_IgnoreCase;
}

