/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <assert.h>
#include <stdlib.h>

#include <string.h>
#include <collstr.h>
#include <hashtab.h>

#include "tvdebug.h"

#include "tvtables.h"
#include "platform.h"
#include "strform.h"
#include "severr.h"
#include "lsevenv.h"
TableHandle counter = 1;

#ifdef __TV_linux__
#define HACK_CLASS_OP(a) a->
#else
#define HACK_CLASS_OP(a) a.
#endif

#ifdef __TV_linux__
RVHashTable *tablesContainer = new RVHashTable(31);
#else
RVHashTable tablesContainer(31);
#endif

int ParseError = TVS_SUCCESS;
extern int lines;

class EnvTableEntry : public RVCollectableString {
  private:
    RVCString value;
    int saveDone;
  public:
    EnvTableEntry(const char* name, const char* v) :

      RVCollectableString(name), value(v), saveDone(0) {}

    ~EnvTableEntry() {}

    const char* getName()     { return (const char*)*this; }
    const char* getValue()     { return (const char*)value; }
    int         getSaveDone() { return saveDone; }

    void setValue(const char* v)      { value = v; }
    void setSaveDone(int sd)        { saveDone = sd; }
};

#define CLObject Object

class EnvTableObject : public RVCollectable {
  private:
    TableHandle h;
    RVHashTable *ht;
    RVHashTableIterator *ti;
    BOOL changed;

  public:
    EnvTableObject(TableHandle th, int size) : h(th), changed(FALSE), ti(NULL)
    {
      ht = new RVHashTable(size);
      ti = new RVHashTableIterator(*ht);
    }

    EnvTableObject(TableHandle th) : h(th), ht(NULL), ti(NULL) {}

    ~EnvTableObject();

    EnvTableEntry* findEntry(const char* entryName)
    {
      EnvTableEntry e(entryName,"");
      return (EnvTableEntry *)ht->find((RVCollectable *)&e);
    }

    int isEmpty()
    {
      return ht->isEmpty();
    }

    void add(EnvTableEntry * te)
    {
      ht->insert(te);
    }

    void resetIterator()
    {
      ti->reset();
      (*ti)();
      setChanged(FALSE);
    }

    void remove (EnvTableEntry* te)
    {
     ht->remove(te);
    }

    EnvTableEntry* getNextEntry()
    {
      if (changed)
      {
        ti->reset();
        (*ti)();
        return NULL;
      }
      if (ti->key())
      {
        EnvTableEntry* entry = (EnvTableEntry *)ti->key();
        (*ti)();
        return entry;
      }
      else
        return NULL;
    }

    BOOL isChanged() { return changed; }

    void setChanged(BOOL newvalue) { changed = newvalue; }

#ifdef __TV_solaris__
#define RW_BOOL_TYPE_HACK bool
#else
#define RW_BOOL_TYPE_HACK int
#endif

    bool  isEqual(const RVCollectable* eto) const
    {
      return (bool)(((EnvTableObject*)eto)->h == h);
    }

    unsigned hash() const
    {
      return (unsigned)h;
    }

    virtual RVClassID isA() const
    {
      return (RVClassID)1;
    }

    virtual char *nameOf() const
    {
      return ("EnvTableObject");
    }

    virtual void printOn( ostream& os) const
    {
    }


};

EnvTableObject::~EnvTableObject()
{
  if (!ht)
    return;

  try {
    ht->clearAndDestroy();
    delete ht;

    if (ti)
      delete ti;
  } catch (...) {
  }
}

void WINAPI TVSTerminate()
{
  // tablesContainer.clearAndDestroy();
  HACK_CLASS_OP(tablesContainer)clearAndDestroy();
}
EnvTableObject* findEnvTable(TableHandle h)
{
  EnvTableObject eto(h);
  //return (EnvTableObject *) tablesContainer.find(&eto);
  return (EnvTableObject *)HACK_CLASS_OP(tablesContainer)find(&eto);
}

EnvTableEntry* findTableEntry(TableHandle h, const char* entry)
{
  EnvTableObject* eto = findEnvTable(h);
  if (!eto)
    return NULL;

  return eto->findEntry(entry);
}

TableHandle WINAPI TVSCreateTable()
{
  counter++;
  EnvTableObject* eto = new EnvTableObject(counter,101);
  // tablesContainer.insert(eto);
  HACK_CLASS_OP(tablesContainer)insert(eto);

  //sergei 3/24/01 - debugPrint(4,("Created table %d\n",counter));

  return counter;
}

void  WINAPI TVSDestroyTable(TableHandle h)
{
  //sergei 3/24/01 - debugPrint(4,("destroying table %d\n",h));

  EnvTableObject* eto = findEnvTable(h);

  if (eto)
    // tablesContainer.remove(eto);
    HACK_CLASS_OP(tablesContainer)remove(eto);
    delete eto;
}

const char* WINAPI TVSGetValue(TableHandle h, const char* e)
{
  //sergei 3/24/01 - debugPrint(4,("TVSGetValue(%d,%s)\n",h,e));

  assert(h && e);

  EnvTableEntry* entry = findTableEntry(h, e);
  if (!entry)  //entry is nil
    return NULL;
  else
  {
    //sergei 3/24/01 - debugPrint(4,("found entry %s=%s\n",entry->getName(),entry->getValue()));
    return entry->getValue();
  }
}

BOOL WINAPI TVSAssignValue(TableHandle h, const char* e, const char* v)
{
//  printf("Inside TVSAssignValue\n");
  ////sergei 3/24/01 - debugPrint(4,("TVSAssignValue(%d,%s,%s)\n",h,e,v));
  assert(h && e && v);

  EnvTableEntry* entry = findTableEntry(h, e);
  if (entry)
  {
    entry->setValue(v);
    return TRUE;
  }
  else
    return TVSAddEntry(h,e,v);
}

BOOL TVSIsSaveDone(TableHandle h,const char *e)
{
  //sergei 3/24/01 - debugPrint(4,("TVSIsSaveDone(%d,%s)\n",h,e));
  assert(h && e);

  EnvTableEntry* entry = findTableEntry(h, e);
  if (!entry)
    return FALSE;
  else
    return entry->getSaveDone();
}

BOOL TVSUpdateSaveDone(TableHandle h, const char *e, int saveDone)
{
  //sergei 3/24/01 - debugPrint(4,("TVSUpdateSaveDone(%d,%s,%d)\n",h,e,saveDone));
  assert(h && e);

  EnvTableEntry* entry = findTableEntry(h, e);
  if (!entry)
    return FALSE;
  else
  {
    entry->setSaveDone(saveDone);
    return TRUE;
  }
}

BOOL TVSAddEntry(TableHandle h, const char *e, const char *v)
{
//  printf("Inside TVSAddEntry\n");
  ////sergei 3/24/01 - debugPrint(4,("TVSAddEntry(%d,%s,%s)\n",h,e,v));
  assert(h && e && v);

  EnvTableObject* eto = findEnvTable(h);
  if (!eto)
    return FALSE;

  EnvTableEntry* entry = new EnvTableEntry(e, v);
  eto->add(entry);
  eto->setChanged(TRUE);
  return TRUE;
}

BOOL WINAPI TVSDeleteEntry(TableHandle h, const char* e)
{
//  printf("Inside TVSDeleteEntry\n");
  //sergei 3/24/01 - debugPrint(4,("TVSDeleteEntry(%d,%s)\n",h,e));
  assert(h && e);

  EnvTableObject* eto = findEnvTable(h);
  if (!eto)
    return FALSE;

  EnvTableEntry* entry = eto->findEntry(e);
  if (!entry)
    return FALSE;
  else
  {
    //sergei 3/24/01 - debugPrint(4,("Deleting entry %s=%s\n",entry->getName(),entry->getValue()));
    eto->remove(entry);
    delete entry;
    eto->setChanged(TRUE);
    return TRUE;
  }
}

BOOL WINAPI TVSIsTableEmpty(TableHandle h)
{
//  printf("Inside TVSIsTableEmpty\n");
  //sergei 3/24/01 - debugPrint(4,("TVSIsTableEmpty(%d)\n",h));
  assert(h);

  EnvTableObject* eto = findEnvTable(h);
  if (!eto)
    return TRUE;

  return (eto->isEmpty());
}

BOOL WINAPI TVSGetNextEntry(int type,TableHandle h,char **e,char **k,int *saveDone)
{
  //sergei 3/24/01 - debugPrint(4,("TVSGetNextEntry(%d,%d)\n",type,h));
  *e = *k = "?";
  EnvTableObject* eto = findEnvTable(h);
  if (!eto)
    return FALSE;

  if (type == FIRST_ENTRY) {
    eto->resetIterator();
  }

  EnvTableEntry* entry = eto->getNextEntry();
  if (!entry)
    return FALSE;

  *e = (char*)entry->getName();
  *k = (char*)entry->getValue();
  *saveDone = entry->getSaveDone();

  //sergei 3/24/01 - debugPrint(4,("Next entry: %s=%s\n",entry->getName(),entry->getValue()));

  return TRUE;
}


int WINAPI TVSGetError()
{
  EnterProcedure("TVSGetError");
  int saveErr = ParseError;
  ParseError = TVS_SUCCESS;
  return saveErr;
}

BOOL WINAPI TVSIsError()
{
  EnterProcedure("TVSIsError");
  if (ParseError != TVS_SUCCESS)
    return TRUE;
  return FALSE;
}
BOOL WINAPI TVSLinesRead()
{
	return lines;
}

