/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// UnicodeCType.cpp : Unicode CType functions
//

#ifdef __TV_WINDOWS__
#include "windows.h"
#else //UNIX
//typedef unsigned char BYTE;
//typedef unsigned short WORD;
#define FALSE 0
#ifndef __TV_linux__
#include <sys/ddi.h>
#endif
#include "gt_portable.h" 
#endif //__TV_WINDOWS__

#include "uniprop.h"
#include "decompose_tab.h"
#include "unitype.h"
#include "typetab.h"

#include "unictype.h"
#include "gt_unicodectype.h"

UNICODE_TYPE UniGetProperty(gt_wchar_t wCode)
{

	WORD wType = unitype_tab[wCode >> 4];
	WORD wPropertyIndex;

	if (wType & 0x8000)
		wPropertyIndex = wType & ~0x8000;
	else
		wPropertyIndex = unitype_tab[wType + (wCode & 0x0F)];

	return unitype_list[wPropertyIndex];
}


int UniDecomposeChar(gt_wchar_t wCode, gt_wchar_t *pDecomposed, int nMaxDecomposed)
{
	WORD *wTemp;
	int nLen = 0;
	bool bDecomposition = FALSE;
	WORD pDecomposedNew[MAX_DECOMPOSED];

	UNICODE_TYPE ut = UniGetProperty(wCode);
	if (ut.pDecomposition[0] == 0xFFFF)
	{	// longer than 2 - found in the decomposition table
		wTemp = &decompose_tab[ut.pDecomposition[1]];
		for (nLen = 0; *wTemp != 0xFFFF && nLen < nMaxDecomposed; wTemp++, nLen++)
		{
			pDecomposedNew[nLen] = *wTemp;
		}
		bDecomposition = 1;
	}
	else if (ut.pDecomposition[0] != 0)
	{	// not longer than 2 - found in ut
		pDecomposedNew[nLen++] = ut.pDecomposition[0];
		if (ut.pDecomposition[1])
			pDecomposedNew[nLen++] = ut.pDecomposition[1];
		bDecomposition = 1;

	}
	else  // no decomposition
	{
		pDecomposedNew[nLen++] = wCode; 
		bDecomposition = 0;
	}

	if (bDecomposition)
	{  // run recursively on all decomposed chars
		int i;
		int nNewLen = 0;

		for (i = 0; i < nLen && (nNewLen < nMaxDecomposed ||
			nMaxDecomposed == 0); i++)
		{
			nNewLen += UniDecomposeChar(pDecomposedNew[i],
						pDecomposed ? pDecomposed + nNewLen : NULL, 
						nMaxDecomposed ? nMaxDecomposed - nNewLen : 0);
		}
		nLen = nNewLen;
	}
	memcpy(pDecomposed, pDecomposedNew, sizeof(WORD) * min(nLen, nMaxDecomposed) );
	
	return nLen;
}


int UniDecompose(gt_wchar_t *pIn, int nInLen, gt_wchar_t *pDecomposed, int nMaxDecomposed)
{
	int i;
	int nNewLen = 0;

	for (i = 0; i < nInLen && pIn[i] && (nNewLen < nMaxDecomposed - 1 
		|| nMaxDecomposed == 0); i++)
	{
		nNewLen += UniDecomposeChar(pIn[i], 
			pDecomposed ? pDecomposed + nNewLen : NULL, 
			nMaxDecomposed ? nMaxDecomposed - nNewLen : 0);
	}

	bool bReordered = pDecomposed ? 1 : 0;

	while (bReordered)
	{	// swap betweem sequences of adjacent combining characters till properly ordered
		// although not optimal, this algorithm should be extrememly fast for typical text
		bReordered = 0;
		for (i = 0; i < nNewLen - 1; i++)
		{
			if (UniCanonicalReorderNeeded(pDecomposed[i], pDecomposed[i+1]))
			{	// swap chars
				WORD wTemp = pDecomposed[i];
				pDecomposed[i] = pDecomposed[i+1];
				pDecomposed[i+1] = wTemp;
				bReordered = 1;
			}
		}
	}
	if(pDecomposed)
		pDecomposed[nNewLen] = 0;
	return nNewLen;
}


bool UniCanonicalReorderNeeded(gt_wchar_t w1, gt_wchar_t w2)
{
	if (UniGetCanonicalCombiningClass(w2) != 0 &&
		(UniGetCanonicalCombiningClass(w1) > UniGetCanonicalCombiningClass(w2)))
		return 1;
	else
		return 0;
}


int UniGetCanonicalCombiningClass(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	return ut.nCanonicalCombiningClass;
}



int w_normalize(gt_wchar_t *pIn, int nInLen, gt_wchar_t *pDecomposed, int nMaxDecomposed)
{
	return UniDecompose(pIn, nInLen, pDecomposed, nMaxDecomposed);
}


bool UniCheckProp(int iProp, int *pPropList)
{
	while (*pPropList)
	{
		if (iProp == *pPropList)
			return 1;
		pPropList++;
	}
	return 0;
}

bool w_isalpha(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Lu, UNI_GC_Ll, UNI_GC_Lm, UNI_GC_Lo, UNI_GC_Lt, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}

bool w_isdigit(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Nd, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}

bool w_isalnum(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Lu, UNI_GC_Ll, UNI_GC_Lm, UNI_GC_Lo, UNI_GC_Lt, UNI_GC_Nd, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}

bool w_iscntrl(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Cc, UNI_GC_Cf, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}

bool w_isspace(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Zs, /*UNI_GC_Ss,*/ 0};
	return (UniCheckProp((int) ut.byGeneralCategory, pPropList) ||
		(wChar >= 0x0009 && wChar <= 0x000d));
}

bool w_isblank(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Zs, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}

bool w_ispunct(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Pd, UNI_GC_Ps, UNI_GC_Pe, UNI_GC_Po, UNI_GC_Pi, UNI_GC_Pf, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}

bool w_isgraph(gt_wchar_t wChar) 
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Mn, UNI_GC_Mc, UNI_GC_Me, UNI_GC_Nd, UNI_GC_Nl, UNI_GC_No, 
				UNI_GC_No, UNI_GC_No, UNI_GC_No, UNI_GC_No, UNI_GC_No, UNI_GC_No, 
				UNI_GC_Lu, UNI_GC_Ll, UNI_GC_Lt, UNI_GC_Lm, UNI_GC_Lo, UNI_GC_Pc, 
				UNI_GC_Pd, UNI_GC_Ps, UNI_GC_Pe, UNI_GC_Po, UNI_GC_Sm, UNI_GC_Sc, 
				UNI_GC_Sk, UNI_GC_So, UNI_GC_Pi, UNI_GC_Pf, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}

bool w_isprint(gt_wchar_t wChar) 
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Mn, UNI_GC_Mc, UNI_GC_Me, UNI_GC_Nd, UNI_GC_Nl, UNI_GC_No, 
				UNI_GC_No, UNI_GC_No, UNI_GC_No, UNI_GC_No, UNI_GC_No, UNI_GC_No, 
				UNI_GC_Lu, UNI_GC_Ll, UNI_GC_Lt, UNI_GC_Lm, UNI_GC_Lo, UNI_GC_Pc, 
				UNI_GC_Pd, UNI_GC_Ps, UNI_GC_Pe, UNI_GC_Po, UNI_GC_Sm, UNI_GC_Sc, 
				UNI_GC_Sk, UNI_GC_So, UNI_GC_Zs, UNI_GC_Pi, UNI_GC_Pf, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}


bool w_isupper(gt_wchar_t wChar) 
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Lu, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}


bool w_islower(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Ll, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}


bool w_istitle(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Lt, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}


//bool w_isxdigit(WORD wChar)
//{
//	UNICODE_TYPE ut = UniGetProperty(wChar);
//	int pPropList[] = {UNI_GC_Hd, 0};
//	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
//}


bool w_isisocntrl(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Cc, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}


bool w_isfmtcntrl(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Cf, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}


bool w_issymbol(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Sm, UNI_GC_Sc, UNI_GC_So, UNI_GC_Sk, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}


bool w_isnumber(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Nd, UNI_GC_No, UNI_GC_Nl, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}


bool w_isnonspacing(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Mn, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}


bool w_isopenpunct(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Ps, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}


bool w_isclosepunct(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Pe, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}


bool w_isinitialpunct(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Pi, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}


bool w_isfinalpunct(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_GC_Pf, 0};
	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
}


bool w_iscomposite(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	return (ut.pDecomposition[0] != 0);
}


//bool w_ishex(WORD wChar)
//{
//	UNICODE_TYPE ut = UniGetProperty(wChar);
//	int pPropList[] = {UNI_GC_Hd, 0};
//	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
//}


//bool w_isquote(WORD wChar)
//{
//	UNICODE_TYPE ut = UniGetProperty(wChar);
//	int pPropList[] = {UNI_GC_Qm, 0};
//	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
//}


//bool w_issymmetric(WORD wChar)
//{
//	UNICODE_TYPE ut = UniGetProperty(wChar);
//	return (ut.byFlag | UNI_FLAG_MIRRORED);
//}


bool w_ismirroring(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	return ((ut.byFlag | UNI_FLAG_MIRRORED) != 0);
}


//bool w_isnonbreaking(WORD wChar)
//{
//	UNICODE_TYPE ut = UniGetProperty(wChar);
//	int pPropList[] = {UNI_GC_Nb, 0};
//	return UniCheckProp((int) ut.byGeneralCategory, pPropList);
//}


/*
 * Directionality macros.
 */
BYTE w_getbiditype(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	return ut.byBiDiCategory;
}
bool w_isrtl(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_BC_R, 0};
	return UniCheckProp((int) ut.byBiDiCategory, pPropList);
}


bool w_isltr(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_BC_L, 0};
	return UniCheckProp((int) ut.byBiDiCategory, pPropList);
}



bool w_isstrong(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_BC_L, UNI_BC_R, 0};
	return UniCheckProp((int) ut.byBiDiCategory, pPropList);
}


bool w_isweak(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_BC_EN, UNI_BC_ES, UNI_BC_ET, UNI_BC_AN, UNI_BC_CS, 0};
	return UniCheckProp((int) ut.byBiDiCategory, pPropList);
}


bool w_isneutral(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_BC_B, UNI_BC_S, UNI_BC_WS, UNI_BC_ON, 0};
	return UniCheckProp((int) ut.byBiDiCategory, pPropList);
}


bool w_isseparator(gt_wchar_t wChar)
{
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int pPropList[] = {UNI_BC_B, UNI_BC_S, 0};
	return UniCheckProp((int) ut.byBiDiCategory, pPropList);
}

bool w_iscjkchar(gt_wchar_t wChar)
{
		return((wChar >= 0x3040 && wChar <= 0x30ff) ||
			(wChar >= 0x1100 && wChar <= 0x11ff) ||
			(wChar >= 0xac00 && wChar <= 0xd7a3) ||
			(wChar >= 0x4e00 && wChar <= 0x9fff));
}


/*
 * Other miscellaneous character property macros.
 */
bool w_ishan(gt_wchar_t wChar) 
{ 
	return ((wChar >= 0x4e00 && wChar <= 0x9fff) ||
                (wChar >= 0xf900 && wChar <= 0xfaff));
}


bool w_ishangul(gt_wchar_t wChar) 
{
	return (wChar >= 0xac00 && wChar <= 0xd7ff);
}


/**************************************************************************
 *
 * Functions for case conversion.
 *
 **************************************************************************/

gt_wchar_t w_toupper(gt_wchar_t wChar)
{
	if (!w_islower(wChar))
		return wChar;
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int nOffset = ut.wLowerUpperOffset;
	if ((ut.byFlag & UNI_FLAG_CASE_OFFSET_POSITIVE) == 0)
		nOffset *= -1;  // negative offset
	return wChar + nOffset;
}


gt_wchar_t w_tolower(gt_wchar_t wChar)
{
	if (!w_isupper(wChar))
		return wChar;
	UNICODE_TYPE ut = UniGetProperty(wChar);
	int nOffset = ut.wLowerUpperOffset;
	if ((ut.byFlag & UNI_FLAG_CASE_OFFSET_POSITIVE) == 0)
		nOffset *= -1;  // negative offset
	return wChar + nOffset;
}

gt_wchar_t *w_strlwr( gt_wchar_t *string )
{
	if(string == NULL)
		return NULL;
	gt_wchar_t *pwChar = string;
    while (*pwChar)
    {
		*pwChar = w_tolower(*pwChar);
		pwChar++;
	}
	return string;
}

gt_wchar_t *w_strupr( gt_wchar_t *string )
{
	if(string == NULL)
		return NULL;
	gt_wchar_t *pwChar = string;
    while (*pwChar)
    {
		*pwChar = w_toupper(*pwChar);
		pwChar++;
	}
	return string;
}
int w_strlen( const gt_wchar_t * str)
{
	int i = 0;
	if (str)
		while (str[i]) i++;
	return (i);
}
int w_stricmp( const gt_wchar_t *string1, const gt_wchar_t *string2 )
{
	gt_wchar_t *string1_norm = new gt_wchar_t[w_strlen((gt_wchar_t *) string1) * MAX_DECOMPOSE + 1];
	gt_wchar_t *string2_norm = new gt_wchar_t[w_strlen((gt_wchar_t *) string2) * MAX_DECOMPOSE + 1];
	
	w_normalize( (gt_wchar_t *) string1, w_strlen((gt_wchar_t *) string1), 
		string1_norm, w_strlen((gt_wchar_t *) string1) * MAX_DECOMPOSE);

	w_normalize( (gt_wchar_t *) string2, w_strlen((gt_wchar_t *) string2), 
		string2_norm, w_strlen((gt_wchar_t *) string2) * MAX_DECOMPOSE);

	gt_wchar_t *pwChar1 = (gt_wchar_t*)string1_norm;
	gt_wchar_t *pwChar2 = (gt_wchar_t*)string2_norm;
	gt_wchar_t wChar1, wChar2;
	int nRes = 0;
	while ((!nRes) && *pwChar1 && *pwChar2)
	{
		wChar1 = w_tolower(*pwChar1);
		wChar2 = w_tolower(*pwChar2);
		pwChar1++;
		pwChar2++;
		nRes = wChar1 - wChar2;
	}
	//one of the strings arrived to NULL termination
	if(nRes == 0)
	{
		nRes = *pwChar1 - *pwChar2;
	}
	delete []string1_norm;
	delete []string2_norm;
	return nRes;
}

int w_strnicmp( const gt_wchar_t *string1, const gt_wchar_t *string2, size_t count )
{
	gt_wchar_t *string1_norm = new gt_wchar_t[count * MAX_DECOMPOSE + 1];
	gt_wchar_t *string2_norm = new gt_wchar_t[count * MAX_DECOMPOSE + 1];
	
	w_normalize( (gt_wchar_t *) string1, count, 
		string1_norm, count * MAX_DECOMPOSE);

	w_normalize( (gt_wchar_t *) string2, count, 
		string2_norm, count * MAX_DECOMPOSE);

	gt_wchar_t *pwChar1 = (gt_wchar_t*)string1_norm;
	gt_wchar_t *pwChar2 = (gt_wchar_t*)string2_norm;
	gt_wchar_t wChar1, wChar2;
	int nRes = 0;
	size_t i = 0;
	while (i < count && (!nRes) && *pwChar1 && *pwChar2)
	{
		if((signed)i >= (signed)count)
			break;
		wChar1 = w_tolower(*pwChar1);
		wChar2 = w_tolower(*pwChar2);
		pwChar1++;
		pwChar2++;
		nRes = wChar1 - wChar2;
		i++;
	}
	//one of the strings arrived to NULL termination
	if(nRes == 0 && (signed)i < (signed)(count -1))
	{
		nRes = *pwChar1 - *pwChar2;
	}
	delete []string1_norm;
	delete []string2_norm;
	return nRes;
}


gt_wchar_t* w_strcat(gt_wchar_t *ws1, const gt_wchar_t *ws2)
{
        gt_wchar_t *ptr1 = ws1;
        const gt_wchar_t *ptr2 = ws2;

        while (*ptr1)
                ptr1++;

        while (*ptr2)
                *ptr1++ = *ptr2++;

        *ptr1 = '\0';
        return ws1;
}

gt_wchar_t* w_strcpy(gt_wchar_t *ws1, const gt_wchar_t *ws2)
{
        gt_wchar_t *ptr1 = ws1;
        const gt_wchar_t *ptr2 = ws2;

        while (*ptr2)
                *ptr1++ = *ptr2++;

        *ptr1 = '\0';
        return ws1;
}

gt_wchar_t* w_strncpy(gt_wchar_t *ws1, const gt_wchar_t *ws2, size_t n)
{
        size_t i;
        for (i = 0; (i < n) && ws2[i]; i++)
                ws1[i] = ws2[i];

        while (i < n)
        {
                ws1[i] = '\0';
                i++;
        }

        return ws1;
}
/*int w_strlen(const gt_wchar_t *string1)
{
	int i;

	for (i = 0; string1[i] != 0; i++)
		;
	return i;
}
*/
/*

/*
 * Other macros inspired by John Cowan.
 *
bool w_ismark(WORD wChar) ucisprop(WORD wChar, UC_MN|UC_MC|UC_ME, 0)
bool ucismodif(WORD wChar) ucisprop(WORD wChar, UC_LM, 0)
bool ucisletnum(WORD wChar) ucisprop(WORD wChar, UC_NL, 0)
bool ucisconnect(WORD wChar) ucisprop(WORD wChar, UC_PC, 0)
bool ucisdash(WORD wChar) ucisprop(WORD wChar, UC_PD, 0)
bool ucismath(WORD wChar) ucisprop(WORD wChar, UC_SM, 0)
bool uciscurrency(WORD wChar) ucisprop(WORD wChar, UC_SC, 0)
bool ucismodifsymbol(WORD wChar) ucisprop(WORD wChar, UC_SK, 0)
bool ucisnsmark(WORD wChar) ucisprop(WORD wChar, UC_MN, 0)
bool ucisspmark(WORD wChar) ucisprop(WORD wChar, UC_MC, 0)
bool ucisenclosing(WORD wChar) ucisprop(WORD wChar, UC_ME, 0)
bool ucisprivate(WORD wChar) ucisprop(WORD wChar, UC_CO, 0)
bool ucissurrogate(WORD wChar) ucisprop(WORD wChar, UC_OS, 0)
bool ucislsep(WORD wChar) ucisprop(WORD wChar, UC_ZL, 0)
bool ucispsep(WORD wChar) ucisprop(WORD wChar, UC_ZP, 0)

bool ucisidentstart(WORD wChar) ucisprop(WORD wChar, UC_LU|UC_LL|UC_LT|UC_LO|UC_NL, 0)
bool ucisidentpart(WORD wChar) ucisprop(WORD wChar, UC_LU|UC_LL|UC_LT|UC_LO|UC_NL|\
                                   UC_MN|UC_MC|UC_ND|UC_PC|UC_CF, 0)

bool ucisdefined(WORD wChar) ucisprop(WORD wChar, 0, UC_CP)
bool ucisundefined(WORD wChar) !ucisprop(WORD wChar, 0, UC_CP)


//unsigned long uctotitle __((unsigned long code));

*/
