/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
//
// (c) 2000 GalTech Soft. All rights reserved.
//
// UniServ.cpp
// Internal functions for code page conversions.
// 
///////////////////////////////////////////////////////////////////////////////

#ifdef __TV_WINDOWS__
#include "windows.h"
#else //UNIX
#include "strings.h"
#include "gt_unicode.h"
//#include "winterface.h"
#endif

#include "gt_portable.h"
#include "uniserv.h"
#include "gt_unidef.h"
#include "unios.h"
#include "uniresource.h"
#include "dbcode.h"
#include "utfcode.h"
#include "gt_portable.h"
#include "gt_uniservapis.h"

extern CONVERT ConvertTable[128];


CONVERT* GetConversionRecord(UINT CodePage)
{
	for(CONVERT *pConvert = ConvertTable;
		pConvert->CodePage != CP_LAST; pConvert++)
	{
		if(pConvert->CodePage == CodePage)
			return pConvert;
	}

	return NULL;
}

BOOL _IsValidCodePage(UINT CodePage)
{
	for(CONVERT *pConvert = ConvertTable;
		pConvert->CodePage != CP_LAST; pConvert++)
	{
		if(pConvert->CodePage == CodePage)
			return TRUE;
	}
	return FALSE;
}

BOOL EnumCodePages(int iIterator, UINT *pCodePage,
		char* szDescription, BOOL* pbMultiTable)
{
	//Get th number records in table
    int nCount =  (sizeof(CPRecTable) / (sizeof(*CPRecTable)));
	//if iterator is out of range return FALSE
	if(iIterator >= nCount)
		return FALSE;
	//copy data from record to parameters and return TRUE
	CPREC* pRecord = &CPRecTable[iIterator];
	*pCodePage = pRecord->CodePage;
	strncpy(szDescription, pRecord->szDescription,
		MAX_RES_NAME);
	*pbMultiTable = pRecord->bMultiTable;

	return TRUE;
}


int MultiToDest(LPSTR lpszDst, gt_wchar_t u, int nMaxWChars, BOOL bNormalize, CONVERT *pConvert)
{
	int iIndx;
	WORD start;
	BOOL bFoundRange = FALSE;

	// first find which range the number belongs to
	// (replace with binary search)
	int nStart = 0;
	int nEnd = pConvert->nRanges - 1;
	if (u <= pConvert->Ranges[nEnd].end && 
		u >= pConvert->Ranges[nStart].start)
	{
		for (; nStart <= nEnd && !bFoundRange;)
		{
			int n = nStart + (nEnd - nStart) / 2;
			if (u >= pConvert->Ranges[n].start)
			{
				if (u <= pConvert->Ranges[n].end)
				{
					start = pConvert->Ranges[n].start;
					iIndx = pConvert->Ranges[n].offset;
					bFoundRange = TRUE;
				}
				else
				{
					nStart = n+1;
				}
			}
			else
			{
				nEnd = n - 1;
			}

		}
	}
//	for (iRange = 0; !bFoundRange && (iRange < nRanges);  iRange++) 
//	{
//		if  (u <= Ranges[iRange].end) 
//		{
//			start = Ranges[iRange].start;
//			iIndx = Ranges[iRange].offset;
//			bFoundRange = TRUE;
//		}
//	}
	if  (!bFoundRange || u < start)
	{
	    return ToDest(lpszDst, u, nMaxWChars, bNormalize);
	}

        // now convert the number
    return ToDest(lpszDst, pConvert->CodeRecs[iIndx+u-start], nMaxWChars, bNormalize);

}


   
int ToDest(LPSTR lpszDst, gt_wchar_t u, int nMaxWChars, BOOL bNormalize)
{

#ifdef _NORMALIZE	
	if(bNormalize)
	{
		gt_wchar_t wChar = u;
		return w_normalize(&wChar, 
				1, (gt_wchar_t*)lpszDst, nMaxWChars);
	}
	else if(lpszDst)
#else
	if(lpszDst)
#endif
	{
		//*((gt_wchar_t*) lpszDst) = u;
		*((gt_wchar_t*) lpszDst) = (u?u:'?');
//		lpszDst += sizeof(gt_wchar_t);	
	}

	return 1;
}

BOOL GetRealCodepage(UINT& SrcCodePage, LPCSTR lpSrcCharStr,
			int ccSrc)
{
	BOOL bDetected = FALSE;
	if(SrcCodePage == CP_JAPANESEAUTODETECT)
	{
		SrcCodePage = UL_JapaneseAutoDetect(lpSrcCharStr, ccSrc, &bDetected);
		return bDetected;
	}
#ifdef __TV_UNIX__
	if(SrcCodePage == CP_ACP)
		SrcCodePage = GetACP();
        if(SrcCodePage == CP_ACP)
              SrcCodePage = CP_MS_LATIN1;
#else //__TV_WINDOWS__
	if(SrcCodePage == CP_ACP)
		SrcCodePage = GetACP();

	if(SrcCodePage == CP_THREAD_ACP)
		SrcCodePage = GetCurrentThreadCP();
#endif //UNIX
	return TRUE;


}
int CheckForAlgorithmConversion(UINT& SrcCodePage, LPCSTR lpSrcCharStr,
			int ccSrc, LPSTR* lppTarget,  e_ConveresionResult *peResult, 
			int *piCharMode)
{
	int nLen = 0;
	int nResult = 0;
	CONVERT* pConvert = NULL;

	switch(SrcCodePage)
	{


		case CP_UTF8:
			nLen = ccSrc != -1 ? (ccSrc + 2) * sizeof(gt_wchar_t) :
				(UL_strlen((LPSTR)lpSrcCharStr, SrcCodePage, -1) + 2);
			*lppTarget = (LPSTR)new gt_wchar_t[nLen];
			nResult = Utf8ToUcs2(lpSrcCharStr, ccSrc, (gt_wchar_t*)(*lppTarget), nLen);
			*peResult = eConvFinished;
			return nResult* sizeof(gt_wchar_t);
		case CP_EUCJP:
			SrcCodePage  = CP_MS_JAPANESE;
			/*pConvert = GetConversionRecord(SrcCodePage);
			if(pConvert->ConvertType == eUnknownConvert)
			{
				*peResult = eConvFailed;
				return 0;
			}*/
			nLen = ccSrc != -1 ? (ccSrc + 2) * sizeof(gt_wchar_t) :
				(UL_strlen((LPSTR)lpSrcCharStr, SrcCodePage, -1) + 2);
			*lppTarget = (LPSTR)new gt_wchar_t[nLen];
			nResult = UL_Euc2SJis(lpSrcCharStr, ccSrc, *lppTarget, nLen* sizeof(gt_wchar_t));
			*peResult = eConvForward;
			return nResult;

		case CP_JIS:
			{
				SrcCodePage  = CP_MS_JAPANESE;
				pConvert = GetConversionRecord(SrcCodePage);
			/*	if(pConvert->ConvertType == eUnknownConvert)
				{
					*peResult = eConvFailed;
					return 0;
				}*/
				int iCharMode = (piCharMode == 0) ? 1 : *piCharMode;
				nLen = ccSrc != -1 ? (ccSrc + 2) * sizeof(gt_wchar_t) :
					(UL_strlen((LPSTR)lpSrcCharStr, SrcCodePage, -1) + 2);
				*lppTarget = (LPSTR)new gt_wchar_t[nLen];
				nResult = UL_Jis2SJis(lpSrcCharStr, ccSrc, 
					*lppTarget, nLen* sizeof(gt_wchar_t), &iCharMode);
				if (piCharMode)
					*piCharMode = iCharMode;
			}
			*peResult = eConvForward;
			return nResult;
		default:
			*peResult = eConvRegular;
			return 0;
	}

}

int UL_strlen(LPSTR str, UINT CodePage, int iLength, int *pnBytesUsed)
{
	if (pnBytesUsed)
		*pnBytesUsed = -1;

	if(IS_DOUBLE_BYTE_ENCODING(CodePage))
		return UL_DBCSstrlen(str, CodePage, iLength, pnBytesUsed);

	if(IS_ESCAPE_SEQUENCE_CODEPAGE(CodePage))
		return UL_EscapeSeqstrlen(str, CodePage, iLength, pnBytesUsed);

	if(CodePage == CP_UTF8)
		return Utf8ToUcs2(str, iLength, NULL, 0, pnBytesUsed);


	else
	{
		BOOL bNullTermination = iLength == -1 ? TRUE : FALSE;
		int i;
		for(i = 0; i < iLength || 
			(bNullTermination && str[i] != '\0'); i++)
		{};
		if (pnBytesUsed)
			*pnBytesUsed = i;
		if(bNullTermination)
			i++;
		return i;
	}

}
#ifdef __TV_WINDOWS__
UINT GetCurrentThreadCP()
{
	LCID lcid =	GetThreadLocale();
	WORD langid = LANGIDFROMLCID(lcid);
	return GetCPFromLanguage(langid);
}

UINT GetCPFromLanguage(WORD langid)
{
	WORD PRIMARYLANGID(langid);
	switch (langid)
	{
		case LANG_THAI		:	return CP_MS_THAI;			
		case LANG_JAPANESE	:	return CP_MS_JAPANESE;	
		case LANG_CHINESE	:	if (SUBLANGID(langid) == SUBLANG_CHINESE_TRADITIONAL)  
									return CP_MS_CHINESE_TRADITIONAL;
								return CP_MS_CHINESE_SIMPLIFIED;
		case LANG_KOREAN	:	return CP_MS_KOREAN;
		case LANG_CROATIAN	:
		case LANG_CZECH		:
		case LANG_HUNGARIAN	:
		case LANG_POLISH	:
		case LANG_ROMANIAN	:
		case LANG_SLOVAK	:	return CP_MS_LATIN2;
		case LANG_RUSSIAN	:	return CP_MS_CYRILLIC; 
		case LANG_GREEK		:	return CP_MS_GREEK;
		case LANG_TURKISH	:	return CP_MS_TURKISH;
		case LANG_HEBREW	:	return CP_MS_HEBREW;
		case LANG_ARABIC	:	return CP_MS_ARABIC;
		case LANG_ESTONIAN	:
		case LANG_LATVIAN	:
		case LANG_LITHUANIAN:	return CP_MS_BALTIC;
		default				:	return 	ANSI_CHARSET;
	}

}
#endif //__TV_WINDOWS__

int ConvertWithErrors(
	UINT SrcCodePage,		// source code page 
	DWORD dwFlags,			// character-type options 	
    LPCSTR lpSrcCharStr,    // source (string) buffer pointer
    int ccSrc,				// bytes in Src
    LPWSTR lpDstCharStr,     // destination buffer pointer
    int cchWideChar				// output buffer limit
	)
{

	BOOL bNullTermination = ccSrc == -1 ? TRUE : FALSE;
	UINT nDst = 0;
	gt_wchar_t wCharReplace = L'?';
	gt_wchar_t* lpDstWStr = lpDstCharStr;
	for (int i = 0; i < ccSrc || 
			(bNullTermination && lpSrcCharStr[i] != '\0'); i++)
	{
		if(nDst == (UINT)cchWideChar)
			break;
		if(IS_DOUBLE_BYTE_ENCODING(SrcCodePage) && 
			UL_IsLeadByte(lpSrcCharStr[i], SrcCodePage))
			/* Double Byte character */
		{
			*lpDstWStr = wCharReplace;	
			i++;        // skip over lower byte of character
		}
		else
		{
			if(lpSrcCharStr[i] > 127)
			{
				*lpDstWStr = wCharReplace;	
			}
			else
			{
				*lpDstWStr = UL_MAKEWORD(lpSrcCharStr[i], 0x00 );
			}
		}
		nDst ++;
		lpDstWStr++;
	}
	if(bNullTermination && cchWideChar - nDst > 1)
	{
		*lpDstWStr++ = L'\0';
		nDst++;
	}

	return nDst*sizeof(gt_wchar_t);
}

