/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/*///////////////////////////////////////////////////////////////////////////////
 * utfcode.cp -- UTF encoding implementation.
 * Copyright (C) GalTech Soft Ltd. 1998-2000. All rights reserved.
 *
///////////////////////////////////////////////////////////////////////////////
*/

#ifdef __TV_WINDOWS__
#include "windows.h"
#else 
#include "no_windows.h"
#endif
#include "stddef.h"

#include "gt_portable.h"
#include "utfcode.h"

// unicode encodings
#define TWOTO6      64
#define TWOTO12     (TWOTO6 * TWOTO6)
#define DOUBLET     0xC0
#define TRIPLET     0xE0
#define TRAIL       0x80
#define QUADRUPLET  0xF0

int Ucs2ToUtf8(LPCWSTR lpUcs2, int ccUcs2, 
	LPSTR lpUtf8, int ccUtf8)
{
    gt_wchar_t compos;
	int  iUcs2 = 0, iUtf8 = 0;
	char *lpCharUcs2 = (char*)lpUcs2;

	BOOL bGetStrLengthOnly = ccUtf8 == 0 ? TRUE : FALSE;
	BOOL bNullTermination = ccUcs2 == -1 ? TRUE : FALSE;

	ccUcs2 *= sizeof(gt_wchar_t);

	while((iUcs2 < ccUcs2|| 
		(bNullTermination && lpUcs2[iUcs2 / sizeof(gt_wchar_t)] != 0 )) &&
		(bGetStrLengthOnly || iUtf8 < ccUtf8))
	{	
		compos = *((gt_wchar_t*)(lpCharUcs2+iUcs2));//lpCharUcs2[iUcs2];

		if(compos < 0x80) 
		{
			// a 7-bit ascii number so write as one "octet"
			if(!bGetStrLengthOnly)
				lpUtf8[iUtf8] = (char)compos;//lo;
			iUtf8++;
		}
		else if(compos < 0x800) 
		{
			// encode to 2 octets
			if(!bGetStrLengthOnly)
				lpUtf8[iUtf8] = (char)(DOUBLET | (compos / TWOTO6)) ;
			iUtf8++;
			if(!bGetStrLengthOnly)
				lpUtf8[iUtf8] = (char)(TRAIL | (compos % TWOTO6)) ;
			iUtf8++;
		}
		else 
		{
				// encode to 3 octets (NO utf-16 encodings)
			if(!bGetStrLengthOnly)
				lpUtf8[iUtf8] = (char)(TRIPLET | (compos / TWOTO12)) ;
			iUtf8++;
			if(!bGetStrLengthOnly)
				lpUtf8[iUtf8] = (char)(TRAIL | ((compos / TWOTO6) % TWOTO6)) ;
			iUtf8++;
			if(!bGetStrLengthOnly)
				lpUtf8[iUtf8] = (char)(TRAIL | (compos % TWOTO6)) ;
			iUtf8++;
		}
		iUcs2 += sizeof(gt_wchar_t);
	}
	if (bNullTermination)
	{
		if(!bGetStrLengthOnly)
		{
			iUtf8 = ccUtf8 - iUtf8 >= sizeof(char) ? 
				iUtf8 : ccUtf8 - sizeof(char);
			lpUtf8[iUtf8] = '\0';

		}
		iUtf8++;
	}
	return 	iUtf8;
}



int Utf8ToUcs2(LPCSTR lpUtf8, int ccUtf8,
	LPWSTR lpUcs2, int ccUcs2, int *pnBytesUsed)
{
  unsigned char cUtf8_1, cUtf8_2, cUtf8_3;
  int  iUcs2 = 0, iUtf8 = 0;
  BOOL bGetStrLengthOnly = (ccUcs2 == 0)&&(lpUcs2==0) ? TRUE : FALSE;
  BOOL bNullTermination = ccUtf8 == -1 ? TRUE : FALSE;
  gt_wchar_t* lpStrUcs2 = lpUcs2;

  while((bGetStrLengthOnly || iUcs2 < ccUcs2) &&
    ((!bNullTermination && (iUtf8 < ccUtf8))|| 
    (bNullTermination && lpUtf8[iUtf8] != '\0')))
  {
    cUtf8_1 = lpUtf8[iUtf8];
    iUtf8 ++;
    if(cUtf8_1 < DOUBLET)
    {
      if(!bGetStrLengthOnly)
        lpStrUcs2[iUcs2] = (gt_wchar_t) cUtf8_1;
    }
    else
    {
      cUtf8_2 = lpUtf8[iUtf8];
      iUtf8 ++;
      if(cUtf8_1 < TRIPLET)
      {
        if ((! bNullTermination && iUtf8 > ccUtf8)|| 
          (bNullTermination && cUtf8_2 == '\0'))
        {   // truncated char
          iUtf8--;
          break;
        }
        if(!bGetStrLengthOnly)
          lpStrUcs2[iUcs2] = (gt_wchar_t)((cUtf8_1- DOUBLET) * TWOTO6 + 
          (cUtf8_2 - TRAIL));
      }
      else
      {
        cUtf8_3 = lpUtf8[iUtf8];
        iUtf8 ++;
        if(cUtf8_1 < QUADRUPLET)
        {
          if ((! bNullTermination && iUtf8 > ccUtf8)|| 
            (bNullTermination && 
            (cUtf8_3 == '\0' || cUtf8_2 == '\0')))
          {   // truncated char
            iUtf8-=2;
            break;
          }
          if(!bGetStrLengthOnly)
            lpStrUcs2[iUcs2] = (gt_wchar_t)((cUtf8_1- TRIPLET) * TWOTO12 + 
            (cUtf8_2 - TRAIL)* TWOTO6 + (cUtf8_3 - TRAIL));
        }
        else
          //error
          break;
      }
    }
    iUcs2++ ;
  }
  if (bNullTermination)
  {
    if(!bGetStrLengthOnly)
    {
      iUcs2 = ccUcs2 - iUcs2 >= 1 ? iUcs2 : ccUcs2 - 1;
      lpStrUcs2[iUcs2] = '\0';
      
    }
    iUcs2++;
  }
  if (pnBytesUsed)
    *pnBytesUsed = iUtf8;
  
  return iUcs2;
}


