/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifdef __TV_WINDOWS__
#include "windows.h"
#endif

#define ID_FILE ID_FILE_UNICODEAPI

#include "gt_portable.h"
#include "gt_unidef.h"
#include "gt_uniservapis.h"
#include "uniserv.h"
#include "utfcode.h"
#include "gt_portable.h"
#include "unios.h"
#include "gt_unicodectype.h"

int PrepareForWideToMulti(UINT& SrcCodePage, LPCWSTR lpWideCharStr, 
	int ccSrc, LPSTR* lppTarget,  e_ConveresionResult *peResult)
{
	int nLen = 0;
	int nResult = 0;
	CONVERT* pConvert = NULL;
	switch(SrcCodePage)
	{


		case CP_UTF8:
			nLen = ccSrc != -1 ? (ccSrc + 2) * 3 :
				(w_strlen(lpWideCharStr) * 3 + 3);
			*lppTarget = new char[nLen];
			nResult =  Ucs2ToUtf8((LPCWSTR)(lpWideCharStr), ccSrc,
				 *lppTarget, nLen );
			*peResult = eConvFinished;
			return nResult;

		case CP_EUCJP:
			SrcCodePage  = CP_MS_JAPANESE;
			*peResult = eConvertAfter;
			return 0;

		case CP_JIS:
			SrcCodePage  = CP_MS_JAPANESE;
			*peResult = eConvertAfter;
			return 0;

		default:
			*peResult = eConvRegular;
			return 0;
	}
	*peResult = eConvRegular;
	return 0;
}
	
	
int ToMultiDst(LPSTR lpszDst, WORD u, int *nCount, CONVERT *pConvert)
{
    int iIndx;
    WORD start;
    BOOL bFoundRange = FALSE;

    // first find which range the number belongs to
    // (replace with binary search)
	int nStart = 0;
	int nEnd = pConvert->nDstRanges - 1;
	if (u <= pConvert->DstRanges[nEnd].end && 
		u >= pConvert->DstRanges[nStart].start)
	{
		for (; nStart <= nEnd && !bFoundRange;)
		{
			int n = nStart + (nEnd - nStart) / 2;
			if (u >= pConvert->DstRanges[n].start)
			{
				if (u <= pConvert->DstRanges[n].end)
				{
					start = pConvert->DstRanges[n].start;
					iIndx = pConvert->DstRanges[n].offset;
					bFoundRange = TRUE;
				}
				else
				{
					nStart = n+1;
				}
			}
			else
			{
				nEnd = n - 1;
			}

		}
	}

    if  (bFoundRange) 
	{
        // now convert the number
        u = pConvert->DstCodeRecs[iIndx+u-start].code;
		int nBytes = 0;
		char cByte1 = (char) (((u & 0xFF00) >> 8) & 0x00FF);
		if (cByte1)
		{
	        *lpszDst++ = cByte1;
			*nCount++;
			nBytes++;
		}
        *lpszDst = (char) (u & 0x00FF);
		nBytes++;
        return nBytes;
    }

    // we get here if we did not find the range or if the code is not found in
    // the range. in either case just output the low order byte and throw away
    // the high order byte

    *lpszDst = (char) (u & 0x00FF);
    *nCount += 1;
    return 1; // sorry, didn't find a 2 byte translattion
}


int ToMultiByteDest(char *lpszDst, gt_wchar_t u, int nCount, bool bMultiTableDst, CONVERT *pConvert)
{
    if  (bMultiTableDst) 
	{
        return ToMultiDst(lpszDst, u, &nCount, pConvert);
    }
    // translate to destination codepage
    BYTE a2 = (u <= END_URANGE_1) ? 
		pConvert->Uni_To_THIS_1 [u] : 
		(u >= BEG_URANGE_2 && u <= END_URANGE_2)
            ? pConvert->Uni_To_THIS_2 [u - BEG_URANGE_2]: (BYTE)NULL;

    if ((u == 0) || 
	(a2 != (BYTE)NULL)) 
	{
        *lpszDst = a2;
        ++(nCount);
        return 1;
    }
	return 0;
}


int ULAPI UL_WideCharToMultiByte(
	UINT CodePage,         // code page
	DWORD dwFlags,         // character-type options
	LPCWSTR lpWideCharStr,  // wide-character buffer
	int cchWideChar,        // size of buffer
	LPSTR lpMultiByteStr, // string to map
	int cbMultiByte,       // number of bytes in string
    LPSTR   lpDefaultChar,
    LPBOOL   lpUsedDefaultChar
)
{
	//If there is the code page for detection detect it
	GetRealCodepage(CodePage, lpMultiByteStr, cbMultiByte);
		
#ifdef _NORMALIZE
	BOOL bNormalize = dwFlags & MB_PRECOMPOSED;
#else
	BOOL bNormalize = 0;
#endif
	int nNormalizedLen = 0;
	gt_wchar_t* pStringNormalized = NULL;

	//if the destination length is 0, the caller wants to get 
	//just the length of the string in WCHARs
//	if(cbMultiByte == 0 /* && (!bNormalize) */)
//		return w_strlen(lpWideCharStr);

	e_ConveresionResult eResult;
	char* pString = NULL;
	BOOL bDeleteString = FALSE;
	BOOL bConvertAfter = FALSE;


	UINT OriginalCodePage = CodePage;
	int nLen = PrepareForWideToMulti(CodePage, lpWideCharStr,
			cchWideChar, &pString, &eResult);
	switch(eResult)
	{
		case eConvFailed:
			return 0;
		case eConvFinished:
			bDeleteString = TRUE;
			if(cbMultiByte > 0)
			{
				nLen = min((signed)nLen, cbMultiByte);
				memcpy((void *)lpMultiByteStr, pString, nLen);
			}
			if(bDeleteString)
			{
				if (pString)
					delete []  pString;
				bDeleteString = FALSE;
			}
			return nLen;
		case eConvForward:
			bDeleteString = TRUE;
			break;
		case eConvertAfter:
			pString = (LPSTR)lpWideCharStr;
			bConvertAfter =TRUE;
			break;
		case eConvRegular:
			pString = (LPSTR)lpWideCharStr;
			break;
		default : return FALSE;
	}
	if(nLen != 0)
		cchWideChar = nLen;
	int iResult = 0;
	CONVERT* pConvert = GetConversionRecord(CodePage);


	BOOL bForceDelete = FALSE;
	if (!cbMultiByte)
	{
		bForceDelete = TRUE;
		cbMultiByte = 2 * (cchWideChar != -1 ? cchWideChar : (w_strlen(lpWideCharStr) + 1)) + 10;
		lpMultiByteStr = new char[cbMultiByte];
	}


#if __TV_WINDOWS__
	if(pConvert->ConvertType == eOSConvert)
	{
		iResult = WideCharToMultiByte(CodePage, dwFlags, (gt_wchar_t *)pString,
			cchWideChar, lpMultiByteStr, cbMultiByte, lpDefaultChar, 
			lpUsedDefaultChar);
		if(bDeleteString)
		{
			if (pString)
				delete []pString;
			bDeleteString = FALSE;
		}
		
		if (bConvertAfter && CodePage!=OriginalCodePage)
		{
			LPSTR pTemp = new char[iResult]; 
			memcpy(pTemp, (void *)lpMultiByteStr, iResult);

			iResult = UL_MultiToMulti(CodePage, OriginalCodePage, 
				pTemp, iResult,
				lpMultiByteStr, 
				cbMultiByte);
			if (pTemp)
				delete []pTemp;
		}

		if (bForceDelete)
			if (lpMultiByteStr)
			delete [] lpMultiByteStr;

		return iResult;
	}
#endif  // __TV_WINDOWS__

	bool bMultiTable = pConvert->bMultiTable ? true : false;

	WORD u;
	LPWSTR  lpwszSrc = (LPWSTR)pString;
	LPSTR  lpszDst =  (LPSTR)lpMultiByteStr;
	int nin = 0, nCount = 0;
	BOOL bNullTermination = cchWideChar == -1 ? TRUE : FALSE;

    while (((!bNullTermination && (nin < cchWideChar)) ||  
			(bNullTermination && *lpwszSrc != L'\0')) &&  cbMultiByte - iResult > 0) 
	{
        u = *lpwszSrc++;
        ++nin;
        nCount = ToMultiByteDest(lpszDst, u,
			 cchWideChar ? cchWideChar - iResult : 0, bMultiTable, pConvert);
		lpszDst += nCount;
		iResult += nCount;

    }           // while
	if(bNullTermination && cbMultiByte - iResult >= 1)
	{
		*lpszDst = '\0';
		iResult ++;
	}
	if (bConvertAfter && CodePage!=OriginalCodePage)
	{
		LPSTR pTemp = new char[iResult]; 
		memcpy(pTemp, (void *)lpMultiByteStr, iResult);

		iResult = UL_MultiToMulti(CodePage, OriginalCodePage, 
			pTemp, iResult,
			lpMultiByteStr, 
			cbMultiByte);
		if (pTemp) 
			delete []pTemp;
	}

	if(bDeleteString && pString)
	{
		delete []pString;
		bDeleteString = FALSE;
	}

	if (bForceDelete)
		if (lpMultiByteStr)
		delete [] lpMultiByteStr;

    return iResult ;
}


#if OBSOLETE_CODE
/////////////////////////////////////////////////
			if(bNormalize)
			{
				nNormalizedLen = w_normalize((gt_wchar_t*)pString, 
					nLen /sizeof(gt_wchar_t), NULL, 0);
				pStringNormalized = new WCHAR[nNormalizedLen + 1];
				w_normalize((gt_wchar_t*)pString, 
					nLen /sizeof(gt_wchar_t), pStringNormalized, nNormalizedLen + 1);
				nLen = (nNormalizedLen + 1) * sizeof(gt_wchar_t);
				if(bDeleteString)
				{
					if (pString) 
						delete [] pString;
					bDeleteString = FALSE;
				}
				pString = (char*)pStringNormalized;
			}


//////////////////////////////////////
			if(pStringNormalized)
			{
				delete [] pStringNormalized;
				pStringNormalized = NULL;
			}
#endif /* 0 */
