/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifdef __TV_WINDOWS__
#include "windows.h"
#endif

#include "gt_unicode.h"
#include <string.h>
#include "gt_unidef.h"

//Finds nth char, starting from 0
int UTF8_NthChar (const char *ptr, int n)
{
	int len, j;
	for(len = 0, j = 0; ptr[j]; j++)
		if (((unsigned char)ptr[j] >> 6) != 2)
		{
			len++;
			if (len >= n)
				return  j;
		}

	return j;
}

//Length of UTF8 string in characters
int UTF8_Length(const char *ptr)
{
   int len;
	for(len = 0; *ptr; ptr++)
		if (((unsigned char)(*ptr) >> 6) != 2)
			len++;

	return len;
}

int ANSIToUCS2(UINT CodePage, const char* pszA, int countA, gt_wchar_t* pszW, int countW)
{
	return UL_MultiByteToWideChar(CodePage, 0,
		pszA, countA, pszW, countW);
}

int UCS2ToANSI(UINT CodePage, const gt_wchar_t* pszW, int countW, char* pszA, int countA)
{
	return UL_WideCharToMultiByte(CodePage, 0,
		pszW,	countW, pszA, countA, NULL, NULL);
}

int ANSIToUTF8(UINT CodePage, const char* pszA, int countA, char* pszUTF, int countUTF)
{
	return ANSIToANSI(CodePage, CP_UTF8, pszA, countA, pszUTF, countUTF);
}

int UTF8ToANSI(UINT CodePage, const char* pszUTF, int countUTF, char* pszA, int countA)
{
	return ANSIToANSI(CP_UTF8, CodePage, pszUTF, countUTF, pszA, countA);
}

int ANSIToANSI(UINT CodePageFrom, UINT CodePageTo, const char* pszFrom, int countFrom, char* pszTo, int countTo)
{
	int  countWide = (countFrom != -1) ? countFrom : strlen(pszFrom) + 1;
	gt_wchar_t *pszWide = new gt_wchar_t[countWide];

	int nWide = ANSIToUCS2(CodePageFrom,
		pszFrom, countFrom, pszWide, countWide);
	int nRet = UCS2ToANSI(CodePageTo,
		pszWide, nWide, pszTo, countTo);

	delete [] pszWide;
	return nRet;
}


#ifdef __TV_UNIX__
unsigned int GetACP ( void )
{
  // Chaim + Daniel (2.1.2002) - Use static variable to avoid repeated calls to setLocale
  static unsigned acp =CP_DEFAULT ;
  static BOOL bInitialized =false ;

  if( bInitialized) 
       return ( acp );

  bInitialized = true;

//  if ( acp != CP_DEFAULT)
    return ( acp );

	char *localeStr = setlocale(LC_ALL, "");
	//char *localeStr = setlocale(LC_MESSAGES, "");

	if (!localeStr)	
		return (0);
	char * dot;
	if (dot = strchr(localeStr,'.'))
	 {
	 	localeStr = dot+1;
	 }
	int count = sizeof(lcname2id_tbl)/sizeof(lcname2id_t)-1;
	for (int i = 0; i < count; i++)
	{
		if (!strcasecmp( localeStr, lcname2id_tbl[i].lc_name))
		{
			return (acp = lcname2id_tbl[i].lc_id);
// 02.01.2002 Old code
//			return (lcname2id_tbl[i].lc_id);
		}
	}
	return (acp = CP_DEFAULT );
// 02.01.2002 Old code
//	return (CP_DEFAULT );
	
}
#else//UNIX

#define _NBSP	0xa0
#define _SP		0x20
void RemoveInvalidHTMLCodes(gt_wchar_t* pStr)
{
	if (!pStr)
		return;

	while (*pStr)
	{
		if (*pStr == _NBSP)
			*pStr = _SP;
		pStr++;
	}
}

int ConvertDialogDimension(HWND hWnd, int Dimension, bool bHDirection, bool bToDlgUnits)
{
	RECT r;
	r.left = Dimension;
	r.top = Dimension;
	r.bottom = Dimension;
	r.right = Dimension;

	MapDialogRect(hWnd, &r);

	if(bHDirection)
	{
		return r.left;
	}
	else
	{
		return r.top;
	}
}

#endif
