/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  


#ifndef dJsd
#define dJsd

//#include <algorithm>

#include "rvjsapi.h"
#include "rvjsdbgapi.h"
#include "rvjsdebug.h"

#include <thr.h>

#include "rvjs.h.h"
#include "rvjsgeneral.h"
#include "njs.h"
#include "rvjsreport.h"

extern "C" {
	#ifdef __TV_WINDOWS__
  __declspec(dllexport) void JSD_SetDebuggerStatus(JSBool dFlag);
  #else
  DllExport void JSD_SetDebuggerStatus(JSBool dFlag);
  #endif
}

// namespace {
	// enum tErrorSeverety { sErrorSeveretyNote, sErrorSeveretyWarning, sErrorSeveretyNormal, sErrorSeveretyFatal };

 	enum tInterruptionControl { sInterruptionControlNone, sInterruptionControlStep, sInterruptionControlAbort };
 	enum tStepping { sSteppingLine, sSteppingInto, sSteppingOver, sSteppingOut, sSteppingNone };
	enum tNormalDMHookReturn { sNormalDMHookReturnContinue, sNormalDMHookReturnAbort };
	enum tThrowDMHookReturn { sThrowDMHookReturnApply, sThrowDMHookReturnClear, sThrowDMHookReturnAbort };
	enum tInvocationDMHookType { sInvocationDMHookTypeEnter, sInvocationDMHookTypeExit };

//	}

namespace nJs {








class	iContext
{
public:
	virtual JSDContext* fGetJSDC() const = 0;
	virtual JSContext* fGetDumbJSContext() const = 0;
};


template<	typename rcType_ >
class ecHandle
{
public:
	typedef rcType_ rcType;
	typedef ecHandle< rcType > jcHandle;

	explicit ecHandle< rcType >(
		rcType Value )
	:
		m(Value)
	{}

	rcType fGet() const {
		return m;
	}
private:
	rcType m;
};

template<	typename rcElementType_ >
class	ecHandlePointer:
	public ecHandle< rcElementType_* >
{
public:
	typedef rcElementType_ rcElementType;
	typedef ecHandlePointer< rcElementType > jcHandlePointer;
// Sun compiler does not inherit typedefs.
#ifdef sun
	typedef ecHandle< rcElementType* > jcHandle;
	typedef typename jcHandle::rcType rcType;
#endif

	explicit ecHandlePointer< rcElementType >( rcType Value )
	:
		jcHandle( Value )
	{}

	bool fIsValid() {
		return (0!=fGet());
	}

};


class iScript
{
protected:
	explicit iScript();
public:
	virtual ~iScript();

	const char* fGetName() const;
	iFile* fGetFile() const;
	tLineNumber fGetBaseLine() const;
	tLineNumber fGetLineExtent() const;
	const char* fGetFunctionName() const;

	virtual JSDScript* vGetJSDScript_iScript() const = 0;
		JSDScript* fGetJSDScript() const { return vGetJSDScript_iScript(); }

	virtual JSDContext* vGetJSDC_iScript() const = 0;
		JSDContext* fGetJSDC() const { return vGetJSDC_iScript(); }

};


class iScriptHookable:
	public iScript
{
friend class iSpace;
protected:
	explicit iScriptHookable();
	~iScriptHookable();
};

class cScriptHookable:
	public ecHandlePointer< JSDScript >,
	public iScriptHookable
{
public:
	explicit cScriptHookable( iContext&, rcType );
	~cScriptHookable();


private:
	iContext& mContext;

	JSDScript* vGetJSDScript_iScript() const;
	JSDContext* vGetJSDC_iScript() const;
};



class	iDMThreadState:
	public ecHandlePointer< JSDThreadState >
{
	friend class iValue;
	friend class iProperty;
protected:
	explicit iDMThreadState( iContext&, rcType );
	virtual ~iDMThreadState();

public:
	tStackFrameIndex fGetStackFramesCount() const;
	iStackFrameLevel* fGetTopStackFrame();
	iValue* fGetException() const;

public:
	mutable iContext& mContext;
};


class cDMThreadState:
	public iDMThreadState
{
public:
	explicit cDMThreadState( iContext&, rcType );
	~cDMThreadState();
};


class iStackFrame:
	public iScript
{
#ifdef __TV_UNIX__
public:
#endif
	typedef JSDStackFrameInfo* cHandle;
protected:
	explicit iStackFrame( iDMThreadState& );
public:
	virtual ~iStackFrame();

	iDMThreadState& fGetThreadState() const;
	virtual JSDStackFrameInfo* vGet_iStackFrame() const = 0;
		JSDStackFrameInfo* fGet() const { return vGet_iStackFrame(); }

	tLineNumber fGetCurrentLineNumber() const;

	iValue* fGetCallObject() const;
	iValue* fGetScopeChain() const;
	iValue* fGetThisValue() const;

	iValue* fEvaluateScript( const char* Bytes, int Length ) const;

public:
	mutable iDMThreadState& mThreadState;

private:
	JSDScript* vGetJSDScript_iScript() const;
	JSDContext* vGetJSDC_iScript() const;
};


class	iStackFrameLevel:
	public iStackFrame
{
public:
	explicit iStackFrameLevel( iDMThreadState&, cHandle );

	iStackFrameLevel* fGetNext() const;

	JSDStackFrameInfo* vGet_iStackFrame() const;

private:
	JSDStackFrameInfo* mHandle;
};


class cIteratableStackFrame:
	public iStackFrame
{
public:
	typedef JSDStackFrameInfo* cHandle;
public:
	explicit cIteratableStackFrame( iDMThreadState& );
	~cIteratableStackFrame();

	void fNext();
	bool fIsValid();

protected:
	JSDStackFrameInfo* vGet_iStackFrame() const;

public:
	cHandle mHandle;
};


class iValue:
	public ecHandlePointer< JSDValue >
{
protected:
	explicit iValue( const iContext&, const iDMThreadState&, const rcType );
public:
	virtual ~iValue();

	const char* fToString();

	tPropertyIndex fGetPropertiesCount();

	iValue* fGetParent();
	iValue* fGetProto();
	iValue* fGetCtor();

	const char* fGetClassName();

	bool fIsObject();
	bool fIsPrimitive();

	bool fIsNull();
	bool fIsVoid();
	bool fIsBoolean();
	bool fIsInt();
	bool fIsDouble();
	bool fIsNumber();
	bool fIsString();

	bool fIsFunction();
	bool fIsNative();

	const char* fGetFunctionName();



public:
	mutable iContext& mContext;
	mutable iDMThreadState mThreadState;
};


class cValueRooted:
	public iValue
{
public:
	explicit cValueRooted( const iContext&, const iDMThreadState&, rcType );
};

class cValueHold:
	public iValue
{
public:
	explicit cValueHold( const iContext&, const iDMThreadState&, rcType );
public:
	virtual ~cValueHold();
};


class cPropertiesIter
{
public:
	explicit cPropertiesIter( iValue& );
	~cPropertiesIter();

	// first time returns first, after last time returns null
	iProperty* fGetNext();

private:
	iValue& mValue;
	JSDProperty* mIter;

};

class iProperty:
	public ecHandlePointer< JSDProperty >
{
protected:
	explicit iProperty( iContext&, iDMThreadState&, const rcType );
public:
	virtual ~iProperty();

	bool fIsNameString() const;
	const char* fGetNameString() const;

	bool fIsNameInt() const;
	tPropertyIndex fGetNameInt() const;

	iValue* fGetValue() const;

	bool fIsEnumerate();
	bool fIsReadOnly();
	bool fIsPermanent();
	bool fIsAlias();
	bool fIsArgument();
	bool fIsVariable();
	bool fIsHinted();

	tPropertyIndex fGetSlotIndex() const;

private:
	mutable iContext& mContext;
	mutable iDMThreadState mThreadState;
	JSDValue* mName;
};

class cProperty:
	public iProperty
{
public:
	explicit cProperty( iContext&, iDMThreadState&, const rcType );
};


class cCallChain {
public:
	enum tCompareResult { sEqual, sCaller, sCallee, sDisjoin };

	explicit cCallChain();
	~cCallChain();

	void fInit( JSDContext*, JSDThreadState*, JSDStackFrameInfo* );
	void fFinal();

	void fMove( cCallChain& );

	JSDScript* fGetTopScript();

	tCompareResult fCompare(const cCallChain&) const;


private:
	JSDContext* mJSDC;
	JSDScript** mChain;
	int mDepth;
};





class iUi {

public:

	virtual void fScriptHook( iScriptHookable&, bool Creating ) = 0;

	virtual tDebugReporterReturn vDebugReporter_iUi( iReportThreadState&, iReportInfo& ) = 0;
	virtual void fReportDMHook( tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState& ) = 0;

	virtual tNormalDMHookReturn vInterruptDMHook_iUi( tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState& ) = 0;
		tNormalDMHookReturn fInterruptDMHook( tInterruptionControl& _X3, bool& _X2, tStepping _X0, iDMThreadState& _X1) {return vInterruptDMHook_iUi(_X3,_X2,_X0,_X1);}


	virtual tNormalDMHookReturn fBreakPointDMHook( tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState& ) = 0;
	virtual tNormalDMHookReturn fKeywordDMHook( tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState& ) = 0;

	virtual tThrowDMHookReturn fThrowDMHook( tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState& ) = 0;

	virtual bool fInvokeDMHook( bool rIsFunction, tInvocationDMHookType, iDMThreadState* ) = 0;

};



class iDebugger {

public:
	virtual void vInterrupt_iDebugger() = 0;
		inline void fInterrupt() {vInterrupt_iDebugger();}

	virtual void vAbort_iDebugger() = 0;
		inline void fAbort() {vAbort_iDebugger();}

	virtual tLineNumber fEnableBreakPoint( iScriptHookable&, tLineNumber ) = 0;
	virtual tLineNumber fToggleBreakPoint( iFile&, tLineNumber, bool State ) = 0;

};


class iExecutor
{
public:

	virtual void fSetRuntime( JSRuntime* ) = 0;

	virtual void vInit_iExecutor() = 0;
		inline void fInit() {vInit_iExecutor();}

	virtual void vFinal_iExecutor() = 0;
		inline void fFinal() {vFinal_iExecutor();}

	virtual void fSignJSContext( JSContext* Cx ) = 0;
};



class iSpace:
	public iExecutor,
	public iDebugger,
	RV_INHERIT_PROTECTED iContext,
	RV_INHERIT_PROTECTED iFileMap
{

friend class cInterruptionControlCommit;
public:

	explicit iSpace( bool IsDebugger, JSRuntime* = 0 ); // the instance is constantly enabled or disabled
	virtual ~iSpace();

// iExecutor
	void fSetRuntime( JSRuntime* );
	void vInit_iExecutor();
	void vFinal_iExecutor();
	void fSignJSContext( JSContext* Cx );

// iDebugger
	void vInterrupt_iDebugger();
	void fAbort_iDebugger();

	tLineNumber fEnableBreakPoint( iScriptHookable&, tLineNumber );
	tLineNumber fToggleBreakPoint( iFile&, tLineNumber, bool State );

protected:

	bool fIsDebugger() const;

	JSDContext* fGetJSDC() const;
	JSContext* fGetDumbJSContext() const;

	virtual iUi& vGetUi_iSpace() = 0;
		iUi& fGetUi() {return vGetUi_iSpace();}

	bool fSetExecutionHook( JSDScript*, jsuword );

public: //for static funcs.
	void fScriptHook(JSDContext*,JSDScript*,JSBool);


	uintN fCallbackErrorReporter(JSDContext*,JSContext*, const char*, JSErrorReport*);
	uintN fCallbackExecutionHook(JSDContext*,JSDThreadState*,uintN,jsval*);

	JSBool fCallbackTopLevelHook(JSDContext*,JSDThreadState*,uintN type);
	JSBool fCallbackFunctionHook(JSDContext*,JSDThreadState*,uintN type);

private:
	bool mIsDebugger;
	JSRuntime* mRuntime;
	JSDContext* mJSDC;
	JSContext* mDumbJSContext;

	jsuword mHookPositionPC;
	tLineNumber mHookPositionLineNumber;
	int mHookPositionStackDepth;
	int mInvokeEnterFlag;
	bool mInvokeIsFunction;

	mutable RVMutexLock* mExecutionLock;

	// mInterruptionControl is synchronized:
	tInterruptionControl fGetInterruptionControl() const;
	void fSetInterruptionControl( tInterruptionControl );
	volatile tInterruptionControl mInterruptionControl;
	mutable RVMutexLock* mInterruptionControlLocker;


	void* fThisVoid();

	void fSetHooks();

	static uintN fsJsdNormalDMHookReturn( tNormalDMHookReturn );
	static uintN fsJsdThrowDMHookReturn( tThrowDMHookReturn );

private: // recursive modes
	int mDMDepth;
	void fEnterDM();
	void fExitDM();

	int mReportDepth;
	void fEnterReport();
	void fExitReport();

	bool fCanDM() const;
	bool fCanReport() const;

};



class iInterruptionControlCommit {
public:
	virtual void fSet( tInterruptionControl ) = 0;
};

class cInterruptionControlCommit: public iInterruptionControlCommit {
public:
	explicit cInterruptionControlCommit( iSpace& );
	void fSet( tInterruptionControl );

private:
	iSpace& mSpace;
};


class iSpaceUi:
	public iSpace,
	public iUi
{

public:
	explicit iSpaceUi( bool, JSRuntime* = 0 );

protected:
	iUi& vGetUi_iSpace();
};



class	cSpaceCompose:
	public iSpace {

public:
	explicit cSpaceCompose( iUi&, bool, JSRuntime* = 0 );

protected:
	iUi& vGetUi_iSpace();

private:
	iUi& mUi;

};

class iVirtualLoadEngine {
public:
  explicit iVirtualLoadEngine() {}
  virtual ~iVirtualLoadEngine() {}
  bool fShouldReportInternalCSJSErrors(JSContext* Cx){
    return vShouldReportInternalCSJSErrors(Cx);
  }

private:
  virtual bool vShouldReportInternalCSJSErrors(JSContext* Cx) = 0;
};


} // nJs



namespace {

using nJs::iContext;
//using nJs::ecHandle;
//using nJs::ecHandlePointer;
using nJs::iScript;
using nJs::iScriptHookable;
using nJs::cScriptHookable;
using nJs::iDMThreadState;
using nJs::cDMThreadState;
using nJs::iStackFrame;
using nJs::iStackFrameLevel;
using nJs::cIteratableStackFrame;
using nJs::iValue;
using nJs::cValueRooted;
using nJs::cValueHold;
using nJs::cPropertiesIter;
using nJs::iProperty;
using nJs::cProperty;
using nJs::iUi;
using nJs::iDebugger;
using nJs::iSpace;
using nJs::iInterruptionControlCommit;
using nJs::cInterruptionControlCommit;
using nJs::iSpaceUi;
using nJs::cSpaceCompose;
using nJs::iVirtualLoadEngine;


}


DllExport void SetVirtualLoadEngine(iVirtualLoadEngine*);



#endif


