/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#ifndef dRVJSGeneral
#define dRVJSGeneral

#include "rvjsapi.h"
#include "rvjsdbgapi.h"

#include "wlerrorlib.h"
#include "tracedll.h"

#include "rvjs.h.h"


namespace nJs {




	class cContextThreadAcquire
	{
	public:
		explicit cContextThreadAcquire( JSContext* );
		~cContextThreadAcquire();
	private:
		JSContext* mCx;
	};


	class cContextThreadRelease
	{
	public:
		explicit cContextThreadRelease( JSContext* );
		~cContextThreadRelease();
	private:
		JSContext* mCx;
	};


	class nZTSEncription {

	private:
		explicit nZTSEncription();

		static const char msCodeBytes[3];
		static const int msCodeLength;

		static int fsPutCode( char* );

		static int fsCheckCode( const char* );


		static const int msPointerEncriptionMaxLength;

	public:

		//typedef char tZTSPointer[ msPointerEncriptionMaxLength ];
		typedef char tZTSPointer[ 14 ];


		static void fsPointerToZTS( const void*, char* );
		static bool fsZTSToPointer( const char*, void*& );
	};

	template< typename rcType_ >
	class ecPointerToZTS 
	{
		typedef rcType_ rcType;
		typedef ecPointerToZTS< rcType > jcPointerToZTS;
	public:
		
		explicit ecPointerToZTS< rcType >( rcType* r ) {
			nZTSEncription::fsPointerToZTS( reinterpret_cast<void*>(r), mZTS );
		}

		explicit ecPointerToZTS< rcType >( rcType& r ) {
			nZTSEncription::fsPointerToZTS( reinterpret_cast<void*>(&r), mZTS );
		}

		operator const char*()
		{
			return mZTS;
		}

	private:
		nZTSEncription::tZTSPointer mZTS;
		
	};


        template< typename rcType_ >
	  class ecfsZTSToPointer 
	  {
	  public:
	  typedef rcType_ rcType;
	  typedef ecfsZTSToPointer< rcType > jcfsZTSToPointer;

	  static rcType* _(
		const char* rZTS )
	{
		void* aX0;
		rcType* rtn;
		if( nZTSEncription::fsZTSToPointer( rZTS, aX0 ) ) {
			rtn = reinterpret_cast<rcType*>(aX0);
                   #if ! defined (__PPC750__ ) && !defined( __TV_linux__ )
			WlAssert(rtn);
                   #else
                        //@@Peter@@Sergei - WlAssert(rtn); - cross GCC does not eat this
                        assert(rtn);
                   #endif
		}	else { 
			rtn = 0;
		}
		return rtn;
	}
	};

	class cFileRef
	{
	public:
		explicit cFileRef();
		explicit cFileRef( iFile& );
		explicit cFileRef( cFileRef& );
		~cFileRef();
		cFileRef& operator=( const cFileRef& );	

	public:
		iFile& fGet() const;

	private:
		iFile* mFile;
	};
	iFile* ZTSEncriptionToFile( const char* );
	
	class iFile {
		friend class cFileRef;
	protected:
		explicit iFile();
		virtual ~iFile();

	public:
		virtual const char* fGetName() const = 0;
		void fIncRef();
		bool fDecRef();
		virtual void vPick() = 0;
		virtual bool vDrop() = 0;

	private:
		int mRefN;

	};

	class iFileMap {
	public:
		virtual void fClearFile( iFile& ) {};
	};


	RVCString fsJSCodeString( const char* rCodeBytes, int rCodeLength );





class cRuntime
{
public:
	explicit cRuntime();
	virtual ~cRuntime();

	void fInit( int rSize );
	void fFinal();

	JSRuntime* fGet();

	static void fsNewScriptHook(	JSContext*, const char* rFilename, uintN rLineno,	JSScript*, JSFunction* );
	static void fsDestroyScriptHook( JSContext*, JSScript* );

private:
	JSRuntime* mJSRuntime;
};



 	} // nJs



namespace {

	
	using nJs::tLineNumber;
	using nJs::tColumn;


	using nJs::cFileRef; 
	using nJs::iFile; 
	using nJs::iFileMap; 
	using nJs::ecPointerToZTS;
	using nJs::ecfsZTSToPointer;
	using nJs::cRuntime;

	}



#endif // dRVJSGeneral
