/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#ifndef dRVJSReport
#define dRVJSReport

#pragma warning( disable : 4250 )

#include "rvjsapi.h"
#include "rvjsdbgapi.h"

#include "wlerrorlib.h"

#include "rvjs.h.h"
#include "rvjsgeneral.h"
#include "njs.h"


	enum tDebugReporterReturn { sDebugReporterReturnApply, sDebugReporterReturnDM, sDebugReporterReturnContinue, sDebugReporterReturnClear };


namespace nJs {



class iReportThreadState
{
protected:
	explicit iReportThreadState();
public:
	virtual ~iReportThreadState();

public:
	virtual const char* vGetName() const = 0;
		const char* fGetName() {return vGetName();}

	virtual iFile* vGetFile() const = 0;
		iFile* fGetFile() {return vGetFile();}

	virtual tLineNumber vGetLineNumber() const = 0;
		tLineNumber fGetLineNumber() {return vGetLineNumber();}

	virtual tColumn vGetColumn() const = 0;
		tColumn fGetColumn() {return vGetColumn();}

	virtual const char* vGetLine() const = 0;
		const char* fGetLine() {return vGetLine();}

};

class cReportThreadState:
	public iReportThreadState
{
public:
	explicit cReportThreadState( const JSErrorReport& );
	virtual ~cReportThreadState();

	const char* vGetName() const;
	iFile* vGetFile() const;
	tLineNumber vGetLineNumber() const;
	tColumn vGetColumn() const;
	const char* vGetLine() const;

protected:
	const char* mName;
	iFile* mFile;
	tLineNumber mLineNumber;
	tColumn mColumn;
	const char* mLine;
};


class coReportThreadState:
	public iReportThreadState
{
public:
	explicit coReportThreadState();
	virtual ~coReportThreadState();

	void fInit( const JSErrorReport& );

	bool fIsValid();

	const char* vGetName() const;
	iFile* vGetFile() const;
	tLineNumber vGetLineNumber() const;
	tColumn vGetColumn() const;
	const char* vGetLine() const;

protected:
	bool mIsValid;
	RVCString mName;
	iFile* mFile;
	tLineNumber mLineNumber;
	tColumn mColumn;
	RVCString mLine;
};



class iEmbedderCallContext 
{

protected:
	explicit iEmbedderCallContext();
	virtual ~iEmbedderCallContext();

public: 
	void fReport( iReportInfo& _) {vReport_iEmbedderCallContext(_);}

protected: 
	virtual void vReport_iEmbedderCallContext( iReportInfo& ) = 0;
};



class cJSEmbedderCallContext: public virtual iEmbedderCallContext 
{

public:
	explicit cJSEmbedderCallContext( JSContext* );
	~cJSEmbedderCallContext();

	bool fGetJSReturn() const;

protected: 
	void vReport_iEmbedderCallContext( iReportInfo& );

private:

	JSContext* mContext;
	bool mJSReturn;

};




class iReportInfo 
{
public:
	virtual ~iReportInfo();


	virtual void vApply_iReportInfo( iReportCommit& , bool CommitAndSetReturnValue) = 0;
		void fApply( iReportCommit& _X0, bool CommitAndSetReturnValue ) { vApply_iReportInfo( _X0, CommitAndSetReturnValue ); };

	virtual bool vIsStop() const = 0;
		bool fIsStop() const {return vIsStop();}
};
class ivReportInfo: public virtual iReportInfo {};


class iDebugExecReportInfo: public iReportInfo 
{
public:
	explicit iDebugExecReportInfo();

	void vApply_iReportInfo( iReportCommit&, bool CommitAndSetReturnValue );

	virtual tDebugReporterReturn vApplyDebug( cJSDReportCommit& ) = 0;
	virtual void vApplyJSExec( iJSExecReportCommit& ) = 0;
};


class iPlainReportInfo: public iDebugExecReportInfo 
{
public:
	explicit iPlainReportInfo();

	tDebugReporterReturn vApplyDebug( cJSDReportCommit& );
	void vApplyJSExec( iJSExecReportCommit& );

public:
	bool mDone;
};


class cInternalReportInfo:
	public iPlainReportInfo 
{

public:

	explicit cInternalReportInfo( const char*, JSErrorReport& );

	const char* fGetMessage() const;
	bool fIsWarning() const;
	bool fIsException() const;
	bool fIsStrict() const;
	uintN fGetErrorNumber() const;
	JSErrorReport& fGetJSReport() const; /* Deprecated, use only when must, and for ReportInfo stuff, not ReportThreadState stuff*/

private:
	const char* mMessage;
	bool mIsException;
	bool mIsStrict;
	bool mIsWarning;
	uintN mErrorNumber;
	JSErrorReport& mJSReport;

	bool vIsStop() const;
};


class iReportCommit 
{
public:
	virtual void vCommit_iReportCommit(bool CommitAndSetReturnValue) = 0;
		void fCommit(bool CommitAndSetReturnValue) { vCommit_iReportCommit(CommitAndSetReturnValue); }

private:
//	virtual void vDebugReporter_iReportCommit( iReportInfo& ) = 0;
//		inline void fDebugReporter( iReportInfo& _ ) {vDebugReporter_iReportCommit(_);}
};



class iExecReportCommit: public virtual iReportCommit 
{
public:
	virtual iJSEmbedContext& vGetEmbedContext() = 0;
		iJSEmbedContext& fGetEmbedContext() { return vGetEmbedContext(); }
};




class iJSReportCommit: public virtual iReportCommit {

public:
	explicit iJSReportCommit( const char* rMessage, JSErrorReport& );

	void vCommit_iReportCommit(bool CommitAndSetReturnValue);

	iReportThreadState& fGetThreadState();
	JSErrorReport& fGetJSReport();

private:
	const char* mMessage;
	JSErrorReport& mErrorReport;
	cReportThreadState mThreadState;

};


class iJSExecReportCommit:
	public iExecReportCommit,
	public iJSReportCommit 
{

public:
	explicit iJSExecReportCommit( JSContext*, const char* rMessage, JSErrorReport* );

	iJSEmbedContext& vGetEmbedContext();

private:
	JSContext* mCx;
};



class cJSDReportCommit:
	public virtual iJSReportCommit 
{
public:
	explicit cJSDReportCommit( iUi&, const char* rMessage, JSErrorReport* );
	uintN fGetReturnValue() const;


	void fSetReturnValue( tDebugReporterReturn );

	iUi& fGetUi() const;

private:
	uintN mReturnValue;
	iUi& mUi;

};







 	} // nJs



namespace {

	using nJs::iReportThreadState;
	using nJs::cReportThreadState;
	using nJs::coReportThreadState;

	using nJs::iEmbedderCallContext;
	using nJs::cJSEmbedderCallContext;
	
	using nJs::iReportInfo;
	using nJs::ivReportInfo;
	using nJs::iDebugExecReportInfo;
	using nJs::iPlainReportInfo;
	using nJs::cInternalReportInfo;

	using nJs::iReportCommit;
	using nJs::iExecReportCommit;
	using nJs::iJSReportCommit;
	using nJs::iJSExecReportCommit;

	using nJs::cJSDReportCommit;

	}



#endif // dRVJSReport
