/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  


#include "rvjsreport.h"
#include "tracedll.h"

#include "njsd.h"
namespace nJs {


iReportThreadState::iReportThreadState()
{}


iReportThreadState::~iReportThreadState() {}


cReportThreadState::cReportThreadState( 
	const JSErrorReport& Report )
:
	iReportThreadState(),
	mName( Report.filename ),
	mFile( NULL ),
	mLineNumber( (tLineNumber)(Report.lineno)-1 ),
	mColumn( ((const char*)Report.tokenptr - (const char*)Report.linebuf) / sizeof(char) ),
	mLine( Report.linebuf )
{
	if( iFile* aFile = ZTSEncriptionToFile( Report.filename ) ) {
		mFile = aFile;
	}
}

cReportThreadState::~cReportThreadState() {}


const char* cReportThreadState::vGetName() const
{
	return mName;
}

iFile* cReportThreadState::vGetFile() const
{
	return mFile;
}

tLineNumber cReportThreadState::vGetLineNumber() const
{
	return mLineNumber;
}

tColumn cReportThreadState::vGetColumn() const
{
	return mColumn;
}

const char* cReportThreadState::vGetLine() const
{
	return mLine;
}



coReportThreadState::coReportThreadState()
:
	iReportThreadState(),
	mIsValid(false),
	mName(),
	mFile(NULL),
	mLineNumber(),
	mColumn(),
	mLine()
{}

coReportThreadState::~coReportThreadState() {}

void coReportThreadState::fInit(
	const JSErrorReport& rReport )
{
	if( rReport.filename ) {
		mName = rReport.filename;
	}
	if( iFile* aFile = ZTSEncriptionToFile( rReport.filename ) ) {
		mFile = aFile;
	}
	mLineNumber = (tLineNumber)(rReport.lineno)-1;
	mColumn = ((const char*)rReport.tokenptr - (const char*)rReport.linebuf) / sizeof(char);
	if( rReport.linebuf ) {
		mLine = rReport.linebuf;
	}
	mIsValid = true;
}

bool coReportThreadState::fIsValid()
{
	return mIsValid;
}

const char* coReportThreadState::vGetName() const
{
	return mName.data();
}

iFile* coReportThreadState::vGetFile() const
{
	return mFile;
}

tLineNumber coReportThreadState::vGetLineNumber() const
{
	return mLineNumber;
}

tColumn coReportThreadState::vGetColumn() const
{
	return mColumn;
}

const char* coReportThreadState::vGetLine() const
{
	return mLine.data();
}







iEmbedderCallContext::iEmbedderCallContext() {}
iEmbedderCallContext::~iEmbedderCallContext() {}


cJSEmbedderCallContext::cJSEmbedderCallContext(
	JSContext* Context ): 
	mContext(Context),
	mJSReturn(true) {}

cJSEmbedderCallContext::~cJSEmbedderCallContext() {}



void cJSEmbedderCallContext::vReport_iEmbedderCallContext(
	iReportInfo& Info ) {

	nZTSEncription::tZTSPointer EncriptedInfoPointer;
	nZTSEncription::fsPointerToZTS( reinterpret_cast<void*>(&Info), EncriptedInfoPointer );

	if(Info.fIsStop()) {
		JS_ReportError( mContext, "%s", EncriptedInfoPointer );
		mJSReturn = false;
		}
	else {
		JS_ReportWarning( mContext, "%s", EncriptedInfoPointer );
		}
	
	}


bool cJSEmbedderCallContext::fGetJSReturn() const {
	return mJSReturn;
	}



iReportInfo::~iReportInfo() {}


iDebugExecReportInfo::iDebugExecReportInfo()
{}

void iDebugExecReportInfo::vApply_iReportInfo( iReportCommit& rCommit_, bool CommitAndSetReturnValue )
{
	if( cJSDReportCommit* aCommit = dynamic_cast<cJSDReportCommit*>(&rCommit_) ) {
    tDebugReporterReturn aReporterReturn;

    // Chaim - if we want to bypass reporting of internal errors,
    // we still need to tell SpiderMonkey to propagate the exception
    if (CommitAndSetReturnValue){
		  aReporterReturn = vApplyDebug( *aCommit );
    }
    else {
      aReporterReturn = sDebugReporterReturnApply;
    }
	  aCommit->fSetReturnValue( aReporterReturn );
	} else if( iJSExecReportCommit* aCommit = dynamic_cast<iJSExecReportCommit*>(&rCommit_) ) {
    // Chaim - do the same thing in LG
    if (CommitAndSetReturnValue){
		vApplyJSExec( *aCommit );
    }
	} else {
		WlAssert(0);
	}
}


iPlainReportInfo::iPlainReportInfo()
:
	mDone(false)
{}


tDebugReporterReturn iPlainReportInfo::vApplyDebug( cJSDReportCommit& rCommit )
{
	WlAssert(!mDone);
	tDebugReporterReturn rtn = rCommit.fGetUi().vDebugReporter_iUi( rCommit.fGetThreadState(), *this );
	mDone=true;
	return rtn;
}


void iPlainReportInfo::vApplyJSExec( iJSExecReportCommit& rCommit )
{
	if(!mDone) {
		rCommit.fGetEmbedContext().fJSExecReportCommit( rCommit, *this );
		mDone=true;
	}
}


		

cInternalReportInfo::cInternalReportInfo(
	const char* rMessage,
	JSErrorReport& rReport )
:
	mMessage(rMessage),
	mIsWarning(JSREPORT_IS_WARNING(rReport.flags)),
	mIsException(JSREPORT_IS_EXCEPTION(rReport.flags)),
	mIsStrict(JSREPORT_IS_STRICT(rReport.flags)),
	mErrorNumber(rReport.errorNumber),
	mJSReport(rReport)
{}


const char* cInternalReportInfo::fGetMessage() const {
	return mMessage;
	}

bool cInternalReportInfo::fIsException() const {
	return mIsException;
	}

bool cInternalReportInfo::fIsWarning() const {
	return mIsWarning;
	}

bool cInternalReportInfo::fIsStrict() const {
	return mIsStrict;
	}

uintN cInternalReportInfo::fGetErrorNumber() const 
{
	return mErrorNumber;
}

JSErrorReport& cInternalReportInfo::fGetJSReport() const
{
	return mJSReport;
}


bool cInternalReportInfo::vIsStop() const {
	return !fIsWarning();
}


iJSReportCommit::iJSReportCommit(
	const char* rMessage,
	JSErrorReport& rErrorReport ) 
:
	mMessage(rMessage),
	mErrorReport(rErrorReport),
	mThreadState(rErrorReport)
{}


void iJSReportCommit::vCommit_iReportCommit(bool CommitAndSetReturnValue) 
{

	void* aPointer;
	if( nZTSEncription::fsZTSToPointer( mMessage, aPointer ) ) {
		iReportInfo* aInfo = reinterpret_cast<iReportInfo*>(aPointer);
		WlAssert(aInfo);
		aInfo->fApply( *this, true );
	} else { 
    // @@Chaim - TBD - what to do about MAJOR internal errors like missing files?
	  cInternalReportInfo aInfo( mMessage, mErrorReport );
	  aInfo.fApply( *this, CommitAndSetReturnValue );
  }
}

iReportThreadState& iJSReportCommit::fGetThreadState()
{
	return mThreadState;
}

JSErrorReport& iJSReportCommit::fGetJSReport()
{
	return mErrorReport;
}


iJSExecReportCommit::iJSExecReportCommit(
	JSContext* rCx,
	const char* rMessage,
	JSErrorReport* rErrorReport )
:
	iJSReportCommit( rMessage, *rErrorReport),
	mCx(rCx)
{}

iJSEmbedContext& iJSExecReportCommit::vGetEmbedContext()
{
	return *iJSEmbedContext::fsGet( mCx );
}









cJSDReportCommit::cJSDReportCommit(
	iUi& rUi,
	const char* rMessage,
	JSErrorReport* rErrorReport )
:
	iJSReportCommit( rMessage, *rErrorReport ),
	mUi(rUi),
  mReturnValue(JSD_ERROR_REPORTER_CLEAR_RETURN)
{}


uintN cJSDReportCommit::fGetReturnValue() const 
{
	return mReturnValue;
}



void cJSDReportCommit::fSetReturnValue( tDebugReporterReturn r )
{
	uintN rtn;
	switch(r) {
	case sDebugReporterReturnApply:
		rtn = JSD_ERROR_REPORTER_PASS_ALONG;
		break;
	case sDebugReporterReturnDM:
		rtn = JSD_ERROR_REPORTER_DEBUG;
		break;
	case sDebugReporterReturnContinue:
		rtn = JSD_ERROR_REPORTER_RETURN;
		break;
	case sDebugReporterReturnClear:
		rtn = JSD_ERROR_REPORTER_CLEAR_RETURN;
		break;
	default:
		WlAssert(0); // switch error - should never get here
		break;
	}
	mReturnValue = rtn;
}


iUi& cJSDReportCommit::fGetUi() const
{
	return mUi;
}







 	} // nJs


