/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "tracedll.h"
#include "wlerrorlib.h"
#include "loadexceptions.h"
#include "loadreportinfo.h"
#include "wlxmlfuncs.h"
#include "wlxmlprivate.h"
#include <xercesc/dom/DOMCharacterData.hpp>
#include <xercesc/dom/DOMText.hpp>
#include <xercesc/dom/DOMComment.hpp>

XERCES_CPP_NAMESPACE_USE

#define CLASSNAME DOMCharacterData

DOMCharacterData* CharacterData_GetThisObject(JSContext *p_pCX, JSObject *p_pObj);

JSBool WLXMLCharacterDataEnumerate(JSContext *cx, JSObject *obj );


static JSClass wl_XMLCharacterDataClass = {
	"XMLCharacterDataObject",
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
	WLXMLCharacterDataEnumerate,//JS_EnumerateStub
	JS_ResolveStub,
	JS_ConvertStub,
	WLXmlGenericFinalize
};




JSBool WLXMLCharacterDataEnumerate(JSContext *cx, JSObject *obj )
{
  //enumerate class is implemented for: (for a in xmlparse), it can be implemented later.
  DOMCharacterData* pObj = CharacterData_GetThisObject(cx, obj);
  
  if (pObj==NULL)
    return JS_TRUE;

	return JS_TRUE;
}



static JSBool
CharacterData_appendData(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMCharacterData* pChData = CharacterData_GetThisObject(cx, obj);
  if (pChData)
  {
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcArg, argv, 0, "CharacterData_appendData")
    if (pcArg) 
    { 
      TRY_XML_PARSER_STANDARD
		    pChData->appendData(pcArg);
      CATCH_XML_PARSER_STANDARD(CharacterData,appendData)
	  }
  }
	return JS_TRUE;					
}

static JSBool
CharacterData_setData(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMCharacterData* pChData = CharacterData_GetThisObject(cx, obj);
  if (pChData)
  {
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcArg, argv, 0, "CharacterData_setData")
    if (pcArg) 
    { 
      TRY_XML_PARSER_STANDARD
		    pChData->setData(pcArg);											
      CATCH_XML_PARSER_STANDARD(CharacterData,setData)
	  }																																								       
  }
	return JS_TRUE;					
}

static JSBool
CharacterData_substringData(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMCharacterData* pChData = CharacterData_GetThisObject(cx, obj);
  if (pChData)
  {
    XML_GET_XMLSIZE_ARGUMENT_CHECK(pcOffset,argv,0,"CharacterData_substringData")
    XML_GET_XMLSIZE_ARGUMENT_CHECK(pcCount,argv,1,"CharacterData_substringData")
    if (pcOffset && pcCount) 
    {
      const XMLCh*  pcData = 0;
      TRY_XML_PARSER_STANDARD
        pcData = pChData->substringData(pcOffset,pcCount);
      CATCH_XML_PARSER_STANDARD(CharacterData,substringData)
		  WLXmlSetXMLChValue(cx, obj, pcData, rval);
	  }
  }
	return JS_TRUE;
}

static JSBool
CharacterData_deleteData(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMCharacterData* pChData = CharacterData_GetThisObject(cx, obj);
  if (pChData)
  {
    XML_GET_XMLSIZE_ARGUMENT_CHECK(pcOffset,argv,0,"CharacterData_deleteData")
    XML_GET_XMLSIZE_ARGUMENT_CHECK(pcCount,argv,1,"CharacterData_deleteData")
    if (pcOffset && pcCount) 
    { 
       TRY_XML_PARSER_STANDARD
        pChData->deleteData(pcOffset,pcCount);
       CATCH_XML_PARSER_STANDARD(CharacterData,deleteData)
	  }
  }
	return JS_TRUE;
}

static JSBool
CharacterData_insertData(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMCharacterData* pChData = CharacterData_GetThisObject(cx, obj);
  if (pChData)
  {
    XML_GET_XMLSIZE_ARGUMENT_CHECK(pcOffset,argv,0,"CharacterData_insertData")
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcArg, argv, 1, "CharacterData_insertData")
    if (pcOffset && pcArg) 
    { 
       TRY_XML_PARSER_STANDARD
        pChData->insertData(pcOffset,pcArg);
       CATCH_XML_PARSER_STANDARD(CharacterData,insertData)
	  }
  }
	return JS_TRUE;
}

static JSBool
CharacterData_replaceData(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMCharacterData* pChData = CharacterData_GetThisObject(cx, obj);
  if (pChData)
  {
    XML_GET_XMLSIZE_ARGUMENT_CHECK(pcOffset,argv,0,"CharacterData_replaceData")
    XML_GET_XMLSIZE_ARGUMENT_CHECK(pcCount,argv,1,"CharacterData_replaceData")
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcArg, argv, 2, "CharacterData_replaceData")
    if (pcOffset && pcCount && pcArg) 
    { 
       TRY_XML_PARSER_STANDARD
        pChData->replaceData(pcOffset,pcCount, pcArg);
       CATCH_XML_PARSER_STANDARD(CharacterData,replaceData)
	  }
  }
	return JS_TRUE;
}


METHOD_RET_INT_NO_ARGS(CharacterData, getLength);
METHOD_RET_XMLCHAR_NO_ARGS(CharacterData,getData);

static JSFunctionSpec xmlCharacterData_methods[] = {
  {"getData",       CharacterData_getData, 0},
  {"getLength",     CharacterData_getLength, 0},
  {"appendData",    CharacterData_appendData, 1},
  {"setData",       CharacterData_setData, 1},
  {"substringData", CharacterData_substringData, 2},
  {"deleteData",    CharacterData_deleteData, 2},
  {"insertData",    CharacterData_insertData, 2},
  {"replaceData",   CharacterData_replaceData, 3},
	{0}
};


PROPERTY_GETTER_RET_INT(CharacterData,length,getLength)
PROPERTY_GETTER_RET_XMLCHAR(CharacterData,data,getData)

PROPERTY_SETTER_FOR_STRING(CharacterData,data,setData)

static JSPropertySpec xmlCharacterData_properties[] = {
  { "length" , 0, READONLY_PROPERTY_FLAGS , CharacterData_prop_getLength , JS_PropertyStub },
  { "data"   , 0, PROPERTY_FLAGS          , CharacterData_prop_getData   , CharacterData_prop_setData },
	{0}
};



//create new CharacterData object
jsval WLCreateCharacterDataObject(JSContext* cx, JSObject* obj, void* value, bool toDelete)
{
  if (!value)
    return JSVAL_NULL;

  CJSGCRequestBeginEnd GCGuard(cx); //lock GC for this action
	CWLXmlParserPrivate* prv = new CWLXmlParserPrivate((void*)value, CWLXmlParserPrivate::CHARACTERDATA, toDelete);
  JSObject* newObj = JS_NewObject(cx, &wl_XMLCharacterDataClass, NULL, NULL);
	JS_SetPrivate(cx, newObj, (void*)prv);
  return OBJECT_TO_JSVAL(newObj);
}

//when we'll define all classes
JSObject* rvAddXMLCharacterDataClass(JSContext *cx, JSObject *obj, JSObject* parent_proto)
{
  return JS_InitClass(cx, obj, parent_proto, &wl_XMLCharacterDataClass, NULL, 0, xmlCharacterData_properties, xmlCharacterData_methods, NULL, NULL);
}


DOMCharacterData* CharacterData_GetThisObject(JSContext *p_pCX, JSObject *p_pObj)
{
  CJSGCRequestBeginEnd GCGuard(p_pCX); //lock GC for this action
	CWLXmlParserPrivate* pPrv = (CWLXmlParserPrivate*)JS_GetPrivate( p_pCX, p_pObj );
	if ( !pPrv )
		return NULL;

  switch (pPrv->m_eClass)
  {
  case CWLXmlParserPrivate::CHARACTERDATA:return (DOMCharacterData*)pPrv->GetCPPObj();
  case CWLXmlParserPrivate::TEXT:return (DOMText*)pPrv->GetCPPObj();
  case CWLXmlParserPrivate::COMMENT:return (DOMComment*)pPrv->GetCPPObj();
  default:WlTraceMsg("characterdata_GetThisObject default class, type = %d",pPrv->m_eClass); assert(0);
  };
  return NULL;

}

