/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "tracedll.h"
#include "wlerrorlib.h"
#include "loadexceptions.h"
#include "loadreportinfo.h"
#include "wlxmlfuncs.h"
#include "wlxmlprivate.h"
#include "wlxmlfuncs.h"
#include <xercesc/dom/DOMDocument.hpp>


XERCES_CPP_NAMESPACE_USE

#define CLASSNAME DOMDocument
DEFAULT_GET_THIS_OBJECT(document);

JSBool WLXMLDocumentEnumerate(JSContext *cx, JSObject *obj );

static JSClass wl_XMLDocumentClass = {
	"XMLDocumentObject",
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
	WLXMLDocumentEnumerate,//JS_EnumerateStub
	JS_ResolveStub,
	JS_ConvertStub,
	WLXmlGenericFinalize
};



JSBool WLXMLDocumentEnumerate(JSContext *cx, JSObject *obj )
{
  //enumerate class is implemented for: (for a in xmlparse), it can be implemented later.
  DOMDocument* pDocument = document_GetThisObject(cx, obj);
  
  if (pDocument==NULL)
    return JS_TRUE;;

	return JS_TRUE;
}


METHOD_RET_OBJECT_ONE_ARG_XMLCH(document, createElement, Element, false);
METHOD_RET_OBJECT_ONE_ARG_XMLCH(document, getElementById, Element, false);
METHOD_RET_OBJECT_ONE_ARG_XMLCH(document, getElementsByTagName, NodeList, false);
METHOD_RET_OBJECT_ONE_ARG_XMLCH(document, createTextNode, Text, false);
METHOD_RET_OBJECT_NO_ARGS(document, getDocumentElement, Element, false);
METHOD_RET_OBJECT_NO_ARGS(document, createDocumentFragment, DocumentFragment, false);
METHOD_RET_OBJECT_NO_ARGS(document, getDoctype, DocumentType, false);
METHOD_RET_OBJECT_ONE_ARG_XMLCH(document, createComment, Comment, false);
METHOD_RET_OBJECT_ONE_ARG_XMLCH(document, createCDATASection, CDATASection, false);
METHOD_RET_OBJECT_ONE_ARG_XMLCH(document, createAttribute, Attr, false);
METHOD_RET_OBJECT_ONE_ARG_XMLCH(document, createEntityReference, EntityReference, false);
METHOD_RET_OBJECT_TWO_ARG_XMLCH(document, createProcessingInstruction, ProcessingInstruction, false);
METHOD_RET_OBJECT_TWO_ARG_XMLCH(document, createElementNS, Element, false);
METHOD_RET_OBJECT_TWO_ARG_XMLCH(document, createAttributeNS, Attr, false);
METHOD_RET_OBJECT_TWO_ARG_XMLCH(document, getElementsByTagNameNS, NodeList, false);

static JSBool
document_importNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMDocument* pDocument = document_GetThisObject(cx, obj);
  
  if (pDocument)
  {
    XML_GET_OBJECT_ARGUMENT_CHECK(pImportedNode, argv, 0, DOMNode, "importNode")
    XML_GET_BOOL_ARGUMENT_CHECK(bDeep, argv, 1, "importNode")
    DOMNode* pNode = 0;
    TRY_XML_PARSER_STANDARD
      pNode = pDocument->importNode(pImportedNode,bDeep);
    CATCH_XML_PARSER_STANDARD(document,importNode)
    *rval = WLCreateNodeObject(cx, obj, pNode, true);//@@yossi - check this
  } 
	return JS_TRUE;
}

/*
 virtual DOMImplementation* getImplementation() const = 0; 
*/

static JSFunctionSpec xmlDocument_methods[] = {
  {"createElement",   document_createElement, 1},
  {"getElementById",   document_getElementById, 1},
  {"getDocumentElement",   document_getDocumentElement, 0},
  {"getElementsByTagName",   document_getElementsByTagName, 1},
  {"createTextNode",   document_createTextNode, 1},
  {"createDocumentFragment", document_createDocumentFragment, 0},
  {"getDoctype", document_getDoctype, 0},
  {"createComment", document_createComment, 1},
  {"createCDATASection", document_createCDATASection, 1},
  {"createAttribute", document_createAttribute, 1},
  {"createEntityReference", document_createEntityReference, 1},
  {"createProcessingInstruction", document_createProcessingInstruction, 2},
  {"createElementNS", document_createElementNS, 2},
  {"createAttributeNS", document_createAttributeNS, 2},
  {"getElementsByTagNameNS", document_getElementsByTagNameNS, 2},
  {"importNode", document_importNode, 2},
  {0}
};

PROPERTY_GETTER_RET_OBJECT(document,documentElement,getDocumentElement,Element,false)

static JSPropertySpec xmlDocument_properties[] = {
  { "documentElement"       , 0, READONLY_PROPERTY_FLAGS   , document_prop_getDocumentElement  , JS_PropertyStub },
	{0}
};




jsval WLCreateDocumentObject(JSContext* cx, JSObject* obj, void* value, bool toDelete)
{
  if (!value)
    return JSVAL_NULL;

  CJSGCRequestBeginEnd GCGuard(cx); //lock GC for this action
	CWLXmlParserPrivate* prv = new CWLXmlParserPrivate((void*)value, CWLXmlParserPrivate::DOCUMENT, toDelete);
  JSObject* newObj = JS_NewObject(cx, &wl_XMLDocumentClass, NULL, NULL);
	JS_SetPrivate(cx, newObj, (void*)prv);
  return OBJECT_TO_JSVAL(newObj);
}


//when we'll define all classes
JSObject* rvAddXMLDocumentClass(JSContext *cx, JSObject *obj, JSObject* parent_proto)
{
  return JS_InitClass(cx, obj, parent_proto, &wl_XMLDocumentClass, NULL, 0, xmlDocument_properties, xmlDocument_methods, NULL, NULL);
}





