/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlxmlfuncs.h"
//#include <collstr.h>
#include <string.h>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/dom/DOMNode.hpp>

extern "C" {
#include "jsapi.h"
}
#include "loadengineglobals.h"
#include "wlxmlprivate.h"

#include "tracedll.h"
#include "wlxmlparserexport.h"


XERCES_CPP_NAMESPACE_USE


void WLXmlGenericFinalize(JSContext* cx, JSObject* obj)
{
	CWLXmlParserPrivate* prv = (CWLXmlParserPrivate*)JS_GetPrivate(cx, obj);
	if (prv) 
  {
		delete prv;
		JS_SetPrivate(cx, obj, NULL);
	}
}


JSBool WLXmlSetIntValue(JSContext* cx, JSObject* obj, unsigned value, jsval* rval)
{
	*rval = INT_TO_JSVAL((int)value);
	return JS_TRUE;
}

JSBool WLXmlSetBoolValue(JSContext* cx, JSObject* obj, bool value, jsval* rval)
{
	if (value)
		*rval = BOOLEAN_TO_JSVAL(JS_TRUE);
	else
		*rval = BOOLEAN_TO_JSVAL(JS_FALSE);
	return JS_TRUE;
}

JSBool WLXmlSetDoubleValue(JSContext* cx, JSObject* obj, const char* name, double value, jsval* rval)
{
	jsdouble* val = JS_NewDouble(cx, (jsdouble)value);
	*rval = DOUBLE_TO_JSVAL(val);

	return JS_TRUE;
}

JSBool WLXmlSetStringValue(JSContext* cx, JSObject* obj, const char* value, jsval* rval, size_t length)
{
	if (!value)
		return JS_TRUE;

	JSString* str = NULL;
	if (length < 0)//length is not known.
		str =JS_NewStringCopyZ(cx, value);
	else
		str =JS_NewStringCopyN(cx, value, length);
	if (!str)
		return JS_FALSE;

	*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}

JSBool WLXmlSetXMLChValue(JSContext* cx, JSObject* obj, const XMLCh* value, jsval* rval)
{
	if (!value)
		return JS_TRUE;

  //XMLCh is utf-16
	JSString* str = NULL;

	str =JS_NewUCStringCopyZ(cx, value);

  if (!str)
		return JS_FALSE;

	*rval = STRING_TO_JSVAL(str);

  return JS_TRUE;
}


bool CreateNodeByType(JSContext* cx, JSObject* obj, DOMNode* p_pNode, jsval* rval, bool toDelete)
{
  if (p_pNode==NULL)
    return false;

  short nNodeType = p_pNode->getNodeType();

  switch(nNodeType)
  {
  case DOMNode::ELEMENT_NODE:
    *rval = WLCreateElementObject(cx, obj, p_pNode, toDelete);
    break;
  case DOMNode::ATTRIBUTE_NODE:
    *rval = WLCreateAttrObject(cx, obj, p_pNode, toDelete);
    break;
  case DOMNode::TEXT_NODE:
    *rval = WLCreateTextObject(cx, obj, p_pNode, toDelete);
    break;
  case DOMNode::CDATA_SECTION_NODE:
    *rval = WLCreateCDATASectionObject(cx, obj, p_pNode, toDelete);
    break;
  case DOMNode::ENTITY_REFERENCE_NODE:
    *rval = WLCreateEntityReferenceObject(cx, obj, p_pNode, toDelete);
    break;
  case DOMNode::ENTITY_NODE:
    *rval = WLCreateEntityObject(cx, obj, p_pNode, toDelete);
    break;
  case DOMNode::PROCESSING_INSTRUCTION_NODE:
    *rval = WLCreateProcessingInstructionObject(cx, obj, p_pNode, toDelete);
    break;
  case DOMNode::COMMENT_NODE:
    *rval = WLCreateCommentObject(cx, obj, p_pNode, toDelete);
    break;
  case DOMNode::DOCUMENT_NODE:
    *rval = WLCreateDocumentObject(cx, obj, p_pNode, toDelete);
    break;
  case DOMNode::DOCUMENT_TYPE_NODE:
    *rval = WLCreateDocumentTypeObject(cx, obj, p_pNode, toDelete);
    break;
  case DOMNode::DOCUMENT_FRAGMENT_NODE:
    *rval = WLCreateDocumentFragmentObject(cx, obj, p_pNode, toDelete);
    break;
  case DOMNode::NOTATION_NODE:
    *rval = WLCreateNotationObject(cx, obj, p_pNode, toDelete);
    break;
  default:
    *rval = WLCreateNodeObject(cx, obj, p_pNode, toDelete);
    break;    
  };

  return true;
}

char* GetNodeTypeString(const DOMNode* p_pNode)
{
  
  
  short nNodeType = p_pNode->getNodeType();

  switch(nNodeType)
  {
  case DOMNode::ELEMENT_NODE:
    return "element";
    break;
  case DOMNode::ATTRIBUTE_NODE:
    return "attribute";
    break;
  case DOMNode::TEXT_NODE:
    return "text";
    break;
  case DOMNode::CDATA_SECTION_NODE:
    return "cdatasection";
    break;
  case DOMNode::ENTITY_REFERENCE_NODE:
    return "entityreference";
    break;
  case DOMNode::ENTITY_NODE:
    return "entity";
    break;
  case DOMNode::PROCESSING_INSTRUCTION_NODE:
    return "processinginstruction";
    break;
  case DOMNode::COMMENT_NODE:
    return "comment";
    break;
  case DOMNode::DOCUMENT_NODE:
    return "document";
    break;
  case DOMNode::DOCUMENT_TYPE_NODE:
    return "documenttype";
    break;
  case DOMNode::DOCUMENT_FRAGMENT_NODE:
    return "documentfragment";
    break;
  case DOMNode::NOTATION_NODE:
    return "notation";
    break;
  default:
    return "UnknownNodeType";
  };
  
}

JSObject* rvAddXMLParserClass(JSContext *cx, JSObject *obj, JSObject* parent_proto);
JSObject* rvAddXMLNodeClass(JSContext *cx, JSObject *obj, JSObject* parent_proto);
JSObject* rvAddXMLDocumentClass(JSContext *cx, JSObject *obj, JSObject* parent_proto);
JSObject* rvAddXMLDocumentFragmentClass(JSContext *cx, JSObject *obj, JSObject* parent_proto);
JSObject* rvAddXMLCharacterDataClass(JSContext *cx, JSObject *obj, JSObject* parent_proto);
JSObject* rvAddXMLAttrClass(JSContext *cx, JSObject *obj, JSObject* parent_proto);
JSObject* rvAddXMLElementClass(JSContext *cx, JSObject *obj, JSObject* parent_proto);
JSObject* rvAddXMLTextClass(JSContext *cx, JSObject *obj, JSObject* parent_proto);
JSObject* rvAddXMLCommentClass(JSContext *cx, JSObject *obj, JSObject* parent_proto);
JSObject* rvAddXMLCDATASectionClass(JSContext *cx, JSObject *obj, JSObject* parent_proto);
JSObject* rvAddXMLDocumentTypeClass(JSContext *cx, JSObject *obj, JSObject* parent_proto);
JSObject* rvAddXMLNotationClass(JSContext *cx, JSObject *obj, JSObject* parent_proto);
JSObject* rvAddXMLEntityClass(JSContext *cx, JSObject *obj, JSObject* parent_proto);
JSObject* rvAddXMLEntityReferenceClass(JSContext *cx, JSObject *obj, JSObject* parent_proto);
JSObject* rvAddXMLProcessingInstructionClass(JSContext *cx, JSObject *obj, JSObject* parent_proto);
JSObject* rvAddXMLNodeListClass(JSContext *cx, JSObject *obj, JSObject* parent_proto);
JSObject* rvAddXMLNamedNodeMapClass(JSContext *cx, JSObject *obj, JSObject* parent_proto);

bool DllExport WLAddXMLClasses(JSContext *cx, JSObject *obj)
{
  bool bRet = true;

  rvAddXMLParserClass(cx, obj, NULL);
  JSObject* pNode = rvAddXMLNodeClass(cx, obj, NULL);
  rvAddXMLDocumentClass(cx, obj, pNode);
  rvAddXMLDocumentFragmentClass(cx, obj, pNode);
  JSObject* pCharacterData = rvAddXMLCharacterDataClass(cx, obj, pNode);
  rvAddXMLAttrClass(cx, obj, pNode);
  rvAddXMLElementClass(cx, obj, pNode);
  JSObject* pText = rvAddXMLTextClass(cx, obj, pCharacterData);
  rvAddXMLCommentClass(cx, obj, pCharacterData);
  rvAddXMLCDATASectionClass(cx, obj, pText);
  rvAddXMLDocumentTypeClass(cx, obj, pNode);
  rvAddXMLNotationClass(cx, obj, pNode);
  rvAddXMLEntityClass(cx, obj, pNode);
  rvAddXMLEntityReferenceClass(cx, obj, pNode);
  rvAddXMLProcessingInstructionClass(cx, obj, pNode);
  rvAddXMLNodeListClass(cx, obj, NULL);
  rvAddXMLNamedNodeMapClass(cx, obj, NULL);

        try 
        {
            XMLPlatformUtils::Initialize();
        }
        catch (const XMLException& toCatch) 
        {
            char* message = XMLString::transcode(toCatch.getMessage());
            WlTraceMsg("XMLPlatformUtils::Initialize failed. xerces message: %s",message);
           	WLErrorException e( XMLPARSER_ERROR_FILE, 10);
      			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );

            XMLString::release(&message);
            return bRet;
        }

  return bRet;
}

